(*============================================================================*)
(* BOARD.PAS -- Chess board object                                            *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: Added bShowSOutline into board object.                         *)
(* 2002.01   : Modified whole bunch of functions to allow bitmap graphics     *)
(* 2002.02.02: Moved routines drawing toups vector pieces to toups.pas        *)
(* 2004.11.15: 1) Added routine drawminus to draw "-" in front of the piece   *)
(*                if the holding is negative (aerial drop)                    *)
(* 2004.11.27: 1) Changed routine onMouseDown to allow aerial drops with a    *)
(*                mouse                                                       *)
(*             2) Added routine tboard.pixels                                 *)
(*             3) Added routine tboard.dropchesstoboard                       *)
(* 2004.12.07  1) Added nPopupDropX, nPopupDropY to allow right mouse drops   *)
(*============================================================================*)
UNIT BOARD;

INTERFACE

USES Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     CHESS, PROMOTE,GLOBAL,toups;


TYPE
    TBoard = CLASS(TObject)
        CONSTRUCTOR Init;
        DESTRUCTOR  Destroy;

        PROCEDURE   LoadPiece;

        PROCEDURE   DrawPawn  (TC: TCanvas);
        PROCEDURE   DrawRook  (TC: TCanvas);
        PROCEDURE   DrawKnight(TC: TCanvas);
        PROCEDURE   DrawBishop(TC: TCanvas);
        PROCEDURE   DrawQueen (TC: TCanvas);
        PROCEDURE   DrawKing  (TC: TCanvas);

        PROCEDURE   Draw11(TC: TCanvas);
        PROCEDURE   Draw0 (TC: TCanvas);
        PROCEDURE   Draw1 (TC: TCanvas);
        PROCEDURE   Draw2 (TC: TCanvas);
        PROCEDURE   Draw3 (TC: TCanvas);
        PROCEDURE   Draw4 (TC: TCanvas);
        PROCEDURE   Draw5 (TC: TCanvas);
        PROCEDURE   Draw6 (TC: TCanvas);
        PROCEDURE   Draw7 (TC: TCanvas);
        PROCEDURE   Draw8 (TC: TCanvas);
        PROCEDURE   Draw9 (TC: TCanvas);
        procedure   drawminus (TC: TCanvas);

        PROCEDURE   DrawBlackPawn  (TC: TCanvas);
        PROCEDURE   DrawBlackRook  (TC: TCanvas);
        PROCEDURE   DrawBlackKnight(TC: TCanvas);
        PROCEDURE   DrawBlackBishop(TC: TCanvas);
        PROCEDURE   DrawBlackQueen (TC: TCanvas);
        PROCEDURE   DrawBlackKing  (TC: TCanvas);

        PROCEDURE   DrawWhitePawn  (TC: TCanvas);
        PROCEDURE   DrawWhiteRook  (TC: TCanvas);
        PROCEDURE   DrawWhiteKnight(TC: TCanvas);
        PROCEDURE   DrawWhiteBishop(TC: TCanvas);
        PROCEDURE   DrawWhiteQueen (TC: TCanvas);
        PROCEDURE   DrawWhiteKing  (TC: TCanvas);

        PROCEDURE   DrawPawnOutline  (TC: TCanvas);
        PROCEDURE   DrawRookOutline  (TC: TCanvas);
        PROCEDURE   DrawKnightOutline(TC: TCanvas);
        PROCEDURE   DrawBishopOutline(TC: TCanvas);
        PROCEDURE   DrawQueenOutline (TC: TCanvas);
        PROCEDURE   DrawKingOutline  (TC: TCanvas);
        PROCEDURE   DrawPieceOutline (TC: TCanvas);

        PROCEDURE   DrawPawnMask  (TC: TCanvas);
        PROCEDURE   DrawRookMask  (TC: TCanvas);
        PROCEDURE   DrawKnightMask(TC: TCanvas);
        PROCEDURE   DrawBishopMask(TC: TCanvas);
        PROCEDURE   DrawQueenMask (TC: TCanvas);
        PROCEDURE   DrawKingMask  (TC: TCanvas);

        PROCEDURE   DrawBlackSquare(TC: TCanvas);
        PROCEDURE   DrawWhiteSquare(TC: TCanvas);
        PROCEDURE   DrawPieceSquare(TC: TCanvas);

        FUNCTION    PieceXY(nI: Integer): Integer;
        FUNCTION    IndexXY(nP: Integer): Integer;
        FUNCTION    GetDropIndex(nBX, nBY: Integer): Integer;

        PROCEDURE   DrawPXY   (CV: TCanvas; nSc, nPX, nPY: Integer; cPiece: Char);
        PROCEDURE   DrawPiece (CV: TCanvas; nBX, nBY: Integer);
        PROCEDURE   DrawNumber(CV: TCanvas; nN: Integer);
        PROCEDURE   DrawDropPc(CV: TCanvas; nBX, nBY: Integer);
        PROCEDURE   DrawBoard (CV: TCanvas);

        PROCEDURE   OnMouseDown(CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
        PROCEDURE   OnMouseMove(CV: TCanvas; nMX, nMY: Integer);
        PROCEDURE   OnMouseUp  (CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
        PROCEDURE   SMouseDown(CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
        PROCEDURE   SMouseMove(CV: TCanvas; nMX, nMY: Integer);
        PROCEDURE   SMouseUp  (CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
        procedure   chesstopixels(square:integer;var X,Y:integer;cmove:string);
        procedure   dropchesstoboard(npc,npi:integer;var x,y:integer);
        function    isOnBoard(X,Y:integer):integer;
        procedure   DropPopupPiece(cPc:char;CV:TCanvas);
        procedure   switchPromoted(nx,ny:integer);
        PRIVATE

            bTmp:         TBitmap;
            bSaveBack:    TBitMap;
            bWorkSpace:   TBitmap;

            bDragPiece:   Boolean;

            nFromX:       Integer;
            nFromY:       Integer;

            nToX:         Integer;
            nToY:         Integer;

            cPromote:     Char;

            cPc:          Char;
            nPi:          Integer;
            nPc:          Integer;



            rRect:        TRect;

            nDropX:       Integer;
            nDropY:       Integer;

            aNum2PX:      ARRAY[0..2] OF Integer;
            aNum2PY:      ARRAY[0..2] OF Integer;

        PUBLIC
            nMXOff:       Integer;
            nMYOff:       Integer;
            nBlkSquare:   TColor;
            nBlkSOutline: TColor;
            nBlkOutline:  TColor;
            nBlkFill:     TColor;

            nWhtSquare:   TColor;
            nWhtSOutline: TColor;
            nWhtOutline:  TColor;
            nWhtFill:     TColor;

            nClickColor:  TColor;
            nPcSquare:    TColor;
            nNumberColor: TColor;

            nMoveI:       Integer;
            Chess:        TChess;
            nPIndex:      Integer;
            nRIndex:      Integer;
            nNIndex:      Integer;
            nBIndex:      Integer;
            nQIndex:      Integer;
            nKIndex:      Integer;

            nPieceSize:   Integer;
            nPieceRatio:  Real;

            rSqrRect:     TRect;

            aSqrClr:      ARRAY[1..8, 1..8] OF Integer;
            aImage:       ARRAY[1..3, 1..6] OF TBitmap;
            aSquare:      ARRAY[1..3]       OF TBitmap;

            bMadeMove:    Boolean;

            bFlip:        Boolean;
            bDisable:     Boolean;
            bAlwaysQueen: Boolean;
            bOutlinePC:   Boolean;
            bDragMove:    Boolean;
            bShwPCCount:  Boolean;
            bSmartMove:   Boolean;
            bShowSOutline:Boolean;
            nPopupDropX:integer;
            nPopupDropY:integer;

    END;

VAR

    aNum2X:       ARRAY[0..2] OF Integer = (2,4,6);
    aNum2Y:       ARRAY[0..2] OF Integer = (1,5,9);

IMPLEMENTATION

CONSTRUCTOR TBoard.Init;
VAR
   nY, nX, nC, nI: Integer;
BEGIN
    nBlkSquare   := clTeal;
    nBlkSOutline := clBlack;
    nBlkOutline  := clGray;
    nBlkFill     := clBlack;

    nWhtSquare   := clWhite;
    nWhtSOutline := clBlack;
    nWhtOutline  := clBlack;
    nWhtFill     := clWhite;

    nClickColor  := clRed;
    nPcSquare    := clBtnFace;
    nNumberColor := clBlack;

    nMoveI       := 0;
    Chess        := TChess.Create;
    Chess.Init;
    nPIndex      := Chess.GetPawnIndex;
    nRIndex      := Chess.GetRookIndex;
    nNIndex      := Chess.GetKnightIndex;
    nBIndex      := Chess.GetBishopIndex;
    nQIndex      := Chess.GetQueenIndex;
    nKIndex      := Chess.GetKingIndex;

    FOR nY := 1 TO 8 DO
    BEGIN
        FOR nX := 1 TO 8 DO
        BEGIN
            IF ((nY = 1) OR (nY = 3) OR (nY = 5) OR (nY = 7)) THEN
            BEGIN
                IF ((nX = 1) OR (nX = 3) OR (nX = 5) OR (nX = 7)) THEN
                    aSqrClr[nX, nY] := BLACK_COLOR
                ELSE
                    aSqrClr[nX, nY] := WHITE_COLOR;
            END
            ELSE
            BEGIN
                IF ((nX = 1) OR (nX = 3) OR (nX = 5) OR (nX = 7)) THEN
                    aSqrClr[nX, nY] := WHITE_COLOR
                ELSE
                    aSqrClr[nX, nY] := BLACK_COLOR;
            END;
        END;
    END;

    nPieceSize  := PIECE_ORG_SIZE;
    nPieceRatio := 1;

    FOR nY := 1 TO 3 DO
        FOR nX := 1 TO 6 DO
            aImage[nY, nX] := TBitmap.Create;



    aSquare[1]        := TBitmap.Create;
    aSquare[2]        := TBitmap.Create;
    asquare[3]:=TbitMap.create;

    bTmp              := TBitmap.Create;
    bTmp.Width        := nPieceSize;
    bTmp.Height       := nPieceSize;

    bSaveBack         := TBitmap.Create;
    bSaveBack.Width   := nPieceSize;
    bSaveBack.Height  := nPieceSize;

    bWorkSpace        := TBitmap.Create;
    bWorkSpace.Width  := nPieceSize;
    bWorkSpace.Height := nPieceSize;

    LoadPiece;

    bDragPiece        := FALSE;

    nFromX            := 0;
    nFromY            := 0;

    nToX              := 0;
    nToY              := 0;

    cPromote          := Chess.GetEmpty;

    cPc               := Chess.GetEmpty;
    nPi               := 0;
    nPc               := 0;

    nMXOff            := 0;
    nMYOff            := 0;

    nDropX            := 0;
    nDropY            := 0;

    bMadeMove         := FALSE;

    bFlip             := FALSE;
    bDisable          := FALSE;
    bAlwaysQueen      := FALSE;
    bOutlinePC        := FALSE;
    bDragMove         := TRUE;
    bShwPCCount       := TRUE;
    bSmartMove        := FALSE;
    bShowSOutline     := TRUE;
END;

DESTRUCTOR TBoard.Destroy;
VAR
   nC, nI: Integer;
BEGIN
    FOR nC := 1 TO 3 DO
        FOR nI := 1 TO 6 DO
            aImage[nC, nI].Free;

    FOR nC := 1 TO 3 DO
        aSquare[nC].Free;

    bTmp.Free;
    bSaveBack.Free;
    bWorkSpace.Free;

    Chess.Free;
END;

PROCEDURE TBoard.LoadPiece;
VAR
    i,j,  nC, nI: Integer;
BEGIN
    { Setup piece rectangle }
    rSqrRect := Rect(0, 0, nPieceSize, nPieceSize);

    { Setup piece image size }
    FOR nC := 1 TO 3 DO
        FOR nI := 1 TO 6 DO
        BEGIN
           aImage[nC, nI].Width  := nPieceSize;
           aImage[nC, nI].Height := nPieceSize;
           if (usepiecebitmap and (nc<3) )then with aimage[nc,ni] do
           begin
            pieces[nc,ni].transparent:=false;
            canvas.stretchdraw(rsqrrect,pieces[nc,ni]);
            transparentcolor:=canvas.Pixels[0,0];
            transparent:=true;
           end;
        END;

    { Setup square and temporary image size }
    aSquare[1].Width  := nPieceSize;
    aSquare[1].Height := nPieceSize;

    aSquare[2].Width  := nPieceSize;
    aSquare[2].Height := nPieceSize;

    aSquare[3].Width  := nPieceSize;
    aSquare[3].Height := nPieceSize;

    bTmp.Width        := nPieceSize;
    bTmp.Height       := nPieceSize;

    bSaveBack.Width   := nPieceSize;
    bSaveBack.Height  := nPieceSize;

    bWorkSpace.Width  := nPieceSize;
    bWorkSpace.Height := nPieceSize;

    { Setup piece images }
if usepiecebitmap then
begin
{    aImage[1, nPIndex].Canvas.stretchdraw(rsqrrect,pieces[1,6]);
    aImage[1, nRIndex].Canvas.stretchdraw(rsqrrect,pieces[1,1]);
    aImage[1, nNIndex].Canvas.stretchdraw(rsqrrect,pieces[1,2]);
    aImage[1, nBIndex].Canvas.stretchdraw(rsqrrect,pieces[1,3]);
    aImage[1, nQIndex].Canvas.stretchdraw(rsqrrect,pieces[1,4]);
    aImage[1, nKIndex].Canvas.stretchdraw(rsqrrect,pieces[1,5]);

    aImage[2, nPIndex].Canvas.stretchdraw(rsqrrect,pieces[2,6]);
    aImage[2, nRIndex].Canvas.stretchdraw(rsqrrect,pieces[2,1]);
    aImage[2, nNIndex].Canvas.stretchdraw(rsqrrect,pieces[2,2]);
    aImage[2, nBIndex].Canvas.stretchdraw(rsqrrect,pieces[2,3]);
    aImage[2, nQIndex].Canvas.stretchdraw(rsqrrect,pieces[2,4]);
    aImage[2, nKIndex].Canvas.stretchdraw(rsqrrect,pieces[2,5]);

    for i:=1 to 2 do
    for j:=1 to 6 do aimage[i,j].transparent:=true;
    // Setup piece masks
   // DrawPawnMask   (aImage[3, nPIndex].Canvas);
   // DrawRookMask   (aImage[3, nRIndex].Canvas);
   // DrawKnightMask (aImage[3, nNIndex].Canvas);
   // DrawBishopMask (aImage[3, nBIndex].Canvas);
   // DrawQueenMask  (aImage[3, nQIndex].Canvas);
   // DrawKingMask   (aImage[3, nKIndex].Canvas);}
end
else begin
    DrawWhitePawn  (aImage[1, nPIndex].Canvas);
    DrawWhiteRook  (aImage[1, nRIndex].Canvas);
    DrawWhiteKnight(aImage[1, nNIndex].Canvas);
    DrawWhiteBishop(aImage[1, nBIndex].Canvas);
    DrawWhiteQueen (aImage[1, nQIndex].Canvas);
    DrawWhiteKing  (aImage[1, nKIndex].Canvas);

    DrawBlackPawn  (aImage[2, nPIndex].Canvas);
    DrawBlackRook  (aImage[2, nRIndex].Canvas);
    DrawBlackKnight(aImage[2, nNIndex].Canvas);
    DrawBlackBishop(aImage[2, nBIndex].Canvas);
    DrawBlackQueen (aImage[2, nQIndex].Canvas);
    DrawBlackKing  (aImage[2, nKIndex].Canvas);

    { Setup piece masks }
    DrawPawnMask   (aImage[3, nPIndex].Canvas);
    DrawRookMask   (aImage[3, nRIndex].Canvas);
    DrawKnightMask (aImage[3, nNIndex].Canvas);
    DrawBishopMask (aImage[3, nBIndex].Canvas);
    DrawQueenMask  (aImage[3, nQIndex].Canvas);
    DrawKingMask   (aImage[3, nKIndex].Canvas);
end;
    { Setup empty board squares }
    if useboardbitmap then
    begin
     asquare[1].canvas.stretchdraw(rsqrrect,boardbmp[1]);
     asquare[2].canvas.stretchdraw(rsqrrect,boardbmp[2]);
    end
    else begin
     DrawWhiteSquare(aSquare[1].Canvas);
     DrawBlackSquare(aSquare[2].Canvas);
    end;
    asquare[3].canvas.copymode:=cmSrcCopy;
    if usebitmap then
    asquare[3].canvas.stretchdraw(rsqrrect,piecesbg) ;
   if not usebitmap then DrawPieceSquare(aSquare[3].Canvas) ;

         { Second digit number positions }
    FOR nI := 0 TO 2 DO
    BEGIN
        aNum2PX[nI] := Trunc(aNum2X[nI] * nPieceRatio);
        aNum2PY[nI] := Trunc(aNum2Y[nI] * nPieceRatio);
    END;
END;

PROCEDURE TBoard.DrawPawn(TC: TCanvas);
begin
 toups.drawpawn(tc,npieceratio);
END;

PROCEDURE TBoard.DrawRook(TC: TCanvas);
BEGIN
 toups.drawrook(tc,npieceratio);
END;

PROCEDURE TBoard.DrawKnight(TC: TCanvas);
begin
 toups.drawknight(tc,npieceratio);
END;

PROCEDURE TBoard.DrawBishop(TC: TCanvas);
begin
 toups.drawbishop(tc,npieceratio);
END;

PROCEDURE TBoard.DrawQueen(TC: TCanvas);
begin
 toups.drawqueen(tc,npieceratio);
END;

PROCEDURE TBoard.DrawKing(TC: TCanvas);
begin
 toups.drawking(tc,npieceratio);
END;

PROCEDURE TBoard.Draw11(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(0, aNum2PY[0]);
    TC.LineTo(0, aNum2PY[2]);
END;

PROCEDURE TBoard.Drawminus(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);
END;


PROCEDURE TBoard.Draw0(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
    TC.LineTo(aNum2PX[0], aNum2PY[2]);
    TC.LineTo(aNum2PX[0], aNum2PY[0]);
END;

PROCEDURE TBoard.Draw1(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[1], aNum2PY[0]);
    TC.LineTo(aNum2PX[1], aNum2PY[2]);
END;

PROCEDURE TBoard.Draw2(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);
    TC.LineTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[0], aNum2PY[2]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
END;

PROCEDURE TBoard.Draw3(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
    TC.LineTo(aNum2PX[0], aNum2PY[2]);

    TC.MoveTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);
END;

PROCEDURE TBoard.Draw4(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);

    TC.MoveTo(aNum2PX[1], aNum2PY[0]);
    TC.LineTo(aNum2PX[1], aNum2PY[2]);
END;

PROCEDURE TBoard.Draw5(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
    TC.LineTo(aNum2PX[0], aNum2PY[2]);
END;

PROCEDURE TBoard.Draw6(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[0], aNum2PY[2]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);
    TC.LineTo(aNum2PX[0], aNum2PY[1]);
END;

PROCEDURE TBoard.Draw7(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
END;

PROCEDURE TBoard.Draw8(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
    TC.LineTo(aNum2PX[0], aNum2PY[2]);
    TC.LineTo(aNum2PX[0], aNum2PY[0]);

    TC.MoveTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[2], aNum2PY[1]);
END;

PROCEDURE TBoard.Draw9(TC: TCanvas);
BEGIN
    TC.Pen.Color := nNumberColor;
    TC.MoveTo(aNum2PX[2], aNum2PY[1]);
    TC.LineTo(aNum2PX[0], aNum2PY[1]);
    TC.LineTo(aNum2PX[0], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[0]);
    TC.LineTo(aNum2PX[2], aNum2PY[2]);
END;

PROCEDURE TBoard.DrawBlackPawn(TC: TCanvas);
BEGIN
 toups.drawblackpawn(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBlackRook(TC: TCanvas);
begin
 toups.drawblackrook(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBlackKnight(TC: TCanvas);
BEGIN
 toups.drawblackknight(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBlackBishop(TC: TCanvas);
BEGIN
 toups.drawblackbishop(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBlackQueen(TC: TCanvas);
BEGIN
 toups.drawblackqueen(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBlackKing(TC: TCanvas);
BEGIN
 toups.drawblackking(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawWhitePawn(TC: TCanvas);
BEGIN
 toups.drawwhitepawn(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawWhiteRook(TC: TCanvas);
BEGIN
toups.drawwhiterook(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawWhiteKnight(TC: TCanvas);
BEGIN
toups.drawwhiteknight(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawWhiteBishop(TC: TCanvas);
BEGIN
toups.drawwhitebishop(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawWhiteQueen(TC: TCanvas);
BEGIN
toups.drawwhitequeen(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawWhiteKing(TC: TCanvas);
BEGIN
toups.drawwhiteking(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawPawnOutline(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nClickColor;
    TC.Brush.Style := bsClear;
    DrawPawn(TC);
END;

PROCEDURE TBoard.DrawRookOutline(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nClickColor;
    TC.Brush.Style := bsClear;
    DrawRook(TC);
END;

PROCEDURE TBoard.DrawKnightOutline(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nClickColor;
    TC.Brush.Style := bsClear;
    DrawKnight(TC);
END;

PROCEDURE TBoard.DrawBishopOutline(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nClickColor;
    TC.Brush.Style := bsClear;
    DrawBishop(TC);
END;

PROCEDURE TBoard.DrawQueenOutline(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nClickColor;
    TC.Brush.Style := bsClear;
    DrawQueen(TC);
END;

PROCEDURE TBoard.DrawKingOutline(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nClickColor;
    TC.Brush.Style := bsClear;
    DrawKing(TC);
END;

PROCEDURE TBoard.DrawPieceOutline(TC: TCanvas);
BEGIN
    IF (nPi = nPIndex) THEN DrawPawnOutline  (TC) ELSE
    IF (nPi = nRIndex) THEN DrawRookOutline  (TC) ELSE
    IF (nPi = nNIndex) THEN DrawKnightOutline(TC) ELSE
    IF (nPi = nBIndex) THEN DrawBishopOutline(TC) ELSE
    IF (nPi = nQIndex) THEN DrawQueenOutline (TC) ELSE
    IF (nPi = nKIndex) THEN DrawKingOutline  (TC);
END;

PROCEDURE TBoard.DrawPawnMask(TC: TCanvas);
BEGIN
 toups.drawpawnmask(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawRookMask(TC: TCanvas);
BEGIN
toups.drawrookmask(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawKnightMask(TC: TCanvas);
BEGIN
toups.drawknightmask(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBishopMask(TC: TCanvas);
BEGIN
toups.drawbishopmask(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawQueenMask(TC: TCanvas);
BEGIN
toups.drawqueenmask(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawKingMask(TC: TCanvas);
BEGIN
toups.drawkingmask(tc,npiecesize,npieceratio);
END;

PROCEDURE TBoard.DrawBlackSquare(TC: TCanvas);
BEGIN
    IF (bShowSOutline) THEN
        TC.Pen.Color := nBlkSOutline
    ELSE
        TC.Pen.Color := nBlkSquare;

    TC.Brush.Color := nBlkSquare;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);
END;

PROCEDURE TBoard.DrawWhiteSquare(TC: TCanvas);
BEGIN
    IF (bShowSOutline) THEN
        TC.Pen.Color := nWhtSOutline
    ELSE
        TC.Pen.Color := nWhtSquare;

    TC.Brush.Color := nWhtSquare;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);
END;

PROCEDURE TBoard.DrawPieceSquare(TC: TCanvas);
BEGIN
    TC.Pen.Color   := nPcSquare;
    TC.Brush.Color := nPcSquare;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);
END;

FUNCTION TBoard.PieceXY(nI: Integer): Integer;
BEGIN
    Result := ((nI - 1) * nPieceSize);
END;

FUNCTION TBoard.IndexXY(nP: Integer): Integer;
BEGIN
    Result := (nP DIV nPieceSize) + 1;
END;

FUNCTION TBoard.GetDropIndex(nBX, nBY: Integer): Integer;
BEGIN
    CASE (nBY) OF
        1: IF (nBX = 9) THEN Result := 1 ELSE Result := 4;
        2: IF (nBX = 9) THEN Result := 2 ELSE Result := 5;
        3: IF (nBX = 9) THEN Result := 3 ELSE Result := 0;

        6: IF (nBX = 9) THEN Result := 1 ELSE Result := 4;
        7: IF (nBX = 9) THEN Result := 2 ELSE Result := 5;
        8: IF (nBX = 9) THEN Result := 3 ELSE Result := 0;
    ELSE
        Result := 0;
    END;
END;

PROCEDURE TBoard.DrawPXY(CV: TCanvas; nSc, nPX, nPY: Integer; cPiece: Char);
VAR
    nI: Integer;
BEGIN
    { Copy }
    bTmp.Canvas.CopyMode := cmSrcCopy;
    bTmp.Canvas.CopyRect(rSqrRect, aSquare[nSc].Canvas, rSqrRect);

    IF (Chess.IsEmpty(cPiece)) THEN
    BEGIN
        { Nothing }
    END
    ELSE
    BEGIN
        nI := Chess.GetIndex(cPiece);

        { And }
        if not usepiecebitmap then
        begin
      bTmp.Canvas.CopyMode := cmSrcAnd;
      bTmp.Canvas.CopyRect(rSqrRect, aImage[3, nI].Canvas, rSqrRect);

        { Or }
        bTmp.Canvas.CopyMode := cmSrcPaint;
        bTmp.Canvas.CopyRect(rSqrRect,
                             aImage[Chess.GetColor(cPiece), nI].Canvas,
                             rSqrRect);
       end
       else
        btmp.canvas.draw(0,0,aimage[Chess.GetColor(cPiece),ni]);
       // begin btmp.Canvas.copymode:=cmSrc.
    //   btmp.canvas.draw(0,0,aimage[Chess.GetColor(cPiece),ni])
    END;

    { Copy }
    CV.CopyRect(Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize),
                bTmp.Canvas, rSqrRect);
END;

PROCEDURE TBoard.DrawPiece(CV: TCanvas; nBX, nBY: Integer);
VAR
    nPX, nPY: Integer;
BEGIN
    IF (bFlip) THEN
    BEGIN
        nPX := PieceXY(Chess.GetRevIndex(nBX));
        nPY := PieceXY(nBY);
    END
    ELSE
    BEGIN
        nPX := PieceXY(nBX);
        nPY := PieceXY(Chess.GetRevIndex(nBY));
    END;
    DrawPXY(CV, aSqrClr[nBX, nBY], nPX, nPY, Chess.aBoard[nBX, nBY]);
END;

PROCEDURE TBoard.DrawNumber(CV: TCanvas; nN: Integer);
BEGIN
    if (nN <0 ) then begin drawminus(CV); exit;end;
    IF ((nN < 2) OR (nN > 19)) THEN
        EXIT;

    IF (nN > 9) THEN
    BEGIN
        Draw11(CV);

        nN := nN - 10;
    END;

    CASE (nN) OF
        0: Draw0(CV);
        1: Draw1(CV);
        2: Draw2(CV);
        3: Draw3(CV);
        4: Draw4(CV);
        5: Draw5(CV);
        6: Draw6(CV);
        7: Draw7(CV);
        8: Draw8(CV);
        9: Draw9(CV);
    END;
END;

PROCEDURE TBoard.DrawDropPc(CV: TCanvas; nBX, nBY: Integer);
VAR
    nP, nPX, nPY, nC, nI: Integer;
BEGIN
    nPX := PieceXY(nBX);
    nPY := PieceXY(nBY);

    nP := GetDropIndex(nBX, nBY);
    IF (nP = 0) THEN
    BEGIN
        { Copy }
        bTmp.Canvas.CopyMode := cmSrcCopy;
        bTmp.Canvas.CopyRect(rSqrRect, aSquare[3].Canvas, rSqrRect);

        { Copy }
      CV.CopyRect(Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize),
                  bTmp.Canvas, rSqrRect);
        EXIT;
    END;

    IF (bFlip) THEN
    BEGIN
        IF (nBY < 4) THEN
            nC := Chess.GetWhiteColor
        ELSE
            nC := Chess.GetBlackColor;
    END
    ELSE
    BEGIN
        IF (nBY < 4) THEN
            nC := Chess.GetBlackColor
        ELSE
            nC := Chess.GetWhiteColor;
    END;

    { Copy }
    bTmp.Canvas.CopyMode := cmSrcCopy;
    bTmp.Canvas.CopyRect(rSqrRect, aSquare[3].Canvas, rSqrRect);

    IF (Chess.IsEmpty(Chess.acPiece[nC, nP])) THEN
    BEGIN
        { Nothing }
    END
    ELSE
    BEGIN
        nI := Chess.GetIndex(Chess.acPiece[nC, nP]);

        { And }
        if not usepiecebitmap then
        begin
         bTmp.Canvas.CopyMode := cmSrcAnd;
         bTmp.Canvas.CopyRect(rSqrRect, aImage[3, nI].Canvas, rSqrRect);

        { Or }
        bTmp.Canvas.CopyMode := cmSrcPaint;
        bTmp.Canvas.CopyRect(rSqrRect, aImage[nC, nI].Canvas, rSqrRect);
        end
        else bTmp.Canvas.draw(0,0, aImage[nC, nI]);

        { Draw # of pieces remaining }
        IF (bShwPCCount) THEN
            DrawNumber(bTmp.Canvas, Chess.anPiece[nC, nP]);
    END;

    { Copy }
   CV.CopyRect(Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize),
               bTmp.Canvas, rSqrRect);
END;

PROCEDURE TBoard.DrawBoard(CV: TCanvas);
VAR
    nPX, nPY, nBX, nBY: Integer;
BEGIN
    { Draw top drop pieces }
    DrawDropPc(CV,  9, 1);
    DrawDropPc(CV,  9, 2);
    DrawDropPc(CV,  9, 3);
    DrawDropPc(CV,  9, 4);
    DrawDropPc(CV,  9, 5);
    DrawDropPc(CV, 10, 1);
    DrawDropPc(CV, 10, 2);

    { Draw board }
    FOR nBY := 1 TO 8 DO
        FOR nBX := 1 TO 8 DO
            DrawPiece(CV, nBX, nBY);

    { Draw bottom drop pieces }
    DrawDropPc(CV,  9, 6);
    DrawDropPc(CV,  9, 7);
    DrawDropPc(CV,  9, 8);
    DrawDropPc(CV, 10, 3);
    DrawDropPc(CV, 10, 4);
    DrawDropPc(CV, 10, 5);
    DrawDropPc(CV, 10, 6);
    DrawDropPc(CV, 10, 7);
    DrawDropPc(CV, 10, 8);
END;

PROCEDURE TBoard.OnMouseDown(CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
VAR
    nBX, nBY, nPX, nPY, nP, nC, nMI, nTmp: Integer;
    blah:integer;
    var fPromote:TFpromote;
BEGIN
    IF (bDisable) THEN
        EXIT;

    bMadeMove := FALSE;

    IF (bDragPiece) THEN
        EXIT;

    nBX := IndexXY(nMX);
    nBY := IndexXY(nMY);

    IF ((nBX > 8) AND (nBY < 4)) THEN    { Top }
    BEGIN
        IF (bFlip) THEN
        BEGIN
            IF (NOT Chess.bWhitesMove) THEN
                EXIT;

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetWhiteColor;
            cPc := Chess.acPiece[nC, nP];

            if ((bairdrop) and (cpc=EMPTY_SQUARE)) then cpc:=adroppiece[nc,np]; //attempt to allow aerial drops

            IF (NOT Chess.IsWhite(cPc)) THEN
                EXIT;
        END
        ELSE
        BEGIN
            IF (Chess.bWhitesMove) THEN
                EXIT;

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetBlackColor;
            cPc := Chess.acPiece[nC, nP];

            if ((bairdrop) and (cpc=EMPTY_SQUARE)) then cpc:=adroppiece[nc,np]; //attempt to allow aerial drops

            IF (NOT Chess.IsBlack(cPc)) THEN
                EXIT;
        END;

        nFromX     := 0;
        nFromY     := 0;

        bDragPiece := TRUE;

        nPi        := Chess.GetIndex(cPc);
        nPc        := Chess.GetColor(cPc);

        nPX        := PieceXY(nBX);
        nPY        := PieceXY(nBY);

        rRect      := Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize);

        nMXOff     := nMX - nPX;
        nMYOff     := nMY - nPY;

        IF (Chess.anPiece[nC, nP] > 1) THEN
        BEGIN
            { Copy }
            bTmp.Canvas.CopyMode := cmSrcCopy;
            bTmp.Canvas.CopyRect(rSqrRect, aSquare[3].Canvas, rSqrRect);

            if not usepiecebitmap then
            begin
            { And }
            bTmp.Canvas.CopyMode := cmSrcAnd;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[3, nPi].Canvas, rSqrRect);

            { Or }
            bTmp.Canvas.CopyMode := cmSrcPaint;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[nC, nPi].Canvas, rSqrRect);
            end
            else  bTmp.Canvas.draw(0,0, aImage[nC, nPi]);
            { Draw # of pieces remaining }
            IF (bShwPCCount) THEN
                DrawNumber(bTmp.Canvas, Chess.anPiece[nC, nP] - 1);

            CV.CopyRect(rRect, bTmp.Canvas, rSqrRect);
        END
        ELSE
            CV.CopyRect(rRect, aSquare[3].Canvas, rSqrRect);

        IF (bDragMove) THEN
        BEGIN
            { Copy }
            bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);
        END;

        { Draw piece }
        nDropX := nBX;
        nDropY := nBY;
        DrawDropPc(CV, nDropX, nDropY);
    END
    ELSE
    IF ((nBX > 8) AND (nBY > 5)) THEN    { Bottom }
    BEGIN
        IF (bFlip) THEN
        BEGIN
            IF (Chess.bWhitesMove) THEN
                EXIT;

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetBlackColor;
            cPc := Chess.acPiece[nC, nP];

            if ((bairdrop) and (cpc=EMPTY_SQUARE)) then cpc:=adroppiece[nc,np]; //attempt to allow aerial drops

            IF (NOT Chess.IsBlack(cPc)) THEN
                EXIT;
        END
        ELSE
        BEGIN
            IF (NOT Chess.bWhitesMove) THEN
                EXIT;

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetWhiteColor;
            cPc := Chess.acPiece[nC, nP];

            if ((bairdrop) and (cpc=EMPTY_SQUARE)) then cpc:=adroppiece[nc,np]; //attempt to allow aerial drops

            IF (NOT Chess.IsWhite(cPc)) THEN
                EXIT;
        END;

        nFromX     := 0;
        nFromY     := 0;

        bDragPiece := TRUE;

        nPi        := Chess.GetIndex(cPc);
        nPc        := Chess.GetColor(cPc);

        nPX        := PieceXY(nBX);
        nPY        := PieceXY(nBY);

        rRect      := Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize);

        nMXOff     := nMX - nPX;
        nMYOff     := nMY - nPY;

        IF (Chess.anPiece[nC, nP] > 1) THEN
        BEGIN
            { Copy }
            bTmp.Canvas.CopyMode := cmSrcCopy;
            bTmp.Canvas.CopyRect(rSqrRect, aSquare[3].Canvas, rSqrRect);
            if not usepiecebitmap then
            begin
            { And }
            bTmp.Canvas.CopyMode := cmSrcAnd;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[3, nPi].Canvas, rSqrRect);

            { Or }
            bTmp.Canvas.CopyMode := cmSrcPaint;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[nC, nPi].Canvas, rSqrRect);
            end
            else  bTmp.Canvas.draw(0,0, aImage[nC, nPi]);
            { Draw # of pieces remaining }
            IF (bShwPCCount) THEN
                DrawNumber(bTmp.Canvas, Chess.anPiece[nC, nP] - 1);

            CV.CopyRect(rRect, bTmp.Canvas, rSqrRect);
        END
        ELSE
            CV.CopyRect(rRect, aSquare[3].Canvas, rSqrRect);

        IF (bDragMove) THEN
        BEGIN
            { Copy }
            bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);
        END;

        { Draw piece }
        nDropX := nBX;
        nDropY := nBY;
        DrawDropPc(CV, nDropX, nDropY);
    END
    ELSE
    BEGIN                                { Board }
        IF (NOT Chess.IndexOk(nBX)) THEN
            EXIT;

        IF (NOT Chess.IndexOk(nBY)) THEN
            EXIT;

        IF (bFlip) THEN
        BEGIN
            nFromX := Chess.GetRevIndex(nBX);
            nFromY := nBY;
        END
        ELSE
        BEGIN
            nFromX := nBX;
            nFromY := Chess.GetRevIndex(nBY);
        END;

        cPc := Chess.aBoard[nFromX, nFromY];

        IF (Chess.bWhitesMove) THEN
        BEGIN
            IF (NOT Chess.IsWhite(cPc)) THEN
                EXIT;
        END
        ELSE
        BEGIN
            IF (NOT Chess.IsBlack(cPc)) THEN
                EXIT;
        END;

        { Check for smart move }
        IF (bSmartMove) THEN
        BEGIN
            { Generate all moves }
            Chess.GenMove;

            { Smart move }
            nMI := Chess.SmartMove(nFromX, nFromY);
            IF (nMI <> 0) THEN
            BEGIN
                IF (NOT bAlwaysQueen) THEN
                BEGIN
                    IF (Chess.IsQueen(Chess.aMove[nMI].cPromote)) THEN
                    BEGIN
                        cPromote := Chess.GetEmpty;
                      try
                        fpromote:=TFPromote.Create(nil);
                        fpromote.nLeft := nLeft + nMX - 15;
                        fPromote.nTop  := nTop  + nMY + 4;

                        fPromote.ShowModal;

                        IF (Chess.bWhitesMove) THEN
                        BEGIN
                            CASE (fPromote.nSelect) OF
                                1: cPromote := Chess.GetWhiteQueen;
                                2: cPromote := Chess.GetWhiteKnight;
                                3: cPromote := Chess.GetWhiteRook;
                                4: cPromote := Chess.GetWhiteBishop;
                            END;
                        END
                        ELSE
                        BEGIN
                            CASE (fPromote.nSelect) OF
                                1: cPromote := Chess.GetBlackQueen;
                                2: cPromote := Chess.GetBlackKnight;
                                3: cPromote := Chess.GetBlackRook;
                                4: cPromote := Chess.GetBlackBishop;
                            END;
                        END;
                      finally
                        fpromote.Free;
                      end;

                        IF (Chess.IsEmpty(cPromote)) THEN
                            EXIT;

                        nTmp := Chess.LegalMove(nFromX,
                                                nFromY,
                                                Chess.aMove[nMI].nToX,
                                                Chess.aMove[nMI].nToY,
                                                Chess.aBoard[nFromX, nFromY],
                                                cPromote);
                        IF (nTmp = 0) THEN
                            EXIT;

                        nMI := nTmp;
                    END;
                END;

                { Execute move }
                nTmp := Chess.ExecMove(nMI);

                { Draw board }
                DrawBoard(CV);

                { Set move drag flag }
                bDragPiece := FALSE;

                { Set move made flag }
                bMadeMove := TRUE;

                { Set move index }
                nMoveI := nMI;

                EXIT;
            END;
        END;

        bDragPiece := TRUE;

        nPi        := Chess.GetIndex(cPc);
        nPc        := Chess.GetColor(cPc);

        nPX        := PieceXY(nBX);
        nPY        := PieceXY(nBY);

        rRect      := Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize);

        nMXOff     := nMX - nPX;
        nMYOff     := nMY - nPY;

        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, aSquare[aSqrClr[nFromX, nFromY]].Canvas, rSqrRect);

            { Copy }
            bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);
        END;

        { Draw piece }
        nDropX := 0;
        nDropY := 0;
        DrawPiece(CV, nFromX, nFromY);
    END;

    { Check to see if click outline needs to be drawn }
    IF ((bOutlinePC) and (not usepiecebitmap))THEN
    BEGIN
        { Outline - copy }
        bTmp.Canvas.CopyRect(rSqrRect, CV, rRect);

        { Outline - draw }
        DrawPieceOutline(bTmp.Canvas);

        { Copy }
        CV.CopyRect(rRect, bTmp.Canvas, rSqrRect);
    END;
END;

PROCEDURE TBoard.OnMouseMove(CV: TCanvas; nMX, nMY: Integer);
VAR
    nPX, nPY, nPX1, nPY1: Integer;
BEGIN
    IF (NOT bDragPiece) THEN
        EXIT;

    if ((npc<1) or (npi<1)) then exit; 
    nPX  := nMX - nMXOff;
    nPY  := nMY - nMYOff;
    nPX1 := nPX + nPieceSize;
    nPY1 := nPY + nPieceSize;

    IF (NOT bDragMove) THEN
        EXIT;

    { Copy }
    CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);

    { Assign }
    rRect := Rect(nPX, nPY, nPX1, nPY1);

    { Copy }
    bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);

    { Copy }
    bWorkSpace.Canvas.CopyMode := cmSrcCopy;
    bWorkSpace.Canvas.CopyRect(rSqrRect, CV, rRect);
    if not usepiecebitmap then
    begin
    { And }
    bWorkSpace.Canvas.CopyMode := cmSrcAnd;
    bWorkSpace.Canvas.CopyRect(rSqrRect, aImage[3, nPi].Canvas, rSqrRect);

    { Or }
    bWorkSpace.Canvas.CopyMode := cmSrcPaint;
    bWorkSpace.Canvas.CopyRect(rSqrRect, aImage[nPc, nPi].Canvas, rSqrRect);
    end
    else bWorkSpace.Canvas.draw(0,0, aImage[nPc, nPi]);
    { Check to see if click outline needs to be drawn }
    IF ((bOutlinePC) and (not usepiecebitmap)) THEN
    BEGIN
        { Outline - draw }
        DrawPieceOutline(bWorkSpace.Canvas);
    END;

    { Copy }
    CV.CopyRect(rRect, bWorkSpace.Canvas, rSqrRect);
END;

PROCEDURE TBoard.OnMouseUp(CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
VAR
    nPX, nPY, nPX1, nPY1, nRevPc, nMI, nTmp: Integer;
    var fpromote:Tfpromote;
BEGIN
    IF (NOT bDragPiece) THEN
        EXIT;

    nPX  := nMX - nMXOff;
    nPY  := nMY - nMYOff;
    nPX1 := nPX + nPieceSize;
    nPY1 := nPY + nPieceSize;

    IF (bDragMove) THEN
        nMX := ((nPX1 - nPX) DIV 2) + nPX;

    nToX := IndexXY(nMX);

    IF (NOT Chess.IndexOk(nToX)) THEN
    BEGIN
        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
        END;

        { Draw piece }
        IF (nDropY = 0) THEN
            DrawPiece (CV, nFromX, nFromY)
        ELSE
            DrawDropPc(CV, nDropX, nDropY);

        bDragPiece := FALSE;
        EXIT;
    END;

    IF (bDragMove) THEN
        nMY := ((nPY1 - nPY) DIV 2) + nPY;

    nToY := IndexXY(nMY);

    IF (NOT Chess.IndexOk(nToY)) THEN
    BEGIN
        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
        END;

        { Draw piece }
        IF (nDropY = 0) THEN
            DrawPiece (CV, nFromX, nFromY)
        ELSE
            DrawDropPc(CV, nDropX, nDropY);

        bDragPiece := FALSE;
        EXIT;
    END;

    IF (bFlip) THEN
        nToX := Chess.GetRevIndex(nToX)
    ELSE
        nToY := Chess.GetRevIndex(nToY);

    { Reset pawn promotion }
    cPromote := Chess.GetEmpty;

    IF (nDropY = 0) THEN
    BEGIN
        IF (Chess.IsPawn(cPc)) THEN
        BEGIN
            IF (Chess.IsPromoteRank(nPc, nToY)) THEN
            BEGIN
                IF (bAlwaysQueen) THEN
                BEGIN
                    IF (Chess.bWhitesMove) THEN
                        cPromote := Chess.GetWhiteQueen
                    ELSE
                        cPromote := Chess.GetBlackQueen;
                END
                ELSE
                BEGIN
                try
                    fpromote:=tfpromote.Create(nil);
                    fPromote.nLeft := nLeft + nMX - 15;
                    fPromote.nTop  := nTop  + nMY + 4;

                    fPromote.ShowModal;

                    IF (Chess.bWhitesMove) THEN
                    BEGIN
                        CASE (fPromote.nSelect) OF
                            1: cPromote := Chess.GetWhiteQueen;
                            2: cPromote := Chess.GetWhiteKnight;
                            3: cPromote := Chess.GetWhiteRook;
                            4: cPromote := Chess.GetWhiteBishop;
                        END;
                    END
                    ELSE
                    BEGIN
                        CASE (fPromote.nSelect) OF
                            1: cPromote := Chess.GetBlackQueen;
                            2: cPromote := Chess.GetBlackKnight;
                            3: cPromote := Chess.GetBlackRook;
                            4: cPromote := Chess.GetBlackBishop;
                        END;
                    END;
                finally
                    fpromote.Free;
                end;

                    IF (Chess.IsEmpty(cPromote)) THEN
                    BEGIN
                        IF (bDragMove) THEN
                        BEGIN
                            { Copy }
                            CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
                        END;

                        { Draw piece }
                        DrawPiece(CV, nFromX, nFromY);

                        bDragPiece := FALSE;
                        EXIT;
                    END;
                END;
            END;
        END;
    END
    ELSE
    BEGIN
        IF (NOT Chess.IsEmpty(Chess.aBoard[nToX, nToY])) THEN
        BEGIN
            IF (bDragMove) THEN
            BEGIN
                { Copy }
                CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
            END;
            
            { Draw piece }
            DrawDropPc(CV, nDropX, nDropY);

            bDragPiece := FALSE;
            EXIT;
        END;

        nPc    := Chess.GetColor(cPc);
        nRevPc := Chess.GetRevColor(cPc);

        IF (Chess.IsPawn(cPc)) THEN
        BEGIN
            IF ((Chess.IsPromoteRank(nPc,    nToY))  OR
                (Chess.IsPromoteRank(nRevPc, nToY))) THEN
            BEGIN
                IF (bDragMove) THEN
                BEGIN
                    { Copy }
                    CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
                END;
                
                { Draw piece }
                DrawDropPc(CV, nDropX, nDropY);

                bDragPiece := FALSE;
                EXIT;
            END;
        END;
    END;

    { Generate all legal moves }
    Chess.GenMove;

    { Check to see if it is a legal move }
    nMI := Chess.LegalMove(nFromX, nFromY, nToX, nToY, cPc, cPromote);
    IF (nMI = 0) THEN
    BEGIN
        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
        END;

        { Draw piece }
        IF (nDropY = 0) THEN
            DrawPiece (CV, nFromX, nFromY)
        ELSE
            DrawDropPc(CV, nDropX, nDropY);

        bDragPiece := FALSE;
        EXIT;
    END;

    { Execute move }
    nTmp := Chess.ExecMove(nMI);

    { Draw board }
    DrawBoard(CV);

    { Set move drag flag }
    bDragPiece := FALSE;

    { Set move made flag }
    bMadeMove := TRUE;

    { Set move index }
    nMoveI := nMI;
END;

procedure tboard.chesstopixels(square:integer;var x,y:integer;cmove:string);
// converts square in chess coordinates to a top left corner of that square in pixels
// also works for drops (returns the top left corner of the corresponding piece in the holding)
var nbx,nby:integer;
piece:char;
begin
 if square=0 then
 begin
  if chess.bWhitesMove then cmove:=uppercase(cmove) else cmove:=lowercase(cmove);
  if length(cmove)<1 then exit else piece:=cmove[1];

  nby:=chess.GetColor(piece);
  nbx:=chess.GetIndex(piece);
  dropchesstoboard(nby,nbx,nbx,nby);
  X := PieceXY(nBX)+1;
  Y := PieceXY(nBY)+1;
  exit;
 end;

 inttoxy(square,nbx,nby);
 IF (bFlip) THEN
    BEGIN
        X := PieceXY(Chess.GetRevIndex(nBX));
        Y := PieceXY(nBY);
    END
    ELSE
    BEGIN
        X := PieceXY(nBX);
        Y := PieceXY(Chess.GetRevIndex(nBY));
    END;


end;

procedure tboard.dropchesstoboard(npc,npi:integer;var x,y:integer);
// returns the coordinates of piece acPiece[npc,npi] in holdings array
//in terms of chessboard: x=9..10; y=1..8
begin
    case npi of
    PAWN_INDEX: begin x:=9; y:=1; end;
    ROOK_INDEX: begin x:=9; y:=2; end;
    KNIGHT_INDEX: begin x:=9; y:=3; end;
    BISHOP_INDEX: begin x:=10; y:=1; end;
    QUEEN_INDEX: begin x:=10; y:=2; end;
    end;

    if (((npc=WHITE_COLOR) and (not bflip)) or ((npc=BLACK_COLOR) and (bflip)))
    then inc(y,5);
end;

PROCEDURE   tboard.SMouseDown(CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
VAR
    nBX, nBY, nPX, nPY, nP, nC, nMI, nTmp: Integer;
    blah:integer;
BEGIN
    IF (bDragPiece) THEN
        EXIT;

    nBX := IndexXY(nMX);
    nBY := IndexXY(nMY);

    IF ((nBX > 8) AND (nBY < 4)) THEN    { Top }
    BEGIN
        IF (bFlip) THEN
        BEGIN

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetWhiteColor;
            cPc := Chess.acPiece[nC, nP];

            IF (NOT Chess.IsWhite(cPc)) THEN
                EXIT;
        END
        ELSE
        BEGIN

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetBlackColor;
            cPc := Chess.acPiece[nC, nP];

            IF (NOT Chess.IsBlack(cPc)) THEN
                EXIT;
        END;

        nFromX     := 0;
        nFromY     := 0;

        bDragPiece := TRUE;

        nPi        := Chess.GetIndex(cPc);
        nPc        := Chess.GetColor(cPc);

        nPX        := PieceXY(nBX);
        nPY        := PieceXY(nBY);

        rRect      := Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize);

        nMXOff     := nMX - nPX;
        nMYOff     := nMY - nPY;

            { Copy }
            bTmp.Canvas.CopyMode := cmSrcCopy;
            bTmp.Canvas.CopyRect(rSqrRect, aSquare[3].Canvas, rSqrRect);

            if not usepiecebitmap then
            begin
            { And }
            bTmp.Canvas.CopyMode := cmSrcAnd;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[3, nPi].Canvas, rSqrRect);

            { Or }
            bTmp.Canvas.CopyMode := cmSrcPaint;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[nC, nPi].Canvas, rSqrRect);
            end
            else  bTmp.Canvas.draw(0,0, aImage[nC, nPi]);
            { Draw # of pieces remaining }
            IF (bShwPCCount) THEN
                DrawNumber(bTmp.Canvas, Chess.anPiece[nC, nP] - 1);

            CV.CopyRect(rRect, bTmp.Canvas, rSqrRect);


        IF (bDragMove) THEN
        BEGIN
            { Copy }
            bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);
        END;

        { Draw piece }
        nDropX := nBX;
        nDropY := nBY;
        DrawDropPc(CV, nDropX, nDropY);
    END
    ELSE
    IF ((nBX > 8) AND (nBY > 5)) THEN    { Bottom }
    BEGIN
        IF (bFlip) THEN
        BEGIN

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetBlackColor;
            cPc := Chess.acPiece[nC, nP];

            IF (NOT Chess.IsBlack(cPc)) THEN
                EXIT;
        END
        ELSE
        BEGIN

            nP := GetDropIndex(nBX, nBY);
            IF (nP = 0) THEN
                EXIT;

            nC  := Chess.GetWhiteColor;
            cPc := Chess.acPiece[nC, nP];

            IF (NOT Chess.IsWhite(cPc)) THEN
                EXIT;
        END;

        nFromX     := 0;
        nFromY     := 0;

        bDragPiece := TRUE;

        nPi        := Chess.GetIndex(cPc);
        nPc        := Chess.GetColor(cPc);

        nPX        := PieceXY(nBX);
        nPY        := PieceXY(nBY);

        rRect      := Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize);

        nMXOff     := nMX - nPX;
        nMYOff     := nMY - nPY;

            { Copy }
            bTmp.Canvas.CopyMode := cmSrcCopy;
            bTmp.Canvas.CopyRect(rSqrRect, aSquare[3].Canvas, rSqrRect);
            if not usepiecebitmap then
            begin
            { And }
            bTmp.Canvas.CopyMode := cmSrcAnd;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[3, nPi].Canvas, rSqrRect);

            { Or }
            bTmp.Canvas.CopyMode := cmSrcPaint;
            bTmp.Canvas.CopyRect(rSqrRect, aImage[nC, nPi].Canvas, rSqrRect);
            end
            else  bTmp.Canvas.draw(0,0, aImage[nC, nPi]);
            { Draw # of pieces remaining }
            IF (bShwPCCount) THEN
                DrawNumber(bTmp.Canvas, Chess.anPiece[nC, nP] - 1);

            CV.CopyRect(rRect, bTmp.Canvas, rSqrRect);

        IF (bDragMove) THEN
        BEGIN
            { Copy }
            bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);
        END;

        { Draw piece }
        nDropX := nBX;
        nDropY := nBY;
        DrawDropPc(CV, nDropX, nDropY);
    END
    ELSE
    BEGIN                                { Board }
        IF (NOT Chess.IndexOk(nBX)) THEN
            EXIT;

        IF (NOT Chess.IndexOk(nBY)) THEN
            EXIT;

        IF (bFlip) THEN
        BEGIN
            nFromX := Chess.GetRevIndex(nBX);
            nFromY := nBY;
        END
        ELSE
        BEGIN
            nFromX := nBX;
            nFromY := Chess.GetRevIndex(nBY);
        END;

        cPc := Chess.aBoard[nFromX, nFromY];



        bDragPiece := TRUE;

        nPi        := Chess.GetIndex(cPc);
        nPc        := Chess.GetColor(cPc);

        nPX        := PieceXY(nBX);
        nPY        := PieceXY(nBY);

        rRect      := Rect(nPX, nPY, nPX + nPieceSize, nPY + nPieceSize);

        nMXOff     := nMX - nPX;
        nMYOff     := nMY - nPY;

        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, aSquare[aSqrClr[nFromX, nFromY]].Canvas, rSqrRect);

            { Copy }
            bSaveBack.Canvas.CopyRect(rSqrRect, CV, rRect);
        END;

        { Draw piece }
        nDropX := 0;
        nDropY := 0;
        DrawPiece(CV, nFromX, nFromY);
    END;

    { Check to see if click outline needs to be drawn }
    IF ((bOutlinePC) and (not usepiecebitmap))THEN
    BEGIN
        { Outline - copy }
        bTmp.Canvas.CopyRect(rSqrRect, CV, rRect);

        { Outline - draw }
        DrawPieceOutline(bTmp.Canvas);

        { Copy }
        CV.CopyRect(rRect, bTmp.Canvas, rSqrRect);
    END;

end;

PROCEDURE   tboard.SMouseMove(CV: TCanvas; nMX, nMY: Integer);
begin
 nmx:=nmy;
end;

PROCEDURE   tboard.SMouseUp  (CV: TCanvas; nMX, nMY, nLeft, nTop: Integer);
VAR
    nPX, nPY, nPX1, nPY1, nRevPc, nMI, nTmp: Integer;
BEGIN
    IF (NOT bDragPiece) THEN
        EXIT;

    nPX  := nMX - nMXOff;
    nPY  := nMY - nMYOff;
    nPX1 := nPX + nPieceSize;
    nPY1 := nPY + nPieceSize;

    IF (bDragMove) THEN
        nMX := ((nPX1 - nPX) DIV 2) + nPX;

    nToX := IndexXY(nMX);

    IF (NOT Chess.IndexOk(nToX)) THEN
    BEGIN
        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
        END;

        { Draw piece }
        IF (nDropY = 0) THEN
            DrawPiece (CV, nFromX, nFromY)
        ELSE
            DrawDropPc(CV, nDropX, nDropY);

        bDragPiece := FALSE;
        EXIT;
    END;

    IF (bDragMove) THEN
        nMY := ((nPY1 - nPY) DIV 2) + nPY;

    nToY := IndexXY(nMY);

    IF (NOT Chess.IndexOk(nToY)) THEN
    BEGIN
        IF (bDragMove) THEN
        BEGIN
            { Copy }
            CV.CopyRect(rRect, bSaveBack.Canvas, rSqrRect);
        END;

        { Draw piece }
        IF (nDropY = 0) THEN
            DrawPiece (CV, nFromX, nFromY)
        ELSE
            DrawDropPc(CV, nDropX, nDropY);

        bDragPiece := FALSE;
        EXIT;
    END;

    IF (bFlip) THEN
        nToX := Chess.GetRevIndex(nToX)
    ELSE
        nToY := Chess.GetRevIndex(nToY);



        nPc    := Chess.GetColor(cPc);
        nRevPc := Chess.GetRevColor(cPc);



    { Execute move }
    chess.aBoard[nTox,ntoy]:=cpc;
    if (nFromX>0) then begin
     if chess.aBoard[nTox,nToy]<>Empty_square then chess.aPromote[nTox,nToy]:=chess.aPromote[nFromx,nFromy];
     chess.aBoard[nfromx,nfromY]:=Empty_square;
     chess.aPromote[nFromx,nFromy]:=false;
    end else chess.aPromote[nTox,nToY]:=false;

    { Set move drag flag }
    bDragPiece := FALSE;


    { Draw board }
    DrawBoard(CV);
END;

function tboard.isOnBoard(x,y:integer):integer;
var nFX,nFY:integer;
begin
      Result:=0;
      nfx:=Indexxy(x);
      nfy:=indexxy(y);
      IF (bFlip) THEN
        nFX := Chess.GetRevIndex(nFX)
      ELSE
        nFY := Chess.GetRevIndex(nFY);
      if not (nfx in [1..8]) then exit;
      if not (nfy in [1..8]) then exit;

      with chess do
      if ((aBoard[nFX,nFY]=EMPTY_SQUARE) or
      (aBoard[nFX,nFY]=WHITE_KING) or
      (aBoard[nFX,nFY]=BLACK_KING) or
      (aBoard[nFX,nFY]=WHITE_PAWN) or
      (aboard[nfx,nfy]=Black_pawn)) then
      begin Result:=1; exit;end;

      if chess.aPromote[nFx,nFY] then Result:=3
      else result:=2;
end;

procedure tboard.DropPopupPiece(cpc:char;CV:TCanvas);
begin
  IF (bFlip) THEN
     nPopupDropX := Chess.GetRevIndex(nPopupDropX)
  ELSE
     nPopupDropY := chess.GetRevIndex(nPopupDropY);
  Chess.aBoard[nPopupDropx,nPopupdropY]:=cpc;
  DrawBoard(CV);
end;

Procedure tboard.switchPromoted(nx,ny:integer);
begin
 IF (bFlip) THEN
        nX := Chess.GetRevIndex(nX)
      ELSE
        nY := Chess.GetRevIndex(nY);
 chess.aPromote[nx,ny]:=not chess.aPromote[nx,ny];
end;
END. { BOARD }
