(*============================================================================*)
(* BOARD1.PAS -- Board #1 window                                              *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: Added bShowSOutline into board object.                         *)
(* 2002.01   : Replaced standard Delphi components with fancier custom ones:  *)
(*             Game Navigation: TSpeedButton --> TLbButton, TRepeatLbButton   *)
(*             Handles,Clocks: TLabel --> TLbButton                           *)
(*             Reason -- Standard components didn't work with bitmap graphics *)
(*             probably because of MS bug in TransparentBlt function or Delphi*)
(*             bug in its analog                                              *)
// 2004.12.23  1) Added gonormal, gosetup that switch setup and normal mode
//             2) Added goclear that clears the board in setup mode
// 2005.02.10  1) Made Board2Form of Tboard1form type. Added goBoard1/goBoard2
//                that set captions etc properly for each board. Modified Resetall
//             2) Added support for lag: label5 and label 6 that display lag,
//                nWlag,nblag that store cumulative lag and nLag for current lag,
//                ShowMove lag that governs lag display
// 2005.02.17  Added MakeMove that replaced displayboard during replay and is faster
// 2005.02.22  Fixed Bug in MakeMove: incorrect refresh after castling
// 2005.02.28  Modified ShowMoveLag to show time in ss.d format
// 2005.03.01  Moved lag display to the left of the board so that board resizes better
//             Apparently when board was small, lag covered handles. Modified AdjustClocks
(*============================================================================*)
UNIT BOARD1;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    ExtCtrls, StdCtrls, Buttons,
    GLOBAL, TOOLBOX, BOARD,INI1, Lbspeedbutton, LbButton, RepeatBtn,misc;

TYPE
    TBoard1Form = CLASS(TForm)
        PaintBox1: TPaintBox;
        PaintBox2: TPaintBox;
        PaintBox3: TPaintBox;
        PaintBox4: TPaintBox;
        PaintBox5: TPaintBox;
    Label1: TLbButton;
    Label2: TLbButton;
    Label3: TLbButton;
    Label4: TLbButton;
    SpeedButton1: TLbButton;
    SpeedButton2: TRepeatLbButton;
    SpeedButton4: TLbButton;
    SpeedButton3: TRepeatLbButton;
    label5: TLbButton;
    Label6: TLbButton;

        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE LoadHandle;
        PROCEDURE LoadTime;
        PROCEDURE ResetAll;
        PROCEDURE FormActivate(Sender: TObject);

        PROCEDURE DisplayBoard;
        PROCEDURE FormPaint(Sender: TObject);
        PROCEDURE AdjustBoards;
        PROCEDURE AdjustOutlines;
        PROCEDURE AdjustClocks;
        PROCEDURE AdjustHandles;
        PROCEDURE AdjustSpeeds;
        PROCEDURE FormResize(Sender: TObject);
        procedure Formdestroy(Sender: Tobject);
        procedure gosetup;
        procedure goclear(Sender:Tobject);
        procedure undoClear;
        procedure GoBoard1;
        procedure GoBoard2;
        procedure gonormal;
        procedure ShowMoveLag;
        procedure MakeMove(nFromX,nFromY,nToX,nToY:integer;cDrop:char);
        procedure SetClockColor;

        PRIVATE
        { Private declarations }
        function needShowLag:boolean;


         procedure bitmappaint;
        PUBLIC
        { Public declarations }
        bActive: Boolean;
        bResize: Boolean;
        bActual: Boolean;

        nSpeedW: Integer;
        nBoard8: Integer;

        Board: TBoard;

        nWaitF: TColor;
        nWaitB: TColor;
        nTurnF: TColor;
        nTurnB: TColor;

        cWHandle: String;
        cBHandle: String;
        cCaption: String;

        cWTime: String;
        cBTime: String;
        nBLag :longint;
        nWLag :longint;
        nLag: longint;
        nWtime: longint;
        nBtime: longint;
    END;

VAR
    Board1Form: TBoard1Form;
    Board2Form: Tboard1Form;
IMPLEMENTATION

{$R *.DFM}

procedure TBoard1Form.FormDestroy;
begin
 //Bitmap.Free;
end;

PROCEDURE TBoard1Form.FormCreate(Sender: TObject);
BEGIN
//    WError('Creating Board');

    bActive := TRUE;
    bResize := FALSE;
    bActual := INI1.INI.bActualBoard;

    nSpeedW := SpeedButton1.Width + SpeedButton1.Width +
               SpeedButton1.Width + SpeedButton1.Width;

    Board := TBoard.Create;
    Board.Init;
    Board.bFlip := INI1.INI.bFlipBd;

    Board.nWhtFill     := INI1.INI.nWhtFill;
    Board.nWhtOutline  := INI1.INI.nWhtOutline;
    Board.nWhtSquare   := INI1.INI.nWhtSquare;
    Board.nWhtSOutline := INI1.INI.nWhtSOutline;

    Board.nBlkFill     := INI1.INI.nBlkFill;
    Board.nBlkOutline  := INI1.INI.nBlkOutline;
    Board.nBlkSquare   := INI1.INI.nBlkSquare;
    Board.nBlkSOutline := INI1.INI.nBlkSOutline;

    Board.bOutlinePC   := INI1.INI.bOutlinePC;
    Board.nClickColor  := INI1.INI.nClickColor;
    Board.bShwPCCount  := INI1.INI.bShwPCCount;
    Board.nNumberColor := INI1.INI.nNumberColor;
    Board.nPcSquare    := INI1.INI.nPcSquare;

    Board.bAlwaysQueen := INI1.INI.bAlwaysQ;
    Board.bDragMove    := INI1.INI.bDragMove;
    Board.bSmartMove   := INI1.INI.bSmartMove;
    Board.bShowSOutline:= INI1.INI.bShowSOutline;
    Color              := INI1.INI.nBoardWindow;

    PaintBox2.Canvas.Pen.Color   := INI1.INI.nBoardOutline;
    PaintBox2.Canvas.Brush.Color := INI1.INI.nBoardOutline;
    PaintBox2.Canvas.Brush.Style := bsSolid;

    PaintBox3.Canvas.Pen.Color   := INI1.INI.nBoardOutline;
    PaintBox3.Canvas.Brush.Color := INI1.INI.nBoardOutline;
    PaintBox3.Canvas.Brush.Style := bsSolid;

    PaintBox4.Canvas.Pen.Color   := INI1.INI.nBoardOutline;
    PaintBox4.Canvas.Brush.Color := INI1.INI.nBoardOutline;
    PaintBox4.Canvas.Brush.Style := bsSolid;

    PaintBox5.Canvas.Pen.Color   := INI1.INI.nBoardOutline;
    PaintBox5.Canvas.Brush.Color := INI1.INI.nBoardOutline;
    PaintBox5.Canvas.Brush.Style := bsSolid;

    nWaitF := INI1.INI.nWaitF;
    nWaitB := INI1.INI.nWaitB;
    nTurnF := INI1.INI.nTurnF;
    nTurnB := INI1.INI.nTurnB;

    Label3.Font.Name   := INI1.INI.fClock.Name;
    Label3.Font.Height := INI1.INI.fClock.Height;
    Label3.Font.Pitch  := INI1.INI.fClock.Pitch;
    Label3.Font.Size   := INI1.INI.fClock.Size;
    Label3.Font.Style  := INI1.INI.fClock.Style;

    Label4.Font.Name   := INI1.INI.fClock.Name;
    Label4.Font.Height := INI1.INI.fClock.Height;
    Label4.Font.Pitch  := INI1.INI.fClock.Pitch;
    Label4.Font.Size   := INI1.INI.fClock.Size;
    Label4.Font.Style  := INI1.INI.fClock.Style;

    Label5.Font.Name   := INI1.INI.fClock.Name;
    Label5.Font.Height := INI1.INI.fClock.Height;
    Label5.Font.Pitch  := INI1.INI.fClock.Pitch;
    Label5.Font.Size   := INI1.INI.fClock.Size;
    Label5.Font.Style  := INI1.INI.fClock.Style;
    label5.Font.Color  := Ini1.INI.nWaitF;
    label5.Color       := ini1.INI.nWaitB;

    Label6.Font.Name   := INI1.INI.fClock.Name;
    Label6.Font.Height := INI1.INI.fClock.Height;
    Label6.Font.Pitch  := INI1.INI.fClock.Pitch;
    Label6.Font.Size   := INI1.INI.fClock.Size;
    Label6.Font.Style  := INI1.INI.fClock.Style;
    label6.Font.Color  := Ini1.INI.nWaitF;
    label6.Color       := ini1.INI.nWaitB;


    Label1.Color       := INI1.INI.nHandleB;
    Label1.Font.Color  := INI1.INI.fHandle.Color;
    Label1.Font.Name   := INI1.INI.fHandle.Name;
    Label1.Font.Height := INI1.INI.fHandle.Height;
    Label1.Font.Pitch  := INI1.INI.fHandle.Pitch;
    Label1.Font.Size   := INI1.INI.fHandle.Size;
    Label1.Font.Style  := INI1.INI.fHandle.Style;

    Label2.Color       := INI1.INI.nHandleB;
    Label2.Font.Color  := INI1.INI.fHandle.Color;
    Label2.Font.Name   := INI1.INI.fHandle.Name;
    Label2.Font.Height := INI1.INI.fHandle.Height;
    Label2.Font.Pitch  := INI1.INI.fHandle.Pitch;
    Label2.Font.Size   := INI1.INI.fHandle.Size;
    Label2.Font.Style  := INI1.INI.fHandle.Style;
END;

PROCEDURE TBoard1Form.LoadHandle;
BEGIN
    IF (Board.bFlip) THEN
    BEGIN
        Label1.Caption := cWHandle;
        Label2.Caption := cBHandle;
    END
    ELSE
    BEGIN
        Label1.Caption := cBHandle;
        Label2.Caption := cWHandle;
    END;
END;

PROCEDURE TBoard1Form.LoadTime;
BEGIN
    IF (Board.bFlip) THEN
    BEGIN
        Label3.Caption := cWTime;
        Label4.Caption := cBTime;
    END
    ELSE
    BEGIN
        Label3.Caption := cBTime;
        Label4.Caption := cWTime;
    END;
END;

PROCEDURE TBoard1Form.ResetAll;
BEGIN
    Caption  := cCaption;

    cWHandle := PLAYERW_NAME;
    cBHandle := PLAYERB_NAME;

    cWTime := TIMEW_STR;
    cBTime := TIMEB_STR;
    nWLag  := 0;
    nBlag  :=0;
    nLag   :=0;

    LoadHandle;
    LoadTime;

    Board.bDisable := TRUE;
    Board.Chess.ResetBoard;

    SpeedButton1.Enabled := FALSE;
    SpeedButton2.Enabled := FALSE;
    SpeedButton3.Enabled := FALSE;
    SpeedButton4.Enabled := FALSE;
END;

PROCEDURE TBoard1Form.FormActivate(Sender: TObject);
BEGIN
    IF (bActive) THEN
    BEGIN
        bActive := FALSE;
        bResize := TRUE;
        FormResize(Sender);
        FormPaint(Sender);
    END;
END;

PROCEDURE TBoard1Form.DisplayBoard;
BEGIN
    PaintBox2.Canvas.MoveTo(0, 0);
    PaintBox2.Canvas.LineTo(PaintBox2.Width - 1, 0);

    PaintBox3.Canvas.MoveTo(0, 0);
    PaintBox3.Canvas.LineTo(0, PaintBox3.Height - 1);

    PaintBox4.Canvas.MoveTo(0, 0);
    PaintBox4.Canvas.LineTo(PaintBox4.Width - 1, 0);

    PaintBox5.Canvas.MoveTo(0, 0);
    PaintBox5.Canvas.LineTo(0, PaintBox5.Height - 1);

    SetClockColor;
    Board.DrawBoard(PaintBox1.Canvas);
END;

procedure Tboard1form.bitmappaint;
var x,y:integer;
BEGIN
 y := 0;
  while y < Height do
  begin
    x := 0;
    while x < Width do
    begin
      Canvas.Draw(x, y, piecesbg);
      x := x + piecesbg.Width;
    end;
    y := y + piecesbg.Height;
  end;
end;

PROCEDURE TBoard1Form.FormPaint(Sender: TObject);
begin
 if usebitmap then bitmappaint;
    DisplayBoard;
END;

PROCEDURE TBoard1Form.AdjustBoards;
VAR
    nCW, nCH, nSW, nSH, nW, nH, nL, nT, nBlockH, nZZ: Integer;
BEGIN
    { Adjust paint boxes }
    nCW := ClientWidth - 1 - 1;

    IF (Label1.Height > Label3.Height) THEN
    BEGIN
        nBlockH := Label1.Height + 1 + 1 + Label2.Height + HEIGHT_GAP + SpeedButton1.Height;
        nZZ := Label1.Height + 1;
    END
    ELSE
    BEGIN
        nBlockH := Label3.Height + 1 + 1 + Label4.Height + HEIGHT_GAP + SpeedButton1.Height;
        nZZ := Label3.Height + 1;
    END;

    nCH := ClientHeight - nBlockH;

    nSW := nCW DIV 10;
    nSH := nCH SHR 3;

    IF (nSW > nSH) THEN
    BEGIN
        nW := nSH * 10;
        nH := nSH SHL 3;

        Board.nPieceSize := nSH;
    END
    ELSE
    BEGIN
        nW := nSW * 10;
        nH := nSW SHL 3;

        Board.nPieceSize := nSW;
    END;
    nCH := Board.nPieceSize SHL 3;

    IF (bActual) THEN
    BEGIN
        nL := 1;
        nT := nZZ;
    END
    ELSE
    BEGIN
        nL := (ClientWidth - nCW) SHR 1;
        nT := ((ClientHeight - nBlockH - nH) SHR 1) + nZZ;
    END;

    PaintBox1.SetBounds(nL, nT, nW, nH);
    Board.nPieceRatio := Board.nPieceSize / 48;
    Board.LoadPiece;

    nBoard8 := nH;

    IF (bActual) THEN
    BEGIN
        ClientWidth  := nW + 2;
        ClientHeight := nH + nBlockH;
    END;
END;

PROCEDURE TBoard1Form.AdjustOutlines;
BEGIN
    PaintBox2.SetBounds(PaintBox1.Left  - 1,
                        PaintBox1.Top   - 1,
                        PaintBox1.Width + 2,
                        1);

    PaintBox3.SetBounds(PaintBox1.Left + PaintBox1.Width,
                        PaintBox1.Top - 1,
                        1,
                        PaintBox1.Height + 3);

    PaintBox4.SetBounds(PaintBox2.Left,
                        PaintBox1.Top + PaintBox1.Height,
                        PaintBox2.Width,
                        1);

    PaintBox5.SetBounds(PaintBox1.Left - 1,
                        PaintBox1.Top - 1,
                        1,
                        PaintBox1.Height + 3);
END;

PROCEDURE TBoard1Form.AdjustClocks;
BEGIN
    Label3.Left := PaintBox1.Left + nBoard8 - Label3.Width;
    Label3.Top  := PaintBox1.Top  - Label3.Height - 1;
    label5.Left := PaintBox1.Left+nBoard8;
    label5.Top  := label3.Top;

    Label4.Left := PaintBox1.Left + nBoard8 - Label4.Width;
    Label4.Top  := PaintBox1.Top  + nBoard8 + 1;
    label6.Left := label5.Left;
    label6.Top  := label4.Top;
END;

PROCEDURE TBoard1Form.AdjustHandles;
BEGIN
    Label1.Left := PaintBox1.Left + ((nBoard8 - Label1.Width) SHR 1);
    Label1.Top  := PaintBox1.Top  - Label1.Height - 1;

    IF ((Label1.Left + Label1.Width) >= Label3.Left) THEN
        Label1.Left := Label1.Left - ((Label1.Left + Label1.Width) - Label3.Left) - WIDTH_GAP;

    Label2.Left := PaintBox1.Left + ((nBoard8 - Label2.Width) SHR 1);
    Label2.Top  := PaintBox1.Top  + nBoard8 + 1;

    IF ((Label2.Left + Label2.Width) >= Label4.Left) THEN
        Label2.Left := Label2.Left - ((Label2.Left + Label2.Width) - Label4.Left) - WIDTH_GAP;
END;

PROCEDURE TBoard1Form.AdjustSpeeds;
BEGIN
    SpeedButton1.Left := PaintBox1.Left + ((nBoard8 - nSpeedW) SHR 1);
    SpeedButton1.Top  := Label2.Top + Label2.Height + HEIGHT_GAP;

    SpeedButton2.Left := SpeedButton1.Left + SpeedButton1.Width;
    SpeedButton2.Top  := SpeedButton1.Top;

    SpeedButton3.Left := SpeedButton2.Left + SpeedButton2.Width;
    SpeedButton3.Top  := SpeedButton1.Top;

    SpeedButton4.Left := SpeedButton3.Left + SpeedButton3.Width;
    SpeedButton4.Top  := SpeedButton1.Top;
END;

PROCEDURE TBoard1Form.FormResize(Sender: TObject);
BEGIN
    IF (bResize) THEN
    BEGIN
        AdjustBoards;
        AdjustOutlines;
        AdjustClocks;
        AdjustHandles;
        AdjustSpeeds;
    END;
END;

procedure tboard1form.gosetup;
begin
        with board.Chess do
        begin
         cSaveBoard:=getBoardStr;
         cSaveWPieces:=GetWhitePieceStr;
         cSaveBPieces:=GetBlackPieceStr;
        end;
        Board.Chess.SetPieces('QRBNP','qrbnp');
        board.DrawBoard(paintbox1.Canvas);
        Board.bDisable:=false;
        Label1.Caption:='Clear board';
        Label2.Caption:='Done';
        label1.OnClick:=goclear;
        label3.Visible:=false;
        label4.Visible:=false;
        label5.Visible:=false;
        label6.Visible:=false;
        Caption:='Setup mode: click Done to return to normal mode';
        speedbutton1.Visible:=false;
        speedbutton2.Visible:=false;
        speedbutton3.Visible:=false;
        speedbutton4.Visible:=false;
end;

procedure tboard1form.goclear(Sender:Tobject);
begin
  board.chess.goclear;
  board.DrawBoard(Paintbox1.Canvas);
end;

procedure tboard1form.undoClear;
begin
 board.Chess.undoSetup;
 board.DrawBoard(Paintbox1.Canvas);
end;

procedure tboard1form.GoBoard1;
begin
    ResetAll;
    IF (INI1.INI.nBoard1S <> 2) THEN
    BEGIN
        SetBounds(INI1.INI.nBoard1L, INI1.INI.nBoard1T,
                  INI1.INI.nBoard1W, INI1.INI.nBoard1H);
    END;
    WindowState := TOOLBOX_IntToWinState(INI1.INI.nBoard1S);
    cCaption := Board1_title;
    Caption     := cCaption;
end;

procedure tboard1form.GoBoard2;
begin
    Board.bFlip := NOT INI1.INI.bFlipBd;
    ResetAll;
    IF (INI1.INI.nBoard2S <> 2) THEN
    BEGIN
        SetBounds(INI1.INI.nBoard2L, INI1.INI.nBoard2T,
                  INI1.INI.nBoard2W, INI1.INI.nBoard2H);
    END;
    WindowState := TOOLBOX_IntToWinState(INI1.INI.nBoard2S);
    cCaption := Board2_title;
    Caption     := cCaption;
end;

procedure tboard1form.gonormal;
begin
        label3.Visible:=true;
        label4.Visible:=true;
        speedbutton1.Visible:=true;
        speedbutton2.Visible:=true;
        speedbutton3.Visible:=true;
        speedbutton4.Visible:=true;
        label5.Visible:=needshowlag;
        label6.Visible:=needshowlag;
end;

function tboard1form.needShowLag:boolean;
begin
 Result:=((ini1.INI.bShowLag) and ((nWlag>0) or (nBlag>0)));
end;

procedure Tboard1form.ShowMoveLag;
var bUp:boolean;
var lab5,lab6:string;
begin

  if needshowlag then
  begin
   if Board.bFlip then
   begin
      lab5:=GetLagStr(nWlag);
      lab6:=GetLagStr(nBlag);
   end
   else begin
      lab5:=GetLagStr(nBlag);
      lab6:=GetLagStr(nWlag);
   end;
   if nLag>0 then
   begin
    bUp:= ((Board.Chess.bWhitesMove) XOR (board.bFlip));
    if bUp then lab5:=lab5+'('+GetLagStr(nlag)+')'
    else lab6:=lab6+'('+GetLagStr(nlag)+')';
   end;
    label5.Caption:=lab5;
    label6.Caption:=lab6;
    label5.Visible:=true;
    label6.Visible:=true;
  end
  else
   begin
      label5.Visible:=false;
      label6.Visible:=false;
   end;
end;

procedure Tboard1Form.MakeMove(nFromX,nFromY,nToX,nToY:integer;cDrop:Char);
var nIndex,nColor,x,y:integer;
begin
 with board.Chess do
 begin
  nIndex:=getIndex(cDrop);
  nColor:=getColor(cDrop);
 end;
 board.dropchesstoboard(nColor,nIndex,x,y);
 board.DrawDropPc(Paintbox1.Canvas,x,y);

 if ((nTox>0) and (nToy>0)) then Board.DrawPiece(Paintbox1.Canvas,nToX,nToY)
 else exit;
 if ((nFromx>0) and (nFromY>0)) then
 begin
  Board.DrawPiece(Paintbox1.Canvas,nFromX,nFromY);
  if ((board.Chess.aBoard[nTox,nToY] in [BLACK_KING,WHITE_KING]) and (abs(nTox-nFromx)>1)) then
  if nTox>nFromx then //0-0
  begin
   board.DrawPiece(Paintbox1.Canvas,nToX+1,nToY);
   board.DrawPiece(Paintbox1.Canvas,nTox-1,nToY);
  end
  else begin //0-0-0
   board.DrawPiece(Paintbox1.Canvas,1,nToY);
   board.DrawPiece(Paintbox1.Canvas,nTox+1,nToY);
  end;

 end
 else Begin
  with board.Chess do begin
   nIndex:=getIndex(aboard[nTox,nToy]);
   nColor:=getColor(aboard[nTox,nToy]);
  end;
  board.dropchesstoboard(nColor,nIndex,x,y);
  board.DrawDropPc(Paintbox1.Canvas,x,y);
 end;
 SetClockColor;
end;

procedure tboard1form.SetClockColor;
begin
    IF (Board.Chess.bWhitesMove) THEN
    BEGIN
        IF (Board.bFlip) THEN
        BEGIN
            Label3.Font.Color := nTurnF;
            Label3.Color      := nTurnB;
            Label4.Font.Color := nWaitF;
            Label4.Color      := nWaitB;
        END
        ELSE
        BEGIN
            Label3.Font.Color := nWaitF;
            Label3.Color      := nWaitB;
            Label4.Font.Color := nTurnF;
            Label4.Color      := nTurnB;
        END;
    END
    ELSE
    BEGIN
        IF (Board.bFlip) THEN
        BEGIN
            Label3.Font.Color := nWaitF;
            Label3.Color      := nWaitB;
            Label4.Font.Color := nTurnF;
            Label4.Color      := nTurnB;
        END
        ELSE
        BEGIN
            Label3.Font.Color := nTurnF;
            Label3.Color      := nTurnB;
            Label4.Font.Color := nWaitF;
            Label4.Color      := nWaitB;
        END;
    END;

end;

END.
