(*============================================================================*)
(* EXTRACT.PAS -- Extract function window                                     *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan                                           *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.09: 1) Removed color selection.                                    *)
(*             2) Added a browse button.                                      *)
(* 1998.11.11: Changed PickDirForm's caption from 'Browse...' to              *)
(*             'Source BPGN directory'.                                       *)
(* 1998.11.23: 1) Added remove all destination comments before extraction.    *)
(*             2) Added generate destination occurrence comment.              *)
(*                                                                            *)
(*============================================================================*)
UNIT EXTRACT;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls, ComCtrls, ExtCtrls,
    MISC, PICKDIR,global;


TYPE
    TExtractForm = CLASS(TForm)
        OpenDialog1: TOpenDialog;

        Panel1:  TPanel;

        Label1:  TLabel;
        Edit1:   TEdit;

        Label2:  TLabel;
        Edit2:   TEdit;

        Label3:  TLabel;
        Edit3:   TEdit;

        Label4:  TLabel;
        Edit4:   TEdit;
        UpDown1: TUpDown;

        CheckBox1: TCheckBox;
        CheckBox2: TCheckBox;

        Button1: TButton;
        Button2: TButton;
        Button3: TButton;
        Button4: TButton;

        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE SetupValues;
        PROCEDURE Edit1Exit(Sender: TObject);
        PROCEDURE Edit2Exit(Sender: TObject);
        PROCEDURE Edit3Exit(Sender: TObject);
        PROCEDURE Button1Click(Sender: TObject);
        PROCEDURE Button2Click(Sender: TObject);
        PROCEDURE Button3Click(Sender: TObject);
        PROCEDURE Button4Click(Sender: TObject);

        PRIVATE
        { Private declarations }

        PUBLIC
        { Public declarations }
        bOK:            Boolean;
        cHandle:        String;
        cSource:        String;
        cDestination:   String;
        nMoveNumber:    Integer;
        bRemoveComment: Boolean;
        bGenOccurrence: Boolean;
    END;


IMPLEMENTATION

{$R *.DFM}

PROCEDURE TExtractForm.FormCreate(Sender: TObject);
BEGIN
    Left           := (Screen.Width  - Width ) SHR 1;
    Top            := (Screen.Height - Height) SHR 1;
    cHandle        := 'Gnejs';
    cSource        := ExtractFilePath(ParamStr(0)) + '*.' + BPGN_EXT;
    cDestination   := ExtractFilePath(ParamStr(0)) + cHandle + '.' + BPGN_EXT;
    nMoveNumber    := DEF_EXTRACT_NUM;
    bRemoveComment := TRUE;
    bGenOccurrence := FALSE;
END;

PROCEDURE TExtractForm.SetupValues;
BEGIN
    bOK            := FALSE;
    ActiveControl  := Edit1;

    Edit1.Text := cHandle;
    Edit2.Text := cSource;
    Edit3.Text := cDestination;

    UpDown1.Min      := 1;
    UpDown1.Max      := MAX_EXTRACT_NUM;
    UpDown1.Position := nMoveNumber;
    Edit4.Text       := IntToStr(nMoveNumber);

    CheckBox1.Checked := bRemoveComment;
    CheckBox2.Checked := bGenOccurrence;

    OpenDialog1.InitialDir := ExtractFilePath(cDestination);
END;

PROCEDURE TExtractForm.Edit1Exit(Sender: TObject);
BEGIN
    Edit1.Text := Trim(Edit1.Text);
    IF (Length(Edit1.Text) > 16) THEN
        Edit1.Text := Copy(Edit1.Text,1,16);
END;

PROCEDURE TExtractForm.Edit2Exit(Sender: TObject);
BEGIN
    Edit2.Text := Trim(Edit2.Text);
    Edit2.Text := ChangeFileExt(Edit2.Text, '.' + BPGN_EXT);
END;

PROCEDURE TExtractForm.Edit3Exit(Sender: TObject);
BEGIN
    Edit3.Text := Trim(Edit3.Text);
    Edit3.Text := ChangeFileExt(Edit3.Text, '.' + BPGN_EXT);
END;

PROCEDURE TExtractForm.Button1Click(Sender: TObject);
VAR
    nI: Integer;
    bE: Boolean;
BEGIN
    IF (Length(Edit1.Text) = 0) THEN
    BEGIN
        wError('Missing handle.');
        ActiveControl := Edit1;
        EXIT;
    END;

    bE := FALSE;
    FOR nI := 1 TO Length(Edit1.Text) DO
    BEGIN
        IF ((UpCase(Edit1.Text[nI]) < 'A') OR (UpCase(Edit1.Text[nI]) > 'Z')) THEN
        BEGIN
            bE := TRUE;
            BREAK;
        END;
    END;
    IF (bE) THEN
    BEGIN
        wError('Handle can only consist of lower and upper case letters.');
        ActiveControl := Edit1;
        EXIT;
    END;

    IF (Length(Edit2.Text) = 0) THEN
    BEGIN
        wError('Missing source BPGN filter.');
        ActiveControl := Edit2;
        EXIT;
    END;

    IF (Length(Edit3.Text) = 0) THEN
    BEGIN
        wError('Missing destination BPGN filename.');
        ActiveControl := Edit3;
        EXIT;
    END;

    IF (CompareText(Edit2.Text, Edit3.Text) = 0) THEN
    BEGIN
        wError('Source and destination must be different.');
        ActiveControl := Edit2;
        EXIT;
    END;

    IF ((CompareText(ExtractFileDrive(Edit2.Text), ExtractFileDrive(Edit3.Text)) = 0) AND
        (CompareText(ExtractFilePath(Edit2.Text),  ExtractFilePath(Edit3.Text)) = 0)) THEN
    BEGIN
        wError('Source and destination directory must be different.');
        ActiveControl := Edit2;
        EXIT;
    END;

    bOK := TRUE;
    IF (Length(Edit1.Text) > 16) THEN
        cHandle := Trim(Copy(Edit1.Text,1,16))
    ELSE
        cHandle := Edit1.Text;

    cSource        := Edit2.Text;
    cDestination   := Edit3.Text;
    nMoveNumber    := StrToInt(Edit4.Text);
    bRemoveComment := CheckBox1.Checked;
    bGenOccurrence := CheckBox2.Checked;

    Close;
END;

PROCEDURE TExtractForm.Button2Click(Sender: TObject);
BEGIN
    bOK := FALSE;
    Close;
END;

PROCEDURE TExtractForm.Button3Click(Sender: TObject);
var PickDirForm:TPickDirForm;
BEGIN
try
    pickDirForm:=TPickDirForm.Create(nil);
    PickDirForm.Caption  := 'Source BPGN directory';
    PickDirForm.cOrgPath := Edit2.Text;
    PickDirForm.cNewPath := '';

    PickDirForm.SetupValues;
    PickDirForm.ShowModal;

    IF (PickDirForm.bOk) THEN
        Edit2.Text := Trim(PickDirForm.cNewPath) + '*.' + BPGN_EXT;
finally
    pickdirform.Free;
end; 
END;

PROCEDURE TExtractForm.Button4Click(Sender: TObject);
BEGIN
    IF (OpenDialog1.Execute) THEN
    BEGIN
        Edit3.Text := ChangeFileExt(OpenDialog1.Filename, '.' + BPGN_EXT);
        OpenDialog1.InitialDir := ExtractFilePath(Edit3.Text);
    END;
END;

END.
