(*============================================================================*)
(* EngineOut.PAS -- Engine Window Form / Engine control                       *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2002.01   : First construction.                                            *)
(* 2004.11.28:1) Changed stop to check whether engine is running              *)
(*            2) stop no longer calls waitfor (fixes a strange invalid handle *)
(*               error)                                                       *)
(*============================================================================*)

unit EngineOut;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  Sjeng,INI1,GLOBAL,TOOLBOX,ZhEngine,EngineRead, Buttons, Menus, ExtCtrls;

const
DiV_STring='-----------------';
ST_IDLE=1;
ST_RUN=2;
ST_SUSPENDED=3;
ST_STICK=4;
ST_UNSTICK=5;
ST_ANNO=6;
ST_TEST=7;



type
  TEngineForm = class(TForm)
    Memo1: TMemo;
    Memo2: TMemo;
    BitBtn1: TBitBtn;
    PopupMenu1: TPopupMenu;
    Suspend1: TMenuItem;
    Unload1: TMenuItem;
    Clear1: TMenuItem;
    Stop1: TMenuItem;
    Options1: TMenuItem;
    SetDefault1: TMenuItem;
    BitBtn2: TBitBtn;
    Stick1: TMenuItem;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    Ann1: TMenuItem;
    Test1: TMenuItem;

    PROCEDURE FormCreate(Sender: TObject);
    Procedure FormDestroy(Sender: Tobject);
    PROCEDURE ResetAll;
    procedure Enginewriteall;
    procedure ReadEng;
    Procedure ShowLine(s:string;mode:integer);
    procedure Run;
    procedure stop;
    procedure suspend;
    procedure resume;
    procedure BitBtn1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Stop1Click(Sender: TObject);
    procedure Options1Click(Sender: TObject);
    procedure SetDefault1Click(Sender: TObject);
    procedure Suspend1Click(Sender: TObject);
    procedure Stick1Click(Sender: TObject);
    procedure Unload1Click(Sender: TObject);
    procedure updatestate(st:integer);
    procedure engfirstinit;
    procedure initmemo2;
    procedure initmemo2moves;
    procedure movefilter(s:string;mode:integer);
    procedure Ann1Click(Sender: TObject);
    procedure Ann2Click(Sender: Tobject);
    procedure Test1Click(Sender: TObject);
    procedure test2click(Sender: Tobject);
    function lastpos(s:string):integer;


  private
   binited:boolean;
   bisrun:boolean;
   bsuspended:boolean;
   bsticks:boolean;
   banno:boolean;
   bmoved:boolean;
   currentout:string;

   procedure reset;


  protected
   procedure RunFirst;
   procedure WMEngineDead(var Message: TMessage); message WM_EngineDead;
   procedure WMEngineRead(var Message: Tmessage); message WM_EngineRead;

    { Private declarations }
  public
    FilterOutput:function(s:string):string;
    HomeWnd:Thandle;
    EngineN:longint;
    EngineRead:TengineRead;
    Engine: Zengine;
    opt:TengineOpt;
   EngDisp:TEnglineArr;
   DispN:integer;
   lastLine:string;
   bestLine:TEngLine;

    procedure updateEng(step:integer);
    procedure setAnnmode;
    procedure setTestmode;
    procedure stoptest;
    procedure stopAnn;
    function getbestline:TEngLine;
    procedure setbestline(s:string);
    { Public declarations }
  end;

var
  EngineForms: array [1..MAX_ENGINE] of TEngineForm;

implementation

{$R *.DFM}

function  ismove(s:string):boolean;
var ps:integer;
res:boolean;
begin
 ps:=pos('move ',s);
 if ps=1 then
 begin
  delete(s,1,5);
  s:=trim(s);
  ps:=pos('score',s);
  if ((s='') or (ps>0)) then res:=false
  else res:=true;
 end
 else res:=false;
 Result:=res;
end;

procedure TEngineForm.setAnnmode;
begin
bAnno:=true;
Updatestate(ST_ANNO);
end;

procedure TengineForm.settestmode;
begin
 banno:=true;
 Updatestate(ST_TEST);
end;

procedure TEngineForm.StopAnn;
begin
 banno:=false;
 Stop;
end;

procedure TengineForm.stoptest;
begin
 banno:=false;
 Stop;
end;


function parseoutput(s:string):TengLine;
var s1,s2,s3,s4,sep:string;
res:Tengline;
ps,i:integer;
begin
sep:=' ';
 s1:=parseNum(s,' ');
 s2:=parseNum(s,' ');
 s3:=parsenum(s,' ');
 s4:=parsenum(s,' ');
 if ((s1='error') or (s2='error') or (s3='error')) then begin Res.score:='';exit;end;
 res.score:=s2;
 res.depth:=s1;
 res.time:=s3;
 res.nodes:=s4;
 s:=trim(s);
 res.line:=s;
 ps:=pos(sep,s);
 if ps>0 then s1:=copy(s,1,ps-1)
 else s1:=s;
  //now delete all ?!
 ps:=pos('?',s1);
 while ps>0 do begin delete(s1,ps,1);ps:=pos('?',s1);end;
 ps:=pos('!',s1);
 while ps>0 do begin delete(s1,ps,1);ps:=pos('!',s1);end;
 res.move:=s1;
 Result:=res;
end;

procedure TEngineForm.setbestline(s:string);
var t:TengLine;
begin
 if bmoved then exit;
 t:=parseoutput(s);
 if length(t.score)>0 then bestline:=t;
end;

function TEngineForm.getbestline:Tengline;
begin
 Result:=bestLine;
end;


function Tostr(t:Tengline):string;
var r,r1:string;
sc:string;
l:integer;
begin
 if t.score[1]<>'-' then t.score:='+'+t.score;
 l:=length(t.score);
 if l>3 then insert('.',t.score,l-1) else insert('0.',t.score,2);
 r:=format('%6s',[t.score]);
 r:=r+format('%6s',[t.move]);
 r1:=format('%7s',['('+t.depth+'ply)']);r:=r+r1;
 r:=r+format('%6s',[t.time] );
 r:=r+format('%9s',[t.nodes] );
 r:=r+' '+t.line;
 result:=r;
 //Result:=t.score+' '+t.move+'('+t.depth+'ply)   '+t.time+'   '+ t.nodes+'   '+t.line;
end;

procedure TEngineform.movefilter(s:string;mode:integer);
var t:Tengline;
lc,fc,ft,fn,ol,i,dc:integer;
found:boolean;
s1,s2:string;
var outbase:integer;
begin
 if bmoved then exit;
 if mode=1 then  exit;
 bmoved:=ismove(s);
 if bmoved then begin memo1.lines.add(s); exit;end;
 t:=parseoutput(s);
 if t.score='' then exit;
 //now delete pondering
 if currentout<>'nothing' then
 begin
  ol:=lastpos(currentout);
  lc:=memo1.lines.count-1;
  for i:=lc downto (ol) do memo1.lines.delete(i);
  currentout:='nothing';
 end;

 // if it's the first line by engine
 if DISpN=0 then
 begin
  dispN:=1;
  EngDisp[1]:=t;
  memo1.lines.add(Tostr(t));
  exit;
 end;

//now find the move in output

 found:=false;
 for i:=1 to dispN do
  if t.move=EngDisp[i].move then
  begin
   found:=true;
   fn:=i;
   break;
  end;
 // this is the new move;
 if not found then
 begin
   dispN:=Dispn+1;
   EngDisp[dispn]:=t;
 end;
 // if we are running out of space
 if dispN=max_next then
 begin
  dispN:=max_next-1;
  fc:=lastpos(tostr(engdisp[DispN]));
  for i:=memo1.lines.count-1 to fc do memo1.lines.delete(i);
  engdisp[dispn]:=t;
  memo1.lines.add(tostr(t));
  exit;
 end;

 if not found then memo1.lines.add(tostr(t))
 else begin
  s1:=tostr(engdisp[fn]);
  if (fn=dispn) then
  begin
   fc:=lastpos(s1);
   ft:=memo1.lines.count-1;
   for i:=ft downto fc do memo1.lines.delete(i);
   memo1.lines.add(tostr(t));
  end
  else begin
   fc:=lastpos(s1);
   ft:=lastpos(tostr(engdisp[fn+1]));
   s2:=memo1.lines[ft]; dc:=0;
   for i:=ft-1 downto fc do
   begin
    inc(dc); memo1.lines.delete(i);
   end;
   memo1.lines[ft-dc]:=s2;
   memo1.lines.insert(fc,tostr(t));
  end;
  engdisp[fn]:=t;
 end;
end;


procedure TEngineForm.updateEng(step:integer);
label lostsync;
begin
 currentout:='nothing';
 if bsticks then exit;
 if Engine.banal then
 begin
  if bmoved then goto lostsync; //engine lost synchronization
  case step of
  1:Engine.writeeng(sjengMove(Line[1,1],linecnt[1])+LF_ASCII_CHAR);
  -1:Engine.Writeeng('undo'+CR_ASCII_CHAR+LF_ASCII_CHAR);
  -INFTY:Engine.writeeng(Engine.command_reset+LF_ASCII_CHAR);
  INFTY:begin
        Engine.writeeng(Engine.command_reset+LF_ASCII_CHAR);
        memo2.clear;
        initmemo2moves;
        enginewriteall;
        end;
  end;
  memo1.lines.add(DIV_STRing);
  DispN:=0;
  exit;
 end;
lostsync:
 if Engine.command_reset='' then
 begin
  stop;
  run;
  exit;
 end;
 memo2.clear;
 Engine.writeeng(engine.command_reset+LF_ASCII_CHAR);
 Engine.writeeng('force'+LF_ASCII_CHAR);
 Engine.writeeng(engine.itime+LF_ASCII_CHAR);
 initmemo2moves;
 memo2.lines.add('go');
 enginewriteall;
 memo1.lines.add(Div_string);
 DispN:=0;
 exit;
end;

procedure Tengineform.engfirstinit;
begin
 Engine.comline:=opt.comline;
 Engine.engname:=opt.name;
 Caption:=opt.comline;
 Engine.prompt:=opt.prompt;
 Engine.command_reset:=opt.reset;
 Engine.banal:=opt.banal;
 Engine.itime:=opt.itime;
 engine.bfilter:=opt.bfilter;
 bmoved:=false;
 str(-MATE_EVAL-1,bestline.score);
end;

PROCEDURE TEngineForm.FormCreate(Sender: TObject);
BEGIN
    left:=screen.width;
    Memo1.Color       := INI1.INI.nEngineB;
    Memo1.Font.Color  := INI1.INI.fEngineF.Color;
    Memo1.Font.Height := INI1.INI.fEngineF.Height;
    Memo1.Font.Name   := INI1.INI.fEngineF.Name;
    Memo1.Font.Pitch  := INI1.INI.fEngineF.Pitch;
    Memo1.Font.Size   := INI1.INI.fEngineF.Size;
    Memo1.Font.Style  := INI1.INI.fEngineF.Style;
    Memo1.MaxLength   := MAX_MEMO_CHAR;
    Memo1.WordWrap    := INI1.INI.bEngineWR;

    IF (Memo1.WordWrap) THEN
        Memo1.ScrollBars := ssVertical
    ELSE
        Memo1.ScrollBars := ssBoth;

    bitbtn1.glyph:=glyphs[2]^.bitmap;
    bitbtn3.glyph:=glyphs[1]^.bitmap;
    bitbtn2.glyph:=glyphs[1]^.bitmap;

    ResetAll;
    bsuspended:=false;
    bisrun:=false;
    bsticks:=false;
    engine:=ZEngine.Create;
    Engine.EngWind:=handle;
    bitbtn1.left:=memo1.left+memo1.width-bitbtn1.width-20;
    bitbtn1.top:=memo1.Top;
    bitbtn4.width:=25;
    bitbtn4.left:=bitbtn1.left-bitbtn4.width;
    bitbtn4.top:=bitbtn1.Top;
    bitbtn2.left:=bitbtn4.left-bitbtn2.width;
    bitbtn2.top:=bitbtn1.top;
    Updatestate(ST_IDLE);
    Engine.mem1:=@memo1;
END;

Procedure TengineForm.FormDestroy(Sender:Tobject);
begin
 stop;
 Engine.Free;
end;

procedure TEngineForm.suspend;
var err,err1:integer;
begin
 bsuspended:=true;
 updateSTate(ST_SUSPENDED);
 if Engine.bLoadOk then
 begin
  EngineRead.suspend;
  err:=suspendthread(Engine.hprocess);
  err1:=getlasterror;
 end;
end;

procedure TEngineForm.resume;
begin
 bsuspended:=false;
 UpdateState(ST_RUN);
 if Engine.BloadOk then
 begin
  resumethread(Engine.hprocess);
  Engineread.Resume;
 end;
end;

procedure TEngineForm.stop;
begin
 if not bisrun then exit;
 bisRun:=false;
 bsticks:=false;
 if not banno then UpdateState(ST_IDLE);
 if (EngineRead<>nil) then EngineRead.terminate;

 reset;
//

 Engine.resetKILL;
 memo1.Lines.add(DIV_String);
// if (EngineRead<>nil) then
//  EngineRead.WaitFor;

 //close handles after we are sure that nobody will use them
 closeHandle(Engine.hto);
 closeHandle(Engine.hfrom);
 getlasterror;
end;

procedure TEngineForm.WMEngineDead(var message:Tmessage);
begin
 //stop;
 inherited;
end;

procedure TEngineForm.WMEngineRead(var message: Tmessage);
begin
 readeng;
 inherited;
end;

PROCEDURE TEngineForm.ResetAll;
BEGIN
    Memo1.Clear;
    memo2.clear;
    binited:=false;
END;

procedure TEngineForm.initmemo2moves;
var i:integer;
begin
 for i:=lineCnt[1] downto 1 do
 Memo2.Lines.add(sjengMove(Line[i,1],lineCnt[1]-i+1));
end;


procedure TEngineForm.initmemo2;
begin
 if opt.bfilter then filterOutput:=sjengFilteroutput
 else FilterOutput:=nofilter;
 memo2.lines.add('xboard');
 memo2.lines.add('post');
 memo2.lines.add('variant crazyhouse');
 memo2.lines.add('force');
 if engine.banal then
 begin
  memo2.lines.add('analyze');
  if engine.command_reset<>'' then memo2.lines.add(engine.command_reset);
 end
 else begin
  memo2.lines.add(opt.itime);
 end;
 initmemo2moves;
 if not engine.banal then memo2.lines.add('go');
end;

procedure TEngineForm.enginewriteall;
var i,cnt:integer;
begin
//write to the programm console here
 cnt:=memo2.lines.count;
 for i:=0 to cnt-1 do
 begin
  Engine.writeEng(memo2.lines[i]+CR_ASCII_CHAR+LF_ASCII_CHAR);
 end;
end;


function TengineForm.lastpos(s:string):integer;
var txt:string;
i:integer;
Begin
 txt:='';
 for i:=memo1.lines.count-1 downto 0 do
 begin
  txt:=memo1.lines[i]+txt;
  if pos(s,txt)>0 then
  begin
   Result:=i;
   exit;
  end;
 end;
 Result:=-1;
end;

Procedure TEngineForm.Showline(s:string;mode:integer);
var i,pl,co:integer;
begin
 if banno then setbestline(s);
 if ((mode=0)and  (opt.bfilter)) then begin movefilter(s,mode); exit; end;
 bmoved:=ismove(s);
 s:=trimright(s);
 if currentout<>'nothing' then   co:=lastpos(currentout)
 else co:=memo1.lines.count;
 if co=-1 then co:=memo1.lines.count;
 for i:=memo1.lines.count-1 downto co do memo1.lines.delete(i);
  memo1.lines.add(s);
 if mode=1 then
 begin
  currentout:=s;
 end
 else begin
  currentout:='nothing';
 end;
end;

procedure TEngineForm.reset;
begin
 memo2.Clear;
 binited:=false;
end;

procedure TEngineForm.ReadEng();
var ps,ps1:integer;
s1,s:string;
begin
 if ((engine.prompt = '') and (not binited)) then
 begin
  binited:=true;enginewriteall;
 end;
 s:=engine.last;
 Engine.last:='';
 s:=engine.next+s;
 engine.next:='';
 // kill CR's

 ps:=pos(EOL_STR,s);
 While ps>0 do begin delete(s,ps,1); ps:=pos(EOL_STR,s); end;
 ps:=pos(LF_ASCII_CHAR,s);
 while ps>0 do
 begin
  s1:=copy(s,1,ps-1);
  ps1:=lastdelimiter(CR_ASCII_CHAR,s1);
  s1:=copy(s1,ps1+1,length(s1));
  showline(s1,0);
  delete(s,1,ps);
  ps:=pos(LF_ASCII_CHAR,s);
 end;
 if ((Engine.prompt<>'') and (pos(Engine.prompt,s)>0)) then
 begin //memo1.lines.add(s);
  enginewriteall;engine.next:='';
 end
 else begin
  ps:=pos(CR_ASCII_CHAR,s);
  s1:=copy(s,1,ps-1);delete(s,1,ps);
  engine.next:=s;
  if length(s1)>0 then  showline(s1,1);
 end;
end;

Procedure TEngineForm.Updatestate(st:integer);
begin
 case st of
 ST_IDLE: begin
           bitbtn4.glyph:=Glyphs[4]^.Bitmap;
           bitbtn4.hint:='Engine is idle. Click to run it';
           bitbtn4.onclick:=stop1click;
           stop1.caption:='Run';
           suspend1.caption:='suspend';
           suspend1.enabled:=false;
           stick1.caption:='Stick to current';
           stick1.enabled:=false;
           bitbtn3.visible:=false;
           Ann1.caption:='Annotate game';
           Ann1.enabled:=true;
           Test1.enabled:=true;
           test1.caption:='Test Engine';
           test1.onclick:=test1click;
           ann1.onclick:=ann1click;
           unload1.enabled:=true;
          end;
 ST_RUN: begin
          bitbtn4.hint:='Engine is running. Click to stop it';
          bitbtn4.glyph:=Glyphs[3]^.bitmap;
          bitbtn4.onclick:=stop1click;
          stop1.caption:='Stop';
          suspend1.caption:='suspend';
          suspend1.enabled:=true;
          stick1.caption:='Stick to current';
          stick1.enabled:=true;
          Unload1.enabled:=true;
          Stop1.enabled:=true;
          ann1.enabled:=true;
          test1.enabled:=true;
         end;
 ST_SUSPENDED: begin
           bitbtn4.glyph:=glyphs[5]^.bitmap;
           bitbtn4.hint:='Engine is suspended. Click to resume';
           bitbtn4.onclick:=suspend1click;
           suspend1.caption:='Resume';
           unload1.enabled:=false;
           stick1.enabled:=false;
           stop1.enabled:=false;
           ann1.enabled:=false;
           test1.enabled:=false;
             end;
 ST_UNSTICK: begin
          stick1.caption:='Stick to current';
          bitbtn3.enabled:=false;
             end;
 ST_STICK: begin
          bitbtn3.caption:='Positition After '+lastmove;
          bitbtn3.enabled:=true;
          bitbtn3.visible:=true;
          stick1.caption:='Stop sticking';
           end;
 ST_ANNO: begin
            bitbtn4.glyph:=glyphs[6]^.bitmap;
            bitbtn4.hint:='Annotation in progress. Click to stop';
            bitBtn4.onClick:=ann2click;
            suspend1.enabled:=false;
            stick1.enabled:=false;
            unload1.enabled:=false;
            Ann1.caption:='Stop Annotating';
            Ann1.onclick:=ann2click;
            stop1.onclick:=ann2click;
            test1.enabled:=false;
          end;
  ST_TEST: Begin
            bitbtn4.glyph:=glyphs[6]^.bitmap;
            bitbtn4.hint:='Test in progress. Click to stop';
            bitBtn4.onClick:=test2click;
            suspend1.enabled:=false;
            stick1.enabled:=false;
            unload1.enabled:=false;
            test1.caption:='Stop Testing';
            test1.onclick:=test2click;
            stop1.onclick:=test2click;
            ann1.enabled:=false;
           end;
 end;
end;


procedure TengineForm.Run;
begin
 bisRun:=true;
 bsticks:=false; bsuspended:=false;
 if not banno then updatestate(ST_RUN);
 DispN:=0;
 currentout:='nothing';
 if Engine.bloadOk then Enginewriteall
 else RunFirst;
end;

procedure TEngineForm.RunFirst;
begin
 engfirstinit;
 initmemo2;
 If not Engine.loadengine(Engine.comline) then exit;
 EngineRead:=TEngineRead.Createit(Engine);
 show;
end;

procedure TEngineForm.BitBtn1Click(Sender: TObject);
begin
  Popupmenu1.Popup(left+width-BitBtn1.width-90,top+bitbtn1.height+65);
end;

procedure TEngineForm.Clear1Click(Sender: TObject);
begin
 memo1.clear;
 DispN:=0;
end;

procedure TEngineForm.Stop1Click(Sender: TObject);
begin
 if bisrun then SendMessage(HomeWnd,WM_ENGINESTOP,engineN,0)
 else sendMessage(homewnd,WM_ENGINERUN,engineN,0);
end;

procedure TEngineForm.Options1Click(Sender: TObject);
begin
 SendMessage(HomeWnd,WM_ENGINEOptions,engineN,0);
end;

procedure TEngineForm.SetDefault1Click(Sender: TObject);
begin
 SendMessage(HomeWnd,WM_ENGINEDEFAULT,engineN,0);
end;

procedure TEngineForm.Suspend1Click(Sender: TObject);
begin
 IF bsuspended then SendMessage(HomeWnd,WM_ENGINERESUME,engineN,0)
 else SendMessage(HomeWnd,WM_ENGINESUSPEND,engineN,0);
end;

procedure TEngineForm.Stick1Click(Sender: TObject);
begin
 bsticks:=not bsticks;
 if bsticks then updateState(ST_Stick)
 else stop1click(Sender);

end;

procedure TEngineForm.Unload1Click(Sender: TObject);
begin
 SendMessage(homewnd,WM_ENGINEUNLOAD,engineN,0);
end;

procedure TEngineForm.Ann1Click(Sender: TObject);
begin
 SendMessage(homewnd,WM_ENGINEAnn,engineN,0);
end;

procedure TEngineForm.Ann2Click(Sender: Tobject);
begin
 SendMessage(homeWnd,WM_EngineANNStop,engineN,0);
end;


procedure TEngineForm.Test1Click(Sender: TObject);
begin
 SendMessage(homeWnd,WM_EngineTest,engineN,0);
end;

procedure TengineForm.Test2click(Sender:Tobject);
begin
 Sendmessage(homeWnd,WM_EngineTeststop,engineN,0);
end;

End.
