(*============================================================================*)
(* EngineRead.PAS -- Engine Read thread                                       *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2001.12.21: First construction.                                            *)
(*                                                                            *)
(*============================================================================*)

unit EngineRead;

interface

uses
  Classes,ZhEngine,windows,global;

type
  TEngineRead = class(TThread)
  private
   ZE:ZEngine;
    { Private declarations }
  protected
    procedure Execute; override;
  published
    constructor CreateIt(aZe : Zengine);
    destructor Destroy; override;
  end;

implementation

{ Important: Methods and properties of objects in VCL can only be used in a
  method called using Synchronize, for example,

      Synchronize(UpdateCaption);

  and UpdateCaption could look like,

    procedure TEngineRead.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end; }

{ TEngineRead }


constructor TengineRead.CreateIt(aZe:Zengine);
begin
  inherited Create(true);      // Create thread suspended
  // Priority := TThreadPriority(PriorityLevel); // Set Priority Level
  FreeOnTerminate := true; // Thread Free Itself when terminated
  Ze:= aZe;    // Set reference
  Suspended := false;         // Continue the thread
end;

destructor TEngineRead.Destroy;
begin
   {PostMessage(form1.Handle,wm_ThreadDoneMsg,self.ThreadID,0);

     This posts a message to the main form, tells us when and which thread
     is done executing.
   }
   inherited destroy;
end;


procedure TEngineRead.Execute;
var q,p:PChar;
prev:char;
i:integer;
begin
  while (not Terminated) do
  begin
   Ze.ReadPipe;
   ze.readbuf;
   if terminated then exit;
   if (Ze.engError<>No_ERROR)  then
   SendMessage(Ze.EngWind,WM_EngineDead,0,0)  // signal an error}
   else SendMessage (Ze.EngWind, WM_ENGINEREAD,0,0);
      // if Terminated is true, this loop exits prematurely so the thread will terminate
  end;
end;

end.

