(*============================================================================*)
(* FCD.PAS -- Four column display window                                      *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2001-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.14: First construction.                                            *)
(* 1998.11.17: Added richedit control.                                        *)
(* 1998.11.21: Changed the code to use chess font keymap.                     *)
(* 1998.11.22: Changed the chess font keymap to hold up to 2 characters.      *)
(* 1998.11.23: 1) Added a tab between board #1 and board #2 display.          *)
(*             2) Added include handle, move, annotation, time, comment,      *)
(*                display board, and result for convert to RTF/TXT function.  *)
(*             3) Added edit move piece keymap in RTF function.               *)
(* 1998.11.24: 1) Added include move piece keymap.                            *)
(*             2) Removed include move piece keymap.                          *)
(* 1998.11.25: Added use zeros instead of Os in castling move to edit move    *)
(*             piece keymap function, RTF function, and TXT function.         *)
// 2004.12.30: Changed Add1 and Add2 to fix the bug with misaligned handles and moves
// 2005.02.15: 1) Modified DisplayBoard to add move indicator
// 2005.02.22: Added button: Edit move piece keymap
(*============================================================================*)
UNIT FCD;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls, ComCtrls, ExtCtrls,
    GLOBAL, MISC, KEYMAP,piecemap,INI1;

const ColSep='          ';

TYPE
    TFCDForm = CLASS(TForm)
        OpenDialog1: TOpenDialog;
        FontDialog1: TFontDialog;

        Panel1:  TPanel;

        Label1:  TLabel;
        Edit1:   TEdit;
        Button1: TButton;

        Label2:  TLabel;
        Edit2:   TEdit;
        Button2: TButton;

        Label3:  TLabel;
        Edit3:   TEdit;
        Button3: TButton;

        Label4:  TLabel;
        Edit4:   TEdit;
        Button4: TButton;

        Label5:  TLabel;
        Edit5:   TEdit;
        Button5: TButton;

        Label6:  TLabel;
        Edit6:   TEdit;
        Button6: TButton;

        CheckBox1: TCheckBox;
        CheckBox2: TCheckBox;
        CheckBox3: TCheckBox;
        CheckBox4: TCheckBox;
        CheckBox5: TCheckBox;
        CheckBox6: TCheckBox;
        CheckBox7: TCheckBox;
        CheckBox8: TCheckBox;

        Button98:  TButton;
        Button99:  TButton;
        Button100: TButton;

        RichEdit1: TRichEdit;
        Memo1:     TMemo;
    Button7: TButton;

        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE SetupValues;
        PROCEDURE Edit5Exit(Sender: TObject);
        PROCEDURE Button1Click(Sender: TObject);
        PROCEDURE Button2Click(Sender: TObject);
        PROCEDURE Button3Click(Sender: TObject);
        PROCEDURE Button4Click(Sender: TObject);
        PROCEDURE Button6Click(Sender: TObject);
        PROCEDURE Button5Click(Sender: TObject);
        PROCEDURE Button98Click(Sender: TObject);
        PROCEDURE Button99Click(Sender: TObject);
        PROCEDURE Button100Click(Sender: TObject);

        PROCEDURE ClearMemo;
        PROCEDURE AddLine(cLine: String;
                          fName: TFontName; fColor: TColor;
                          fSize: Integer; fHeight: Integer;
                          fPitch: TFontPitch; fStyle: TFontStyles);
        PROCEDURE Add1(cWhiteA,cBlackA,cWhiteB,cBlackB: String);
        PROCEDURE Add2(cMoveAW,cMoveAB,cMoveBW,cMoveBB: String);
        PROCEDURE Add3(cLine: String);
        PROCEDURE Add4(cLine: String);
        PROCEDURE Add5(cLine: String);
        PROCEDURE AddBlankLine;
        PROCEDURE SaveToFile;
        PROCEDURE ClearComment;
        FUNCTION  GetOffPiece(cP: Char): String;
        FUNCTION  GetBoardPiece(bWhiteSquare: Boolean; cP: Char): String;
        PROCEDURE DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2,cwt1,cbt1,cwt2,cbt2: String;bwm1,bwm2:boolean);
        PROCEDURE AdjustFile;
        procedure SetColumnWidth;
        procedure backupfont;
        procedure restorefont;
        function AdjustTxtW(s:string;nGoal:integer):string;
    procedure Button7Click(Sender: TObject);

        PRIVATE
        { Private declarations }
        nColW:       integer;
        fBkupFont:   Tfont;
        PUBLIC
        { Public declarations }
        bOK:         Boolean;
        fHandle:     TFont;
        fMove:       TFont;
        fComment:    TFont;
        fChess:      TFont;
        fResult:     TFont;
        bHandle:     Boolean;
        bMove:       Boolean;
        bAnnotation: Boolean;
        bTime:       Boolean;
        bComment:    Boolean;
        bDisplay:    Boolean;
        bResult:     Boolean;
        bZero:       Boolean;
        cRTFFile:    String;
    END;



IMPLEMENTATION

{$R *.DFM}
 {$I-}
PROCEDURE TFCDForm.FormCreate(Sender: TObject);
BEGIN
    Left     := (Screen.Width  - Width ) SHR 1;
    Top      := (Screen.Height - Height) SHR 1;
    fHandle  := TFont.Create;
    fMove    := TFont.Create;
    fComment := TFont.Create;
    fChess   := TFont.Create;
    fResult  := TFont.Create;

    fHandle.Color  := clBlack;
    fHandle.Height := -11;
    fHandle.Name   := 'MS Sans Serif';
    fHandle.Pitch  := fpDefault;
    fHandle.Size   := 10;
    fHandle.Style  := [];

    fMove.Color  := clBlack;
    fMove.Height := -11;
    fMove.Name   := 'MS Sans Serif';
    fMove.Pitch  := fpDefault;
    fMove.Size   := 10;
    fMove.Style  := [];

    fComment.Color  := clBlack;
    fComment.Height := -11;
    fComment.Name   := 'MS Sans Serif';
    fComment.Pitch  := fpDefault;
    fComment.Size   := 10;
    fComment.Style  := [];

    fChess.Color  := clBlack;
    fChess.Height := -11;
    fChess.Name   := 'MS Sans Serif';
    fChess.Pitch  := fpDefault;
    fChess.Size   := 10;
    fChess.Style  := [];

    fResult.Color  := clBlack;
    fResult.Height := -11;
    fResult.Name   := 'MS Sans Serif';
    fResult.Pitch  := fpDefault;
    fResult.Size   := 10;
    fResult.Style  := [];

    bHandle     := TRUE;
    bMove       := TRUE;
    bAnnotation := TRUE;
    bTime       := TRUE;
    bComment    := TRUE;
    bDisplay    := TRUE;
    bresult     := TRUE;
    bZero       := FALSE;

    cRTFFile := ExtractFilePath(ParamStr(0)) + 'GAME.' + RTF_EXT;
END;

PROCEDURE TFCDForm.SetupValues;
BEGIN
    bOK            := FALSE;
    ActiveControl  := Edit1;

    Edit1.Text := fHandle.Name;
    Edit2.Text := fMove.Name;
    Edit3.Text := fComment.Name;
    Edit4.Text := fChess.Name;
    Edit6.Text := fResult.Name;
    Edit5.Text := cRTFFile;

    CheckBox1.Checked := bHandle;
    CheckBox2.Checked := bMove;
    CheckBox3.Checked := bAnnotation;
    CheckBox4.Checked := bTime;
    CheckBox5.Checked := bComment;
    CheckBox6.Checked := bDisplay;
    CheckBox7.Checked := bResult;
    CheckBox8.Checked := bZero;

    OpenDialog1.InitialDir := ExtractFilePath(cRTFFile);
END;

PROCEDURE TFCDForm.Edit5Exit(Sender: TObject);
BEGIN
    Edit5.Text := Trim(Edit5.Text);
    Edit5.Text := ChangeFileExt(Edit5.Text, '.' + RTF_EXT);
END;

PROCEDURE TFCDForm.Button1Click(Sender: TObject);
BEGIN
    FontDialog1.Device      := fdScreen;
    FontDialog1.Font.Name   := fHandle.Name;
    FontDialog1.Font.Color  := fHandle.Color;
    FontDialog1.Font.Height := fHandle.Height;
    FontDialog1.Font.Size   := fHandle.Size;
    FontDialog1.Font.Pitch  := fHandle.Pitch;
    FontDialog1.Font.Style  := fHandle.Style;

    IF (FontDialog1.Execute) THEN
    BEGIN
        fHandle.Name   := FontDialog1.Font.Name;
        fHandle.Color  := FontDialog1.Font.Color;
        fHandle.Height := FontDialog1.Font.Height;
        fHandle.Size   := FontDialog1.Font.Size;
        fHandle.Pitch  := FontDialog1.Font.Pitch;
        fHandle.Style  := FontDialog1.Font.Style;
        Edit1.Text     := fHandle.Name;
    END;
END;

PROCEDURE TFCDForm.Button2Click(Sender: TObject);
BEGIN
    FontDialog1.Device      := fdScreen;
    FontDialog1.Font.Name   := fMove.Name;
    FontDialog1.Font.Color  := fMove.Color;
    FontDialog1.Font.Height := fMove.Height;
    FontDialog1.Font.Size   := fMove.Size;
    FontDialog1.Font.Pitch  := fMove.Pitch;
    FontDialog1.Font.Style  := fMove.Style;

    IF (FontDialog1.Execute) THEN
    BEGIN
        fMove.Name   := FontDialog1.Font.Name;
        fMove.Color  := FontDialog1.Font.Color;
        fMove.Height := FontDialog1.Font.Height;
        fMove.Size   := FontDialog1.Font.Size;
        fMove.Pitch  := FontDialog1.Font.Pitch;
        fMove.Style  := FontDialog1.Font.Style;
        Edit2.Text   := fMove.Name;
    END;
END;

PROCEDURE TFCDForm.Button3Click(Sender: TObject);
BEGIN
    FontDialog1.Device      := fdScreen;
    FontDialog1.Font.Name   := fComment.Name;
    FontDialog1.Font.Color  := fComment.Color;
    FontDialog1.Font.Height := fComment.Height;
    FontDialog1.Font.Size   := fComment.Size;
    FontDialog1.Font.Pitch  := fComment.Pitch;
    FontDialog1.Font.Style  := fComment.Style;

    IF (FontDialog1.Execute) THEN
    BEGIN
        fComment.Name   := FontDialog1.Font.Name;
        fComment.Color  := FontDialog1.Font.Color;
        fComment.Height := FontDialog1.Font.Height;
        fComment.Size   := FontDialog1.Font.Size;
        fComment.Pitch  := FontDialog1.Font.Pitch;
        fComment.Style  := FontDialog1.Font.Style;
        Edit3.Text      := fComment.Name;
    END;
END;

PROCEDURE TFCDForm.Button4Click(Sender: TObject);
BEGIN
    FontDialog1.Device      := fdScreen;
    FontDialog1.Font.Name   := fChess.Name;
    FontDialog1.Font.Color  := fChess.Color;
    FontDialog1.Font.Height := fChess.Height;
    FontDialog1.Font.Size   := fChess.Size;
    FontDialog1.Font.Pitch  := fChess.Pitch;
    FontDialog1.Font.Style  := fChess.Style;

    IF (FontDialog1.Execute) THEN
    BEGIN
        fChess.Name   := FontDialog1.Font.Name;
        fChess.Color  := FontDialog1.Font.Color;
        fChess.Height := FontDialog1.Font.Height;
        fChess.Size   := FontDialog1.Font.Size;
        fChess.Pitch  := FontDialog1.Font.Pitch;
        fChess.Style  := FontDialog1.Font.Style;
        Edit4.Text    := fChess.Name;
    END;
END;

PROCEDURE TFCDForm.Button6Click(Sender: TObject);
BEGIN
    FontDialog1.Device      := fdScreen;
    FontDialog1.Font.Name   := fResult.Name;
    FontDialog1.Font.Color  := fResult.Color;
    FontDialog1.Font.Height := fResult.Height;
    FontDialog1.Font.Size   := fResult.Size;
    FontDialog1.Font.Pitch  := fResult.Pitch;
    FontDialog1.Font.Style  := fResult.Style;

    IF (FontDialog1.Execute) THEN
    BEGIN
        fResult.Name   := FontDialog1.Font.Name;
        fResult.Color  := FontDialog1.Font.Color;
        fResult.Height := FontDialog1.Font.Height;
        fResult.Size   := FontDialog1.Font.Size;
        fResult.Pitch  := FontDialog1.Font.Pitch;
        fResult.Style  := FontDialog1.Font.Style;
        Edit6.Text     := fResult.Name;
    END;
END;

PROCEDURE TFCDForm.Button5Click(Sender: TObject);
BEGIN
    IF (OpenDialog1.Execute) THEN
    BEGIN
        Edit5.Text := ChangeFileExt(OpenDialog1.Filename, '.' + RTF_EXT);
        OpenDialog1.InitialDir := ExtractFilePath(Edit5.Text);
    END;
END;

PROCEDURE TFCDForm.Button98Click(Sender: TObject);
BEGIN
    IF (Length(Edit1.Text) = 0) THEN
    BEGIN
        wError('Missing handle font.');
        ActiveControl := Button1;
        EXIT;
    END;

    IF (Length(Edit2.Text) = 0) THEN
    BEGIN
        wError('Missing move font.');
        ActiveControl := Button2;
        EXIT;
    END;

    IF (Length(Edit3.Text) = 0) THEN
    BEGIN
        wError('Missing comment font.');
        ActiveControl := Button3;
        EXIT;
    END;

    IF (Length(Edit4.Text) = 0) THEN
    BEGIN
        wError('Missing chess font.');
        ActiveControl := Button4;
        EXIT;
    END;

    IF (Length(Edit6.Text) = 0) THEN
    BEGIN
        wError('Missing result font.');
        ActiveControl := Edit6;
        EXIT;
    END;

    IF (Length(Edit5.Text) = 0) THEN
    BEGIN
        wError('Missing destination RTF filename.');
        ActiveControl := Edit5;
        EXIT;
    END;

    bOK := TRUE;

    bHandle     := CheckBox1.Checked;
    bMove       := CheckBox2.Checked;
    bAnnotation := CheckBox3.Checked;
    bTime       := CheckBox4.Checked;
    bComment    := CheckBox5.Checked;
    bDisplay    := CheckBox6.Checked;
    bResult     := CheckBox7.Checked;
    bZero       := CheckBox8.Checked;

    cRTFFile := Edit5.Text;
    Close;
END;

PROCEDURE TFCDForm.Button99Click(Sender: TObject);
BEGIN
    bOK := FALSE;
    Close;
END;

PROCEDURE TFCDForm.ClearMemo;
BEGIN
    RichEdit1.Clear;
END;

PROCEDURE TFCDForm.AddLine(cLine: String;
                           fName: TFontName; fColor: TColor;
                           fSize: Integer; fHeight: Integer;
                           fPitch: TFontPitch; fStyle: TFontStyles);
BEGIN
    RichEdit1.SelAttributes.Name   := fName;
    RichEdit1.SelAttributes.Color  := fColor;
    RichEdit1.SelAttributes.Size   := fSize;
    RichEdit1.SelAttributes.Height := fHeight;
    RichEdit1.SelAttributes.Pitch  := fPitch;
    RichEdit1.SelAttributes.Style  := fStyle;

    RichEdit1.Lines.Add(cLine);
END;

PROCEDURE TFCDForm.Add1(cWhiteA,cBlackA,cWhiteB,cBlackB: String);
var cLine:string;
BEGIN
canvas.Font:=fhandle;
cLine:=AdjustTxtW(cWhiteA,nColW)+AdjustTxtW(cBlackA,nColW)+ColSep+
AdjustTxtW(cWhiteB,nColW)+AdjustTxtW(cBlackB,nColW);
    AddLine(cLine, fhandle.Name,  fhandle.Color,
                   fhandle.Size,  fhandle.Height,
                   fhandle.Pitch, fhandle.Style);
END;

PROCEDURE TFCDForm.Add2(cMoveAW,cmoveAB,cmoveBW,cmoveBB: String);
var cline:string;
BEGIN
canvas.Font:=fmove;
cLine:=AdjustTxtW(cmoveAW,nColW)+AdjustTxtW(cmoveAB,nColW)+ColSep+
AdjustTxtW(cMoveBW,nColW)+AdjustTxtW(cMoveBB,nColW);
AddLine(cLine, fmove.Name,  fmove.Color,
                   fmove.Size,  fmove.Height,
                   fmove.Pitch, fmove.Style);;
END;

PROCEDURE TFCDForm.Add3(cLine: String);
BEGIN
    AddLine(cLine, INI1.INI.fFCD3.Name,  INI1.INI.fFCD3.Color,
                   INI1.INI.fFCD3.Size,  INI1.INI.fFCD3.Height,
                   INI1.INI.fFCD3.Pitch, INI1.INI.fFCD3.Style);
END;

PROCEDURE TFCDForm.Add4(cLine: String);
BEGIN
    AddLine(cLine, INI1.INI.fFCD4.Name,  INI1.INI.fFCD4.Color,
                   INI1.INI.fFCD4.Size,  INI1.INI.fFCD4.Height,
                   INI1.INI.fFCD4.Pitch, INI1.INI.fFCD4.Style);
END;

PROCEDURE TFCDForm.Add5(cLine: String);
BEGIN
    AddLine(cLine, INI1.INI.fFCD5.Name,  INI1.INI.fFCD5.Color,
                   INI1.INI.fFCD5.Size,  INI1.INI.fFCD5.Height,
                   INI1.INI.fFCD5.Pitch, INI1.INI.fFCD5.Style);
END;

PROCEDURE TFCDForm.AddBlankLine;
BEGIN
    RichEdit1.Lines.Add('');
END;

PROCEDURE TFCDForm.SaveToFile;
BEGIN
    RichEdit1.Lines.SaveToFile(INI1.INI.cRTFFile);
END;

PROCEDURE TFCDForm.ClearComment;
BEGIN
    Memo1.Clear;
END;

FUNCTION TFCDForm.GetOffPiece(cP: Char): String;
BEGIN
    CASE (cP) OF
        WHITE_PAWN:   Result := INI1.INI.cWPWS;
        WHITE_ROOK:   Result := INI1.INI.cWRWS;
        WHITE_KNIGHT: Result := INI1.INI.cWNWS;
        WHITE_BISHOP: Result := INI1.INI.cWBWS;
        WHITE_QUEEN:  Result := INI1.INI.cWQWS;
        WHITE_KING:   Result := INI1.INI.cWKWS;

        BLACK_PAWN:   Result := INI1.INI.cBPWS;
        BLACK_ROOK:   Result := INI1.INI.cBRWS;
        BLACK_KNIGHT: Result := INI1.INI.cBNWS;
        BLACK_BISHOP: Result := INI1.INI.cBBWS;
        BLACK_QUEEN:  Result := INI1.INI.cBQWS;
        BLACK_KING:   Result := INI1.INI.cBKWS;
//        MOVE_INDICATOR: Result:=ini1.INI.cMoveInd;
    ELSE
        Result := INI1.INI.cWS;
    END;
END;

FUNCTION TFCDForm.GetBoardPiece(bWhiteSquare: Boolean; cP: Char): String;
BEGIN
    IF (bWhiteSquare) THEN
    BEGIN
        CASE (cP) OF
            WHITE_PAWN:   Result := INI1.INI.cWPWS;
            WHITE_ROOK:   Result := INI1.INI.cWRWS;
            WHITE_KNIGHT: Result := INI1.INI.cWNWS;
            WHITE_BISHOP: Result := INI1.INI.cWBWS;
            WHITE_QUEEN:  Result := INI1.INI.cWQWS;
            WHITE_KING:   Result := INI1.INI.cWKWS;

            BLACK_PAWN:   Result := INI1.INI.cBPWS;
            BLACK_ROOK:   Result := INI1.INI.cBRWS;
            BLACK_KNIGHT: Result := INI1.INI.cBNWS;
            BLACK_BISHOP: Result := INI1.INI.cBBWS;
            BLACK_QUEEN:  Result := INI1.INI.cBQWS;
            BLACK_KING:   Result := INI1.INI.cBKWS;
        ELSE
            Result := INI1.INI.cWS;
        END;
    END
    ELSE
    BEGIN
        CASE (cP) OF
            WHITE_PAWN:   Result := INI1.INI.cWPBS;
            WHITE_ROOK:   Result := INI1.INI.cWRBS;
            WHITE_KNIGHT: Result := INI1.INI.cWNBS;
            WHITE_BISHOP: Result := INI1.INI.cWBBS;
            WHITE_QUEEN:  Result := INI1.INI.cWQBS;
            WHITE_KING:   Result := INI1.INI.cWKBS;

            BLACK_PAWN:   Result := INI1.INI.cBPBS;
            BLACK_ROOK:   Result := INI1.INI.cBRBS;
            BLACK_KNIGHT: Result := INI1.INI.cBNBS;
            BLACK_BISHOP: Result := INI1.INI.cBBBS;
            BLACK_QUEEN:  Result := INI1.INI.cBQBS;
            BLACK_KING:   Result := INI1.INI.cBKBS;
        ELSE
            Result := INI1.INI.cBS;
        END;
    END;
END;

PROCEDURE TFCDForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2: String;bwm1,bwm2:boolean);
VAR
    cWhite, cBlack: String;
    nY, nX: Integer;
    aWhite: ARRAY[1..8, 1..8] OF Boolean;

    cL1:   String;
    cL2:   String;
    aStr1: ARRAY[1..8] OF String;
    aStr2: ARRAY[1..8] OF String;

    nL, nI, nJ: Integer;
    cTmp: String;
BEGIN
    { Initialize variables }
    cWhite := GetBoardPiece(TRUE,  EMPTY_SQUARE);
    cBlack := GetBoardPiece(FALSE, EMPTY_SQUARE);


    { Initalize color array }
    FOR nY := 1 TO 8 DO
    BEGIN
        FOR nX := 1 TO 8 DO
        BEGIN
            IF ((nY = 1) OR (nY = 3) OR (nY = 5) OR (nY = 7)) THEN
            BEGIN
                IF ((nX = 1) OR (nX = 3) OR (nX = 5) OR (nX = 7)) THEN
                    aWhite[nX, nY] := FALSE
                ELSE
                    aWhite[nX, nY] := TRUE;
            END
            ELSE
            BEGIN
                IF ((nX = 1) OR (nX = 3) OR (nX = 5) OR (nX = 7)) THEN
                    aWhite[nX, nY] := TRUE
                ELSE
                    aWhite[nX, nY] := FALSE;
            END;
        END;
    END;


    { Print top off board pieces }
    cL1 := cWhite;
    if bwm1 then cL2 := cWhite else cL2:=ini1.INI.cMoveInd;
    nL  := Length(cBP1);
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 9 TO nL DO
            cL1 := cL1 + GetOffPiece(cBP1[nI]);

        FOR nI := (nL + 1) TO 16 DO
            cL1 := cL1 + cWhite;

        FOR nI := 1 TO 8 DO
            cL2 := cL2 + GetOffPiece(cBP1[nI]);
    END
    ELSE
    BEGIN
        FOR nI := 9 TO 16 DO
            cL1 := cL1 + cWhite;

        FOR nI := 1 TO nL DO
            cL2 := cL2 + GetOffPiece(cBP1[nI]);

        FOR nI := (nL + 1) TO 8 DO
            cL2 := cL2 + cWhite;
    END;
    cL1 := cL1 + cWhite + TAB_STR + cWhite;
    cL2 := cL2 + cWhite + TAB_STR;
    if bwm2 then cl2:=cl2 + ini1.INI.cMoveInd else cl2:=cl2+cWhite;
    nL  := Length(cWP2);
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 9 TO nL DO
            cL1 := cL1 + GetOffPiece(cWP2[nI]);

        FOR nI := (nL + 1) TO 16 DO
            cL1 := cL1 + cWhite;

        FOR nI := 1 TO 8 DO
            cL2 := cL2 + GetOffPiece(cWP2[nI]);
    END
    ELSE
    BEGIN
        FOR nI := 9 TO 16 DO
            cL1 := cL1 + cWhite;

        FOR nI := 1 TO nL DO
            cL2 := cL2 + GetOffPiece(cWP2[nI]);

        FOR nI := (nL + 1) TO 8 DO
            cL2 := cL2 + cWhite;
    END;
    cL1 := cL1 + cWhite;
    cL2 := cL2 + cWhite;
    Add4(cL1);
    Add4(cL2);


    { Print top border }
    cTmp := INI1.INI.cTopLC +
            INI1.INI.cTopB + INI1.INI.cTopB + INI1.INI.cTopB + INI1.INI.cTopB +
            INI1.INI.cTopB + INI1.INI.cTopB + INI1.INI.cTopB + INI1.INI.cTopB +
            INI1.INI.cTopRC;
    Add4(cTmp + TAB_STR + cTmp);


    { Print board position }
    nI := 0;
    nJ := 9;
    FOR nY := 1 TO 8 DO
    BEGIN
        Dec(nJ);
        aStr1[nJ] := '';
        aStr2[nY] := '';
        FOR nX := 1 TO 8 DO
        BEGIN
            Inc(nI);
            aStr1[nJ] := aStr1[nJ] + GetBoardPiece(aWhite[nX, nY], cB1[nI]);
            aStr2[nY] := GetBoardPiece(aWhite[nX, nY], cB2[nI]) + aStr2[nY];
        END;
    END;
    FOR nI := 1 TO 8 DO
    BEGIN
        Add4(INI1.INI.cLeftB + aStr1[nI] + INI1.INI.cRightB +
             TAB_STR +
             INI1.INI.cLeftB + aStr2[nI] + INI1.INI.cRightB);
    END;


    { Print bottom border }
    cTmp := INI1.INI.cBotLC +
            INI1.INI.cBotB + INI1.INI.cBotB + INI1.INI.cBotB + INI1.INI.cBotB +
            INI1.INI.cBotB + INI1.INI.cBotB + INI1.INI.cBotB + INI1.INI.cBotB +
            INI1.INI.cBotRC;
    Add4(cTmp + TAB_STR + cTmp);


    { Print bottom off board pieces }
    cL2 := cWhite;
    if bwm1 then cl1:=ini1.INI.cMoveInd else cl1 := cWhite;

    nL  := Length(cWP1);
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 1 TO 8 DO
            cL1 := cL1 + GetOffPiece(cWP1[nI]);

        FOR nI := 9 TO nL DO
            cL2 := cL2 + GetOffPiece(cWP1[nI]);

        FOR nI := (nL + 1) TO 16 DO
            cL2 := cL2 + cWhite;
    END
    ELSE
    BEGIN
        FOR nI := 1 TO nL DO
            cL1 := cL1 + GetOffPiece(cWP1[nI]);

        FOR nI := (nL + 1) TO 8 DO
            cL1 := cL1 + cWhite;

        FOR nI := 9 TO 16 DO
            cL2 := cL2 + cWhite;
    END;
    cL1 := cL1 + cWhite + TAB_STR;
    if bwm2 then cl1:=cl1+cWhite else cl1:=cl1+ini1.INI.cMoveInd;
    cL2 := cL2 + cWhite + TAB_STR + cWhite;
    nL  := Length(cBP2);
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 1 TO 8 DO
            cL1 := cL1 + GetOffPiece(cBP2[nI]);

        FOR nI := 9 TO nL DO
            cL2 := cL2 + GetOffPiece(cBP2[nI]);

        FOR nI := (nL + 1) TO 16 DO
            cL2 := cL2 + cWhite;
    END
    ELSE
    BEGIN
        FOR nI := 1 TO nL DO
            cL1 := cL1 + GetOffPiece(cBP2[nI]);

        FOR nI := (nL + 1) TO 8 DO
            cL1 := cL1 + cWhite;

        FOR nI := 9 TO 16 DO
            cL2 := cL2 + cWhite;
    END;
    cL1 := cL1 + cWhite;
    cL2 := cL2 + cWhite;
    Add4(cL1);
    Add4(cL2);
END;

PROCEDURE TFCDForm.AdjustFile;
VAR
    cFind, cReplace, cNewFn, cLine: String;
    OrgFile, NewFile: TextFile;
    nP, nFind: Integer;
BEGIN
    IF (NOT FileExists(INI1.INI.cRTFFile)) THEN
       EXIT;

    cFind    := '\fnil ' + INI1.INI.fFCD4.Name + ';';
    nFind    := Length(cFind);

    cReplace := '\fswiss\fcharset2\fprq2 ' + INI1.INI.fFCD4.Name + ';';

    cNewFn   := ChangeFileExt(INI1.INI.cRTFFile, '.' + TMP1_EXT);

    AssignFile(OrgFile, INI1.INI.cRTFFile);
    Reset(OrgFile);

    AssignFile(NewFile, cNewFn);
    Rewrite(NewFile);

    WHILE (NOT EOF(OrgFile)) DO
    BEGIN
        Readln(OrgFile, cLine);

        IF (Length(cLine) > 0) THEN
        BEGIN
            nP := Pos(cFind, cLine);
            IF (nP > 0) THEN
            BEGIN
                IF (nP = 1) THEN
                BEGIN
                    Write(NewFile, cReplace);
                    Writeln(NewFile, Copy(cLine, nP + nFind, (Length(cLine) - nP - nFind) + 1));
                END
                ELSE
                BEGIN
                    Write(NewFile, Copy(cLine, 1, nP - 1));
                    Write(NewFile, cReplace);
                    Writeln(NewFile, Copy(cLine, nP + nFind, (Length(cLine) - nP - nFind) + 1));
                END;
            END
            ELSE
                Writeln(NewFile, cLine);
        END
        ELSE
            Writeln(NewFile, cLine);
    END;

    CloseFile(NewFile);
    CloseFile(OrgFile);

    DeleteFile(INI1.INI.cRTFFile);
    RenameFile(cNewFn, INI1.INI.cRTFFile);
END;

PROCEDURE TFCDForm.Button100Click(Sender: TObject);
var Keymapform:TKeyMapForm;
BEGIN
try
    KeyMapForm:=TKeyMapForm.Create(nil);
    KeyMapForm.SetupValues;
    KeyMapForm.ShowModal;
finally
    KeyMapForm.Free;
end;
END;

procedure tfcdform.SetColumnWidth;
var shandle:string;
smove:string;
cwc,cwn:string;
i,nmaxmovel,n1,n2:integer;
begin
 nMaxMoveL:=7;
 cwc:='W'; cwn:='9';
 shandle:='';
 for i:=1 to FICS_NAME_LEN do
 shandle:=shandle+cwc;
 shandle:=shandle+'('+cwn+cwn+cwn+cwn+')';
 smove:=cwn+cwn+'.'+cwc+' ';
 for i:=1 to nMaxMoveL do
 smove:=smove+cwc;
 smove:=smove+' {'+cwn+cwn+':'+cwn+cwn+'}';

 Canvas.Font:=fhandle;
 n1:=Canvas.TextWidth(shandle);
 Canvas.Font:=fMove;
 n2:=Canvas.TextWidth(smove);
 nColW:=n2;
// if n1>n2 then ncolw:=n1 else ncolw:=n2;
end;

procedure tfcdform.backupfont;
begin
 fBkupFont:=Canvas.Font;
end;

procedure tfcdform.restorefont;
begin
 Canvas.Font:=fbkupfont;
end;

function tfcdform.AdjustTxtW(s:string;nGoal:integer):string;
var nl,ns,i,nc:integer;
begin
 s:=trim(s);
 nL:=Canvas.TextWidth(s);
 if nL<nGoal then
 begin
  nS:=Canvas.TextWidth(' ');
  nc:=(nGoal-nl) div ns;
  for i:=1 to nc do s:=s+' ';
 end;
 Result:=s;
end;


procedure TFCDForm.Button7Click(Sender: TObject);
var PieceMapform:TPiecemapForm;
begin
    try
    piecemapform:=tpiecemapform.create(self);
    piecemapform.GoRTF;
    PieceMapForm.SetupValues;
    PieceMapForm.ShowModal;
    finally piecemapform.free;
    end;
end;

END.
