(*============================================================================*)
(* GAMELIST.PAS -- Game list window                                           *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan                                           *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.05: Reseted game list caption in ResetAll().                       *)
// 2004.12.29: Made listbox1 virtual. This speeds up game list updates 10 times
// 2004.12.30: 1) Replaced listbox with TListView because listbox is limited to
//                32767 items on win98
//             2) Added Outofdate, restoreIndex,backupIndex
// 2005.01.08: 1) Introdused states, added setCaption that sets game list caption
//                according to the given state.
//             2) Changed format in which stat is displayed (now it's +x-y=z)
(*============================================================================*)
UNIT GAMELIST;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
//    StdCtrls,
    GLOBAL, TOOLBOX,INI1,  ComCtrls,misc;


TYPE
TGLState=(GlNormal,GlFilter,GlDoubles,GLCurrent,GlDelete,GlShort,glBroken,glDabort,glSelect);
    TGameListForm = CLASS(TForm)
    ListBox1: TListView;

        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE ClearAll;
        PROCEDURE ResetAll;
        FUNCTION  AddItem: Integer;
    procedure ListBox1Resize(Sender: TObject);
    procedure Outofdate;
    procedure backupIndex;
    procedure restoreindex;
    procedure SetCaption(fState:TGlState);
        PRIVATE
        { Private declarations }
        fOldIndex:longint;
        fCurState:TGLstate;
        PUBLIC
        cInfo:string;
        { Public declarations }
    END;

VAR
    GameListForm: TGameListForm;

IMPLEMENTATION
const
 T_Normal = 'Games';
 T_Filter = 'Filtered games';
 T_Doubles = 'Doubles';
 T_Delete  = 'Deleted games';
 T_short  =  'Short games';
 T_Broken =  'Games with broken results';
 T_DAbort =  'Doubles (incomplete)';
 T_Select =  'Selected';

{$R *.DFM}

PROCEDURE TGameListForm.FormCreate(Sender: TObject);
BEGIN
//    Werror('Creating game list Form');
    ListBox1.Color       := INI1.INI.nGameListB;
    ListBox1.Font.Color  := INI1.INI.fGameListF.Color;
    ListBox1.Font.Height := INI1.INI.fGameListF.Height;
    ListBox1.Font.Name   := INI1.INI.fGameListF.Name;
    ListBox1.Font.Pitch  := INI1.INI.fGameListF.Pitch;
    ListBox1.Font.Size   := INI1.INI.fGameListF.Size;
    ListBox1.Font.Style  := INI1.INI.fGameListF.Style;
    cInfo:='';
    ResetAll;

    IF (INI1.INI.nGameListS <> 2) THEN
    BEGIN
        SetBounds(INI1.INI.nGameListL, INI1.INI.nGameListT,
                  INI1.INI.nGameListW, INI1.INI.nGameListH);
    END;
    listbox1.Column[0].Width:=listbox1.ClientWidth;
    WindowState := TOOLBOX_IntToWinState(INI1.INI.nGameListS);
    fCurState:=GLNormal;
    SetCaption(fCurState);
END;

PROCEDURE TGameListForm.ClearAll;
BEGIN
    listbox1.items.Count:=0;
    listbox1.Clear;
    cInfo:='';
    setcaption(GlNormal);
END;

PROCEDURE TGameListForm.ResetAll;
BEGIN
    foldindex:=-1;
    ClearAll;
    ListBox1.Enabled := FALSE;
END;

function tgamelistform.AddItem:integer;
begin
 listbox1.items.Count:=listbox1.items.Count+1;
 result:=listbox1.items.Count;
end;

procedure TGameListForm.ListBox1Resize(Sender: TObject);
begin
 listbox1.Column[0].Width:=Listbox1.ClientWidth;
end;

procedure tgamelistform.Outofdate;
begin
 listbox1.Invalidate;
end;

procedure tgamelistform.backupIndex;
begin
 foldIndex:=listbox1.ItemIndex;
end;

procedure tgamelistform.restoreindex;
begin
 listbox1.ItemIndex:=foldIndex;
end;

procedure tgamelistform.SetCaption(fState:TGLState);
var s:string;
i:integer;
begin
 if Fstate=GLCurrent then fstate:=FCurState;
 i:=listbox1.ItemIndex+1;
 if i>0 then s:=' ('+Inttostr(i)+'/'+Inttostr(listbox1.Items.Count)+')'
 else s:=' ('+Inttostr(listbox1.Items.Count)+')';
 case fstate of
 GlNormal: Caption:=T_Normal+s+cInfo;
 GlFilter: Caption:=T_Filter+s+cInfo;
 GlDoubles: Caption:=T_Doubles+s;
 GLDelete:  Caption:=T_Delete+s;
 GlShort :  Caption:=T_short+s;
 GlBroken:  Caption:=T_Broken+s;
 GlDAbort: Caption:=T_Dabort+s;
 GlSelect: Caption:=T_Select+s;
 end;
 FcurState:=Fstate;
end;

END.
