(*============================================================================*)
(* GLOBAL.PAS -- Global variables                                             *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 2001.12.11: Added Crazyhouze_Id to identify zh games                       *)
(* 2002.01-05: Defined Windows Messages WM_EngineRead-WM_EngineTestStop       *)
(*             Defined Engine Constants (Mate evaluation, etc.)               *)
(*             Defined Constants for default directory structure              *)
(*             Defined Constants for default board configuration              *)
(*                                                                            *)
(*             Defined Records:                                               *)
(*             TEngineOpt --- Crazyhouse Engine options                       *)
(*             TEngineLayout --- Layout of Engine Window                      *)
(*             TEngineLine   --- Stores Engine Analysis of current position   *)
(*             TFilterRec --- Stores parameters of (nonpositional) search     *)
(*                                                                            *)
(*             Added functions that load/free board bitmap graphics           *)
(*             Added functions that check if directory contains valid graphics*)
(*             set                                                            *)
(*                                                                            *)
(*             Defined global variables: Line -- Defaultmaxdate               *)
(*2002.11.27 1) Added progbar: global variable for progress bar form          *)
(*           2) Added bairdrop. bAirdrop=true => airdrops are allowed         *)
(*2004.12.02 Added Default_DB_SAVE_DEPTH=20                                   *)
//2005.01.16 Changed FICS_NAME_LEN to 18 to accomodate TyrionsApprentice
//2005.01.02 Added fMainHandle, a handle of the main form to which all messages
//           are sent
//2005.02.17 Added StartMove used by replay functions 
//2005.02.22 Major cleanup: removed a bunch of global variables used only in one
//           place
//2005.02.28 Modified TfilterRec to include bSelected and nRatingMode
(*============================================================================*)
UNIT GLOBAL;

INTERFACE

USES Classes,Messages,Windows,Graphics,controls,sysutils,filectrl;

const
MATE_EVAL=9000;
INFTY=32000;
DIR_SEP='\';
BUFFERSIZE=8096;
WM_ENGINEDEAD=WM_APP+400;
WM_ENGINEREAD=WM_APP+401;
WM_ENGINERUN=WM_APP+601;
WM_ENGINESTOP=WM_APP+402;
WM_ENGINEUNLOAD=WM_APP+603;
WM_ENGINEOPTIONS=WM_APP+604;
WM_ENGINEDEFAULT=WM_APP+605;
WM_ENGINESUSPEND=WM_APP+606;
WM_ENGINERESUME=WM_APP+607;
WM_ENGINEANN=WM_APP+608;
WM_ENGINEANNSTOP=WM_APP+700;
WM_ENGINETEST=WM_APP+701;
WM_ENGINETESTSTOP=WM_APP+702;
WM_PROGRESS_STEP=WM_APP+703;
WM_BPGNError=WM_APP+704;
WM_ProgressUpdate=WM_APP+705;
WM_BPGNinfo=WM_APP+706;
WM_UpdateDB=WM_APP+707;
WM_UpdateTree=WM_App+800;
WM_Cancel_OP=WM_APP+801;

MAX_ENGINE=5;

SJENG_ON='ON';
SJENG_OFF='OFF';
SJENG_RESUME='RESUME';
Crazyhouse_ID='crazyhouse';

    PROGRAM_TITLE  = 'BPGN viewer';
    BOARD1_TITLE   = 'Board #1';
    BOARD2_TITLE   = 'Board #2';
    CLOCK_TITLE    = 'Edit clock';
    HANDLE_TITLE   = 'Edit handle';
    RATING_TITLE   = 'Edit rating';
    PIECEMAP_TITLE = 'Edit move piece keymap';
    TXTMAP_TITLE   = 'Edit move piece keymap (TXT)';
    TXTCMAP_TITLE  = 'Edit chess piece keymap (TXT)';

    INI_EXT        = 'INI';
    BPGN_EXT       = 'BPGN';
    TMP_EXT        = '@@@';
    TMP1_EXT       = '###';
    RTF_EXT        = 'RTF';
    TXT_EXT        = 'TXT';
    DB_HEADER_EXT  = '.bh';
    db_white_EXT= '.bpw';
    db_black_ext= '.bpb';
    Default_db_save_depth=20;
    Default_Sh_Game_Max=5;

    PROGBAR_SHOW_LIM=100;
    MAX_STRING     = 65000; //65536;
    MAX_LINE       = 100;
    MAX_GAME       = 10000;
    MAX_NEXT       = 50;
    MAX_MEMO_CHAR  = 64000;
    MAX_BPGN_SAVE  = 10;
    MAX_BPGN_LEN   = 79;
    MAX_POSITION = 10000;

    MAX_EXTRACT_NUM= 20;
    DEF_EXTRACT_NUM= 10;
    EXTRACT_ERRFN  = 'EXTRACT.ERR';
    SAVE_HEADER_BUF= 65000;

    WIDTH_GAP      = 5;
    HEIGHT_GAP     = 3;

    NAVIGATE_WIDTH = 484;
    NAVIGATE_HEIGHT= 25;

    FICS_NAME_LEN  = 18;

    PLAYERW_NAME   = 'WHITE';
    PLAYERB_NAME   = 'BLACK';

    TIMEW_STR      = '00:00';
    TIMEB_STR      = '00:00';


    PIECE_ORG_SIZE = 48;
    CLOCK_SPACEY   = 10;

    SPACEW_GAP     = 2;
    SPACEH_GAP     = 2;
    SPACEB_GAP     = 3;

    BPGN_NORMAL    = 0;
    BPGN_STRING    = 1;
    BPGN_INTEGER   = 2;
    BPGN_COMMENT1  = 3;
    BPGN_COMMENT2  = 4;
    BPGN_NAG       = 5;
    BPGN_SYMBOL    = 6;

    BPGN_WHITE_WINS= '1-0';
    BPGN_BLACK_WINS= '0-1';
    BPGN_DRAW      = '1/2-1/2';
    BPGN_UNKNOWN   = '*';

    DEFAULT_EVENT  = 'fics rated bughouse match';
    DEFAULT_SITE   = 'fics, Oklahoma City, OK USA';
    DEFAULT_DATE   = '????.??.??';
    DEFAULT_WHITEA = 'WhiteA';
    DEFAULT_BLACKA = 'BlackA';
    DEFAULT_WHITEB = 'WhiteB';
    DEFAULT_BLACKB = 'BlackB';
    DEFAULT_TIME   = '180';
    DEFAULT_INC    = '0';
    DEFAULT_RATING = '----';
    DEFAULT_REASON = '{*}';
    DEFAULT_RESULT = '*';
    DEFAULT_SAVEPOS_DEPTH=20;

    TAG_OK         = ' ';
    TAG_DELETE     = 'D';
    TAG_Select     = '+';

    BPGN_OCCUR_STR = '<Occurrence: ';

    BPGN_EVENT_STR = '[Event ';
    BPGN_SITE_STR  = '[Site ';
    BPGN_DATE_STR  = '[Date ';
    BPGN_WHITEA_STR= '[WhiteA ';
    BPGN_BLACKA_STR= '[BlackA ';
    BPGN_WHITEB_STR= '[WhiteB ';
    BPGN_BLACKB_STR= '[BlackB ';
    BPGN_TIMEC_STR = '[TimeControl ';
    BPGN_RESULT_STR= '[Result ';
    BPGN_OPENA_STR = '[BECOA ';
    BPGN_OPENB_STR = '[BECOB ';
    BPGN_WHITEA_ELO= '[WhiteAElo ';
    BPGN_BLACKA_ELO= '[BlackAElo ';
    BPGN_WHITEB_ELO= '[WhiteBElo ';
    BPGN_BLACKB_ELO= '[BlackBElo ';
    BPGN_SETUP_STR = '[Setup ';
    BPGN_FEN_STR = '[FEN ';

    BPGN_REASON_NUM=15;
    BPGN_LAG_STR = '[Lag ';

    UNKNOWN_RATING = '????';

    TAB_ASCII_NUM  = 9;
    CR_ASCII_NUM   = 13;
    LF_ASCII_NUM   = 10;

    TAB_ASCII_CHAR = #9;
    CR_ASCII_CHAR  = #13;
    LF_ASCII_CHAR  = #10;
    ESC_ASCII_CHAR = #27;
    COMMENT_CR_CHAR= #254;

    TAB_STR        = TAB_ASCII_CHAR;
    EOL_STR        = CR_ASCII_CHAR + LF_ASCII_CHAR;
    UEOL_STR       = LF_ASCII_CHAR;

    INIT_GAME_POS  = 'RNBQKBNRPPPPPPPP                                pppppppprnbqkbnr';
    INIT_WHITE_PC  = '';
    INIT_BLACK_PC  = '';
    BLANK_GAME_POS = '                                                                ';

    WHITE_COLOR    = 1;
    BLACK_COLOR    = 2;
    EMPTY_COLOR    = 0;
    UNKNOWN_COLOR  = -1;

    WHITE_PAWN     = 'P';
    WHITE_ROOK     = 'R';
    WHITE_KNIGHT   = 'N';
    WHITE_BISHOP   = 'B';
    WHITE_QUEEN    = 'Q';
    WHITE_KING     = 'K';
    MOVE_INDICATOR = '>';

    BLACK_PAWN     = 'p';
    BLACK_ROOK     = 'r';
    BLACK_KNIGHT   = 'n';
    BLACK_BISHOP   = 'b';
    BLACK_QUEEN    = 'q';
    BLACK_KING     = 'k';

    EMPTY_SQUARE   = ' ';

    PAWN_INDEX     = 1;
    ROOK_INDEX     = 2;
    KNIGHT_INDEX   = 3;
    BISHOP_INDEX   = 4;
    QUEEN_INDEX    = 5;
    KING_INDEX     = 6;
    EMPTY_INDEX    = 0;
    UNKNOWN_INDEX  = -1;

    MOVE_REGULAR   = ' ';
    MOVE_O_O       = 'K';
    MOVE_O_O_O     = 'Q';
    MOVE_PROMOTE   = 'P';
    MOVE_ENPASANT  = 'E';
    MOVE_DROP      = 'D';

    MAX_MOVE       = 1000;
    MAX_DIR        = 4;
    MAX_DIST       = 7;
    MAX_KNIGHT     = 8;
    MAX_KING       = 8;

    FICS_AVERAGE_RATING=1720;
    DATE_TIME_TO_MS=1000*60*60*24;
    MOVE_OLD_DELAY=200;

DEFcWorkBpgn='work.bpgn';

Relativeboarddir='Square\';
Relativepiecedir='Piece\Bitmap\';
RelativeBgdir='BGround\';
Relativethemedir='Theme\';

DEFnWhtSquare=9094641; //$008AC5F1;
DEFnBlkSquare=1589108; //$00183f74;
DEFnNumberColor=clBlack;
DEFnPcSquare= clBtnFace;
DEFnboardoutline=ClBtnFace;
DEFBgColor=clBtnFace;
DEFnWaitf=clBlack;
DEFnwaitb=clBtnFace;
DEFnturnb=clBlack;
DEFnturnf=clWhite;
DEFnHandleB=clBtnFace;
DefFormB=clsilver; //14211288;

defusebmp=true;
defusepiecebmp=true;
defuseboardbmp=true;
defbmpFile='cork.bmp';
defp_dir='wcn\';
defb_dir='paper1\';
defgraphhome='..\bugstuff\';

deffHandle_Color = clBlack;
deffHandle_Height= -11;
deffHandle_Name= 'MS Sans Serif';
deffHandle_Pitch= fpDefault;
deffHandle_Size= 10;

deffClock_Color  = clBlack;
deffClock_Height = -11;
deffClock_Name   = 'MS Sans Serif';
deffClock_Pitch  = fpDefault;
deffClock_Size   = 10;

rmAll=7;
rmAllTeams=6;
rmPteam=4;
rmOteam=5;
rmPlayer=0;
rmOpponent=1;
rmPartner=2;
rmPardOp=3;


TYPE
TintBoard= array [1..8] of longint;
TEngLine=record
score:string;
Depth:string;
Line:string;
Move:string;
time:string;
nodes:string;
end;

TFilterRec=record
Player1:string;
Player2:string;
Player3:string;
Player4:string;
MinRMin   :integer;
MinRMax   :integer;
MaxRMin:integer;
MaxRMax:integer;
RAvMin :integer;
RAvMax :integer;
TimeMin:integer;
TimeMax:integer;
dateMin:TdateTime;
dateMax:TdateTime;
bOpponent:boolean;
bPartner:boolean;
bWhite:boolean;
bBlack:boolean;
bWin:boolean;
bLoss:boolean;
bdraw:boolean;
bunknown:boolean;
brated:boolean;
bunrated:boolean;
bbug:Boolean;
bcrazy:boolean;
possearchmode:integer;
bautosearch:boolean;
boardnum:integer;
position:TintBoard;
bSelected:boolean;
nRatingMode:integer;
end;

Pbmp=^TPicture;

Tpieces=array[1..2,1..6] of Tbitmap;
Tboardbmp=array[1..2] of Tbitmap;

TEnglinearr=array[1..MaX_NEXT] of Tengline;

Tbuffer=array[0..BUFFERSIZE] of char;

TEngineOpt=record
name:string;
comline:string;
etype:integer;
prompt:string;
reset:string;
itime:string;
bfilter:boolean;
banal:boolean;
end;

TEngineOptArr = array [1..MAX_ENGINE] of TEngineOpt;

TEngineLayout=record
nEngineL: Integer;
nEngineT: Integer;
nEngineW: Integer;
nEngineH: Integer;
nEngineS: Integer;
end;

TEngineLayoutArr = array [1..MAX_ENGINE] of TEngineLayout;

    FileString = ARRAY[0..(MAX_STRING - 1)] OF Char;

    BoardArray   = ARRAY[1..8, 1..8] OF Char;
    PromoteArray = ARRAY[1..8, 1..8] OF Boolean;

    MoveRecord = RECORD
        nFromX:                   Integer;
        nFromY:                   Integer;
        nToX:                     Integer;
        nToY:                     Integer;
        cPiece:                   Char;
        cPromote:                 Char;
        cCapture:                 Char;
        cType:                    Char;
        bCapturedIsPromotedPiece: Boolean;
    END;

    BPGNRecord = RECORD
        nWTime:      longint;
        nBTime:      longint;
        nWLag :      longint;
        nBLag :      longint;

        aKingBX:     ARRAY[1..2] OF Integer;
        aKingBY:     ARRAY[1..2] OF Integer;

        aKCastle:    ARRAY[1..2] OF Boolean;
        aQCastle:    ARRAY[1..2] OF Boolean;

        mPrev:       MoveRecord;

        aBoard:      BoardArray;
        aPromote:    PromoteArray;
        acPiece:     ARRAY[1..2,1..8] OF Char;
        anPiece:     ARRAY[1..2,1..8] OF Integer;
        bWhitesMove: Boolean;

        cCaption:    String;
    END;


var
{ nTotalLag:integer;
 nStartIndex:integer;
 sIdealT,sDelayT,sTProblem:string;
 StartPlay,EndPlay,} StartMove:TdateTime;
// nLoadDelay:longint;
 fmainHandle:THandle;
 Line:array [1..MAX_MOVE,1..2] of string;
 LineCnt:array [1..2] of integer;
 usebitmap:boolean;
 usepiecebitmap:boolean;
 useboardbitmap:boolean;
 Tecumseh:boolean;
 graphhome:string=defgraphhome;
 bg_file:string;
 piece_dir:string;
 board_dir:string;
 lastmove:string;
 boardbmp:Tboardbmp;
 pieces:TPieces;
 Piecesbg:Tbitmap;
 Glyphs: array [1..10] of pbmp;
 defaultmindate:Tdatetime;
 defaultmaxdate:Tdatetime;
 bAirDrop:boolean;
 cCurrentInfo:string;


function readpieces(var p:Tpieces;s:string):boolean;
procedure freepieces(var p:Tpieces);
procedure createpieces(var p:Tpieces);
function readboard(var bd:Tboardbmp;s:string):boolean;
procedure freeboard(var bd:Tboardbmp);
procedure createboard(var bd:Tboardbmp);
function validpiecedir(s:string):boolean;
function validboarddir(s:string):boolean;
function validgraphhome(s:string):boolean;
procedure resetfilterrec(var fr:TfilterRec);
procedure inttoxy(n:integer;var x,y:integer);

Implementation
{$I-}

procedure freepieces(var p:Tpieces);
var i,j:integer;
begin
 for i:=1 to 2 do
 for j:=1 to 6 do if p[i,j]<>nil then P[i,j].free;
end;

procedure freeboard(var bd:Tboardbmp);
var i:integer;
begin
 for i:=1 to 2 do if bd[i]<>nil then boardbmp[i].free;
end;

procedure createboard(var bd:Tboardbmp);
var i:integer;
begin
 for i:=1 to 2 do bd[i]:=Tbitmap.Create;
end;

procedure createpieces(var p:Tpieces);
var i,j:integer;
begin
 for i:=1 to 2 do
  for j:=1 to 6 do
  P[i,j]:=Tbitmap.create;
end;

function readpieces(var p:Tpieces;s:string):boolean;
var i,j:integer;
begin
 try
  P[1,6].loadfromfile(s+'wking.bmp');
  P[1,2].loadfromfile(s+'wrook.bmp');
  P[1,3].loadfromfile(s+'wknight.bmp');
  P[1,4].loadfromfile(s+'wbishop.bmp');
  P[1,5].loadfromfile(s+'wqueen.bmp');
  P[1,1].loadfromfile(s+'wpawn.bmp');
  P[2,2].loadfromfile(s+'brook.bmp');
  P[2,3].loadfromfile(s+'bknight.bmp');
  P[2,4].loadfromfile(s+'bbishop.bmp');
  P[2,5].loadfromfile(s+'bqueen.bmp');
  P[2,6].loadfromfile(s+'bking.bmp');
  P[2,1].loadfromfile(s+'bpawn.bmp');
{for i:=1 to 2 do
for j:=1 to 6 do pieces[i,j].transparent:=true;}
  Result:=true;
 except
  Result:=false;
 end;
end;

function readboard(var bd:Tboardbmp;s:string):boolean;
var i:integer;
begin
 try
  bd[1].loadfromfile(s+'wsquare.bmp');
  bd[2].loadfromfile(s+'bsquare.bmp');
  Result:=true;
 except
  Result:=false;
 end;
end;

function validgraphhome(s:string):boolean;
begin
 if not directoryexists(s+relativepiecedir) then begin result:=false;exit;end;
 if not directoryexists(s+relativeboarddir) then begin result:=false;exit;end;
 result:=true;
end;

function validpiecedir(s:string):boolean;
begin
 if not fileexists(s+'\wpawn.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\wrook.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\wknight.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\wbishop.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\wqueen.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\wking.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\bpawn.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\brook.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\bknight.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\bbishop.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\bqueen.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\bking.bmp') then begin Result:=false;exit;end;
 Result:=True;
end;

function validboarddir(s:string):boolean;
begin
 if not fileexists(s+'\wsquare.bmp') then begin Result:=false;exit;end;
 if not fileexists(s+'\bsquare.bmp') then begin Result:=false;exit;end;
 Result:=true;
end;

procedure resetFilterrec(var fr:TfilterRec);
begin
fr.Player1:='';
fr.Player2:='';
fr.Player3:='';
fr.Player4:='';
fr.MinRMin:=0;
fr.MinRMax:=Infty;
fr.MaxRMin:=0;
fr.MaxRMax:=INFTY;
fr.RAvMin:=0;
fr.RAvMax:=INFTY;
fr.TimeMin:=0;
fr.TimeMax:=Infty;
fr.datemin:=defaultmindate;
fr.datemax:=defaultmaxdate;
fr.bOpponent:=false;
fr.bPartner:=false;
fr.bwhite:=true;
fr.bBlack:=true;
fr.bwin:=true;
fr.bloss:=true;
fr.bunknown:=true;
fr.bdraw:=true;
fr.brated:=true;
fr.bunrated:=true;
fr.bbug:=true;
fr.bcrazy:=true;
fr.possearchmode:=3;
fr.bautosearch:=false;
fr.bSelected:=false;
fr.nRatingMode:=rmAll;
end;

procedure inttoxy(n:integer;var x,y:integer);
begin
 x:=n div 10;
 y:=n mod 10;
end;

END.
