(*============================================================================*)
(* INI.PAS -- .INI file object                                                *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: Added control window's width and height.                       *)
(* 1998.11.04: Added previous next move window.                               *)
(* 1998.11.05: 1) Added show ratings under game list options.                 *)
(*             2) Added show time under game list options.                    *)
(*             3) Added show increment under game list options.               *)
(*             4) Added show result under game list options.                  *)
(*             5) Added show square outline under board options.              *)
(* 1998.11.07: 1) Added cNewPath.                                             *)
(*             2) Added cDeletePath.                                          *)
(* 1998.11.09: Added show game # under game list options.                     *)
(* 1998.11.11: Added user select maximum # of positions per bpgn game.        *)
(* 1998.11.12: Added show event under game list options.                      *)
(* 1998.11.17: 1) Added FCD stuff.                                            *)
(*             2) Added view RTF file.                                        *)
(* 1998.11.18: 1) Added TXT stuff.                                            *)
(*             2) Added view TXT file.                                        *)
(* 1998.11.21: Added chess font keymap.                                       *)
(* 1998.11.22: Changed the chess font keymap to hold up to 2 characters.      *)
(* 1998.11.23: 1) Fixed a bug where it didn't load more than 1 character for  *)
(*                chess font keymap.                                          *)
(*             2) Added include handle, move, annotation, time, comment,      *)
(*                display board, and result for convert to RTF/TXT function.  *)
(*             3) Added remove all destination comments before extraction.    *)
(*             4) Added generate destination occurrence comment.              *)
(*             5) Added edit move piece keymap in RTF function.               *)
(* 1998.11.24: 1) Added include move piece keymap.                            *)
(*             2) Removed include move piece keymap.                          *)
(*             3) Added move piece keymap (TXT).                              *)
(*             4) Added chess piece keymap (TXT).                             *)
(* 1998.11.25: Added use zeros instead of Os in castling move to edit move    *)
(*             piece keymap function, RTF function, and TXT function.         *)
(* 1998.11.27: Added word wrap into comment/option.                           *)
(* 2001.12   : Renamed file to ini1.pas to resolve Scope conflicts in Delphi 5*)
(* 2002.01-05: 1) Added Engine1-Engine5 Options/Layout                        *)
(*             2) Added Bitmap Graphics                                       *)
(*             3) Added Initialization code that tries to find Directory with *)
(*                bitmap graphics                                             *)
(*             4) Added cworkBpgn --- Work bpgn filename                      *)
(*             5) Added Tecumseh --- unused var intended for jokes on Tecumseh*)
(* 2004.11.29: Added bairdropini---responsible for handling airdrops          *)
(* 2004.12.02: Added savedepth ---number of moves to save in db               *)
(* 2004.12.03: 1) Added bfullstat                                             *)
(*             2) Added nMoveOrder --- sorting column for movelist in filter  *)
(* 2004.12.08  Added nfilterL-nfilterH, the coordinates of filter window      *)
(*============================================================================*)
UNIT Ini1;

INTERFACE

USES Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     ExtCtrls, StdCtrls, Menus, ComCtrls, IniFiles,
     GLOBAL,registry;


TYPE
    TConfig = CLASS(TObject)
        CONSTRUCTOR Init;
        DESTRUCTOR  Destroy;  override;
        PROCEDURE   InitConfig;

        PROCEDURE SetFilename(cFn: String);
        PROCEDURE ReadAll;
        PROCEDURE WriteAll;
        PROCEDURE WriteLayout;
        PROCEDURE WriteOptions;

        PRIVATE

        PUBLIC

        tmpgraphhome:string;
        tecumseh:boolean;
           EngineOpt:TEngineOptArr;
           EngineLay:TEngineLayoutArr;
           defeng:integer;
           usebmp:boolean;
           usepiecebmp:boolean;
           useboardbmp:boolean;
           btheme:boolean;
           b_dir:string;
           p_Dir:string;
           bmpfile:string;
           stheme:string;


            cName: String;
            cFile: String;

            nMainL: Integer;
            nMainT: Integer;
            nMainW: Integer;
            nMainH: Integer;
            nMainS: Integer;

            nWhtSquare:  TColor;
            nWhtSOutline:TColor;
            nWhtOutline: TColor;
            nWhtFill:    TColor;

            nBlkSquare:  TColor;
            nBlkSOutline:TColor;
            nBlkOutline: TColor;
            nBlkFill:    TColor;

            bOutlinePC:   Boolean;
            nClickColor:  TColor;

            bShwPCCount:  Boolean;
            nNumberColor: TColor;
            nPcSquare:    TColor;

            bFlipBd:      Boolean;
            bAlwaysQ:     Boolean;
            bDragMove:    Boolean;
            bSmartMove:   Boolean;
            bActualBoard: Boolean;
            nBoardOutline:TColor;
            nBoardWindow: TColor;
            bShowSOutline:Boolean;
            bShowLag:boolean;
            bShowTenth:boolean;
            bairdropini:boolean;
            savedepth:integer;
//            nShortg:integer;
            bOpenTree:boolean;
            bROgames:boolean;
            bfullstat:boolean;
            nMoveOrder:integer;

            nWaitF: TColor;
            nWaitB: TColor;
            nTurnF: TColor;
            nTurnB: TColor;
            fClock: TFont;

            nHandleB: TColor;
            fHandle:  TFont;

            nBoard1L: Integer;
            nBoard1T: Integer;
            nBoard1W: Integer;
            nBoard1H: Integer;
            nBoard1S: Integer;

            nBoard2L: Integer;
            nBoard2T: Integer;
            nBoard2W: Integer;
            nBoard2H: Integer;
            nBoard2S: Integer;

            nNavigateL: Integer;
            nNavigateT: Integer;
            nNavigateSticky: boolean;
            nNavigateW:integer;
            nNavigateH:integer;
            nNavigateS: Integer;
            bNavigateSimple:boolean;

            nGameListL: Integer;
            nGameListT: Integer;
            nGameListW: Integer;
            nGameListH: Integer;
            nGameListS: Integer;
            nGameListB: TColor;
            fGameListF: TFont;
            bShowGameNum: Boolean;
            bShowEvent:   Boolean;
            bShowRatings: Boolean;
            bShowTime:    Boolean;
            bShowInc:     Boolean;
            bShowResult:  Boolean;

            nNextMoveL: Integer;
            nNextMoveT: Integer;
            nNextMoveW: Integer;
            nNextMoveH: Integer;
            nNextMoveS: Integer;
            nNextMoveB: TColor;
            fNextMoveF: TFont;

            nPrevMoveL: Integer;
            nPrevMoveT: Integer;
            nPrevMoveW: Integer;
            nPrevMoveH: Integer;
            nPrevMoveS: Integer;
            nPrevMoveB: TColor;
            fPrevMoveF: TFont;

            nCommentL: Integer;
            nCommentT: Integer;
            nCommentW: Integer;
            nCommentH: Integer;
            nCommentS: Integer;
            nCommentB: TColor;
            fCommentF: TFont;
            bCommentWR:Boolean;

            bsoundon:boolean;
            csoundmove:string;
            csoundcap:string;
            csoundcheck:string;
            csoundend:string;
            

            nEngineB: TColor;
            fEngineF: TFont;
            bEngineWR:Boolean;
            nEngineAT:Real;
            nEngineTT:Real;
            nEngineIT:Real;
            nEngineET:Real;
            bEngineSM:Boolean;

            nVATL: Integer;
            nVATT: Integer;
            nVATW: Integer;
            nVATH: Integer;

            nVEXL: Integer;
            nVEXT: Integer;
            nVEXW: Integer;
            nVEXH: Integer;

            nRTFL:   Integer;
            nRTFT:   Integer;
            nRTFW:   Integer;
            nRTFH:   Integer;
            cRTFDir: String;

            nTXTL:   Integer;
            nTXTT:   Integer;
            nTXTW:   Integer;
            nTXTH:   Integer;
            cTXTDir: String;

            nStatL:integer;
            nStatT:integer;
            nStatW:integer;
            nStatH:integer;

            nfilterL:integer;
            nfilterT:integer;
            nfilterW:integer;
            nfilterH:integer;
            bFilterSt:boolean;
            nTreeL:integer;
            nTreeT:integer;
            nTreeW:integer;
            nTreeH:integer;
            bTreeFil:boolean;
//            bTreeAuto:boolean;

            bSaveOptionsOnExit: Boolean;
            bSaveLayoutsOnExit: Boolean;
            cNewPath:    String;
            cOpenPath:   String;
            cDeletePath: String;
            cSaveAsPath: String;
            cExportPath: String;
            cWorkBpgn  :String;

            cHandle:        String;
            cSource:        String;
            cDestination:   String;
            nMoveNumber:    Integer;
            bRemoveComment: Boolean;
            bGenOccurrence: Boolean;

            fFCD1:    TFont;
            fFCD2:    TFont;
            fFCD3:    TFont;
            fFCD4:    TFont;
            fFCD5:    TFont;

            bRTFHand: Boolean;
            bRTFMove: Boolean;
            bRTFAnno: Boolean;
            bRTFTime: Boolean;
            bRTFNote: Boolean;
            bRTFDisp: Boolean;
            bRTFRest: Boolean;
            bRTFZero: Boolean;

            cRTFFile: String;

            bTXTHand: Boolean;
            bTXTMove: Boolean;
            bTXTAnno: Boolean;
            bTXTTime: Boolean;
            bTXTNote: Boolean;
            bTXTDisp: Boolean;
            bTXTRest: Boolean;
            bTXTZero: Boolean;

            cTXTWP: String;
            cTXTWN: String;
            cTXTWB: String;
            cTXTWR: String;
            cTXTWQ: String;
            cTXTWK: String;

            cTXTBP: String;
            cTXTBN: String;
            cTXTBB: String;
            cTXTBR: String;
            cTXTBQ: String;
            cTXTBK: String;

            cTXTCWP: String;
            cTXTCWN: String;
            cTXTCWB: String;
            cTXTCWR: String;
            cTXTCWQ: String;
            cTXTCWK: String;

            cTXTCBP: String;
            cTXTCBN: String;
            cTXTCBB: String;
            cTXTCBR: String;
            cTXTCBQ: String;
            cTXTCBK: String;

            cTXTFile: String;

            cWP: String;
            cWN: String;
            cWB: String;
            cWR: String;
            cWQ: String;
            cWK: String;

            cBP: String;
            cBN: String;
            cBB: String;
            cBR: String;
            cBQ: String;
            cBK: String;
            bZero: Boolean;

            cWPWS:   String;
            cWPBS:   String;
            cWNWS:   String;
            cWNBS:   String;
            cWBWS:   String;
            cWBBS:   String;
            cWRWS:   String;
            cWRBS:   String;
            cWQWS:   String;
            cWQBS:   String;
            cWKWS:   String;
            cWKBS:   String;

            cBPWS:   String;
            cBPBS:   String;
            cBNWS:   String;
            cBNBS:   String;
            cBBWS:   String;
            cBBBS:   String;
            cBRWS:   String;
            cBRBS:   String;
            cBQWS:   String;
            cBQBS:   String;
            cBKWS:   String;
            cBKBS:   String;
            cMoveInd:String;

            cWS:     String;
            cBS:     String;

            cTopLC:  String;
            cTopB:   String;
            cTopRC:  String;

            cLeftB:  String;
            cRightB: String;

            cBotLC:  String;
            cBotB:   String;
            cBotRC:  String;

            cColorA: String;
            cColorB: String;
            cColorC: String;
            cColorD: String;
            cColorE: String;
            cColorF: String;
            cColorG: String;
            cColorH: String;
            cColorI: String;
            cColorJ: String;
            cColorK: String;
            cColorL: String;
            cColorM: String;
            cColorN: String;
            cColorO: String;
            cColorP: String;
            bTimeError:boolean;

            cDBSite:string;
    END;

VAR
    INI: TConfig;

PROCEDURE GLOBAL_Init;
PROCEDURE GLOBAL_Destroy;

IMPLEMENTATION
{$I-}
CONSTRUCTOR TConfig.Init;
BEGIN

    fGameListF := TFont.Create;
    fNextMoveF := TFont.Create;
    fPrevMoveF := TFont.Create;
    fCommentF  := TFont.Create;
    fEngineF   := Tfont.Create;
    fClock     := TFont.Create;
    fHandle    := TFont.Create;
    fFCD1      := TFont.Create;
    fFCD2      := TFont.Create;
    fFCD3      := TFont.Create;
    fFCD4      := TFont.Create;
    fFCD5      := TFont.Create;

    SetFilename( ChangeFileExt(ParamStr(0), '.' + INI_EXT) );
    InitConfig;
END;

DESTRUCTOR TConfig.Destroy;
BEGIN
    { Nothing }
END;

PROCEDURE GLOBAL_Init;
BEGIN
    defaultmindate:=EncodeDate(1,1,1);
    defaultmaxdate:=EncodeDate(9999,1,1);
    INI := TConfig.Create;
    INI.Init;

    IF (NOT FileExists(INI.cFile)) THEN
        INI.WriteAll;

    INI.ReadAll;
END;

PROCEDURE GLOBAL_Destroy;
BEGIN
    { Destroy }
END;


PROCEDURE TConfig.InitConfig;
var empEngopt:Tengineopt;
i:integer;
curdir:string;
BEGIN
curdir:=ExcludeTrailingBackslash(ExtractFilePath(paramstr(0)));

 defeng:=1;
 empEngopt.name:='';
 empEngopt.comline:='';
 empEngopt.etype:=2;
 empEngOpt.prompt:='';
 empEngOpt.reset:='';
 empEngOpt.itime:='time 10000000';
 empEngOpt.bfilter:=false;
 empEngOpt.banal:=false;

 for i:=1 to MAX_ENGINE do EngineOpt[i]:=empEngOpt;

 EngineOpt[2].name:='Sjeng 10';
 EngineOpt[2].comline:=curdir+'\Engines\sjeng10.exe';
 EngineOpt[2].etype:=0;
 EngineOpt[2].prompt:='Sjeng:';
 EngineOpt[2].reset:='new';
 EngineOpt[2].itime:='st 10000000';
 EngineOpt[2].bfilter:=true;
 EngineOpt[2].banal:=true;

 EngineOpt[1].name:='Sunsetter';
 EngineOpt[1].comline:=curdir+'\Engines\sunsetter.exe';
 EngineOpt[1].etype:=1;
 EngineOpt[1].prompt:='';
 EngineOpt[1].reset:='reset';
 EngineOpt[1].itime:='sd 24';
 EngineOpt[1].bfilter:=true;
 EngineOpt[1].banal:=true;

 EngineOpt[3].name:='Sjeng 7';
 EngineOpt[3].comline:=curdir+'\Engines\sjeng7.exe';
 EngineOpt[3].etype:=2;
 EngineOpt[3].prompt:='Sjeng:';
 EngineOpt[3].reset:='';
 EngineOpt[3].itime:='st 10000000';
 EngineOpt[3].bfilter:=true;
 EngineOpt[3].banal:=false;

    nMainL := 0;
    nMainT := 0;
    nMainW := Screen.Width;
    nMainH := Screen.Height - 30;
    nMainS := 0;

    nWhtSquare  :=defnwhtsquare; // $008AC5F1; //clWhite;
    nWhtSOutline:=clBlack;
    nWhtOutline := clBlack;
    nWhtFill    := clWhite;

    nBlkSquare  :=defnblksquare;//$00183f74; //clTeal;
    nBlkSOutline:= clBlack;
    nBlkOutline := clGray;
    nBlkFill    := clBlack;

    bOutlinePC   := TRUE;
    nClickColor  := clRed;

    bShwPCCount  := TRUE;
    nNumberColor := defnnumbercolor;
    nPcSquare    := defnpcsquare;

    usebmp:=defusebmp;
    usepiecebmp:=defusepiecebmp;
    useboardbmp:=defuseboardbmp;
    tmpgraphhome:=Extractfilepath(paramstr(0))+defgraphhome;
    bmpFile:=tmpgraphhome+relativebgdir+defbmpfile;
    p_dir:=tmpgraphhome+relativepiecedir+defp_dir;
    b_dir:=tmpgraphhome+relativeboarddir+defb_dir;


    bFlipBd      := FALSE;
    bAlwaysQ     := FALSE;
    bDragMove    := TRUE;
    bSmartMove   := FALSE;
    bActualBoard := FALSE;
    nBoardOutline:= defnboardoutline;
    nBoardWindow := defbgcolor;
    bShowSOutline:= FALSE;
    bShowLag:=true;
    bShowTenth:=true;

    nWaitF        := defnwaitf;
    nWaitB        := defnwaitb;
    nTurnF        := defnturnf;
    nTurnB        := defnturnb;
    fClock.Color  := deffclock_color;
    fClock.Height := deffclock_height;
    fClock.Name   := deffclock_name;
    fClock.Pitch  := deffclock_pitch;
    fClock.Size   := deffclock_size;
    fClock.Style  := [];

    nHandleB       := defnhandleb;
    fHandle.Color  := deffhandle_color;
    fHandle.Height := deffhandle_height;
    fHandle.Name   := deffhandle_name;
    fHandle.Pitch  := deffhandle_pitch;
    fHandle.Size   := deffhandle_size;
    fHandle.Style  := [];

    nBoard1L := 0;
    nBoard1T := 0;
    nBoard1W := (nMainW SHR 1) - 10;
    nBoard1H := ((nMainH DIV 3) SHL 1) - 30;
    nBoard1S := 0;

    nBoard2L := nBoard1L + nBoard1W;
    nBoard2T := nBoard1T;
    nBoard2W := nBoard1W;
    nBoard2H := nBoard1H;
    nBoard2S := 0;

    nNavigateL := ((nBoard1L + nBoard1W + nBoard1W) - NAVIGATE_WIDTH) SHR 1;
    nNavigateT := nBoard1T + nBoard1H;
    nNavigateSticky:=false;
    nNavigateW:=Navigate_width;
    nNavigateH:=Navigate_height;
    nNavigateS := 0;
    bnavigatesimple:=false;

    nGameListL        := nBoard1L;
    nGameListT        := nNavigateT + Navigate_height;
    nGameListW        := (nMainW DIV 3) - 5;
    nGameListH        := nMainH - nGameListT - 70;
    nGameListS        := 0;
    nGameListB        := DefFormB;
    fGameListF.Color  := clBlack;
    fGameListF.Height := -11;
    fGameListF.Name   := 'MS Sans Serif';
    fGameListF.Pitch  := fpDefault;
    fGameListF.Size   := 10;
    fGameListF.Style  := [];
    bShowGameNum      := FALSE;
    bShowEvent        := FALSE;
    bShowRatings      := TRUE;
    bShowTime         := TRUE;
    bShowInc          := TRUE;
    bShowResult       := TRUE;

    nNextMoveL        := nGameListL + nGameListW;
    nNextMoveT        := nGameListT;
    nNextMoveW        := nGameListW SHR 1;
    nNextMoveH        := nGameListH;
    nNextMoveS        := 0;
    nNextMoveB        := DefFormB;
    fNextMoveF.Color  := clBlack;
    fNextMoveF.Height := -11;
    fNextMoveF.Name   := 'MS Sans Serif';
    fNextMoveF.Pitch  := fpDefault;
    fNextMoveF.Size   := 10;
    fNextMoveF.Style  := [];

    nPrevMoveL        := nNextMoveL + nNextMoveW;
    nPrevMoveT        := nGameListT;
    nPrevMoveW        := nNextMoveW;
    nPrevMoveH        := nGameListH;
    nPrevMoveS        := 0;
    nPrevMoveB        := DefFormB;
    fPrevMoveF.Color  := clBlack;
    fPrevMoveF.Height := -11;
    fPrevMoveF.Name   := 'MS Sans Serif';
    fPrevMoveF.Pitch  := fpDefault;
    fPrevMoveF.Size   := 10;
    fPrevMoveF.Style  := [];

    nCommentL        := nPrevMoveL + nPrevMoveW;
    nCommentT        := nNextMoveT;
    nCommentW        := nGameListW;
    nCommentH        := nNextMoveH;
    nCommentS        := 0;
    nCommentB        := defFormB;
    fCommentF.Color  := clBlack;
    fCommentF.Height := -11;
    fCommentF.Name   := 'MS Sans Serif';
    fCommentF.Pitch  := fpDefault;
    fCommentF.Size   := 10;
    fCommentF.Style  := [];
    bCommentWR       := True;

    bsoundon:=true;
    csoundmove:=curdir+'\Sound\move.wav';
    csoundcap:=curdir+'\Sound\capture.wav';
    csoundcheck:=curdir+'\Sound\check.wav';
    csoundend:=curdir+'\Sound\final.wav';


for i:=1 to 3 do
begin
 enginelay[i].nEngineL:=nboard2l;
 enginelay[i].nenginew:=nboard2w+10;
 enginelay[i].nEngineT:=nboard2t+6+((nnavigatet-nboard2t)div 3)*(i-1);
 enginelay[i].nEngineH:=(nnavigatet-nboard2t-6) div 3;
 enginelay[i].nengineS:=0;
end;
enginelay[5].nEngineL:=nmainl;
enginelay[5].nEngineT:=nnavigatet+nNavigateh;
enginelay[5].nengineH:=ncommenth;
enginelay[5].nengineW:=nmainW shr 1;
enginelay[5].nengines:=0;
enginelay[4].nenginel:=nmainl+enginelay[5].nenginew;
enginelay[4].nenginew:=enginelay[5].nenginew;
enginelay[4].nengineh:=enginelay[5].nengineh;
enginelay[4].nengines:=0;
enginelay[4].nenginet:=enginelay[5].nEngineT;

    nEngineB        := defFormB;
    fEngineF.Color  := clBlack;
    fEngineF.Height := -11;
    fEngineF.Name   := 'Courier New';
    fEngineF.Pitch  := fpDefault;
    fEngineF.Size   := 10;
    fEngineF.Style  := [];
    bEngineWR       := False;
    nEngineTT:=30;
    nEngineAT:=30;
    nEngineIT:=0.5;
    nEngineET:=6.0;
    bEngineSM:=true;
    Tecumseh:=false;

    nVATW := Screen.Width SHR 1;
    nVATH := Screen.Height SHR 1;
    nVATL := (Screen.Width  - nVATW) SHR 1;
    nVATT := (Screen.Height - nVATH) SHR 1;

    nVEXW := Screen.Width SHR 1;
    nVEXH := Screen.Height SHR 1;
    nVEXL := (Screen.Width  - nVEXW) SHR 1;
    nVEXT := (Screen.Height - nVEXH) SHR 1;

    nRTFW   := Screen.Width SHR 1;
    nRTFH   := Screen.Height SHR 1;
    nRTFL   := (Screen.Width  - nRTFW) SHR 1;
    nRTFT   := (Screen.Height - nRTFH) SHR 1;
    cRTFDir := ExtractFilePath(ParamStr(0));

    nTXTW   := Screen.Width SHR 1;
    nTXTH   := Screen.Height SHR 1;
    nTXTL   := (Screen.Width  - nTXTW) SHR 1;
    nTXTT   := (Screen.Height - nTXTH) SHR 1;
    cTXTDir := ExtractFilePath(ParamStr(0));

    nstatw:=697;
    nstath:=504;
    nstatL:=(screen.Width-nstatw) shr 1;
    nstatT:=(screen.Height-nstath) div 3;

    nfilterw:=369;
    nfilterH:=582;
    bFilterSt:=false;
    nTreeW:=0;
    nTreeH:=0;
    bTreeFil:=false;
//    bTreeAuto:=false;



    bSaveOptionsOnExit := TRUE;
    bSaveLayoutsOnExit := TRUE;

    cNewPath    := ExtractFilePath(ParamStr(0));
    cOpenPath   := ExtractFilePath(ParamStr(0));
    cDeletePath := ExtractFilePath(ParamStr(0));
    cSaveAsPath := ExtractFilePath(ParamStr(0));
    cExportPath := ExtractFilePath(ParamStr(0));
    Cworkbpgn   :=ExtractFilePath(Paramstr(0))+defcworkbpgn;

    cHandle        := 'Gnejs';
    cSource        := ExtractFilePath(ParamStr(0)) + '*.' + BPGN_EXT;
    cDestination   := ExtractFileDrive(ParamStr(0)) + '\' + cHandle + '.' + BPGN_EXT;
    nMoveNumber    := DEF_EXTRACT_NUM;
    bRemoveComment := TRUE;
    bGenOccurrence := FALSE;

    fFCD1.Color  := clBlack;
    fFCD1.Height := -11;
    fFCD1.Name   := 'MS Sans Serif';
    fFCD1.Pitch  := fpDefault;
    fFCD1.Size   := 10;
    fFCD1.Style  := [];

    fFCD2.Color  := clBlack;
    fFCD2.Height := -11;
    fFCD2.Name   := 'MS Sans Serif';
    fFCD2.Pitch  := fpDefault;
    fFCD2.Size   := 10;
    fFCD2.Style  := [];

    fFCD3.Color  := clBlack;
    fFCD3.Height := -11;
    fFCD3.Name   := 'MS Sans Serif';
    fFCD3.Pitch  := fpDefault;
    fFCD3.Size   := 10;
    fFCD3.Style  := [];

    fFCD4.Color  := clBlack;
    fFCD4.Height := -11;
    fFCD4.Name   := 'MS Sans Serif';
    fFCD4.Pitch  := fpDefault;
    fFCD4.Size   := 10;
    fFCD4.Style  := [];

    fFCD5.Color  := clBlack;
    fFCD5.Height := -11;
    fFCD5.Name   := 'MS Sans Serif';
    fFCD5.Pitch  := fpDefault;
    fFCD5.Size   := 10;
    fFCD5.Style  := [];

    bRTFHand := TRUE;
    bRTFMove := TRUE;
    bRTFAnno := TRUE;
    bRTFTime := TRUE;
    bRTFNote := TRUE;
    bRTFDisp := TRUE;
    bRTFRest := TRUE;
    bRTFZero := FALSE;

    cRTFFile := ExtractFilePath(ParamStr(0)) + 'GAME.' + RTF_EXT;

    bTXTHand := TRUE;
    bTXTMove := TRUE;
    bTXTAnno := TRUE;
    bTXTTime := TRUE;
    bTXTNote := TRUE;
    bTXTDisp := TRUE;
    bTXTRest := TRUE;
    bTXTZero := FALSE;

    cTXTWP := 'P';
    cTXTWN := 'N';
    cTXTWB := 'B';
    cTXTWR := 'R';
    cTXTWQ := 'Q';
    cTXTWK := 'K';

    cTXTBP := 'P';
    cTXTBN := 'N';
    cTXTBB := 'B';
    cTXTBR := 'R';
    cTXTBQ := 'Q';
    cTXTBK := 'K';

    cTXTCWP := 'P';
    cTXTCWN := 'N';
    cTXTCWB := 'B';
    cTXTCWR := 'R';
    cTXTCWQ := 'Q';
    cTXTCWK := 'K';

    cTXTCBP := '*P';
    cTXTCBN := '*N';
    cTXTCBB := '*B';
    cTXTCBR := '*R';
    cTXTCBQ := '*Q';
    cTXTCBK := '*K';

    cTXTFile := ExtractFilePath(ParamStr(0)) + 'GAME.' + TXT_EXT;

    cWP := 'P';
    cWN := 'N';
    cWB := 'B';
    cWR := 'R';
    cWQ := 'Q';
    cWK := 'K';

    cBP := 'P';
    cBN := 'N';
    cBB := 'B';
    cBR := 'R';
    cBQ := 'Q';
    cBK := 'K';
    bZero := FALSE;

    cWPWS   := 'p';
    cWPBS   := 'P';
    cWNWS   := 'n';
    cWNBS   := 'N';
    cWBWS   := 'b';
    cWBBS   := 'B';
    cWRWS   := 'r';
    cWRBS   := 'R';
    cWQWS   := 'q';
    cWQBS   := 'Q';
    cWKWS   := 'k';
    cWKBS   := 'K';
    cMoveInd:= MOVE_INDICATOR;

    cBPWS   := 'o';
    cBPBS   := 'O';
    cBNWS   := 'm';
    cBNBS   := 'M';
    cBBWS   := 'v';
    cBBBS   := 'V';
    cBRWS   := 't';
    cBRBS   := 'T';
    cBQWS   := 'w';
    cBQBS   := 'W';
    cBKWS   := 'l';
    cBKBS   := 'L';

    cWS     := '*';
    cBS     := '+';

    cTopLC  := '!';
    cTopB   := '"';
    cTopRC  := '#';

    cLeftB  := '$';
    cRightB := '%';

    cBotLC  := '/';
    cBotB   := '(';
    cBotRC  := ')';

    cColorA := 'ColorA=';
    cColorB := 'ColorB=';
    cColorC := 'ColorC=';
    cColorD := 'ColorD=';
    cColorE := 'ColorE=';
    cColorF := 'ColorF=';
    cColorG := 'ColorG=';
    cColorH := 'ColorH=';
    cColorI := 'ColorI=';
    cColorJ := 'ColorJ=';
    cColorK := 'ColorK=';
    cColorL := 'ColorL=';
    cColorM := 'ColorM=';
    cColorN := 'ColorN=';
    cColorO := 'ColorO=';
    cColorP := 'ColorP=';
    bairdropini:=true;
    savedepth:=default_DB_SAVE_DEPTH;
    bROgames:=false;
    bOpenTree:=true;
//    nshortG:=default_Sh_game_max;
    bfullstat:=false;
    nMoveOrder:=0;
    cDBSite:='http://www.bughouse-db.org/';
END;

PROCEDURE TConfig.SetFilename(cFn: String);
BEGIN
    cName := ExtractFileName(cFn);
    cFile := cFn;
END;

PROCEDURE TConfig.ReadAll;
VAR
    IniFile: TIniFile;
    cTmp: String;
    tmp:string;
    i:integer;
    reg:Tregistry;
    nokey:boolean;
    wrongkey:boolean;
    graphhome0:string;
    graphhome1:string;
    graphhome2:string;
    graphhome3:string;
BEGIN
   try
    IniFile := TIniFile.Create(cFile);

    { Main }
    nMainL := IniFile.ReadInteger('Main', 'Left',   nMainL);
    nMainT := IniFile.ReadInteger('Main', 'Top',    nMainT);
    nMainW := IniFile.ReadInteger('Main', 'Width',  nMainW);
    nMainH := IniFile.ReadInteger('Main', 'Height', nMainH);
    nMainS := IniFile.ReadInteger('Main', 'State',  nMainS);

    { Board }
    usebmp:=IniFile.Readbool('Board','use bmp',usebmp);
    usepiecebmp:=IniFile.Readbool('Board','use piece bmp',usepiecebmp);
    useboardbmp:=INIFile.readbool('Board','use board bmp',useboardbmp);
    p_dir:=IniFile.Readstring('Board', 'piece dir',p_dir);
    bmpfile:=IniFile.Readstring('Board', 'bmp file',bmpfile);
    b_dir:=iniFile.ReadString('Board','board dir',b_dir);
    graphhome1:=inifile.readstring('Board', 'graphics home',tmpgraphhome);

    nWhtSquare   := IniFile.ReadInteger('Board', 'White square color',  nWhtSquare);
    nWhtSOutline := IniFile.ReadInteger('Board', 'White square outline color', nWhtSOutline);
    nWhtFill     := IniFile.ReadInteger('Board', 'White piece color',   nWhtFill);
    nWhtOutLine  := IniFile.ReadInteger('Board', 'White piece outline color', nWhtOutline);
    nBlkSquare   := IniFile.ReadInteger('Board', 'Black square color',  nBlkSquare);
    nBlkSOutline := IniFile.ReadInteger('Board', 'Black square outline color', nBlkSOutline);
    nBlkFill     := IniFile.ReadInteger('Board', 'Black piece color',   nBlkFill);
    nBlkOutline  := IniFile.ReadInteger('Board', 'Black piece outline color', nBlkOutline);
    bOutlinePC   := IniFile.ReadBool('Board', 'Show piece outline when mouse clicked', bOutlinePC);
    nClickColor  := IniFile.ReadInteger('Board', 'Piece outline color when mouse clicked', nClickColor);
    bShwPCCount  := IniFile.ReadBool('Board', 'Show piece count', bShwPCCount);
    nNumberColor := IniFile.ReadInteger('Board', 'Piece count color', nNumberColor);
    nPcSquare    := IniFile.ReadInteger('Board', 'Piece square color', nPcSquare);
    bFlipBd      := IniFile.ReadBool('Board', 'Flip board',   bFlipBd);
    bAlwaysQ     := IniFile.ReadBool('Board', 'Always queen', bAlwaysQ);
    bDragMove    := IniFile.ReadBool('Board', 'Drag move',    bDragMove);
    bSmartMove   := IniFile.ReadBool('Board', 'Smart move',   bSmartMove);
    bActualBoard := IniFile.ReadBool('Board', 'Restrict board window to actual board size', bActualBoard);
    nBoardOutline:= IniFile.ReadInteger('Board', 'Board outline color', nBoardOutline);
    nBoardWindow := IniFile.ReadInteger('Board', 'Board window background color', nBoardWindow);
    bShowSOutline:= IniFile.ReadBool('Board', 'Show square outline', bShowSOutline);
    bShowlag:= IniFile.ReadBool('Board', 'Show lag', bShowLag);
    bShowtenth:= IniFile.ReadBool('Board', 'Show tenth', bShowtenth);

    { Clock }
    nWaitF := IniFile.ReadInteger('Clock', 'Wait foreground color', nWaitF);
    nWaitB := IniFile.ReadInteger('Clock', 'Wait background color', nWaitB);
    nTurnF := IniFile.ReadInteger('Clock', 'Turn foreground color', nTurnF);
    nTurnB := IniFile.ReadInteger('Clock', 'Turn background color', nTurnB);

    fClock.Color  := nWaitF;
    fClock.Name   := IniFile.ReadString ('Clock', 'Font name',        fClock.Name);
    fClock.Height := IniFile.ReadInteger('Clock', 'Font height',      fClock.Height);
    fClock.Size   := IniFile.ReadInteger('Clock', 'Font size',        fClock.Size);

    cTmp := IniFile.ReadString('Clock', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fClock.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fClock.Pitch := fpFixed
    else
        fClock.Pitch := fpDefault;

    fClock.Style := [];

    IF (IniFile.ReadBool('Clock', 'Font style bold', FALSE)) THEN
        fClock.Style := fClock.Style + [fsBold];

    IF (IniFile.ReadBool('Clock', 'Font style italic', FALSE)) THEN
        fClock.Style := fClock.Style + [fsItalic];

    IF (IniFile.ReadBool('Clock', 'Font style underline', FALSE)) THEN
        fClock.Style := fClock.Style + [fsUnderline];

    IF (IniFile.ReadBool('Clock', 'Font style strikeout', FALSE)) THEN
        fClock.Style := fClock.Style + [fsStrikeOut];

    { Handle }
    nHandleB := IniFile.ReadInteger('Handle', 'Background color', nHandleB);

    fHandle.Color  := IniFile.ReadInteger('Handle', 'Foreground color', fHandle.Color);
    fHandle.Name   := IniFile.ReadString ('Handle', 'Font name',        fHandle.Name);
    fHandle.Height := IniFile.ReadInteger('Handle', 'Font height',      fHandle.Height);
    fHandle.Size   := IniFile.ReadInteger('Handle', 'Font size',        fHandle.Size);

    cTmp := IniFile.ReadString('Handle', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fHandle.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fHandle.Pitch := fpFixed
    else
        fHandle.Pitch := fpDefault;

    fHandle.Style := [];

    IF (IniFile.ReadBool('Handle', 'Font style bold', FALSE)) THEN
        fHandle.Style := fHandle.Style + [fsBold];

    IF (IniFile.ReadBool('Handle', 'Font style italic', FALSE)) THEN
        fHandle.Style := fHandle.Style + [fsItalic];

    IF (IniFile.ReadBool('Handle', 'Font style underline', FALSE)) THEN
        fHandle.Style := fHandle.Style + [fsUnderline];

    IF (IniFile.ReadBool('Handle', 'Font style strikeout', FALSE)) THEN
        fHandle.Style := fHandle.Style + [fsStrikeOut];

    { Board #1 }
    nBoard1L := IniFile.ReadInteger('Board #1', 'Left',   nBoard1L);
    nBoard1T := IniFile.ReadInteger('Board #1', 'Top',    nBoard1T);
    nBoard1W := IniFile.ReadInteger('Board #1', 'Width',  nBoard1W);
    nBoard1H := IniFile.ReadInteger('Board #1', 'Height', nBoard1H);
    nBoard1S := IniFile.ReadInteger('Board #1', 'State',  nBoard1S);

    { Board #2 }
    nBoard2L := IniFile.ReadInteger('Board #2', 'Left',   nBoard2L);
    nBoard2T := IniFile.ReadInteger('Board #2', 'Top',    nBoard2T);
    nBoard2W := IniFile.ReadInteger('Board #2', 'Width',  nBoard2W);
    nBoard2H := IniFile.ReadInteger('Board #2', 'Height', nBoard2H);
    nBoard2S := IniFile.ReadInteger('Board #2', 'State',  nBoard2S);

    { Naviagate }
    nNavigateL := IniFile.ReadInteger('Navigate', 'Left',   nNavigateL);
    nNavigateT := IniFile.ReadInteger('Navigate', 'Top',    nNavigateT);
    nNavigateSticky:=inifile.ReadBool('Navigate','Sticky',nNavigateSticky);
    nNavigateW := IniFile.ReadInteger('Navigate', 'Width',   nNavigateW);
    nNavigateH := IniFile.ReadInteger('Navigate', 'Height',    nNavigateH);
    nNavigateS := IniFile.ReadInteger('Navigate', 'State',  nNavigateS);
    bnavigateSimple:=inifile.Readbool('Navigate','Simple',bnavigatesimple);

    { Game list }
    nGameListL := IniFile.ReadInteger('Game list', 'Left',   nGameListL);
    nGameListT := IniFile.ReadInteger('Game list', 'Top',    nGameListT);
    nGameListW := IniFile.ReadInteger('Game list', 'Width',  nGameListW);
    nGameListH := IniFile.ReadInteger('Game list', 'Height', nGameListH);
    nGameListS := IniFile.ReadInteger('Game list', 'State',  nGameListS);
    nGameListB := IniFile.ReadInteger('Game list', 'Background color', nGameListB);

    fGameListF.Color  := IniFile.ReadInteger('Game list', 'Foreground color', fGameListF.Color);
    fGameListF.Name   := IniFile.ReadString ('Game list', 'Font name',        fGameListF.Name);
    fGameListF.Height := IniFile.ReadInteger('Game list', 'Font height',      fGameListF.Height);
    fGameListF.Size   := IniFile.ReadInteger('Game list', 'Font size',        fGameListF.Size);

    cTmp := IniFile.ReadString('Game list', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fGameListF.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fGameListF.Pitch := fpFixed
    else
        fGameListF.Pitch := fpDefault;

    fGameListF.Style := [];

    IF (IniFile.ReadBool('Game list', 'Font style bold', FALSE)) THEN
        fGameListF.Style := fGameListF.Style + [fsBold];

    IF (IniFile.ReadBool('Game list', 'Font style italic', FALSE)) THEN
        fGameListF.Style := fGameListF.Style + [fsItalic];

    IF (IniFile.ReadBool('Game list', 'Font style underline', FALSE)) THEN
        fGameListF.Style := fGameListF.Style + [fsUnderline];

    IF (IniFile.ReadBool('Game list', 'Font style strikeout', FALSE)) THEN
        fGameListF.Style := fGameListF.Style + [fsStrikeOut];

    bShowGameNum := IniFile.ReadBool('Game list', 'Show game #',    bShowGameNum);
    bShowEvent   := IniFile.ReadBool('Game list', 'Show event',     bShowEvent);
    bShowRatings := IniFile.ReadBool('Game list', 'Show ratings',   bShowRatings);
    bShowTime    := IniFile.ReadBool('Game list', 'Show time',      bShowTime);
    bShowInc     := IniFile.ReadBool('Game list', 'Show increment', bShowInc);
    bShowResult  := IniFile.ReadBool('Game list', 'Show result',    bShowResult);

    { Next move }
    nNextMoveL := IniFile.ReadInteger('Next move', 'Left',   nNextMoveL);
    nNextMoveT := IniFile.ReadInteger('Next move', 'Top',    nNextMoveT);
    nNextMoveW := IniFile.ReadInteger('Next move', 'Width',  nNextMoveW);
    nNextMoveH := IniFile.ReadInteger('Next move', 'Height', nNextMoveH);
    nNextMoveS := IniFile.ReadInteger('Next move', 'State',  nNextMoveS);
    nNextMoveB := IniFile.ReadInteger('Next move', 'Background color', nNextMoveB);

    fNextMoveF.Color  := IniFile.ReadInteger('Next move', 'Foreground color', fNextMoveF.Color);
    fNextMoveF.Name   := IniFile.ReadString ('Next move', 'Font name',        fNextMoveF.Name);
    fNextMoveF.Height := IniFile.ReadInteger('Next move', 'Font height',      fNextMoveF.Height);
    fNextMoveF.Size   := IniFile.ReadInteger('Next move', 'Font size',        fNextMoveF.Size);

    cTmp := IniFile.ReadString('Next move', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fNextMoveF.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fNextMoveF.Pitch := fpFixed
    else
        fNextMoveF.Pitch := fpDefault;

    fNextMoveF.Style := [];

    IF (IniFile.ReadBool('Next move', 'Font style bold', FALSE)) THEN
        fNextMoveF.Style := fNextMoveF.Style + [fsBold];

    IF (IniFile.ReadBool('Next move', 'Font style italic', FALSE)) THEN
        fNextMoveF.Style := fNextMoveF.Style + [fsItalic];

    IF (IniFile.ReadBool('Next move', 'Font style underline', FALSE)) THEN
        fNextMoveF.Style := fNextMoveF.Style + [fsUnderline];

    IF (IniFile.ReadBool('Next move', 'Font style strikeout', FALSE)) THEN
        fNextMoveF.Style := fNextMoveF.Style + [fsStrikeOut];

    { Previous next move }
    nPrevMoveL := IniFile.ReadInteger('Previous next move', 'Left',   nPrevMoveL);
    nPrevMoveT := IniFile.ReadInteger('Previous next move', 'Top',    nPrevMoveT);
    nPrevMoveW := IniFile.ReadInteger('Previous next move', 'Width',  nPrevMoveW);
    nPrevMoveH := IniFile.ReadInteger('Previous next move', 'Height', nPrevMoveH);
    nPrevMoveS := IniFile.ReadInteger('Previous next move', 'State',  nPrevMoveS);
    nPrevMoveB := IniFile.ReadInteger('Previous next move', 'Background color', nPrevMoveB);

    fPrevMoveF.Color  := IniFile.ReadInteger('Previous next move', 'Foreground color', fPrevMoveF.Color);
    fPrevMoveF.Name   := IniFile.ReadString ('Previous next move', 'Font name',        fPrevMoveF.Name);
    fPrevMoveF.Height := IniFile.ReadInteger('Previous next move', 'Font height',      fPrevMoveF.Height);
    fPrevMoveF.Size   := IniFile.ReadInteger('Previous next move', 'Font size',        fPrevMoveF.Size);

    cTmp := IniFile.ReadString('Previous next move', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fPrevMoveF.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fPrevMoveF.Pitch := fpFixed
    else
        fPrevMoveF.Pitch := fpDefault;

    fPrevMoveF.Style := [];

    IF (IniFile.ReadBool('Previous next move', 'Font style bold', FALSE)) THEN
        fPrevMoveF.Style := fPrevMoveF.Style + [fsBold];

    IF (IniFile.ReadBool('Previous next move', 'Font style italic', FALSE)) THEN
        fPrevMoveF.Style := fPrevMoveF.Style + [fsItalic];

    IF (IniFile.ReadBool('Previous next move', 'Font style underline', FALSE)) THEN
        fPrevMoveF.Style := fPrevMoveF.Style + [fsUnderline];

    IF (IniFile.ReadBool('Previous next move', 'Font style strikeout', FALSE)) THEN
        fPrevMoveF.Style := fPrevMoveF.Style + [fsStrikeOut];

    { Comment }
    nCommentL := IniFile.ReadInteger('Comment', 'Left',   nCommentL);
    nCommentT := IniFile.ReadInteger('Comment', 'Top',    nCommentT);
    nCommentW := IniFile.ReadInteger('Comment', 'Width',  nCommentW);
    nCommentH := IniFile.ReadInteger('Comment', 'Height', nCommentH);
    nCommentS := IniFile.ReadInteger('Comment', 'State',  nCommentS);
    nCommentB := IniFile.ReadInteger('Comment', 'Background color', nCommentB);

    fCommentF.Color  := IniFile.ReadInteger('Comment', 'Foreground color', fCommentF.Color);
    fCommentF.Name   := IniFile.ReadString ('Comment', 'Font name',        fCommentF.Name);
    fCommentF.Height := IniFile.ReadInteger('Comment', 'Font height',      fCommentF.Height);
    fCommentF.Size   := IniFile.ReadInteger('Comment', 'Font size',        fCommentF.Size);

    cTmp := IniFile.ReadString('Comment', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fCommentF.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fCommentF.Pitch := fpFixed
    else
        fCommentF.Pitch := fpDefault;

    fCommentF.Style := [];

    IF (IniFile.ReadBool('Comment', 'Font style bold', FALSE)) THEN
        fCommentF.Style := fCommentF.Style + [fsBold];

    IF (IniFile.ReadBool('Comment', 'Font style italic', FALSE)) THEN
        fCommentF.Style := fCommentF.Style + [fsItalic];

    IF (IniFile.ReadBool('Comment', 'Font style underline', FALSE)) THEN
        fCommentF.Style := fCommentF.Style + [fsUnderline];

    IF (IniFile.ReadBool('Comment', 'Font style strikeout', FALSE)) THEN
        fCommentF.Style := fCommentF.Style + [fsStrikeOut];

    bCommentWR := IniFile.ReadBool('Comment', 'Word wrap', bCommentWR);

    bsoundOn:=inifile.readbool('Sound', 'on', bsoundon);
    csoundmove:=inifile.readstring('Sound', 'move',csoundmove);
    csoundcheck:=inifile.readstring('Sound', 'check', csoundcheck);
    csoundcap:=inifile.readstring('Sound', 'capture', csoundcap);
    csoundend:=inifile.readstring('Sound', 'end', csoundend);



    for i:=1 to MAX_ENGINE do
    begin
     str(i,tmp);
     tmp:='Engine'+tmp;
     engineopt[i].name:=IniFile.ReadString(tmp, 'name', engineopt[i].name);
     engineopt[i].comline:=IniFile.ReadString(tmp, 'command line', engineopt[i].comline);
     engineopt[i].etype:=IniFile.ReadInteger(tmp, 'type', engineopt[i].etype);
     engineopt[i].prompt:=IniFile.ReadString(tmp, 'prompt', engineopt[i].prompt);
     engineopt[i].reset:=IniFile.ReadString(tmp, 'reset', engineopt[i].reset);
     engineopt[i].itime:=IniFile.ReadString(tmp, 'infinite time', engineopt[i].itime);
     engineopt[i].bFilter:=IniFile.ReadBool(tmp, 'filter output', engineopt[i].bfilter);
     engineopt[i].banal:=IniFile.ReadBool(tmp,'analysis',engineopt[i].banal);

     enginelay[i].nEngineL := IniFile.ReadInteger(tmp, 'Left',   enginelay[i].nEngineL);
     enginelay[i].nEngineT := IniFile.ReadInteger(tmp, 'Top',    enginelay[i].nEngineT);
     enginelay[i].nEngineW := IniFile.ReadInteger(tmp, 'Width',  enginelay[i].nEngineW);
     enginelay[i].nEngineH := IniFile.ReadInteger(tmp, 'Height', enginelay[i].nEngineH);
     enginelay[i].nEngineS := IniFile.ReadInteger(tmp, 'State',  enginelay[i].nEngineS);
    end;


    nEngineB := IniFile.ReadInteger('Engine', 'Background color', nEngineB);

    fEngineF.Color  := IniFile.ReadInteger('Engine', 'Foreground color', fEngineF.Color);
    fEngineF.Name   := IniFile.ReadString ('Engine', 'Font name',        fEngineF.Name);
    fEngineF.Height := IniFile.ReadInteger('Engine', 'Font height',      fEngineF.Height);
    fEngineF.Size   := IniFile.ReadInteger('Engine', 'Font size',        fEngineF.Size);

    cTmp := IniFile.ReadString('Engine', 'Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fEngineF.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fEngineF.Pitch := fpFixed
    else
        fEngineF.Pitch := fpDefault;

    fEngineF.Style := [];

    IF (IniFile.ReadBool('Engine', 'Font style bold', FALSE)) THEN
        fEngineF.Style := fEngineF.Style + [fsBold];

    IF (IniFile.ReadBool('Engine', 'Font style italic', FALSE)) THEN
        fEngineF.Style := fEngineF.Style + [fsItalic];

    IF (IniFile.ReadBool('Engine', 'Font style underline', FALSE)) THEN
        fEngineF.Style := fEngineF.Style + [fsUnderline];

    IF (IniFile.ReadBool('Engine', 'Font style strikeout', FALSE)) THEN
        fEngineF.Style := fEngineF.Style + [fsStrikeOut];

    bEngineWR := IniFile.ReadBool('Engine', 'Word wrap', bEngineWR);
    nEngineTT:=Inifile.readfloat('Engine', 'test time', nengineTT);
    nEngineAT:=INiFile.ReadFloat('Engine', 'annotate time', nEngineat);
    nEngineIT:=INiFile.ReadFloat('Engine', 'include threshold', nEngineIt);
    nEngineeT:=INiFile.ReadFloat('Engine', 'exclude threshold', nEngineet);
    bEngineSM:=INIFile.readbool('Engine', 'show mates', bEngineSM);


    TEcumseh:=Inifile.ReadBool('Engine','Tecumseh',tecumseh);
    defeng:=IniFile.ReadInteger('Engine', 'Default engine', defeng);


    { Extract }
    cHandle := IniFile.ReadString('Extract', 'Handle', cHandle);
    IF (Length(cHandle) = 0) THEN
        cHandle := 'Gnejs';

    cSource := IniFile.ReadString('Extract', 'Source BGPN filter', cSource);
    IF (Length(cSource) = 0) THEN
        cSource := ExtractFilePath(ParamStr(0)) + '*.' + BPGN_EXT;

    cDestination := IniFile.ReadString('Extract', 'Destination BGPN filename', cDestination);
    IF (Length(cDestination) = 0) THEN
        cDestination := ExtractFilePath(ParamStr(0)) + cHandle + '.' + BPGN_EXT;

    nMoveNumber := IniFile.ReadInteger('Extract', 'Number of moves', nMoveNumber);
    IF (nMoveNumber < 1) THEN
        nMoveNumber := DEF_EXTRACT_NUM
    ELSE
    IF (nMoveNumber > MAX_EXTRACT_NUM) THEN
        nMoveNumber := MAX_EXTRACT_NUM;

    bRemoveComment := IniFile.ReadBool('Extract', 'Remove all destination comment before extraction', bRemoveComment);
    bGenOccurrence := IniFile.ReadBool('Extract', 'Generate destination occurrence comment', bGenOccurrence);

    { FCD - handle }
    fFCD1.Color  := IniFile.ReadInteger('FCD', 'Handle Foreground color', fFCD1.Color);
    fFCD1.Name   := IniFile.ReadString ('FCD', 'Handle Font name',        fFCD1.Name);
    fFCD1.Height := IniFile.ReadInteger('FCD', 'Handle Font height',      fFCD1.Height);
    fFCD1.Size   := IniFile.ReadInteger('FCD', 'Handle Font size',        fFCD1.Size);

    cTmp := IniFile.ReadString('FCD', 'Handle Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fFCD1.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fFCD1.Pitch := fpFixed
    else
        fFCD1.Pitch := fpDefault;

    fFCD1.Style := [];

    IF (IniFile.ReadBool('FCD', 'Handle Font style bold', FALSE)) THEN
        fFCD1.Style := fFCD1.Style + [fsBold];

    IF (IniFile.ReadBool('FCD', 'Handle Font style italic', FALSE)) THEN
        fFCD1.Style := fFCD1.Style + [fsItalic];

    IF (IniFile.ReadBool('FCD', 'Handle Font style underline', FALSE)) THEN
        fFCD1.Style := fFCD1.Style + [fsUnderline];

    IF (IniFile.ReadBool('FCD', 'Handle Font style strikeout', FALSE)) THEN
        fFCD1.Style := fFCD1.Style + [fsStrikeOut];

    { FCD - move }
    fFCD2.Color  := IniFile.ReadInteger('FCD', 'Move Foreground color', fFCD2.Color);
    fFCD2.Name   := IniFile.ReadString ('FCD', 'Move Font name',        fFCD2.Name);
    fFCD2.Height := IniFile.ReadInteger('FCD', 'Move Font height',      fFCD2.Height);
    fFCD2.Size   := IniFile.ReadInteger('FCD', 'Move Font size',        fFCD2.Size);

    cTmp := IniFile.ReadString('FCD', 'Move Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fFCD2.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fFCD2.Pitch := fpFixed
    else
        fFCD2.Pitch := fpDefault;

    fFCD2.Style := [];

    IF (IniFile.ReadBool('FCD', 'Move Font style bold', FALSE)) THEN
        fFCD2.Style := fFCD2.Style + [fsBold];

    IF (IniFile.ReadBool('FCD', 'Move Font style italic', FALSE)) THEN
        fFCD2.Style := fFCD2.Style + [fsItalic];

    IF (IniFile.ReadBool('FCD', 'Move Font style underline', FALSE)) THEN
        fFCD2.Style := fFCD2.Style + [fsUnderline];

    IF (IniFile.ReadBool('FCD', 'Move Font style strikeout', FALSE)) THEN
        fFCD2.Style := fFCD2.Style + [fsStrikeOut];

    { FCD - comment }
    fFCD3.Color  := IniFile.ReadInteger('FCD', 'Comment Foreground color', fFCD3.Color);
    fFCD3.Name   := IniFile.ReadString ('FCD', 'Comment Font name',        fFCD3.Name);
    fFCD3.Height := IniFile.ReadInteger('FCD', 'Comment Font height',      fFCD3.Height);
    fFCD3.Size   := IniFile.ReadInteger('FCD', 'Comment Font size',        fFCD3.Size);

    cTmp := IniFile.ReadString('FCD', 'Comment Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fFCD3.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fFCD3.Pitch := fpFixed
    else
        fFCD3.Pitch := fpDefault;

    fFCD3.Style := [];

    IF (IniFile.ReadBool('FCD', 'Comment Font style bold', FALSE)) THEN
        fFCD3.Style := fFCD3.Style + [fsBold];

    IF (IniFile.ReadBool('FCD', 'Comment Font style italic', FALSE)) THEN
        fFCD3.Style := fFCD3.Style + [fsItalic];

    IF (IniFile.ReadBool('FCD', 'Comment Font style underline', FALSE)) THEN
        fFCD3.Style := fFCD3.Style + [fsUnderline];

    IF (IniFile.ReadBool('FCD', 'Comment Font style strikeout', FALSE)) THEN
        fFCD3.Style := fFCD3.Style + [fsStrikeOut];

    { FCD - chess }
    fFCD4.Color  := IniFile.ReadInteger('FCD', 'Chess Foreground color', fFCD4.Color);
    fFCD4.Name   := IniFile.ReadString ('FCD', 'Chess Font name',        fFCD4.Name);
    fFCD4.Height := IniFile.ReadInteger('FCD', 'Chess Font height',      fFCD4.Height);
    fFCD4.Size   := IniFile.ReadInteger('FCD', 'Chess Font size',        fFCD4.Size);

    cTmp := IniFile.ReadString('FCD', 'Chess Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fFCD4.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fFCD4.Pitch := fpFixed
    else
        fFCD4.Pitch := fpDefault;

    fFCD4.Style := [];

    IF (IniFile.ReadBool('FCD', 'Chess Font style bold', FALSE)) THEN
        fFCD4.Style := fFCD4.Style + [fsBold];

    IF (IniFile.ReadBool('FCD', 'Chess Font style italic', FALSE)) THEN
        fFCD4.Style := fFCD4.Style + [fsItalic];

    IF (IniFile.ReadBool('FCD', 'Chess Font style underline', FALSE)) THEN
        fFCD4.Style := fFCD4.Style + [fsUnderline];

    IF (IniFile.ReadBool('FCD', 'Chess Font style strikeout', FALSE)) THEN
        fFCD4.Style := fFCD4.Style + [fsStrikeOut];

    { FCD - result }
    fFCD5.Color  := IniFile.ReadInteger('FCD', 'Result Foreground color', fFCD5.Color);
    fFCD5.Name   := IniFile.ReadString ('FCD', 'Result Font name',        fFCD5.Name);
    fFCD5.Height := IniFile.ReadInteger('FCD', 'Result Font height',      fFCD5.Height);
    fFCD5.Size   := IniFile.ReadInteger('FCD', 'Result Font size',        fFCD5.Size);

    cTmp := IniFile.ReadString('FCD', 'Result Font pitch', 'Default');
    IF (CompareStr(cTmp, 'Variable') = 0) THEN
        fFCD5.Pitch := fpVariable
    else
    IF (CompareStr(cTmp, 'Fixed') = 0) THEN
        fFCD5.Pitch := fpFixed
    else
        fFCD5.Pitch := fpDefault;

    fFCD5.Style := [];

    IF (IniFile.ReadBool('FCD', 'Result Font style bold', FALSE)) THEN
        fFCD5.Style := fFCD5.Style + [fsBold];

    IF (IniFile.ReadBool('FCD', 'Result Font style italic', FALSE)) THEN
        fFCD5.Style := fFCD5.Style + [fsItalic];

    IF (IniFile.ReadBool('FCD', 'Result Font style underline', FALSE)) THEN
        fFCD5.Style := fFCD5.Style + [fsUnderline];

    IF (IniFile.ReadBool('FCD', 'Result Font style strikeout', FALSE)) THEN
        fFCD5.Style := fFCD5.Style + [fsStrikeOut];

    { FCD - includes }
    bRTFHand := IniFile.ReadBool('FCD', 'Include handle',     bRTFHand);
    bRTFMove := IniFile.ReadBool('FCD', 'Include move',       bRTFMove);
    bRTFAnno := IniFile.ReadBool('FCD', 'Include annotation', bRTFAnno);
    bRTFTime := IniFile.ReadBool('FCD', 'Include time',       bRTFTime);
    bRTFNote := IniFile.ReadBool('FCD', 'Include comment',    bRTFNote);
    bRTFDisp := IniFile.ReadBool('FCD', 'Include display',    bRTFDisp);
    bRTFRest := IniFile.ReadBool('FCD', 'Include result',     bRTFRest);
    bRTFZero := IniFile.ReadBool('FCD', 'Use zeros instead of Os in castling move', bRTFZero);

    { FCD - RTF file }
    cRTFFile := IniFile.ReadString('FCD', 'Destination RTF filename', cRTFFile);
    IF (Length(cRTFFile) = 0) THEN
        cRTFFile := ExtractFilePath(ParamStr(0)) + 'GAME.' + RTF_EXT;

    { TXT - includes }
    bTXTHand := IniFile.ReadBool('TXT', 'Include handle',     bTXTHand);
    bTXTMove := IniFile.ReadBool('TXT', 'Include move',       bTXTMove);
    bTXTAnno := IniFile.ReadBool('TXT', 'Include annotation', bTXTAnno);
    bTXTTime := IniFile.ReadBool('TXT', 'Include time',       bTXTTime);
    bTXTNote := IniFile.ReadBool('TXT', 'Include comment',    bTXTNote);
    bTXTDisp := IniFile.ReadBool('TXT', 'Include display',    bTXTDisp);
    bTXTRest := IniFile.ReadBool('TXT', 'Include result',     bTXTRest);
    bTXTZero := IniFile.ReadBool('TXT', 'Use zeros instead of Os in castling move', bTXTZero);

    { TXT - move piece keymap }
    cTmp := IniFile.ReadString('TXT', 'White pawn',   '"P"');
    IF (Length(cTmp) < 3) THEN cTXTWP := cTmp ELSE cTXTWP := 'P';

    cTmp := IniFile.ReadString('TXT', 'White knight', '"N"');
    IF (Length(cTmp) < 3) THEN cTXTWN := cTmp ELSE cTXTWN := 'N';

    cTmp := IniFile.ReadString('TXT', 'White bishop', '"B"');
    IF (Length(cTmp) < 3) THEN cTXTWB := cTmp ELSE cTXTWB := 'B';

    cTmp := IniFile.ReadString('TXT', 'White rook',   '"R"');
    IF (Length(cTmp) < 3) THEN cTXTWR := cTmp ELSE cTXTWR := 'R';

    cTmp := IniFile.ReadString('TXT', 'White queen',  '"Q"');
    IF (Length(cTmp) < 3) THEN cTXTWQ := cTmp ELSE cTXTWQ := 'Q';

    cTmp := IniFile.ReadString('TXT', 'White king',  '"K"');
    IF (Length(cTmp) < 3) THEN cTXTWK := cTmp ELSE cTXTWK := 'K';

    cTmp := IniFile.ReadString('TXT', 'Black pawn',   '"*P"');
    IF (Length(cTmp) < 3) THEN cTXTBP := cTmp ELSE cTXTBP := 'P';

    cTmp := IniFile.ReadString('TXT', 'Black knight', '"*N"');
    IF (Length(cTmp) < 3) THEN cTXTBN := cTmp ELSE cTXTBN := 'N';

    cTmp := IniFile.ReadString('TXT', 'Black bishop', '"*B"');
    IF (Length(cTmp) < 3) THEN cTXTBB := cTmp ELSE cTXTBB := 'B';

    cTmp := IniFile.ReadString('TXT', 'Black rook',   '"*R"');
    IF (Length(cTmp) < 3) THEN cTXTBR := cTmp ELSE cTXTBR := 'R';

    cTmp := IniFile.ReadString('TXT', 'Black queen',  '"*Q"');
    IF (Length(cTmp) < 3) THEN cTXTBQ := cTmp ELSE cTXTBQ := 'Q';

    cTmp := IniFile.ReadString('TXT', 'Black king',  '"*K"');
    IF (Length(cTmp) < 3) THEN cTXTBK := cTmp ELSE cTXTBK := 'K';

    { TXT - chess piece keymap }
    cTmp := IniFile.ReadString('TXT', 'Chess white pawn',   '"P"');
    IF (Length(cTmp) < 3) THEN cTXTCWP := cTmp ELSE cTXTCWP := 'P';

    cTmp := IniFile.ReadString('TXT', 'Chess white knight', '"N"');
    IF (Length(cTmp) < 3) THEN cTXTCWN := cTmp ELSE cTXTCWN := 'N';

    cTmp := IniFile.ReadString('TXT', 'Chess white bishop', '"B"');
    IF (Length(cTmp) < 3) THEN cTXTCWB := cTmp ELSE cTXTCWB := 'B';

    cTmp := IniFile.ReadString('TXT', 'Chess white rook',   '"R"');
    IF (Length(cTmp) < 3) THEN cTXTCWR := cTmp ELSE cTXTCWR := 'R';

    cTmp := IniFile.ReadString('TXT', 'Chess white queen',  '"Q"');
    IF (Length(cTmp) < 3) THEN cTXTCWQ := cTmp ELSE cTXTCWQ := 'Q';

    cTmp := IniFile.ReadString('TXT', 'Chess white king',  '"K"');
    IF (Length(cTmp) < 3) THEN cTXTCWK := cTmp ELSE cTXTCWK := 'K';

    cTmp := IniFile.ReadString('TXT', 'Chess black pawn',   '"*P"');
    IF (Length(cTmp) < 3) THEN cTXTCBP := cTmp ELSE cTXTCBP := '*P';

    cTmp := IniFile.ReadString('TXT', 'Chess black knight', '"*N"');
    IF (Length(cTmp) < 3) THEN cTXTCBN := cTmp ELSE cTXTCBN := '*N';

    cTmp := IniFile.ReadString('TXT', 'Chess black bishop', '"*B"');
    IF (Length(cTmp) < 3) THEN cTXTCBB := cTmp ELSE cTXTCBB := '*B';

    cTmp := IniFile.ReadString('TXT', 'Chess black rook',   '"*R"');
    IF (Length(cTmp) < 3) THEN cTXTCBR := cTmp ELSE cTXTCBR := '*R';

    cTmp := IniFile.ReadString('TXT', 'Chess black queen',  '"*Q"');
    IF (Length(cTmp) < 3) THEN cTXTCBQ := cTmp ELSE cTXTCBQ := '*Q';

    cTmp := IniFile.ReadString('TXT', 'Chess black king',  '"*K"');
    IF (Length(cTmp) < 3) THEN cTXTCBK := cTmp ELSE cTXTCBK := '*K';

    { TXT - TXT file }
    cTXTFile := IniFile.ReadString('TXT', 'Destination TXT filename', cTXTFile);
    IF (Length(cTXTFile) = 0) THEN
        cTXTFile := ExtractFilePath(ParamStr(0)) + 'GAME.' + TXT_EXT;

    { System }
    bSaveOptionsOnExit := IniFile.ReadBool('System', 'Save options on exit', bSaveOptionsOnExit);
    bSaveLayoutsOnExit := IniFile.ReadBool('System', 'Save layouts on exit', bSaveLayoutsOnExit);

    cNewPath := IniFile.ReadString('System', 'New directory', cNewPath);
    IF (Length(cNewPath) = 0) THEN
        cNewPath := ExtractFilePath(ParamStr(0));

    cOpenPath := IniFile.ReadString('System', 'Open directory', cOpenPath);
    IF (Length(cOpenPath) = 0) THEN
        cOpenPath := ExtractFilePath(ParamStr(0));

    cDeletePath := IniFile.ReadString('System', 'Delete directory', cDeletePath);
    IF (Length(cDeletePath) = 0) THEN
        cDeletePath := ExtractFilePath(ParamStr(0));

    cSaveAsPath := IniFile.ReadString('System', 'Save as directory', cSaveAsPath);
    IF (Length(cSaveAsPath) = 0) THEN
        cSaveAsPath := ExtractFilePath(ParamStr(0));

    cworkbpgn := IniFile.ReadString('System', 'Work Bpgn File', cWorkBpgn);
    IF (Length(cworkbpgn) = 0) THEN
        cworkbpgn := ExtractFilePath(ParamStr(0))+defcworkbpgn;


    cExportPath := IniFile.ReadString('System', 'Export directory', cExportPath);
    IF (Length(cExportPath) = 0) THEN
        cExportPath := ExtractFilePath(ParamStr(0));


    { View actual text }
    nVATL := IniFile.ReadInteger('View actual text', 'Left',   nVATL);
    nVATT := IniFile.ReadInteger('View actual text', 'Top',    nVATT);
    nVATW := IniFile.ReadInteger('View actual text', 'Width',  nVATW);
    nVATH := IniFile.ReadInteger('View actual text', 'Height', nVATH);

    { View extract.err }
    nVEXL := IniFile.ReadInteger('View extract.err', 'Left',   nVEXL);
    nVEXT := IniFile.ReadInteger('View extract.err', 'Top',    nVEXT);
    nVEXW := IniFile.ReadInteger('View extract.err', 'Width',  nVEXW);
    nVEXH := IniFile.ReadInteger('View extract.err', 'Height', nVEXH);

    { View RTF file }
    nRTFL   := IniFile.ReadInteger('View RTF file', 'Left',   nRTFL);
    nRTFT   := IniFile.ReadInteger('View RTF file', 'Top',    nRTFT);
    nRTFW   := IniFile.ReadInteger('View RTF file', 'Width',  nRTFW);
    nRTFH   := IniFile.ReadInteger('View RTF file', 'Height', nRTFH);
    cRTFDir := IniFile.ReadString('View RTF file', 'RTF directory', cRTFDir);
    IF (Length(cRTFDir) = 0) THEN
        cRTFDir := ExtractFilePath(ParamStr(0));

    { View TXT file }
    nTXTL   := IniFile.ReadInteger('View TXT file', 'Left',   nTXTL);
    nTXTT   := IniFile.ReadInteger('View TXT file', 'Top',    nTXTT);
    nTXTW   := IniFile.ReadInteger('View TXT file', 'Width',  nTXTW);
    nTXTH   := IniFile.ReadInteger('View TXT file', 'Height', nTXTH);
    cTXTDir := IniFile.ReadString('View TXT file', 'TXT directory', cTXTDir);
    IF (Length(cTXTDir) = 0) THEN
        cTXTDir := ExtractFilePath(ParamStr(0));

        { Player Stats }
    nstatL   := IniFile.ReadInteger('Player stats', 'Left',   nstatL);
    nStatT   := IniFile.ReadInteger('Player stats', 'Top',    nstatT);
    nStatW   := IniFile.ReadInteger('Player stats', 'Width',  nstatW);
    nStatH   := IniFile.ReadInteger('Player stats', 'Height', nstath);

    {filter window}
    nfilterL   := IniFile.ReadInteger('Filter', 'Left',nmainL+nmainW-nFilterW-6);
    nfilterT   := IniFile.ReadInteger('Filter', 'Top', nmaint+nmainH-nfilterH-10);
    nfilterW   := IniFile.ReadInteger('Filter', 'Width',  nFilterW);
    nfilterH   := IniFile.ReadInteger('Filter', 'Height', nFilterH);
    bFilterSt  :=inifile.ReadBool('Filter','Stats',bfilterst);

    nTreeL   := IniFile.ReadInteger('Tree', 'Left',nmainL+nmainW-nFilterW-6);
    nTreeT   := IniFile.ReadInteger('Tree', 'Top', nmaint+nmainH-nfilterH-10);
    nTreeW   := IniFile.ReadInteger('Tree', 'Width',  nTreeW);
    nTreeH   := IniFile.ReadInteger('Tree', 'Height', nTreeH);
    bTreeFil :=inifile.ReadBool('Tree','Filter',bTreeFil);
//    bTreeAuto:=inifile.ReadBool('Tree','Auto',bTreeAuto);

    { Move piece keymap }
    cTmp := IniFile.ReadString('Move piece keymap', 'White pawn',   '"P"');
    IF (Length(cTmp) < 3) THEN cWP := cTmp ELSE cWP := 'P';

    cTmp := IniFile.ReadString('Move piece keymap', 'White knight', '"N"');
    IF (Length(cTmp) < 3) THEN cWN := cTmp ELSE cWN := 'N';

    cTmp := IniFile.ReadString('Move piece keymap', 'White bishop', '"B"');
    IF (Length(cTmp) < 3) THEN cWB := cTmp ELSE cWB := 'B';

    cTmp := IniFile.ReadString('Move piece keymap', 'White rook',   '"R"');
    IF (Length(cTmp) < 3) THEN cWR := cTmp ELSE cWR := 'R';

    cTmp := IniFile.ReadString('Move piece keymap', 'White queen',  '"Q"');
    IF (Length(cTmp) < 3) THEN cWQ := cTmp ELSE cWQ := 'Q';

    cTmp := IniFile.ReadString('Move piece keymap', 'White king',   '"K"');
    IF (Length(cTmp) < 3) THEN cWK := cTmp ELSE cWK := 'K';

    cTmp := IniFile.ReadString('Move piece keymap', 'Black pawn',   '"P"');
    IF (Length(cTmp) < 3) THEN cBP := cTmp ELSE cBP := 'P';

    cTmp := IniFile.ReadString('Move piece keymap', 'Black knight', '"N"');
    IF (Length(cTmp) < 3) THEN cBN := cTmp ELSE cBN := 'N';

    cTmp := IniFile.ReadString('Move piece keymap', 'Black bishop', '"B"');
    IF (Length(cTmp) < 3) THEN cBB := cTmp ELSE cBB := 'B';

    cTmp := IniFile.ReadString('Move piece keymap', 'Black rook',   '"R"');
    IF (Length(cTmp) < 3) THEN cBR := cTmp ELSE cBR := 'R';

    cTmp := IniFile.ReadString('Move piece keymap', 'Black queen',  '"Q"');
    IF (Length(cTmp) < 3) THEN cBQ := cTmp ELSE cBQ := 'Q';

    cTmp := IniFile.ReadString('Move piece keymap', 'Black king',   '"K"');
    IF (Length(cTmp) < 3) THEN cBK := cTmp ELSE cBK := 'K';

    bZero := IniFile.ReadBool('Move piece keymap', 'Use zeros instead of Os in castling move', bZero);

    { Chess font keymap }
    cTmp := IniFile.ReadString('Chess font keymap', 'White pawn on white square',   '"p"');
    IF (Length(cTmp) < 3) THEN cWPWS := cTmp ELSE cWPWS := 'p';

    cTmp := IniFile.ReadString('Chess font keymap', 'White knight on white square', '"n"');
    IF (Length(cTmp) < 3) THEN cWNWS := cTmp ELSE cWNWS := 'n';

    cTmp := IniFile.ReadString('Chess font keymap', 'White bishop on white square', '"b"');
    IF (Length(cTmp) < 3) THEN cWBWS := cTmp ELSE cWBWS := 'b';

    cTmp := IniFile.ReadString('Chess font keymap', 'White rook on white square',   '"r"');
    IF (Length(cTmp) < 3) THEN cWRWS := cTmp ELSE cWRWS := 'r';

    cTmp := IniFile.ReadString('Chess font keymap', 'White queen on white square',  '"q"');
    IF (Length(cTmp) < 3) THEN cWQWS := cTmp ELSE cWQWS := 'q';

    cTmp := IniFile.ReadString('Chess font keymap', 'White king on white square',   '"k"');
    IF (Length(cTmp) < 3) THEN cWKWS := cTmp ELSE cWKWS := 'k';

    cTmp := IniFile.ReadString('Chess font keymap', 'White pawn on black square',   '"P"');
    IF (Length(cTmp) < 3) THEN cWPBS := cTmp ELSE cWPBS := 'P';

    cTmp := IniFile.ReadString('Chess font keymap', 'White knight on black square', '"N"');
    IF (Length(cTmp) < 3) THEN cWNBS := cTmp ELSE cWNBS := 'N';

    cTmp := IniFile.ReadString('Chess font keymap', 'White bishop on black square', '"B"');
    IF (Length(cTmp) < 3) THEN cWBBS := cTmp ELSE cWBBS := 'B';

    cTmp := IniFile.ReadString('Chess font keymap', 'White rook on black square',   '"R"');
    IF (Length(cTmp) < 3) THEN cWRBS := cTmp ELSE cWRBS := 'R';

    cTmp := IniFile.ReadString('Chess font keymap', 'White queen on black square',  '"Q"');
    IF (Length(cTmp) < 3) THEN cWQBS := cTmp ELSE cWQBS := 'Q';

    cTmp := IniFile.ReadString('Chess font keymap', 'White king on black square',   '"K"');
    IF (Length(cTmp) < 3) THEN cWKBS := cTmp ELSE cWKBS := 'K';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black pawn on white square',   '"o"');
    IF (Length(cTmp) < 3) THEN cBPWS := cTmp ELSE cBPWS := 'o';

    cTmp := IniFile.ReadString('Chess font keymap', 'Move indicator',   '">"');
    IF (Length(cTmp) < 3) THEN cMoveInd := cTmp ELSE cMoveInd := MOVE_INDICATOR;

    cTmp := IniFile.ReadString('Chess font keymap', 'Black knight on white square', '"m"');
    IF (Length(cTmp) < 3) THEN cBNWS := cTmp ELSE cBNWS := 'm';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black bishop on white square', '"v"');
    IF (Length(cTmp) < 3) THEN cBBWS := cTmp ELSE cBBWS := 'v';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black rook on white square',   '"t"');
    IF (Length(cTmp) < 3) THEN cBRWS := cTmp ELSE cBRWS := 't';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black queen on white square',  '"w"');
    IF (Length(cTmp) < 3) THEN cBQWS := cTmp ELSE cBQWS := 'w';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black king on white square',   '"l"');
    IF (Length(cTmp) < 3) THEN cBKWS := cTmp ELSE cBKWS := 'l';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black pawn on black square',   '"O"');
    IF (Length(cTmp) < 3) THEN cBPBS := cTmp ELSE cBPBS := 'O';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black knight on black square', '"M"');
    IF (Length(cTmp) < 3) THEN cBNBS := cTmp ELSE cBNBS := 'M';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black bishop on black square', '"V"');
    IF (Length(cTmp) < 3) THEN cBBBS := cTmp ELSE cBBBS := 'V';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black rook on black square',   '"T"');
    IF (Length(cTmp) < 3) THEN cBRBS := cTmp ELSE cBRBS := 'T';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black queen on black square',  '"W"');
    IF (Length(cTmp) < 3) THEN cBQBS := cTmp ELSE cBQBS := 'W';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black king on black square',   '"L"');
    IF (Length(cTmp) < 3) THEN cBKBS := cTmp ELSE cBKBS := 'L';

    cTmp := IniFile.ReadString('Chess font keymap', 'White square', '"*"');
    IF (Length(cTmp) < 3) THEN cWS := cTmp ELSE cWS := '*';

    cTmp := IniFile.ReadString('Chess font keymap', 'Black square', '"+"');
    IF (Length(cTmp) < 3) THEN cBS := cTmp ELSE cBS := '+';

    cTmp := IniFile.ReadString('Chess font keymap', 'Top left corner', '"!"');
    IF (Length(cTmp) < 3) THEN cTopLC := cTmp ELSE cTopLC := '!';

    cTmp := IniFile.ReadString('Chess font keymap', 'Top border', '"""');
    IF (Length(cTmp) < 3) THEN cTopB := cTmp ELSE cTopB := '"';

    cTmp := IniFile.ReadString('Chess font keymap', 'Top right corner', '"#"');
    IF (Length(cTmp) < 3) THEN cTopRC := cTmp ELSE cTopRC := '#';

    cTmp := IniFile.ReadString('Chess font keymap', 'Left border', '"$"');
    IF (Length(cTmp) < 3) THEN cLeftB := cTmp ELSE cLeftB := '$';

    cTmp := IniFile.ReadString('Chess font keymap', 'Right border', '"%"');
    IF (Length(cTmp) < 3) THEN cRightB := cTmp ELSE cRightB := '%';

    cTmp := IniFile.ReadString('Chess font keymap', 'Bottom left corner', '"/"');
    IF (Length(cTmp) < 3) THEN cBotLC := cTmp ELSE cBotLC := '/';

    cTmp := IniFile.ReadString('Chess font keymap', 'Bottom border', '"("');
    IF (Length(cTmp) < 3) THEN cBotB := cTmp ELSE cBotB := '(';

    cTmp := IniFile.ReadString('Chess font keymap', 'Bottom right corner', '")"');
    IF (Length(cTmp) < 3) THEN cBotRC := cTmp ELSE cBotRC := ')';

    { Custom colors }
    cColorA := IniFile.ReadString('Custom colors', 'A', cColorA);
    cColorB := IniFile.ReadString('Custom colors', 'B', cColorB);
    cColorC := IniFile.ReadString('Custom colors', 'C', cColorC);
    cColorD := IniFile.ReadString('Custom colors', 'D', cColorD);
    cColorE := IniFile.ReadString('Custom colors', 'E', cColorE);
    cColorF := IniFile.ReadString('Custom colors', 'F', cColorF);
    cColorG := IniFile.ReadString('Custom colors', 'G', cColorG);
    cColorH := IniFile.ReadString('Custom colors', 'H', cColorH);
    cColorI := IniFile.ReadString('Custom colors', 'I', cColorI);
    cColorJ := IniFile.ReadString('Custom colors', 'J', cColorJ);
    cColorK := IniFile.ReadString('Custom colors', 'K', cColorK);
    cColorL := IniFile.ReadString('Custom colors', 'L', cColorL);
    cColorM := IniFile.ReadString('Custom colors', 'M', cColorM);
    cColorN := IniFile.ReadString('Custom colors', 'N', cColorN);
    cColorO := IniFile.ReadString('Custom colors', 'O', cColorO);
    cColorP := IniFile.ReadString('Custom colors', 'P', cColorP);

    bairdropini:=IniFile.ReadBool('Bpgn', 'Air drops',bairdropini);
    savedepth:=IniFile.ReadInteger('Database','Save depth',default_db_save_depth);
    bROgames:=inifile.readBool('Database','RO games', bROgames);
    bOpenTree:=inifile.ReadBool('Database','Open tree',bOpenTree);
//    nshortg:=IniFile.ReadInteger('Bpgn','Short game max',nShortG);
    bfullstat:=inifile.ReadBool('Filter', 'Full stat',bfullstat);
    nMoveOrder:=inifile.ReadInteger('Filter','Move order',nMoveOrder);
    bTimeError:=iniFile.ReadBool('Bpgn','Time error',false);
    cDBSite:=IniFile.ReadString('Web','Database',cDBSite);
   except
    IniFile.Free;
   end;

   try
     reg:=tregistry.Create;
     Reg.RootKey:=HKEY_Local_Machine;
     nokey:=false;
     if reg.openKey('\Software\Bughouse',false) then
      graphhome0:=reg.readstring('graphpath')
     else nokey:=true;
     if trim(graphhome0)='' then nokey:=true;
     if not nokey then wrongkey:=not validgraphhome(graphhome0);
     if nokey or wrongkey then
     begin
       graphhome2:=tmpgraphhome;
       graphhome3:=extractfilepath(ExcludeTrailingbackslash(b_dir));
       if validgraphhome(graphhome1) then
       begin
        global.graphhome:=graphhome1;
        if nokey then
         if reg.openkey('\Software\Bughouse',true)
         then reg.writestring('graphpath',global.graphhome);
        tmpgraphhome:=global.graphhome;
       end else
       if validgraphhome(graphhome2) then
       begin
        graphhome:=graphhome2;
        if nokey then
         if reg.openkey('\Software\Bughouse',true)
         then reg.writestring('graphpath',global.graphhome);
        tmpgraphhome:=global.graphhome;
       end else
       if validgraphhome(graphhome3) then
       begin
        graphhome:=graphhome1;
        if nokey then
         if reg.openkey('\Software\Bughouse',true)
         then reg.writestring('graphpath',global.graphhome);
         tmpgraphhome:=graphhome;
       end
       else graphhome:=graphhome2;
     end
     else
      graphhome:=graphhome0;
   finally
     reg.closekey;
     reg.free;
   end;
END;

PROCEDURE TConfig.WriteAll;
VAR
    IniFile: TIniFile;
    i:integer;
    tmp:string;
BEGIN
    IniFile := TIniFile.Create(cFile);

    { Main }
    IniFile.WriteInteger('Main', 'Left',   nMainL);
    IniFile.WriteInteger('Main', 'Top',    nMainT);
    IniFile.WriteInteger('Main', 'Width',  nMainW);
    IniFile.WriteInteger('Main', 'Height', nMainH);
    IniFile.WriteInteger('Main', 'State',  nMainS);

    { Board }
    inifile.writestring('Board', 'graphics home',graphhome);
    IniFile.WriteInteger('Board', 'White square color',  nWhtSquare);
    IniFile.WriteInteger('Board', 'White square outline color', nWhtSOutline);
    IniFile.WriteInteger('Board', 'White piece color',   nWhtFill);
    IniFile.WriteInteger('Board', 'White piece outline color', nWhtOutline);
    IniFile.WriteInteger('Board', 'Black square color',  nBlkSquare);
    IniFile.WriteInteger('Board', 'Black square outline color', nBlkSOutline);
    IniFile.WriteInteger('Board', 'Black piece color',   nBlkFill);
    IniFile.WriteInteger('Board', 'Black piece outline color', nBlkOutline);
    IniFile.WriteBool('Board', 'Show piece outline when mouse clicked', bOutlinePC);
    IniFile.WriteInteger('Board', 'Piece outline color when mouse clicked', nClickColor);
    IniFile.WriteBool('Board', 'Show piece count', bShwPCCount);
    IniFile.WriteInteger('Board', 'Piece count color', nNumberColor);
    IniFile.WriteInteger('Board', 'Piece square color', nPcSquare);
    IniFile.WriteBool('Board', 'Flip board',   bFlipBd);
    inifile.WriteBool('Board', 'use bmp', usebmp);
    inifile.WriteBool('Board', 'use piece bmp', usepiecebmp);
    inifile.writebool('Board', 'use board bmp', useboardbmp);
    Inifile.WriteString('Board', 'bmp file', bmpfile);
    Inifile.WriteString('Board', 'piece dir', p_dir);
    inifile.WRiteSTring('Board', 'board dir', b_dir);
    IniFile.WriteBool('Board', 'Always queen', bAlwaysQ);
    IniFile.WriteBool('Board', 'Drag move',    bDragMove);
    IniFile.WriteBool('Board', 'Smart move',   bSmartMove);
    IniFile.WriteBool('Board', 'Restrict board window to actual board size', bActualBoard);
    IniFile.WriteInteger('Board', 'Board outline color', nBoardOutline);
    IniFile.WriteInteger('Board', 'Board window background color', nBoardWindow);
    IniFile.WriteBool('Board', 'Show square outline', bShowSOutline);
    IniFile.WriteBool('Board', 'Show lag', bShowlag);
    IniFile.WriteBool('Board', 'Show tenth', bShowtenth);

    { Clock }
    IniFile.WriteInteger('Clock', 'Wait foreground color', nWaitF);
    IniFile.WriteInteger('Clock', 'Wait background color', nWaitB);
    IniFile.WriteInteger('Clock', 'Turn foreground color', nTurnF);
    IniFile.WriteInteger('Clock', 'Turn background color', nTurnB);
    IniFile.WriteString ('Clock', 'Font name', fClock.Name);
    IniFile.WriteInteger('Clock', 'Font height', fClock.Height);
    IniFile.WriteInteger('Clock', 'Font size', fClock.Size);

    CASE (fClock.Pitch) OF
        fpVariable: IniFile.WriteString('Clock', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Clock', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Clock', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Clock', 'Font style bold',      (fsBold      IN fClock.Style));
    IniFile.WriteBool('Clock', 'Font style italic',    (fsItalic    IN fClock.Style));
    IniFile.WriteBool('Clock', 'Font style underline', (fsUnderline IN fClock.Style));
    IniFile.WriteBool('Clock', 'Font style strikeout', (fsStrikeOut IN fClock.Style));

    { Handle }
    IniFile.WriteInteger('Handle', 'Background color', nHandleB);
    IniFile.WriteInteger('Handle', 'Foreground color', fHandle.Color);
    IniFile.WriteString ('Handle', 'Font name', fHandle.Name);
    IniFile.WriteInteger('Handle', 'Font height', fHandle.Height);
    IniFile.WriteInteger('Handle', 'Font size', fHandle.Size);

    CASE (fHandle.Pitch) OF
        fpVariable: IniFile.WriteString('Handle', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Handle', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Handle', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Handle', 'Font style bold',      (fsBold      IN fHandle.Style));
    IniFile.WriteBool('Handle', 'Font style italic',    (fsItalic    IN fHandle.Style));
    IniFile.WriteBool('Handle', 'Font style underline', (fsUnderline IN fHandle.Style));
    IniFile.WriteBool('Handle', 'Font style strikeout', (fsStrikeOut IN fHandle.Style));

    { Board #1 }
    IniFile.WriteInteger('Board #1', 'Left',   nBoard1L);
    IniFile.WriteInteger('Board #1', 'Top',    nBoard1T);
    IniFile.WriteInteger('Board #1', 'Width',  nBoard1W);
    IniFile.WriteInteger('Board #1', 'Height', nBoard1H);
    IniFile.WriteInteger('Board #1', 'State',  nBoard1S);

    { Board #2 }
    IniFile.WriteInteger('Board #2', 'Left',   nBoard2L);
    IniFile.WriteInteger('Board #2', 'Top',    nBoard2T);
    IniFile.WriteInteger('Board #2', 'Width',  nBoard2W);
    IniFile.WriteInteger('Board #2', 'Height', nBoard2H);
    IniFile.WriteInteger('Board #2', 'State',  nBoard2S);

    { Navigate }
    IniFile.WriteInteger('Navigate', 'Left',   nNavigateL);
    IniFile.WriteInteger('Navigate', 'Top',    nNavigateT);
    inifile.WriteBool('Navigate','Sticky', nNavigateSticky);
    IniFile.WriteInteger('Navigate', 'Width',   nNavigateW);
    IniFile.WriteInteger('Navigate', 'Height',    nNavigateH);
    IniFile.WriteInteger('Navigate', 'State',  nNavigateS);
    inifile.writebool('Navigate','Simple',bnavigatesimple);

    { Game list }
    IniFile.WriteInteger('Game list', 'Left',   nGameListL);
    IniFile.WriteInteger('Game list', 'Top',    nGameListT);
    IniFile.WriteInteger('Game list', 'Width',  nGameListW);
    IniFile.WriteInteger('Game list', 'Height', nGameListH);
    IniFile.WriteInteger('Game list', 'State',  nGameListS);
    IniFile.WriteInteger('Game list', 'Background color', nGameListB);
    IniFile.WriteInteger('Game list', 'Foreground color', fGameListF.Color);
    IniFile.WriteString ('Game list', 'Font name', fGameListF.Name);
    IniFile.WriteInteger('Game list', 'Font height', fGameListF.Height);
    IniFile.WriteInteger('Game list', 'Font size', fGameListF.Size);

    CASE (fGameListF.Pitch) OF
        fpVariable: IniFile.WriteString('Game list', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Game list', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Game list', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Game list', 'Font style bold',      (fsBold      IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Font style italic',    (fsItalic    IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Font style underline', (fsUnderline IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Font style strikeout', (fsStrikeOut IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Show game #',    bShowGameNum);
    IniFile.WriteBool('Game list', 'Show event',     bShowEvent);
    IniFile.WriteBool('Game list', 'Show ratings',   bShowRatings);
    IniFile.WriteBool('Game list', 'Show time',      bShowTime);
    IniFile.WriteBool('Game list', 'Show increment', bShowInc);
    IniFile.WriteBool('Game list', 'Show result',    bShowResult);

    { Next move }
    IniFile.WriteInteger('Next move', 'Left',   nNextMoveL);
    IniFile.WriteInteger('Next move', 'Top',    nNextMoveT);
    IniFile.WriteInteger('Next move', 'Width',  nNextMoveW);
    IniFile.WriteInteger('Next move', 'Height', nNextMoveH);
    IniFile.WriteInteger('Next move', 'State',  nNextMoveS);
    IniFile.WriteInteger('Next move', 'Background color', nNextMoveB);
    IniFile.WriteInteger('Next move', 'Foreground color', fNextMoveF.Color);
    IniFile.WriteString ('Next move', 'Font name', fNextMoveF.Name);
    IniFile.WriteInteger('Next move', 'Font height', fNextMoveF.Height);
    IniFile.WriteInteger('Next move', 'Font size', fNextMoveF.Size);

    CASE (fNextMoveF.Pitch) OF
        fpVariable: IniFile.WriteString('Next move', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Next move', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Next move', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Next move', 'Font style bold',      (fsBold      IN fNextMoveF.Style));
    IniFile.WriteBool('Next move', 'Font style italic',    (fsItalic    IN fNextMoveF.Style));
    IniFile.WriteBool('Next move', 'Font style underline', (fsUnderline IN fNextMoveF.Style));
    IniFile.WriteBool('Next move', 'Font style strikeout', (fsStrikeOut IN fNextMoveF.Style));

    { Previous next move }
    IniFile.WriteInteger('Previous next move', 'Left',   nPrevMoveL);
    IniFile.WriteInteger('Previous next move', 'Top',    nPrevMoveT);
    IniFile.WriteInteger('Previous next move', 'Width',  nPrevMoveW);
    IniFile.WriteInteger('Previous next move', 'Height', nPrevMoveH);
    IniFile.WriteInteger('Previous next move', 'State',  nPrevMoveS);
    IniFile.WriteInteger('Previous next move', 'Background color', nPrevMoveB);
    IniFile.WriteInteger('Previous next move', 'Foreground color', fPrevMoveF.Color);
    IniFile.WriteString ('Previous next move', 'Font name', fPrevMoveF.Name);
    IniFile.WriteInteger('Previous next move', 'Font height', fPrevMoveF.Height);
    IniFile.WriteInteger('Previous next move', 'Font size', fPrevMoveF.Size);

    CASE (fPrevMoveF.Pitch) OF
        fpVariable: IniFile.WriteString('Previous next move', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Previous next move', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Previous next move', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Previous next move', 'Font style bold',      (fsBold      IN fPrevMoveF.Style));
    IniFile.WriteBool('Previous next move', 'Font style italic',    (fsItalic    IN fPrevMoveF.Style));
    IniFile.WriteBool('Previous next move', 'Font style underline', (fsUnderline IN fPrevMoveF.Style));
    IniFile.WriteBool('Previous next move', 'Font style strikeout', (fsStrikeOut IN fPrevMoveF.Style));

    { Comment }
    IniFile.WriteInteger('Comment', 'Left',   nCommentL);
    IniFile.WriteInteger('Comment', 'Top',    nCommentT);
    IniFile.WriteInteger('Comment', 'Width',  nCommentW);
    IniFile.WriteInteger('Comment', 'Height', nCommentH);
    IniFile.WriteInteger('Comment', 'State',  nCommentS);
    IniFile.WriteInteger('Comment', 'Background color', nCommentB);
    IniFile.WriteInteger('Comment', 'Foreground color', fCommentF.Color);
    IniFile.WriteString ('Comment', 'Font name', fCommentF.Name);
    IniFile.WriteInteger('Comment', 'Font height', fCommentF.Height);
    IniFile.WriteInteger('Comment', 'Font size', fCommentF.Size);

    CASE (fCommentF.Pitch) OF
        fpVariable: IniFile.WriteString('Comment', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Comment', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Comment', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Comment', 'Font style bold',      (fsBold      IN fCommentF.Style));
    IniFile.WriteBool('Comment', 'Font style italic',    (fsItalic    IN fCommentF.Style));
    IniFile.WriteBool('Comment', 'Font style underline', (fsUnderline IN fCommentF.Style));
    IniFile.WriteBool('Comment', 'Font style strikeout', (fsStrikeOut IN fCommentF.Style));

    IniFile.WriteBool('Comment', 'Word wrap', bCommentWR);

    inifile.writebool('Sound', 'on', bsoundon);
    inifile.writestring('Sound', 'move', csoundmove);
    inifile.writestring('Sound', 'check', csoundcheck);
    inifile.writestring('Sound', 'end', csoundend);
    inifile.writestring('Sound', 'capture', csoundcap);

    { Engine }
    for i:=1 to MAX_ENGINE do
    begin
     str(i,tmp);
     tmp:='Engine'+tmp;
     IniFile.WriteString(tmp, 'name', engineopt[i].name);
     IniFile.WriteString(tmp, 'command line', engineopt[i].comline);
     IniFile.WriteInteger(tmp, 'type', engineopt[i].etype);
     IniFile.WriteString(tmp, 'prompt', engineopt[i].prompt);
     IniFile.WriteString(tmp, 'reset', engineopt[i].reset);
     IniFile.WriteString(tmp, 'infinite time', engineopt[i].itime);
     IniFile.WriteBool(tmp, 'filter output', engineopt[i].bfilter);
     iniFile.Writebool(tmp,'analysis',engineopt[i].banal);
     IniFile.WriteInteger(tmp, 'Left',   enginelay[i].nEngineL);
     IniFile.WriteInteger(tmp, 'Top',    enginelay[i].nEngineT);
     IniFile.WriteInteger(tmp, 'Width',  enginelay[i].nEngineW);
     IniFile.WriteInteger(tmp, 'Height', enginelay[i].nEngineH);
     IniFile.WriteInteger(tmp, 'State',  enginelay[i].nEngineS);
    end;


    IniFile.WriteInteger('Engine', 'Background color', nEngineB);
    IniFile.WriteInteger('Engine', 'Foreground color', fEngineF.Color);
    IniFile.WriteString ('Engine', 'Font name', fEngineF.Name);
    IniFile.WriteInteger('Engine', 'Font height', fEngineF.Height);
    IniFile.WriteInteger('Engine', 'Font size', fEngineF.Size);

    CASE (fEngineF.Pitch) OF
        fpVariable: IniFile.WriteString('Engine', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Engine', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Engine', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Engine', 'Font style bold',      (fsBold      IN fEngineF.Style));
    IniFile.WriteBool('Engine', 'Font style italic',    (fsItalic    IN fEngineF.Style));
    IniFile.WriteBool('Engine', 'Font style underline', (fsUnderline IN fEngineF.Style));
    IniFile.WriteBool('Engine', 'Font style strikeout', (fsStrikeOut IN fEngineF.Style));

    IniFile.WriteBool('Engine', 'Word wrap', bEngineWR);
    inifile.Writefloat('Engine', 'test time', nEngineTT);
    IniFile.WriteFloat('Engine','annotate time', nengineat);
    IniFile.WriteFloat('Engine','exclude threshold', nengineet);
    inifile.WriteFloat('Engine','include threshold', nengineit);
    inifile.WriteBool('Engine', 'show mates', bEngineSM);
    IniFile.WriteBool('Engine', 'Tecumseh', tecumseh);
    IniFile.WriteInteger('Engine', 'Default engine', defeng);


    { Extract }
    IniFile.WriteString('Extract', 'Handle', cHandle);
    IniFile.WriteString('Extract', 'Source BGPN filter', cSource);
    IniFile.WriteString('Extract', 'Destination BGPN filename', cDestination);
    IniFile.WriteInteger('Extract', 'Number of moves', nMoveNumber);
    IniFile.WriteBool('Extract', 'Remove all destination comment before extraction', bRemoveComment);
    IniFile.WriteBool('Extract', 'Generate destination occurrence comment', bGenOccurrence);

        { FCD - handle }
    IniFile.WriteString ('FCD', 'Handle Font name', fFCD1.Name);
    IniFile.WriteInteger('FCD', 'Handle Foreground color', fFCD1.Color);
    IniFile.WriteInteger('FCD', 'Handle Font height', fFCD1.Height);
    IniFile.WriteInteger('FCD', 'Handle Font size', fFCD1.Size);

    CASE (fFCD1.Pitch) OF
        fpVariable: IniFile.WriteString('FCD', 'Handle Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('FCD', 'Handle Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('FCD', 'Handle Font pitch', 'Default');
    END;

    IniFile.WriteBool('FCD', 'Handle Font style bold',      (fsBold      IN fFCD1.Style));
    IniFile.WriteBool('FCD', 'Handle Font style italic',    (fsItalic    IN fFCD1.Style));
    IniFile.WriteBool('FCD', 'Handle Font style underline', (fsUnderline IN fFCD1.Style));
    IniFile.WriteBool('FCD', 'Handle Font style strikeout', (fsStrikeOut IN fFCD1.Style));

    { FCD - move }
    IniFile.WriteString ('FCD', 'Move Font name', fFCD2.Name);
    IniFile.WriteInteger('FCD', 'Move Foreground color', fFCD2.Color);
    IniFile.WriteInteger('FCD', 'Move Font height', fFCD2.Height);
    IniFile.WriteInteger('FCD', 'Move Font size', fFCD2.Size);

    CASE (fFCD2.Pitch) OF
        fpVariable: IniFile.WriteString('FCD', 'Move Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('FCD', 'Move Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('FCD', 'Move Font pitch', 'Default');
    END;

    IniFile.WriteBool('FCD', 'Move Font style bold',      (fsBold      IN fFCD2.Style));
    IniFile.WriteBool('FCD', 'Move Font style italic',    (fsItalic    IN fFCD2.Style));
    IniFile.WriteBool('FCD', 'Move Font style underline', (fsUnderline IN fFCD2.Style));
    IniFile.WriteBool('FCD', 'Move Font style strikeout', (fsStrikeOut IN fFCD2.Style));

    { FCD - comment }
    IniFile.WriteString ('FCD', 'Comment Font name', fFCD3.Name);
    IniFile.WriteInteger('FCD', 'Comment Foreground color', fFCD3.Color);
    IniFile.WriteInteger('FCD', 'Comment Font height', fFCD3.Height);
    IniFile.WriteInteger('FCD', 'Comment Font size', fFCD3.Size);

    CASE (fFCD3.Pitch) OF
        fpVariable: IniFile.WriteString('FCD', 'Comment Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('FCD', 'Comment Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('FCD', 'Comment Font pitch', 'Default');
    END;

    IniFile.WriteBool('FCD', 'Comment Font style bold',      (fsBold      IN fFCD3.Style));
    IniFile.WriteBool('FCD', 'Comment Font style italic',    (fsItalic    IN fFCD3.Style));
    IniFile.WriteBool('FCD', 'Comment Font style underline', (fsUnderline IN fFCD3.Style));
    IniFile.WriteBool('FCD', 'Comment Font style strikeout', (fsStrikeOut IN fFCD3.Style));

    { FCD - chess }
    IniFile.WriteString ('FCD', 'Chess Font name', fFCD4.Name);
    IniFile.WriteInteger('FCD', 'Chess Foreground color', fFCD4.Color);
    IniFile.WriteInteger('FCD', 'Chess Font height', fFCD4.Height);
    IniFile.WriteInteger('FCD', 'Chess Font size', fFCD4.Size);

    CASE (fFCD4.Pitch) OF
        fpVariable: IniFile.WriteString('FCD', 'Chess Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('FCD', 'Chess Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('FCD', 'Chess Font pitch', 'Default');
    END;

    IniFile.WriteBool('FCD', 'Chess Font style bold',      (fsBold      IN fFCD4.Style));
    IniFile.WriteBool('FCD', 'Chess Font style italic',    (fsItalic    IN fFCD4.Style));
    IniFile.WriteBool('FCD', 'Chess Font style underline', (fsUnderline IN fFCD4.Style));
    IniFile.WriteBool('FCD', 'Chess Font style strikeout', (fsStrikeOut IN fFCD4.Style));

    { FCD - result }
    IniFile.WriteString ('FCD', 'Result Font name', fFCD5.Name);
    IniFile.WriteInteger('FCD', 'Result Foreground color', fFCD5.Color);
    IniFile.WriteInteger('FCD', 'Result Font height', fFCD5.Height);
    IniFile.WriteInteger('FCD', 'Result Font size', fFCD5.Size);

    CASE (fFCD5.Pitch) OF
        fpVariable: IniFile.WriteString('FCD', 'Result Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('FCD', 'Result Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('FCD', 'Result Font pitch', 'Default');
    END;

    IniFile.WriteBool('FCD', 'Result Font style bold',      (fsBold      IN fFCD5.Style));
    IniFile.WriteBool('FCD', 'Result Font style italic',    (fsItalic    IN fFCD5.Style));
    IniFile.WriteBool('FCD', 'Result Font style underline', (fsUnderline IN fFCD5.Style));
    IniFile.WriteBool('FCD', 'Result Font style strikeout', (fsStrikeOut IN fFCD5.Style));

    { FCD - includes }
    IniFile.WriteBool('FCD', 'Include handle',     bRTFHand);
    IniFile.WriteBool('FCD', 'Include move',       bRTFMove);
    IniFile.WriteBool('FCD', 'Include annotation', bRTFAnno);
    IniFile.WriteBool('FCD', 'Include time',       bRTFTime);
    IniFile.WriteBool('FCD', 'Include comment',    bRTFNote);
    IniFile.WriteBool('FCD', 'Include display',    bRTFDisp);
    IniFile.WriteBool('FCD', 'Include result',     bRTFRest);
    IniFile.WriteBool('FCD', 'Use zeros instead of Os in castling move', bRTFZero);

    { FCD - RTF file }
    IniFile.WriteString('FCD', 'Destination RTF filename', cRTFFile);


    { TXT - includes }
    IniFile.WriteBool('TXT', 'Include handle',     bTXTHand);
    IniFile.WriteBool('TXT', 'Include move',       bTXTMove);
    IniFile.WriteBool('TXT', 'Include annotation', bTXTAnno);
    IniFile.WriteBool('TXT', 'Include time',       bTXTTime);
    IniFile.WriteBool('TXT', 'Include comment',    bTXTNote);
    IniFile.WriteBool('TXT', 'Include display',    bTXTDisp);
    IniFile.WriteBool('TXT', 'Include result',     bTXTRest);
    IniFile.WriteBool('TXT', 'Use zeros instead of Os in castling move', bTXTZero);

    { TXT - move piece keymap }
    IniFile.WriteString('TXT', 'White pawn',   '"' + cTXTWP + '"');
    IniFile.WriteString('TXT', 'White knight', '"' + cTXTWN + '"');
    IniFile.WriteString('TXT', 'White bishop', '"' + cTXTWB + '"');
    IniFile.WriteString('TXT', 'White rook',   '"' + cTXTWR + '"');
    IniFile.WriteString('TXT', 'White queen',  '"' + cTXTWQ + '"');
    IniFile.WriteString('TXT', 'White king',   '"' + cTXTWK + '"');

    IniFile.WriteString('TXT', 'Black pawn',   '"' + cTXTBP + '"');
    IniFile.WriteString('TXT', 'Black knight', '"' + cTXTBN + '"');
    IniFile.WriteString('TXT', 'Black bishop', '"' + cTXTBB + '"');
    IniFile.WriteString('TXT', 'Black rook',   '"' + cTXTBR + '"');
    IniFile.WriteString('TXT', 'Black queen',  '"' + cTXTBQ + '"');
    IniFile.WriteString('TXT', 'Black king',   '"' + cTXTBK + '"');

    { TXT - chess piece keymap }
    IniFile.WriteString('TXT', 'Chess white pawn',   '"' + cTXTCWP + '"');
    IniFile.WriteString('TXT', 'Chess white knight', '"' + cTXTCWN + '"');
    IniFile.WriteString('TXT', 'Chess white bishop', '"' + cTXTCWB + '"');
    IniFile.WriteString('TXT', 'Chess white rook',   '"' + cTXTCWR + '"');
    IniFile.WriteString('TXT', 'Chess white queen',  '"' + cTXTCWQ + '"');
    IniFile.WriteString('TXT', 'Chess white king',   '"' + cTXTCWK + '"');

    IniFile.WriteString('TXT', 'Chess black pawn',   '"' + cTXTCBP + '"');
    IniFile.WriteString('TXT', 'Chess black knight', '"' + cTXTCBN + '"');
    IniFile.WriteString('TXT', 'Chess black bishop', '"' + cTXTCBB + '"');
    IniFile.WriteString('TXT', 'Chess black rook',   '"' + cTXTCBR + '"');
    IniFile.WriteString('TXT', 'Chess black queen',  '"' + cTXTCBQ + '"');
    IniFile.WriteString('TXT', 'Chess black king',   '"' + cTXTCBK + '"');

    { TXT - TXT file }
    IniFile.WriteString('TXT', 'Destination TXT filename', cTXTFile);

    { System }
    IniFile.WriteBool('System', 'Save options on exit', bSaveOptionsOnExit);
    IniFile.WriteBool('System', 'Save layouts on exit', bSaveLayoutsOnExit);

    IniFile.WriteString('System', 'New directory',     cNewPath);
    IniFile.WriteString('System', 'Open directory',    cOpenPath);
    IniFile.WriteString('System', 'Delete directory',  cDeletePath);
    IniFile.WriteString('System', 'Save as directory', cSaveAsPath);
    iniFile.WriteString('System', 'Work Bpgn File', cworkbpgn);
    IniFile.WriteString('System', 'Export directory',  cExportPath);

    { View actual text }
    IniFile.WriteInteger('View actual text', 'Left',   nVATL);
    IniFile.WriteInteger('View actual text', 'Top',    nVATT);
    IniFile.WriteInteger('View actual text', 'Width',  nVATW);
    IniFile.WriteInteger('View actual text', 'Height', nVATH);

    { View extract.err }
    IniFile.WriteInteger('View extract.err', 'Left',   nVEXL);
    IniFile.WriteInteger('View extract.err', 'Top',    nVEXT);
    IniFile.WriteInteger('View extract.err', 'Width',  nVEXW);
    IniFile.WriteInteger('View extract.err', 'Height', nVEXH);

    { View RTF file }
    IniFile.WriteInteger('View RTF file', 'Left',   nRTFL);
    IniFile.WriteInteger('View RTF file', 'Top',    nRTFT);
    IniFile.WriteInteger('View RTF file', 'Width',  nRTFW);
    IniFile.WriteInteger('View RTF file', 'Height', nRTFH);
    IniFile.WriteString('View RTF file', 'RTF directory', cRTFDir);

    { View TXT file }
    IniFile.WriteInteger('View TXT file', 'Left',   nTXTL);
    IniFile.WriteInteger('View TXT file', 'Top',    nTXTT);
    IniFile.WriteInteger('View TXT file', 'Width',  nTXTW);
    IniFile.WriteInteger('View TXT file', 'Height', nTXTH);
    IniFile.WriteString('View TXT file', 'TXT directory', cTXTDir);

        { Player stats }
    IniFile.WriteInteger('Player stats', 'Left',   nstatL);
    IniFile.WriteInteger('Player stats', 'Top',    nstatT);
    IniFile.WriteInteger('Player stats', 'Width',  nstatW);
    IniFile.WriteInteger('Player stats', 'Height', nstatH);

    {filter window}
    IniFile.WriteInteger('Filter', 'Left',   nFilterL);
    IniFile.WriteInteger('Filter', 'Top',    nFilterT);
    IniFile.WriteInteger('Filter', 'Width',  nFilterW);
    IniFile.WriteInteger('Filter', 'Height', nFilterH);
    IniFile.WriteBool('Filter','Stats',bfilterSt);

    IniFile.WriteInteger('Tree', 'Left',   nTreeL);
    IniFile.WriteInteger('Tree', 'Top',    nTreeT);
    IniFile.WriteInteger('Tree', 'Width',  nTreeW);
    IniFile.WriteInteger('Tree', 'Height', nTreeH);
    IniFile.WriteBool('Tree','Filter',bTreeFil);
//    IniFile.WriteBool('Tree','Auto',bTreeAuto);

    { Move piece keymap }
    IniFile.WriteString('Move piece keymap', 'White pawn',   '"' + cWP + '"');
    IniFile.WriteString('Move piece keymap', 'White knight', '"' + cWN + '"');
    IniFile.WriteString('Move piece keymap', 'White bishop', '"' + cWB + '"');
    IniFile.WriteString('Move piece keymap', 'White rook',   '"' + cWR + '"');
    IniFile.WriteString('Move piece keymap', 'White queen',  '"' + cWQ + '"');
    IniFile.WriteString('Move piece keymap', 'White king',   '"' + cWK + '"');

    IniFile.WriteString('Move piece keymap', 'Black pawn',   '"' + cBP + '"');
    IniFile.WriteString('Move piece keymap', 'Black knight', '"' + cBN + '"');
    IniFile.WriteString('Move piece keymap', 'Black bishop', '"' + cBB + '"');
    IniFile.WriteString('Move piece keymap', 'Black rook',   '"' + cBR + '"');
    IniFile.WriteString('Move piece keymap', 'Black queen',  '"' + cBQ + '"');
    IniFile.WriteString('Move piece keymap', 'Black king',   '"' + cBK + '"');

    IniFile.WriteBool('Move piece keymap', 'Use zeros instead of Os in castling move', bZero);

    { Chess font keymap }
    IniFile.WriteString('Chess font keymap', 'White pawn on white square',   '"' + cWPWS + '"');
    IniFile.WriteString('Chess font keymap', 'White knight on white square', '"' + cWNWS + '"');
    IniFile.WriteString('Chess font keymap', 'White bishop on white square', '"' + cWBWS + '"');
    IniFile.WriteString('Chess font keymap', 'White rook on white square',   '"' + cWRWS + '"');
    IniFile.WriteString('Chess font keymap', 'White queen on white square',  '"' + cWQWS + '"');
    IniFile.WriteString('Chess font keymap', 'White king on white square',   '"' + cWKWS + '"');

    IniFile.WriteString('Chess font keymap', 'White pawn on black square',   '"' + cWPBS + '"');
    IniFile.WriteString('Chess font keymap', 'White knight on black square', '"' + cWNBS + '"');
    IniFile.WriteString('Chess font keymap', 'White bishop on black square', '"' + cWBBS + '"');
    IniFile.WriteString('Chess font keymap', 'White rook on black square',   '"' + cWRBS + '"');
    IniFile.WriteString('Chess font keymap', 'White queen on black square',  '"' + cWQBS + '"');
    IniFile.WriteString('Chess font keymap', 'White king on black square',   '"' + cWKBS + '"');

    IniFile.WriteString('Chess font keymap', 'Move indicator',   '"' + cMoveInd + '"');
    IniFile.WriteString('Chess font keymap', 'Black pawn on white square',   '"' + cBPWS + '"');
    IniFile.WriteString('Chess font keymap', 'Black knight on white square', '"' + cBNWS + '"');
    IniFile.WriteString('Chess font keymap', 'Black bishop on white square', '"' + cBBWS + '"');
    IniFile.WriteString('Chess font keymap', 'Black rook on white square',   '"' + cBRWS + '"');
    IniFile.WriteString('Chess font keymap', 'Black queen on white square',  '"' + cBQWS + '"');
    IniFile.WriteString('Chess font keymap', 'Black king on white square',   '"' + cBKWS + '"');

    IniFile.WriteString('Chess font keymap', 'Black pawn on black square',   '"' + cBPBS + '"');
    IniFile.WriteString('Chess font keymap', 'Black knight on black square', '"' + cBNBS + '"');
    IniFile.WriteString('Chess font keymap', 'Black bishop on black square', '"' + cBBBS + '"');
    IniFile.WriteString('Chess font keymap', 'Black rook on black square',   '"' + cBRBS + '"');
    IniFile.WriteString('Chess font keymap', 'Black queen on black square',  '"' + cBQBS + '"');
    IniFile.WriteString('Chess font keymap', 'Black king on black square',   '"' + cBKBS + '"');

    IniFile.WriteString('Chess font keymap', 'White square', '"' + cWS + '"');
    IniFile.WriteString('Chess font keymap', 'Black square', '"' + cBS + '"');

    IniFile.WriteString('Chess font keymap', 'Top left corner',  '"' + cTopLC + '"');
    IniFile.WriteString('Chess font keymap', 'Top border',       '"' + cTopB  + '"');
    IniFile.WriteString('Chess font keymap', 'Top right corner', '"' + cTopRC + '"');

    IniFile.WriteString('Chess font keymap', 'Left border',  '"' + cLeftB  + '"');
    IniFile.WriteString('Chess font keymap', 'Right border', '"' + cRightB + '"');

    IniFile.WriteString('Chess font keymap', 'Bottom left corner',  '"' + cBotLC + '"');
    IniFile.WriteString('Chess font keymap', 'Bottom border',       '"' + cBotB  + '"');
    IniFile.WriteString('Chess font keymap', 'Bottom right corner', '"' + cBotRC + '"');

    { Custom colors }
    IniFile.WriteString('Custom colors', 'A', cColorA);
    IniFile.WriteString('Custom colors', 'B', cColorB);
    IniFile.WriteString('Custom colors', 'C', cColorC);
    IniFile.WriteString('Custom colors', 'D', cColorD);
    IniFile.WriteString('Custom colors', 'E', cColorE);
    IniFile.WriteString('Custom colors', 'F', cColorF);
    IniFile.WriteString('Custom colors', 'G', cColorG);
    IniFile.WriteString('Custom colors', 'H', cColorH);
    IniFile.WriteString('Custom colors', 'I', cColorI);
    IniFile.WriteString('Custom colors', 'J', cColorJ);
    IniFile.WriteString('Custom colors', 'K', cColorK);
    IniFile.WriteString('Custom colors', 'L', cColorL);
    IniFile.WriteString('Custom colors', 'M', cColorM);
    IniFile.WriteString('Custom colors', 'N', cColorN);
    IniFile.WriteString('Custom colors', 'O', cColorO);
    IniFile.WriteString('Custom colors', 'P', cColorP);

    IniFile.WriteBool('Bpgn','Air drops',bairdropini);
    IniFile.WriteInteger('Database','Save depth',savedepth);
    Inifile.WriteBool('Database','RO games',bROgames);
    Inifile.WriteBool('Database','Open tree',bOpentree);
//    IniFile.WriteInteger('Bpgn','Short game max',nShortG);
    inifile.WriteBool('Filter','Full stat',bfullstat);
    inifile.WriteInteger('Filter','Move order',nmoveorder);

    IniFile.Free;
END;

PROCEDURE TConfig.WriteLayout;
VAR
    IniFile: TIniFile;
    i:integer;
    tmp:string;
BEGIN
try
    IniFile := TIniFile.Create(cFile);

    { Main }
    IniFile.WriteInteger('Main', 'Left',   nMainL);
    IniFile.WriteInteger('Main', 'Top',    nMainT);
    IniFile.WriteInteger('Main', 'Width',  nMainW);
    IniFile.WriteInteger('Main', 'Height', nMainH);
    IniFile.WriteInteger('Main', 'State',  nMainS);

    { Board #1 }
    IniFile.WriteInteger('Board #1', 'Left',   nBoard1L);
    IniFile.WriteInteger('Board #1', 'Top',    nBoard1T);
    IniFile.WriteInteger('Board #1', 'Width',  nBoard1W);
    IniFile.WriteInteger('Board #1', 'Height', nBoard1H);
    IniFile.WriteInteger('Board #1', 'State',  nBoard1S);

    { Board #2 }
    IniFile.WriteInteger('Board #2', 'Left',   nBoard2L);
    IniFile.WriteInteger('Board #2', 'Top',    nBoard2T);
    IniFile.WriteInteger('Board #2', 'Width',  nBoard2W);
    IniFile.WriteInteger('Board #2', 'Height', nBoard2H);
    IniFile.WriteInteger('Board #2', 'State',  nBoard2S);

    { Navigate }
    IniFile.WriteInteger('Navigate', 'Left',   nNavigateL);
    IniFile.WriteInteger('Navigate', 'Top',    nNavigateT);
    IniFile.WriteBool('Navigate', 'Sticky',  nNavigateSticky);
    IniFile.WriteInteger('Navigate', 'Width',   nNavigateW);
    IniFile.WriteInteger('Navigate', 'Height',    nNavigateH);
    IniFile.WriteInteger('Navigate', 'State',  nNavigateS);
    inifile.writebool('Navigate','Simple',bnavigatesimple);

    { Game list }
    IniFile.WriteInteger('Game list', 'Left',   nGameListL);
    IniFile.WriteInteger('Game list', 'Top',    nGameListT);
    IniFile.WriteInteger('Game list', 'Width',  nGameListW);
    IniFile.WriteInteger('Game list', 'Height', nGameListH);
    IniFile.WriteInteger('Game list', 'State',  nGameListS);

    { Next move }
    IniFile.WriteInteger('Next move', 'Left',   nNextMoveL);
    IniFile.WriteInteger('Next move', 'Top',    nNextMoveT);
    IniFile.WriteInteger('Next move', 'Width',  nNextMoveW);
    IniFile.WriteInteger('Next move', 'Height', nNextMoveH);
    IniFile.WriteInteger('Next move', 'State',  nNextMoveS);

    { Previous next move }
    IniFile.WriteInteger('Previous next move', 'Left',   nPrevMoveL);
    IniFile.WriteInteger('Previous next move', 'Top',    nPrevMoveT);
    IniFile.WriteInteger('Previous next move', 'Width',  nPrevMoveW);
    IniFile.WriteInteger('Previous next move', 'Height', nPrevMoveH);
    IniFile.WriteInteger('Previous next move', 'State',  nPrevMoveS);

    { Comment }
    IniFile.WriteInteger('Comment', 'Left',   nCommentL);
    IniFile.WriteInteger('Comment', 'Top',    nCommentT);
    IniFile.WriteInteger('Comment', 'Width',  nCommentW);
    IniFile.WriteInteger('Comment', 'Height', nCommentH);
    IniFile.WriteInteger('Comment', 'State',  nCommentS);

    { Engine }
    for i:=1 to MAX_ENGINE do
    begin
     str(i,tmp);
     tmp:='Engine'+tmp;
     IniFile.WriteInteger(tmp, 'Left',   enginelay[i].nEngineL);
     IniFile.WriteInteger(tmp, 'Top',    enginelay[i].nEngineT);
     IniFile.WriteInteger(tmp, 'Width',  enginelay[i].nEngineW);
     IniFile.WriteInteger(tmp, 'Height', enginelay[i].nEngineH);
     IniFile.WriteInteger(tmp, 'State',  enginelay[i].nEngineS);
    end;

    { View actual text }
    IniFile.WriteInteger('View actual text', 'Left',   nVATL);
    IniFile.WriteInteger('View actual text', 'Top',    nVATT);
    IniFile.WriteInteger('View actual text', 'Width',  nVATW);
    IniFile.WriteInteger('View actual text', 'Height', nVATH);

    { View extract.err }
    IniFile.WriteInteger('View extract.err', 'Left',   nVEXL);
    IniFile.WriteInteger('View extract.err', 'Top',    nVEXT);
    IniFile.WriteInteger('View extract.err', 'Width',  nVEXW);
    IniFile.WriteInteger('View extract.err', 'Height', nVEXH);

    { View RTF file }
    IniFile.WriteInteger('View RTF file', 'Left',   nRTFL);
    IniFile.WriteInteger('View RTF file', 'Top',    nRTFT);
    IniFile.WriteInteger('View RTF file', 'Width',  nRTFW);
    IniFile.WriteInteger('View RTF file', 'Height', nRTFH);
    IniFile.WriteString('View RTF file', 'RTF directory', cRTFDir);

    { View TXT file }
    IniFile.WriteInteger('View TXT file', 'Left',   nTXTL);
    IniFile.WriteInteger('View TXT file', 'Top',    nTXTT);
    IniFile.WriteInteger('View TXT file', 'Width',  nTXTW);
    IniFile.WriteInteger('View TXT file', 'Height', nTXTH);
    IniFile.WriteString('View TXT file', 'TXT directory', cTXTDir);

        { Player stats }
    IniFile.WriteInteger('Player stats', 'Left',   nstatL);
    IniFile.WriteInteger('Player stats', 'Top',    nstatT);
    IniFile.WriteInteger('Player stats', 'Width',  nstatW);
    IniFile.WriteInteger('Player stats', 'Height', nstatH);

    {filter window}
    IniFile.WriteInteger('Filter', 'Left',   nFilterL);
    IniFile.WriteInteger('Filter', 'Top',    nFilterT);
    IniFile.WriteInteger('Filter', 'Width',  nFilterW);
    IniFile.WriteInteger('Filter', 'Height', nFilterH);
    IniFile.WriteBool('Filter','Stats',bfilterSt);

    IniFile.WriteInteger('Tree', 'Left',   nTreeL);
    IniFile.WriteInteger('Tree', 'Top',    nTreeT);
    IniFile.WriteInteger('Tree', 'Width',  nTreeW);
    IniFile.WriteInteger('Tree', 'Height', nTreeH);
    IniFile.WriteBool('Tree','Filter',bTreeFil);
//    IniFile.WriteBool('Tree','Auto',bTreeAuto);


except
    IniFile.Free;
end;
END;

PROCEDURE TConfig.WriteOptions;
VAR
    IniFile: TIniFile;
    i:integer;
    tmp:string;
BEGIN
 try
    IniFile := TIniFile.Create(cFile);

    { Board }
    inifile.writestring('Board', 'graphics home', graphhome);
    IniFile.WriteInteger('Board', 'White square color',  nWhtSquare);
    IniFile.WriteInteger('Board', 'White square outline color', nWhtSOutline);
    IniFile.WriteInteger('Board', 'White piece color',   nWhtFill);
    IniFile.WriteInteger('Board', 'White piece outline color', nWhtOutline);
    IniFile.WriteInteger('Board', 'Black square color',  nBlkSquare);
    IniFile.WriteInteger('Board', 'Black square outline color', nBlkSOutline);
    IniFile.WriteInteger('Board', 'Black piece color',   nBlkFill);
    IniFile.WriteInteger('Board', 'Black piece outline color', nBlkOutline);
    IniFile.WriteBool('Board', 'Show piece outline when mouse clicked', bOutlinePC);
    IniFile.WriteInteger('Board', 'Piece outline color when mouse clicked', nClickColor);
    IniFile.WriteBool('Board', 'Show piece count', bShwPCCount);
    IniFile.WriteInteger('Board', 'Piece count color', nNumberColor);
    IniFile.WriteInteger('Board', 'Piece square color', nPcSquare);
    IniFile.WriteBool('Board', 'Flip board',   bFlipBd);
    IniFile.WriteBool('Board', 'use bmp', usebmp);
    IniFile.WriteBool('Board', 'use piece bmp', usepiecebmp);
    Inifile.WRitebool('Board', 'use board bmp', useboardbmp);
    Inifile.WriteString('Board', 'bmp file', bmpfile);
    Inifile.WriteString('Board', 'piece dir', p_dir);
    Inifile.WriteString('Board', 'board dir', b_dir);
    IniFile.WriteBool('Board', 'Always queen', bAlwaysQ);
    IniFile.WriteBool('Board', 'Drag move',    bDragMove);
    IniFile.WriteBool('Board', 'Smart move',   bSmartMove);
    IniFile.WriteBool('Board', 'Restrict board window to actual board size', bActualBoard);
    IniFile.WriteInteger('Board', 'Board outline color', nBoardOutline);
    IniFile.WriteInteger('Board', 'Board window background color', nBoardWindow);
    IniFile.WriteBool('Board', 'Show square outline', bShowSOutline);
    IniFile.WriteBool('Board', 'Show lag', bShowlag);
    IniFile.WriteBool('Board', 'Show tenth', bShowtenth);

    { Clock }
    IniFile.WriteInteger('Clock', 'Wait foreground color', nWaitF);
    IniFile.WriteInteger('Clock', 'Wait background color', nWaitB);
    IniFile.WriteInteger('Clock', 'Turn foreground color', nTurnF);
    IniFile.WriteInteger('Clock', 'Turn background color', nTurnB);
    IniFile.WriteString ('Clock', 'Font name', fClock.Name);
    IniFile.WriteInteger('Clock', 'Font height', fClock.Height);
    IniFile.WriteInteger('Clock', 'Font size', fClock.Size);

    CASE (fClock.Pitch) OF
        fpVariable: IniFile.WriteString('Clock', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Clock', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Clock', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Clock', 'Font style bold',      (fsBold      IN fClock.Style));
    IniFile.WriteBool('Clock', 'Font style italic',    (fsItalic    IN fClock.Style));
    IniFile.WriteBool('Clock', 'Font style underline', (fsUnderline IN fClock.Style));
    IniFile.WriteBool('Clock', 'Font style strikeout', (fsStrikeOut IN fClock.Style));

    { Handle }
    IniFile.WriteInteger('Handle', 'Background color', nHandleB);
    IniFile.WriteInteger('Handle', 'Foreground color', fHandle.Color);
    IniFile.WriteString ('Handle', 'Font name', fHandle.Name);
    IniFile.WriteInteger('Handle', 'Font height', fHandle.Height);
    IniFile.WriteInteger('Handle', 'Font size', fHandle.Size);

    CASE (fHandle.Pitch) OF
        fpVariable: IniFile.WriteString('Handle', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Handle', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Handle', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Handle', 'Font style bold',      (fsBold      IN fHandle.Style));
    IniFile.WriteBool('Handle', 'Font style italic',    (fsItalic    IN fHandle.Style));
    IniFile.WriteBool('Handle', 'Font style underline', (fsUnderline IN fHandle.Style));
    IniFile.WriteBool('Handle', 'Font style strikeout', (fsStrikeOut IN fHandle.Style));

    { Game list }
    IniFile.WriteInteger('Game list', 'Background color', nGameListB);
    IniFile.WriteInteger('Game list', 'Foreground color', fGameListF.Color);
    IniFile.WriteString ('Game list', 'Font name', fGameListF.Name);
    IniFile.WriteInteger('Game list', 'Font height', fGameListF.Height);
    IniFile.WriteInteger('Game list', 'Font size', fGameListF.Size);

    CASE (fGameListF.Pitch) OF
        fpVariable: IniFile.WriteString('Game list', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Game list', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Game list', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Game list', 'Font style bold',      (fsBold      IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Font style italic',    (fsItalic    IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Font style underline', (fsUnderline IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Font style strikeout', (fsStrikeOut IN fGameListF.Style));
    IniFile.WriteBool('Game list', 'Show game #',    bShowGameNum);
    IniFile.WriteBool('Game list', 'Show event',     bShowEvent);
    IniFile.WriteBool('Game list', 'Show ratings',   bShowRatings);
    IniFile.WriteBool('Game list', 'Show time',      bShowTime);
    IniFile.WriteBool('Game list', 'Show increment', bShowInc);
    IniFile.WriteBool('Game list', 'Show result',    bShowResult);

    { Next move }
    IniFile.WriteInteger('Next move', 'Background color', nNextMoveB);
    IniFile.WriteInteger('Next move', 'Foreground color', fNextMoveF.Color);
    IniFile.WriteString ('Next move', 'Font name', fNextMoveF.Name);
    IniFile.WriteInteger('Next move', 'Font height', fNextMoveF.Height);
    IniFile.WriteInteger('Next move', 'Font size', fNextMoveF.Size);

    CASE (fNextMoveF.Pitch) OF
        fpVariable: IniFile.WriteString('Next move', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Next move', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Next move', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Next move', 'Font style bold',      (fsBold      IN fNextMoveF.Style));
    IniFile.WriteBool('Next move', 'Font style italic',    (fsItalic    IN fNextMoveF.Style));
    IniFile.WriteBool('Next move', 'Font style underline', (fsUnderline IN fNextMoveF.Style));
    IniFile.WriteBool('Next move', 'Font style strikeout', (fsStrikeOut IN fNextMoveF.Style));

    { Previous next move }
    IniFile.WriteInteger('Previous next move', 'Background color', nPrevMoveB);
    IniFile.WriteInteger('Previous next move', 'Foreground color', fPrevMoveF.Color);
    IniFile.WriteString ('Previous next move', 'Font name', fPrevMoveF.Name);
    IniFile.WriteInteger('Previous next move', 'Font height', fPrevMoveF.Height);
    IniFile.WriteInteger('Previous next move', 'Font size', fPrevMoveF.Size);

    CASE (fPrevMoveF.Pitch) OF
        fpVariable: IniFile.WriteString('Previous next move', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Previous next move', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Previous next move', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Previous next move', 'Font style bold',      (fsBold      IN fPrevMoveF.Style));
    IniFile.WriteBool('Previous next move', 'Font style italic',    (fsItalic    IN fPrevMoveF.Style));
    IniFile.WriteBool('Previous next move', 'Font style underline', (fsUnderline IN fPrevMoveF.Style));
    IniFile.WriteBool('Previous next move', 'Font style strikeout', (fsStrikeOut IN fPrevMoveF.Style));

    { Comment }
    IniFile.WriteInteger('Comment', 'Background color', nCommentB);
    IniFile.WriteInteger('Comment', 'Foreground color', fCommentF.Color);
    IniFile.WriteString ('Comment', 'Font name', fCommentF.Name);
    IniFile.WriteInteger('Comment', 'Font height', fCommentF.Height);
    IniFile.WriteInteger('Comment', 'Font size', fCommentF.Size);

    CASE (fCommentF.Pitch) OF
        fpVariable: IniFile.WriteString('Comment', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Comment', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Comment', 'Font pitch', 'Default');
    END;
                                     //  www.randomly.com/icarus
    IniFile.WriteBool('Comment', 'Font style bold',      (fsBold      IN fCommentF.Style));
    IniFile.WriteBool('Comment', 'Font style italic',    (fsItalic    IN fCommentF.Style));
    IniFile.WriteBool('Comment', 'Font style underline', (fsUnderline IN fCommentF.Style));
    IniFile.WriteBool('Comment', 'Font style strikeout', (fsStrikeOut IN fCommentF.Style));

    IniFile.WriteBool('Comment', 'Word wrap', bCommentWR);

    inifile.writebool('Sound', 'on', bsoundon);
    inifile.writestring('Sound', 'move', csoundmove);
    inifile.writestring('Sound', 'check', csoundcheck);
    inifile.writestring('Sound', 'end', csoundend);
    inifile.writestring('Sound', 'capture', csoundcap);

     { Engine }
    for i:=1 to MAX_ENGINE do
    begin
     str(i,tmp);
     tmp:='Engine'+tmp;
     IniFile.WriteString(tmp, 'name', engineopt[i].name);
     IniFile.WriteString(tmp, 'command line', engineopt[i].comline);
     IniFile.WriteInteger(tmp, 'type', engineopt[i].etype);
     IniFile.WriteString(tmp, 'prompt', engineopt[i].prompt);
     IniFile.WriteString(tmp, 'reset', engineopt[i].reset);
     IniFile.WriteString(tmp, 'infinite time', engineopt[i].itime);
     IniFile.WriteBool(tmp, 'filter output', engineopt[i].bfilter);
     inifile.writebool(tmp,'analysis',engineopt[i].banal);
    end;

    IniFile.WriteInteger('Engine', 'Background color', nEngineB);
    IniFile.WriteInteger('Engine', 'Foreground color', fEngineF.Color);
    IniFile.WriteString ('Engine', 'Font name', fEngineF.Name);
    IniFile.WriteInteger('Engine', 'Font height', fEngineF.Height);
    IniFile.WriteInteger('Engine', 'Font size', fEngineF.Size);

    CASE (fEngineF.Pitch) OF
        fpVariable: IniFile.WriteString('Engine', 'Font pitch', 'Variable');
        fpFixed:    IniFile.WriteString('Engine', 'Font pitch', 'Fixed');
    ELSE
        IniFile.WriteString('Engine', 'Font pitch', 'Default');
    END;

    IniFile.WriteBool('Engine', 'Font style bold',      (fsBold      IN fEngineF.Style));
    IniFile.WriteBool('Engine', 'Font style italic',    (fsItalic    IN fEngineF.Style));
    IniFile.WriteBool('Engine', 'Font style underline', (fsUnderline IN fEngineF.Style));
    IniFile.WriteBool('Engine', 'Font style strikeout', (fsStrikeOut IN fEngineF.Style));

    IniFile.WriteBool('Engine', 'Word wrap', bEngineWR);
    inifile.Writefloat('Engine', 'test time', nengineTT);
    IniFile.WriteFloat('Engine', 'annotate time', nengineat);
    inifile.WriteFloat('Engine', 'include threshold', nengineit);
    inifile.WriteFloat('Engine', 'exclude threshold', nengineet);
    inifile.WriteBool('Engine', 'show mates', bengineSM);

    IniFile.WRiteBool('Engine', 'Tecumseh', Tecumseh);
    IniFile.WriteInteger('Engine','Default engine', defeng);

    { System }
    IniFile.WriteBool('System', 'Save options on exit', bSaveOptionsOnExit);
    IniFile.WriteBool('System', 'Save layouts on exit', bSaveLayoutsOnExit);

    { Custom colors }
    IniFile.WriteString('Custom colors', 'A', cColorA);
    IniFile.WriteString('Custom colors', 'B', cColorB);
    IniFile.WriteString('Custom colors', 'C', cColorC);
    IniFile.WriteString('Custom colors', 'D', cColorD);
    IniFile.WriteString('Custom colors', 'E', cColorE);
    IniFile.WriteString('Custom colors', 'F', cColorF);
    IniFile.WriteString('Custom colors', 'G', cColorG);
    IniFile.WriteString('Custom colors', 'H', cColorH);
    IniFile.WriteString('Custom colors', 'I', cColorI);
    IniFile.WriteString('Custom colors', 'J', cColorJ);
    IniFile.WriteString('Custom colors', 'K', cColorK);
    IniFile.WriteString('Custom colors', 'L', cColorL);
    IniFile.WriteString('Custom colors', 'M', cColorM);
    IniFile.WriteString('Custom colors', 'N', cColorN);
    IniFile.WriteString('Custom colors', 'O', cColorO);
    IniFile.WriteString('Custom colors', 'P', cColorP);

    IniFile.WriteBool('Bpgn','Air drops',bairdropini);
    IniFile.WriteInteger('DataBase','Save depth',savedepth);
    Inifile.WriteBool('Database','RO games',bROgames);
    Inifile.WriteBool('Database','Open tree',bOpentree);
//    IniFile.WriteInteger('Bpgn','Short game max',nShortG);    
    iniFile.WriteBool('Filter','Full stat',bfullstat);
    inifile.WriteInteger('Filter','Move order',nmoveorder);
except
    IniFile.Free;
end;
END;



END.
