(*============================================================================*)
(* MISC.PAS -- Miscellaneous functions                                        *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 2002.01-05: Added functions Parseint -- longstrtodate                      *)
(* 2002.06.22: Renamed function Readfile to Readfiletostr to avoid scope con- *)
(*             flicts in Delphi 6.0                                           *)
(* 2004.11.07: Changed Readfiletostr to speed it up (using Zon's method)      *)
(*             and to accomodate EOF behavior                                 *)
(* 2004.11.27: Added routines boardtobyte, boardnum, xytoint, getworb,        *)
(*             perfrating, boardcolortoside, fromtotoint, inttofromto         *)
(* 2004.12.08: Added wNoYesCancel                                             *)
(* 2004.12.17: Added iscurentbpgn                                             *)
// 2004.12.26: Added extractDelimitedString
// 2005.01.02: Added raiseForm
// 2005.01.10: Added ValidReason
// 2005.01.25: Added getLagDelay
// 2005.02.10: Added DispTimeToSec, BPGNtime, InternalTime
// 2005.02.28: Added WindowsLinebreaks that converts Linux->Windows linebreaks
// 2005.03.03: Added FileSaveAs
(*============================================================================*)
UNIT MISC;

INTERFACE

USES Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     ExtCtrls, StdCtrls,Math,
     GLOBAL,noyes,ToolBox;

PROCEDURE wOK         (cSt: String);
PROCEDURE wWarn       (cSt: String);
PROCEDURE wError      (cSt: String);
FUNCTION  wYesNo      (cSt: String): Boolean;
FUNCTION  wYesNoCancel(cSt: String): Integer;
function  wNoYesCancel(cst: string): integer;
FUNCTION  wOKCancel   (cSt: String): Boolean;
FUNCTION  wIgnoreCan  (cSt: String): Boolean;
FUNCTION  wRetryCan   (cSt: String): Boolean;
function ParseInt(s,d:string):integer;
function lastintpos(s,d:string):integer;
function uniquefilename(dir:string):string;
function standmove(move:string):string;
function isnumber(c:char):boolean;
procedure mergefiles(src,dest:string);
function readfiletostr(name:string):string;
function longstrtodate(s:String;var er:integer):Tdatetime;
function resultStrToByte(s:string):byte;
function resultByteToStr(r:byte):string;
function StrTimeC(time,inc:integer):string;
function isbpgnheader(fname:string):boolean;
function boardtobyte(cboard:char):byte;
function boardnum(cboard:char):byte;
function xytoint(x,y:integer):integer;
function getworb(c:char):integer;
function perfrating(pardRating,OppRating:longint;winP:real;NGames:integer):real;
function boardcolortoside(nboard,ncolor:integer):integer;
function fromtotoint(x,y:byte):integer;
procedure inttofromto(n:integer;var nFrom,nTo:integer);
function IsCurrentBPGN(fname,bpgnname:string):boolean;
function extractDelimitedStr(s,del1,del2:string):string;
function DispTimeTosec(dtime:string):string;
procedure RaiseForm(Fform:tform);
function GetLagDelay(Lag:integer):integer;
function ValidReason(s:string):string;
function Bpgntime(nInternaltime:integer;nFracL:integer =3):string;
function InternalTime(cBPGNtime:string):integer;
Procedure WindowsLineBreaks(var s:string);
function FileSaveAs(source,dest:string):boolean;

//function TeamRating(nOppRat:longint; winP: real; Ngames:longint):real;
IMPLEMENTATION

function longstrtodate(s:String;var er:integer):Tdatetime;
label error;
var yy,mm,dd:integer;
ps,err:integer;
s1:string;
dt:TDateTime;
begin
 er:=0;
 ps:=pos('.',s);
 if ps=0 then goto error;
 s1:=copy(s,1,ps-1);
 val(s1,yy,err);
 if ((err<>0)or (yy<1) or (yy>9999)) then goto error;
 delete(s,1,ps);

 ps:=pos('.',s);
 if ps=0 then goto error;
 s1:=copy(s,1,ps-1);
 val(s1,mm,err);
 if ((err<>0)or (mm<1) or (mm>12)) then goto error;
 delete(s,1,ps);

 val(s,dd,err);
 if ((err<>0)or (dd>31)) then goto error;

 try
  dt:=encodeDate(yy,mm,dd);
 except
  on EconvertError do begin er:=-1;dt:=defaultmindate;end;
 end;
 Result:=dt;
 exit;

error:
  er:=-1;
  Result:=defaultmindate;
end;


function readfiletostr(name:string):string;
var st,res:string;
inf:File of Filestring;
ef: boolean;
tfstr:FileString;
ni,dm:integer;
begin
 res:='';
 AssignFile(inf,name);
 dm:=ioresult;
 {$I-}
 reset(inf);
 if ioresult<>0 then begin wError('Can not open file '+name); Result:=res;exit;end;
 repeat
// while not Eof(inf) do
// begin
//    Readln(inf, st);
//    res:=res+st+CR+LF;
// end;
        ef:=EOF(inf);
        FOR nI := 0 TO (MAX_STRING - 1) DO
            tfstr[nI] := #0;
        try
         Read(inf, tfstr);
        EXCEPT
                ON E: EInOutError DO
                    BEGIN
                    END;
        end;
  //      FOR nI := 0 TO (Strlen(tfstr) - 1) DO
 //           res := res + tfstr[nI];
        res:=res+tfstr;
 until ef;
 dm:=ioresult;
 CloseFile(inf);

 Result:=res;
end;

procedure mergefiles(src,dest:string);
var tmpfname,res,er:string;
outfile:TextFile;
renfile:File;
sr:TSearchRec;
FileAttrs:integer;
begin
tmpfname:=uniquefilename(extractfilepath(dest));
FileAttrs := faAnyFile;
{$I-}
if FindFirst(src, FileAttrs, sr) = 0 then
begin
 try
  assignFile(outfile,tmpfname);
  rewrite(outfile);
  res:=readfiletostr(sr.Name);
  Write(outfile,res);
  deletefile(sr.name);
  while FindNext(sr) = 0 do
  begin
   res:=readfiletostr(sr.Name);
   Write(outfile,res);
   deletefile(sr.name);
  end;
 finally
  flush(outfile);
  closefile(outfile);
  FindClose(sr);
  str(Ioresult,er);
  if er<>'0' then wError('I/O Error '+er+' occured');
 end;
 AssignFile(renfile,tmpfname);
 try
  rename(renfile,dest);
 finally
 //nothing
  str(Ioresult,er);
  if er<>'0' then wError('I/O Error '+er+' occured');
 end;
end
Else wOk('No files found');
end;

function  isnumber(c:char):boolean;
begin
 if (pos(c,'1234567890')>0) then Result:=true else Result:=false;
end;

function standmove(move:string):string;
begin
 if length(move)<2 then begin Result:=move;exit;end;
 if move[2]='@' then Result:=lowercase(move)
 else result:=move[1]+lowercase(copy(move,2,length(move)));
end;

function uniquefilename(dir:string):string;
var i:integer;
fname,base,ext,s:string;
begin
 ext:='.tmp';
 base:='bpgn';
 i:=1;
 str(i,s);
 fname:=dir+base+s+ext;
 while fileexists(fname) do
 begin
  inc(i,1);
  str(i,s);
  fname:=dir+base+s+ext;
 end;
 Result:=fname;
end;

function parseint(s,d:string):integer;
var ps,i,code:integer;
s1:string;
begin
 ps:=pos(d,s);
 s1:=copy(s,ps+length(d),length(s));
 val(s1,i,code);
 if code<>0 then result:=-99
 else result:=i;
end;

function lastintpos(s,d:string):integer;
var ps,code:integer;
s1:string;
i:real;
begin
 ps:=lastdelimiter(d,s);
 s1:=copy(s,ps+length(d),length(s));
 val(s1,i,code);
 if code>0 then result:=-1 else result:=ps;
end;

PROCEDURE wOK(cSt: String);
BEGIN
 //   Beep;
    MessageDlg(cSt, mtInformation, [mbOK], 0);
END;

PROCEDURE wWarn(cSt: String);
BEGIN
//    Beep;
    MessageDlg(cSt, mtWarning, [mbOK], 0);
END;

PROCEDURE wError(cSt: String);
BEGIN
//    Beep;
    MessageDlg(cSt, mtError, [mbOK], 0);
END;

FUNCTION wYesNo(cSt: String): Boolean;
BEGIN
//    Beep;
    Result := (MessageDlg(cSt, mtConfirmation, [mbYes, mbNo], 0) = mrYes);
END;

FUNCTION wYesNoCancel(cSt: String): Integer;
VAR
    nWord: Word;
BEGIN
//    Beep;
    nWord := MessageDlg(cSt, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
    IF (nWord = mrYes) THEN
        Result := 1
    ELSE
    IF (nWord = mrNo) THEN
        Result := 2
    ELSE
        Result :=3;
END;

FUNCTION wNoYesCancel(cSt: String): Integer;
VAR
    nWord,res: Word;
    wform:TNoyesForm;
BEGIN
 //   Beep;
    res:=3;
    try
     wform:=TNoyesform.Create(nil);
     wform.Label1.Caption:=cst;
     wform.nchoice:=res;
     wform.ActiveControl:=wform.Button2;
     wform.preparepic;
     wform.ShowModal;
     res:=wform.nchoice;
    finally
     wform.Free;
    end;
    Result:=res;
END;

FUNCTION wOKCancel(cSt: String): Boolean;
BEGIN
//    Beep;
    Result := (MessageDlg(cSt, mtConfirmation, [mbOK, mbCancel], 0) = mrOK);
END;

FUNCTION wIgnoreCan(cSt: String): Boolean;
BEGIN
 //   Beep;
    Result := (MessageDlg(cSt, mtConfirmation, [mbIgnore, mbCancel], 0) = mrIgnore);
END;

FUNCTION wRetryCan(cSt: String): Boolean;
BEGIN
 //   Beep;
    Result := (MessageDlg(cSt, mtConfirmation, [mbRetry, mbCancel], 0) = mrRetry);
END;

function resultStrToByte(s:string):byte;
begin
 if s=BPGN_WHITE_WINS then
 begin
  result:=1;
  exit;
 end;
 if s=BPGN_BLACK_WINS then
 begin
  result:=2;
  exit;
 end;
 if s=BPGN_DRAW then
 begin
  result:=3;
  exit;
 end;
 result:=4
end;

function resultByteToStr(r:byte):string;
begin
 case r of
  1: result:=BPGN_WHITE_WINS;
  2: result:=BPGN_BLACK_WINS;
  3: result:=BPGN_DRAW;
 else
  result:=BPGN_UNKNOWN;
 end;
end;

function StrTimeC(time,inc:integer):string;
var
s,s1:string;
begin
 str(time,s);str(inc,s1);
 Result:=s+'+'+s1;
end;

function isbpgnheader(fname:string):boolean;
begin
 If (extractfileext(fname)=DB_HEADER_EXT) then result:=true
 else result:=false;
end;

function boardtobyte(cboard:char):byte;
begin
 case cboard of
  'A': Result:=1;
  'B': Result:=2;
  'a': Result:=3;
  'b': Result:=4;
 end;
end;

function boardnum(cboard:char):byte;
begin
 if ((cboard='A')or (cboard='a')) then result:=1
 else Result:=2;
end;

function xytoint(x,y:integer):integer;
begin
 result:=10*x+y;
end;

function getworb(c:char):integer;
begin
 if ((c='A') or (c='B')) then Result:=1 else result:=2;
end;

function perfrating(pardRating,OppRating:longint;winP:real;NGames:integer):real;
var k:integer;
delta:real;
begin
 k:=400;
 delta:=0.01;
 winp:=winp/100;
 if nGames>0 then
 begin
  if winp>1-delta then winp:=1-delta;
  if winp<delta then winp:=delta;

  Result:=(Opprating-pardrating)/nGames-k*log10(1/winp-1);
 // Result:=Opprating/(2*ngames)-k*log10(1/winp-1);
 end
 else Result:=-1;
end;

{function teamrating(Opprating:longint; winp:real; Ngames:longint):real
begin
 Result:=PerfRating(0,round(opprating/2),winp,nGames);
end;}

function boardcolortoside(nboard,ncolor:integer):integer;
begin
 Result:=(nboard-1)*2+ncolor;
end;

function fromtotoint(x,y:byte):integer;
begin
 Result:=100*x+y;
end;

procedure inttofromto(n:integer;var nFrom, nTo:integer);
begin
 nFrom:=n div 100;
 nTo:=n mod 100;
end;

function iscurrentbpgn(fname,bpgnname:string):boolean;
var cext:string;
bc:boolean;
begin
  fname:=Uppercase(fname);bpgnname:=uppercase(bpgnname);
  If changeFileExt(fname,'.'+BPGN_EXT)=bpgnname then
  begin
   cext:=ExtractFileExt(fname);
   if ((cext='.'+BPGN_EXT) or (lowercase(cext)=DB_HEADER_EXT)) then bc:=true else bc:=false;
  end
  else bc:=false;
  Result:=bc;
end;

function extractDelimitedStr(s,del1,del2:string):string;
var nbeg,nend:integer;
s1:string;
begin
 nbeg:=pos(del1,s);
 if nbeg>0 then delete(s,1,nbeg-1+length(del1));
 nend:=pos(del2,s);
 if nend>0 then Result:=copy(s,1,nend-1) else Result:=s;
end;

function disptimetosec(dtime:string):string;
var
ps,n1,n2,err:integer;
r2:real;
begin
 ps:=pos(':',dtime);
 if ps>0 then begin
  val(copy(dtime,1,ps-1),n1,err);
  if err>0 then begin result:='0';exit;end;
  val(copy(dtime,ps+1,length(dtime)),r2,err);
  if err>0 then begin result:='0';exit;end;
  n2:=Round(r2*1000);
  ps:=n1*60*1000+n2; //ms
  Result:=bpgntime(ps);
 end
 else Result:='0';
end;

procedure raiseform(fform:tform);
begin
  with fform do begin
  if windowstate=wsMinimized then windowstate:=wsnormal;
  BringToFront;
 end;
end;

function getLagDelay(Lag:integer):integer;
begin
 Result:=abs(lag) div 2;
// inc(nTotalLag,abs(Lag));
end;

function ValidReason(s:string):string;
var ps:integer;
s1:string;
begin
 result:='';
 s:=trim(s);
 if pos('{',s)<>1 then exit;
 ps:=LastDelimiter('}',s);
 if ps<1 then exit;
 s1:=copy(s,ps+1,length(s));
 s1:=trim(s1);
 if ((s1=BPGN_WHITE_WINS) OR (S1=BPGN_BLACK_WINS) OR (s1=BPGN_DRAW) OR (s1=BPGN_UNKNOWN)) then Result:=s;
end;

function Bpgntime(nInternaltime:integer;nFracL:integer=3):string;
var s:string;
nW,nF:integer;
begin
 nW:=nInternaltime div 1000;
 nF:=nInternaltime mod 1000;
 if nFracL<3 then nF:=nF div (10*(3-nFracL));
 Result:=Inttostr(nw);
 if nF>0 then
 begin
  str(nF:nFracl,s);
  TOOLBOX_CharTran(s,' ', '0');
  Result:=Result+'.'+s;
 end;
end;

function InternalTime(cBPGNtime:string):integer;
var r:real;
er:integer;
begin
 val(cBpgntime,r,er);
 if er>0 then Result:=0 else Result:=Round(1000*r);
end;

procedure WindowsLinebreaks(var s:string);
var i:longint;
begin
 for i:=length(s) downto 2 do
 if ((s[i]=LF_ASCII_CHAR) and (s[i-1]<>CR_ASCII_CHAR)) then insert(CR_ASCII_CHAR,s,i);
 if (s[1]=LF_ASCII_CHAR) then insert(CR_ASCII_CHAR,s,1);
end;

function FileSaveAs(source,dest:string):boolean;
var PCsource,PCdest:array[0..512] of char;
begin
  Result:=false;
  if not fileexists(source) then exit;
  strpcopy(pcSource,source);
  strpcopy(pcDest,dest);

  if FileExists(dest) then
  FileSetAttr(PCDest,0);

  if not copyfile(PCsource,PCDest,false) then exit;
  FileSetAttr(PCDest,0);
  Result:=true;
end;

END. { MISC }
