(*============================================================================*)
(* NAVIGATE.PAS -- Navigators window                                          *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: Disabled window key preview.                                   *)
(* 1998.11.05: 1) Converted flip, start, and stop buttons into speed buttons. *)
(*             2) Removed key preview.                                        *)
(* 2002.01   : Added Previous/next Annotation/comment buttons; open,          *)
(*             engine ON/OFF, plus, sound ON/OFF, Simple/Advanced controls    *)
(*             buttons                                                        *)
(* 2002.12.08: 1) Moved Play and open buttons to the position after last      *)
(*             2) Added configure button                                      *)
(*             3) Replaced all speedbuttons with lbbuttons                    *)
(*             4) changed adjustbuttons to leave 1pixel between buttons       *)
(*             5) Modified setnavigate width to account for 4)                *)
// 2005.02.10  Changed caption on the Display button to RTF, added glyph for it
(*============================================================================*)

UNIT NAVIGATE;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls, Buttons,
    GLOBAL, TOOLBOX,INI1, Menus, RepeatBtn, LbButton, ExtCtrls;


TYPE
    TNavigateForm = CLASS(TForm)
    SpeedButton2: TRepeatLBbutton;
    SpeedButton3: TRepeatLbButton;
    SpeedButton4: TLbButton;
    SpeedButton5: TLbButton;
    SpeedButton6: TLbButton;
    SpeedButton7: TLbButton;
    SpeedButton8: TLbButton;
    Button1: TLbButton;
    BitBtn1: TLbButton;
    PopupMenu1: TPopupMenu;
    Eng_1: TMenuItem;
    Eng_2: TMenuItem;
    Eng_3: TMenuItem;
    Eng_4: TMenuItem;
    Eng_5: TMenuItem;
    SpeedButton9: TLbButton;
    SpeedButton10: TLbButton;
    SpeedButton1: TLbButton;
    SpeedButton11: TLbButton;
    SpeedButton12: TLbButton;
    SpeedButton13: TLbButton;
    SpeedButton14: TLbButton;
    LbButton1: TLbButton;


        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE ResetAll;
        PROCEDURE Enable(nI: Integer);
        PROCEDURE Disable(nI: Integer);
        Procedure EngineBackup;
        Procedure EngineRestore;
    procedure BitBtn1Click(Sender: TObject);
    procedure SpeedButton14Click(Sender: TObject);
    procedure SpeedButton14MouseEnter(Sender: TObject);
    procedure SpeedButton14MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
      procedure makeitmovable;
      procedure makeitstick;
      procedure SetDisplay(bShow:boolean);


        PRIVATE
        bEngBackup:boolean;
        { Private declarations }

        PUBLIC
        { Public declarations }
        procedure setnavigatewidth;
        procedure adjustbuttons;
    END;

VAR
    NavigateForm: TNavigateForm;

IMPLEMENTATION
{$R *.DFM}

procedure TNavigateForm.setnavigatewidth;
var tw,th:integer;
i:integer;
begin

  adjustbuttons;
  if left+speedbutton14.Left+speedbutton14.Width div 2>screen.Width then
  makeitmovable;
  clientheight:=25;
  clientwidth:=speedbutton14.Left+speedbutton14.Width;

end;

procedure tnavigateform.Adjustbuttons;
begin
 if speedbutton13.Visible then
 begin
  speedbutton13.left:=speedbutton1.Left+speedbutton1.Width+1;
  speedbutton9.left:=speedbutton13.Left+speedbutton13.Width+1;
  speedbutton2.left:=speedbutton9.Left+speedbutton9.Width+1;
  speedbutton3.left:=speedbutton2.Left+speedbutton2.Width+1;
  speedbutton10.left:=speedbutton3.Left+speedbutton3.Width+1;
  speedbutton12.left:=speedbutton10.Left+speedbutton10.Width+1;
  speedbutton4.left:=speedbutton12.Left+speedbutton12.Width+1;
  speedbutton6.left:=speedbutton4.Left+speedbutton4.Width+1;
  speedbutton7.left:=speedbutton6.Left+speedbutton6.Width+1;
  speedbutton5.left:=speedbutton7.Left+speedbutton7.Width+1;
  speedbutton8.left:=speedbutton5.Left+speedbutton5.Width+1;
  button1.left:=speedbutton8.Left+speedbutton8.Width+1;
  bitbtn1.left:=button1.Left+button1.Width+1;
  speedbutton11.left:=bitbtn1.Left+bitbtn1.Width+1;
  lbbutton1.Left:=speedbutton11.Left+speedbutton11.Width+1;
  speedbutton14.left:=lbbutton1.Left+lbbutton1.Width+1;
 end
 else begin
  speedbutton2.left:=speedbutton1.Left+speedbutton9.Width+1;
  speedbutton3.left:=speedbutton2.Left+speedbutton2.Width+1;
  speedbutton4.left:=speedbutton3.Left+speedbutton12.Width+1;
  speedbutton6.left:=speedbutton4.Left+speedbutton4.Width+1;
  speedbutton7.left:=speedbutton6.Left+speedbutton6.Width+1;
  speedbutton5.left:=speedbutton7.Left+speedbutton7.Width+1;
  speedbutton11.left:=speedbutton5.left+speedbutton5.Width+1;
  speedbutton14.left:=speedbutton11.Left+speedbutton11.Width+1;
 end;
end;

procedure TNavigateForm.EngineBackup;
begin
 bEngBackup:=button1.Enabled;
 button1.Enabled:=false;
end;

procedure TNavigateForm.EngineRestore;
begin
 button1.Enabled:=bEngBackup;
end;

PROCEDURE TNavigateForm.FormCreate(Sender: TObject);
var i:integer;
BEGIN
    height:=speedbutton1.Height;
    for i:=0 to controlcount-1 do begin
     controls[i].controlstyle:=controls[i].controlstyle+[csOpaque];

    end;

    ResetAll;

    IF (INI1.INI.nNavigateS = 2) THEN  ini1.INI.nNavigateS:=0;

    if ini1.INI.nNavigateSticky then
    begin
        with ini1.INI do if (nnavigateL+(nnavigateW div 2))>screen.Width then
        nnavigateL:=screen.Width-(nnavigateW div 2);
        with ini1.ini do if (nnavigateT+(speedbutton1.Height div 2))>screen.Height
        then nnavigatet:=screen.Height-(speedbutton1.Height div 2);
        transparentcolor:=true;
        borderIcons:=[];
        borderStyle:=bsNone;
        formstyle:=fsStayOnTop;
    end;


        setBounds(INI1.INI.nNavigateL, INI1.INI.nNavigateT,
                  INI1.INI.nNavigateW, INI1.INI.nNavigateH);


    if not ini1.INI.nNavigateSticky then
    WindowState := TOOLBOX_IntToWinState(INI1.INI.nNavigateS)
    else windowState:=wsNormal;

    Caption     := 'Control';
    button1.Caption:=Sjeng_ON;
    speedbutton6.glyph:=glyphs[7]^.bitmap;
    speedbutton6.numglyphs:=2;
    speedbutton8.Glyph:=glyphs[9]^.bitmap;
END;

PROCEDURE TNavigateForm.ResetAll;
BEGIN
    Disable(1);
    Disable(2);
    Disable(3);
    Disable(4);
    Enable (5);
    Disable(6);
    enable(7);
    Disable(8);
    Button1.Enabled:=false;
    bitBtn1.Enabled:=false;
END;

PROCEDURE TNavigateForm.Enable(nI: Integer);
BEGIN
    CASE (nI) OF
        1: if not Speedbutton1.enabled then SpeedButton1.Enabled := TRUE;
        2: if not Speedbutton2.enabled then SpeedButton2.Enabled := TRUE;
        3: if not Speedbutton3.enabled then SpeedButton3.Enabled := TRUE;
        4: if not Speedbutton4.enabled then SpeedButton4.Enabled := TRUE;
        5: if not Speedbutton5.enabled then SpeedButton5.Enabled := TRUE;
        6: if not Speedbutton6.enabled then SpeedButton6.Enabled := TRUE;
        7: if not Speedbutton7.enabled then SpeedButton7.Enabled := TRUE;
        8: if not Speedbutton8.enabled then SpeedButton8.Enabled := TRUE;
    END;
END;

PROCEDURE TNavigateForm.Disable(nI: Integer);
BEGIN
    CASE (nI) OF
        1: if Speedbutton1.enabled then SpeedButton1.Enabled := FALSE;
        2: if Speedbutton2.enabled then SpeedButton2.Enabled := FALSE;
        3: if Speedbutton3.enabled then SpeedButton3.Enabled := FALSE;
        4: if Speedbutton4.enabled then SpeedButton4.Enabled := FALSE;
        5: if Speedbutton5.enabled then SpeedButton5.Enabled := FALSE;
        6: if Speedbutton6.enabled then SpeedButton6.Enabled := FALSE;
        7: if Speedbutton7.enabled then SpeedButton7.Enabled := FALSE;
        8: if Speedbutton8.enabled then SpeedButton8.Enabled := FALSE;
    END;
END;


procedure TNavigateForm.BitBtn1Click(Sender: TObject);
begin
 Popupmenu1.popup(left+bitbtn1.left,top-20);
end;

procedure TNavigateForm.SpeedButton14Click(Sender: TObject);
begin
 if Speedbutton13.Visible then
 begin
  speedbutton9.visible:=false;
  speedbutton10.visible:=false;
  speedbutton12.visible:=false;
  speedbutton13.visible:=false;
  speedbutton8.visible:=false;
  speedbutton9.visible:=false;
  button1.visible:=false;
  bitbtn1.visible:=false;
  lbbutton1.Visible:=false;
 end
 else begin
  speedbutton9.visible:=TRUE;
  speedbutton10.visible:=true;
  speedbutton12.visible:=true;
  speedbutton13.visible:=true;
  speedbutton8.visible:=true;
  speedbutton9.visible:=true;
  button1.visible:=true;
  bitbtn1.visible:=true;
  lbbutton1.Visible:=true;
 end;
 setnavigatewidth;
end;

procedure TNavigateForm.SpeedButton14MouseEnter(Sender: TObject);
begin
 if borderStyle=bsnone then
 begin
  makeitmovable;
 end
 else begin
  makeitstick;
 end;
 setnavigatewidth;
end;

procedure TNavigateForm.SpeedButton14MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if button=mbRight then SpeedButton14MouseEnter(Sender)
 else if button=mbLeft then SpeedButton14Click(Sender);
end;

procedure tnavigateform.makeitmovable;
var prevtop,deltax,deltay,prevleft:integer;
pnt:tpoint;
begin
  prevtop:=top;prevleft:=left;
  windowstate:=wsminimized;
  borderIcons:=[biSystemMenu,bIMinimize];
  borderStyle:=bsSingle;
  transparentColor:=false;
  formstyle:=fsMdiChild;
  left:=prevleft+screen.Width;
  top:=prevtop;
  windowstate:=wsNormal;
  pnt:=clientOrigin;
  deltaY:=pnt.Y-top;
  deltax:=pnt.X-left;
//  delta:=prevtop+Height-clientHeight-pnt.Y;
   top:=top-deltay;
   left:=prevleft-deltax;
end;

procedure tnavigateform.makeitstick;
var deltay,deltax,prevtop,prevleft:integer;
pnt:Tpoint;
begin
  pnt:=clientOrigin;
  deltay:=pnt.Y-top;
  deltax:=pnt.X-left;
  prevtop:=top;           prevleft:=left;
  windowstate:=wsMinimized;
  transparentcolor:=true;
  borderIcons:=[];
  borderStyle:=bsNone;
  formstyle:=fsStayOnTop;
  top:=prevtop+deltay;
  left:=prevleft+deltax;
  windowstate:=wsNormal;
end;

procedure tnavigateform.SetDisplay(bshow:boolean);
begin
    with speedbutton8 do
    IF (bShow) THEN
    BEGIN
        Tag     := 1;
        Glyph:=glyphs[10]^.bitmap;
        Hint:='Unmark current position for diagramming in the generated Rtf or Txt file (menu Utility)';
    END
    ELSE
    BEGIN
        Tag     := 0;
        Glyph:=glyphs[9]^.bitmap;
        Hint:='Mark current position for diagramming in the generated Rtf or Txt file (menu Utility)';
    END;
end;

END.
