(*============================================================================*)
(* PIECEMAP.PAS -- Edit move piece keymap window                              *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.23: First construction.                                            *)
(* 1998.11.25: Added use zeros instead of Os in castling move to edit move    *)
(*             piece keymap function, RTF function, and TXT function.         *)
(*                                                                            *)
// 2005.02.22: Modified to work both with RTF and TXT generating forms: added
//             goRTF and goTXT
(*============================================================================*)
UNIT PIECEMAP;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls, ComCtrls, ExtCtrls,
    GLOBAL,ini1;


TYPE
    TPieceMapForm = CLASS(TForm)
        Label1:  TLabel;
        Edit1:   TEdit;
        Label2:  TLabel;
        Edit2:   TEdit;
        Label3:  TLabel;
        Edit3:   TEdit;
        Label4:  TLabel;
        Edit4:   TEdit;
        Label5:  TLabel;
        Edit5:   TEdit;
        Label6:  TLabel;
        Edit6:   TEdit;
        Label7:  TLabel;
        Edit7:   TEdit;
        Label8:  TLabel;
        Edit8:   TEdit;
        Label9:  TLabel;
        Edit9:   TEdit;
        Label10: TLabel;
        Edit10:  TEdit;
        Label11: TLabel;
        Edit11:  TEdit;
        Label12: TLabel;
        Edit12:  TEdit;

        CheckBox1: TCheckBox;

        Button98:  TButton;
        Button99:  TButton;
        Button100: TButton;

        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE SetupValues;
        PROCEDURE Button98Click(Sender: TObject);
        PROCEDURE Button99Click(Sender: TObject);
        PROCEDURE Button100Click(Sender: TObject);
        procedure GoRTF;
        procedure GoTXT;

        PRIVATE
        { Private declarations }
        bRTF:boolean;
        PUBLIC
        { Public declarations }
    END;


IMPLEMENTATION

{$R *.DFM}

PROCEDURE TPieceMapForm.FormCreate(Sender: TObject);
BEGIN
    Left    := (Screen.Width  - Width ) SHR 1;
    Top     := (Screen.Height - Height) SHR 1;
END;

procedure TPieceMapForm.GoRTF;
begin
 bRTF:=true;
 Caption := PIECEMAP_TITLE;
end;

procedure TPieceMapForm.GoTXT;
begin
 bRTF:=false;
 Caption := TXTMAP_TITLE;
 checkBox1.Visible:=false;
end;

PROCEDURE TPieceMapForm.SetupValues;
BEGIN
    ActiveControl := Edit1;
if bRTF then
begin
    Edit1.Text  := INI1.INI.cWP;
    Edit2.Text  := INI1.INI.cWN;
    Edit3.Text  := INI1.INI.cWB;
    Edit4.Text  := INI1.INI.cWR;
    Edit5.Text  := INI1.INI.cWQ;
    Edit6.Text  := INI1.INI.cWK;

    Edit7.Text  := INI1.INI.cBP;
    Edit8.Text  := INI1.INI.cBN;
    Edit9.Text  := INI1.INI.cBB;
    Edit10.Text := INI1.INI.cBR;
    Edit11.Text := INI1.INI.cBQ;
    Edit12.Text := INI1.INI.cBK;

    CheckBox1.Checked := INI1.INI.bZero;
end
else begin
    Edit1.Text  := INI1.INI.cTXTWP;
    Edit2.Text  := INI1.INI.cTXTWN;
    Edit3.Text  := INI1.INI.cTXTWB;
    Edit4.Text  := INI1.INI.cTXTWR;
    Edit5.Text  := INI1.INI.cTXTWQ;
    Edit6.Text  := INI1.INI.cTXTWK;

    Edit7.Text  := INI1.INI.cTXTBP;
    Edit8.Text  := INI1.INI.cTXTBN;
    Edit9.Text  := INI1.INI.cTXTBB;
    Edit10.Text := INI1.INI.cTXTBR;
    Edit11.Text := INI1.INI.cTXTBQ;
    Edit12.Text := INI1.INI.cTXTBK;
end;
END;

PROCEDURE TPieceMapForm.Button98Click(Sender: TObject);
BEGIN
    IF (Length(Edit1.Text) > 2)  THEN Edit1.Text  := Copy(Edit1.Text,1,2);
    IF (Length(Edit2.Text) > 2)  THEN Edit2.Text  := Copy(Edit2.Text,1,2);
    IF (Length(Edit3.Text) > 2)  THEN Edit3.Text  := Copy(Edit3.Text,1,2);
    IF (Length(Edit4.Text) > 2)  THEN Edit4.Text  := Copy(Edit4.Text,1,2);
    IF (Length(Edit5.Text) > 2)  THEN Edit5.Text  := Copy(Edit5.Text,1,2);
    IF (Length(Edit6.Text) > 2)  THEN Edit6.Text  := Copy(Edit6.Text,1,2);

    IF (Length(Edit7.Text) > 2)  THEN Edit7.Text  := Copy(Edit7.Text,1,2);
    IF (Length(Edit8.Text) > 2)  THEN Edit8.Text  := Copy(Edit8.Text,1,2);
    IF (Length(Edit9.Text) > 2)  THEN Edit9.Text  := Copy(Edit9.Text,1,2);
    IF (Length(Edit10.Text) > 2) THEN Edit10.Text := Copy(Edit10.Text,1,2);
    IF (Length(Edit11.Text) > 2) THEN Edit11.Text := Copy(Edit11.Text,1,2);
    IF (Length(Edit12.Text) > 2) THEN Edit12.Text := Copy(Edit12.Text,1,2);
if bRTF then
begin
    INI1.INI.cWP := Edit1.Text;
    INI1.INI.cWN := Edit2.Text;
    INI1.INI.cWB := Edit3.Text;
    INI1.INI.cWR := Edit4.Text;
    INI1.INI.cWQ := Edit5.Text;
    INI1.INI.cWK := Edit6.Text;

    INI1.INI.cBP := Edit7.Text;
    INI1.INI.cBN := Edit8.Text;
    INI1.INI.cBB := Edit9.Text;
    INI1.INI.cBR := Edit10.Text;
    INI1.INI.cBQ := Edit11.Text;
    INI1.INI.cBK := Edit12.Text;

    INI1.INI.bZero := CheckBox1.Checked;
end
else begin
    INI1.INI.cTXTWP := Edit1.Text;
    INI1.INI.cTXTWN := Edit2.Text;
    INI1.INI.cTXTWB := Edit3.Text;
    INI1.INI.cTXTWR := Edit4.Text;
    INI1.INI.cTXTWQ := Edit5.Text;
    INI1.INI.cTXTWK := Edit6.Text;

    INI1.INI.cTXTBP := Edit7.Text;
    INI1.INI.cTXTBN := Edit8.Text;
    INI1.INI.cTXTBB := Edit9.Text;
    INI1.INI.cTXTBR := Edit10.Text;
    INI1.INI.cTXTBQ := Edit11.Text;
    INI1.INI.cTXTBK := Edit12.Text;
end;
    Close;
END;

PROCEDURE TPieceMapForm.Button99Click(Sender: TObject);
BEGIN
    Close;
END;

PROCEDURE TPieceMapForm.Button100Click(Sender: TObject);
BEGIN
    Edit1.Text  := 'P';
    Edit2.Text  := 'N';
    Edit3.Text  := 'B';
    Edit4.Text  := 'R';
    Edit5.Text  := 'Q';
    Edit6.Text  := 'K';

    Edit7.Text  := 'P';
    Edit8.Text  := 'N';
    Edit9.Text  := 'B';
    Edit10.Text := 'R';
    Edit11.Text := 'Q';
    Edit12.Text := 'K';
    if bRTF then
    CheckBox1.Checked := FALSE;
END;

END.
