(*============================================================================*)
(* TOOLBOX.PAS -- Some useful functions                                       *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.06: Added TOOLBOX_GetReason() and TOOLBOX_GetResult().             *)
// 2005.02.10: 1) Modified TOOLBOX_GetDispTimeStr to work with decimal time
//             2) Added GetLagStr
(*                                                                            *)
(*============================================================================*)
UNIT TOOLBOX;

INTERFACE

USES Forms, SysUtils, Windows,ini1;

FUNCTION TOOLBOX_WinStateToInt(AState: TWindowState): Word;
FUNCTION TOOLBOX_IntToWinState(nInt: Word): TWindowState;

PROCEDURE TOOLBOX_PadR(VAR cS: String; nN: Integer; cC: Char);
PROCEDURE TOOLBOX_PadL(VAR cS: String; nN: Integer; cC: Char);
PROCEDURE TOOLBOX_CharTran(VAR cS: String; cF, cT: Char);

PROCEDURE TOOLBOX_TxtWrite  (cFn, cSt: String);
PROCEDURE TOOLBOX_TxtWriteln(cFn, cSt: String);
PROCEDURE TOOLBOX_BinWrite  (cFn, cSt: String);
PROCEDURE TOOLBOX_BinRewrite(cFn, cSt: String);

FUNCTION TOOLBOX_GetDispTimeStr(nTime: longint): String;
FUNCTION TOOLBOX_GetTimeStr(cTimeC: String): String;
FUNCTION TOOLBOX_GetTimeVal(cTimeC: String): Integer;
FUNCTION TOOLBOX_GetTimeIncVal(cTimeC: String): Integer;
FUNCTION TOOLBOX_GetCommentTime(cT: String): LongInt;

FUNCTION TOOLBOX_StartTime: TDateTime;
FUNCTION TOOLBOX_EndTime: TDateTime;
FUNCTION TOOLBOX_ElapsedTime(tStart, tEnd: TDateTime): String;

FUNCTION TOOLBOX_GetReason(cR: String): String;
FUNCTION TOOLBOX_GetReasult(cR: String): String;
Function GetLagStr(nlag:integer):string;

IMPLEMENTATION

FUNCTION TOOLBOX_WinStateToInt(AState: TWindowState): Word;
BEGIN
    Result := Ord(AState);
END;

FUNCTION TOOLBOX_IntToWinState(nInt: Word): TWindowState;
BEGIN
    CASE (nInt) OF
        0: Result := wsNormal;
        1: Result := wsMinimized;
        2: Result := wsMaximized
    ELSE
        Result := wsNormal;
    END;
END;

PROCEDURE TOOLBOX_PadR(VAR cS: String; nN: Integer; cC: Char);
VAR
   nL, nI: Integer;
BEGIN
    cS := TrimRight(cS);
    nL := Length(cS);

    IF (nL > nN) THEN
    BEGIN
        cS := Copy(cS, 1, nN);
        EXIT;
    END;

    FOR nI := nL + 1 TO nN DO
        cS := cS + cC;
END;

PROCEDURE TOOLBOX_PadL(VAR cS: String; nN: Integer; cC: Char);
VAR
   nL, nI: Integer;
   cT: String;
BEGIN
    cS := TrimLeft(cS);
    nL := Length(cS);

    IF (nL > nN) THEN
    BEGIN
        cS := Copy(cS, 1, nN);
        EXIT;
    END;

    cT := '';
    FOR nI := nL + 1 TO nN DO
        cT := cT + cC;

    cS := cT + cS;
END;

PROCEDURE TOOLBOX_CharTran(VAR cS: String; cF, cT: Char);
VAR
    nI: Integer;
BEGIN
    FOR nI := 1 TO Length(cS) DO
        IF (cS[nI] = cF) THEN cS[nI] := cT;
END;

PROCEDURE TOOLBOX_TxtWrite(cFn, cSt: String);
VAR
    OutFile: TextFile;
BEGIN
    IF (FileExists(cFn)) THEN
    BEGIN
        AssignFile(OutFile, cFn);
        Append(OutFile);
    END
    ELSE
    BEGIN
        AssignFile(OutFile, cFn);
        Rewrite(OutFile);
    END;

    Write(OutFile, cSt);

    CloseFile(OutFile);
END;

PROCEDURE TOOLBOX_TxtWriteln(cFn, cSt: String);
VAR
    OutFile: TextFile;
BEGIN
    IF (FileExists(cFn)) THEN
    BEGIN
        AssignFile(OutFile, cFn);
        Append(OutFile);
    END
    ELSE
    BEGIN
        AssignFile(OutFile, cFn);
        Rewrite(OutFile);
    END;

    Writeln(OutFile, cSt);

    CloseFile(OutFile);
END;

PROCEDURE TOOLBOX_BinWrite(cFn, cSt: String);
VAR
    OutFile: File;
    nI:      Integer;
    nSize:   LongInt;
    Buffer:  Array[1..2048] OF Char;
BEGIN
    FOR nI := 1 TO Length(cSt) DO
        Buffer[nI] := cSt[nI];

    IF (FileExists(cFn)) THEN
    BEGIN
        AssignFile(OutFile, cFn);
        Reset(OutFile, 1);
        nSize := FileSize(OutFile);
        Seek(OutFile, nSize);
    END
    ELSE
    BEGIN
        AssignFile(OutFile, cFn);
        Rewrite(OutFile, 1);
    END;

    BlockWrite(OutFile, Buffer, Length(cSt));

    CloseFile(OutFile);
END;

PROCEDURE TOOLBOX_BinRewrite(cFn, cSt: String);
VAR
    OutFile: File;
    nI:      Integer;
    nSize:   LongInt;
    Buffer:  Array[1..2048] OF Char;
BEGIN
    FOR nI := 1 TO Length(cSt) DO
        Buffer[nI] := cSt[nI];

    AssignFile(OutFile, cFn);
    Rewrite(OutFile, 1);

    BlockWrite(OutFile, Buffer, Length(cSt));

    CloseFile(OutFile);
END;

FUNCTION TOOLBOX_GetDispTimeStr(nTime: longint): String;
VAR
    nHr, nMin,nps,nMs: longInt;
    cSign, cHr, cMin, cfrac: String;
BEGIN
//nTime is now the number of time units (currently millisecs, hence nps=1000)
    nps:=1000;
    IF (nTime >= 0) THEN
        cSign := ''
    ELSE
    BEGIN
        cSign := '-';
        nTime := 0 - nTime;
    END;

    nHr  := (nTime DIV (60*nps));
    nMin := ((nTime div nps) MOD 60);
    nMs:=nTime Mod nps;

    IF (nHr >= 60) THEN nHr := 59;

    Str(nHr: 2, cHr );
    Str(nMin:2, cMin);


    TOOLBOX_CharTran(cHr,  ' ', '0');
    TOOLBOX_CharTran(cMin, ' ', '0');

    Result := Trim(cSign) + cHr + ':' + cMin;
    if not Ini1.INI.bShowTenth then exit;
     nMs:=nMs div 100;
     str(nMs:1,cfrac);
     ToolBox_CharTran(cFrac, ' ', '0');
     Result:=Result+'.'+cFrac;
END;

FUNCTION TOOLBOX_GetTimeStr(cTimeC: String): String;
VAR
    nS, nI: longint;
    cTmp: String;
BEGIN
    nS := Length(cTimeC);
    IF (nS = 0) THEN
    BEGIN
        Result := '??:??';
        EXIT;
    END;

    cTmp := '';
    FOR nI := 1 TO nS DO
    BEGIN
        IF ((cTimeC[nI] >= '0') AND (cTimeC[nI] <= '9')) THEN
            cTmp := cTmp + cTimeC[nI]
        ELSE
            BREAK;
    END;

    VAL(cTmp, nI, nS);

    IF (nS <> 0) THEN
    BEGIN
        Result := '??:??';
        EXIT;
    END;

    Result := TOOLBOX_GetDispTimeStr(nI);
END;

FUNCTION TOOLBOX_GetTimeVal(cTimeC: String): Integer;
VAR
    nS, nI: Integer;
    cTmp: String;
BEGIN
    nS := Length(cTimeC);
    IF (nS = 0) THEN
    BEGIN
        Result := 0;
        EXIT;
    END;

    cTmp := '';
    FOR nI := 1 TO nS DO
    BEGIN
        IF ((cTimeC[nI] >= '0') AND (cTimeC[nI] <= '9')) THEN
            cTmp := cTmp + cTimeC[nI]
        ELSE
            BREAK;
    END;

    VAL(cTmp, nI, nS);

    IF (nS <> 0) THEN
    BEGIN
        Result := 0;
        EXIT;
    END;

    Result := nI;
END;

FUNCTION TOOLBOX_GetTimeIncVal(cTimeC: String): Integer;
VAR
    nS, nI, nJ: Integer;
    cTmp: String;
BEGIN
    nS := Length(cTimeC);
    IF (nS = 0) THEN
    BEGIN
        Result := 0;
        EXIT;
    END;

    cTmp := '';
    FOR nI := 1 TO nS DO
    BEGIN
        IF ((cTimeC[nI] < '0') OR (cTimeC[nI] > '9')) THEN
        BEGIN
            FOR nJ := (nI + 1) TO nS DO
            BEGIN
                IF ((cTimeC[nJ] >= '0') AND (cTimeC[nJ] <= '9')) THEN
                    cTmp := cTmp + cTimeC[nJ]
                ELSE
                    BREAK;
            END;

            BREAK;
        END;
    END;

    VAL(cTmp, nI, nS);

    IF (nS <> 0) THEN
    BEGIN
        Result := 0;
        EXIT;
    END;

    Result := nI;
END;

FUNCTION TOOLBOX_GetCommentTime(cT: String): LongInt;
VAR
    nS, nI: Integer;
    cTmp: String;
    bErr: Boolean;
BEGIN
    nS := Length(cT);
    IF (nS = 0) THEN
    BEGIN
        Result := -1;
        EXIT;
    END;

    cTmp := '';
    bErr := FALSE;
    FOR nI := 1 TO nS DO
    BEGIN
        IF ((cT[nI] < '0') OR (cT[nI] > '9')) THEN
        BEGIN
            bErr := TRUE;
            BREAK;
        END;

        cTmp := cTmp + cT[nI];
    END;
    IF (bErr) THEN
    BEGIN
        Result := -1;
        EXIT;
    END;

    VAL(cTmp, nI, nS);

    IF (nS <> 0) THEN
    BEGIN
        Result := -1;
        EXIT;
    END;

    Result := nI;
END;

FUNCTION TOOLBOX_StartTime: TDateTime;
BEGIN
    Result := Now;
END;

FUNCTION TOOLBOX_EndTime: TDateTime;
BEGIN
    Result := Now;
END;

FUNCTION TOOLBOX_ElapsedTime(tStart, tEnd: TDateTime): String;
BEGIN
    Result := FormatDateTime('hh:nn:ss', tEnd - tStart);
END;

FUNCTION TOOLBOX_GetReason(cR: String): String;
VAR
    nL, nI, nJ: Integer;
BEGIN
    nL := Length(cR);
    IF (nL = 0) THEN
    BEGIN
        Result := '';
        EXIT;
    END;
    nI := 0;
    FOR nJ := nL DOWNTO 1 DO
    BEGIN
        IF (nJ < nL) THEN
        BEGIN
            IF (cR[nJ] = ' ') THEN
            BEGIN
                nI := nJ;
                BREAK;
            END;
        END;
    END;
    IF (nI <= 1) THEN
    BEGIN
        Result := '';
        EXIT;
    END;
    Result := Copy(cR, 1, nJ - 1);
END;

FUNCTION TOOLBOX_GetReasult(cR: String): String;
VAR
    nL, nI, nJ: Integer;
BEGIN
    nL := Length(cR);
    IF (nL = 0) THEN
    BEGIN
        Result := '';
        EXIT;
    END;
    nI := 0;
    FOR nJ := nL DOWNTO 1 DO
    BEGIN
        IF (nJ < nL) THEN
        BEGIN
            IF (cR[nJ] = ' ') THEN
            BEGIN
                nI := nJ;
                BREAK;
            END;
        END;
    END;
    IF (nI = 0) THEN
    BEGIN
        Result := '';
        EXIT;
    END;
    Result := Copy(cR, nJ + 1, nL - nJ);
END;

function GetLagStr(nlag:integer):string;
var n:integer;
s:string;
begin
 if nlag<=0 then begin Result:=''; exit; end;
 n:=Round(nLag / 100);
 Result:=Inttostr(n div 10)+'.'+Inttostr(n mod 10);
// insert('.',s,length(S));
// if s[1]='.' then s:='0'+s;
// Result:=s;
end;

END. { TOOLBOX }
