(*============================================================================*)
(* TXT.PAS -- Four column display window                                      *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan                                           *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.18: First construction.                                            *)
(* 1998.11.23: Added include handle, move, annotation, time, comment,         *)
(*             display board, and result for convert to RTF/TXT function.     *)
(* 1998.11.24: 1) Added edit move piece map.                                  *)
(*             2) Added edit chess piece map.                                 *)
(* 1998.11.25: Added use zeros instead of Os in castling move to edit move    *)
(*             piece keymap function, RTF function, and TXT function.         *)
(*                                                                            *)
(*============================================================================*)
UNIT TXT;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls, ComCtrls, ExtCtrls,
    GLOBAL, MISC, TOOLBOX, PieceMap, TXTCMAP,ini1;


TYPE
    TTXTForm = CLASS(TForm)
        OpenDialog1: TOpenDialog;

        Panel1:  TPanel;

        CheckBox1: TCheckBox;
        CheckBox2: TCheckBox;
        CheckBox3: TCheckBox;
        CheckBox4: TCheckBox;
        CheckBox5: TCheckBox;
        CheckBox6: TCheckBox;
        CheckBox7: TCheckBox;
        CheckBox8: TCheckBox;

        Label1:  TLabel;
        Edit1:   TEdit;
        Button1: TButton;

        Button98:  TButton;
        Button99:  TButton;
        Button100: TButton;
        Button101: TButton;

        Memo1: TMemo;
        Memo2: TMemo;

        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE SetupValues;
        PROCEDURE Edit1Exit(Sender: TObject);
        PROCEDURE Button1Click(Sender: TObject);
        PROCEDURE Button98Click(Sender: TObject);
        PROCEDURE Button99Click(Sender: TObject);
        PROCEDURE Button100Click(Sender: TObject);
        PROCEDURE Button101Click(Sender: TObject);

        PROCEDURE ClearMemo;
        PROCEDURE AddMemo(cLine: String);
        PROCEDURE AddBlankLine;
        PROCEDURE ClearComment;
        FUNCTION  PiecePadR(cP: String): String;
        FUNCTION  GetPiece(cP: Char): String;
        PROCEDURE DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2: String);
        PROCEDURE SaveToFile;

        PRIVATE
        { Private declarations }

        PUBLIC
        { Public declarations }
        bOK:         Boolean;
        bHandle:     Boolean;
        bMove:       Boolean;
        bAnnotation: Boolean;
        bTime:       Boolean;
        bComment:    Boolean;
        bDisplay:    Boolean;
        bResult:     Boolean;
        bZero:       Boolean;
        cTXTFile:    String;
    END;


IMPLEMENTATION

{$R *.DFM}

PROCEDURE TTXTForm.FormCreate(Sender: TObject);
BEGIN
    Left     := (Screen.Width  - Width ) SHR 1;
    Top      := (Screen.Height - Height) SHR 1;

    bHandle     := TRUE;
    bMove       := TRUE;
    bAnnotation := TRUE;
    bTime       := TRUE;
    bComment    := TRUE;
    bDisplay    := TRUE;
    bResult     := TRUE;
    bZero       := FALSE;
    
    cTXTFile := ExtractFilePath(ParamStr(0)) + 'GAME.' + TXT_EXT;
END;

PROCEDURE TTXTForm.SetupValues;
BEGIN
    bOK           := FALSE;
    ActiveControl := CheckBox1;
    Edit1.Text    := cTXTFile;

    CheckBox1.Checked := bHandle;
    CheckBox2.Checked := bMove;
    CheckBox3.Checked := bAnnotation;
    CheckBox4.Checked := bTime;
    CheckBox5.Checked := bComment;
    CheckBox6.Checked := bDisplay;
    CheckBox7.Checked := bResult;
    CheckBox8.Checked := bZero;

    OpenDialog1.InitialDir := ExtractFilePath(cTXTFile);
END;

PROCEDURE TTXTForm.Edit1Exit(Sender: TObject);
BEGIN
    Edit1.Text := Trim(Edit1.Text);
    Edit1.Text := ChangeFileExt(Edit1.Text, '.' + TXT_EXT);
END;

PROCEDURE TTXTForm.Button1Click(Sender: TObject);
BEGIN
    IF (OpenDialog1.Execute) THEN
    BEGIN
        Edit1.Text := ChangeFileExt(OpenDialog1.Filename, '.' + TXT_EXT);
        OpenDialog1.InitialDir := ExtractFilePath(Edit1.Text);
    END;
END;

PROCEDURE TTXTForm.Button98Click(Sender: TObject);
BEGIN
    IF (Length(Edit1.Text) = 0) THEN
    BEGIN
        wError('Missing destination TXT filename.');
        ActiveControl := Edit1;
        EXIT;
    END;

    bOK := TRUE;

    bHandle     := CheckBox1.Checked;
    bMove       := CheckBox2.Checked;
    bAnnotation := CheckBox3.Checked;
    bTime       := CheckBox4.Checked;
    bComment    := CheckBox5.Checked;
    bDisplay    := CheckBox6.Checked;
    bResult     := CheckBox7.Checked;
    bZero       := CheckBox8.Checked;

    cTXTFile := Edit1.Text;
    Close;
END;

PROCEDURE TTXTForm.Button99Click(Sender: TObject);
BEGIN
    bOK := FALSE;
    Close;
END;

PROCEDURE TTXTForm.ClearMemo;
BEGIN
    Memo1.Clear;
END;

PROCEDURE TTXTForm.AddMemo(cLine: String);
BEGIN
    Memo1.Lines.Add(cLine);
END;

PROCEDURE TTXTForm.AddBlankLine;
BEGIN
    Memo1.Lines.Add('');
END;

PROCEDURE TTXTForm.ClearComment;
BEGIN
    Memo2.Clear;
END;

FUNCTION TTXTForm.PiecePadR(cP: String): String;
BEGIN
    cP := ' ' + cP;
    TOOLBOX_PadR(cP, 3, ' ');
    Result := cP;
END;

FUNCTION TTXTForm.GetPiece(cP: Char): String;
BEGIN
    CASE (cP) OF
        WHITE_PAWN:   Result := PiecePadR(INI1.INI.cTXTcWP);
        WHITE_ROOK:   Result := PiecePadR(INI1.INI.cTXTcWR);
        WHITE_KNIGHT: Result := PiecePadR(INI1.INI.cTXTcWN);
        WHITE_BISHOP: Result := PiecePadR(INI1.INI.cTXTcWB);
        WHITE_QUEEN:  Result := PiecePadR(INI1.INI.cTXTcWQ);
        WHITE_KING:   Result := PiecePadR(INI1.INI.cTXTcWK);

        BLACK_PAWN:   Result := PiecePadR(INI1.INI.cTXTcBP);
        BLACK_ROOK:   Result := PiecePadR(INI1.INI.cTXTcBR);
        BLACK_KNIGHT: Result := PiecePadR(INI1.INI.cTXTcBN);
        BLACK_BISHOP: Result := PiecePadR(INI1.INI.cTXTcBB);
        BLACK_QUEEN:  Result := PiecePadR(INI1.INI.cTXTcBQ);
        BLACK_KING:   Result := PiecePadR(INI1.INI.cTXTcBK);
    ELSE
        Result := '   ';
    END;
END;

PROCEDURE TTXTForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2: String);
VAR
    nL, nI, nJ, nY, nX: Integer;
    cL1, cL2, cL3, cL4: String;
    aLine1, aLine2: ARRAY[1..8] OF String;
BEGIN
    { Get board 1 black off board pieces (top) }
    nL  := Length(cBP1);
    cL1 := '';
    cL2 := '';
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 9 TO nL DO
        BEGIN
            IF (nI = 9) THEN
                cL1 := cL1 + GetPiece(cBP1[nI])
            ELSE
                cL1 := cL1 + ' ' + GetPiece(cBP1[nI]);
        END;

        FOR nI := 1 TO 8 DO
        BEGIN
            IF (nI = 1) THEN
                cL2 := cL2 + GetPiece(cBP1[nI])
            ELSE
                cL2 := cL2 + ' ' + GetPiece(cBP1[nI]);
        END;
    END
    ELSE
    BEGIN
        FOR nI := 1 TO nL DO
        BEGIN
            IF (nI = 1) THEN
                cL2 := cL2 + GetPiece(cBP1[nI])
            ELSE
                cL2 := cL2 + ' ' + GetPiece(cBP1[nI]);
        END;
    END;


    { Get board 2 white off board pieces (top) }
    nL  := Length(cWP2);
    cL3 := '';
    cL4 := '';
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 9 TO nL DO
        BEGIN
            IF (nI = 9) THEN
                cL3 := cL3 + GetPiece(cWP2[nI])
            ELSE
                cL3 := cL3 + ' ' + GetPiece(cWP2[nI]);
        END;

        FOR nI := 1 TO 8 DO
        BEGIN
            IF (nI = 1) THEN
                cL4 := cL4 + GetPiece(cWP2[nI])
            ELSE
                cL4 := cL4 + ' ' + GetPiece(cWP2[nI]);
        END;
    END
    ELSE
    BEGIN
        FOR nI := 1 TO nL DO
        BEGIN
            IF (nI = 1) THEN
                cL4 := cL4 + GetPiece(cWP2[nI])
            ELSE
                cL4 := cL4 + ' ' + GetPiece(cWP2[nI]);
        END;
    END;


    { Print top pieces }
    TOOLBOX_PadR(cL1, 33, ' ');
    TOOLBOX_PadR(cL2, 33, ' ');
    TOOLBOX_PadR(cL3, 33, ' ');
    TOOLBOX_PadR(cL4, 33, ' ');

    AddMemo(' ' + cL1 + '  ' + cL3);
    AddMemo(' ' + cL2 + '  ' + cL4);


    { Print border }
    AddMemo('---------------------------------  ---------------------------------');


    { Print board 1 and board 2 positions }
    nI := 0;
    nJ := 9;
    FOR nY := 1 TO 8 DO
    BEGIN
        Dec(nJ);
        aLine1[nJ] := '';
        aLine2[nY] := '';
        FOR nX := 1 TO 8 DO
        BEGIN
            Inc(nI);
            aLine1[nJ] := aLine1[nJ] + GetPiece(cB1[nI]) + '|';
            aLine2[nY] := GetPiece(cB2[nI]) + '|' + aLine2[nY];
        END;
    END;

    FOR nI := 1 TO 8 DO
    BEGIN
        AddMemo('|' + aLine1[nI] + '  ' + '|' + aLine2[nI]);
        IF (nI < 8) THEN
            AddMemo('|---+---+---+---+---+---+---+---|  |---+---+---+---+---+---+---+---|');
    END;


    { Print border }
    AddMemo('---------------------------------  ---------------------------------');

    
    { Get board 1 white off board pieces (bottom) }
    nL  := Length(cWP1);
    cL1 := '';
    cL2 := '';
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 9 TO nL DO
        BEGIN
            IF (nI = 9) THEN
                cL2 := cL2 + ' ' + GetPiece(cWP1[nI])
            ELSE
                cL2 := cL2 + GetPiece(cWP1[nI]);
        END;

        FOR nI := 1 TO 8 DO
        BEGIN
            IF (nI = 1) THEN
                cL1 := cL1 + GetPiece(cWP1[nI])
            ELSE
                cL1 := cL1 + ' ' + GetPiece(cWP1[nI]);
        END;
    END
    ELSE
    BEGIN
        FOR nI := 1 TO nL DO
        BEGIN
            IF (nI = 1) THEN
                cL1 := cL1 + GetPiece(cWP1[nI])
            ELSE
                cL1 := cL1 + ' ' + GetPiece(cWP1[nI]);
        END;
    END;


    { Get board 2 black off board pieces (bottom) }
    nL  := Length(cBP2);
    cL3 := '';
    cL4 := '';
    IF (nL > 8) THEN
    BEGIN
        FOR nI := 9 TO nL DO
        BEGIN
            IF (nI = 9) THEN
                cL4 := cL4 + GetPiece(cBP2[nI])
            ELSE
                cL4 := cL4 + ' ' + GetPiece(cBP2[nI]);
        END;

        FOR nI := 1 TO 8 DO
        BEGIN
            IF (nI = 1) THEN
                cL3 := cL3 + GetPiece(cBP2[nI])
            ELSE
                cL3 := cL3 + ' ' + GetPiece(cBP2[nI]);
        END;
    END
    ELSE
    BEGIN
        FOR nI := 1 TO nL DO
        BEGIN
            IF (nI = 1) THEN
                cL3 := cL3 + GetPiece(cBP2[nI])
            ELSE
                cL3 := cL3 + ' ' + GetPiece(cBP2[nI]);
        END;
    END;


    { Print bottom pieces }
    TOOLBOX_PadR(cL1, 33, ' ');
    TOOLBOX_PadR(cL2, 33, ' ');
    TOOLBOX_PadR(cL3, 33, ' ');
    TOOLBOX_PadR(cL4, 33, ' ');
    
    AddMemo(' ' + cL1 + '  ' + cL3);
    AddMemo(' ' + cL2 + '  ' + cL4);
END;

PROCEDURE TTXTForm.SaveToFile;
BEGIN
    Memo1.Lines.SaveToFile(INI1.INI.cTXTFile);
END;

PROCEDURE TTXTForm.Button100Click(Sender: TObject);
var TXTMapForm:TPieceMapForm;
BEGIN
try
    TXTMapform:=TPiecemapform.Create(nil);
    TXTMapForm.GoTXT;
    TXTMapForm.SetupValues;
    TXTMapForm.ShowModal;
finally
    TXTMapform.Free;
end; 
END;

PROCEDURE TTXTForm.Button101Click(Sender: TObject);
var TxtCMapForm:TTxtcmapform;
BEGIN
try
    TxtCmapForm:=tTxtcMapForm.Create(nil);
    TXTCMapForm.SetupValues;
    TXTCMapForm.ShowModal;
finally
    txtcmapform.Free;
end;
END;

END.
