(*============================================================================*)
(* ZhEngine.PAS -- Winboard Engine object                                     *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2001.12      First Construction                                            *)
(*                                                                            *)
(*============================================================================*)


unit ZhEngine;

interface
uses Global,Wintypes,WinProcs,Sysutils,stdctrls,messages,misc;
type
 ZEngine = class(TObject)
 EngWind: HWND;
 comline: string;
 command_quit:string;
 engname:string;
 hProcess:Thandle;
 hTo:Thandle;
 hFrom:Thandle;
 buf:Tbuffer ;
 last:string;
 next:string;
 bLoadOK:boolean ;
 hThread:Thandle;
 id:DWORD;
 count:DWORD;
 engerror:integer;
 prompt:string;
 command_reset:string;
 banal:boolean;
 itime:string;
 bfilter:boolean;


 function loadengine(comline:string):boolean;
 procedure WriteEng(s:string);
 procedure Readbuf;
 procedure ReadPipe;
 procedure resetKill;
 procedure reset;

 private
    { Private declarations }
 public
  mem1:^Tmemo;

    { Public declarations }
 end;

implementation
{$I-}

procedure Zengine.reset;
Begin
 resetKill ;
end;

function Zengine.loadengine(comline:string):boolean;
var
zAppName:array[0..512] of char;
lpcurrentdirectory:Pchar;
saAttr:TSecurityAttributes;
StartInfo:TStartupInfo;
piProcInfo:TProcessInformation;
hChildStdinRd,hChildStdinWr,hChildStdoutRd:Thandle;
hChildStdoutWr,hChildStdinWrDup, hChildStdoutRdDup:Thandle;
fSuccess:boolean;
tmp:string;

begin
StrPCopy(zAppName,comline);
tmp:=extractFilepath(comline);
lpcurrentdirectory:=Pchar(tmp);
bLoadOk:=false;
//set the bInheritHandle flag so pipe handles are inherited

    saAttr.nLength              := sizeof (SECURITY_ATTRIBUTES) ;
    saAttr.bInheritHandle       := TRUE ;
    saAttr.lpSecurityDescriptor := Nil ;

    //
    // the steps for redirecting child's STDOUT:
    //     1. Create anonymous pipe to be STDOUT for child.
    //     2. Create a noninheritable duplicate of read handle,
    //         and close the inheritable read handle.
    //

    // create a pipe for the child's STDOUT

if (not CreatePipe (hChildStdoutRd,hChildStdoutWr,@saAttr,0)) then
begin
 tmp:=engname+': CreatePipe () failed.';
 wError(tmp);
 Result:=false; exit;
end;

    // duplicate the read handle to the pipe, so it is not inherited
fSuccess:= DuplicateHandle (GetCurrentProcess(),
                                hChildStdoutRd,
                                GetCurrentProcess(),
                                @hChildStdoutRdDup,
                                0,
                                FALSE,                      // not inherited
                                DUPLICATE_SAME_ACCESS) ;
if (not fSuccess) then
begin
 tmp:=engname+': DuplicateHandle () failed.';
 wError(tmp);
 closeHandle(hChildStdoutRd);
 CloseHandle(hChildStdoutWr);
 Result:=false; exit;
end;
CloseHandle (hChildStdoutRd) ;


    //
    // the steps for redirecting child's STDIN:
    //     1. Create anonymous pipe to be STDIN for child.
    //     2. Create a noninheritable duplicate of write handle,
    //         and close the inheritable write handle.
    //

    // create a pipe for the child's STDIN

if (not CreatePipe (hChildStdinRd, hChildStdinWr, @saAttr, 0)) then
begin
 tmp:=engname+': CreatePipe () failed.';
 wError(tmp);
 CloseHandle(hChildStdoutRdDup);
 closeHandle(hChildStdoutWr);
 Result:=false; exit;
end;

    // duplicate the write handle to the pipe, so it is not inherited

fSuccess:=DuplicateHandle (GetCurrentProcess(),
                                hChildStdinWr,
                                GetCurrentProcess(),
                                @hChildStdinWrDup,
                                0,
                                FALSE,                      // not inherited
                                DUPLICATE_SAME_ACCESS) ;
if (not fSuccess) then
begin
 tmp:=engname+': DuplicateHandle () failed.';
 wError(tmp);
 CloseHandle(hChildStdoutRdDup);
 closeHandle(hChildStdinWr);
 closeHandle(hChildStdinRd);
 closeHandle(hChildStdOutWr);
 Result:=false; exit;
end;

CloseHandle (hChildStdinWr) ;

    // now create the child process

    StartInfo.cb          := sizeof (STARTUPINFO) ;
    StartInfo.lpReserved  := nil ;
    StartInfo.lpDesktop   := nil ;
    StartInfo.lpTitle     := nil;
    StartInfo.dwFlags     := STARTF_USESTDHANDLES;
    StartInfo.cbReserved2 := 0 ;
    StartInfo.lpReserved2 := nil ;
    StartInfo.hStdInput   := hChildStdinRd ;
    StartInfo.hStdOutput  := hChildStdoutWr ;
    StartInfo.hStdError   := hChildStdoutWr ;

fSuccess :=CreateProcess (nil,
                              zAppName,  // command line
                              Nil,             // process security attributes
                              Nil,             // primary thread security attrs
                              true,             // handles are inherited
                              DETACHED_PROCESS+CReate_NEW_PROCESS_GROUP,
                              nil,             // use parent's environment
                              lpcurrentdirectory,  //set engine's work directory
                              StartInfo,     // STARTUPINFO pointer
                              piProcInfo ) ;   // receives PROCESS_INFORMATION

    // close the handles we don't need in the parent     getStdHandle

CloseHandle (hChildStdinRd) ;
CloseHandle (hChildStdoutWr) ;

if not fSuccess then
begin
 tmp:=engname+': CreateProcess () failed. Command line:'+comline;
 wError(tmp);
 CloseHandle(hChildStdoutRdDup);
 CloseHandle(hChildStdinWrDup);
 result:=false;
 exit;
end;

id      :=piprocinfo.dwProcessId;
hProcess := piProcInfo.hProcess ;
hFrom    := hChildStdoutRdDup ;
hTo      := hChildStdinWrDup ;

buf:=#0;
last:='';
next:='';
bLoadOK:= true ;
Result:=true;
end;

procedure Zengine.WriteEng(s:string);
var tmp:Pchar;
s1:TBuffer;
nw:LongWord;
begin
 strPCopy(s1,s);
 if (not WriteFile(hTo,s1,strlen(s1),nW,nil)) then    // send it to stdin
 begin
  engError:=GetLastError;
  SendMessage(EngWind,WM_EngineDead,0,0);
  tmp:=pchar(engname+' -> WriteFile failed'+CR_ASCII_CHAR  = #13);
  wError(tmp);
 end;
 if not bfilter then mem1^.Lines.add('Engine <===='+s1);
end;

procedure Zengine.ReadBuf;
begin
buf[count]:=#0;
last:=last+buf;
end;

procedure Zengine.Readpipe;
begin
  if ReadFile(hFrom,buf,BufferSize,count,nil)
  then EngError:=NO_ERROR
  else  EngError:=getlastError();
end;

procedure Zengine.resetKill;
var fsuccess:boolean;
status:DWORD;
begin
 if (bLoadOK) then
 begin
  bLoadOk:=false;
  generateConsoleCTrlEvent(CTRL_BREAK_EVENT,id);
  fsuccess:=getExitCodeThread(hprocess,status);
  if ((not fsuccess) or (status=STILL_ACTIVE)) then
  TerminateProcess(hprocess,22);
  fsuccess:=CloseHandle (hProcess);
  if not fsuccess then EngError:=GetlastError;
 end;
 buf:=#0;
 last:='';
 next:=buf;
end;

end.
