(*============================================================================*)
(* bpgnlist.PAS -- Headers datbase                                            *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, , 2002-2005 Fermy                        *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2004.11.08: First construction.                                            *)
(* 2004.12.13: Added apositionw, apositionb to the tbpgnlist                  *)
(* 2004.12.17: Added ReaddbSyncInfo, WriteDBSyncInfo
// 2005.01.16: Changed aWhiteA,etc size from 16 to FICS_NAME_LEN
// 2005.02.22: Added DBStampOk
(*----------------------------------------------------------------------------*)
unit bpgnlist;

interface

  uses sysutils,classes,global   ;

  type
   tbpgnrec=record
    aTag:  Char;
    aEvent: String;
    aSite:  String;
    aDate:  String;
    aPosition:  LongInt;
    aWhiteA:    String[FICS_NAME_LEN];
    aBlackA:    String[FICS_NAME_LEN];
    aWhiteB:    String[FICS_NAME_LEN];
    aBlackB:    String[FICS_NAME_LEN];
    aRWA:       String[4];
    aRBA:       String[4];
    aRWB:       String[4];
    aRBB:       String[4];
    aTimeC:     String[10];
    aResult:    String[10];
    aOpenA:     String;
    aOpenB:     String;
    aSetup:     String;
    aFen:       String;

    aFilterN:   longint;
    aReason:    string;
    aPositionW: longint;
    aPositionB: longint;
   end;
   Pbpgnrec=^tbpgnrec;

   tbpgnlist=class(Tlist)
    fBkuprec:tbpgnrec;
    function add(p:pbpgnrec):integer;
    procedure delete(n:integer);
    procedure clear;
    procedure backup(n:integer);
    procedure restore(n:integer);
   end;

   theadersave=record
    GameStart:longint;
    GameEnd: longint;
    aTag:  Char;
    EventSh: integer;
    EventLn: integer;
    SiteSh: integer;
    Siteln: integer;
    DateSh: integer;
    DateLn:  integer;
    WhiteASh:    integer;
    WhiteALn:    integer;
    WhiteBSh:    integer;
    WhitebLn:    integer;
    BlackBSh:    integer;
    BlackBLn:  integer;
    BlackASh: integer;
    BlackALn: integer;
    RWA:       String[4];
    RBA:       String[4];
    RWB:       String[4];
    RBB:       String[4];
    time:      integer;
    inc:       integer;
    Result:    byte;
    OpenASh:   integer  ;
    OpenALn:   integer;
    OpenBSh:   integer ;
    OpenBLn:   integer;
    SetupSh:   integer;
    SetupLn:   integer;
    FenSh:     integer;
    FenLn:     integer;
    ReasonSh:  integer;
    ReasonLn:  integer;
    positionw: longint;
    positionb: longint;
   end;
   pbpgnlist=^tbpgnlist;
   THeadlist=array[0..SAVE_HEADER_BUF] of theadersave;
   Pheadlist=^THeadlist;
   pheadersave=^theadersave;
   function  ReadDbSyncInfo(fname:string;var nFileSize,nHeaderCount:longint):integer;
   function  writeDbSyncInfo(var F:file;fname:string; nFileSize:longint;ctype:boolean;nHeaderCount:longint):boolean;
   function  DBStampOk(d1,d2:TDateTime):boolean;
   procedure BPGNrecFillempty(p:pbpgnRec);
implementation

function tbpgnlist.add(p:pbpgnrec):integer;
begin
 try
  New(p);
  inherited add(p);
 except on EOutOfMemory do
  result:=-1;
 end;
end;


procedure tbpgnlist.delete(n:integer);
var p:pbpgnrec;
begin
 p:=items[n];
 try
  dispose(p);
 except on EInvalidPointer do begin end;
 end;
 inherited delete(n);
end;

procedure tbpgnlist.clear;
var b:integer;
begin
 while (count>0) do delete(count-1);
end;

procedure BPGNrecFillempty(p:pbpgnRec);
begin
    p^.aTag := TAG_OK;
    p^.aEvent:= DEFAULT_EVENT;
    p^.aSite := DEFAULT_SITE;
    p^.aDate  := FormatDateTime('yyyy.mm.dd', Date);
    p^.aPosition := 0;
    p^.aWhiteA := DEFAULT_WHITEA;
    p^.aBlackA := DEFAULT_BLACKA;
    p^.aWhiteB := DEFAULT_WHITEB;
    p^.aBlackB := DEFAULT_BLACKB;
    P^.aTimeC  := DEFAULT_TIME + '+' + DEFAULT_INC;
    p^.aResult := DEFAULT_RESULT;
    p^.aReason := Default_REASON+' '+DEFAULT_RESULT;
    p^.aOpenA  := '';
    p^.aOpenB  := '';
    p^.aRWA    := DEFAULT_RATING;
    p^.aRBA    := DEFAULT_RATING;
    p^.aRWB    := DEFAULT_RATING;
    p^.aRBB    := DEFAULT_RATING;
    p^.aSetup  := '';
    p^.aFEN    := '';
    p^.aPositionW:=0;
    p^.aPositionB:=0;
end;

function  ReadDBSyncInfo(fname:string;var nFileSize,nHeaderCount:longint):integer;
var f:file;
bbugdb:boolean;
res:integer;
begin
 res:=0;
 if not FileExists(fname) then begin Result:=2; exit;end;
 AssignFile(f,fname);
 try
  Reset(f,1); seek(f,0);
  blockRead(f,nHeaderCount,sizeof(nheadercount));
  blockread(f,bbugdb,sizeof(bbugdb));
  blockread(f,nFileSize,SizeOf(nfilesize));
  if bbugdb then res:=1 else res:=-1;
 finally
  closeFile(f);
 end;
 Result:=res;
end;

function WriteDBSyncInfo(var f:File;fname:string;nFileSize:longint;ctype:boolean;nHeadercount:longint):boolean;
var res:integer;
bpgndate:TdateTime;
begin
 try
  seek(f,0);
  BlockWrite(f,nHeaderCount,sizeof(nHeaderCount),res);
  blockWrite(f,ctype,sizeof(ctype),res);
  //add some data for  sync control between bpgn and headers;
  blockwrite(f,nFileSize,sizeof(nfilesize),res);
  bpgndate:=FileDateToDateTime(FileAge(fname));
  blockwrite(f,bpgndate,sizeof(bpgndate));
  Result:=true;
 except
  Result:=false;
 end;
end;

procedure tbpgnlist.backup(n:integer);
begin
 fbkuprec:=pbpgnrec(items[n])^;
end;

procedure tbpgnlist.restore(n:integer);
begin
 pbpgnrec(items[n])^:=fbkuprec;
end;

function DBStampOk(d1,d2:TDateTime):boolean;
var n1,n2,n:longint;
begin
 n1:=Round(d1*DATE_TIME_TO_MS);
 n2:=Round(d2*DATE_TIME_TO_MS);
 n:=abs(n1-n2);
 n1:=n MOD (1000*60*60);
 if n1=0 then Result:=true else Result:=False;
end;

end.
