(*============================================================================*)
(* CONST.INC -- Constants                                                     *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan                                           *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: Changed the maximm # of bpgn games in a bpgn file from 1000 to *)
(*             10,000.                                                        *)
(* 1998.11.05: Added view actual text window title.                           *)
(* 1998.11.06: 1) Added escape key.                                           *)
(*             2) Added event title.                                          *)
(*             3) Added site title.                                           *)
(* 1998.11.07: 1) Added tag constants.                                        *)
(*             2) Added TMP_EXT.                                              *)
(* 1998.11.08: 1) Changed the maximum # of positions from 10,000 to 20,000.   *)
(* 1998.11.09: 1) Changed the maximum # of moves to extract from 30 to 20.    *)
(*             2) Added EXTRCT_ERRFN.                                         *)
(*             3) Added VEX_TITLE.                                            *)
(*             4) Removed the limitation for comments.                        *)
(*             5) Changed the maximum # of positions from 20,000 to 10,000.   *)
(* 1998.11.11: 1) Changed the maximum # of comment characters from 0 to 64000.*)
(*             2) Added user select maximum # of positions per bpgn game.     *)
(*             3) Added BPGN_2K.                                              *)
(* 1998.11.13: Added opponent, bpgn filename, and game # into extract's       *)
(*             occurrence tag.                                                *)
(* 1998.11.14: 1) Added character 254 as the comment's carriage return symbol.*)
(*             2) Added FCD_TITLE.                                            *)
(*             3) Added RTF extension.                                        *)
(* 1998.11.17: 1) Added TAB_STR.                                              *)
(*             2) Added RTF_TITLE.                                            *)
(* 1998.11.18: 1) Added TXT_EXT.                                              *)
(*             2) Added TXT_TITLE.                                            *)
(*             3) Added VTXT_TITLE.                                           *)
(* 1998.11.21: Added KEYMAP_TITLE.                                            *)
(* 1998.11.23: Added PIECEMAP_TITLE.                                          *)
(* 1998.11.24: 1) Added TXTMAP_TITLE.                                         *)
(*             2) Added TXTCMAP_TITLE.                                        *)
(*                                                                            *)
(*============================================================================*)
CONST
    PROGRAM_TITLE  = 'BPGN viewer';
    BOARD1_TITLE   = 'Board #1';
    BOARD2_TITLE   = 'Board #2';
    NAVIGATE_TITLE = 'Control';
    GAMELIST_TITLE = 'Game list';
    NEXTMOVE_TITLE = 'Next move';
    PREVMOVE_TITLE = 'Previous next move';
    COMMENT_TITLE  = 'Comment';
    ANNOTATE_TITLE = 'Annotate';
    CLOCK_TITLE    = 'Edit clock';
    HANDLE_TITLE   = 'Edit handle';
    RATING_TITLE   = 'Edit rating';
    EXTRACT_TITLE  = 'Extract moves from BPGN files';
    ABOUT_TITLE    = 'About BPGN viewer';
    VAT_TITLE      = 'View actual text';
    EVENT_TITLE    = 'Edit event';
    SITE_TITLE     = 'Edit site';
    DATE_TITLE     = 'Edit date';
    TIMECTRL_TITLE = 'Edit time control';
    RESULT_TITLE   = 'Edit result';
    VEX_TITLE      = 'View extract.err';
    FCD_TITLE      = 'Convert current BPGN game to four column display RTF file';
    RTF_TITLE      = 'View RTF file';
    TXT_TITLE      = 'Convert current BPGN game to four column display TXT file';
    VTXT_TITLE     = 'View TXT file';
    KEYMAP_TITLE   = 'Edit chess font keymap';
    PIECEMAP_TITLE = 'Edit move piece keymap';
    TXTMAP_TITLE   = 'Edit move piece keymap (TXT)';
    TXTCMAP_TITLE  = 'Edit chess piece keymap (TXT)';

    INI_EXT        = 'INI';
    BPGN_EXT       = 'BPGN';
    TMP_EXT        = '@@@';
    TMP1_EXT       = '###';
    RTF_EXT        = 'RTF';
    TXT_EXT        = 'TXT';

    BPGN_2K        = 0;
    BPGN_5K        = 1;
    BPGN_10K       = 2;
    BPGN_20K       = 3;
    BPGN_30K       = 4;
    BPGN_40K       = 5;

    MAX_STRING     = 65536;
    MAX_LINE       = 100;
    MAX_GAME       = 10000;
    MAX_NEXT       = 50;
    MAX_MEMO_CHAR  = 64000;
    MAX_BPGN_SAVE  = 10;
    MAX_BPGN_LEN   = 79;
    MAX_EXTRACT_NUM= 20;
    DEF_EXTRACT_NUM= 10;
    EXTRACT_ERRFN  = 'EXTRACT.ERR'; 

    WIDTH_GAP      = 5;
    HEIGHT_GAP     = 3;

    NAVIGATE_WIDTH = 298;
    NAVIGATE_HEIGHT= 52;

    FICS_NAME_LEN  = 16;

    PLAYERW_NAME   = 'WHITE';
    PLAYERB_NAME   = 'BLACK';

    TIMEW_STR      = '00:00';
    TIMEB_STR      = '00:00';

    PIECE_ORG_SIZE = 48;
    CLOCK_SPACEY   = 10;

    SPACEW_GAP     = 2;
    SPACEH_GAP     = 2;
    SPACEB_GAP     = 3;

    BPGN_NORMAL    = 0;
    BPGN_STRING    = 1;
    BPGN_INTEGER   = 2;
    BPGN_COMMENT1  = 3;
    BPGN_COMMENT2  = 4;
    BPGN_NAG       = 5;
    BPGN_SYMBOL    = 6;

    BPGN_WHITE_WINS= '1-0';
    BPGN_BLACK_WINS= '0-1';
    BPGN_DRAW      = '1/2-1/2';
    BPGN_UNKNOWN   = '*';

    DEFAULT_EVENT  = 'fics rated bughouse match';
    DEFAULT_SITE   = 'fics, Oklahoma City, OK USA';
    DEFAULT_DATE   = '????.??.??';
    DEFAULT_WHITEA = 'WhiteA';
    DEFAULT_BLACKA = 'BlackA';
    DEFAULT_WHITEB = 'WhiteB';
    DEFAULT_BLACKB = 'BlackB';
    DEFAULT_TIME   = '180';
    DEFAULT_INC    = '0';
    DEFAULT_RATING = '----';
    DEFAULT_REASON = '{*}';
    DEFAULT_RESULT = '*'; 

    TAG_OK         = ' ';
    TAG_DELETE     = 'D';

    BPGN_OCCUR_STR = '<Occurrence: ';
    BPGN_OCCUR_NUM = 13;

    BPGN_EVENT_STR = '[Event ';
    BPGN_EVENT_NUM = 7;

    BPGN_SITE_STR  = '[Site ';
    BPGN_SITE_NUM  = 6;

    BPGN_DATE_STR  = '[Date ';
    BPGN_DATE_NUM  = 6;

    BPGN_WHITEA_STR= '[WhiteA ';
    BPGN_WHITEA_NUM= 8;

    BPGN_BLACKA_STR= '[BlackA ';
    BPGN_BLACKA_NUM= 8;

    BPGN_WHITEB_STR= '[WhiteB ';
    BPGN_WHITEB_NUM= 8;

    BPGN_BLACKB_STR= '[BlackB ';
    BPGN_BLACKB_NUM= 8;

    BPGN_TIMEC_STR = '[TimeControl ';
    BPGN_TIMEC_NUM = 13;

    BPGN_RESULT_STR= '[Result ';
    BPGN_RESULT_NUM= 8;

    BPGN_OPENA_STR = '[BECOA ';
    BPGN_OPENA_NUM = 7;

    BPGN_OPENB_STR = '[BECOB ';
    BPGN_OPENB_NUM = 7;

    BPGN_WHITEA_ELO= '[WhiteAElo ';
    BPGN_BLACKA_ELO= '[BlackAElo ';
    BPGN_WHITEB_ELO= '[WhiteBElo ';
    BPGN_BLACKB_ELO= '[BlackBElo ';

    UNKNOWN_RATING = '????';

    TAB_ASCII_NUM  = 9;
    CR_ASCII_NUM   = 13;
    LF_ASCII_NUM   = 10;

    TAB_ASCII_CHAR = #9;
    CR_ASCII_CHAR  = #13;
    LF_ASCII_CHAR  = #10;
    ESC_ASCII_CHAR = #27;
    COMMENT_CR_CHAR= #254;

    TAB_STR        = TAB_ASCII_CHAR;
    EOL_STR        = CR_ASCII_CHAR + LF_ASCII_CHAR;

    INIT_GAME_POS  = 'RNBQKBNRPPPPPPPP                                pppppppprnbqkbnr';
    INIT_WHITE_PC  = '';
    INIT_BLACK_PC  = '';
    BLANK_GAME_POS = '                                                                ';

    WHITE_COLOR    = 1;
    BLACK_COLOR    = 2;
    EMPTY_COLOR    = 0;
    UNKNOWN_COLOR  = -1;

    WHITE_PAWN     = 'P';
    WHITE_ROOK     = 'R';
    WHITE_KNIGHT   = 'N';
    WHITE_BISHOP   = 'B';
    WHITE_QUEEN    = 'Q';
    WHITE_KING     = 'K';

    BLACK_PAWN     = 'p';
    BLACK_ROOK     = 'r';
    BLACK_KNIGHT   = 'n';
    BLACK_BISHOP   = 'b';
    BLACK_QUEEN    = 'q';
    BLACK_KING     = 'k';

    EMPTY_SQUARE   = ' ';

    PAWN_INDEX     = 1;
    ROOK_INDEX     = 2;
    KNIGHT_INDEX   = 3;
    BISHOP_INDEX   = 4;
    QUEEN_INDEX    = 5;
    KING_INDEX     = 6;
    EMPTY_INDEX    = 0;
    UNKNOWN_INDEX  = -1;

    MOVE_REGULAR   = ' ';
    MOVE_O_O       = 'K';
    MOVE_O_O_O     = 'Q';
    MOVE_PROMOTE   = 'P';
    MOVE_ENPASANT  = 'E';
    MOVE_DROP      = 'D';

    MAX_MOVE       = 1000;
    MAX_DIR        = 4;
    MAX_DIST       = 7;
    MAX_KNIGHT     = 8;
    MAX_KING       = 8;
