(*============================================================================*)
(* filter.PAS -- Filter form                                                  *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2002.04-05: First construction.                                            *)
(* 2004.11.15: Set the default button to Ok                                   *)
(* 2004.11.27: 1) Added Move/Stats button, radiogroup1                        *)
(*             2) Added stringgrid1 to show game stats                        *)
(*             3) Changed caption of "Ok" button to "GAmes"                   *)
(*             4) Changed form layout, Set cancel button to react to esc      *)
(* 2004.12.03: Added bfullstats, if true, items in () are shown in stats      *)
// 2004.12.29: 1) Changed form layout and captions
//             2) Added gofilter, gotree, gofiltree that change the form appearance
//             3) Added makeitnormal/makeitchild to change the form state
//             4) Added Formresize to resize the grid
//             5) Added gobpgn/Godatabase
//             6) Added FilterCaptions and TreeCaptions
// 2005.01.02: Changed the cursor over the clickable grid to hand. Changed
//             the cursor to hourglass while the tree is being built: gobusy/goready
// 2005.01.07  1) Fixed bug: Tree is not refreshed when one unchecks "Use filter"
// 2005.01.29: 1) Introduced states. Added ChangeState
//             3) Modified radiogroup1click
//             4) Added notamove
//             5) Modified goBpgn to uncheck Use Filter checkbox
// 2005.02.28  1) Added Checkbox "Selected only"
//             2) Added selector "Apply ratings to"
(*============================================================================*)

unit filter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,global,misc, ExtCtrls, Grids;

const
nTitleShift=4;
nStandardGridSize=60;
nStandardMaxWidth=385;

type
TFilterState=(fsFilter,fsTree,fsEmpty);
  TFForm = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    CheckBox5: TCheckBox;
    CheckBox6: TCheckBox;
    CheckBox7: TCheckBox;
    CheckBox8: TCheckBox;
    Edit5: TEdit;
    Edit6: TEdit;
    Label9: TLabel;
    Label8: TLabel;
    Edit7: TEdit;
    Label10: TLabel;
    Edit8: TEdit;
    Edit9: TEdit;
    Edit10: TEdit;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Edit11: TEdit;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label14: TLabel;
    Edit12: TEdit;
    Edit13: TEdit;
    Edit14: TEdit;
    Label16: TLabel;
    Label17: TLabel;
    CheckBox9: TCheckBox;
    CheckBox10: TCheckBox;
    CheckBox11: TCheckBox;
    CheckBox12: TCheckBox;
    RadioGroup1: TRadioGroup;
    CheckBox13: TCheckBox;
    StringGrid1: TStringGrid;
    Button4: TButton;
    CheckBox14: TCheckBox;
    RadioGroup2: TRadioGroup;
    CheckBox15: TCheckBox;
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure setupvalues(f:TfilterRec);
    procedure clearstats;
    procedure gofilter;
    procedure gotree;
    procedure gofiltree;
    procedure makeitchild;
    procedure makeitnormal;
    procedure CheckBox13Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure CheckBox14Click(Sender: TObject);
    procedure godatabase;
    procedure gobpgn;
    procedure filtercaptions;
    procedure treecaptions;
    procedure RadioGroup1Click(Sender: TObject);
    procedure Gobusy;
    procedure goready;
    function NotaMove(row:integer):boolean;
    procedure ChangeState(state:TfilterState);
    procedure Resetall;
    procedure RadioGroup2Click(Sender: TObject);
  private
   breset:boolean;
   fState:TfilterState;
   bPosSearch:boolean;
    { Private declarations }
  public
    { Public declarations }
    bOk:boolean;
    filter:TfilterRec;
    bfullstat:boolean;
    nStandardHeight:integer;
    nDeltavertical:integer;
  end;

implementation

{$R *.DFM}
procedure TFForm.Setupvalues(f:TfilterRec);
var n,er:integer;
st:string;
//rect:TGridRect;
begin
 filter:=f;
 if breset then resetfilterrec(filter);
 breset:=false;
 bOk:=false;
 with filter do
 begin
  edit1.text:=player1;
  edit2.text:=player2;
  edit3.text:=player3;
  edit4.text:=player4;
  checkbox1.checked:=bwin;
  checkbox2.checked:=bloss;
  checkbox3.checked:=bdraw;
  checkbox4.checked:=bunknown;
  checkbox5.checked:=bwhite;
  checkbox6.checked:=bblack;
  checkbox7.checked:=bopponent;
  checkbox8.checked:=bpartner;
  checkbox9.checked:=brated;
  checkbox10.checked:=bunrated;
  checkbox11.checked:=bbug;
  checkbox12.checked:=bcrazy;
  edit5.text:=formatdatetime('yyyy.mm.dd',datemin);
  edit6.text:=formatdatetime('yyyy.mm.dd',datemax);
  str(timemin,st);
  edit7.text:=st;
  str(timemax,st);
  edit8.text:=st;
  str(minrmin,st);
  edit9.text:=st;
  str(minrmax,st);
  edit10.text:=st;
  str(ravmin,st);
  edit11.text:=st;
  str(ravmax,st);
  edit12.text:=st;
  str(maxrmin,st);
  edit13.text:=st;
  str(maxrmax,st);
  edit14.text:=st;
  radiogroup2.itemindex:=nRatingMode;
  checkbox15.Checked:=bSelected;
  if ((radiogroup1.items.Count=2) and (possearchmode=3)) then radiogroup1.ItemIndex:=0
  else radiogroup1.ItemIndex:=possearchmode-1;
  if radiogroup1.ItemIndex=2 then stringgrid1.Cursor:=crDefault else stringgrid1.Cursor:=crHandPoint;
 end;
end;

procedure TFForm.Button2Click(Sender: TObject);
begin
 close;
end;

procedure TFForm.Button1Click(Sender: TObject);
var n,er:integer;
begin
 bOk:=false;
 with filter do
  if edit1.Visible then
  begin
   datemin:=longstrtodate(edit5.text,er);
   if er<0 then
   begin
    werror('Invalid minimal date');
    ActiveControl:=edit5;
    exit;
   end;
   datemax:=longstrtodate(edit6.text,er);
   if er<0 then
   begin
    werror('Invalid maximal date');
    ActiveControl:=edit6;
    exit;
   end;
   edit7.Text:=trim(edit7.text);
   edit8.Text:=trim(edit8.text);
   edit9.Text:=trim(edit9.text);
   edit10.Text:=trim(edit10.text);
   edit11.Text:=trim(edit11.text);
   edit12.Text:=trim(edit12.text);
   val(edit7.text,timemin,er);
   if er<>0 then
   begin
     Werror('Invalid Minimal Time');
     ActiveControl:=edit7;
     exit;
   end;
   val(edit8.text,timemax,er);
   if er<>0 then
   begin
     Werror('Invalid Maximal Time');
     ActiveControl:=edit8;
     exit;
   end;
   If edit9.Enabled then
   begin
   val(edit9.text,MinRmin,er);
   if er<>0 then
   begin
     Werror('Invalid low boundary for minimal rating');
     ActiveControl:=edit9;
     exit;
   end;
   val(edit10.text,MinRMax,er);
   if er<>0 then
   begin
     Werror('Invalid high boundary for minimal rating');
     ActiveControl:=edit10;
     exit;
   end;

   val(edit13.text,MaxRMin,er);
   if er<>0 then
   begin
     Werror('Invalid low boundary for maximal rating');
     ActiveControl:=edit13;
     exit;
   end;
   val(edit14.text,MaxRMax,er);
   if er<>0 then
   begin
     Werror('Invalid high boundary for maximal rating');
     ActiveControl:=edit14;
     exit;
   end;
   end; {edit9.enabled}

   val(edit11.text,RavMin,er);
   if er<>0 then
   begin
     Werror('Invalid low boundary for (average) rating');
     ActiveControl:=edit11;
     exit;
   end;
  
   val(edit12.text,RavMax,er);
   if er<>0 then
   begin
     Werror('Invalid high boundary for (average) rating');
     ActiveControl:=edit12;
     exit;
   end;

   if (not edit9.Enabled) then
   begin
    MinRMin:=RavMin; MinRMax:=RavMax;
    MaxRMin:=RavMin; MaxRMax:=RavMax;
   end;

   player1:=LowerCase(trim(edit1.text));
   player2:=LowerCase(trim(edit2.text));
   player3:=LowerCase(trim(edit3.text));
   player4:=LowerCase(trim(edit4.text));

   if ((player1='') or (player2='')) then checkbox7.checked:=false;
   if ((player1='') or (player3='')) then checkbox8.checked:=false;
   if Player1='' then
   begin
    checkbox5.checked:=true;
    checkbox6.checked:=true;
//    checkbox2.checked:=checkbox1.checked;
   end;

   bwin:=checkbox1.checked;
   bloss:=checkbox2.checked;
   bdraw:=checkbox3.checked;
   bunknown:=checkbox4.checked;

   bwhite:=checkbox5.checked;
   bblack:=checkbox6.checked;

   bopponent:=checkbox7.checked;
   bpartner:=checkbox8.checked;
   brated:=checkbox9.checked;
   bunrated:=checkbox10.checked;
   bbug:=checkbox11.checked;
   bcrazy:=checkbox12.checked;
   possearchmode:=Radiogroup1.ItemIndex+1;
   bautosearch:=((checkbox13.Checked) and (checkbox13.Visible));
   bSelected:=checkbox15.Checked;
   nRatingMode:=RadioGroup2.ItemIndex;
  end
  else begin
    resetfilterrec(filter);
    possearchmode:=radiogroup1.ItemIndex+1;
    bautosearch:=checkbox13.Checked;
  end;
 boK:=true;
end;

procedure TFForm.Button3Click(Sender: TObject);
var f:Tfilterrec;
begin
 resetfilterrec(f);
 setupvalues(f);
 breset:=true;
end;

procedure tFForm.clearstats;
var i,j:integer;
begin
 with StringGrid1 do
    for i:=1 to rowcount-1 do
    for j:=0 to colcount-1 do
     cells[j,i]:='';
end;


procedure tFForm.gofilter;
var sh:integer;
begin
 checkbox13.visible:=false;
 button4.Visible:=false;
 stringgrid1.Visible:=false;

 radiogroup1.top:=168;
 radiogroup1.ItemIndex:=2;
 radiogroup1.Enabled:=false;
 checkbox13.Top:=radiogroup1.Top;
 checkbox14.Top:=checkbox13.Top+16;
 button1.Top:=208; button1.Left:=radiogroup1.Left;
 button2.Top:=button1.Top;
 button3.Top:=button1.Top; button3.Left:=272;
 button4.Top:=button1.Top;
 button2.Left:=button1.Left+button1.Width+(button3.Left-button1.left-button1.Width-button2.Width) div 2;

 sh:=24;
 label6.Top:=button1.Top+button1.Height+8; edit5.Top:=label6.Top; label8.top:=label6.top; edit6.Top:=label6.Top;
 label9.Top:=label6.Top+sh; edit7.Top:=label9.Top;label10.top:=label9.top;edit8.Top:=label9.Top;
 label12.Top:=label9.Top+sh; edit9.Top:=label12.Top;label11.top:=label12.top;edit10.Top:=label12.Top;
 label16.Top:=label12.Top+sh; edit13.Top:=label16.Top;label14.top:=label16.top;edit14.Top:=label16.Top;
 label13.Top:=label16.Top+sh; edit11.Top:=label13.Top;label17.top:=label13.top; edit12.Top:=label13.Top;
 radiogroup2.Top:=edit8.Top;

 button3.Visible:=true;
 checkbox14.Visible:=true;
 label1.Visible:=true;
 label2.Visible:=true;
 label3.Visible:=true;
 label4.Visible:=true;
 edit1.Visible:=true;
 edit2.Visible:=true;
 edit3.Visible:=true;
 edit4.Visible:=true;
 label5.Visible:=true;
 label7.Visible:=true;
 checkbox1.Visible:=true;
 checkbox2.Visible:=true;
 checkbox3.Visible:=true;
 checkbox4.Visible:=true;
 checkbox5.Visible:=true;
 checkbox6.Visible:=true;
 checkbox7.Visible:=true;
 checkbox8.Visible:=true;
 checkbox9.Visible:=true;
 checkbox10.Visible:=true;
 checkbox11.Visible:=true;
 checkbox12.Visible:=true;
 label6.Visible:=true;
 label9.Visible:=true;
 label12.Visible:=true;
 label16.Visible:=true;
 label13.Visible:=true;
 edit5.Visible:=true;
 edit6.Visible:=true;
 edit7.Visible:=true;
 edit8.Visible:=true;
 edit9.Visible:=true;
 edit10.Visible:=true;
 edit11.Visible:=true;
 edit12.Visible:=true;
 edit13.Visible:=true;
 edit14.Visible:=true;
 label8.visible:=true;
 label10.visible:=true;
 label11.visible:=true;
 label14.visible:=true;
 label17.visible:=true;
 checkBox15.Visible:=true;
 Radiogroup2.Visible:=true;
 nDeltaVertical:=edit12.top+edit12.height+nTitleshift;
// nDeltaVertical:=RadioGroup2.top+RadioGroup2.height+nTitleshift;
 clientHeight:=nDeltaVertical;
 Constraints.MaxHeight:=height;
end;

procedure tFForm.gotree;
begin
 constraints.MaxHeight:=0;
 button4.Visible:=false;
 button3.Visible:=false;
 label1.Visible:=false;
 label2.Visible:=false;
 label3.Visible:=false;
 label4.Visible:=false;
 edit1.Visible:=false;
 edit2.Visible:=false;
 edit3.Visible:=false;
 edit4.Visible:=false;
 label5.Visible:=false;
 label7.Visible:=false;
 checkbox1.Visible:=false;
 checkbox2.Visible:=false;
 checkbox3.Visible:=false;
 checkbox4.Visible:=false;
 checkbox5.Visible:=false;
 checkbox6.Visible:=false;
 checkbox7.Visible:=false;
 checkbox8.Visible:=false;
 checkbox9.Visible:=false;
 checkbox10.Visible:=false;
 checkbox11.Visible:=false;
 checkbox12.Visible:=false;
 label6.Visible:=false;
 label9.Visible:=false;
 label12.Visible:=false;
 label16.Visible:=false;
 label13.Visible:=false;
 edit5.Visible:=false;
 edit6.Visible:=false;
 edit7.Visible:=false;
 edit8.Visible:=false;
 edit9.Visible:=false;
 edit10.Visible:=false;
 edit11.Visible:=false;
 edit12.Visible:=false;
 edit13.Visible:=false;
 edit14.Visible:=false;
 label8.visible:=false;
 label10.visible:=false;
 label11.visible:=false;
 label14.visible:=false;
 label17.visible:=false;
 checkbox15.Visible:=false;
 Radiogroup2.Visible:=false;
 radiogroup1.top:=edit1.Top-4;
 checkbox13.Top:=radiogroup1.Top;
 checkbox14.Top:=checkbox13.Top+16;
 stringgrid1.Top:=radiogroup1.top+radiogroup1.height+8;

 button1.Top:=stringgrid1.top+stringgrid1.height+8; button1.Left:=button4.Left;
 button2.Top:=button1.Top; button2.Left:=184;
 button3.Top:=button1.Top;
 button4.Top:=button1.Top;
// label19.Top:=button4.Top+button4.Height+nTitleshift;
 nDeltaVertical:=button4.top+button4.Height-stringgrid1.Height+nTitleShift;
 clientheight:=nDeltaVertical+stringgrid1.Height;

 stringgrid1.Visible:=true;
 checkbox13.Visible:=true;
 checkbox14.Visible:=true;
end;

procedure tFForm.gofiltree;
var sh:integer;
begin
 stringgrid1.Visible:=false;
 constraints.MaxHeight:=0;
 radiogroup1.top:=168;
 if bPosSearch then radiogroup1.Enabled:=true;
 checkbox13.Top:=radiogroup1.Top;
 checkbox14.Top:=checkbox13.Top+16;
 button1.Top:=208; button1.Left:=radiogroup1.Left;
 button2.Top:=button1.Top; button2.Left:=184;
 button3.Top:=button1.Top; button3.Left:=272;
 button4.Top:=button1.Top;

 stringgrid1.Top:=240;

 sh:=24;
 label6.Top:=stringgrid1.Top+stringgrid1.Height+8; edit5.Top:=label6.Top; label8.top:=label6.top; edit6.Top:=label6.Top;
 label9.Top:=label6.Top+sh; edit7.Top:=label9.Top; label10.top:=label9.top; edit8.Top:=label9.Top;
 label12.Top:=label9.Top+sh; edit9.Top:=label12.Top; label11.top:=label12.top; edit10.Top:=label12.Top;
 label16.Top:=label12.Top+sh; edit13.Top:=label16.Top;label14.top:=label16.top;edit14.Top:=label16.Top;
 label13.Top:=label16.Top+sh; edit11.Top:=label13.Top; label17.top:=label13.top; edit12.Top:=label13.Top;
 radiogroup2.Top:=edit8.Top;
 
 label1.Visible:=true;
 label2.Visible:=true;
 label3.Visible:=true;
 label4.Visible:=true;
 edit1.Visible:=true;
 edit2.Visible:=true;
 edit3.Visible:=true;
 edit4.Visible:=true;
 label5.Visible:=true;
 label7.Visible:=true;
 checkbox1.Visible:=true;
 checkbox2.Visible:=true;
 checkbox3.Visible:=true;
 checkbox4.Visible:=true;
 checkbox5.Visible:=true;
 checkbox6.Visible:=true;
 checkbox7.Visible:=true;
 checkbox8.Visible:=true;
 checkbox9.Visible:=true;
 checkbox10.Visible:=true;

 button3.Visible:=true;
 button4.Visible:=true;
 CheckBox13.Visible:=true;
 checkbox14.Visible:=true;
 label6.Visible:=true;
 label9.Visible:=true;
 label12.Visible:=true;
 label16.Visible:=true;
 label13.Visible:=true;
 edit5.Visible:=true;
 edit6.Visible:=true;
 edit7.Visible:=true;
 edit8.Visible:=true;
 edit9.Visible:=true;
 edit10.Visible:=true;
 edit11.Visible:=true;
 edit12.Visible:=true;
 edit13.Visible:=true;
 edit14.Visible:=true;
 label8.visible:=true;
 label10.visible:=true;
 label11.visible:=true;
 label14.visible:=true;
 label17.visible:=true;
 checkbox15.Visible:=true;
 radiogroup2.Visible:=true;
 nDeltaVertical:=edit12.top+edit12.height+nTitleShift-stringgrid1.Height;
// nDeltaVertical:=radiogroup2.top+radiogroup2.height+nTitleShift-stringgrid1.Height;
 clientheight:=stringgrid1.height+nDeltavertical;
 stringgrid1.Visible:=true;
end;

procedure TFForm.CheckBox13Click(Sender: TObject);
begin
 if formStyle=fsNormal then
 begin
  makeitchild;
 end
 else begin
  makeitnormal;
 end;
end;

procedure tFForm.makeitchild;
var prevtop,deltax,deltay,prevleft,prevh,prevw:integer;
pntold,pnt:tpoint;
begin
  prevtop:=top;prevleft:=left;
  prevw:=width; prevh:=height;
  pntold:=clientOrigin;
  visible:=false;
  bordericons:=[];
  windowState:=wsMinimized;
  formstyle:=fsMdiChild;
  constraints.MaxWidth:=nStandardMaxWidth;
  setbounds(prevleft,prevtop,prevw,prevh);
  windowState:=wsNormal;
  pnt:=clientOrigin;
  deltaY:=pnt.Y-pntold.Y;
  deltax:=pnt.X-pntold.X;
  borderIcons:=[biSystemMenu,bIMinimize];
  setbounds(prevleft-deltax,prevtop-deltay,prevw,prevh);
end;

procedure tFForm.makeitnormal;
var deltay,deltax,prevtop,prevleft,prevh,prevw:integer;
pnt,pntold:Tpoint;
begin
  pntold:=clientOrigin;
  prevtop:=top; prevleft:=left;
  prevw:=width;prevh:=height;
  formstyle:=fsNormal;
  visible:=false;
  constraints.MaxWidth:=nStandardMaxWidth;
  windowstate:=wsNormal;
  pnt:=clientOrigin;
  deltax:=pntold.X-pnt.X;
  deltay:=pntold.Y-pnt.Y;
  borderIcons:=[biSystemmenu];
  top:=prevtop+deltaY;left:=prevleft+deltax;
  setbounds(prevleft+deltax,prevtop+deltaY,prevw,prevh);
  visible:=true;
end;

procedure TFForm.FormResize(Sender: TObject);
var i,ndelta:integer;
begin
 if (stringgrid1.Visible) then
 begin
  if clientheight>(ndeltavertical+nStandardgridsize) then
  ndelta:=clientheight-ndeltavertical-Stringgrid1.Height
  else ndelta:=nStandardgridsize-stringgrid1.Height;
  stringgrid1.Height:=stringgrid1.Height+ndelta;
  for i:=0 to Controlcount-1 do
  with controls[i] do
  begin
   if ((visible) and (Top>stringgrid1.Top)) then top:=top+ndelta;
  end;
 end;
end;

procedure TFForm.CheckBox14Click(Sender: TObject);
begin
 if ((stringgrid1.Visible) and (not edit1.Visible)) then
 begin gofiltree; exit; end;
 if ((not stringgrid1.Visible) and (edit1.Visible)) then
 begin gofiltree; exit; end;
 if ((stringgrid1.Visible) and (edit1.Visible)) then
 if fState=fsTree then
 begin
  gotree;
  SendMessage(fMainHandle,WM_UpdateTree,0,0);
 end
 else gofilter;
end;

procedure tFForm.godatabase;
begin
   checkbox11.Checked:=true;
   CheckBox12.Checked:=true;
   checkbox11.Enabled:=false;
   CheckBox12.Enabled:=false;
   RadioGroup1.Enabled:=true;
   CheckBox13.Enabled:=true;
   StringGrid1.Enabled:=true;
   if Radiogroup1.ItemIndex=2 then button4.Caption:='Stats' else button4.Caption:='Tree';
   bPosSearch:=true;
end;

procedure tFForm.gobpgn;
begin
   with radiogroup1 do if Items.Count=2 then
   begin columns:=3; items.Add('None'); end;
   checkbox13.Checked:=false;
   Radiogroup1.ItemIndex:=2;
   RadioGroup1.Enabled:=false;
   StringGrid1.Enabled:=false;
   CheckBox13.Enabled:=false;
   button4.Caption:='Stats';
   checkbox11.Enabled:=true;
   checkbox12.Enabled:=true;
   bPosSearch:=false;
   if fState=fsTree then ChangeState(fsFilter);
end;

procedure tFForm.filtercaptions;
begin
 Caption:='Filter games';
 checkbox14.Caption:='Advanced stats';
 with radiogroup1.Items do if Count=2 then begin radiogroup1.Columns:=3; add('None');end;
 if radiogroup1.ItemIndex=2 then button4.Caption:='Stats' else button4.Caption:='Tree';
 StringGrid1.RowCount:=MAX_NEXT+3;
// StringGrid1.cells[1,0]:='Games';
// StringGrid1.cells[2,0]:='Win %';
// StringGrid1.Cells[3,0]:='Av. Rating';
// StringGrid1.Cells[4,0]:='Perf';
end;

procedure tFForm.treecaptions;
begin
 Caption:='Tree';
 checkbox14.Caption:='Use filter';
 button4.Caption:='Refresh';
 StringGrid1.RowCount:=MAX_NEXT+3;
// StringGrid1.cells[1,0]:='Games';
// StringGrid1.cells[2,0]:='Win %';
// StringGrid1.Cells[3,0]:='Av. Rating';
// StringGrid1.Cells[4,0]:='Perf';
 end;

procedure TFForm.RadioGroup1Click(Sender: TObject);
begin
 if radiogroup1.ItemIndex=2 then
 begin button4.Caption:='Stats'; stringgrid1.Cursor:=crDefault;
  end
 else begin
   if fstate=fsFilter then button4.caption:='Tree' else button4.caption:='Refresh';
   stringgrid1.Cursor:=crHandPoint;
  end;
 filter.possearchmode:=radiogroup1.ItemIndex+1;
 if fstate=fsTree then SendMessage(fMainHandle,WM_UpdateTree,0,0);
end;

procedure tFForm.Gobusy;
begin
 Cursor:=crHourGlass;
end;

procedure tFForm.GoReady;
begin
 Cursor:=crDefault;
end;

function tFForm.NotaMove(row:integer):boolean;
var cmove:string;
begin
 if ((row<1) or (row>stringgrid1.RowCount-3)) then
 begin
  Result:=true; exit;
 end;
 cMove:=trim(stringgrid1.Cells[0,row]);
 if ((Cmove='') or (cMove='End')) then Result:=true else result:=false;
end;

procedure tFForm.ChangeState(state:TfilterState);
begin
 if State=fState then exit;
 fstate:=State;
 case state of
 fsFilter: begin
           with radiogroup1 do if items.count=2 then begin columns:=3; items.add('None'); end;
           if not Checkbox14.Checked then goFilter;
           filterCaptions;
           end;
 fsTree: begin
         radiogroup1.enabled:=true;
         with radiogroup1 do if itemindex=2 then ItemIndex:=0;
         with radiogroup1.Items do if Count=3 then begin radiogroup1.Columns:=2; delete(count-1);end;
         if not Checkbox14.Checked then goTree;
         TreeCaptions;
         end;
 end;
 if checkbox14.Checked then begin goFiltree; end;
 if radiogroup1.ItemIndex=2 then stringgrid1.Cursor:=crDefault else stringgrid1.Cursor:=crHandPoint;
end;

procedure tFForm.Resetall;
begin
 ResetFilterRec(filter);
 SetupValues(filter);
 fState:=fsEmpty;
 nDeltaVertical:=362;
end;

procedure TFForm.RadioGroup2Click(Sender: TObject);
begin
 Case RadioGroup2.ItemIndex of
 rmPlayer..rmPardop: Begin
  Edit9.Enabled:=false;
  Edit10.Enabled:=false;
  Edit13.Enabled:=false;
  Edit14.Enabled:=false;
  Label13.Caption:='Rating';
 end;
 else Begin
  Edit9.Enabled:=true;
  Edit10.Enabled:=true;
  Edit13.Enabled:=true;
  Edit14.Enabled:=true;
  Label13.Caption:='Average rating';
 end;
 end;
end;

end.
