(*============================================================================*)
(* posdb.PAS -- Positional datbase                                            *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2004.11.08: First construction.                                            *)
(* 2004.11.18: Added tposdb.findInsPos,tposdb.findsup, tposdb.comprec         *)
(* 2004.11.20: Added tposdb.compPosition, emptyrecord, tposdb.findMax,        *)
(*             tposdb.findMin, tposdb.getGameN, tposdb.getPositionBounds      *)
(*============================================================================*)

unit posdb;


interface

uses sysutils,classes,misc,math,Global;

const POS_SAVE_BUF_SIZE=10000;

type
 TintBoard= array [1..8] of longint;
type
 tposDBrec=record
  position: TintBoard;
  holding : array [1..2,1..5] of shortint;
  timedif: integer;
  cBoard:byte;
  mfrom:byte;
  mto:byte;
  mIndex:integer;
  cMove:string[5];
  GameN:longint;
 end;

type tPosSaveBuf=array [0..POS_SAVE_BUF_SIZE] of tPosDBrec;

type PposDBRec=^TposDBrec;

type tPosDB=class(TList)
 public
  searchrec:tposdbrec;
  psearchrec:pposdbrec;

  constructor Create;
  function add(p:pposdbrec):integer;
  function insert(var rec:tposdbrec):integer;
  procedure delete(n:longint);
  procedure clear;

  function Save(fname:string):boolean;
  function Load(fname:string):boolean;
  function findInsPos(var rec:tposdbrec):longint;
  function findsup(a,b:longint):longint;
  function comprec(r1,r2:pposdbrec):integer;
  function compPosition(r1,r2:pposdbrec):integer;
  procedure emptyrecord(var rec:TPosDBRec);
  procedure getPositionBounds(var rec:TposDBrec;var nMin,nMax:longint);
  function findMax(nMin,nMax:longint):longint;
  function findMin(nMin,nMax:longint):longint;
  function getGameN(i:longint):longint;
  function getboardnum(i:longint):integer;
end;

procedure BoardtoInt(var board:boardarray; var intboard:TintBoard);

implementation

function piecetoint(piece:char):longint;
begin
 case piece of
    WHITE_PAWN: result:=1;
    WHITE_ROOK: result:= 2;
    WHITE_KNIGHT: result:=3;
    WHITE_BISHOP: result:=4;
    WHITE_QUEEN : result:=5;
    WHITE_KING  : result:=6;

    BLACK_PAWN : result:=7;
    BLACK_ROOK :result:=8;
    BLACK_KNIGHT:result:=9;
    BLACK_BISHOP:result:=10;
    BLACK_QUEEN :result:=11;
    BLACK_KING  :result:=12;

    EMPTY_SQUARE: result:=0;
 end;
end;

procedure BoardtoInt(var board:boardarray; var intboard:TintBoard);
var i,j:integer;
tmp:longint;
begin
 for i:=1 to 8 do
 begin
  intboard[i]:=0;
  for j:=1 to 8 do
  begin
   tmp:=piecetoint(board[i,j]);
   tmp:=tmp shl 4*(j-1);
   inc(intboard[i],tmp);
  end;
 end;
end;

constructor tposdb.Create;
begin
 inherited Create;
 psearchrec:=@searchrec;
end;

function tposdb.add(p:pposdbrec):integer;
begin
 try
  New(p);
  Result:=inherited add(p);
 except on EOutOfMemory do
  result:=-1;
 end;
end;

function tposdb.insert(var rec:tposdbrec):integer;
var n:longint;
p:pPosDBrec;
begin
  n:=findinspos(rec);
  if n<0 then begin Result:=n;exit;end;
  try
   New(p);
   inherited insert(n,p);
  except on EOutOfMemory do
   result:=-2;
  end;
  p^:=rec;
  result:=n;
end;

procedure tposdb.delete(n:longint);
var p:pPosDBrec;
begin
 p:=items[n];
 try
  dispose(p);
 except on EInvalidPointer do begin end;
 end;
 inherited delete(n);
end;

procedure tposdb.clear;
var b:integer;
begin
 while (count>0) do delete(count-1);
end;

function tposdb.Save(fname:string):boolean;
var i,hc,k,listsz:longint;
f:file;
psave:^tPosSaveBuf;
p:pPosDBrec;
begin
  //allocating the memory for buffer
 try
  New(psave);
 except
  Werror('Not enough memory');
  Result:=false;
  exit;
 end;

 AssignFile(f,fname);
 try
  Rewrite(f,1);
 except
  CloseFile(f);
  Werror('Can not write to file ' + fname);
  dispose(psave);
  Result:=false;
  exit;
 end;
 try
  listsz:=count; hc:=0;
  BlockWrite(f,listsz,sizeof(listsz));
  //save the number or records in the position list
  for i:=0 to listsz-1 do
  begin
   p:=items[i];
   psave[hc]:=p^;
   inc(hc,1);
   if (hc>POS_SAVE_BUF_SIZE) then
   begin
    blockwrite(f,psave^,sizeof(tPosSaveBuf));
    hc:=0;
   end;
  end;

 //flush buffer at the end;
  blockwrite(f,psave^,hc*sizeof(tPosDBrec));
 finally
  CloseFile(f);
 end;
 dispose(psave);
 Result:=true;
end;

function tposdb.Load(fname:string):boolean;
var i,k,rc,hr:longint;
f:file;
pload:^tpossavebuf;
bufsize,listsz,bufmemsize:longint;
p,p1:pposDBrec;
begin
 //allocate memory
 try
  new(pload);
 except
  Werror('Not enough memory.'+EOL_STR+'Positional database was not loaded');
  Result:=False;
  exit;
 end;

 //open header file
 AssignFile(f,fname);
 try
  reset(f,1);
 except
  Werror('Can not open file '+fname+EOL_STR+'Positional database was not loaded');
  dispose(pload);
  Result:=False;
  exit;
 end;


 try
  blockread(f,listsz,sizeof(listsz),rc);

  bufmemsize:=sizeof(tpossavebuf); //memory size of the buffer
  bufsize:=POS_SAVE_BUF_SIZE+1;    //number of rec in the buffer

  //since the tpossavebuf is 0 based array

  clear;
  hr:=0; i:=1;
  while(listSz>=i*bufsize) do
  begin
   //read a chunk of position file;
   blockread(f,pload^,bufmemsize,rc);
   if rc<>bufmemsize then // corrupt file
   begin
    wError('Corrupted position file ' + fname+EOL_STR+'Positional database was not loaded');
    dispose(pload);
    clear;
    Result:=false; exit;
   end;

   //populate alist from the chunk
   for k:=0 to bufsize-1 do
   begin
    if add(p)=-1 then //not enough memory
    begin
     wError('Not enough memory'+EOL_STR+'Positional database was not loaded');
     dispose(pload);
     clear;
     Result:=false;
     exit;
    end;
    p:=last;
    p^:=pload[k];
    inc(hr,1);
   end;
   inc(i,1);
  end;

  //read the last chunk of positions
  blockread(f,pload^,(listsz-hr)*sizeof(tPosDBrec),rc);
  if ((listsz-hr)*sizeof(tPosDBrec)<>rc) then
  begin
   wError('Corrupted position file ' + fname+EOL_STR+'Positional database was not loaded');
   dispose(pload);
   clear;
   Result:=false; exit;
  end;

  //populate alist from the chunk
  for k:=0 to listsz-hr-1 do
  begin
   if add(p)=-1 then //not enough memory
   begin
    wError('Not enough memory'+EOL_STR+'Positional database was not loaded');
    dispose(pload);
    clear;
    Result:=false;
    exit;
   end;
   p:=last;
   p^:=pload[k];
  end;

 finally
  CloseFile(f);
 end;
 dispose(pload);
 Result:=true;
end;

function tPosDB.findInsPos(var rec: tPosDBrec):longint;
var a,b:longint;
res:integer;
n2:longint;
begin
 if count=0 then begin Result:=0; exit; end;
 searchrec:=rec;
 a:=0;
 b:=count-1;
 res:=findsup(a,b);
 if comprec(@rec,items[b])=0 then Result:=-1 else result:=res;
end;

function tPosdb.findsup(a,b:integer):longint;
var
p1:pPosdbRec;
res:integer;
n2:longint;
begin
 p1:=items[b];
 res:=comprec(psearchrec,p1);
 case res of
 0:  result:=b;
 1:  result:=b+1;
 else
  if b-a<2 then
  begin
   if comprec(psearchrec,items[a])<=0 then result:=a
   else result:=b;
   exit;
  end;
  n2:=(b+a) div 2; p1:=items[n2];
  res:=comprec(psearchrec,p1);
  if res<=0 then result:=findsup(a,n2) else result:=findsup(n2,b);
 end;
end;

function tposdb.compPosition(r1,r2:pposdbrec):integer;
var res:integer;
i:integer;
begin
 for i:=1 to 8 do
 begin
  res:=sign(r1^.position[i]-r2^.position[i]);
  if res<>0 then begin result:=res; exit; end;
 end;

 result:=(r1^.cboard-r2^.cboard) div 2;

end;

function tposdb.comprec(r1,r2:pposdbrec):integer;
var res:integer;
begin
 res:=compPosition(r1,r2);
 if res<>0 then begin result:=res;exit;end;

 res:=sign(r1^.GameN-r2^.gameN);
 if res<>0 then begin result:=res; exit; end;

 res:=sign(r1^.mfrom-r2^.mfrom);
 if res<>0 then begin result:=res; exit; end;

 res:=sign(r1^.mTo-r2^.mTo);
 if res<>0 then begin result:=res; exit; end;

 result:=sign(r1^.cBoard-r2^.cBoard);

end;

procedure tposdb.emptyrecord(var rec:TposDBRec);
var i,j:integer;
begin
 for i:=1 to 8 do rec.position[i]:=0;
 for i:=1 to 2 do
 for j:=1 to 5 do
  rec.holding[i,j]:=0;
 rec.timedif:=0;
 rec.cBoard:=0;
 rec.mfrom:=0;
 rec.mto:=0;
 rec.mIndex:=0;
 rec.cMove:='';
 rec.GameN:=-1;
end;

function tposdb.findMaX(nMin,Nmax:longint):longint;
var n2:longint;
res:integer;
begin
   res:=compPosition(psearchrec,items[nMax]);
   case res of
   0:  result:=nMax;
   1:  result:=-1;
   else
    if Nmax-Nmin<2 then
    begin
     if (compPosition(psearchrec,items[nMin])=0) then Result:=nMin else Result:=-1;
     exit;
    end;
    n2:=(Nmax+Nmin) div 2;
    if compPosition(psearchrec,items[n2])>=0
    then result:=findMaX(n2,nMax)
    else Result:=findMax(nMin,n2);
   end;
end;

function tposdb.findMin(nMin,Nmax:longint):longint;
var n2:longint;
res:integer;
begin
   res:=compPosition(psearchrec,items[nMin]);
   case res of
   0:  result:=nMin;
  -1:  result:=count;
   else
    if Nmax-Nmin<2 then
    begin
     if (compPosition(psearchrec,items[nMax])=0) then Result:=nMaX else Result:=count;
     exit;
    end;
    n2:=(Nmax+Nmin) div 2;
    if compPosition(psearchrec,items[n2])<=0 then result:=findMin(nMin,n2)
    else Result:=findMin(n2,nMax);
   end;
end;

procedure tposdb.getPositionBounds(var rec:TposDBrec;var nMin,nMax:longint);
begin
 searchrec:=rec;
 nMax:=findMax(0,count-1);
 if nMax>=0 then nMin:=findMin(0,nMax) else nMin:=count;
end;

function tposdb.getGameN(i:longint):longint;
var p:PposDbrec;
begin
 p:=items[i];
 Result:=p^.GameN;
end;

function tposdb.getboardnum(i:longint):integer;
begin
 result:=Pposdbrec(items[i])^.cBoard mod 2;
end;

end.
