(*============================================================================*)
(* posdb1.PAS -- Positional datbase                                            *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2004.11.08: First construction.                                            *)
(* 2004.11.18: Added tposdb.findInsPos,tposdb.findsup, tposdb.comprec         *)
(* 2004.11.20: Added tposdb.compPosition, emptyrecord, tposdb.findMax,        *)
(*             tposdb.findMin, tposdb.getGameN, tposdb.getPositionBounds      *)
(* 2004.11.22: 1) Changed order of records in the db. Now the db is sorted on *)
(*             GameN (blackcomb method), unlike in the first attempt, where   *)
(*             games where sorted based on position. This is done to make     *)
(*             Creation time of the db reasonable.                            *)
(*             2) functions tposdb.findMax, tposdb.getPositionBound,          *)
(*             tposdb.findMin are gone as unneeded,                           *)
(*             3) added functions findPosInGame, findMinGameX                 *)
(*             4) Modified function add to initialize the added element       *)
(* 2004.11.27  1) Added bisdb to bposdbrec                                    *)
(*             2) Modified function clear to free all allocated memory first  *)
(*                instead of freeing elements 1 by 1                          *)
(*             3) Modified findminxgame to use range and internal searchrec   *)
(*             4) added cleardbnext                                           *)
(* 2004.11.29  Added RemoveGame, defaultInsertN,incDefaultInsertN used to     *)
(*             update a game in the db                                        *)
(* 2004.12.04  Fixed bug is boardtoint that caused different board positions  *)
(*             to be converted to one                                         *)
(* 2004.12.12  Removed FindMinGameX as no longer needed                       *)
(* 2004.12.17  Added Append,GetPositionCount                                  *)
(*============================================================================*)

unit posdb1;


interface

uses sysutils,classes,misc,math,Global,bpgnlist;

const POS_SAVE_BUF_SIZE=10000;

type
 tposDBrec=record
  position: TintBoard;
 // holding : array [1..2,1..5] of shortint;
  timedif: integer;
  cBoard:byte;
  mfrom:byte;
  mto:byte;
  mIndex:integer;
  cMove:string[5];
  GameN:longint;
  bIsMain:boolean;
 end;

type tPosSaveBuf=array [0..POS_SAVE_BUF_SIZE] of tPosDBrec;

type PposDBRec=^TposDBrec;

type tPosDB=class(TList)
 public
  searchrec:tposdbrec;
  psearchrec:pposdbrec;
  lastfound:longint;
  defaultInsertN:longint;
  incDefaultInsertN:integer;
  phead:pbpgnlist;
  cLastError:string;

  constructor Create;
  function add(p:pposdbrec):integer;
  procedure delete(n:longint);
  procedure clear;

  function Save(fname:string):boolean;
  function Load(fname:string):boolean;
  function Append(fname:string;nStart:longint):boolean;
  procedure RemoveGame(Gamen,nPosition:longint);

  procedure emptyrecord(var rec:TPosDBRec);
  function getGameN(i:longint):longint;
  function getboardnum(i:longint):integer;
  function findPosInGame(GameN,nmin,nmax:longint):boolean;
  procedure shiftgames(nStart,nShift:longint);
end;

tDBPosNext=Record
 nFromTo:integer;
 cMove:string[5];
 nWin:integer;
 nDraw:integer;
 nLoss:integer;
 nGame:integer;
 Perc:real;
 nPerf:integer;
 nRwin:integer;
 nrDraw:integer;
 nrLoss:integer;
 nSelfRat:integer;
 nPardRat:integer;
 nOppRat:integer;
end;

tDBPosNextAr=array[1..Max_Next] of tdbPosNext;

procedure BoardtoInt(var board:boardarray; var intboard:TintBoard);
procedure cleardbnext(var rec:tdbposnext);
function comprec(r1,r2:pposdbrec):integer;
function compPosition(r1,r2:pposdbrec):integer;
function GetPositionCount(fname:string):integer;
implementation

function piecetoint(piece:char):longint;
begin
 case piece of
    WHITE_PAWN: result:=1;
    WHITE_ROOK: result:= 2;
    WHITE_KNIGHT: result:=3;
    WHITE_BISHOP: result:=4;
    WHITE_QUEEN : result:=5;
    WHITE_KING  : result:=6;

    BLACK_PAWN : result:=7;
    BLACK_ROOK :result:=8;
    BLACK_KNIGHT:result:=9;
    BLACK_BISHOP:result:=10;
    BLACK_QUEEN :result:=11;
    BLACK_KING  :result:=12;

    EMPTY_SQUARE: result:=0;
 end;
end;

procedure BoardtoInt(var board:boardarray; var intboard:TintBoard);
var i,j:integer;
tmp:longint;
begin
 for i:=1 to 8 do
 begin
  intboard[i]:=0;
  for j:=1 to 8 do
  begin
   tmp:=piecetoint(board[i,j]);
   tmp:=tmp shl (4*(j-1));
   inc(intboard[i],tmp);
  end;
 end;
end;

constructor tposdb.Create;
begin
 inherited Create;
 psearchrec:=@searchrec;
 defaultInsertN:=-1;
 incdefaultInsertN:=1;
 cLastError:='';
end;

function tposdb.add(p:pposdbrec):integer;
var pnew:pposdbrec;
begin
 try
  New(pnew);
  If defaultInsertN<0 then Result:=inherited add(pnew)
  else begin
   inherited insert(defaultInsertN,pnew);
   inc(defaultInsertN,incDefaultInsertN);
  end;
 except on EOutOfMemory do
  result:=-1;
 end;
 pnew^:=p^;
end;

procedure tposdb.delete(n:longint);
var p:pPosDBrec;
begin
 p:=items[n];
 try
  dispose(p);
 except on EInvalidPointer do begin end;
 end;
 inherited delete(n);
end;

procedure tposdb.clear;
var i:longint;
begin
 for i:=0 to count-1 do
 try
  dispose(items[i]);
 except on EInvalidPointer do begin end;
 end;
 inherited clear;
end;

function tposdb.Save(fname:string):boolean;
var i,hc,k,listsz:longint;
f:file;
psave:^tPosSaveBuf;
p:pPosDBrec;
begin
  //allocating the memory for buffer
 try
  New(psave);
 except
  Werror('Not enough memory');
  Result:=false;
  exit;
 end;

 AssignFile(f,fname);
 try
  Rewrite(f,1);
 except
  CloseFile(f);
  Werror('Can not write to file ' + fname);
  dispose(psave);
  Result:=false;
  exit;
 end;
 try
  listsz:=count; hc:=0;
  BlockWrite(f,listsz,sizeof(listsz));
  //save the number or records in the position list
  for i:=0 to listsz-1 do
  begin
   p:=items[i];
   psave[hc]:=p^;
   inc(hc,1);
   if (hc>POS_SAVE_BUF_SIZE) then
   begin
    blockwrite(f,psave^,sizeof(tPosSaveBuf));
    hc:=0;
   end;
  end;

 //flush buffer at the end;
  blockwrite(f,psave^,hc*sizeof(tPosDBrec));
 finally
  CloseFile(f);
 end;
 dispose(psave);
 Result:=true;
end;

function tposdb.Load(fname:string):boolean;
var i,k,rc,hr:longint;
f:file;
pload:^tpossavebuf;
bufsize,listsz,bufmemsize:longint;
p:pposDBrec;
begin
 cLastError:='';
 //allocate memory
 try
  new(pload);
 except
  cLastError:='Not enough memory.'+EOL_STR+'Positional database was not loaded';
  Result:=False;
  exit;
 end;

 //open header file
 AssignFile(f,fname);
 try
  reset(f,1);
 except
  cLastError:='Can not open file '+fname+EOL_STR+'Positional database was not loaded';
  dispose(pload);
  Result:=False;
  exit;
 end;

 try
  blockread(f,listsz,sizeof(listsz),rc);

  bufmemsize:=sizeof(tpossavebuf); //memory size of the buffer
  bufsize:=POS_SAVE_BUF_SIZE+1;    //number of rec in the buffer

  //since the tpossavebuf is 0 based array

  clear;
  hr:=0; i:=1;
  while(listSz>=i*bufsize) do
  begin
   //read a chunk of position file;
   blockread(f,pload^,bufmemsize,rc);
   if rc<>bufmemsize then // corrupt file
   begin
    cLastError:='Corrupted position file ' + fname+EOL_STR+'Positional database was not loaded';
    dispose(pload);
    clear;
    Result:=false; exit;
   end;

   //populate alist from the chunk
   for k:=0 to bufsize-1 do
   begin
    if add(@pload[k])=-1 then //not enough memory
    begin
     cLastError:='Not enough memory'+EOL_STR+'Positional database was not loaded';
     dispose(pload);
     clear;
     Result:=false;
     exit;
    end;
    inc(hr,1);
   end;
   inc(i,1);
  end;

  //read the last chunk of positions
  blockread(f,pload^,(listsz-hr)*sizeof(tPosDBrec),rc);
  if ((listsz-hr)*sizeof(tPosDBrec)<>rc) then
  begin
   cLastError:='Corrupted position file ' + fname+EOL_STR+'Positional database was not loaded';
   dispose(pload);
   clear;
   Result:=false; exit;
  end;

  //populate alist from the chunk
  for k:=0 to listsz-hr-1 do
  begin
   if add(@pload[k])=-1 then //not enough memory
   begin
    cLastError:='Not enough memory'+EOL_STR+'Positional database was not loaded';
    dispose(pload);
    clear;
    Result:=false;
    exit;
   end;
//   p:=last;
//   p^:=pload[k];
  end;

 finally
  CloseFile(f);
 end;
 dispose(pload);
 Result:=true;
end;

function compPosition(r1,r2:pposdbrec):integer;
var res:integer;
i:integer;
begin
 for i:=1 to 8 do
 begin
  res:=sign(r1^.position[i]-r2^.position[i]);
  if res<>0 then begin result:=res; exit; end;
 end;
 Result:=0;
// result:=(r1^.cboard-r2^.cboard) div 2;

end;

function comprec(r1,r2:pposdbrec):integer;
var res:integer;
begin
 res:=compPosition(r1,r2);
 if res<>0 then begin result:=res;exit;end;

 res:=sign(r1^.GameN-r2^.gameN);
 if res<>0 then begin result:=res; exit; end;

 res:=sign(r1^.mfrom-r2^.mfrom);
 if res<>0 then begin result:=res; exit; end;

 res:=sign(r1^.mTo-r2^.mTo);
 if res<>0 then begin result:=res; exit; end;

 result:=sign(r1^.cBoard-r2^.cBoard);

end;

procedure tposdb.emptyrecord(var rec:TposDBRec);
var i,j:integer;
begin
 for i:=1 to 8 do rec.position[i]:=0;
// for i:=1 to 2 do
// for j:=1 to 5 do
//  rec.holding[i,j]:=0;
 rec.timedif:=0;
 rec.cBoard:=0;
 rec.mfrom:=0;
 rec.mto:=0;
 rec.mIndex:=0;
 rec.cMove:='';
 rec.GameN:=-1;
 rec.bIsMain:=true;
end;



function tposdb.getGameN(i:longint):longint;
var p:PposDbrec;
begin
  p:=items[i];
  Result:=p^.GameN;
end;

function tposdb.getboardnum(i:longint):integer;
begin
 result:=Pposdbrec(items[i])^.cBoard;
end;

function tposdb.findPosInGame(GameN,nmin,nmax:longint):boolean;
var n:longint;
begin
 lastFound:=-1;
 if nmin>count-1 then begin Result:=false; exit;end;
 if nmax>count-1 then nmax:=count-1;
 n:=nmin;
 if n<0 then Begin Result:=false;exit;end;
 while GetGameN(n)=GameN do
 begin
  if compPosition(pposdbRec(items[n]),@searchrec)=0 then
  begin Result:=true;LastFound:=n;exit;end;
  inc(n,1);
  if n>count-1 then Begin Result:=false; exit;end;
 end;
 Result:=false;
end;

procedure cleardbnext(var rec:tdbposnext);
begin
 rec.nFromTo:=0;
 rec.cMove:='';
 rec.nWin:=0;
 rec.nDraw:=0;
 rec.nLoss:=0;
 rec.nGame:=0;
 rec.nPerf:=0;
 rec.nRwin:=0;
 rec.nrDraw:=0;
 rec.nrLoss:=0;
 rec.nSelfRat:=0;
 rec.nPardRat:=0;
 rec.nOppRat:=0;
end;

procedure tposdb.RemoveGame(Gamen,nPosition:longint);
begin
 if ((nposition<0) or (nposition>count-1)) then exit;
 while GetGameN(nPosition)=GameN do
 begin
  delete(nPosition);
  if nPosition>count-1 then exit;
 end;
end;

procedure tposdb.shiftgames(nstart,nshift:longint);
var i:longint;
begin
 if ((nstart<0) or (nstart>count-1)) then exit;
 for i:=nstart to count-1 do
   dec(Pposdbrec(items[i])^.GameN,nshift);
end;

function tposdb.Append(fname:string;nStart:longint):boolean;
 var i,hc,k,listsz,nPrevCount:longint;
f:file;
psave:^tPosSaveBuf;
p:pPosDBrec;
begin
  //allocating the memory for buffer
 try
  New(psave);
 except
  Werror('Not enough memory');
  Result:=false;
  exit;
 end;

 AssignFile(f,fname);
 try
  if FileExists(fname) then
  begin
   Reset(f,1); Blockread(f,nPrevCount,sizeof(nprevcount));
  end
  else begin nPrevCount:=0; Rewrite(f,1);end;
  seek(f,0);
 except
  CloseFile(f);
  Werror('Can not write to file ' + fname);
  dispose(psave);
  Result:=false;
  exit;
 end;

 try
  listsz:=count+nPrevCount; hc:=0;
  BlockWrite(f,listsz,sizeof(listsz));
  //save the number or records in the position list
  seek(f,FileSize(f));
  for i:=0 to count-1 do
  begin
   p:=items[i];
   psave[hc]:=p^;
   inc(psave[hc].GameN,nStart);
   inc(hc,1);
   if (hc>POS_SAVE_BUF_SIZE) then
   begin
    blockwrite(f,psave^,sizeof(tPosSaveBuf));
    hc:=0;
   end;
  end;

 //flush buffer at the end;
  blockwrite(f,psave^,hc*sizeof(tPosDBrec));
 finally
  CloseFile(f);
 end;
 dispose(psave);
 Result:=true;
end;

function getPositionCount(fname:string):longint;
var f:file;
nC:longint;
begin
 If not FileExists(fname) then Begin result:=0;exit;end;
 nc:=0;
 AssignFile(f,fname);
 try
  Reset(f,1);
  seek(f,0);
  Blockread(f,nc,sizeof(nc));
 finally
  CloseFile(f);
 end;
 Result:=nc;
end;


end.
