(*============================================================================*)
(* toups.PAS -- Toups vector set                                              *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002 Fermy                               *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 2002.02.02: First construction.                                            *)
(*                                                                            *)
(*============================================================================*)

unit toups;

interface
uses graphics,windows,classes;

const
nwhtfill=clwhite;
nwhtoutline=clblack;
nblkoutline=clgray;
nblkfill=clblack;


var
    aPawnX:       ARRAY[0..41] OF Integer =
                  (12,13,13,14,14,15,15,19,19,17,18,17,17,18,18,21,22,20,20,23,
                   26,29,29,27,28,31,31,32,32,31,32,30,30,34,34,35,35,36,36,37,
                   37,12);
    aPawnY:       ARRAY[0..41] OF Integer =
                  (37,36,34,33,32,31,30,26,25,23,22,21,18,17,16,13,13,11, 8, 5,
                    5, 8,11,13,13,16,17,18,21,22,23,25,26,30,31,32,33,34,36,37,
                   42,42);

    aRookX:       ARRAY[0..30] OF Integer =
                  ( 9,39,10,10,14,34,15,15,34,14,10,10,16,16,21,21,28,28,33,33,
                   39,39,35,34,34,35,39,39,40,40, 9);
    aRookY:       ARRAY[0..30] OF Integer =
                  (39,39,39,35,31,31,31,18,18,18,14, 6, 6,11,11, 6, 6,11,11, 6,
                    6,14,18,18,31,31,35,39,39,44,44);

    aKnightX:     ARRAY[0..68] OF Integer =
                  (14,21,21,22,22,23,23,24,24,25,25,25,24,24,23,22,21,21,20,19,
                   18,14,14,11, 9, 9,10, 9, 9, 6, 3, 3, 4, 4, 8, 9, 9,13,11,10,
                   12, 9,12,17,12,11,11,12,13,14,15,19,19,19,17,19,20,23,25,23,
                   29,31,38,39,42,42,43,43,14);
    aKnightY:     ARRAY[0..68] OF Integer =
                  (41,34,33,32,31,30,23,22,21,20,18,20,21,22,23,23,24,25,25,26,
                   26,30,31,34,34,31,29,31,33,33,30,28,27,25,17,16,15,15,17,17,
                   15,15,12, 9,12, 7, 4, 5, 5, 6, 6,10,12,10, 8, 8, 4, 8,12, 8,
                    8, 9,16,18,27,33,33,43,44);

    aBishopX:     ARRAY[0..76] OF Integer =
                  ( 5, 5, 7,10,18,19,19,29,14,18,30,18,18,17,19,21,24,24,24,20,
                   28,24,24,27,28,29,31,29,28,27,21,19,17,13,13,14,14,18,19,21,
                   22,23,24,23,21,21,23,25,27,27,25,24,25,26,27,29,30,34,34,35,
                   35,30,30,34,29,29,30,38,41,43,43,42,29,24,19, 6, 6);
    aBishopY:     ARRAY[0..76] OF Integer =
                  (43,41,39,38,38,37,35,35,35,31,31,31,29,28,26,25,25,15,19,19,
                   19,19,25,25,26,26,28,26,26,25,25,26,28,24,19,18,17,13,13,11,
                   11,10,10,10, 8, 6, 4, 4, 6, 8,10,10,10,11,11,13,13,17,18,19,
                   24,29,31,35,35,37,38,38,39,41,43,44,44,39,44,44,44);

    aQueenX:      ARRAY[0..76] OF Integer =
                  (12,12,24,36,24,12,12, 9,17,24,31,39,31,24,17, 9, 8, 8,18,30,
                   40,30,18, 8, 8, 5, 5, 3, 5, 7, 5, 8, 9,13,13,13,11,13,15,13,
                   19,19,19,22,24,22,24,26,24,27,29,29,29,35,33,35,37,35,35,35,
                   39,40,43,41,43,45,43,43,40,40,40,39,36,36,36,34,14);
    aQueenY:      ARRAY[0..76] OF Integer =
                  (42,39,37,39,37,39,37,34,34,31,34,34,34,31,34,34,33,30,26,26,
                   30,26,26,30,26,20,14,12,10,12,14,17,20,24,28,10, 8, 6, 8,10,
                   22,26,22,15, 8, 6, 4, 6, 8,15,22,26,22,10, 8, 6, 8,10,28,24,
                   20,17,14,12,10,12,14,20,26,30,33,34,37,39,42,44,44);

    aKingX:       ARRAY[0..81] OF Integer =
                  (15,13,12,10, 9, 9,16,17,29,30,37,30,29,17,16, 9, 9,13,14,32,
                   33,37,33,32,14,13, 9, 9, 8, 8, 3, 3, 4, 4, 6, 7, 8,13,14,16,
                   19,23,23,23,27,23,19,19,22,22,19,19,22,22,25,25,28,28,25,25,
                   28,28,31,33,34,39,40,41,43,43,44,44,39,39,38,38,37,35,34,32,
                   31,16);
    aKingY:       ARRAY[0..81] OF Integer =
                  (43,43,42,42,41,39,39,38,38,39,39,39,38,38,39,39,35,35,34,34,
                   35,35,35,34,34,35,35,32,31,30,25,20,19,18,16,16,15,15,16,16,
                   19,26,34,26,19,26,19,15,12, 9, 9, 7, 7, 3, 3, 7, 7, 9, 9,12,
                   15,19,16,16,15,15,16,16,18,19,20,25,30,31,32,41,42,42,43,43,
                   44,44);


procedure DrawPawn(TC: TCanvas; nPieceRatio:real);
procedure drawking(TC: TCanvas; nPieceRatio:real);
procedure drawrook(TC: TCanvas; nPieceRatio:real);
procedure drawbishop(TC: TCanvas; nPieceRatio:real);
procedure drawqueen(TC: TCanvas; nPieceRatio:real);
procedure drawknight(tc: tcanvas;npieceratio:real);

procedure drawblackpawn(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawblackknight(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawblackbishop(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawblackqueen(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawblackking(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawblackrook(tc: tcanvas; npiecesize:integer;npieceratio:real);

procedure drawwhitepawn(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawwhiterook(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawwhiteknight(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawwhitebishop(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawwhitequeen(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawwhiteking(tc: tcanvas; npiecesize:integer;npieceratio:real);

procedure drawpawnmask(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawrookmask(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawknightmask(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawbishopmask(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawqueenmask(tc: tcanvas; npiecesize:integer;npieceratio:real);
procedure drawkingmask(tc: tcanvas; npiecesize:integer;npieceratio:real);


implementation

PROCEDURE  DrawPawn(TC: TCanvas; nPieceRatio:real);
VAR
    nC:    Integer;
    aPoly: ARRAY[0..41] OF TPoint;
BEGIN
    FOR nC := 0 TO 41 DO
    BEGIN
        aPoly[nC].X := Trunc(aPawnX[nC] * nPieceRatio);
        aPoly[nC].Y := Trunc(aPawnY[nC] * nPieceRatio);
    END;

    TC.Polygon(aPoly);
END;

PROCEDURE  DrawRook(TC: TCanvas; nPieceRatio:real);
VAR
    nC:    Integer;
    aPoly: ARRAY[0..30] OF TPoint;
BEGIN
    FOR nC := 0 TO 30 DO
    BEGIN
        aPoly[nC].X := Trunc(aRookX[nC] * nPieceRatio);
        aPoly[nC].Y := Trunc(aRookY[nC] * nPieceRatio);
    END;

    TC.Polygon(aPoly);
END;

PROCEDURE  DrawKnight(TC: TCanvas; nPieceRatio:real);
VAR
    nC:    Integer;
    aPoly: ARRAY[0..68] OF TPoint;
BEGIN
    FOR nC := 0 TO 68 DO
    BEGIN
        aPoly[nC].X := Trunc(aKnightX[nC] * nPieceRatio);
        aPoly[nC].Y := Trunc(aKnightY[nC] * nPieceRatio);
    END;

    TC.Polygon(aPoly);
END;

PROCEDURE  DrawBishop(TC: TCanvas; nPieceRatio:real);
VAR
    nC:    Integer;
    aPoly: ARRAY[0..76] OF TPoint;
BEGIN
    FOR nC := 0 TO 76 DO
    BEGIN
        aPoly[nC].X := Trunc(aBishopX[nC] * nPieceRatio);
        aPoly[nC].Y := Trunc(aBishopY[nC] * nPieceRatio);
    END;

    TC.Polygon(aPoly);
END;

PROCEDURE  DrawQueen(TC: TCanvas; nPieceRatio:real);
VAR
    nC:    Integer;
    aPoly: ARRAY[0..76] OF TPoint;
BEGIN
    FOR nC := 0 TO 76 DO
    BEGIN
        aPoly[nC].X := Trunc(aQueenX[nC] * nPieceRatio);
        aPoly[nC].Y := Trunc(aQueenY[nC] * nPieceRatio);
    END;

    TC.Polygon(aPoly);
END;

PROCEDURE  DrawKing(TC: TCanvas; nPieceRatio:real);
VAR
    nC:    Integer;
    aPoly: ARRAY[0..81] OF TPoint;
BEGIN
    FOR nC := 0 TO 81 DO
    BEGIN
        aPoly[nC].X := Trunc(aKingX[nC] * nPieceRatio);
        aPoly[nC].Y := Trunc(aKingY[nC] * nPieceRatio);
    END;

    TC.Polygon(aPoly);
END;

PROCEDURE  DrawBlackPawn(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nBlkFill;
    DrawPawn(tc,npieceratio);

    TC.Pen.Color   := nBlkOutline;
    TC.Brush.Style := bsClear;
    DrawPawn(tc,npieceratio);
END;

PROCEDURE  DrawBlackRook(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nBlkFill;
    DrawRook(tc,npieceratio);

    TC.Pen.Color   := nBlkOutline;
    TC.Brush.Style := bsClear;
    DrawRook(tc,npieceratio);
END;

PROCEDURE  DrawBlackKnight(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nBlkFill;
    DrawKnight(tc,npieceratio);

    TC.Pen.Color   := nBlkOutline;
    TC.Brush.Style := bsClear;
    DrawKnight(tc,npieceratio);
END;

PROCEDURE  DrawBlackBishop(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nBlkFill;
    DrawBishop(tc,npieceratio);

    TC.Pen.Color   := nBlkOutline;
    TC.Brush.Style := bsClear;
    DrawBishop(tc,npieceratio);
END;

PROCEDURE  DrawBlackQueen(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nBlkFill;
    DrawQueen(tc,npieceratio);

    TC.Pen.Color   := nBlkOutline;
    TC.Brush.Style := bsClear;
    DrawQueen(tc,npieceratio);
END;

PROCEDURE  DrawBlackKing(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nBlkFill;
    DrawKing(tc,npieceratio);

    TC.Pen.Color   := nBlkOutline;
    TC.Brush.Style := bsClear;
    DrawKing(tc,npieceratio);
END;

PROCEDURE  DrawWhitePawn(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nWhtFill;
    DrawPawn(tc,npieceratio);

    TC.Pen.Color   := nWhtOutline;
    TC.Brush.Style := bsClear;
    DrawPawn(tc,npieceratio);
END;

PROCEDURE  DrawWhiteRook(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nWhtFill;
    DrawRook(tc,npieceratio);

    TC.Pen.Color   := nWhtOutline;
    TC.Brush.Style := bsClear;
    DrawRook(tc,npieceratio);
END;

PROCEDURE  DrawWhiteKnight(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nWhtFill;
    DrawKnight(tc,npieceratio);

    TC.Pen.Color   := nWhtOutline;
    TC.Brush.Style := bsClear;
    DrawKnight(tc,npieceratio);
END;

PROCEDURE  DrawWhiteBishop(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nWhtFill;
    DrawBishop(tc,npieceratio);

    TC.Pen.Color   := nWhtOutline;
    TC.Brush.Style := bsClear;
    DrawBishop(tc,npieceratio);
END;

PROCEDURE  DrawWhiteQueen(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nwhtfill;
    DrawQueen(TC,npieceratio);

    TC.Pen.Color   := nwhtoutline;
    TC.Brush.Style := bsClear;
    DrawQueen(TC,npieceratio);
END;

PROCEDURE  DrawWhiteKing(TC: TCanvas; nPieceSize:integer;nPieceRatio:real);
BEGIN
    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Brush.Color := nwhtfill;
    DrawKing(TC,npieceratio);

    TC.Pen.Color   := nwhtoutline;
    TC.Brush.Style := bsClear;
    DrawKing(tc,npieceratio);
END;

PROCEDURE  DrawPawnMask(tc: tcanvas; npiecesize: integer; npieceratio: real);
BEGIN
    TC.Pen.Color   := clWhite;
    TC.Brush.Color := clWhite;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    DrawPawn(tc,npieceratio);
END;

PROCEDURE  DrawRookMask(tc: tcanvas; npiecesize: integer; npieceratio: real);
BEGIN
    TC.Pen.Color   := clWhite;
    TC.Brush.Color := clWhite;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    DrawRook(tc,npieceratio);
END;

PROCEDURE  DrawKnightMask(tc: tcanvas; npiecesize: integer; npieceratio: real);
BEGIN
    TC.Pen.Color   := clWhite;
    TC.Brush.Color := clWhite;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    DrawKnight(tc,npieceratio);
END;

PROCEDURE  DrawBishopMask(tc: tcanvas; npiecesize: integer; npieceratio: real);
BEGIN
    TC.Pen.Color   := clWhite;
    TC.Brush.Color := clWhite;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    DrawBishop(tc,npieceratio);
END;

PROCEDURE  DrawQueenMask(tc: tcanvas; npiecesize: integer; npieceratio: real);
BEGIN
    TC.Pen.Color   := clWhite;
    TC.Brush.Color := clWhite;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    DrawQueen(tc,npieceratio);
END;

PROCEDURE  DrawKingMask(tc: tcanvas; npiecesize: integer; npieceratio: real);
BEGIN
    TC.Pen.Color   := clWhite;
    TC.Brush.Color := clWhite;
    TC.Brush.Style := bsSolid;
    TC.Polygon([ Point(             0,              0),
                 Point(nPieceSize - 1,              0),
                 Point(nPieceSize - 1, nPieceSize - 1),
                 Point(             0, nPieceSize - 1),
                 Point(             0,              0) ]);

    TC.Pen.Color   := clBlack;
    TC.Brush.Color := clBlack;
    DrawKing(tc,npieceratio);
END;



end.
