(*============================================================================*)
(* BPGN40K.PAS -- BPGN object (40000)                                         *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: 1) Added previous next move support.                           *)
(*             2) Added control tag <Occurrence:xxx> into comment tag for     *)
(*                extract function.                                           *)
(* 1998.11.05: Added into bShowRatings, bShowTime, bShowInc, and bShowResult  *)
(*             into GetHeaderItem().                                          *)
(* 1998.11.06: Added BPGNIsMain[].                                            *)
(* 1998.11.07: 1) Added new file.                                             *)
(*             2) Added new game.                                             *)
(*             3) Added duplicate game.                                       *)
(*             4) Added pack file.                                            *)
(* 1998.11.09: 1) Added show game # under game list options.                  *)
(* 1998.11.11: 1) Changed the maximum # of comment characters from 0 to 64000.*)
(* 1998.11.12: Added show event under game list options.                      *)
(* 1998.11.13: Added opponent, bpgn filename, and game # into extract's       *)
(*             occurrence tag.                                                *)
(* 1998.11.14: 1) Substituted comment's carriage return symbol with ASCII 254.*)
(*             2) Changed the 'vs.' to 'vs' inside game list.                 *)
(*             3) Changed GetActualGameText() to reflect actual game.         *)
(* 1998.11.17: Patched a bug in LoadGame() where the first comment caused an  *)
(*             error message.                                                 *)
(* 1998.11.23: Added generate destination occurrence comment.                 *)
(* 1999.01.20: 1) Added code to read in Crazyhouse games (determined by event *)
(*               tag in BPGN file: [Event "fics unrated crazyhouse match"]).  *)
(*               If it's a crazyhouse game then the captured piece's color    *)
(*               will be reversed and into player's piece buffer instead of   *)
(*               going to player partner's piece buffer as in regular         *)
(*               bughouse games.                                              *)
(*            2) Changed version ID to 1.13 beta.                             *)
(* 2002.04.09: 1) Scrapped memory models. Retained 40K memory model           *)
(*             2) Transferred routines from BPGN40K so that files             *)
(*                bpgn2K--bpgn40K are no longer needed                        *)
(* 2002.04-05: 1) Added Setup/BFen Tag support                                *)
(*             2) Added code for non-positional searches(filter)              *)
(*             3) Added code to support filtered gamelists (navigating/saving)*)
(*             4) Added code to determine the reason of game end              *)
(*             5) Added splitbpgn function that separates bug and zh games    *)
(* 2004.11.06: 1) Changed code of getline,getactualgame,getgame,savegame      *)
(*                to accomodate the weird behaviour of eof() . EOF() is       *)
(*                returnin false if the unread chunk in a file is smaller     *)
(*                than record size.                                           *)
(* 2004.11.11  1) Replaced fixed arrays in bpgn object with the list of       *)
(*                records alist. Now the memory is allocated dynamically, the *)
(*                restriction of 10000 games per bpgn is lifted.              *)
(*             2) Changed the various function to account for the fact that   *)
(*                formerly indexes in all arrays in tbpgn started from 1, but *)
(*                index in alist starts from 0 (as in any list).              *)
(*             3) LoadHeader no longer checks whether BPGN file has more than *)
(*                MAX_GAMES games                                             *)
(*             4) PackGameList completely rewritten                           *)
(* 2004.11.13  1) Added tbpgn.DBtoHeader, tbpgn.LoadDBheaders,                *)
(*                tbpgn.gametoheader, Tbpgn.SaveHeaders to enable work with   *)
(*                *.bh files = databases of headers.                          *)
(*             2) Changed tbpgn.savefiltered to start filtering from 0        *)
(*             3) Changed BPGN.GetHeaderItem to start game numbers from 1     *)
(*             4) Changed bgpn.setPosition to NOT set bgameChange             *)
(* 2004.11.15  5) Rewrote bpgn.packgamelist to fix bugs                       *)
(* 2004.11.16  1) Modified TBPGN.LoadBPGNMove, TBPGN.MakeBoardMove to call    *)
(*                tchess.adddropPiece instead of SetPieces in order to allow  *)
(*                aerial drops                                                *)
(*             2) Fixed bug in TBPGN.LoadBPGNMove, TBPGN.MakeBoardMove that   *)
(*                caused pawns captured enpasant to disappear without a trace *)
(*             3) Modified tbpgn.exec move to allow negative holdings         *)
(* 2004.11.18  1) Added functions tbpgn.SavePosDb, tbpgn.AddGameToPosDb,      *)
(*                tbpgn.gettimedif, tbpgn.BPGNtoPosRec                        *)
(*             2) Changed tbpgn.getHeaderItem back to start numbering from 0  *)
(* 2004.11.20  1) Added  tbpgn.addChildrenPos and rewrote tbpgn.addgametoPosDB*)
(*             2) renamed tbpgn.setfilter to tbpgn.setNonPosFilter;  added    *)
(*                tbpgn.setfilter and tbpgn.setPosFilter                      *)
(* 2004.11.20  1) setposFilter is not used anymore                            *)
(*             2) addchildren calls tposdb.add instead of insert because now  *)
(*                new records are appended and not sorted by position         *)
(*             3) Patched bug in getgame caused by the fact that strlen does  *)
(*                not work for char arrays if the array is full               *)
(* 2004.11.27  1) modified addchildrenpos to save also tdbposrec.bismain      *)
(*             2) Added routines setstats,setnonposstats,setstats to handle   *)
(*                clicks on the Move/stats button in fform                    *)
(*             3) Added bIsBugDb -> true if the opened db is a bughouse db;   *)
(*                NextDBList -> contains the list of all next moves in db in  *)
(*                a current position along with stats; nCurrentWorB ->        *)
(*                1 if the last search was in white db, black otherwise       *)
(*             4) added routine checkDbtype, checks if it's a bug db or zh   *)
(* 2004.11.28  Modified LoadBPGNMove, MakeBoardMove to check for bairdrop     *)
(* 2004.11.29  1) Added function updateGameToPosDB                            *)
(*             2) Modified savegame to update also a db (if it is opened)     *)
(*             3) Modified AddChildrenPos to use defaultInsertN and           *)
(*                incDefaultInsertN in order to use it for inserting a game   *)
(*                into the positional db file                                 *)
(* 2004.12.01  Modified CheckGameSaveChanges to account for the fact that     *)
(*             indexes in alist now run from 0 to nHeadercount-1              *)
(* 2004.12.03  1) Added CompareNextMove, sortmovestats to sort the moves in   *)
(*             stats window in filter form                                    *)
(*             2) Extracted part of setposstat into displayPosStats           *)
(*             3) Fixed bug in setposstat that made all drops on the same     *)
(*                square identical to the programm                            *)
(* 2004.12.06 1) Added updatedb, modification to a db now saved to disk       *)
(*            2) Fixed bug in savegame identical to 2004.11.20 3)             *)
(*            3) Fixed bug in setposstat, first game didn't affect performance*)
(*            4) Added progress bar to time consuming functions               *)
(*            5) Modified savegame and packfile to update databases in memory *)
(*            6) Completely rewritten packgamelist                            *)
(*            7) Modified AddChildrenPos to save terminal positions           *)
(*            8) Modified format of bh file to ensure sync between bpgn and bh*)
(*                files. Added code to check db integrity to LoaddbHeaders    *)
(*            9) Modified LoadGame to load searched position if the gamelist  *)
(*               is the result of positional search                           *)
(*            10) Modified addchildrenpos to save nothing on board2 for zh db *)
(* 2004.12.13 1) Fixed bug in loaddbheaders that caused headers to be shifted *)
(*               by 1 character                                               *)
(*            2) Added killdoubles, isdouble, shiftgamesdb to implement eli-  *)
(*               mination of doubles                                          *)
(*            3) added gamexstart instead of pdblist[a].findmingamex to speed *)
(*               up search. Changed format of bh file to include apositionW/B *)
(* 2004.12.14 1) Fixed bug in getline that caused games to be saved incorectly*)
(*               into other files                                             *)
(*            2) Progress bar now is shown only if files are large            *)
(*            3) Modified RebuildHeaders to work with filters                 *)
(* 2004.12.15 1) Modified Duplicategame to work with db                       *)
(* 2004.12.17 1) Fixed bug in SaveHeaders that resulted in shifted headers in *)
(*               the first game                                               *)
(*            2) Modified AppendGame to append to databases as well           *)
(*            3) Modified SaveFiltered to save to databases as well           *)
(*            4) Addded AppendGameDB                                          *)
(*            5) Fixed bug in FilterMatch that caused incorrect filter        *)
(*               results if Player1 is non-empty                              *)
(* 2004.12.20 1) Changed loadgame, getheaderitem, addgametoposdb to number    *)
(*               games from 1                                                 *)
// 2005.01.03 Modified DisplayPosStats to get rid of unused rows in tree grid
// 2005.01.07 Added underscored functions to be called by DoLongRun. They call
//            non-underscored functions with the same name which are now private
//            (e.g. packFile)
// 2005.01.10 1) Added FilterTag, unTagFiltered
//            2) Added FilterBroken, isBroken, getRsideResult, deleteFiltered,
//               SelectFiltered
//            3) Modified SetTag to set file change only if Delete tag is set
// 2005.01.12 1) Added SynchFilter that makes sure that board is in synch with tree
//            2) Added SynchResultfromtag and SynchResultFromString to repair
//               broken result tags/strings
// 2005.01.16 1) Modified isBroken to also check if confirmation string(i.e 1-0
//               after {jtp checkmated} matches the result tag and string
//            2) Modified SynchResultfromtag to set the confirmation string
//            3) Added SaveUpdates that should do massive header and result string
//               changes
// 2005.01.25 1) Added GetUpdatedGame that puts updated game score into cGameline
//               Works for massive header changes ulike getActualGame
//            2) Modified SaveUpdates to call GetUpdatedGame instead of getActualgame
//            3) split getActualgame into itself and GetUpdatedHeader
// 2005.01.29 1) Added bpgn.bAbortLong and checks for it in fuctions performing long
//               operations
//            2) Fixed bug in SaveposDb. Errors reported by other bpgns routines
//               interfered with report of damaged games reported by this function
// 2005.02.09 1) Fixed bug in SynchResultfromString: if the result string is
//               incorrect or it is {*}*, the result tag was set incorrectly
//            2) Removed calls setting alist.capacity in attempt to solve stack
//               overflow on win98. Temporarily worked, yet apparently the culprit
//               was in the TmainForm.FormShow, with the line
//               if bFilterSt then fform.CheckBox14.Checked:=true;
// 2005.02.10 1) Changed internal representation of time to milliseconds.
//            2) Added support for Lag tag
//            3) Added support for decimal bpgn time (e.g. {5.2}).
//            4) To do 1)-3) modified parsing in LoadBPGNMove, added getNextLag
//            5) Added BPGNLag to store lag times and added 4 lag fields to bpgnrecord
//            6) Added GetCurrentLag that returns lag on the last move
//            7) Added GetCurrentWlag/GetCurrentBlag that reports cumulative lag
//            8) Modified SaveMove to save lag
//            9) Added variable bLagTag that is true if current game has lag tag
// 2005.02.15 1) Modified GetBpgnPosition to return also times and side to move
//            2) Modified GetLine to work with Unix style linebreaks
// 2005.02.17 1) Modified LoadBPGNmove to catch time inconsistencies (if
//               ini1.ini.btimerror is true)
//            2) Fixed bug in getline introduced in the previous version: All
//               tags after the result tag were lost.
// 2005.02.22 1) Fixed parsing bug in getLine: Access violation
//            2) Modified LoadBpgnStuff, LoadGame, LoadBPGNmove to treat {X:} as
//               an unknown bpgn tag, i.e. to skip it, but not signal error
//            3) Modified PackFile to improve progress display on databases
//            4) Modified database synch check code in LoadDBHeaders: Fixes the
//               problem that timezone change also changes file timestamp in Windows
// 2005.02.28 1) Modified FilterMatch to do more sofisticated rating search and
//               search selected games
//            2) Added InvertTag
// 2005.03.01 1) Modified parsfen1,parsefen2 to work with non-integer time
//            2) Fixed bug in MakeBoardMove: lag appeared out of nowhere when move
//               was manually added
// 2005.03.02 Improved generation of new bpgns: Modified NewGameString and
//            AppendNewGame to take pbpgnrec argument. Fixes bug: Invalid result
//            string after adding a new game with New game and immediately editing it
(*============================================================================*)

UNIT Bpgnx;

INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    Menus, ExtCtrls, StdCtrls,syncobjs,
    BOARD, CHESS, GLOBAL, MISC, TOOLBOX,INI1,math,bpgnlist,posdb1,
    filter, viewact,progressdb;


TYPE
    tStrBoolRec=record
    s:string;
    b:boolean;
    end;
    tboardCount=array [1..2] of integer;
    plistbox=^Tlistbox;
    TBPGN = CLASS(TObject)
        CONSTRUCTOR Init;
        DESTRUCTOR  Destroy;

        { Private routines }
        FUNCTION  GetCommentTime(cT: String): LongInt;
        FUNCTION  GetQuoteString(VAR cSt: String): Boolean;
        FUNCTION  GetRating(nWhich: Integer): String;
        FUNCTION  GetLine: Boolean;
        PROCEDURE GetActualGame(nBeginPos, nEndPos: LongInt);
 // routines from bpgnx
        FUNCTION  GetBPGNMain(nIndex: Integer): Integer;
        FUNCTION  GetBPGNBoard(nIndex: Integer): Char;
        FUNCTION  GetBPGNNumber(nIndex, nBoard: Integer): Integer;
        FUNCTION  GetBPGNMove(nIndex: Integer): String;
        FUNCTION  GetBPGNAnno(nIndex: Integer): String;
        FUNCTION  GetBPGNDisplay(nIndex: Integer): Boolean;
        FUNCTION  GetBPGNComment(nIndex: Integer): String;
        FUNCTION  GetBPGNnWTime(nIndex, nBoard: longint): longint;
        FUNCTION  GetBPGNnBTime(nIndex, nBoard: longint): longint;
        PROCEDURE GetBPGNPosition(nIndex: Integer; VAR cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2: String; var bwm1,bwm2:boolean);


        PROCEDURE GetActualGameText(nIndex: Integer);
        PROCEDURE GetGame(nBeginPos, nEndPos: LongInt);
        PROCEDURE CopyChessIntoBPGNRecord(nIndex: Integer);
        PROCEDURE CopyBPGNRecordIntoChess(nIndex: Integer);
        PROCEDURE CopyBoardPiecesIntoBPGNRecord(cC1, cC2: TChess; nIndex: Integer);
        PROCEDURE CopyBoardsIntoBPGNRecord(cC1, cC2: TChess; nIndex: Integer; cCap1, cCap2: String);
        PROCEDURE CopyBPGNRecordIntoBoards(cC1, cC2: TChess; nIndex: Integer);
        FUNCTION  GetToken: Boolean;
        PROCEDURE Backup(nI: Integer);
        PROCEDURE Restore(nI: Integer);
        FUNCTION  LoadBPGNStuff: Boolean;
        PROCEDURE LoadBPGNResult(nIndex: Integer);
        PROCEDURE LoadBPGNMove(nParent: Integer; VAR nMoveA, nMoveB: Integer; VAR cCap1, cCap2: String; bMain: Boolean);
        PROCEDURE LoadBPGNBody(nParent, nMoveA, nMoveB: Integer; cCap1, cCap2: String; bMain: Boolean);
        PROCEDURE ReleaseBPGN(nIndex: Integer);
        PROCEDURE RemoveStuff(nWhich, nIndex: Integer);
        PROCEDURE SaveMove(nIndex: Integer; bSpace: Boolean; VAR cLineStr: String);
        PROCEDURE SaveTime(nIndex: Integer; VAR cLineStr: String);
        PROCEDURE SaveAnnotation(nIndex: Integer; VAR cLineStr: String);
        PROCEDURE SaveDisplay(nIndex: Integer; VAR cLineStr: String);
        PROCEDURE SaveComment(nIndex: Integer; VAR cLineStr: String);
        PROCEDURE SavePosition(nParent: Integer; VAR cLineStr: String);
        procedure SaveHeaders(fname:string);

        { Public routines }
        FUNCTION  OpenBPGN(cFilename: String): Boolean;
        FUNCTION  CloseBPGN(bSilent: Boolean): Boolean;


        FUNCTION  LoadGame(nIndex: Integer): Boolean;

        PROCEDURE DeleteAllAnnotations;
        PROCEDURE DeleteAllComments;
        PROCEDURE DeleteAllDisplays;
        PROCEDURE DeleteCurrentMove;

        procedure AppendGame(nIndex: Integer; filename:string);

        FUNCTION  GetTag     (nIndex: Integer): Char;
        FUNCTION  GetEvent   (nIndex: Integer): String;
        FUNCTION  GetSite    (nIndex: Integer): String;
        FUNCTION  GetDate    (nIndex: Integer): String;
        FUNCTION  GetPosition(nIndex: Integer): LongInt;
        FUNCTION  GetWhiteA  (nIndex: Integer): String;
        FUNCTION  GetBlackA  (nIndex: Integer): String;
        FUNCTION  GetWhiteB  (nIndex: Integer): String;
        FUNCTION  GetBlackB  (nIndex: Integer): String;
        FUNCTION  GetRWA     (nIndex: Integer): String;
        FUNCTION  GetRBA     (nIndex: Integer): String;
        FUNCTION  GetRWB     (nIndex: Integer): String;
        FUNCTION  GetRBB     (nIndex: Integer): String;
        FUNCTION  GetTimeC   (nIndex: Integer): String;
        FUNCTION  GetReason  : String;
        FUNCTION  GetResult  (nIndex: Integer): String;
        FUNCTION  GetOpenA   (nIndex: Integer): String;
        FUNCTION  GetOpenB   (nIndex: Integer): String;
        function  getsetup   (nIndex: Integer): String;
        function  getfen     (nIndex: Integer): String;
        function  getaReason  (nIndex: Integer): String;

        FUNCTION  GetWhiteAHandle(nIndex: Integer): String;
        FUNCTION  GetBlackAHandle(nIndex: Integer): String;
        FUNCTION  GetWhiteBHandle(nIndex: Integer): String;
        FUNCTION  GetBlackBHandle(nIndex: Integer): String;

        FUNCTION  GetHeaderCount: Integer;
        FUNCTION  GetHeaderItem(nIndex: Integer): String;

        PROCEDURE SetTag   (nIndex: Integer; cTag:    Char);
        PROCEDURE SetEvent (nIndex: Integer; cEvent:  String);
        PROCEDURE SetSite  (nIndex: Integer; cSite:   String);
        PROCEDURE SetDate  (nIndex: Integer; cDate:   String);
        procedure SetPosition (nIndex: Integer; nPos:longint);
        PROCEDURE SetWhiteA(nIndex: Integer; cWhiteA: String);
        PROCEDURE SetBlackA(nIndex: Integer; cBlackA: String);
        PROCEDURE SetWhiteB(nIndex: Integer; cWhiteB: String);
        PROCEDURE SetBlackB(nIndex: Integer; cBlackB: String);
        PROCEDURE SetRWA   (nIndex: Integer; cRWA:    String);
        PROCEDURE SetRBA   (nIndex: Integer; cRBA:    String);
        PROCEDURE SetRWB   (nIndex: Integer; cRWB:    String);
        PROCEDURE SetRBB   (nIndex: Integer; cRBB:    String);
        PROCEDURE SetTimeC (nIndex: Integer; cTimeC:  String);
        PROCEDURE SetReason(nIndex: longint; cReason: String);
        PROCEDURE SetResult(nIndex: Integer; cRC:     String);
        PROCEDURE SetOpenA (nIndex: Integer; cOpenA:  String);
        PROCEDURE SetOpenB (nIndex: Integer; cOpenB:  String);

        FUNCTION  GetCurrentBoard1WTime: String;
        FUNCTION  GetCurrentBoard1BTime: String;
        FUNCTION  GetCurrentBoard2WTime: String;
        FUNCTION  GetCurrentBoard2BTime: String;
        FUNCTION  GetCurrentBoard1Caption: String;
        FUNCTION  GetCurrentBoard2Caption: String;
        FUNCTION  GetCurrentComment: String;
        FUNCTION  GetCurrentDisplay: Boolean;

        FUNCTION  GetCurrentNumber: Integer;
        FUNCTION  GetCurrentBoard: Char;
        FUNCTION  GetCurrentMove: String;
        FUNCTION  GetCurrentAnno: String;
        Function  GetCurrentLag(nBoard:integer):integer;
        Function  GetCurrentWLag(nBoard:integer):integer;
        Function  GetCurrentBlag(nBoard:integer):integer;

        PROCEDURE SetCurrentBoard1WTime(nWTime: Integer);
        PROCEDURE SetCurrentBoard1BTime(nBTime: Integer);
        PROCEDURE SetCurrentBoard2WTime(nWTime: Integer);
        PROCEDURE SetCurrentBoard2BTime(nBTime: Integer);
        PROCEDURE SetCurrentBoard1Caption(cCap1: String);
        PROCEDURE SetCurrentBoard2Caption(cCap2: String);
        PROCEDURE SetCurrentComment(cComment: String);
        PROCEDURE SetCurrentDisplay(bDisplay: Boolean);
        PROCEDURE SetCurrentAnno(cAnno: String);

        FUNCTION  GetNextNumber(nIndex: Integer): Integer;
        FUNCTION  GetNextBoard (nIndex: Integer): Char;
        FUNCTION  GetNextMove  (nIndex: Integer): String;
        FUNCTION  GetNextAnno  (nIndex: Integer): String;
        FUNCTION  GetNextIndex (nIndex: Integer): Integer;

        function GetNextCnt(nIndex: Integer):Integer;
        FUNCTION  GetNextCount: Integer;
        FUNCTION  GetNextItem(nIndex: Integer): String;

        PROCEDURE LoadNextMove;

        FUNCTION  GetPNextNumber(nIndex: Integer): Integer;
        FUNCTION  GetPNextBoard (nIndex: Integer): Char;
        FUNCTION  GetPNextMove  (nIndex: Integer): String;
        FUNCTION  GetPNextAnno  (nIndex: Integer): String;
        FUNCTION  GetPNextIndex (nIndex: Integer): Integer;

        FUNCTION  GetPNextCount: Integer;
        FUNCTION  GetPNextItem(nIndex: Integer): String;

        PROCEDURE LoadPNextMove;

        PROCEDURE LoadNavigator;

        FUNCTION  GetAllNavigator   (nIndex: Integer): Boolean;
        FUNCTION  GetBoard1Navigator(nIndex: Integer): Boolean;
        FUNCTION  GetBoard2Navigator(nIndex: Integer): Boolean;

        PROCEDURE MakeAllNav1Move;
        PROCEDURE MakeAllNav2Move;
        PROCEDURE MakeAllNav3Move;
        PROCEDURE MakeAllNav4Move;
        Procedure makeallNav5Move;
        procedure makeallNav6Move;
        Procedure makeallNav7Move;
        Procedure MakeallNav8Move;

        PROCEDURE MakeBoard1Nav1Move;
        PROCEDURE MakeBoard1Nav2Move;
        PROCEDURE MakeBoard1Nav3Move;
        PROCEDURE MakeBoard1Nav4Move;

        PROCEDURE MakeBoard2Nav1Move;
        PROCEDURE MakeBoard2Nav2Move;
        PROCEDURE MakeBoard2Nav3Move;
        PROCEDURE MakeBoard2Nav4Move;

        PROCEDURE MakeNextMove(nIndex: Integer);
        PROCEDURE MakePrevMove(nIndex: Integer);
        PROCEDURE MakeBoardMove(cC1, cC2: TChess; cBoard: Char; cMove: String; nMI: Integer; cCap1, cCap2: String);
        PROCEDURE UpdateComment(nIndex: Integer; cGame: String);
        FUNCTION  AddExtractMove(nParent, nMove: Integer; cBoard: Char; cMove: String; nTime: Integer; cGame: String): Integer;

        FUNCTION  IsOpen: Boolean;
        FUNCTION  IsEmpty: Boolean;
        FUNCTION  IsAtStartingPosition: Boolean;
        FUNCTION  IsGameChange: Boolean;
        FUNCTION  IsFileChange: Boolean;
        FUNCTION  GetBPGNFilename: String;
        FUNCTION  GetPositionIndex: Integer;
        PROCEDURE SetStartTime(nST: Integer);

        procedure NewGameString(p:pbpgnrec;var s:string);
        PROCEDURE NewFile(cFn, cWhiteA, cBlackA, cWhiteB, cBlackB: String);
        PROCEDURE AppendNewGame(pnew:pbpgnrec);
        PROCEDURE DuplicateGame(nIndex: Integer);
        FUNCTION  CanAddNewGame: Boolean;
        FUNCTION  GameIsValid(nIndex: Integer): Boolean;
        PROCEDURE SetGameChange(bC: Boolean);
        PROCEDURE SetFileChange(bC: Boolean);
        PROCEDURE RebuildHeaders(VAR ListBox1: TListBox);
        FUNCTION  GetErrorMessage: String;
        PROCEDURE SetPromptError(bE: Boolean);

        function GetBPGNLineUpTo(bd:integer):integer;
        procedure parsefen(fen:string);
        procedure parsefen1(fen:string);
        procedure parsefen2(fen:string);
        procedure splitbpgn(d1,d2:string;m1,m2:integer);
        procedure resetfilter;
        procedure setfilter;
        function filtermatch(n:longint):boolean;
        function filteron():boolean;
        function getFIndex(n:longint):longint;
        procedure setfindex(n,findex:longint);
        function getplayerside(player:string;n:longint):integer;
        function isblack(side:integer):boolean;
        function israted(n:longint):boolean;
        function getsideresult(side:integer;n:longint):integer;
        function getopponentside(side:integer):integer;
        function getpartnerside(side:integer):integer;
        function getsiderating(side:integer;n:longint):integer;
        function getsideplayer(side:integer;n:longint):string;
        function isreason(s:string):boolean;
        procedure gametoheader(var gametext:string;n:longint;var head:theadersave);

        procedure DBtoHeader(var s:string;p:pheadersave;p1:pbpgnrec);
        procedure BPGNtoPosRec(n:longint;cboard:char; var rec: tposDBrec);
        function gettimedif(nIndex:longint;cboard:char):integer;
        function AddGameToPosDb(n:longint):integer;
        function addChildrenPos(aleft,bleft:integer;ParentN:integer;lastA,lastB:integer;ismain:boolean):integer;
//        Procedure SetPosFilter(board:integer);
        procedure SetNonPosFilter;
        function  checkDBtype:integer;
        procedure setStats(fform:Tfform);
        procedure setNonPosStats(fform:tfform);
        procedure setposstats(fform:tfform);
        procedure sortmovestats(fform:tfform;col:integer);
        function UpdateGameToPosDB(n:longint):integer;
        function CompareNextMove(i,j,col:integer):integer;
        procedure DisplayPosStats(fform:tfform);
        procedure goposition(var rec:TPosDBRec);
        function IsDouble(i,j:longint):boolean;
        function gamexstart(gamen: longint;ncolor:integer):longint;
        procedure removeGameFromDB(gamen:longint);
        procedure shiftgamesDb(gamen:longint);
        procedure appendGameDB(fname:string;nIndex,nPrevSize,nPrevCount,nAddSize:longint);
        function iszh(nIndex:longint):boolean;
        procedure Createdatabase(pFname:Pointer);
        procedure SaveGame_(p:Pointer);
        procedure PackFile_(p:pointer);
        procedure updateDB_(p:pointer);
        procedure killdoubles_(p:pointer);
        procedure loadheader_(p:pointer);
        procedure LoadDBHeader_(p:pointer);
        procedure LoadDBPos_(p:pointer);
        procedure SaveFiltered_(p:pointer);
        procedure filterTag(cTag:char);
        procedure filterBroken;
        function getRSideResult(side:integer;n:longint):integer;
        procedure DeleteFiltered;
        procedure SelectFiltered;
        procedure untagFiltered(ctag:char);
        procedure invertTag(cTag:char);
        function SynchFilter(var filter:TfilterRec):integer;
        procedure SyncResultFromTag(gameN:longint);
        procedure SyncResultFromString(gamen:longint);
        procedure SaveUpdates_(p:pointer);
//        procedure killshort(nMaxMove:integer);
//        function isShortgame(i:longint;nMaxMove:integer):boolean;
        function isBroken(n:longint):boolean;
        PRIVATE
        bGameChange: Boolean;
        FUNCTION  PackFile: Boolean;
        PROCEDURE SaveGame(nIndex: Longint);
        function  SavePosDb(fname:string):integer;
        procedure updatedb;
        procedure killdoubles;

        FUNCTION  LoadHeader: Boolean;
        function LoadDBheaders(fname:string):boolean;
        procedure savefiltered(fname:string;bdel:boolean);
        procedure SaveUpdates;

//        function GameScoreStart(bScrewFirstMove:boolean):longint;
        procedure GetUpdatedGame(nIndex:longint);
        procedure getUpdatedHeader(nIndex:longint);
        function FindHeader(cHeaderTag:string;var cGame:string):string;
        function GetNextLag(nbpgnIndex:longint):integer;

        PUBLIC
            { Common }
            bScrewFirstMove:boolean;
            bPromptError: Boolean;
            cFile:        String;
            cName:        String;
            cResult:      String;
            ByteFile:     FILE OF Byte;
            BPGNFile:     FILE OF FileString;
            bOpen:        Boolean;
            nFileSize:    LongInt;
            cFileString:  FileString;

            { BPGN file }
            bFirst:       Boolean;
            nFilePos:     LongInt;
            nLastPos:     LongInt;
            nStartString: LongInt;
            nEndString:   LongInt;
            cLine:        String;

            { BPGN Game }
            cActualGame:  String;
            cGameLine:    String;
            nGameSize:    LongInt;
            nGameIndex:   LongInt;
            cToken:       String;
            cTmpSt:       String;
            nState:       Integer;
            nLastState:   Integer;
            aGameIndex:   ARRAY[1..MAX_BPGN_SAVE] OF LongInt;
            aState:       ARRAY[1..MAX_BPGN_SAVE] OF Integer;

            { BPGN Header }
            nTmpCounter  :integer;
            nHeaderCount: Integer;
            nGameHeader:  Integer;
            aList: tbpgnlist;

            { BPGN Tree }
            nBPGNIndex:  Integer;
            cBPGNResult: String;

            cStartCap1:  String;
            cStartCap2:  String;
            bGameDone:   Boolean;
            bGameError:  Boolean;
            nCurrent:    Integer;

            bFileChange: Boolean;
            bLagTag    : boolean;
            cLagString : string;

            aFound:      ARRAY[1..2] OF Boolean;
            aFirst:     TboardCount;// ARRAY[1..2] OF Integer;
            aPrevious:  TboardCount;// ARRAY[1..2] OF Integer;
            aNext:      TboardCount;// ARRAY[1..2] OF Integer;
            nLast:       Integer;

            Chess1:      TChess;
            Chess2:      TChess;

            BPGNArray:   ARRAY[1..MAX_POSITION, 1..2] OF BPGNRecord;
            BPGNInUse:   ARRAY[1..MAX_POSITION] OF Boolean;
            BPGNIsMain:  ARRAY[1..MAX_POSITION] OF Boolean;
            BPGNParent:  ARRAY[1..MAX_POSITION] OF Integer;
            BPGNMain:    ARRAY[1..MAX_POSITION] OF Integer;
            BPGNNext:    ARRAY[1..MAX_POSITION, 1..MAX_NEXT] OF Integer;
            BPGNNumber:  ARRAY[1..MAX_POSITION, 1..2] OF Integer;
            BPGNBoard:   ARRAY[1..MAX_POSITION] OF Char;
            BPGNMove:    ARRAY[1..MAX_POSITION] OF String;
            BPGNAnno:    ARRAY[1..MAX_POSITION] OF String;
            BPGNDisplay: ARRAY[1..MAX_POSITION] OF Boolean;
            BPGNComment: ARRAY[1..MAX_POSITION] OF String;
            BPGNLag:     ARRAY[1..MAX_POSITION] of integer;

            { Next move }
            nNextCount: Integer;
            aNumber:    ARRAY[1..MAX_NEXT] OF Integer;
            aBoard:     ARRAY[1..MAX_NEXT] OF Char;
            aMove:      ARRAY[1..MAX_NEXT] OF String;
            aAnno:      ARRAY[1..MAX_NEXT] OF String;
            aIndex:     ARRAY[1..MAX_NEXT] OF Integer;

            { Previous next move }
            nPNextCount: Integer;
            aPNumber:    ARRAY[1..MAX_NEXT] OF Integer;
            aPBoard:     ARRAY[1..MAX_NEXT] OF Char;
            aPMove:      ARRAY[1..MAX_NEXT] OF String;
            aPAnno:      ARRAY[1..MAX_NEXT] OF String;
            aPIndex:     ARRAY[1..MAX_NEXT] OF Integer;

            { Navigator }
            aNavAll:    ARRAY[1..4] OF Boolean;
            aNavBoard1: ARRAY[1..4] OF Boolean;
            aNavBoard2: ARRAY[1..4] OF Boolean;

            RFilter:TFilterRec;
            nFilterCount: longint;
            ndraw:integer;
            nwin:integer;
            nloss:integer;
            bfilterset:boolean;
            {positional database}
            bIsDB:boolean;
            pdblist : array [1..2] of tPosDB;
            savedepth:integer;
            bIsBugDb: boolean;
            NextDBList:TDBPosNextar;
            nCurrentWorB:integer;
            bdbchange:boolean;
            gamelistpos:TPosDBRec;
            bAbortLong:boolean;
    END;

IMPLEMENTATION
function tbpgn.getsideplayer(side:integer;n:longint):string;
begin
 case side of
 1: Result:=getWhitea(n);
 2: Result:=getBlacka(n);
 3: Result:=GetWhiteb(n);
 4: Result:=getblackb(n);
 end;
end;

function tbpgn.isreason(s:string):boolean;
var ps:integer;
begin
 if s[1]<>'}' then begin Result:=false;exit;end;
 delete(s,1,1);
 if pos('-',s)=0 then begin Result:=false;exit;end;
 ps:=pos('}',s);
 if ps=0 then begin Result:=false;exit;end;
 delete(s,ps,length(s));
 if pos(' checkmated',s)>0 then begin Result:=true;exit;end;
 if pos(' resignes',s)>0 then begin Result:=true;exit;end;
 if pos(' forfeits on time',s)>0 then begin Result:=true;exit;end;
 result:=false;
end;

function tbpgn.getopponentside(side:integer):integer;
begin
 case side of
 1: Result:=2;
 2: Result:=1;
 3: Result:=4;
 4: Result:=3;
 end;
end;

function tbpgn.getpartnerside(side:integer):integer;
begin
 Result:=5-side;
end;

function tbpgn.getsiderating(side:integer;n:longint):integer;
var s:string;
res,err:integer;
begin
 case side of
 1: s:=getRWA(n);
 2: s:=getRBA(n);
 3: s:=getRWB(n);
 4: s:=getRBB(n);
 end;
 val(s,res,err);
 if err<>0 then res:=FICS_AVERAGE_RATING;
 Result:=res;
end;

function tbpgn.getsideresult(side:integer;n:longint):integer;
var s:string;
r1:integer;
begin
  s:=getresult(n);
  if s=BPGN_UNKNOWN then begin REsult:=0; exit; end;
  if s=BPGN_DRAW then begin Result:=3;exit; end;
  r1:=3;
  if s=BPGN_WHITE_WINS then r1:=1;
  if s=BPGN_BLACK_WINS then r1:=2;
  if ((side=2) or (side=3)) then r1:=3-r1;
  Result:=r1
end;

function tbpgn.isblack(side:integer):boolean;
begin
 if ((side=1) or (side=3)) then Result:=false else result:=true;
end;

function tbpgn.israted(n:longint):boolean;
begin
 if pos('unrated',lowercase(getevent(n)))>0 then Result:=false else result:=true;
end;

function tbpgn.getplayerside(player:string;n:longint):integer;
begin
 player:=LowerCase(player);
 if player=lowercase(getwhitea(n)) then
 begin result:=1; exit;end;
 if player=lowercase(getwhiteb(n)) then
 begin result:=3; exit;end;
 if player=lowercase(getblacka(n)) then
 begin result:=2; exit;end;
 if player=lowercase(getblackb(n)) then
 begin result:=4; exit;end;
 Result:=0;
end;

procedure tbpgn.savefiltered(fname:string;bdel:boolean);
var f:TextFile;
hfile:File;
i,j,n,maxj:longint;
bdbsave:boolean;
cLine,hname,wname,bname:string;
wlist:tposdb;
blist:tposdb;
ngamechunk,nc:integer;
lc,hc,nPrevCount,nPrevSize,nPrevCountW,nPrevCountB:longint;
headlist:pheadlist;
barstep,nbarc:integer;
begin
 nGameChunk:=MAX_STRING div 1000;
 bdbsave:=((ExtractFileExt(fname)=DB_HEADER_EXT) and (bisdb));
 cResult:='';
 if bdbsave then
 begin
  try
   New(headlist);
  except
   cResult:='Not enough memory';
   SendMessage(fmainHandle,WM_BpgnError,0,0);
   exit;
  end;

  hname:=fname;
  fname:=ChangeFileExt(hname,'.'+BPGN_EXT);
  wname:=ChangeFileExt(hname,DB_WHITE_EXT);
  bname:=ChangeFileExt(hname,DB_BLACK_EXT);
  nc:=ReadDbSyncInfo(hname,nPrevSize,nPrevCount);
  if nc=0 then begin
   cresult:='Can not add games to'+EOL_STR+'The database file is damaged';
   dispose(headlist);
   SendMessage(fmainHandle,WM_BpgnError,0,0);
   exit;
  end;
  if (((bisbugdb) and (nc=-1)) or ((not bisbugdb) and (nc=1))) then begin
   cResult:='Types of source and destination databases do not match'+EOL_STR;
   dispose(headlist);
   SendMessage(fmainHandle,WM_BpgnError,0,0);exit;
  end;
  Assign(hfile,hname);
 end;

 Assign(f,fname);
 try

  if bdbsave then
  begin
   wlist:=tposdb.Create;
   blist:=tposdb.Create;
   if nc=2 then begin Rewrite(hfile,1);
    if not WriteDBSyncInfo(hfile,getbpgnfilename,0,bisbugdb,0)
    then begin
     cresult:='Can not write to file'+EOL_STR+hname;
     PostMessage(fmainHandle,WM_BpgnError,0,0);
    end;
    nPrevCount:=0;nPrevSize:=0; nPrevCountW:=0;nPrevCountB:=0;
   end
   else begin
    Reset(hfile,1);nPrevCountW:=GetPositionCount(wname);
    nPrevCountB:=getPositionCount(bname);
   end;
   seek(hfile,filesize(hfile));
  end;


  if fileexists(fname) then  Append(f) else rewrite(f);
  nc:=0; cline:='';lc:=0;hc:=0;
  PostMessage(fmainHandle,WM_ProgressUpdate,1,nFilterCount);
  barstep:=100; nbarc:=0;
  for I:=0 to  nfiltercount-1 do
  begin
   n:=getFindex(i);
   GetActualGameText(n);
   if (length(cactualgame)<1) then continue;
   cLine:=cLine+cActualgame;
   if nc=nGamechunk then
   begin
    write(f,cLine);
    cLine:=''; nc:=0;
   end else inc(nc,1);

   if bdbsave then begin
    gametoheader(cActualgame,n,headlist[hc]);
    with headlist[hc] do begin
     GameStart:=lc+nPrevSize;
     GameEnd:=Gamestart+length(cActualgame)-1;
     positionw:=wlist.Count+nPrevCountW;
     positionb:=blist.Count+nPrevCountB;
    end;
    {white positions}
    if n<(alist.Count-1) then maxj:=pbpgnrec(alist.Items[n+1]).apositionw-1
    else maxj:=pdblist[1].Count-1;
    for j:=pbpgnrec(alist.Items[n])^.apositionw to maxj do
    with wlist do
    begin
      add(pdblist[1].items[j]);
      pposdbrec(last)^.GameN:=i;
    end;
    {black positions}
    if n<(alist.Count-1) then maxj:=pbpgnrec(alist.Items[n+1]).apositionb-1
    else maxj:=pdblist[2].Count-1;
    for j:=pbpgnrec(alist.Items[n])^.apositionb to maxj do
    with blist do
    begin
      add(pdblist[2].items[j]);
      pposdbrec(last)^.GameN:=i;
    end;

    {headers}
    inc(hc,1);
    if (hc>SAVE_HEADER_BUF) then
    begin
     blockwrite(hfile,Headlist^,sizeof(theadlist));
     hc:=0;
    end;

   end; {if bdbsave}

   if bdel then SetTag(n, TAG_DELETE);
   inc(lc,length(cActualgame));
   if nbarc=barstep then
   begin
    PostMessage(fmainHandle,WM_ProgressUpdate,i+1,nFilterCount); nbarc:=0;
   end else inc(nbarc,1);
  end; {for i}
  write(f,cLine);

  if bdbsave then begin
   blockwrite(hfile,headlist^,hc*sizeof(theadersave));
   wlist.Append(wname,nPrevCount); blist.Append(bname,nPrevCount);
  end;
 finally
  flush(f);
  closeFile(f);
  if bdbsave then
  begin
   writedbSyncInfo(hfile,fname,lc+nPrevSize,bisbugdb,nFilterCount+nPrevCount);
   dispose(headlist);
   wlist.clear;blist.clear;
   wlist.Free;blist.Free;
   closeFile(hfile);
  end;

 end; {try finally}

end;

function tbpgn.getFindex(n:longint):longint;
var p:pbpgnrec;
begin
 p:=alist.Items[n];
 Result:=p^.aFilterN;
end;

procedure tbpgn.setfindex(n,findex:longint);
var p:pbpgnrec;
begin
 p:=alist.Items[n];
 p^.aFilterN:=findex;
end;


function tbpgn.filteron():boolean;
begin
Result:=not (nheadercount=nfiltercount)
end;

function tbpgn.filtermatch(n:longint):boolean;
var n1,er,r1,r2,r3,r4,rc,rc1,rmax,rmin,rav,rav1,i:integer;
bzh:boolean;
d1:tdatetime;
s,wa,wb,ba,bb,tc:string;
p:pbpgnrec;
worb:integer;
prec:tposdbrec;
cTag:char;
begin
 p:=alist.items[n];
 wa:=lowercase(getwhitea(n));
 wb:=lowercase(getwhiteb(n));
 ba:=lowercase(getblacka(n));
 bb:=lowercase(getblackb(n));
 with rfilter do
 begin
  n1:=0;
  if player1=wa then n1:=1;
  if player1=bb then n1:=4;
  if player1=ba then n1:=3;
  if player1=wb then n1:=2;
  if player1='' then n1:=5;
  if (n1=0) then
  begin
   result:=false;
   exit;
  end;
  //exclude non-selected;
  if bSelected then
   if (gettag(n)<>Tag_Select) then
   begin
      Result:=false; exit;
   end;

  //exclude white
  if ((not bwhite) and (n1<3)) then
  begin
   result:=false;
   exit;
  end;
  //exclude black
  if ((not bblack) and (n1>2)) then
  begin
   result:=false;
   exit;
  end;

  if ((bOpponent) and (((n1=1)and(player2<>ba)) or ((n1=2)and(player2<>bb)) or((n1=3)and(player2<>wa)) or ((n1=4)and(player2<>wb))))  then
  begin
    result:=false;
    exit;
  end else
  if ((player2<>'')and(player2<>wa) and (player2<>wb)and(player2<>ba)and (player2<>bb)) then  begin
   result:=false;
   exit;
  end;
  if (Pos(Crazyhouse_Id,getevent(n))<>0) then bzh:=true else bzh:=false;
  if (((bPartner) and (not bzh)) and (((n1=1)and(player3<>bb)) or ((n1=2)and(player3<>ba)) or((n1=3)and(player3<>wb)) or ((n1=4)and(player3<>wa))))  then
  begin
    result:=false;
    exit;
  end else
  if ((player3<>'')and(player3<>wa) and (player3<>wb)and(player3<>ba)and (player3<>bb)) then
  begin
   result:=false;
   exit;
  end;
  if ((player4<>'')and(player4<>wa) and (player4<>wb)and(player4<>ba)and (player4<>bb)) then
  begin
   result:=false;
   exit;
  end;
  //filter bug/zh
  if not bbug then
   if not bzh then begin
    result:=false;
    exit;
   end;
  if not bcrazy then
   if bzh then begin
    result:=false;
    exit;
   end;

  //filter time
  tc:=gettimec(n);
  if (timemin>(TOOLBOX_GetTimeVal(tc) div 60)) then
  begin
   result:=false;
   exit;
  end;
  if (timemax<(TOOLBOX_GetTimeVal(tc)/60)) then
  begin
   result:=false;
   exit;
  end;
 //filter date
   d1:=longstrtodate(getdate(n),er);
   if er<0 then d1:=datemin;
  if ((d1<datemin) or (d1>datemax)) then
  begin
   result:=false;
   exit;
  end;

  if not brated then
   if israted(n) then
   begin
    Result:=false ;
    exit;
   end;
  if not bunrated then
   if not israted(n) then
   begin
    Result:=false ;
    exit;
   end;


  if (possearchmode<3) then
  begin
   if(not pdblist[nCurrentworb].findPosInGame(n,gamexstart(n,ncurrentworb),pdblist[nCurrentworb].Count-1)) then begin Result:=false; exit;end;
   // if positional search, filter the games where position occured not on the board of player 1;
   with pdblist[ncurrentworb] do begin
    if n1<>5 then
    repeat
     if pposdbrec(items[lastfound]).cBoard=(2-(n1 mod 2)) then break;
     if not findposingame(n,LastFound+1,count-1) then
     begin
      result:=false;
      exit;
     end;
    until false;
    if n1=5 then n1:=boardcolortoside(pposdbrec(items[lastfound]).cBoard,1);
   end;
  end;

  if ((n1=2) or (n1=3)) then n1:=5-n1; //translate to normal side;

  // filterRatings
  if n1=5 then
   case nRatingMode of
   rmPlayer..rmPardop: nRatingMode:=rmAll;
   rmPteam..rmOteam: nRatingMode:=rmAllteams;
   end;


  case nRatingMode of
  rmAll: begin
  rc:=4;
  val(p^.aRwa,r1,er);
  if er<>0 then begin r1:=0; dec(rc,1);end;
  val(p^.arwb,r2,er);
  if er<>0 then begin r2:=0; dec(rc,1);end;
  val(p^.arba,r3,er);
  if er<>0 then begin r3:=0; dec(rc,1);end;
  val(p^.arbb,r4,er);
  if er<>0 then begin r4:=0; dec(rc,1);end;
  rmax:=max(r1,r2);rmax:=max(rmax,r3);rmax:=max(rmax,r4);
  rmin:=min(r1,r2);rmin:=min(rmin,r3);rmin:=min(rmin,r4);
  if rc>0 then rav:=round((r1+r2+r3+r4)/rc) else rav:=0;
  end;{case 0}
  rmAllTeams: Begin
   rc:=2;rc1:=2;
   val(p^.aRwa,r1,er);
   if er<>0 then begin r1:=0; dec(rc,1);end;
   val(p^.arwb,r2,er);
   if er<>0 then begin r2:=0; dec(rc1,1);end;
   val(p^.arba,r3,er);
   if er<>0 then begin r3:=0; dec(rc1,1);end;
   val(p^.arbb,r4,er);
   if er<>0 then begin r4:=0; dec(rc,1);end;
   rmax:=max(r1,r2);rmax:=max(rmax,r3);rmax:=max(rmax,r4);
   rmin:=min(r1,r2);rmin:=min(rmin,r3);rmin:=min(rmin,r4);
   if rc1>0 then rav:=round((r2+r3)/rc1) else rav:=0;
   if ((rav<ravmin)or(rav>ravmax)) then
   begin
    result:=false;
    exit;
   end;
   if rc>0 then rav:=round((r1+r4)/rc) else rav:=0;
  end; {case 1}
  rmPlayer: rav:=GetSideRating(n1,n);
  rmOpponent: rav:=GetSideRating(getOpponentSide(n1),n);
  rmPartner: rav:=GetSideRating(GetPartnerSide(n1),n);
  rmPardop: rav:=GetSideRating(GetPartnerSide(getOpponentSide(n1)),n);
  rmPteam: Begin
   r1:=GetSiderating(n1,n); r2:=GetSideRating(GetPartnerSide(n1),n);
   rmax:=max(r1,r2); rmin:=min(r1,r2);
   rav:=Round((r1+r2)/2);
  end;
  rmOteam: Begin
   rc1:=GetOpponentSide(n1);
   r1:=GetSiderating(rc1,n); r2:=GetSideRating(GetPartnerSide(rc1),n);
   rmax:=max(r1,r2); rmin:=min(r1,r2);
   rav:=Round((r1+r2)/2);
  end;

  end; {case}

  if ((rav<ravmin)or(rav>ravmax)) then
  begin
   result:=false;
   exit;
  end;
  if ((nRatingMode<rmPlayer) or (nRatingMode>rmPardop)) then //maximum and minimum ratings make sense.
  begin
          if ((rmax<maxrmin)or(rmax>maxrmax)) then
          begin
           result:=false;
           exit;
          end;
          if ((rmin<minrmin)or(rmin>minrmax)) then
          begin
           result:=false;
           exit;
          end;
  end;

  //filter result;
  s:=getresult(n);
  r1:=getSideResult(n1,n);
  if (((r1=0) and (not bunknown))or ((r1=3)and (not bdraw)) or ((r1=1) and (n1<5) and (not bwin))or ((r1=2) and (n1<5) and (not bloss)) or ((n1=5) and (not bWin) and (not Bloss) and ((r1=1) or (r1=2)))) then
  begin
    result:=false;
    exit;
  end;

 end;{with rfilter do}


 if r1=1 then inc(nwin,1);
 if r1=2 then inc(nloss,1);
 if r1=3 then inc(ndraw,1);
 Result:=true;
end;

procedure tbpgn.setNonPosfilter;
var i,fc:longint;
begin
 fc:=0;
 ndraw:=0;nwin:=0;nloss:=0;
 if rfilter.possearchmode<3 then
   begin
    if bpgnarray[ncurrent,rfilter.possearchmode].bWhitesMove then nCurrentworb:=1
    else nCurrentworb:=2;
    boardtoint(bpgnarray[ncurrent,rfilter.possearchmode].aboard,pdblist[ncurrentWorb].searchrec.position);
    rfilter.position:=pdblist[ncurrentWorb].searchrec.position;
    gamelistpos:=pdblist[ncurrentworb].searchrec;
    gamelistpos.cBoard:=ncurrentworb;
   end
 else gamelistpos.cBoard:=0;

 for i:=0 to nHeaderCount-1 do
 if filtermatch(i) then
 begin
  setfindex(fc,i);
  inc(fc,1);
 end;
 nFilterCount:=fc;
 bfilterset:=true;
end;

{procedure tbpgn.setPosFilter(board:integer);
var i,nMin,nMax,nc,gameN:longint;
rec:tPosDbrec;
cboard:char;
worb:integer;
begin
 nc:=0;
 boardtoInt(bpgnarray[ncurrent,board].aboard,rec.position);

 if board=1 then
  if bpgnarray[ncurrent,board].bWhitesMove then begin cboard:='A'; worb:=1;end
  else begin cboard:='a';worb:=2;end;
 if board=2 then
  if bpgnarray[ncurrent,board].bWhitesMove then begin cboard:='B'; worb:=1; end
  else begin cboard:='b'; worb:=2; end;
 rec.cBoard:=boardtobyte(cboard);

 pdblist[worb].getPositionBounds(rec,nMin,Nmax);
 for i:=nMin to nMax do
 begin
  gameN:=pdblist[worb].getGameN(i);
  rfilter.boardnum:=pdblist[worb].getBoardnum(i);
  if nc>0 then
   if getfindex(nc-1)=GameN then continue;

  if filtermatch(gameN) then
  begin
   setfindex(nc,gameN);
   inc(nc,1);
  end;
 end;
 nFilterCount:=nc;
end;
}
procedure tbpgn.setfilter;
begin
 setNonPosfilter;
end;

procedure tbpgn.resetfilter;
var i:longint;
begin
 for i:=0 to getHeaderCount-1 do begin setfindex(i,i);end;
 nFilterCount:=nHeaderCount;
 bfilterset:=false;
 resetfilterrec(rfilter);
end;

procedure tbpgn.splitbpgn(d1,d2:string;m1,m2:integer);
var f1,f2:TextFile;
i:integer;
begin
 Assign(f1,d1);
 Assign(f2,d2);
 try
  If m1=1 then rewrite(f1) else Append(f1);
  if m2=1 then rewrite(f2) else Append(f2);
  for I:=0 to  nheadercount-1 do
  begin
   GetActualGameText(i);
   if (length(cactualgame)<1) then continue;
   IF (Pos(Crazyhouse_Id,lowercase(getevent(i))) = 0)
   THEN  write(f1,cactualgame)   else write(F2,cactualgame);
  end;
 finally

  flush(f1);flush(f2);
  closeFile(f1);closeFile(f2);
 end;
end;

procedure tbpgn.parsefen1(fen:string);
var
ps:integer;
s:string;
begin
 chess1.setfenpos(fen);
 ps:=lastintpos(fen,' ');
 if (ps<0) then exit;
 s:=copy(fen,ps+1,length(fen));
 BPGNArray[1,1].nBTime:=Internaltime(s);
 delete(fen,ps,length(fen));
 fen:=trim(fen);
 ps:=lastintpos(fen,' ');
 if (ps<0) then exit;
 s:=copy(fen,ps+1,length(fen));
 BPGNArray[1,1].nWTime:=Internaltime(s);
 Bpgnarray[1,1].nwLag:=0;
 bpgnarray[1,1].nblag:=0;
 exit;
end;

procedure tbpgn.parsefen2(fen:string);
var
ps:integer;
s:string;
begin
 chess2.setfenpos(fen);
 ps:=lastintpos(fen,' ');
 if (ps<0) then exit;
 s:=copy(fen,ps+1,length(fen));
 BPGNArray[1,2].nBTime:=InternalTime(s);
 delete(fen,ps,length(fen));
 fen:=trim(fen);
 ps:=lastintpos(fen,' ');
 if (ps<0) then exit;
 s:=copy(fen,ps+1,length(fen));
 BPGNArray[1,2].nWTime:=InternalTime(s);
 Bpgnarray[1,2].nwLag:=0;
 bpgnarray[1,2].nblag:=0;
 exit;
end;

procedure TBPGN.parsefen(fen:string);
var fen1,fen2:string;
ps:integer;
begin
 fen:=trim(fen);
 ps:=pos('|',fen);
 if (ps<0) then  parsefen1(fen)
 else begin
  fen1:=trim(copy(fen,1,ps-1));
  fen2:=trim(copy(fen,ps+1,length(fen)));
  parsefen1(fen1);
  parsefen2(fen2);
 end;
end;

function TBPGN.GetNextCnt(nIndex:Integer):Integer;
var ni,nc:integer;
begin
FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nIndex,nI] = 0) THEN
        BEGIN
            nC := nI-1;
            BREAK;
        END;
    END;
Result:=nc;
end;

FUNCTION TBPGN.GetBPGNNumber(nIndex, nBoard: Integer): Integer;
BEGIN
 Result := BPGNNumber[nIndex, nBoard];
END;

FUNCTION TBPGN.GetBPGNBoard(nIndex: Integer): Char;
BEGIN
    Result := BPGNBoard[nIndex];
END;

FUNCTION TBPGN.GetBPGNMain(nIndex: Integer): Integer;
BEGIN
 Result := BPGNMain[nIndex];
END;

FUNCTION TBPGN.GetBPGNMove(nIndex: Integer): String;
BEGIN
 Result := BPGNMove[nIndex];
END;

FUNCTION TBPGN.GetBPGNAnno(nIndex: Integer): String;
BEGIN
    Result := BPGNAnno[nIndex];
END;

FUNCTION TBPGN.GetBPGNDisplay(nIndex: Integer): Boolean;
BEGIN
  Result :=BPGNDisplay[nIndex];
END;

FUNCTION TBPGN.GetBPGNComment(nIndex: Integer): String;
BEGIN
  Result := BPGNComment[nIndex];
END;

FUNCTION TBPGN.GetBPGNnWTime(nIndex, nBoard: longint): longint;
BEGIN
  Result := BPGNArray[nIndex, nBoard].nWTime;
END;

FUNCTION TBPGN.GetBPGNnBTime(nIndex, nBoard: Integer): Integer;
BEGIN
  Result := BPGNArray[nIndex, nBoard].nBTime;
END;

PROCEDURE TBPGN.GetBPGNPosition(nIndex: Integer; VAR cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2: String; var bwm1,bwm2:boolean);
BEGIN
        CopyBPGNRecordIntoChess(nIndex);
        cB1  := Chess1.GetBoardStr;
        cB2  := Chess2.GetBoardStr;
        cWP1 := Chess1.GetWhitePieceStr;
        cBP1 := Chess1.GetBlackPieceStr;
        cWP2 := Chess2.GetWhitePieceStr;
        cBP2 := Chess2.GetBlackPieceStr;

        cwt1:=TOOLBOX_GetDispTimeStr(BPGNArray[nIndex,1].nWTime);
        cwt2:=TOOLBOX_GetDispTimeStr(BPGNArray[nIndex,2].nWTime);
        cbt1:=TOOLBOX_GetDispTimeStr(BPGNArray[nIndex,1].nBTime);
        cbt2:=TOOLBOX_GetDispTimeStr(BPGNArray[nIndex,2].nBTime);
        bwm1:=chess1.bWhitesMove;
        bwm2:=chess2.bWhitesMove;
END;

PROCEDURE TBPGN.SetPromptError(bE: Boolean);
BEGIN
   bPromptError := bE;
END;



FUNCTION TBPGN.GetBPGNLineUpTo(bd:integer):integer;
var ind,cnt:integer;
begin
 ind:=nCurrent; cnt:=0;
 While (ind>1) do begin
  inc(cnt,1);
  line[cnt,bd]:=AlgebraicMove(BPGNArray[ind,bd].mPrev);
  ind:=BPGNParent[ind];
 end;
 linecnt[1]:=cnt;
 Result:=cnt;
end;


CONSTRUCTOR TBPGN.Init;
VAR
   nI: Integer;
BEGIN
    {filter}
    ResetFilterRec(Rfilter);

    { Common }
    bPromptError := TRUE;
    cFile        := '';
    cName        := '';
    cResult      := '';
    bOpen        := FALSE;

    { BPGN file }
    bFirst       := TRUE;
    nFileSize    := 0;
    nFilePos     := 0;
    nLastPos     := 0;
    nStartString := 0;
    nEndString   := -1;
    cLine        := '';
    bIsDB        :=false;
    savedepth:=DEFAULt_SAVEPOS_Depth;
    bdbchange:=false;

    { BPGN Game }
    cActualGame:= '';
    cGameLine  := '';
    nGameSize  := 0;
    nGameIndex := 0;
    cToken     := '';
    cTmpSt     := '';
    nState     := BPGN_NORMAL;
    nLastState := BPGN_NORMAL;

    { BPGN Header }
    nHeaderCount := 0;
    nGameHeader  := 0;
    nFilterCount := 0;

    { BPGN Tree }
    nBPGNIndex   := 0;
    cBPGNResult  := '';
    cStartCap1   := '';
    cStartCap2   := '';
    bGameDone    := FALSE;
    bGameError   := FALSE;
    nCurrent     := 0;
    bGameChange  := FALSE;
    bFileChange  := FALSE;
    aFound[1]    := FALSE;
    aFound[2]    := FALSE;
    aFirst[1]    := 0;
    aFirst[2]    := 0;
    aPrevious[1] := 0;
    aPrevious[2] := 0;
    aNext[1]     := 0;
    aNext[2]     := 0;
    nLast        := 0;

    Chess1 := TChess.Create;
    Chess1.Init;

    Chess2 := TChess.Create;
    Chess2.Init;

    alist:=tbpgnlist.Create;
//    alist.Capacity:=MAX_GAME;

    pdblist[1]:=tposdb.Create;
    pdblist[2]:=tposdb.Create;


    { Next move }
    nNextCount := 0;

    { Previous next move }
    nPNextCount := 0;

    { Navigator }
    FOR nI := 1 TO 4 DO
    BEGIN
        aNavAll   [nI] := FALSE;
        aNavBoard1[nI] := FALSE;
        aNavBoard2[nI] := FALSE;
    END;

END;

DESTRUCTOR TBPGN.Destroy;
BEGIN
    { Nothing }
END;

FUNCTION TBPGN.GetCommentTime(cT: String): LongInt;
VAR
    nS:integer;
    r:real;
    cTmp: String;
    bErr: Boolean;
BEGIN
    nS := Length(cT);
    IF (nS = 0) THEN
    BEGIN
        Result := -1;
        EXIT;
    END;

    VAL(cT, r, nS);

    IF (nS <> 0) THEN
    BEGIN
        Result := -1;
        EXIT;
    END;

    Result := Round(1000*r);
END;


FUNCTION TBPGN.GetQuoteString(VAR cSt: String): Boolean;
VAR
    nS, nB, nE, nI: Integer;
BEGIN
    cSt := '';
    nS  := Length(cLine);
    IF (nS = 0) THEN
    BEGIN
        Result := FALSE;
        EXIT;
    END;

    nB := 0;
    nE := 0;
    FOR nI := 1 TO nS DO
    BEGIN
        IF (cLine[nI] = '"') THEN
        BEGIN
            IF (nB = 0) THEN
                nB := nI
            ELSE
            IF (nE = 0) THEN
                nE := nI
            ELSE
                BREAK;
        END;
    END;

    IF ((nB = 0) OR (nE = 0) OR (nB > nE)) THEN
    BEGIN
        Result := FALSE;
        EXIT;
    END;

    IF (nB = nE) THEN
    BEGIN
        Result := TRUE;
        EXIT;
    END;

    cSt    := Copy(cLine, nB + 1, nE - nB - 1);
    Result := TRUE;
END;

FUNCTION TBPGN.GetRating(nWhich: Integer): String;
VAR
    nP, nS, nB, nC, nI: Integer;
    cSt: String;
BEGIN
    IF (nWhich = 1) THEN
    BEGIN
        { WhiteAElo }
        nP := Pos(BPGN_WHITEA_ELO, cLine);
    END
    ELSE
    IF (nWhich = 2) THEN
    BEGIN
        { BlackAElo }
        nP := Pos(BPGN_BLACKA_ELO, cLine);
    END
    ELSE
    IF (nWhich = 3) THEN
    BEGIN
        { WhiteBElo }
        nP := Pos(BPGN_WHITEB_ELO, cLine);
    END
    ELSE
    BEGIN
        { BlackBElo }
        nP := Pos(BPGN_BLACKB_ELO, cLine);
    END;

    IF (nP = 0) THEN
    BEGIN
        Result := '';
        EXIT;
    END;

    nS := Length(cLine);
    nB := 0;
    FOR nI := nP TO nS DO
    BEGIN
        IF (cLine[nI] = '"') THEN
        BEGIN
            nB := nI;
            BREAK;
        END;
    END;
    IF (nB = 0) THEN
    BEGIN
        Result := '';
        EXIT;
    END;

    Inc(nB);

    nC := 0;
    FOR nI := nB TO nS DO
    BEGIN
        IF (cLine[nI] = '"') THEN
            BREAK;

        Inc(nC);
    END;
    IF (nC = 0) THEN
    BEGIN
        Result := '';
        EXIT;
    END;

    cSt := Copy(cLine, nB, nC);
    IF (Length(cSt) > 4) THEN
        cSt := Copy(cSt,1,4);

    IF (CompareStr(cSt, UNKNOWN_RATING) = 0) THEN
        Result := ''
    ELSE
        Result := cSt;
END;

FUNCTION TBPGN.GetLine: Boolean;
VAR
    bC: Boolean;
    nI: LongInt;
BEGIN
    cLine := '';

    IF (bFirst) THEN
    BEGIN
        bFirst := FALSE;

            fillchar(cFileString,sizeof(cFileString),#0);

            TRY
                Read(BPGNFile, cFileString);
            EXCEPT
                ON E: EInOutError DO
                    BEGIN
                    END;
            END;

            nStartString := 0;
            nEndString   := StrLen(cFileString) - 1;
            if nEndstring>=Max_String then nEndstring:=max_string-1;
            nFilePos     := nLastPos;
    END
    ELSE
        nFilePos := nLastPos;

    IF (nStartString <= nEndString) THEN
    BEGIN
        bC := FALSE;

        WHILE (NOT bC) DO
        BEGIN
            REPEAT
                Inc(nLastPos);

                if cFileString[nStartString]=LF_ASCII_CHAR then
                begin
                    bC := TRUE;
                    Inc(nStartString);
//                    nFilePos:=nLastPos;
                     nI:=length(cline);
                     if nI>0 then
                     if cline[nI]=CR_ASCII_CHAR then delete(cline,ni,1);
                    BREAK;
                end;

{                IF (cFileString[nStartString] = CR_ASCII_CHAR) THEN
                BEGIN
                    bC := TRUE;
                    Inc(nStartString);
                    BREAK;
                END;
}
                cLine := cLine + cFileString[nStartString];

                Inc(nStartString);
            UNTIL (nStartString > nEndString);

            IF (NOT bC) THEN
            BEGIN
 //               IF (EOF(BPGNFile) OR (nLastPos >= nFileSize)) THEN
                IF (nLastPos >= nFileSize) THEN
                    BREAK;

                fillchar(cFileString,sizeof(cFileString),#0);
                TRY
                    Read(BPGNFile, cFileString);
                EXCEPT
                    ON E: EInOutError DO
                        BEGIN
                        END;
                END;

                nStartString := 0;
                nEndString   := StrLen(cFileString) - 1;
                if nendstring>=MAX_string then nendstring:=MAX_string-1;
            END;
        END;

        IF (bC) THEN
            Result := TRUE
        ELSE
            Result := (Length(cLine) > 0);

        EXIT;
    END;

//    IF (EOF(BPGNFile) OR (nLastPos >= nFileSize)) THEN
    IF (nlastPos>=nFileSize) Then
    BEGIN
        Result := FALSE;
        EXIT;
    END;

    FOR nI := 0 TO (MAX_STRING - 1) DO
        cFileString[nI] := #0;

    TRY
        Read(BPGNFile, cFileString);
    EXCEPT
        ON E: EInOutError DO
            BEGIN
            END;
    END;

    nStartString := 0;
    nEndString   := StrLen(cFileString) - 1;
    if nEndString>=MaX_String then nendstring:=max_string-1;
    Result       := (nEndString >= 0);
END;

PROCEDURE TBPGN.GetActualGame(nBeginPos, nEndPos: LongInt);
VAR
    nRn, nStartPos, nI: LongInt;
    ef:boolean;
    strl:longint;
BEGIN
    cActualGame := '';

    IF ((nBeginPos + 1) <= MAX_STRING) THEN
    BEGIN
        Seek(BPGNFile, 0);
        nStartPos := -1;
    END
    ELSE
    IF (((nBeginPos + 1) MOD MAX_STRING) = 0) THEN
    BEGIN
        nRn := ((nBeginPos + 1) DIV MAX_STRING) - 1;
        Seek(BPGNFile, nRn);
        nStartPos := (nRn * MAX_STRING) - 1;
    END
    ELSE
    BEGIN
        nRn := (nBeginPos + 1) DIV MAX_STRING;
        Seek(BPGNFile, nRn);
        nStartPos := (nRn * MAX_STRING) - 1;
    END;


//    WHILE ((NOT EOF(BPGNFile)) AND (nStartPos < nEndPos)) DO
    while (nstartpos<nendpos) do
    BEGIN
        ef:=eof(bpgnfile);
        Fillchar(cFileString,sizeof(cFileString),#0);

        TRY

            Read(BPGNFile, cFileString);
        EXCEPT
            ON E: EInOutError DO
                BEGIN
                END;
        END;

        strl:=strlen(cfilestring);
        if strl>MAX_STRING then strl:=Max_STRING;
        FOR nI := 0 TO (Strl - 1) DO
        BEGIN
            Inc(nStartPos);
            IF (nStartPos > nEndPos) THEN
                BREAK;

            IF (nStartPos >= nBeginPos) THEN
                cActualGame := cActualGame + cFileString[nI];
        END;
        if ef then break;
    END;
END;

PROCEDURE TBPGN.GetActualGameText(nIndex: Integer);
VAR
    nBeginPos, nEndPos: LongInt;
    cLineStr: String;
    ps:longint;
BEGIN
//sets cActualGame to updated bpgn score for a current game, old(read from file) for all others.
//Does not work for massive header changes. Use GetUpdatedGame for that
    nBeginPos := getPosition(nIndex);
    IF (nIndex = nHeaderCount - 1) THEN
        nEndPos := nFileSize
    ELSE begin
        nEndPos := getPosition(nIndex+1) - 1;
    end;
    IF ((NOT bGameChange) or (nIndex<>nGameHeader)) THEN
    BEGIN
        GetActualGame(nBeginPos, nEndPos);
        EXIT;
    END;

    cLagString:='';
    GetUpdatedHeader(nIndex); // puts updated headers into cgameline

    { First display }
    IF (BPGNDisplay[1]) THEN
        cGameLine := cGameLine + '{D:}' + EOL_STR;


    { Initialize variable }
    cLineStr := '';


    { First comment }
    IF (Length(BPGNComment[1]) > 0) THEN
    BEGIN
        SaveComment(1, cLineStr);
        cGameLine := cGameLine + cLineStr + EOL_STR;
        cLineStr := '';
    END;


    { Go through all moves }
    SavePosition(1, cLineStr);
    IF (Length(cLineStr) > 0) THEN
        cGameLine := cGameLine + cLineStr + EOL_STR + EOL_STR
    ELSE
    IF (nBPGNIndex > 1) THEN
        cGameLine := cGameLine + EOL_STR;


    { Get result }
    cGameLine := cGameLine + GetaReason(nIndex) + EOL_STR;

    if bLagTag then
    begin
     ps:=Pos(LF_ASCII_CHAR+BPGN_RESULT_STR,cGameLine);
     insert('[Lag "'  + cLagString + '"]' + EOL_STR,cGameLine,ps+1);
    end;

    { Assign game to cActualGame }
    cActualGame := cGameLine;
END;

PROCEDURE TBPGN.GetGame(nBeginPos, nEndPos: LongInt);
VAR
    nRn, nStartPos, nI: LongInt;
    ef:boolean;
BEGIN
    cGameLine  := '';
    nGameSize  := 0;
    nGameIndex := 0;
    cToken     := '';
    cTmpSt     := '';
    nState     := BPGN_NORMAL;
    nLastState := BPGN_NORMAL;

    IF ((nBeginPos + 1) <= MAX_STRING) THEN
    BEGIN
        Seek(BPGNFile, 0);
        nStartPos := -1;
    END
    ELSE
    IF (((nBeginPos + 1) MOD MAX_STRING) = 0) THEN
    BEGIN
        nRn := ((nBeginPos + 1) DIV MAX_STRING) - 1;
        Seek(BPGNFile, nRn);
        nStartPos := (nRn * MAX_STRING) - 1;
    END
    ELSE
    BEGIN
        nRn := (nBeginPos + 1) DIV MAX_STRING;
        Seek(BPGNFile, nRn);
        nStartPos := (nRn * MAX_STRING) - 1;
    END;

//   WHILE ((NOT EOF(BPGNFile)) AND (nStartPos < nEndPos)) DO
    while (nstartpos<nendpos) do
    BEGIN
        ef:=eof(bpgnfile);
           fillchar(cFileSTring,sizeof(cFileString),#0);

        TRY
            Read(BPGNFile, cFileString);
        EXCEPT
            ON E: EInOutError DO
                BEGIN
                END;
        END;

        FOR nI := 0 TO (strlen(cFileString) - 1) DO
        BEGIN
            if ni>(MAX_STRING-1) then break;
            Inc(nStartPos);
            IF (nStartPos > nEndPos) THEN
                BREAK;

            IF (nStartPos >= nBeginPos) THEN
            BEGIN
                IF (cFileString[nI] = COMMENT_CR_CHAR) THEN
                    cGameLine := cGameLine + EOL_STR
                ELSE
                IF ((cFileString[nI] <> CR_ASCII_CHAR) AND (cFileString[nI] <> LF_ASCII_CHAR)) THEN
                    cGameLine := cGameLine + cFileString[nI];
            END;
        END;
        if ef then break;
    END;

    nGameSize := Length(cGameLine);

    IF (nGameSize > 0) THEN
        nGameIndex := 1
    ELSE
        nGameIndex := 0;
END;

PROCEDURE TBPGN.CopyChessIntoBPGNRecord(nIndex: Integer);
VAR
    nI, nJ: Integer;
BEGIN
    { Copy chess #1 }
    FOR nI := 1 TO 2 DO
    BEGIN
        BPGNArray[nIndex,1].aKingBX[nI] := Chess1.aKingBX[nI];
        BPGNArray[nIndex,1].aKingBY[nI] := Chess1.aKingBY[nI];

        BPGNArray[nIndex,1].aKCastle[nI] := Chess1.aKCastle[nI];
        BPGNArray[nIndex,1].aQCastle[nI] := Chess1.aQCastle[nI];
    END;

    BPGNArray[nIndex,1].mPrev.nFromX                   := Chess1.mPrev.nFromX;
    BPGNArray[nIndex,1].mPrev.nFromY                   := Chess1.mPrev.nFromY;
    BPGNArray[nIndex,1].mPrev.nToX                     := Chess1.mPrev.nToX;
    BPGNArray[nIndex,1].mPrev.nToY                     := Chess1.mPrev.nToY;
    BPGNArray[nIndex,1].mPrev.cPiece                   := Chess1.mPrev.cPiece;
    BPGNArray[nIndex,1].mPrev.cPromote                 := Chess1.mPrev.cPromote;
    BPGNArray[nIndex,1].mPrev.cCapture                 := Chess1.mPrev.cCapture;
    BPGNArray[nIndex,1].mPrev.cType                    := Chess1.mPrev.cType;
    BPGNArray[nIndex,1].mPrev.bCapturedIsPromotedPiece := Chess1.mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,1].aBoard  [nI, nJ] := Chess1.aBoard  [nI, nJ];
            BPGNArray[nIndex,1].aPromote[nI, nJ] := Chess1.aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,1].acPiece[nI, nJ] := Chess1.acPiece[nI, nJ];
            BPGNArray[nIndex,1].anPiece[nI, nJ] := Chess1.anPiece[nI, nJ];
        END;
    END;

    BPGNArray[nIndex,1].bWhitesMove := Chess1.bWhitesMove;
    BPGNArray[nIndex,1].cCaption    := cStartCap1;


    { Copy chess #2 }
    FOR nI := 1 TO 2 DO
    BEGIN
        BPGNArray[nIndex,2].aKingBX[nI] := Chess2.aKingBX[nI];
        BPGNArray[nIndex,2].aKingBY[nI] := Chess2.aKingBY[nI];

        BPGNArray[nIndex,2].aKCastle[nI] := Chess2.aKCastle[nI];
        BPGNArray[nIndex,2].aQCastle[nI] := Chess2.aQCastle[nI];
    END;

    BPGNArray[nIndex,2].mPrev.nFromX                   := Chess2.mPrev.nFromX;
    BPGNArray[nIndex,2].mPrev.nFromY                   := Chess2.mPrev.nFromY;
    BPGNArray[nIndex,2].mPrev.nToX                     := Chess2.mPrev.nToX;
    BPGNArray[nIndex,2].mPrev.nToY                     := Chess2.mPrev.nToY;
    BPGNArray[nIndex,2].mPrev.cPiece                   := Chess2.mPrev.cPiece;
    BPGNArray[nIndex,2].mPrev.cPromote                 := Chess2.mPrev.cPromote;
    BPGNArray[nIndex,2].mPrev.cCapture                 := Chess2.mPrev.cCapture;
    BPGNArray[nIndex,2].mPrev.cType                    := Chess2.mPrev.cType;
    BPGNArray[nIndex,2].mPrev.bCapturedIsPromotedPiece := Chess2.mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,2].aBoard  [nI, nJ] := Chess2.aBoard  [nI, nJ];
            BPGNArray[nIndex,2].aPromote[nI, nJ] := Chess2.aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,2].acPiece[nI, nJ] := Chess2.acPiece[nI, nJ];
            BPGNArray[nIndex,2].anPiece[nI, nJ] := Chess2.anPiece[nI, nJ];
        END;
    END;

    BPGNArray[nIndex,2].bWhitesMove := Chess2.bWhitesMove;
    BPGNArray[nIndex,2].cCaption    := cStartCap2;


    { Initialize other arrays }
    BPGNInUse [nIndex] := TRUE;
    BPGNParent[nIndex] := 0;
    BPGNMain  [nIndex] := 0;

    FOR nI := 1 TO MAX_NEXT DO
        BPGNNext[nIndex, nI] := 0;

    BPGNNumber[nIndex,1] := 0;
    BPGNNumber[nIndex,2] := 0;

    BPGNBoard  [nIndex] := ' ';
    BPGNMove   [nIndex] := '';
    BPGNAnno   [nIndex] := '';
    BPGNDisplay[nIndex] := FALSE;
    BPGNComment[nIndex] := '';
END;

PROCEDURE TBPGN.CopyBPGNRecordIntoChess(nIndex: Integer);
VAR
    nI, nJ: Integer;
BEGIN
    { Copy chess #1 }
    FOR nI := 1 TO 2 DO
    BEGIN
        Chess1.aKingBX[nI] := BPGNArray[nIndex,1].aKingBX[nI];
        Chess1.aKingBY[nI] := BPGNArray[nIndex,1].aKingBY[nI];

        Chess1.aKCastle[nI] := BPGNArray[nIndex,1].aKCastle[nI];
        Chess1.aQCastle[nI] := BPGNArray[nIndex,1].aQCastle[nI];
    END;

    Chess1.mPrev.nFromX                   := BPGNArray[nIndex,1].mPrev.nFromX;
    Chess1.mPrev.nFromY                   := BPGNArray[nIndex,1].mPrev.nFromY;
    Chess1.mPrev.nToX                     := BPGNArray[nIndex,1].mPrev.nToX;
    Chess1.mPrev.nToY                     := BPGNArray[nIndex,1].mPrev.nToY;
    Chess1.mPrev.cPiece                   := BPGNArray[nIndex,1].mPrev.cPiece;
    Chess1.mPrev.cPromote                 := BPGNArray[nIndex,1].mPrev.cPromote;
    Chess1.mPrev.cCapture                 := BPGNArray[nIndex,1].mPrev.cCapture;
    Chess1.mPrev.cType                    := BPGNArray[nIndex,1].mPrev.cType;
    Chess1.mPrev.bCapturedIsPromotedPiece := BPGNArray[nIndex,1].mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            Chess1.aBoard  [nI, nJ] := BPGNArray[nIndex,1].aBoard  [nI, nJ];
            Chess1.aPromote[nI, nJ] := BPGNArray[nIndex,1].aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            Chess1.acPiece[nI, nJ] := BPGNArray[nIndex,1].acPiece[nI, nJ];
            Chess1.anPiece[nI, nJ] := BPGNArray[nIndex,1].anPiece[nI, nJ];
        END;
    END;

    Chess1.bWhitesMove := BPGNArray[nIndex,1].bWhitesMove;


    { Copy chess #2 }
    FOR nI := 1 TO 2 DO
    BEGIN
        Chess2.aKingBX[nI] := BPGNArray[nIndex,2].aKingBX[nI];
        Chess2.aKingBY[nI] := BPGNArray[nIndex,2].aKingBY[nI];

        Chess2.aKCastle[nI] := BPGNArray[nIndex,2].aKCastle[nI];
        Chess2.aQCastle[nI] := BPGNArray[nIndex,2].aQCastle[nI];
    END;

    Chess2.mPrev.nFromX                   := BPGNArray[nIndex,2].mPrev.nFromX;
    Chess2.mPrev.nFromY                   := BPGNArray[nIndex,2].mPrev.nFromY;
    Chess2.mPrev.nToX                     := BPGNArray[nIndex,2].mPrev.nToX;
    Chess2.mPrev.nToY                     := BPGNArray[nIndex,2].mPrev.nToY;
    Chess2.mPrev.cPiece                   := BPGNArray[nIndex,2].mPrev.cPiece;
    Chess2.mPrev.cPromote                 := BPGNArray[nIndex,2].mPrev.cPromote;
    Chess2.mPrev.cCapture                 := BPGNArray[nIndex,2].mPrev.cCapture;
    Chess2.mPrev.cType                    := BPGNArray[nIndex,2].mPrev.cType;
    Chess2.mPrev.bCapturedIsPromotedPiece := BPGNArray[nIndex,2].mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            Chess2.aBoard  [nI, nJ] := BPGNArray[nIndex,2].aBoard  [nI, nJ];
            Chess2.aPromote[nI, nJ] := BPGNArray[nIndex,2].aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            Chess2.acPiece[nI, nJ] := BPGNArray[nIndex,2].acPiece[nI, nJ];
            Chess2.anPiece[nI, nJ] := BPGNArray[nIndex,2].anPiece[nI, nJ];
        END;
    END;

    Chess2.bWhitesMove := BPGNArray[nIndex,2].bWhitesMove;
END;

PROCEDURE TBPGN.CopyBoardPiecesIntoBPGNRecord(cC1, cC2: TChess; nIndex: Integer);
VAR
    nI, nJ: Integer;
BEGIN
    { Copy from board #1 }
    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,1].acPiece[nI, nJ] := cC1.acPiece[nI, nJ];
            BPGNArray[nIndex,1].anPiece[nI, nJ] := cC1.anPiece[nI, nJ];
        END;
    END;

    { Copy from board #2 }
    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,2].acPiece[nI, nJ] := cC2.acPiece[nI, nJ];
            BPGNArray[nIndex,2].anPiece[nI, nJ] := cC2.anPiece[nI, nJ];
        END;
    END;
END;

PROCEDURE TBPGN.CopyBoardsIntoBPGNRecord(cC1, cC2: TChess; nIndex: Integer; cCap1, cCap2: String);
VAR
    nI, nJ: Integer;
BEGIN
    { Copy from board #1 }
    FOR nI := 1 TO 2 DO
    BEGIN
        BPGNArray[nIndex,1].aKingBX[nI] := cC1.aKingBX[nI];
        BPGNArray[nIndex,1].aKingBY[nI] := cC1.aKingBY[nI];

        BPGNArray[nIndex,1].aKCastle[nI] := cC1.aKCastle[nI];
        BPGNArray[nIndex,1].aQCastle[nI] := cC1.aQCastle[nI];
    END;

    BPGNArray[nIndex,1].mPrev.nFromX                   := cC1.mPrev.nFromX;
    BPGNArray[nIndex,1].mPrev.nFromY                   := cC1.mPrev.nFromY;
    BPGNArray[nIndex,1].mPrev.nToX                     := cC1.mPrev.nToX;
    BPGNArray[nIndex,1].mPrev.nToY                     := cC1.mPrev.nToY;
    BPGNArray[nIndex,1].mPrev.cPiece                   := cC1.mPrev.cPiece;
    BPGNArray[nIndex,1].mPrev.cPromote                 := cC1.mPrev.cPromote;
    BPGNArray[nIndex,1].mPrev.cCapture                 := cC1.mPrev.cCapture;
    BPGNArray[nIndex,1].mPrev.cType                    := cC1.mPrev.cType;
    BPGNArray[nIndex,1].mPrev.bCapturedIsPromotedPiece := cC1.mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,1].aBoard  [nI, nJ] := cC1.aBoard  [nI, nJ];
            BPGNArray[nIndex,1].aPromote[nI, nJ] := cC1.aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,1].acPiece[nI, nJ] := cC1.acPiece[nI, nJ];
            BPGNArray[nIndex,1].anPiece[nI, nJ] := cC1.anPiece[nI, nJ];
        END;
    END;

    BPGNArray[nIndex,1].bWhitesMove := cC1.bWhitesMove;
    BPGNArray[nIndex,1].cCaption    := cCap1;


    { Copy from board #2 }
    FOR nI := 1 TO 2 DO
    BEGIN
        BPGNArray[nIndex,2].aKingBX[nI] := cC2.aKingBX[nI];
        BPGNArray[nIndex,2].aKingBY[nI] := cC2.aKingBY[nI];

        BPGNArray[nIndex,2].aKCastle[nI] := cC2.aKCastle[nI];
        BPGNArray[nIndex,2].aQCastle[nI] := cC2.aQCastle[nI];
    END;

    BPGNArray[nIndex,2].mPrev.nFromX                   := cC2.mPrev.nFromX;
    BPGNArray[nIndex,2].mPrev.nFromY                   := cC2.mPrev.nFromY;
    BPGNArray[nIndex,2].mPrev.nToX                     := cC2.mPrev.nToX;
    BPGNArray[nIndex,2].mPrev.nToY                     := cC2.mPrev.nToY;
    BPGNArray[nIndex,2].mPrev.cPiece                   := cC2.mPrev.cPiece;
    BPGNArray[nIndex,2].mPrev.cPromote                 := cC2.mPrev.cPromote;
    BPGNArray[nIndex,2].mPrev.cCapture                 := cC2.mPrev.cCapture;
    BPGNArray[nIndex,2].mPrev.cType                    := cC2.mPrev.cType;
    BPGNArray[nIndex,2].mPrev.bCapturedIsPromotedPiece := cC2.mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,2].aBoard  [nI, nJ] := cC2.aBoard  [nI, nJ];
            BPGNArray[nIndex,2].aPromote[nI, nJ] := cC2.aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            BPGNArray[nIndex,2].acPiece[nI, nJ] := cC2.acPiece[nI, nJ];
            BPGNArray[nIndex,2].anPiece[nI, nJ] := cC2.anPiece[nI, nJ];
        END;
    END;

    BPGNArray[nIndex,2].bWhitesMove := cC2.bWhitesMove;
    BPGNArray[nIndex,2].cCaption    := cCap2;
END;

PROCEDURE TBPGN.CopyBPGNRecordIntoBoards(cC1, cC2: TChess; nIndex: Integer);
VAR
    nI, nJ: Integer;
BEGIN
    { Copy into board #1 }

    FOR nI := 1 TO 2 DO
    BEGIN
        cC1.aKingBX[nI] := BPGNArray[nIndex,1].aKingBX[nI];
        cC1.aKingBY[nI] := BPGNArray[nIndex,1].aKingBY[nI];

        cC1.aKCastle[nI] := BPGNArray[nIndex,1].aKCastle[nI];
        cC1.aQCastle[nI] := BPGNArray[nIndex,1].aQCastle[nI];
    END;

    cC1.mPrev.nFromX                   := BPGNArray[nIndex,1].mPrev.nFromX;
    cC1.mPrev.nFromY                   := BPGNArray[nIndex,1].mPrev.nFromY;
    cC1.mPrev.nToX                     := BPGNArray[nIndex,1].mPrev.nToX;
    cC1.mPrev.nToY                     := BPGNArray[nIndex,1].mPrev.nToY;
    cC1.mPrev.cPiece                   := BPGNArray[nIndex,1].mPrev.cPiece;
    cC1.mPrev.cPromote                 := BPGNArray[nIndex,1].mPrev.cPromote;
    cC1.mPrev.cCapture                 := BPGNArray[nIndex,1].mPrev.cCapture;
    cC1.mPrev.cType                    := BPGNArray[nIndex,1].mPrev.cType;
    cC1.mPrev.bCapturedIsPromotedPiece := BPGNArray[nIndex,1].mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            cC1.aBoard  [nI, nJ] := BPGNArray[nIndex,1].aBoard  [nI, nJ];
            cC1.aPromote[nI, nJ] := BPGNArray[nIndex,1].aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            cC1.acPiece[nI, nJ] := BPGNArray[nIndex,1].acPiece[nI, nJ];
            cC1.anPiece[nI, nJ] := BPGNArray[nIndex,1].anPiece[nI, nJ];
        END;
    END;

    cC1.bWhitesMove := BPGNArray[nIndex,1].bWhitesMove;


    { Copy into board #2 }

    FOR nI := 1 TO 2 DO
    BEGIN
        cC2.aKingBX[nI] := BPGNArray[nIndex,2].aKingBX[nI];
        cC2.aKingBY[nI] := BPGNArray[nIndex,2].aKingBY[nI];

        cC2.aKCastle[nI] := BPGNArray[nIndex,2].aKCastle[nI];
        cC2.aQCastle[nI] := BPGNArray[nIndex,2].aQCastle[nI];
    END;

    cC2.mPrev.nFromX                   := BPGNArray[nIndex,2].mPrev.nFromX;
    cC2.mPrev.nFromY                   := BPGNArray[nIndex,2].mPrev.nFromY;
    cC2.mPrev.nToX                     := BPGNArray[nIndex,2].mPrev.nToX;
    cC2.mPrev.nToY                     := BPGNArray[nIndex,2].mPrev.nToY;
    cC2.mPrev.cPiece                   := BPGNArray[nIndex,2].mPrev.cPiece;
    cC2.mPrev.cPromote                 := BPGNArray[nIndex,2].mPrev.cPromote;
    cC2.mPrev.cCapture                 := BPGNArray[nIndex,2].mPrev.cCapture;
    cC2.mPrev.cType                    := BPGNArray[nIndex,2].mPrev.cType;
    cC2.mPrev.bCapturedIsPromotedPiece := BPGNArray[nIndex,2].mPrev.bCapturedIsPromotedPiece;

    FOR nI := 1 TO 8 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            cC2.aBoard  [nI, nJ] := BPGNArray[nIndex,2].aBoard  [nI, nJ];
            cC2.aPromote[nI, nJ] := BPGNArray[nIndex,2].aPromote[nI, nJ];
        END;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 8 DO
        BEGIN
            cC2.acPiece[nI, nJ] := BPGNArray[nIndex,2].acPiece[nI, nJ];
            cC2.anPiece[nI, nJ] := BPGNArray[nIndex,2].anPiece[nI, nJ];
        END;
    END;

    cC2.bWhitesMove := BPGNArray[nIndex,2].bWhitesMove;
END;

FUNCTION TBPGN.GetToken: Boolean;
VAR
    nI: LongInt;
BEGIN
    IF ((nGameIndex = 0) OR (nGameSize = 0) OR (nGameIndex > nGameSize)) THEN
    BEGIN
        cToken := '';
        Result := FALSE;
        EXIT;
    END;


    nI := (nGameIndex - 1);
    WHILE (nI < nGameSize) DO
    BEGIN
        cToken := '';
        cTmpSt := '';
        WHILE (nI < nGameSize) DO
        BEGIN
            Inc(nI);


            { Normal state }
            IF (nState = BPGN_NORMAL) THEN
            BEGIN
                IF ((cGameLine[nI] = ' ') OR (Ord(cGameLine[nI]) = TAB_ASCII_NUM)) THEN
                    BREAK;

                IF ((cGameLine[nI] = '[') OR (cGameLine[nI] = ']') OR
                    (cGameLine[nI] = '(') OR (cGameLine[nI] = ')') OR
                    (cGameLine[nI] = '<') OR (cGameLine[nI] = '>') OR
                    (cGameLine[nI] = '(') OR (cGameLine[nI] = ')') OR
                    (cGameLine[nI] = '.') OR (cGameLine[nI] = '*') OR
                    (cGameLine[nI] = '}')) THEN
                BEGIN
                    cToken := cGameLine[nI];
                    BREAK;
                END;

                IF (cGameLine[nI] = '"') THEN
                BEGIN
                    nState := BPGN_STRING;
                    CONTINUE;
                END;

                IF (cGameLine[nI] = '0') THEN
                BEGIN
                    IF ((nI + 2) <= nGameSize) THEN
                    BEGIN
                        IF ((cGameLine[nI + 1] = '-') AND (cGameLine[nI + 2] = '1')) THEN
                        BEGIN
                            cToken := '0-1';
                            nI     := nI + 2;
                            BREAK;
                        END;
                    END;
                END;

                IF (cGameLine[nI] = '1') THEN
                BEGIN
                    IF ((nI + 2) <= nGameSize) THEN
                    BEGIN
                        IF ((cGameLine[nI + 1] = '-') AND (cGameLine[nI + 2] = '0')) THEN
                        BEGIN
                            cToken := '1-0';
                            nI     := nI + 2;
                            BREAK;
                        END;
                    END;

                    IF ((nI + 6) <= nGameSize) THEN
                    BEGIN
                        IF ((cGameLine[nI + 1] = '/') AND (cGameLine[nI + 2] = '2') AND
                            (cGameLine[nI + 3] = '-') AND (cGameLine[nI + 4] = '1') AND
                            (cGameLine[nI + 5] = '/') AND (cGameLine[nI + 6] = '2')) THEN
                        BEGIN
                            cToken := '1/2-1/2';
                            nI     := nI + 6;
                            BREAK;
                        END;
                    END;
                END;

                IF ((cGameLine[nI] >= '0') AND (cGameLine[nI] <= '9')) THEN
                BEGIN
                    nState := BPGN_INTEGER;
                    cTmpSt := cGameLine[nI];
                    CONTINUE;
                END;

                IF (cGameLine[nI] = ';') THEN
                BEGIN
                    nState := BPGN_COMMENT1;
                    cTmpSt := '';
                    BREAK;
                END;

                IF (cGameLine[nI] = '{') THEN
                BEGIN
                    nState := BPGN_COMMENT2;
                    cTmpSt := cGameLine[nI];
                    BREAK;
                END;

                IF (cGameLine[nI] = '$') THEN
                BEGIN
                    nState := BPGN_NAG;
                    cTmpSt := cGameLine[nI];
                    CONTINUE;
                END;

                IF (((cGameLine[nI] >= 'a') AND (cGameLine[nI] <= 'z')) OR
                    ((cGameLine[nI] >= 'A') AND (cGameLine[nI] <= 'Z'))) THEN
                BEGIN
                    nState := BPGN_SYMBOL;
                    cTmpSt := cGameLine[nI];
                    CONTINUE;
                END;
            END;


            { String token state }
            IF (nState = BPGN_STRING) THEN
            BEGIN
                IF (cGameLine[nI] = '\') THEN
                BEGIN
                    IF (nI < nGameSize) THEN
                    BEGIN
                        Inc(nI);

                        IF (cGameLine[nI] = '"') THEN
                        BEGIN
                            cTmpSt := cTmpSt + cGameLine[nI];
                            CONTINUE;
                        END;

                        IF (cGameLine[nI] = '\') THEN
                        BEGIN
                            cTmpSt := cTmpSt + cGameLine[nI];
                            CONTINUE;
                        END;

                        cTmpSt := cTmpSt + '\' + cGameLine[nI];
                        CONTINUE;
                    END;

                    cTmpSt := cTmpSt + cGameLine[nI];
                    BREAK;
                END;

                IF (cGameLine[nI] = '"') THEN
                    BREAK;

                cTmpSt := cTmpSt + cGameLine[nI];
                CONTINUE;
            END;


            { Integer token state }
            IF (nState = BPGN_INTEGER) THEN
            BEGIN
                IF ((cGameLine[nI] >= '0') AND (cGameLine[nI] <= '9')) THEN
                BEGIN
                    cTmpSt := cTmpSt + cGameLine[nI];
                    CONTINUE;
                END;

                Dec(nI);
                BREAK;
            END;


            { Comment token state }
            IF (nState = BPGN_COMMENT2) THEN
            BEGIN
                IF (cGameLine[nI] = '}') THEN
                BEGIN
                    nState := BPGN_NORMAL;
                    cToken := cTmpSt;

                    Dec(nI);
                    BREAK;
                END;

                cTmpSt := cTmpSt + cGameLine[nI];
                CONTINUE;
            END;


            { NAG (Numeric Annotation Glyph) token state }
            IF (nState = BPGN_NAG) THEN
            BEGIN
                IF ((cGameLine[nI] >= '0') AND (cGameLine[nI] <= '9')) THEN
                BEGIN
                    cTmpSt := cTmpSt + cGameLine[nI];
                    CONTINUE;
                END;

                Dec(nI);
                BREAK;
            END;


            { Symbol token state }
            IF (nState = BPGN_SYMBOL) THEN
            BEGIN
                IF ((cGameLine[nI] = ' ') OR (Ord(cGameLine[nI]) = TAB_ASCII_NUM)) THEN
                    BREAK;

                IF (((cGameLine[nI] >= 'a') AND (cGameLine[nI] <= 'z')) OR
                    ((cGameLine[nI] >= 'A') AND (cGameLine[nI] <= 'Z'))) THEN
                BEGIN
                    cTmpSt := cTmpSt + cGameLine[nI];
                    CONTINUE;
                END;

                IF ((cGameLine[nI] >= '0') AND (cGameLine[nI] <= '9')) THEN
                BEGIN
                    cTmpSt := cTmpSt + cGameLine[nI];
                    CONTINUE;
                END;

                IF ((cGameLine[nI] = '_') OR (cGameLine[nI] = '+') OR
                    (cGameLine[nI] = '#') OR (cGameLine[nI] = '=') OR
                    (cGameLine[nI] = ':') OR (cGameLine[nI] = '-') OR
                    (cGameLine[nI] = '@')) THEN
                BEGIN
                    cTmpSt := cTmpSt + cGameLine[nI];
                    CONTINUE;
                END;

                Dec(nI);
                BREAK;
            END;
        END;


        { Set BPGN state }
        nLastState := nState;

        IF ((nState = BPGN_STRING  )  OR
            (nState = BPGN_INTEGER )  OR
            (nState = BPGN_COMMENT1)  OR
            (nState = BPGN_NAG     )  OR
            (nState = BPGN_SYMBOL  )  OR
            (nState = BPGN_COMMENT1)) THEN
        BEGIN
            nState := BPGN_NORMAL;
            cToken := cTmpSt;
        END
        ELSE
        IF (nState = BPGN_COMMENT2) THEN
            cToken := cTmpSt;


        IF (Length(cToken) > 0) THEN
            BREAK;
    END;


    { Set game index }
    nGameIndex := nI + 1;


    Result := (Length(cToken) > 0);
END;

PROCEDURE TBPGN.Backup(nI: Integer);
BEGIN
    aGameIndex[nI] := nGameIndex;
    aState    [nI] := nState;
END;

PROCEDURE TBPGN.Restore(nI: Integer);
BEGIN
    nGameIndex := aGameIndex[nI];
    nState     := aState    [nI];
END;

FUNCTION TBPGN.LoadBPGNStuff: Boolean;
VAR
    cTmp: String;
BEGIN
    (* Find { *)
    Backup(3);
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        Result    := FALSE;
        EXIT;
    END;
    IF (nLastState <> BPGN_COMMENT2) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;


    (* Find A:, D:, C: *)
    IF (NOT GetToken) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;
{    IF ((CompareStr(Copy(cToken,1,2), 'A:') <> 0)  AND
        (CompareStr(Copy(cToken,1,2), 'D:') <> 0)  AND
        (CompareStr(Copy(cToken,1,2), 'C:') <> 0)) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;}
    //commented to treat {X:} as unknown TAG, i.e legal.
    if copy(cToken,2,1)<>':' then
    begin
        Restore(3);
        Result := FALSE;
        EXIT;
    end;

    cTmp := cToken;

    (* Find } *)
    IF (NOT GetToken) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;
    IF (CompareStr(cToken, '}') <> 0) THEN
    BEGIN
        Restore(3);
        Result := FALSE;
        EXIT;
    END;


    IF (CompareStr(Copy(cTmp,1,2), 'A:') = 0) THEN
        BPGNAnno[nBPGNIndex] := Copy(cTmp, 3, Length(cTmp) - 2)
    ELSE
    IF (CompareStr(Copy(cTmp,1,2), 'D:') = 0) THEN
        BPGNDisplay[nBPGNIndex] := TRUE
    ELSE
    IF (CompareStr(Copy(cTmp,1,2), 'C:') = 0) THEN
    BEGIN
        BPGNComment[nBPGNIndex] := Copy(cTmp, 3, Length(cTmp) - 2);

        IF (Length(BPGNComment[nBPGNIndex]) > MAX_MEMO_CHAR) THEN
            BPGNComment[nBPGNIndex] := Copy(BPGNComment[nBPGNIndex],1,MAX_MEMO_CHAR);
    END;

    Result := TRUE;
END;

PROCEDURE TBPGN.LoadBPGNResult(nIndex: Integer);
VAR
    cTmp, cTmp1, cTmp2: String;
BEGIN
    (* Game over?  Find { *)
    Backup(4);
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_COMMENT2) THEN
    BEGIN
        Restore(4);
        EXIT;
    END;
    cTmp := cToken;


    (* Find game over reason *)
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        Restore(4);
        EXIT;
    END;
    cTmp1 := cToken;

    (* Find } *)
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        Restore(4);
        EXIT;
    END;
    IF (CompareStr(cToken, '}') <> 0) THEN
    BEGIN
        Restore(4);
        EXIT;
    END;
    cTmp2 := cToken;


    { Find game result }
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        Restore(4);
        EXIT;
    END;
    IF ((CompareStr(cToken, BPGN_WHITE_WINS) = 0)  OR
        (CompareStr(cToken, BPGN_BLACK_WINS) = 0)  OR
        (CompareStr(cToken, BPGN_DRAW)       = 0)  OR
        (CompareStr(cToken, BPGN_UNKNOWN)    = 0)) THEN
    BEGIN
        bGameDone   := TRUE;
        cBPGNResult := cTmp + cTmp1 + cTmp2 + ' ' + cToken;
        EXIT;
    END;


    Restore(4);
END;

PROCEDURE TBPGN.LoadBPGNMove(nParent: Integer; VAR nMoveA, nMoveB: Integer; VAR cCap1, cCap2: String; bMain: Boolean);
VAR
    i:integer;
    cBoard: Char;
    cMove, cTmp: String;
    nMove, nTime, nTime1W, nTime1B, nTime2W, nTime2B, nCur, nI, nTmp: Integer;
BEGIN
    { Load result }
    LoadBPGNResult(nParent);
    IF (bGameDone) THEN
        EXIT;


    { Find move # }
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_INTEGER) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Invalid move number [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    VAL(cToken, nMove, nTmp);
    IF (nTmp <> 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Invalid move number [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;


    { Find symbol A, a, B, b }
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_SYMBOL) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Expecting move symbol A, a, B, b but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    IF ((CompareStr(cToken, 'A') <> 0)  AND
        (CompareStr(cToken, 'a') <> 0)  AND
        (CompareStr(cToken, 'B') <> 0)  AND
        (CompareStr(cToken, 'b') <> 0)) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Expecting move symbol A, a, B, b but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;


    { Check who's move and move number }
    IF (CompareStr(cToken, 'A') = 0) THEN
    BEGIN
        IF (nMoveA = 0) THEN
           nMoveA := 1
        ELSE
           Inc(nMoveA);

        IF (nMove <> nMoveA)  THEN
        BEGIN
         if ((nmovea=1)and (bscrewfirstmove)) then
         begin
          nmovea:=nmove;
          i:=nparent;
          while (i>0) do
          begin
           BPGNnumber[i,1]:=nmove-1;
           i:=bpgnparent[i];
          end;
         end
         else begin
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #1 should be move number ' +
                          IntToStr(nMoveA) + ' not [' + IntToStr(nMove) + '].';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
         end;
        END;
        IF (NOT Chess1.bWhitesMove) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #1 should be black''s move (' + IntToStr(nMoveA) + ').';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;
        cBoard := 'A';
    END
    ELSE
    IF (CompareStr(cToken, 'a') = 0) THEN
    BEGIN
        IF (nMoveA = 0) THEN
           nMoveA := 1;

        IF (nMove <> nMoveA) THEN
        BEGIN
         if ((nmovea=1)and (bscrewfirstmove)) then
         begin
          nmovea:=nmove;
          i:=nparent;
          while (i>0) do
          begin
           BPGNnumber[i,1]:=nmove;
           i:=bpgnparent[i];
          end;
         end
         else begin
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #1 should be move number ' +
                          IntToStr(nMoveA) + ' not [' + IntToStr(nMove) + '].';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
         end;
        END;
        IF (Chess1.bWhitesMove) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #1 should be white''s move (' + IntToStr(nMoveA) + ').';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;
        cBoard := 'a';
    END
    ELSE
    IF (CompareStr(cToken, 'B') = 0) THEN
    BEGIN
        IF (nMoveB = 0) THEN
           nMoveB := 1
        ELSE
           Inc(nMoveB);

        IF (nMove <> nMoveB) THEN
        BEGIN
         if ((nmoveb=1)and (bscrewfirstmove)) then
         begin
          nmoveb:=nmove;
          i:=nparent;
          while (i>0) do
          begin
           BPGNnumber[i,2]:=nmove-1;
           i:=bpgnparent[i];
          end;
         end
         else begin
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #2 should be move number ' +
                          IntToStr(nMoveB) + ' not [' + IntToStr(nMove) + '].';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
         end;
        END;
        IF (NOT Chess2.bWhitesMove) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #2 should be black''s move (' + IntToStr(nMoveB) + ').';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;
        cBoard := 'B';
    END
    ELSE
    IF (CompareStr(cToken, 'b') = 0) THEN
    BEGIN
        IF (nMoveB = 0) THEN
           nMoveB := 1;

        IF (nMove <> nMoveB) THEN
        BEGIN
         if ((nmoveb=1)and (bscrewfirstmove)) then
         begin
          nmoveb:=nmove;
          i:=nparent;
          while (i>0) do
          begin
           BPGNnumber[i,2]:=nmove;
           i:=bpgnparent[i];
          end;
         end
         else begin
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #2 should be move number ' +
                          IntToStr(nMoveB) + ' not [' + IntToStr(nMove) + '].';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
         end;
        END;
        IF (Chess2.bWhitesMove) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                          'Board #2 should be white''s move (' + IntToStr(nMoveB) + ').';
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;
        cBoard := 'b';
    END;


    { Find symbol . }
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Expecting move symbol . but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    IF (CompareStr(cToken, '.') <> 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Expecting move symbol . but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;


    { Get move }
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_SYMBOL) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Expecting move but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;

    { Look at move }
    IF (cBoard = 'A') THEN
    BEGIN
        cTmp := Chess1.ExecAlgebraicMove(nMove, cBoard, cToken, nTmp);
        IF (nTmp = 0) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Board #1: ' + cTmp;
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;

        cMove := Chess1.GetRealMove(nTmp, cToken);

        Chess1.ExecMove(nTmp);

        IF (Pos(Crazyhouse_Id,lowercase(getevent(nGameHeader))) = 0) THEN
        BEGIN
            IF ((Chess1.aMove[nTmp].bCapturedIsPromotedPiece) or (Chess1.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
if not bairdrop then Chess2.SetPieces(Chess2.GetWhitePieceStr, Chess2.GetBlackPieceStr + Chess1.GetBlackPawn)
              else   chess2.AddDropPiece(chess1.GetBlackPawn)
            ELSE
            BEGIN
                IF (NOT Chess1.IsEmpty(Chess1.aMove[nTmp].cCapture)) THEN
if not bairdrop then Chess2.SetPieces(Chess2.GetWhitePieceStr, Chess2.GetBlackPieceStr + Chess1.aMove[nTmp].cCapture)
               else chess2.AddDropPiece(chess1.amove[ntmp].cCapture);
            END;
        END
        ELSE
        BEGIN
            IF ((Chess1.aMove[nTmp].bCapturedIsPromotedPiece) or (Chess1.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
              Chess1.SetPieces(Chess1.GetWhitePieceStr + Chess1.GetWhitePawn, Chess1.GetBlackPieceStr)
//             else chess1.AddDropPiece(chess1.GetWhitePawn)
            ELSE
            BEGIN
                IF (NOT Chess1.IsEmpty(Chess1.aMove[nTmp].cCapture)) THEN
                    Chess1.SetPieces(Chess1.GetWhitePieceStr + Chess1.GetRevPiece(Chess1.aMove[nTmp].cCapture), Chess1.GetBlackPieceStr);
//                 chess1.AddDropPiece(chess1.GetRevPiece(chess1.amove[ntmp].cCapture));
            END;
        END;
    END
    ELSE
    IF (cBoard = 'a') THEN
    BEGIN
        cTmp := Chess1.ExecAlgebraicMove(nMove, cBoard, cToken, nTmp);
        IF (nTmp = 0) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Board #1: ' + cTmp;
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;

        cMove := Chess1.GetRealMove(nTmp, cToken);

        Chess1.ExecMove(nTmp);

        IF (Pos(Crazyhouse_Id,lowercase(getevent(nGameHeader)))= 0) THEN
        BEGIN
            IF ((Chess1.aMove[nTmp].bCapturedIsPromotedPiece)or (Chess1.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
if not bairdrop then Chess2.SetPieces(Chess2.GetWhitePieceStr + Chess1.GetWhitePawn, Chess2.GetBlackPieceStr)
                else chess2.AddDropPiece(chess1.GetWhitePawn)
            ELSE
            BEGIN
                IF (NOT Chess1.IsEmpty(Chess1.aMove[nTmp].cCapture)) THEN
if not bairdrop then Chess2.SetPieces(Chess2.GetWhitePieceStr + Chess1.aMove[nTmp].cCapture, Chess2.GetBlackPieceStr)
                else chess2.AddDropPiece(chess1.amove[ntmp].cCapture);
            END;
        END
        ELSE
        BEGIN
            IF ((Chess1.aMove[nTmp].bCapturedIsPromotedPiece)or (Chess1.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
                Chess1.SetPieces(Chess1.GetWhitePieceStr, Chess1.GetBlackPieceStr + Chess1.GetBlackPawn)
//                chess1.AddDropPiece(chess1.GetBlackPawn)
            ELSE
            BEGIN
                IF (NOT Chess1.IsEmpty(Chess1.aMove[nTmp].cCapture)) THEN
                    Chess1.SetPieces(Chess1.GetWhitePieceStr, Chess1.GetBlackPieceStr + Chess1.GetRevPiece(Chess1.aMove[nTmp].cCapture));
//                chess1.AddDropPiece(chess1.GetRevPiece(chess1.amove[ntmp].cCapture));
            END;
        END;
    END
    ELSE
    IF (cBoard = 'B') THEN
    BEGIN
        cTmp := Chess2.ExecAlgebraicMove(nMove, cBoard, cToken, nTmp);
        IF (nTmp = 0) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Board #2: ' + cTmp;
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;

        cMove := Chess2.GetRealMove(nTmp, cToken);

        Chess2.ExecMove(nTmp);

        IF (Pos(Crazyhouse_Id,lowercase(getevent(nGameHeader))) = 0) THEN
        BEGIN
            IF ((Chess2.aMove[nTmp].bCapturedIsPromotedPiece)or (Chess2.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
if not bairdrop then Chess1.SetPieces(Chess1.GetWhitePieceStr, Chess1.GetBlackPieceStr + Chess2.GetBlackPawn)
                else chess1.AddDropPiece(chess2.GetBlackPawn)
            ELSE
            BEGIN
                IF (NOT Chess2.IsEmpty(Chess2.aMove[nTmp].cCapture)) THEN
if not bairdrop then  Chess1.SetPieces(Chess1.GetWhitePieceStr, Chess1.GetBlackPieceStr + Chess2.aMove[nTmp].cCapture)
                else chess1.AddDropPiece(chess2.amove[ntmp].cCapture);
            END;
        END
        ELSE
        BEGIN
            IF ((Chess2.aMove[nTmp].bCapturedIsPromotedPiece)or (Chess2.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
                Chess2.SetPieces(Chess2.GetWhitePieceStr + Chess2.GetWhitePawn, Chess2.GetBlackPieceStr)
//                chess2.adddroppiece(chess2.GetWhitePawn)
            ELSE
            BEGIN
                IF (NOT Chess2.IsEmpty(Chess2.aMove[nTmp].cCapture)) THEN
                    Chess2.SetPieces(Chess2.GetWhitePieceStr + Chess2.GetRevPiece(Chess2.aMove[nTmp].cCapture), Chess2.GetBlackPieceStr);
//                chess2.AddDropPiece(chess2.GetRevPiece(chess2.amove[ntmp].cCapture));
            END;
        END;
    END
    ELSE
    IF (cBoard = 'b') THEN
    BEGIN
        cTmp := Chess2.ExecAlgebraicMove(nMove, cBoard, cToken, nTmp);
        IF (nTmp = 0) THEN
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR + 'Board #2: ' + cTmp;
            IF (bPromptError) THEN wError(cResult);
            EXIT;
        END;

        cMove := Chess2.GetRealMove(nTmp, cToken);

        Chess2.ExecMove(nTmp);

        IF (Pos(Crazyhouse_Id,lowercase(getevent(nGameHeader))) = 0) THEN
        BEGIN
            IF ((Chess2.aMove[nTmp].bCapturedIsPromotedPiece)or (Chess2.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
if not bairdrop then Chess1.SetPieces(Chess1.GetWhitePieceStr + Chess2.GetWhitePawn, Chess1.GetBlackPieceStr)
                else chess1.AddDropPiece(chess2.GetWhitePawn)
            ELSE
            BEGIN
                IF (NOT Chess2.IsEmpty(Chess2.aMove[nTmp].cCapture)) THEN
if not bairdrop then Chess1.SetPieces(Chess1.GetWhitePieceStr + Chess2.aMove[nTmp].cCapture, Chess1.GetBlackPieceStr)
                else chess1.AddDropPiece(Chess2.aMove[nTmp].cCapture);
            END;
        END
        ELSE
        BEGIN
            IF ((Chess2.aMove[nTmp].bCapturedIsPromotedPiece)or (Chess2.aMove[nTmp].ctype=MOVE_ENPASANT)) THEN
                Chess2.SetPieces(Chess2.GetWhitePieceStr, Chess2.GetBlackPieceStr + Chess2.GetBlackPawn)
//                else chess2.AddDropPiece(chess2.GetBlackPawn)
            ELSE
            BEGIN
                IF (NOT Chess2.IsEmpty(Chess2.aMove[nTmp].cCapture)) THEN
                   Chess2.SetPieces(Chess2.GetWhitePieceStr, Chess2.GetBlackPieceStr + Chess2.GetRevPiece(Chess2.aMove[nTmp].cCapture));
//                chess2.AddDropPiece(chess2.GetRevPiece(Chess2.aMove[nTmp].cCapture));
            END;
        END;
    END;


    { Find time }
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_COMMENT2) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Invalid time [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Invalid time [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    nTime := GetCommentTime(cToken);
    IF (NOT GetToken) THEN
    BEGIN
        bGameDone := TRUE;
        EXIT;
    END;
    IF (nLastState <> BPGN_NORMAL) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Expecting } but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    IF (CompareStr(cToken, '}') <> 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Expecting } but found [' + cToken + '].';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;


    { Store game }
    IF (nBPGNIndex >= MAX_POSITION) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Maximum # of positions exceeded (' + IntToStr(MAX_POSITION) + ').';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    Inc(nBPGNIndex);
    CopyChessIntoBPGNRecord(nBPGNIndex);

    nCur := 0;
    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nParent,nI] = 0) THEN
        BEGIN
            nCur := nI;
            BREAK;
        END;
    END;
    IF (nCur = 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Maximum # of next moves per position exceeded (' + IntToStr(MAX_NEXT) + ').';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    BPGNNext[nParent,nCur] := nBPGNIndex;
    BPGNIsMain[nBPGNIndex] := bMain;
    BPGNParent[nBPGNIndex] := nParent;

    IF (BPGNMain[nParent] = 0) THEN
        BPGNMain[nParent] := nBPGNIndex;

    BPGNNumber[nBPGNIndex,1] := nMoveA;
    BPGNNumber[nBPGNIndex,2] := nMoveB;

    BPGNBoard [nBPGNIndex] := cBoard;
    BPGNMove  [nBPGNIndex] := cMove;


    { Adjust time }
    nTime1W := BPGNArray[nParent,1].nWTime;
    nTime1B := BPGNArray[nParent,1].nBTime;
    nTime2W := BPGNArray[nParent,2].nWTime;
    nTime2B := BPGNArray[nParent,2].nBTime;
    BPGNArray[nBPGNIndex,1].nWLag := BPGNArray[nParent,1].nWLag;
    BPGNArray[nBPGNIndex,1].nbLag := BPGNArray[nParent,1].nBLag;
    BPGNArray[nBPGNIndex,2].nWLag := BPGNArray[nParent,2].nWLag;
    BPGNArray[nBPGNIndex,2].nbLag := BPGNArray[nParent,2].nBLag;
    BPGNLag[nBPGNIndex]:=GetNextLag(nBPGNIndex);
    if BPGNLag[nBPGNindex]=-1 then
    begin
     BPGNLag[nBPGNindex]:=0;
     cResult:= cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) +', '+
     'move:'+Inttostr(nmove)+cboard+'.'+cmove+EOL_STR+cresult;
     if bPromptError then wError(cResult);
    end;

    IF (cBoard = 'A') THEN
    BEGIN
        IF (nTime <=(nTime1W+BPGNLag[nBPGNIndex])) THEN
        BEGIN
            IF (Chess2.bWhitesMove) THEN
                nTime2W := nTime2W - (nTime1W - nTime)-BPGNLag[nBpgnIndex]
            ELSE
                nTime2B := nTime2B - (nTime1W - nTime)-BPGNLag[nBpgnIndex];
        END
        else if ini1.INI.bTimeError then begin
         cResult:=cFile+EOL_STR + 'Game #' + IntToStr(nGameHeader+1) +', '+
         'move:'+Inttostr(nmove)+cboard+'.'+cmove+EOL_STR+
         'BPGN time=' + Inttostr(nTime)+'ms is bigger than maximum possible time ='+Inttostr(nTime1W+BPGNLag[nBPGNIndex])+'ms';
         if bPromptError then wError(cResult);
        end;
        nTime1W := nTime;
        inc(BpgnArray[nBpgnIndex,1].nWLag,BPGNLag[nBpgnIndex]);

        IF (NOT aFound[1]) THEN
        BEGIN
            aFound[1] := TRUE;
            aFirst[1] := nParent;
        END;
    END
    ELSE
    IF (cBoard = 'a') THEN
    BEGIN
        IF (nTime <=(nTime1B+BPGNLag[nBpgnIndex])) THEN
        BEGIN
            IF (Chess2.bWhitesMove) THEN
                nTime2W := nTime2W - (nTime1B - nTime)-BPGNLag[nBpgnIndex]
            ELSE
                nTime2B := nTime2B - (nTime1B - nTime)-BPGNLag[nBpgnIndex];
        END
        else if ini1.INI.bTimeError then begin
         cResult:=cFile+EOL_STR + 'Game #' + IntToStr(nGameHeader+1) +', '+
         'move:'+Inttostr(nmove)+cboard+'.'+cmove+EOL_STR+
         'BPGN time=' + Inttostr(nTime)+'ms is bigger than maximum possible time ='+Inttostr(nTime1B+BPGNLag[nBPGNIndex])+'ms';
         if bPromptError then wError(cResult);
        end;

        nTime1B := nTime;
        inc(BpgnArray[nBpgnIndex,1].nBLag,BPGNLag[nBpgnIndex]);
    END
    ELSE
    IF (cBoard = 'B') THEN
    BEGIN
        IF (nTime <= (nTime2W+BPGNLag[nBpgnIndex])) THEN
        BEGIN
            IF (Chess1.bWhitesMove) THEN
                nTime1W := nTime1W - (nTime2W - nTime)-BPGNLag[nBpgnIndex]
            ELSE
                nTime1B := nTime1B - (nTime2W - nTime)-BPGNLag[nBpgnIndex];
        END
        else if ini1.INI.bTimeError then begin
         cResult:=cFile+EOL_STR + 'Game #' + IntToStr(nGameHeader+1) +', '+
         'move:'+Inttostr(nmove)+cboard+'.'+cmove+EOL_STR+
         'BPGN time=' + Inttostr(nTime)+'ms is bigger than maximum possible time ='+Inttostr(nTime2W+BPGNLag[nBPGNIndex])+'ms';
         if bPromptError then wError(cResult);
        end;

        nTime2W := nTime;
        inc(BpgnArray[nBpgnIndex,2].nWLag,BPGNLag[nBpgnIndex]);

        IF (NOT aFound[2]) THEN
        BEGIN
            aFound[2] := TRUE;
            aFirst[2] := nParent;
        END;
    END
    ELSE
    IF (cBoard = 'b') THEN
    BEGIN
        IF (nTime <=(nTime2B+BPGNLag[nBpgnIndex])) THEN
        BEGIN
            IF (Chess1.bWhitesMove) THEN
                nTime1W := nTime1W - (nTime2B - nTime)-BPGNLag[nBpgnIndex]
            ELSE
                nTime1B := nTime1B - (nTime2B - nTime)-BPGNLag[nBpgnIndex];
        END
        else if ini1.INI.bTimeError then begin
         cResult:=cFile+EOL_STR + 'Game #' + IntToStr(nGameHeader+1) +', '+
         'move:'+Inttostr(nmove)+cboard+'.'+cmove+EOL_STR+
         'BPGN time=' + Inttostr(nTime)+'ms is bigger than maximum possible time ='+Inttostr(nTime2b+BPGNLag[nBPGNIndex])+'ms';
         if bPromptError then wError(cResult);
        end;

        nTime2B := nTime;
        inc(BpgnArray[nBpgnIndex,2].nBLag,BPGNLag[nBpgnIndex]);
    END;

    BPGNArray[nBPGNIndex,1].nWTime := nTime1W;
    BPGNArray[nBPGNIndex,1].nBTime := nTime1B;
    BPGNArray[nBPGNIndex,2].nWTime := nTime2W;
    BPGNArray[nBPGNIndex,2].nBTime := nTime2B;

    { Load stuff }
    while LoadBPGNStuff do Begin end;
//    LoadBPGNStuff;
//    LoadBPGNStuff;

    IF (UpCase(cBoard) = 'A') THEN
        cCap1 := BOARD1_TITLE + ' (' + IntToStr(nMove) + cBoard + '. ' + cMove + Trim(BPGNAnno[nBPGNIndex]) + ')'
    ELSE
        cCap2 := BOARD2_TITLE + ' (' + IntToStr(nMove) + cBoard + '. ' + cMove + Trim(BPGNAnno[nBPGNIndex]) + ')';

    BPGNArray[nBPGNIndex,1].cCaption := cCap1;
    BPGNArray[nBPGNIndex,2].cCaption := cCap2;

    IF (bGameDone) THEN
        EXIT;

    { Load result }
    LoadBPGNResult(nBPGNIndex);
END;

PROCEDURE TBPGN.LoadBPGNBody(nParent, nMoveA, nMoveB: Integer; cCap1, cCap2: String; bMain: Boolean);
VAR
    nOrgMoveA, nOrgMoveB, nIndex: Integer;
    bExitLoop: Boolean;
BEGIN
    WHILE (NOT bGameDone) DO
    BEGIN
        nOrgMoveA := nMoveA;
        nOrgMoveB := nMoveB;
        LoadBPGNMove(nParent, nMoveA, nMoveB, cCap1, cCap2, bMain);
        IF (bGameDone) THEN
            BREAK;

        nIndex    := nBPGNIndex;
        bExitLoop := FALSE;
        WHILE (NOT bGameDone) DO
        BEGIN
            Backup(2);
            IF (NOT GetToken) THEN
            BEGIN
                bGameDone := TRUE;
                bExitLoop := TRUE;
                BREAK;
            END;
            IF (nLastState <> BPGN_NORMAL) THEN
            BEGIN
                Restore(2);
                BREAK;
            END;
            IF (CompareStr(cToken, ')') = 0) THEN
            BEGIN
                bExitLoop := TRUE;
                BREAK;
            END;
            IF (CompareStr(cToken, '(') <> 0) THEN
            BEGIN
                Restore(2);
                BREAK;
            END;

            CopyBPGNRecordIntoChess(nParent);
            LoadBPGNBody(nParent, nOrgMoveA, nOrgMoveB, cCap1, cCap2, FALSE);
            CopyBPGNRecordIntoChess(nIndex);
        END;

        IF (bExitLoop) THEN
            BREAK;

        nParent := nIndex;
    END;
END;

PROCEDURE TBPGN.ReleaseBPGN(nIndex: Integer);
VAR
    nI: Integer;
BEGIN
    IF (nIndex < 1) THEN
        EXIT;

    BPGNInUse  [nIndex] := FALSE;
    BPGNParent [nIndex] := 0;
    BPGNMain   [nIndex] := 0;

    BPGNNumber [nIndex,1] := 0;
    BPGNNumber [nIndex,2] := 0;

    BPGNBoard  [nIndex] := ' ';
    BPGNMove   [nIndex] := '';
    BPGNAnno   [nIndex] := '';
    BPGNDisplay[nIndex] := FALSE;
    BPGNComment[nIndex] := '';

    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nIndex,nI] > 0) THEN
        BEGIN
            ReleaseBPGN(BPGNNext[nIndex,nI]);
            BPGNNext[nIndex,nI] := 0;
        END;
    END;
END;

PROCEDURE TBPGN.RemoveStuff(nWhich, nIndex: Integer);
VAR
    nI: Integer;
BEGIN
    IF (nIndex < 1) THEN
        EXIT;

    IF (nWhich = 1) THEN
    BEGIN
        IF (NOT bGameChange) THEN
            bGameChange := (Length(BPGNAnno[nIndex]) > 0);

        BPGNAnno[nIndex] := '';
    END
    ELSE
    IF (nWhich = 2) THEN
    BEGIN
        IF (NOT bGameChange) THEN
            bGameChange := (Length(BPGNComment[nIndex]) > 0);

        BPGNComment[nIndex] := '';
    END
    ELSE
    IF (nWhich = 3) THEN
    BEGIN
        IF (NOT bGameChange) THEN
            bGameChange := BPGNDisplay[nIndex];

        BPGNDisplay[nIndex] := FALSE;
    END;

    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nIndex,nI] > 0) THEN
            RemoveStuff(nWhich, BPGNNext[nIndex,nI]);
    END;
END;

PROCEDURE TBPGN.SaveMove(nIndex: Integer; bSpace: Boolean; VAR cLineStr: String);
VAR
    nL: Integer;
    cS: String;
BEGIN
    { Get move string }
    IF (UpCase(BPGNBoard[nIndex]) = 'A') THEN
        cS := IntToStr(BPGNNumber[nIndex,1]) + BPGNBoard[nIndex] + '. ' + BPGNMove[nIndex]
    ELSE
        cS := IntToStr(BPGNNumber[nIndex,2]) + BPGNBoard[nIndex] + '. ' + BPGNMove[nIndex];

    { Save move string }
    nL := Length(cLineStr);
    IF (nL = 0) THEN
        cLineStr := cS
    ELSE
    IF ((bSpace) AND ((nL + 1 + Length(cS)) <= MAX_BPGN_LEN)) THEN
        cLineStr := cLineStr + ' ' + cS
    ELSE
    IF ((NOT bSpace) AND ((nL + Length(cS)) <= MAX_BPGN_LEN)) THEN
        cLineStr := cLineStr + cS
    ELSE
    BEGIN
        cGameLine := cGameLine + cLineStr + EOL_STR;
        cLineStr  := cS;
    END;
    if BPGNismain[nIndex] then cLagString:=cLagString+' '+Inttostr(BPGNLag[nIndex]);
END;

PROCEDURE TBPGN.SaveTime(nIndex: Integer; VAR cLineStr: String);
VAR
    nL: Integer;
    cS: String;
BEGIN
    { Get time string }
    IF (BPGNBoard[nIndex] = 'A') THEN
        cS := '{' + BPGNTime(BPGNArray[nIndex, 1].nWTime) + '}'
    ELSE
    IF (BPGNBoard[nIndex] = 'a') THEN
        cS := '{' + bpgntime(BPGNArray[nIndex, 1].nBTime) + '}'
    ELSE
    IF (BPGNBoard[nIndex] = 'B') THEN
        cS := '{' + bpgntime(BPGNArray[nIndex, 2].nWTime) + '}'
    ELSE
    IF (BPGNBoard[nIndex] = 'b') THEN
        cS := '{' + bpgntime(BPGNArray[nIndex, 2].nBTime) + '}'
    ELSE
        cS := '';

    { Save time string }
    nL := Length(cLineStr);
    IF (nL = 0) THEN
        cLineStr := cS
    ELSE
    IF ((nL + 1 + Length(cS)) <= MAX_BPGN_LEN) THEN
        cLineStr := cLineStr + ' ' + cS
    ELSE
    BEGIN
        cGameLine := cGameLine + cLineStr + EOL_STR;
        cLineStr  := cS;
    END;
END;

PROCEDURE TBPGN.SaveAnnotation(nIndex: Integer; VAR cLineStr: String);
VAR
    nL: Integer;
    cS: String;
BEGIN
    { Get annotation string }
    IF (Length(BPGNAnno[nIndex]) > 0) THEN
    BEGIN
        cS := '{A:' + BPGNAnno[nIndex] + '}';

        { Save annotation string }
        nL := Length(cLineStr);
        IF (nL = 0) THEN
            cLineStr := cS
        ELSE
        IF ((nL + 1 + Length(cS)) <= MAX_BPGN_LEN) THEN
            cLineStr := cLineStr + ' ' + cS
        ELSE
        BEGIN
            cGameLine := cGameLine + cLineStr + EOL_STR;
            cLineStr  := cS;
        END;
    END;
END;

PROCEDURE TBPGN.SaveDisplay(nIndex: Integer; VAR cLineStr: String);
VAR
    nL: Integer;
    cS: String;
BEGIN
    { Get display string }
    IF (BPGNDisplay[nIndex]) THEN
    BEGIN
        cS := '{D:}';

        { Save display string }
        nL := Length(cLineStr);
        IF (nL = 0) THEN
            cLineStr := cS
        ELSE
        IF ((nL + 1 + Length(cS)) <= MAX_BPGN_LEN) THEN
            cLineStr := cLineStr + ' ' + cS
        ELSE
        BEGIN
            cGameLine := cGameLine + cLineStr + EOL_STR;
            cLineStr  := cS;
        END;
    END;
END;

PROCEDURE TBPGN.SaveComment(nIndex: Integer; VAR cLineStr: String);
VAR
    nL, nI: LongInt;
    nT: Integer;
    cC, cS: String;
BEGIN
    { Substitue comment string's carriage return with ASCII 254 }
    cC := '';
    nL := Length(BPGNComment[nIndex]);
    FOR nI := 1 TO nL DO
    BEGIN
        IF (BPGNComment[nIndex][nI] = CR_ASCII_CHAR) THEN
            cC := cC + COMMENT_CR_CHAR
        ELSE
        IF (BPGNComment[nIndex][nI] <> LF_ASCII_CHAR) THEN
            cC := cC + BPGNComment[nIndex][nI];
    END;

    { Get comment string }
    nL := Length(cC);
    IF (nL > 0) THEN
    BEGIN
        { Comment header }
        cS := '{C:';

        nT := Length(cLineStr);
        IF (nT = 0) THEN
            cLineStr := cS
        ELSE
        IF ((nT + 1 + Length(cS)) <= MAX_BPGN_LEN) THEN
            cLineStr := cLineStr + ' ' + cS
        ELSE
        BEGIN
            cGameLine := cGameLine + cLineStr + EOL_STR;
            cLineStr  := cS;
        END;

        { Comment body }
        nI := 0;
        WHILE (nI < nL) DO
        BEGIN
            Inc(nI);

            nT := Length(cLineStr);
            IF ((nT + 1) = MAX_BPGN_LEN) THEN
            BEGIN
                IF (cC[nI] = ' ') THEN
                BEGIN
                    cGameLine := cGameLine + cLineStr + EOL_STR;
                    cLineStr  := ' ';
                END
                ELSE
                    cLineStr := cLineStr + cC[nI]
            END
            ELSE
            IF (nT < MAX_BPGN_LEN) THEN
                cLineStr := cLineStr + cC[nI]
            ELSE
            BEGIN
                cGameLine := cGameLine + cLineStr + EOL_STR;
                cLineStr  := cC[nI];
            END;
        END;

        { Comment trailer }
        IF ((Length(cLineStr) + 1) <= MAX_BPGN_LEN) THEN
            cLineStr := cLineStr + '}'
        ELSE
        BEGIN
            cGameLine := cGameLine + cLineStr + EOL_STR;
            cLineStr  := '}';
        END;
    END;
END;

PROCEDURE TBPGN.SavePosition(nParent: Integer; VAR cLineStr: String);
VAR
    nI: Integer;
BEGIN
    IF (BPGNMain[nParent] < 1) THEN
        EXIT;

    { Generate main line }
    SaveMove      (BPGNMain[nParent], TRUE, cLineStr);
    SaveTime      (BPGNMain[nParent], cLineStr);
    SaveAnnotation(BPGNMain[nParent], cLineStr);
    SaveDisplay   (BPGNMain[nParent], cLineStr);
    SaveComment   (BPGNMain[nParent], cLineStr);


    { Generate variations }
    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF ((BPGNNext[nParent,nI] > 0) AND (BPGNMain[nParent] <> BPGNNext[nParent,nI])) THEN
        BEGIN
            IF (Length(cLineStr) = 0) THEN
                cLineStr := cLineStr + '('
            ELSE
            IF ((Length(cLineStr) + 2) < MAX_BPGN_LEN) THEN
                cLineStr := cLineStr + ' ('
            ELSE
            BEGIN
                cGameLine := cGameLine + cLineStr + EOL_STR;
                cLineStr  := '(';
            END;

            SaveMove      (BPGNNext[nParent,nI], FALSE, cLineStr);
            SaveTime      (BPGNNext[nParent,nI], cLineStr);
            SaveAnnotation(BPGNNext[nParent,nI], cLineStr);
            SaveDisplay   (BPGNNext[nParent,nI], cLineStr);
            SaveComment   (BPGNNext[nParent,nI], cLineStr);

            SavePosition(BPGNNext[nParent,nI], cLineStr);

            IF ((Length(cLineStr) + 1) <= MAX_BPGN_LEN) THEN
                cLineStr := cLineStr + ')'
            ELSE
            BEGIN
                cGameLine := cGameLine + cLineStr + EOL_STR;
                cLineStr  := ')';
            END;
        END;
    END;

    { Generate remaining of main line }
    SavePosition(BPGNMain[nParent], cLineStr);
END;

FUNCTION TBPGN.OpenBPGN(cFilename: String): Boolean;
VAR
    nI:  LongInt;
BEGIN
    cFilename := Trim(cFilename);
    IF (Length(cFilename) = 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := 'Missing BPGN filename.';
        IF (bPromptError) THEN wError(cResult);
        Result     := FALSE;
        EXIT;
    END;

    IF (NOT FileExists(cFilename)) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFilename + EOL_STR + 'BPGN file not found.';
        IF (bPromptError) THEN wError(cResult);
        Result     := FALSE;
        EXIT;
    END;

    cFile := cFilename;
    cName := ExtractFilename(cFilename);

    AssignFile(ByteFile, cFile);
    Reset(ByteFile);
    nFileSize := FileSize(ByteFile);
    nFilePos  := 0;
    nLastPos  := 0;
    CloseFile(ByteFile);

    IF (bOpen) THEN CloseFile(BPGNFile);

    AssignFile(BPGNFile, cFile);
    Reset(BPGNFile);

    cResult := '';
    bOpen   := TRUE;
    bFirst  := TRUE;
    cLine   := '';
    Result  := TRUE;
END;

FUNCTION TBPGN.CloseBPGN(bSilent: Boolean): Boolean;
VAR
    nI: Integer;
BEGIN
    IF (NOT bOpen) THEN
    BEGIN
        Result := FALSE;
        IF (bSilent) THEN
            cResult := ''
        ELSE
        BEGIN
            bGameDone  := TRUE;
            bGameError := TRUE;
            cResult := cFile + EOL_STR + 'BPGN file is not opened.';
            IF (bPromptError) THEN wError(cResult);
        END;
        EXIT;
    END;
    if bisdb and bdbchange then SendMessage(fMainHandle,WM_UpdateDB,0,0);
    CloseFile(BPGNFile);

    { Common }
    cFile   := '';
    cName   := '';
    cResult := '';
    bOpen   := FALSE;

    { BPGN file }
    bFirst       := TRUE;
    nFileSize    := 0;
    nFilePos     := 0;
    nLastPos     := 0;
    nStartString := 0;
    nEndString   := -1;
    cLine        := '';
    bISDB        :=false;
    bdbchange  :=false;

    { BPGN Game }
    cGameLine  := '';
    nGameSize  := 0;
    nGameIndex := 0;
    cToken     := '';
    cTmpSt     := '';
    nState     := BPGN_NORMAL;
    nLastState := BPGN_NORMAL;

    { BPGN Header }
    nHeaderCount := 0;
    nGameHeader  := 0;
    nFilterCount :=0;
    bFilterSet:=false;

    { BPGN Tree }
    nBPGNIndex   := 0;
    cBPGNResult  := '';
    cStartCap1   := '';
    cStartCap2   := '';
    bGameDone    := FALSE;
    bGameError   := FALSE;
    nCurrent     := 0;
    bGameChange  := FALSE;
    bFileChange  := FALSE;
    aFound[1]    := FALSE;
    aFound[2]    := FALSE;
    aFirst[1]    := 0;
    aFirst[2]    := 0;
    aPrevious[1] := 0;
    aPrevious[2] := 0;
    aNext[1]     := 0;
    aNext[2]     := 0;
    nLast        := 0;

    { Next move }
    nNextCount := 0;

    { Previous next move }
    nPNextCount := 0;

    { Navigator }
    FOR nI := 1 TO 4 DO
    BEGIN
        aNavAll   [nI] := FALSE;
        aNavBoard1[nI] := FALSE;
        aNavBoard2[nI] := FALSE;
    END;

    { Game list stuff }


    alist.clear;
    pdblist[1].Clear;
    pdblist[2].clear;
    { BPGN stuff }
    FOR nI := 1 TO MAX_POSITION DO
    BEGIN
        BPGNArray[nI, 1].cCaption := '';
        BPGNArray[nI, 2].cCaption := '';

        BPGNInUse [nI] := FALSE;
        BPGNIsMain[nI] := FALSE;
        BPGNParent[nI] := 0;
        BPGNMain  [nI] := 0;

        BPGNNumber [nI,1] := 0;
        BPGNNumber [nI,2] := 0;
        BPGNBoard  [nI] := ' ';
        BPGNMove   [nI] := '';
        BPGNAnno   [nI] := '';
        BPGNDisplay[nI] := FALSE;
        BPGNComment[nI] := '';
    END;

    Result := TRUE;
END;

FUNCTION TBPGN.LoadHeader: Boolean;
VAR
    bResult: Boolean;
    nPos: LongInt;
    bEvent: Boolean;
    cEvent, cSite, cDate: String;
    cWhiteA, cBlackA, cWhiteB, cBlackB: String;
    cTimeC, cBResult, cOpenA, cOpenB: String;
    cRWA, cRBA, cRWB, cRBB,cSetup,cFen: String;
    nLine, nS: Integer;
    aCnt: ARRAY[1..6] OF Integer;
    creason:String;
    apointer:pbpgnrec;
    nc1,barstep:longint;
    BPGN_EVENT_NUM,
    BPGN_SITE_NUM,
    BPGN_DATE_NUM,
    BPGN_WHITEA_NUM,
    BPGN_BLACKA_NUM,
    BPGN_WHITEB_NUM,
    BPGN_BLACKB_NUM,
    BPGN_TIMEC_NUM,
    BPGN_RESULT_NUM,
    BPGN_OPENA_NUM,
    BPGN_OPENB_NUM,
    BPGN_SETUP_NUM,
    BPGN_FEN_NUM:integer;
BEGIN
    BPGN_EVENT_NUM:=length(BPGN_EVENT_STR);
    BPGN_site_NUM:=length(BPGN_site_STR);
    BPGN_date_NUM:=length(BPGN_date_STR);
    BPGN_whitea_NUM:=length(BPGN_whitea_STR);
    BPGN_blacka_NUM:=length(BPGN_blacka_STR);
    BPGN_whiteb_NUM:=length(BPGN_whiteb_STR);
    BPGN_blackb_NUM:=length(BPGN_blackb_STR);
    BPGN_timec_NUM:=length(BPGN_timec_STR);
    BPGN_result_NUM:=length(BPGN_result_STR);
    BPGN_opena_NUM:=length(BPGN_opena_STR);
    BPGN_setup_NUM:=length(BPGN_setup_STR);
    BPGN_fen_NUM:=length(BPGN_fen_STR);

    barstep:=1000; nc1:=0;
    bResult      := TRUE;
    nGameHeader  := 0;
    bEvent       := FALSE;
    cEvent       := DEFAULT_EVENT;
    cSite        := DEFAULT_SITE;
    cDate        := DEFAULT_DATE;
    nPos         := -1;
    cWhiteA      := '';
    cBlackA      := '';
    cWhiteB      := '';
    cBlackB      := '';
    cTimeC       := '';
    cBResult     := '';
    cOpenA       := '';
    cOpenB       := '';
    cRWA         := '';
    cRBA         := '';
    cRWB         := '';
    cRBB         := '';
    cSetup       := '';
    cFen         := '';
    aCnt[1]      := 0;
    aCnt[2]      := 0;
    aCnt[3]      := 0;
    aCnt[4]      := 0;
    aCnt[5]      := 0;
    aCnt[6]      := 0;
    cReason      :=DEFAULT_REASON;


    nLine := 0;
    WHILE (GetLine) DO
    BEGIN
        Inc(nLine);
        IF (nLine >= MAX_LINE) THEN
            nLine := 0;

        IF ((aCnt[1] <> 0) AND (aCnt[2] <> 0) AND (aCnt[3] <> 0) AND
            (aCnt[4] <> 0) AND (aCnt[5] <> 0) AND (aCnt[6] <> 0) AND
            (nPos >= 0)    AND (CompareStr(Copy(cLine,1,1), '[') <> 0)) THEN
        BEGIN

            if nc1=barstep then begin
            PostMessage(fMainHandle,WM_ProgressUpdate,nFilePos div 1000,nFileSize div 1000);nc1:=0;
            end
            else inc(nc1,1);
            if bAbortLong then exit;
//            Inc(nHeaderCount);
            alist.add(apointer);
            apointer:=alist.Last;
            apointer^.atag := TAG_OK;
            apointer^.aEvent := cEvent;
            apointer^.aSite    := cSite;
            apointer^.aDate    := cDate;
            apointer^.aPosition:= nPos;
            apointer^.aWhiteA  := cWhiteA;
            apointer^.aBlackA  := cBlackA;
            apointer^.aWhiteB  := cWhiteB;
            apointer^.aBlackb  := cBlackB;
            apointer^.aTimeC   := cTimeC;
            apointer^.aResult  := cBResult;
            apointer^.aOpenA   := cOpenA;
            apointer^.aOpenB   := cOpenB;
            apointer^.aRWA     := cRWA;
            apointer^.aRBA     := cRBA;
            apointer^.aRWB     := cRWB;
            apointer^.aRBB     := cRBB;
            apointer^.aSetup   := cSetup;
            apointer^.aFen     := cFen;
            apointer^.aFilterN :=alist.count-1;

            bEvent  := FALSE;
            cEvent  := DEFAULT_EVENT;
            cSite   := DEFAULT_SITE;
            cDate   := DEFAULT_DATE;
            nPos    := -1;
            cWhiteA := '';
            cBlackA := '';
            cWhiteB := '';
            cBlackB := '';
            cTimeC  := '';
            cBResult:= '';
            cOpenA  := '';
            cOpenB  := '';
            cRWA    := '';
            cRBA    := '';
            cRWB    := '';
            cRBB    := '';
            cSetup  := '';
            cFen    := '';
            aCnt[1] := 0;
            aCnt[2] := 0;
            aCnt[3] := 0;
            aCnt[4] := 0;
            aCnt[5] := 0;
            aCnt[6] := 0;
            cReason:=Default_Reason;
        END;

        nS := Length(cLine);
        IF (nS = 0) THEN
            CONTINUE;

        { Event }
        IF (nS > BPGN_EVENT_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1,BPGN_EVENT_NUM), BPGN_EVENT_STR) = 0) THEN
            BEGIN
                bEvent  := TRUE;
                if alist.Count>0 then apointer^.areason:=ValidReason(creason);
                IF (NOT GetQuoteString(cEvent)) THEN
                    cEvent := DEFAULT_EVENT;

                cSite   := DEFAULT_SITE;
                cDate   := DEFAULT_DATE;
                nPos    := nFilePos;
                cWhiteA := '';
                cBlackA := '';
                cWhiteB := '';
                cBlackB := '';
                cTimeC  := '';
                cBResult:= '';
                cOpenA  := '';
                cOpenB  := '';
                cRWA    := '';
                cRBA    := '';
                cRWB    := '';
                cRBB    := '';
                cSetup  :='';
                cFen    :='';
                aCnt[1] := 0;
                aCnt[2] := 0;
                aCnt[3] := 0;
                aCnt[4] := 0;
                aCnt[5] := 0;
                aCnt[6] := 0;
                CONTINUE;
            END;
        END;

        IF (NOT bEvent) THEN
        begin
            cReason:=cline;
            CONTINUE;
        end;

        { Site }
        IF (nS > BPGN_SITE_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_SITE_NUM), BPGN_SITE_STR) = 0) THEN
            BEGIN
                IF (NOT GetQuoteString(cSite)) THEN
                    cSite := DEFAULT_SITE;
                CONTINUE;
            END;
        END;

        { Date }
        IF (nS > BPGN_DATE_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_DATE_NUM), BPGN_DATE_STR) = 0) THEN
            BEGIN
                IF (NOT GetQuoteString(cDate)) THEN
                    cDate := DEFAULT_DATE;
                CONTINUE;
            END;
        END;

        { WhiteA }
        IF (nS > BPGN_WHITEA_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_WHITEA_NUM), BPGN_WHITEA_STR) = 0) THEN
            BEGIN
                IF (GetQuoteString(cWhiteA)) THEN aCnt[1] := aCnt[1] + 1;
                cRWA := GetRating(1);
                CONTINUE;
            END;
        END;

        { BlackA }
        IF (nS > BPGN_BLACKA_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_BLACKA_NUM), BPGN_BLACKA_STR) = 0) THEN
            BEGIN
                IF (GetQuoteString(cBlackA)) THEN aCnt[2] := aCnt[2] + 1;
                cRBA := GetRating(2);
                CONTINUE;
            END;
        END;

        { WhiteB }
        IF (nS > BPGN_WHITEB_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_WHITEB_NUM), BPGN_WHITEB_STR) = 0) THEN
            BEGIN
                IF (GetQuoteString(cWhiteB)) THEN aCnt[3] := aCnt[3] + 1;
                cRWB := GetRating(3);
                CONTINUE;
            END;
        END;

        { BlackB }
        IF (nS > BPGN_BLACKB_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_BLACKB_NUM), BPGN_BLACKB_STR) = 0) THEN
            BEGIN
                IF (GetQuoteString(cBlackB)) THEN aCnt[4] := aCnt[4] + 1;
                cRBB := GetRating(4);
                CONTINUE;
            END;
        END;

        { TimeControl }
        IF (nS > BPGN_TIMEC_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_TIMEC_NUM), BPGN_TIMEC_STR) = 0) THEN
            BEGIN
                IF (GetQuoteString(cTimeC)) THEN aCnt[5] := aCnt[5] + 1;
                CONTINUE;
            END;
        END;

        { Result }
        IF (nS > BPGN_RESULT_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_RESULT_NUM), BPGN_RESULT_STR) = 0) THEN
            BEGIN
                IF (GetQuoteString(cBResult)) THEN aCnt[6] := aCnt[6] + 1;
                CONTINUE;
            END;
        END;

        { BECOA }
        IF (nS > BPGN_OPENA_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_OPENA_NUM), BPGN_OPENA_STR) = 0) THEN
            BEGIN
                GetQuoteString(cOpenA);
                CONTINUE;
            END;
        END;

        { BECOB }
        IF (nS > BPGN_OPENB_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_OPENB_NUM), BPGN_OPENB_STR) = 0) THEN
            BEGIN
                GetQuoteString(cOpenB);
                CONTINUE;
            END;
        END;

        {Setup}
        IF (nS > BPGN_SETUP_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_SETUP_NUM), BPGN_SETUP_STR) = 0) THEN
            BEGIN
                GetQuoteString(csetup);
                CONTINUE;
            END;
        END;

        {FEN}
        IF (nS > BPGN_FEN_NUM) THEN
        BEGIN
            IF (CompareText(Copy(cLine, 1, BPGN_FEN_NUM), BPGN_FEN_STR) = 0) THEN
            BEGIN
                GetQuoteString(cFEn);
                CONTINUE;
            END;
        END;

    END;
    IF alist.Count>0 then apointer^.areason:=validreason(creason);
    nHeaderCount:=alist.count;
//    nFilterCount:=nHeaderCount;
    Result := bResult;
END;

FUNCTION TBPGN.LoadGame(nIndex: Integer): Boolean;
VAR
    nBeginPos, nEndPos: LongInt;
    nI: Integer;
    bFound, bNumber, bDisplay, bComment, bDidIt: Boolean;
BEGIN
    nGameHeader := nIndex;
    alist.backup(nIndex);
    nBeginPos := getPosition(nIndex);
    IF (nIndex = nHeaderCount-1) THEN
        nEndPos := nFileSize
    ELSE
        nEndPos := getposition(nIndex+1) - 1;
    GetGame(nBeginPos, nEndPos);

    cLagString:=trim(FindHeader(BPGN_LAG_STR,cGameLine));
    bLagTag:=not (cLagString='');
    nBPGNIndex  := 0;
    cBPGNResult := DEFAULT_REASON + ' ' + DEFAULT_RESULT;

    IF (Length(getopena(nIndex)) = 0) THEN
        cStartCap1 := BOARD1_TITLE
    ELSE
        cStartCap1 := BOARD1_TITLE + ' (BECO "' + getOpenA(nIndex) + '")';

    IF (Length(getopenB(nIndex)) = 0) THEN
        cStartCap2 := BOARD2_TITLE
    ELSE
        cStartCap2 := BOARD2_TITLE + ' (BECO "' + getopenB(nIndex) + '")';

    bGameDone    := FALSE;
    bGameError   := FALSE;
    nCurrent     := 0;
    bGameChange  := FALSE;
    aFound[1]    := FALSE;
    aFound[2]    := FALSE;
    aFirst[1]    := 0;
    aFirst[2]    := 0;
    aPrevious[1] := 0;
    aPrevious[2] := 0;
    aNext[1]     := 0;
    aNext[2]     := 0;
    nLast        := 0;

    FOR nI := 1 TO 4 DO
    BEGIN
        aNavAll   [nI] := FALSE;
        aNavBoard1[nI] := FALSE;
        aNavBoard2[nI] := FALSE;
    END;

    FOR nI := 1 TO MAX_POSITION DO
        BPGNInUse[nI] := FALSE;

    Inc(nBPGNIndex);

    Chess1.ResetBoard;
    Chess2.ResetBoard;

    BPGNArray[nBPGNIndex,1].nWTime := 1000*TOOLBOX_GetTimeVal(gettimec(nIndex));
    BPGNArray[nBPGNIndex,1].nBTime := BPGNArray[nBPGNIndex,1].nWTime;

    BPGNArray[nBPGNIndex,2].nWTime := BPGNArray[nBPGNIndex,1].nWTime;
    BPGNArray[nBPGNIndex,2].nBTime := BPGNArray[nBPGNIndex,1].nWTime;
    BPGNArray[1,1].nWLag:=0;
    BPGNarray[1,1].nBLag:=0;
    BpgnArray[1,2].nWlag:=0;
    bpgnarray[1,2].nBLag:=0;

    bscrewfirstmove:=false;
    if (getsetup(nIndex)='1') then
    begin
     parsefen(getFen(nIndex));
     bScrewFirstMove:=true;
    end;
    CopyChessIntoBPGNRecord(nBPGNIndex);

    BPGNIsMain[nBPGNIndex] := TRUE;


    { Find the first move number 1 }
    bFound   := FALSE;
    bNumber  := FALSE;
    bDisplay := FALSE;
    bComment := FALSE;
    WHILE (TRUE) DO
    BEGIN
        Backup(1);
        IF (NOT GetToken) THEN
            BREAK;

        IF (nLastState = BPGN_INTEGER) THEN
        BEGIN
            bNumber := TRUE;
            IF ((CompareStr(cToken,'1') = 0) or (bscrewfirstmove)) THEN
            BEGIN
                Restore(1);
                bFound := TRUE;
                BREAK;
            END;
        END
        ELSE
        IF (nLastState = BPGN_COMMENT2) THEN
        BEGIN
            bDidIt := FALSE;
            Restore(1);
            IF (NOT bDisplay) THEN
            BEGIN
                IF (LoadBPGNStuff) THEN
                BEGIN
                    bDidiT:=TRUE;
                    IF (BPGNDisplay[nBPGNIndex]) THEN
                    BEGIN
                        bDisplay := TRUE;
//                        bDidIt   := TRUE;
                    END
                    ELSE
                    IF (Length(BPGNComment[nBPGNIndex]) > 0) THEN
                    BEGIN
                        bComment := TRUE;
//                        bDidIt   := TRUE;
                    END;
                END;
            END
            ELSE
            IF (NOT bComment) THEN
            BEGIN
                IF (LoadBPGNStuff) THEN
                BEGIN
                    bDidIt:=true;
                    IF (BPGNDisplay[nBPGNIndex]) THEN
                    BEGIN
                        bDisplay := TRUE;
//                        bDidIt   := TRUE;
                    END
                    ELSE
                    IF (Length(BPGNComment[nBPGNIndex]) > 0) THEN
                    BEGIN
                        bComment := TRUE;
//                        bDidIt   := TRUE;
                    END;
                END;
            END;
            IF (NOT bDidIt) THEN
                GetToken;
        END;
    END;
    IF ((NOT bFound) AND bNumber) THEN
    BEGIN
        nBPGNIndex := 0;
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Move number [1] not found.';
        IF (bPromptError) THEN wError(cResult);
        Result := FALSE;
        EXIT;
    END;


    { Process rest }
    LoadBPGNBody(nBPGNIndex, 0, 0, cStartCap1, cStartCap2, TRUE);

    nCurrent := 1;
    if ((bfilterset) and (gamelistpos.cBoard>0)) then
     goposition(gamelistpos);
//    nTotalLag:=0;
    Result   := (NOT bGameError);
END;

PROCEDURE TBPGN.DeleteAllAnnotations;
BEGIN
    RemoveStuff(1, 1);
END;

PROCEDURE TBPGN.DeleteAllComments;
BEGIN
    RemoveStuff(2, 1);
END;

PROCEDURE TBPGN.DeleteAllDisplays;
BEGIN
    RemoveStuff(3, 1);
END;

PROCEDURE TBPGN.DeleteCurrentMove;
VAR
    nP, nI: Integer;
BEGIN
    bGameChange := TRUE;
    nP          := BPGNParent[nCurrent];

    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nP, nI] = nCurrent) THEN
        BEGIN
            BPGNNext[nP, nI] := 0;
            BREAK;
        END;
    END;

    IF (BPGNMain[nP] = nCurrent) THEN
    BEGIN
        BPGNMain[nP] := 0;
        FOR nI := 1 TO MAX_NEXT DO
        BEGIN
            IF (BPGNNext[nP, nI] <> 0) THEN
            BEGIN
                BPGNMain[nP] := BPGNNext[nP, nI];
                BREAK;
            END;
        END;
    END;

    aFound[1] := FALSE;
    aFound[2] := FALSE;
    aFirst[1] := 0;
    aFirst[2] := 0;
    nP        := 1;
    nI        := BPGNMain[nP];
    WHILE (nI > 0) DO
    BEGIN
        IF (BPGNBoard[nI] = 'A') THEN
        BEGIN
            IF (NOT aFound[1]) THEN
            BEGIN
                aFound[1] := TRUE;
                aFirst[1] := nP;
                IF (aFound[2]) THEN
                    BREAK;
            END;
        END
        ELSE
        IF (BPGNBoard[nI] = 'B') THEN
        BEGIN
            IF (NOT aFound[2]) THEN
            BEGIN
                aFound[2] := TRUE;
                aFirst[2] := nP;
                IF (aFound[1]) THEN
                    BREAK;
            END;
        END;
        nP := nI;
        nI := BPGNMain[nP];
    END;

    nP := BPGNParent[nCurrent];
    ReleaseBPGN(nCurrent);

    nCurrent := nP;
END;

procedure TBPGN.AppendGame(nIndex:integer;filename:string);
var s,hname:string;
f:TextFile;
bc,bdbsave:boolean;
nc:integer;
nPrevCount,nPrevSize:longint;
begin
    bdbsave:=((ExtractFileExt(filename)=DB_HEADER_EXT) and (bisdb));
    if bdbsave then begin
     nc:=ReadDBSyncInfo(filename,nPrevSize,nPrevCount);
     if nc=0 then begin
      wError('Can not add games to'+EOL_STR+'The database file is damaged');
      exit;
     end;
     if (((bisbugdb) and (nc=-1)) or ((not bisbugdb) and (nc=1))) then begin
      Werror('Types of source and destination databases do not match'+EOL_STR);
      exit;
     end;
     if nc=2 then begin nPrevSize:=0;nPrevCount:=0; end;
     hname:=filename;filename:=changeFileExt(hname,'.'+bpgn_EXT);
    end;

    getactualgametext(nIndex);
    cgameline:=cactualgame;
    if (Length(cGameLine)<1) then exit;
  try
    AssignFile(f, filename);
    if not fileexists(filename) then begin rewrite(f);close(f);end;
    Append(f);
    Write(f, cgameline);
    { insert code here that would require a Flush before closing the file }
    Flush(f);  { ensures that the text was actually written to file }
  except
   str(getlasterror,s);
   Werror('Error '+s+' occured during attempt to write to file '+ filename);
   CloseFile(f);
   exit;
  end;
  closefile(f);
  if bdbsave then appendGameDB(hname,nIndex,nPrevSize,nPrevCount,length(cActualGame));
end;


PROCEDURE TBPGN.SaveGame(nIndex: LongInt);
VAR
    cFileStr, cLineStr: String;
    nBeginPos, nEndPos, nOrgFileLen, nNewFileLen, nI, nJ: LongInt;
    OutText: TextFile;
    nF: Integer;
    ef:boolean;
    nstrl:longint;
    stepinc:integer;
BEGIN
    { Game file position }
    nBeginPos := getPosition(nIndex);
    IF (nIndex = nHeaderCount -1 ) THEN
        nEndPos := nFileSize
    ELSE begin
        nEndPos := getPosition(nIndex+1) - 1;
    end;

    getactualgametext(nIndex);
    cgameline:=cactualgame;
    IF ((nHeaderCount > 1) AND (nIndex <> nHeaderCount-1)) THEN
        cGameLine := cGameLine + EOL_STR;



    { Adjust game line }
    nGameSize := Length(cGameLine);

    IF (nGameSize > 0) THEN
        nGameIndex := 1
    ELSE
        nGameIndex := 0;


    { Close BPGN file }
    CloseFile(BPGNFile);


    { Read entire file }
    cFileStr := '';
    AssignFile(BPGNFile, cFile);
    Reset(BPGNFile);
    repeat
        ef:=EOF(bpgnfile);
        FOR nI := 0 TO (MAX_STRING - 1) DO
            cFileString[nI] := #0;

        TRY
            Read(BPGNFile, cFileString);
        EXCEPT
            ON E: EInOutError DO
                BEGIN
                END;
        END;
        nstrl:=strlen(Cfilestring)-1;
        if nstrl>MAX_STRING-1 then nstrl:=Max_STRING-1;
        FOR nI := 0 TO nstrl DO
            cFileStr := cFileStr + cFileString[nI];
    until ef;
//    END;
    CloseFile(BPGNFile);

    nOrgFileLen := Length(cFileStr);

PostMessage(fMainHandle,WM_PROGRESSUpdate,1,3);
        { Write into file }
    IF (nHeaderCount = 1) THEN
    BEGIN
        AssignFile(OutText, cFile);
        Rewrite(OutText);
        Write(OutText, cGameLine);
        CloseFile(OutText);
        nNewFileLen := Length(cGameLine);
    END
    ELSE
    BEGIN
        { Alter file string }
        IF (nBeginPos = 0) THEN
            cFileStr := cGameLine + Copy(cFileStr, nEndPos + 2, nOrgFileLen - nEndPos - 1)
        ELSE
        IF (nIndex = nHeaderCount -1) THEN
            cFileStr := Copy(cFileStr, 1, nBeginPos) + cGameLine
        ELSE
            cFileStr := Copy(cFileStr, 1, nBeginPos) + cGameLine + Copy(cFileStr, nEndPos + 2, nOrgFileLen - nEndPos - 1);

        nNewFileLen := Length(cFileStr);

        { Write into file }
        nF := 0;
        nJ := 0;

        WHILE (nJ < nNewFileLen) DO
        BEGIN
            IF ((nJ + 1) = MAX_STRING) THEN
            BEGIN
                IF (nF = 0) THEN
                BEGIN
                    AssignFile(BPGNFile, cFile);
                    ReWrite(BPGNFile);
                END;
                FOR nI := 0 TO (MAX_STRING - 1) DO
                BEGIN
                    Inc(nJ);
                    cFileString[nI] := cFileStr[nJ];
                END;
                Write(BPGNFile, cFileString);
                CloseFile(BPGNFile);
                BREAK;
            END;

            IF ((nJ + 1) > MAX_STRING) THEN
            BEGIN
                IF (nF = 0) THEN
                BEGIN
                    nF := 1;
                    AssignFile(BPGNFile, cFile);
                    ReWrite(BPGNFile);
                END;
                FOR nI := 0 TO (MAX_STRING - 1) DO
                BEGIN
                    Inc(nJ);
                    cFileString[nI] := cFileStr[nJ];
                END;
                Write(BPGNFile, cFileString);
                CONTINUE;
            END;

            IF (nF = 0) THEN
            BEGIN
                AssignFile(OutText, cFile);
                Rewrite(OutText);
            END
            ELSE
            BEGIN
                CloseFile(BPGNFile);
                AssignFile(OutText, cFile);
                Append(OutText);
            END;
            Write(OutText, Copy(cFileStr, nJ + 1, nNewFileLen - nJ));
            CloseFile(OutText);
            BREAK;
        END;
    END;
    PostMessage(fMainHandle,WM_PROGressUpdate,2,3);


    { Adjust gamelist's starting position value }
    nJ := nNewFileLen - nOrgFileLen;

    FOR nI := (nIndex + 1) TO nHeaderCount-1 DO
    begin
        setposition(nI,getposition(ni)+nj);
    end;
    nEndPos := nEndPos + nJ;


    { Assign new file size }
    nFileSize := nNewFileLen;


    { Open BPGN file again }
    AssignFile(BPGNFile, cFile);
    Reset(BPGNFile);

    { Set changed flag }
    bGameChange := FALSE;

     PostMessage(fmainHandle,WM_progressUpdate,3,3);
    {Save Headers and positions if db file}
  if bisdb then
  begin
   updateGameToPosDB(nIndex);
   bdbchange:=true;
  end;
END;


FUNCTION TBPGN.GetTag(nIndex: Integer): Char;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nindex];
    Result := p^.aTag;
END;

FUNCTION TBPGN.GetEvent(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nindex];
    Result := p^.aEvent;
END;

FUNCTION TBPGN.GetSite(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aSite;
END;

FUNCTION TBPGN.GetDate(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aDate;
END;

FUNCTION TBPGN.GetPosition(nIndex: Integer): LongInt;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aPosition;
END;

FUNCTION TBPGN.GetWhiteA(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aWhiteA;
END;

FUNCTION TBPGN.GetBlackA(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aBlackA;
END;

FUNCTION TBPGN.GetWhiteB(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aWhiteB;
END;

FUNCTION TBPGN.GetBlackB(nIndex: Integer): String;
var p:pbpgnrec;
begin
    p:=alist.Items[nIndex];
    Result := p^.aBlackB;
END;

FUNCTION TBPGN.GetRWA(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aRWA;
END;

FUNCTION TBPGN.GetRBA(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aRBA;
END;

FUNCTION TBPGN.GetRWB(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aRWB;
END;

FUNCTION TBPGN.GetRBB(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aRBB;
END;

FUNCTION TBPGN.GetTimeC(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aTimeC;
END;

FUNCTION TBPGN.GetReason: String;
BEGIN
    Result := cBPGNResult;
END;

function TBPGN.getaReason(nIndex: Integer):String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aReason;
END;

FUNCTION TBPGN.GetResult(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aResult;
END;

FUNCTION TBPGN.GetOpenA(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aOpena;
END;

FUNCTION TBPGN.GetOpenB(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aOpenB;
END;

FUNCTION TBPGN.GetSetup(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aSetup;
END;

FUNCTION TBPGN.GetFen(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    Result := p^.aFen;
END;


FUNCTION TBPGN.GetWhiteAHandle(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    IF (Length(p^.aRWA) = 0) THEN
        Result := p^.aWhiteA
    ELSE
        Result := p^.aWhiteA + '(' + p^.aRWA + ')';
END;

FUNCTION TBPGN.GetBlackAHandle(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    IF (Length(p^.aRBA) = 0) THEN
        Result := p^.aBlackA
    ELSE
        Result := p^.aBlackA + '(' + p^.aRBA + ')';
END;

FUNCTION TBPGN.GetWhiteBHandle(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    IF (Length(p^.aRWB) = 0) THEN
        Result := p^.aWhiteB
    ELSE
        Result := p^.aWhiteB + '(' + p^.aRWB + ')';
END;

FUNCTION TBPGN.GetBlackBHandle(nIndex: Integer): String;
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    IF (Length(p^.aRBB) = 0) THEN
        Result := p^.aBlackB
    ELSE
        Result := p^.aBlackB + '(' + p^.aRBB + ')';
END;

FUNCTION TBPGN.GetHeaderCount: Integer;
BEGIN
    Result := nHeaderCount;
END;

FUNCTION TBPGN.GetHeaderItem(nIndex: Integer): String;
VAR
    cItem: String;
    p:pbpgnrec;
BEGIN
    p:= alist.Items[nindex];
    cItem := p^.aTag + ' ';

    IF (INI1.INI.bShowGameNum) THEN
        cItem := cItem + IntToStr(nIndex+1) + '. ';

    IF (INI1.INI.bShowEvent) THEN
        cItem := cItem + p^.aEvent + ' ';

    IF ((Length(p^.aRWA) = 0) OR (NOT INI1.INI.bShowRatings)) THEN
        cItem := cItem + p^.aWhiteA + '/'
    ELSE
        cItem := cItem + p^.aWhiteA + '(' + p^.aRWA + ')/';

    IF ((Length(p^.aRBB) = 0) OR (NOT INI1.INI.bShowRatings)) THEN
        cItem := cItem + p^.aBlackB + ' vs '
    ELSE
        cItem := cItem + p^.aBlackB + '(' + p^.aRBB + ') vs ';

    IF ((Length(P^.aRBA) = 0) OR (NOT INI1.INI.bShowRatings)) THEN
        cItem := cItem + P^.aBlackA + '/'
    ELSE
        cItem := cItem + P^.aBlackA + '(' + P^.aRBA + ')/';

    IF ((Length(P^.aRWB) = 0) OR (NOT INI1.INI.bShowRatings)) THEN
        cItem := cItem + P^.aWhiteB
    ELSE
        cItem := cItem + P^.aWhiteB + '(' + P^.aRWB + ')';

    IF (INI1.INI.bShowTime) THEN
        cItem := cItem + ' ' + IntToStr((TOOLBOX_GetTimeVal(P^.aTimeC) DIV 60));

    IF (INI1.INI.bShowInc) THEN
        cItem := cItem + ' ' + IntToStr(TOOLBOX_GetTimeIncVal(P^.aTimeC));

    IF (INI1.INI.bShowResult) THEN
        cItem := cItem + ' ' + P^.aResult;

    Result := cItem;
END;

PROCEDURE TBPGN.SetTag(nIndex: Integer; cTag: Char);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nindex];
    p^.aTag := cTag;
    if cTag=Tag_delete then bFileChange  := TRUE;
END;

PROCEDURE TBPGN.SetEvent(nIndex: Integer; cEvent: String);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nindex];
    p^.aEvent := cEvent;
    bGameChange    := TRUE;
END;

PROCEDURE TBPGN.SetSite(nIndex: Integer; cSite: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nindex];
    p^.aSite := cSite;
    bGameChange   := TRUE;
END;

PROCEDURE TBPGN.SetDate(nIndex: Integer; cDate: String);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    p^.aDate:= cDate;
    bGameChange   := TRUE;
END;

PROCEDURE TBPGN.SetPosition(nIndex: Integer; nPos: longint);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    p^.aPosition:= nPos;
END;

PROCEDURE TBPGN.SetWhiteA(nIndex: Integer; cWhiteA: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aWhiteA := cWhiteA;
    bGameChange     := TRUE;
END;

PROCEDURE TBPGN.SetBlackA(nIndex: Integer; cBlackA: String);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    p^.aBlackA := cBlackA;
    bGameChange     := TRUE;
END;

PROCEDURE TBPGN.SetWhiteB(nIndex: Integer; cWhiteB: String);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    p^.aWhiteB := cWhiteB;
    bGameChange     := TRUE;
END;

PROCEDURE TBPGN.SetBlackB(nIndex: Integer; cBlackB: String);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    p^.aBlackB := cBlackB;
    bGameChange     := TRUE;
END;

PROCEDURE TBPGN.SetRWA(nIndex: Integer; cRWA: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aRWA := cRWA;
    bGameChange  := TRUE;
END;

PROCEDURE TBPGN.SetRBA(nIndex: Integer; cRBA: String);
var p:pbpgnrec;
BEGIN
    p:=alist.Items[nIndex];
    p^.aRBA := cRBA;
    bGameChange  := TRUE;
END;

PROCEDURE TBPGN.SetRWB(nIndex: Integer; cRWB: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aRWB := cRWB;
    bGameChange  := TRUE;
END;

PROCEDURE TBPGN.SetRBB(nIndex: Integer; cRBB: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aRBB := cRBB;
    bGameChange  := TRUE;
END;

PROCEDURE TBPGN.SetTimeC(nIndex: Integer; cTimeC: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aTimeC := cTimeC;
    bGameChange    := TRUE;
END;

PROCEDURE TBPGN.SetReason(nIndex:longint; cReason: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aReason:=cReason;
    cBPGNResult := cReason;
    bGameChange := TRUE;
END;

PROCEDURE TBPGN.SetResult(nIndex: Integer; cRC: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aResult := cRC;
    bGameChange     := TRUE;
END;

PROCEDURE TBPGN.SetOpenA(nIndex: Integer; cOpenA: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aOpenA := cOpenA;
    bGameChange    := TRUE;
END;

PROCEDURE TBPGN.SetOpenB(nIndex: Integer; cOpenB: String);
var p:pbpgnrec;
BEGIN
    p:=alist.items[nIndex];
    p^.aOpenB := cOpenB;
    bGameChange    := TRUE;
END;

FUNCTION TBPGN.GetCurrentBoard1WTime: String;
BEGIN
    Result := TOOLBOX_GetDispTimeStr(BPGNArray[nCurrent,1].nWTime);
END;

FUNCTION TBPGN.GetCurrentBoard1BTime: String;
BEGIN
    Result := TOOLBOX_GetDispTimeStr(BPGNArray[nCurrent,1].nBTime);
END;

FUNCTION TBPGN.GetCurrentBoard2WTime: String;
BEGIN
    Result := TOOLBOX_GetDispTimeStr(BPGNArray[nCurrent,2].nWTime);
END;

FUNCTION TBPGN.GetCurrentBoard2BTime: String;
BEGIN
    Result :=TOOLBOX_GetDispTimeStr(BPGNArray[nCurrent,2].nBTime);
END;

FUNCTION TBPGN.GetCurrentBoard1Caption: String;
BEGIN
    IF (BPGNIsMain[nCurrent] AND (nCurrent = nLast)) THEN
        Result := BOARD1_TITLE + ' (' + cBPGNResult + ')'
    ELSE
        Result := BPGNArray[nCurrent,1].cCaption;
END;

FUNCTION TBPGN.GetCurrentBoard2Caption: String;
BEGIN
    IF (BPGNIsMain[nCurrent] AND (nCurrent = nLast)) THEN
        Result := BOARD2_TITLE + ' (' + cBPGNResult + ')'
    ELSE
        Result := BPGNArray[nCurrent,2].cCaption;
END;

FUNCTION TBPGN.GetCurrentComment: String;
BEGIN
    Result := BPGNComment[nCurrent];
END;

FUNCTION TBPGN.GetCurrentDisplay: Boolean;
BEGIN
    Result := BPGNDisplay[nCurrent];
END;

FUNCTION TBPGN.GetCurrentNumber: Integer;
BEGIN
    IF (UpCase(BPGNBoard[nCurrent]) = 'A') THEN
        Result := BPGNNumber[nCurrent,1]
    ELSE
        Result := BPGNNumber[nCurrent,2];
END;

FUNCTION TBPGN.GetCurrentBoard: Char;
BEGIN
    Result := BPGNBoard[nCurrent];
END;

FUNCTION TBPGN.GetCurrentMove: String;
BEGIN
    Result := BPGNMove[nCurrent];
END;

FUNCTION TBPGN.GetCurrentAnno: String;
BEGIN
    Result := BPGNAnno[nCurrent];
END;

PROCEDURE TBPGN.SetCurrentBoard1WTime(nWTime: Integer);
BEGIN
    BPGNArray[nCurrent,1].nWTime := nWTime;
    bGameChange                  := TRUE;
END;

PROCEDURE TBPGN.SetCurrentBoard1BTime(nBTime: Integer);
BEGIN
    BPGNArray[nCurrent,1].nBTime := nBTime;
    bGameChange                  := TRUE;
END;

PROCEDURE TBPGN.SetCurrentBoard2WTime(nWTime: Integer);
BEGIN
    BPGNArray[nCurrent,2].nWTime := nWTime;
    bGameChange                  := TRUE;
END;

PROCEDURE TBPGN.SetCurrentBoard2BTime(nBTime: Integer);
BEGIN
    BPGNArray[nCurrent,2].nBTime := nBTime;
    bGameChange                  := TRUE;
END;

PROCEDURE TBPGN.SetCurrentBoard1Caption(cCap1: String);
BEGIN
    BPGNArray[nCurrent,1].cCaption := cCap1;
    bGameChange                    := TRUE;
END;

PROCEDURE TBPGN.SetCurrentBoard2Caption(cCap2: String);
BEGIN
    BPGNArray[nCurrent,2].cCaption := cCap2;
    bGameChange                    := TRUE;
END;

PROCEDURE TBPGN.SetCurrentComment(cComment: String);
BEGIN
    BPGNComment[nCurrent] := cComment;
    bGameChange           := TRUE;
END;

PROCEDURE TBPGN.SetCurrentDisplay(bDisplay: Boolean);
BEGIN
    BPGNDisplay[nCurrent] := bDisplay;
    bGameChange           := TRUE;
END;

PROCEDURE TBPGN.SetCurrentAnno(cAnno: String);
BEGIN
    BPGNAnno[nCurrent] := cAnno;
    bGameChange        := TRUE;
END;

FUNCTION TBPGN.GetNextNumber(nIndex: Integer): Integer;
BEGIN
    Result := aNumber[nIndex];
END;

FUNCTION TBPGN.GetNextBoard(nIndex: Integer): Char;
BEGIN
    Result := aBoard[nIndex];
END;

FUNCTION TBPGN.GetNextMove(nIndex: Integer): String;
BEGIN
    Result := aMove[nIndex];
END;

FUNCTION TBPGN.GetNextAnno(nIndex: Integer): String;
BEGIN
    Result := aAnno[nIndex];
END;

FUNCTION TBPGN.GetNextIndex(nIndex: Integer): Integer;
BEGIN
    Result := aIndex[nIndex];
END;

FUNCTION TBPGN.GetNextCount: Integer;
BEGIN
    Result := nNextCount;
END;

FUNCTION TBPGN.GetNextItem(nIndex: Integer): String;
BEGIN
    Result := IntToStr(aNumber[nIndex]) + aBoard[nIndex] + '. ' + aMove[nIndex] + Trim(aAnno[nIndex]);
END;

PROCEDURE TBPGN.LoadNextMove;
VAR
    nI, nJ, nK: Integer;
BEGIN
    nNextCount := 0;

    nI := BPGNMain[nCurrent];
    IF (nI > 0) THEN
    BEGIN
        Inc(nNextCount);
        IF (UpCase(BPGNBoard[nI]) = 'A') THEN
            aNumber[nNextCount] := BPGNNumber[nI,1]
        ELSE
            aNumber[nNextCount] := BPGNNumber[nI,2];

        aBoard[nNextCount] := BPGNBoard[nI];
        aMove [nNextCount] := BPGNMove [nI];
        aAnno [nNextCount] := BPGNAnno [nI];
        aIndex[nNextCount] := nI;

        FOR nJ := 1 TO MAX_NEXT DO
        BEGIN
            nK := BPGNNext[nCurrent,nJ];
            IF ((nK > 0) AND (nK <> nI)) THEN
            BEGIN
                Inc(nNextCount);
                IF (UpCase(BPGNBoard[nK]) = 'A') THEN
                    aNumber[nNextCount] := BPGNNumber[nK,1]
                ELSE
                    aNumber[nNextCount] := BPGNNumber[nK,2];

                aBoard[nNextCount] := BPGNBoard[nK];
                aMove [nNextCount] := BPGNMove [nK];
                aAnno [nNextCount] := BPGNAnno [nK];
                aIndex[nNextCount] := nK;
            END;
        END;
    END;
END;

FUNCTION TBPGN.GetPNextNumber(nIndex: Integer): Integer;
BEGIN
    Result := aPNumber[nIndex];
END;

FUNCTION TBPGN.GetPNextBoard(nIndex: Integer): Char;
BEGIN
    Result := aPBoard[nIndex];
END;

FUNCTION TBPGN.GetPNextMove(nIndex: Integer): String;
BEGIN
    Result := aPMove[nIndex];
END;

FUNCTION TBPGN.GetPNextAnno(nIndex: Integer): String;
BEGIN
    Result := aPAnno[nIndex];
END;

FUNCTION TBPGN.GetPNextIndex(nIndex: Integer): Integer;
BEGIN
    Result := aPIndex[nIndex];
END;

FUNCTION TBPGN.GetPNextCount: Integer;
BEGIN
    Result := nPNextCount;
END;

FUNCTION TBPGN.GetPNextItem(nIndex: Integer): String;
BEGIN
    Result := IntToStr(aPNumber[nIndex]) + aPBoard[nIndex] + '. ' + aPMove[nIndex] + Trim(aPAnno[nIndex]);
END;

PROCEDURE TBPGN.LoadPNextMove;
VAR
    nI, nJ, nK, nM: Integer;
BEGIN
    nPNextCount := 0;

    nI := BPGNParent[nCurrent];
    IF (nI > 0) THEN
    BEGIN
        nM := BPGNMain[nI];
        IF (nM <> nCurrent) THEN
        BEGIN
            Inc(nPNextCount);
            IF (UpCase(BPGNBoard[nM]) = 'A') THEN
                aPNumber[nPNextCount] := BPGNNumber[nM,1]
            ELSE
                aPNumber[nPNextCount] := BPGNNumber[nM,2];

            aPBoard[nPNextCount] := BPGNBoard[nM];
            aPMove [nPNextCount] := BPGNMove [nM];
            aPAnno [nPNextCount] := BPGNAnno [nM];
            aPIndex[nPNextCount] := nM;
        END;

        FOR nJ := 1 TO MAX_NEXT DO
        BEGIN
            nK := BPGNNext[nI,nJ];
            IF ((nK > 0) AND (nK <> nCurrent) AND (nK <> nM)) THEN
            BEGIN
                Inc(nPNextCount);
                IF (UpCase(BPGNBoard[nK]) = 'A') THEN
                    aPNumber[nPNextCount] := BPGNNumber[nK,1]
                ELSE
                    aPNumber[nPNextCount] := BPGNNumber[nK,2];

                aPBoard[nPNextCount] := BPGNBoard[nK];
                aPMove [nPNextCount] := BPGNMove [nK];
                aPAnno [nPNextCount] := BPGNAnno [nK];
                aPIndex[nPNextCount] := nK;
            END;
        END;
    END;
END;

PROCEDURE TBPGN.LoadNavigator;
VAR
    nI, nJ, nK: Integer;
BEGIN
    { Adjust navigate first and previous }
    aNavAll[1] := (BPGNParent[nCurrent] > 0);
    aNavAll[2] := aNavAll[1];


    { Adjust navigate next and last }
    aNavAll[3] := (BPGNMain[nCurrent] > 0);
    aNavAll[4] := aNavAll[3];


    { Adjust board #1's first and previous }
    IF (nCurrent = 1) THEN
    BEGIN
        aNavBoard1[1] := FALSE;
        aNavBoard1[2] := FALSE;
    END
    ELSE
    BEGIN
        aPrevious[1] := 0;
        nJ           := 0;
        nI           := BPGNParent[nCurrent];
        WHILE (nI > 0) DO
        BEGIN
            IF ((aPrevious[1] = 0) AND (UpCase(BPGNBoard[nI]) = 'A')) THEN
            BEGIN
                IF (UpCase(BPGNBoard[nCurrent]) = 'A') THEN
                BEGIN
                    aPrevious[1] := nI;
                    BREAK;
                END
                ELSE
                BEGIN
                    IF (nJ = 0) THEN
                        nJ := 1
                    ELSE
                    BEGIN
                        aPrevious[1] := nI;
                        BREAK;
                    END;
                END;
            END;
            nI := BPGNParent[nI];
        END;

        IF (aPrevious[1] = 0) THEN
        BEGIN
            IF (aFirst[1] = 0) THEN
            BEGIN
                aNavBoard1[1] := FALSE;
                aNavBoard1[2] := FALSE;
            END
            ELSE
            BEGIN
                IF (aFirst[1] = nCurrent) THEN
                BEGIN
                    aNavBoard1[1] := FALSE;
                    aNavBoard1[2] := FALSE;
                END
                ELSE
                BEGIN
                    nJ := 0;
                    nI := aFirst[1];
                    WHILE (nI > 0) DO
                    BEGIN
                        IF (nI = nCurrent) THEN
                        BEGIN
                            nJ := nI;
                            BREAK;
                        END;
                        nI := BPGNParent[nI];
                    END;
                    aPrevious [1] := aFirst[1];
                    aNavBoard1[1] := (nJ = 0);
                    aNavBoard1[2] := aNavBoard1[1];
                END;
            END;
        END
        ELSE
        BEGIN
            aNavBoard1[1] := TRUE;
            aNavBoard1[2] := TRUE;
        END;
    END;


    { Adjust board #2's first and previous }
    IF (nCurrent = 1) THEN
    BEGIN
        aNavBoard2[1] := FALSE;
        aNavBoard2[2] := FALSE;
    END
    ELSE
    BEGIN
        aPrevious[2] := 0;
        nJ           := 0;
        nI           := BPGNParent[nCurrent];
        WHILE (nI > 0) DO
        BEGIN
            IF ((aPrevious[2] = 0) AND (UpCase(BPGNBoard[nI]) = 'B')) THEN
            BEGIN
                IF (UpCase(BPGNBoard[nCurrent]) = 'B') THEN
                BEGIN
                    aPrevious[2] := nI;
                    BREAK;
                END
                ELSE
                BEGIN
                    IF (nJ = 0) THEN
                        nJ := 1
                    ELSE
                    BEGIN
                        aPrevious[2] := nI;
                        BREAK;
                    END;
                END;
            END;
            nI := BPGNParent[nI];
        END;

        IF (aPrevious[2] = 0) THEN
        BEGIN
            IF (aFirst[2] = 0) THEN
            BEGIN
                aNavBoard2[1] := FALSE;
                aNavBoard2[2] := FALSE;
            END
            ELSE
            BEGIN
                IF (aFirst[2] = nCurrent) THEN
                BEGIN
                    aNavBoard2[1] := FALSE;
                    aNavBoard2[2] := FALSE;
                END
                ELSE
                BEGIN
                    nJ := 0;
                    nI := aFirst[2];
                    WHILE (nI > 0) DO
                    BEGIN
                        IF (nI = nCurrent) THEN
                        BEGIN
                            nJ := nI;
                            BREAK;
                        END;
                        nI := BPGNParent[nI];
                    END;
                    aPrevious [2] := aFirst[2];
                    aNavBoard2[1] := (nJ = 0);
                    aNavBoard2[2] := aNavBoard2[1];
                END;
            END;
        END
        ELSE
        BEGIN
            aNavBoard2[1] := TRUE;
            aNavBoard2[2] := TRUE;
        END;
    END;


    { Adjust board #1 and board #2's next }
    nLast    := nCurrent;
    aNext[1] := 0;
    aNext[2] := 0;
    nI       := BPGNMain[nCurrent];
    WHILE (nI > 0) DO
    BEGIN
        nLast := nI;
        IF ((aNext[1] = 0) AND (UpCase(BPGNBoard[nI]) = 'A')) THEN
        BEGIN
            aNext[1] := nI;
            IF (aNext[2] > 0) THEN
                BREAK;
        END;
        IF ((aNext[2] = 0) AND (UpCase(BPGNBoard[nI]) = 'B')) THEN
        BEGIN
            aNext[2] := nI;
            IF (aNext[1] > 0) THEN
                BREAK;
        END;
        nI := BPGNMain[nI];
    END;


    { Adjust board #1 and board #2's last }
    WHILE (nI > 0) DO
    BEGIN
        nLast := nI;
        nI    := BPGNMain[nI];
    END;
    IF (aNext[1] = 0) THEN
        aNext[1] := nLast;

    IF (aNext[2] = 0) THEN
        aNext[2] := nLast;

    aNavBoard1[3] := (nCurrent <> nLast);
    aNavBoard1[4] := aNavBoard1[3];
    aNavBoard2[3] := aNavBoard1[3];
    aNavBoard2[4] := aNavBoard1[3];
END;

FUNCTION TBPGN.GetAllNavigator(nIndex: Integer): Boolean;
BEGIN
    Result := aNavAll[nIndex];
END;

FUNCTION TBPGN.GetBoard1Navigator(nIndex: Integer): Boolean;
BEGIN
    Result := aNavBoard1[nIndex];
END;

FUNCTION TBPGN.GetBoard2Navigator(nIndex: Integer): Boolean;
BEGIN
    Result := aNavBoard2[nIndex];
END;

PROCEDURE TBPGN.MakeAllNav1Move;
BEGIN
    nCurrent := 1;
END;

PROCEDURE TBPGN.MakeAllNav2Move;
BEGIN
    nCurrent := BPGNParent[nCurrent];
END;

PROCEDURE TBPGN.MakeAllNav3Move;
BEGIN
    nCurrent := BPGNMain[nCurrent];
END;

PROCEDURE TBPGN.MakeAllNav4Move;
BEGIN
    nCurrent := nLast;
END;

Procedure TBPGN.MakeAllNav5Move;   //prev comment
Begin
 nCurrent:=BPGNParent[ncurrent];
 While ((ncurrent>1) and (trim(GetBpgnComment(ncurrent))='')) do
  nCurrent:=BPGNParent[ncurrent];
end;

Procedure TBPGN.MakeAllNav8Move;  //next comment
Begin
 nCurrent:=BPGNMain[ncurrent];
 While ((ncurrent<nLast) and (trim(GetBpgnComment(ncurrent))='')) do
  nCurrent:=BPGNMain[ncurrent];
end;

Procedure TBPGN.MakeAllNav6Move;   //prev fork
Begin
 nCurrent:=BPGNParent[ncurrent];
 While ((ncurrent>1) and (getNextCnt(nCurrent)=1)) do
  nCurrent:=BPGNParent[ncurrent];
end;

Procedure TBPGN.MakeAllNav7Move;   //next fork
Begin
 nCurrent:=BPGNMain[ncurrent];
 While ((ncurrent<nLast) and (getNextCnt(nCurrent)=1)) do
  nCurrent:=BPGNMain[ncurrent];
end;


PROCEDURE TBPGN.MakeBoard1Nav1Move;
BEGIN
    nCurrent := aFirst[1];
END;

PROCEDURE TBPGN.MakeBoard1Nav2Move;
BEGIN
    nCurrent := aPrevious[1];
END;

PROCEDURE TBPGN.MakeBoard1Nav3Move;
BEGIN
    nCurrent := aNext[1];
END;

PROCEDURE TBPGN.MakeBoard1Nav4Move;
BEGIN
    nCurrent := nLast;
END;

PROCEDURE TBPGN.MakeBoard2Nav1Move;
BEGIN
    nCurrent := aFirst[2];
END;

PROCEDURE TBPGN.MakeBoard2Nav2Move;
BEGIN
    nCurrent := aPrevious[2];
END;

PROCEDURE TBPGN.MakeBoard2Nav3Move;
BEGIN
    nCurrent := aNext[2];
END;

PROCEDURE TBPGN.MakeBoard2Nav4Move;
BEGIN
    nCurrent := nLast;
END;

PROCEDURE TBPGN.MakeNextMove(nIndex: Integer);
BEGIN
    nCurrent := aIndex[nIndex];
END;

PROCEDURE TBPGN.MakePrevMove(nIndex: Integer);
BEGIN
    nCurrent := aPIndex[nIndex];
END;

PROCEDURE TBPGN.MakeBoardMove(cC1, cC2: TChess; cBoard: Char; cMove: String; nMI: Integer; cCap1, cCap2: String);
VAR
    nI, nJ, nP: Integer;
BEGIN
    nJ := -1;
    FOR nI := 1 TO nNextCount DO
    BEGIN
        IF ((cBoard = aBoard[nI]) AND (CompareStr(cMove, aMove[nI]) = 0)) THEN
        BEGIN
            nJ := nI;
            BREAK;
        END;
    END;
    IF (nJ <> -1) THEN
    BEGIN
        nCurrent := aIndex[nJ];
        EXIT;
    END;

    nBPGNIndex := 0;
    FOR nI := 1 TO MAX_POSITION DO
    BEGIN
        IF (NOT BPGNInUse[nI]) THEN
        BEGIN
            nBPGNIndex := nI;
            BREAK;
        END;
    END;
    IF (nBPGNIndex = 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Maximum # of positions exceeded (' + IntToStr(MAX_POSITION) + ').';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;
    CopyBoardsIntoBPGNRecord(cC1, cC2, nBPGNIndex, cCap1, cCap2);

    nJ := 0;
    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nCurrent,nI] = 0) THEN
        BEGIN
            nJ := nI;
            BREAK;
        END;
    END;
    IF (nJ = 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Maximum # of next moves per position exceeded (' + IntToStr(MAX_NEXT) + ').';
        IF (bPromptError) THEN wError(cResult);
        EXIT;
    END;

    BPGNNext[nCurrent,nJ]  := nBPGNIndex;
    BPGNInUse [nBPGNIndex] := TRUE;
    BPGNParent[nBPGNIndex] := nCurrent;

    IF (BPGNMain[nCurrent] = 0) THEN
    BEGIN
        BPGNMain[nCurrent]     := nBPGNIndex;
        BPGNIsMain[nBPGNIndex] := BPGNIsMain[nCurrent];
    END
    ELSE
        BPGNIsMain[nBPGNIndex] := FALSE;

    IF (cBoard = 'A') THEN
    BEGIN
        BPGNNumber[nBPGNIndex,1] := BPGNNumber[nCurrent,1] + 1;
        BPGNNumber[nBPGNIndex,2] := BPGNNumber[nCurrent,2];
        BPGNArray[nBPGNIndex,1].cCaption := BOARD1_TITLE + ' (' + IntToStr(BPGNNumber[nBPGNIndex,1]) + cBoard + '. ' + cMove + ')';

        IF ((cC1.aMove[nMI].bCapturedIsPromotedPiece) or (cc1.aMove[nMi].cType=MOVE_ENPASANT))THEN
        BEGIN
            IF (Pos(Crazyhouse_Id,lowercase(getevent(nGameHeader))) = 0) THEN
if not bairdrop then cC2.SetPieces(cC2.GetWhitePieceStr, cC2.GetBlackPieceStr + cC1.GetBlackPawn)
               else cc2.AddDropPiece(cc1.GetBlackPawn)
            ELSE
                cC1.SetPieces(cC1.GetWhitePieceStr + cC1.GetWhitePawn, cC1.GetBlackPieceStr);
//        cc1.AddDropPiece(cc1.GetWhitePawn);

            CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
        END
        ELSE
        BEGIN
            IF (NOT cC1.IsEmpty(cC1.aMove[nMI].cCapture)) THEN
            BEGIN
                IF (Pos(Crazyhouse_Id,lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC2.SetPieces(cC2.GetWhitePieceStr, cC2.GetBlackPieceStr + cC1.aMove[nMI].cCapture)
                else cc2.AddDropPiece(Cc1.aMove[nmi].cCapture)
                ELSE
                    cC1.SetPieces(cC1.GetWhitePieceStr + cC1.GetRevPiece(cC1.aMove[nMI].cCapture), cC1.GetBlackPieceStr);
//                cc1.AddDropPiece(cc1.GetRevPiece(Cc1.aMove[nmi].cCapture));
                CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
            END;
        END;
    END
    ELSE
    IF (cBoard = 'a') THEN
    BEGIN
        BPGNNumber[nBPGNIndex,1] := BPGNNumber[nCurrent,1];
        BPGNNumber[nBPGNIndex,2] := BPGNNumber[nCurrent,2];
        BPGNArray[nBPGNIndex,1].cCaption := BOARD1_TITLE + ' (' + IntToStr(BPGNNumber[nBPGNIndex,1]) + cBoard + '. ' + cMove + ')';

        IF ((cC1.aMove[nMI].bCapturedIsPromotedPiece)or (cc1.aMove[nMi].cType=MOVE_ENPASANT)) THEN
        BEGIN
            IF (Pos(Crazyhouse_Id,lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC2.SetPieces(cC2.GetWhitePieceStr + cC1.GetWhitePawn, cC2.GetBlackPieceStr)
                else cc2.AddDropPiece(cc1.GetWhitePawn)
            ELSE
                cC1.SetPieces(cC1.GetWhitePieceStr, cC1.GetBlackPieceStr + cC1.GetBlackPawn);
//                cc1.AddDropPiece(cc1.GetBlackPawn);

            CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
        END
        ELSE
        BEGIN
            IF (NOT cC1.IsEmpty(cC1.aMove[nMI].cCapture)) THEN
            BEGIN
                IF (Pos(Crazyhouse_Id,lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC2.SetPieces(cC2.GetWhitePieceStr + cC1.aMove[nMI].cCapture, cC2.GetBlackPieceStr)
                else cc2.AddDropPiece(Cc1.aMove[nmi].cCapture)
                ELSE
                    cC1.SetPieces(cC1.GetWhitePieceStr, cC1.GetBlackPieceStr + cC1.GetRevPiece(cC1.aMove[nMI].cCapture));
//                cc1.AddDropPiece(cc1.GetRevPiece(Cc1.aMove[nmi].cCapture));

                CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
            END;
        END;
    END
    ELSE
    IF (cBoard = 'B') THEN
    BEGIN
        BPGNNumber[nBPGNIndex,1] := BPGNNumber[nCurrent,1];
        BPGNNumber[nBPGNIndex,2] := BPGNNumber[nCurrent,2] + 1;
        BPGNArray[nBPGNIndex,2].cCaption := BOARD2_TITLE + ' (' + IntToStr(BPGNNumber[nBPGNIndex,2]) + cBoard + '. ' + cMove + ')';

        IF ((cC2.aMove[nMI].bCapturedIsPromotedPiece)or (cc2.aMove[nMi].cType=MOVE_ENPASANT)) THEN
        BEGIN
            IF (Pos(Crazyhouse_Id,lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC1.SetPieces(cC1.GetWhitePieceStr, cC1.GetBlackPieceStr + cC2.GetBlackPawn)
                else cc1.AddDropPiece(cc2.GetBlackPawn)
            ELSE
                cC2.SetPieces(cC2.GetWhitePieceStr + cC2.GetWhitePawn, cC2.GetBlackPieceStr);
//                cc2.AddDropPiece(cc2.GetWhitePawn);

            CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
        END
        ELSE
        BEGIN
            IF (NOT cC2.IsEmpty(cC2.aMove[nMI].cCapture)) THEN
            BEGIN
                IF (Pos(Crazyhouse_Id,lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC1.SetPieces(cC1.GetWhitePieceStr, cC1.GetBlackPieceStr + cC2.aMove[nMI].cCapture)
                else cc1.AddDropPiece(Cc2.aMove[nmi].cCapture)
                ELSE
                    cC2.SetPieces(cC2.GetWhitePieceStr + cC2.GetRevPiece(cC2.aMove[nMI].cCapture), cC2.GetBlackPieceStr);
//                cc2.AddDropPiece(cc2.GetRevPiece(Cc2.aMove[nmi].cCapture));

                CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
            END;
        END;
    END
    ELSE
    IF (cBoard = 'b') THEN
    BEGIN
        BPGNNumber[nBPGNIndex,1] := BPGNNumber[nCurrent,1];
        BPGNNumber[nBPGNIndex,2] := BPGNNumber[nCurrent,2];
        BPGNArray[nBPGNIndex,2].cCaption := BOARD2_TITLE + ' (' + IntToStr(BPGNNumber[nBPGNIndex,2]) + cBoard + '. ' + cMove + ')';

        IF ((cC2.aMove[nMI].bCapturedIsPromotedPiece)or (cc2.aMove[nMi].cType=MOVE_ENPASANT)) THEN
        BEGIN
            IF (Pos(Crazyhouse_Id,lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC1.SetPieces(cC1.GetWhitePieceStr + cC2.GetWhitePawn, cC1.GetBlackPieceStr)
                else cc1.AddDropPiece(cc2.GetWhitePawn)
            ELSE
                cC2.SetPieces(cC2.GetWhitePieceStr, cC2.GetBlackPieceStr + cC2.GetBlackPawn);
//                cc2.AddDropPiece(cc2.GetBlackPawn);

            CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
        END
        ELSE
        BEGIN
            IF (NOT cC2.IsEmpty(cC2.aMove[nMI].cCapture)) THEN
            BEGIN
                IF (Pos(Crazyhouse_Id, lowercase(getEvent(nGameHeader))) = 0) THEN
if not bairdrop then cC1.SetPieces(cC1.GetWhitePieceStr + cC2.aMove[nMI].cCapture, cC1.GetBlackPieceStr)
                else cc1.AddDropPiece(Cc2.aMove[nmi].cCapture)
                ELSE
                    cC2.SetPieces(cC2.GetWhitePieceStr, cC2.GetBlackPieceStr + cC2.GetRevPiece(cC2.aMove[nMI].cCapture));
//                cc2.AddDropPiece(Cc2.aMove[nmi].cCapture);

                CopyBoardPiecesIntoBPGNRecord(cC1, cC2, nBPGNIndex);
            END;
        END;
    END;

    FOR nI := 1 TO MAX_NEXT DO
        BPGNNext[nBPGNIndex, nI] := 0;

    BPGNMain   [nBPGNIndex] := 0;
    BPGNBoard  [nBPGNIndex] := cBoard;
    BPGNMove   [nBPGNIndex] := cMove;
    BPGNAnno   [nBPGNIndex] := '';
    BPGNDisplay[nBPGNIndex] := FALSE;
    BPGNComment[nBPGNIndex] := '';

    BPGNArray[nBPGNIndex,1].nWTime := BPGNArray[nCurrent,1].nWTime;
    BPGNArray[nBPGNIndex,1].nBTime := BPGNArray[nCurrent,1].nBTime;
    BPGNArray[nBPGNIndex,2].nWTime := BPGNArray[nCurrent,2].nWTime;
    BPGNArray[nBPGNIndex,2].nBTime := BPGNArray[nCurrent,2].nBTime;

    BPGNArray[nBPGNIndex,1].nWLag := BPGNArray[nCurrent,1].nWLag;
    BPGNArray[nBPGNIndex,1].nBLag := BPGNArray[nCurrent,1].nBLag;
    BPGNArray[nBPGNIndex,2].nWLag := BPGNArray[nCurrent,2].nWLag;
    BPGNArray[nBPGNIndex,2].nBLag := BPGNArray[nCurrent,2].nBLag;
    BPGNLag[nBPGNIndex]:=0;

    aFound[1] := FALSE;
    aFound[2] := FALSE;
    aFirst[1] := 0;
    aFirst[2] := 0;
    nP        := 1;
    nI        := BPGNMain[nP];
    WHILE (nI > 0) DO
    BEGIN
        IF (BPGNBoard[nI] = 'A') THEN
        BEGIN
            IF (NOT aFound[1]) THEN
            BEGIN
                aFound[1] := TRUE;
                aFirst[1] := nP;
                IF (aFound[2]) THEN
                    BREAK;
            END;
        END
        ELSE
        IF (BPGNBoard[nI] = 'B') THEN
        BEGIN
            IF (NOT aFound[2]) THEN
            BEGIN
                aFound[2] := TRUE;
                aFirst[2] := nP;
                IF (aFound[1]) THEN
                    BREAK;
            END;
        END;
        nP := nI;
        nI := BPGNMain[nP];
    END;


    nCurrent    := nBPGNIndex;
    bGameChange := TRUE;
END;

PROCEDURE TBPGN.UpdateComment(nIndex: Integer; cGame: String);
VAR
    bFound: Boolean;
    nLength, nStart, nLast, nNumber, nB, nE, nI, nL, nN, nP: LongInt;
    cT, cN: String;
BEGIN
    nLength := Length(BPGNComment[nIndex]);
    IF (nLength = 0) THEN
    BEGIN
        BPGNComment[nIndex] := '<Occurrence: 1' + cGame;
        EXIT;
    END;

    bFound  := FALSE;
    nStart  := 1;
    nLast   := 0;
    nNumber := 0;
    WHILE (nStart < nLength) DO
    BEGIN
        { Find < }
        nB := 0;
        FOR nI := nStart TO nLength DO
        BEGIN
            IF (BPGNComment[nIndex][nI] = '<') THEN
            BEGIN
                nB := nI;
                BREAK;
            END;
        END;
        IF (nB = 0) THEN
            BREAK;

        { Find > }
        nE := 0;
        FOR nI := nB TO nLength DO
        BEGIN
            IF (BPGNComment[nIndex][nI] = '>') THEN
            BEGIN
                nE := nI;
                BREAK;
            END;
        END;
        IF (nE = 0) THEN
            BREAK;

        { Get tag string }
        cT := Copy(BPGNComment[nIndex], nB, (nE - nB) + 1);

        { Check for begining of tag string }
        IF (CompareText(Copy(cT, 1, length(BPGN_OCCUR_STR)), BPGN_OCCUR_STR) <> 0) THEN
        BEGIN
            nStart := nE + 1;
            CONTINUE;
        END;

        { Set last position }
        nLast := nE + 1;
        FOR nI := (nE + 1) TO (nLength - 1) DO
        BEGIN
            IF ((BPGNComment[nIndex][nI    ] = CR_ASCII_CHAR)  AND
                (BPGNComment[nIndex][nI + 1] = LF_ASCII_CHAR)) THEN
            BEGIN
                nLast := nI + 2;
                BREAK;
            END;
        END;

        { Get occurrence # }
        cN := '';
        nL := Length(cT);
        FOR nI := length(BPGN_OCCUR_STR) + 1 TO nL DO
        BEGIN
            IF ((cT[nI] < '0') OR (cT[nI] > '9')) THEN
                BREAK;

            cN := cN + cT[nI];
        END;
        IF (Length(cN) > 0) THEN
        BEGIN
            nN := StrToInt(cN);
            IF (nN > nNumber) THEN
                nNumber := nN;
        END;

        { Check for end of tag string }
        nP := Pos(cGame, cT);
        IF (nP = 0) THEN
        BEGIN
            nStart := nE + 1;
            CONTINUE;
        END;

        { Found }
        bFound := TRUE;
        BREAK;
    END;
    IF (NOT bFound) THEN
    BEGIN
        cT := '<Occurrence: ' + IntToStr(nNumber + 1) + cGame;

        IF (nLast = 0) THEN
            BPGNComment[nIndex] := cT
        ELSE
        IF (nLast >= nLength) THEN
            BPGNComment[nIndex] := BPGNComment[nIndex] + EOL_STR + cT
        ELSE
        BEGIN
            BPGNComment[nIndex] := Copy(BPGNComment[nIndex], 1, nLast - 1) +
                                   cT + EOL_STR +
                                   Copy(BPGNComment[nIndex], nLast, nLength - nLast - 1);
        END;
    END;

    IF (Length(BPGNComment[nIndex]) > MAX_MEMO_CHAR) THEN
        BPGNComment[nIndex] := Copy(BPGNComment[nIndex],1,MAX_MEMO_CHAR);
END;

FUNCTION TBPGN.AddExtractMove(nParent, nMove: Integer; cBoard: Char; cMove: String; nTime: Integer; cGame: String): Integer;
VAR
    nI, nJ, nK: Integer;
BEGIN
    nJ := -1;
    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        nK := BPGNNext[nParent,nI];
        IF (nK > 0) THEN
        BEGIN
            IF ((nMove = BPGNNumber[nK,1]) AND (cBoard = BPGNBoard[nK]) AND (CompareStr(cMove, BPGNMove[nK]) = 0)) THEN
            BEGIN
                nJ := nK;
                BREAK;
            END;
        END;
    END;
    IF (nJ <> -1) THEN
    BEGIN
        IF (INI1.INI.bGenOccurrence) THEN
        BEGIN
            UpdateComment(nJ, cGame);
        END;
        bGameChange := TRUE;
        Result      := nJ;
        EXIT;
    END;

    nBPGNIndex := 0;
    FOR nI := 1 TO MAX_POSITION DO
    BEGIN
        IF (NOT BPGNInUse[nI]) THEN
        BEGIN
            nBPGNIndex := nI;
            BREAK;
        END;
    END;
    IF (nBPGNIndex = 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Maximum # of positions exceeded (' + IntToStr(MAX_POSITION) + ').';
        IF (bPromptError) THEN wError(cResult);
        Result     := -1;
        EXIT;
    END;

    FOR nI := 1 TO 2 DO
    BEGIN
        FOR nJ := 1 TO 2 DO
        BEGIN
            BPGNArray[nBPGNIndex,nI].aKingBX[nJ] := 0;
            BPGNArray[nBPGNIndex,nI].aKingBY[nJ] := 0;

            BPGNArray[nBPGNIndex,nI].aKCastle[nJ] := FALSE;
            BPGNArray[nBPGNIndex,nI].aQCastle[nJ] := FALSE;
        END;

        BPGNArray[nBPGNIndex,nI].mPrev.nFromX                   := 0;
        BPGNArray[nBPGNIndex,nI].mPrev.nFromY                   := 0;
        BPGNArray[nBPGNIndex,nI].mPrev.nToX                     := 0;
        BPGNArray[nBPGNIndex,nI].mPrev.nToY                     := 0;
        BPGNArray[nBPGNIndex,nI].mPrev.cPiece                   := ' ';
        BPGNArray[nBPGNIndex,nI].mPrev.cPromote                 := ' ';
        BPGNArray[nBPGNIndex,nI].mPrev.cCapture                 := ' ';
        BPGNArray[nBPGNIndex,nI].mPrev.cType                    := ' ';
        BPGNArray[nBPGNIndex,nI].mPrev.bCapturedIsPromotedPiece := FALSE;

        FOR nJ := 1 TO 8 DO
        BEGIN
            FOR nK := 1 TO 8 DO
            BEGIN
                BPGNArray[nBPGNIndex,nI].aBoard  [nJ,nK] := ' ';
                BPGNArray[nBPGNIndex,nI].aPromote[nJ,nK] := FALSE;
            END;
        END;

        FOR nJ := 1 TO 2 DO
        BEGIN
            FOR nK := 1 TO 8 DO
            BEGIN
                BPGNArray[nBPGNIndex,nI].acPiece[nJ,nK] := ' ';
                BPGNArray[nBPGNIndex,nI].anPiece[nJ,nK] := 0;
            END;
        END;

        BPGNArray[nBPGNIndex,nI].bWhitesMove := FALSE;
        BPGNArray[nBPGNIndex,nI].cCaption    := '';
    END;

    nJ := 0;
    FOR nI := 1 TO MAX_NEXT DO
    BEGIN
        IF (BPGNNext[nParent,nI] = 0) THEN
        BEGIN
            nJ := nI;
            BREAK;
        END;
    END;
    IF (nJ = 0) THEN
    BEGIN
        bGameDone  := TRUE;
        bGameError := TRUE;
        cResult    := cFile + EOL_STR + 'Game #' + IntToStr(nGameHeader+1) + EOL_STR +
                      'Maximum # of next moves per position exceeded (' + IntToStr(MAX_NEXT) + ').';
        IF (bPromptError) THEN wError(cResult);
        Result     := 0;
        EXIT;
    END;

    BPGNNext[nParent,nJ]   := nBPGNIndex;
    BPGNInUse [nBPGNIndex] := TRUE;
    BPGNParent[nBPGNIndex] := nParent;

    IF (BPGNMain[nParent] = 0) THEN
    BEGIN
        BPGNMain[nParent]      := nBPGNIndex;
        BPGNIsMain[nBPGNIndex] := BPGNIsMain[nParent];
    END
    ELSE
        BPGNIsMain[nBPGNindex] := FALSE;

    BPGNArray[nBPGNIndex,1].nWTime := BPGNArray[nParent,1].nWTime;
    BPGNArray[nBPGNIndex,1].nBTime := BPGNArray[nParent,1].nBTime;
    BPGNArray[nBPGNIndex,2].nWTime := BPGNArray[nParent,2].nWTime;
    BPGNArray[nBPGNIndex,2].nBTime := BPGNArray[nParent,2].nBTime;

    IF (cBoard = 'A') THEN BPGNArray[nBPGNIndex,1].nWTime := nTime ELSE
    IF (cBoard = 'a') THEN BPGNArray[nBPGNIndex,1].nBTime := nTime;

    FOR nI := 1 TO MAX_NEXT DO
        BPGNNext[nBPGNIndex, nI] := 0;

    BPGNNumber[nBPGNIndex,1] := nMove;
    BPGNNumber[nBPGNIndex,2] := BPGNNumber[nParent,2];

    BPGNMain   [nBPGNIndex] := 0;
    BPGNBoard  [nBPGNIndex] := cBoard;
    BPGNMove   [nBPGNIndex] := cMove;
    BPGNAnno   [nBPGNIndex] := '';
    BPGNDisplay[nBPGNIndex] := FALSE;

    IF (INI1.INI.bGenOccurrence) THEN
        BPGNComment[nBPGNIndex] := '<Occurrence: 1' + cGame
    ELSE
        BPGNComment[nBPGNIndex] := '';

    bGameChange := TRUE;
    Result      := nBPGNIndex;
END;



FUNCTION TBPGN.IsOpen: Boolean;
BEGIN
    Result := bOpen;
END;

FUNCTION TBPGN.IsEmpty: Boolean;
BEGIN
    Result := (nBPGNIndex <= 0);
END;

FUNCTION TBPGN.IsAtStartingPosition: Boolean;
BEGIN
    Result := (nCurrent <= 1);
END;

FUNCTION TBPGN.IsGameChange: Boolean;
BEGIN
    Result := bGameChange;
END;

FUNCTION TBPGN.IsFileChange: Boolean;
BEGIN
    Result := bFileChange;
END;

FUNCTION TBPGN.GetBPGNFilename: String;
BEGIN
    Result := cFile;
END;

FUNCTION TBPGN.GetPositionIndex: Integer;
BEGIN
    Result := nCurrent;
END;

PROCEDURE TBPGN.SetStartTime(nST: Integer);
BEGIN
    BPGNArray[1,1].nWTime := nST;
    BPGNArray[1,1].nBTime := nST;
    BPGNArray[1,2].nWTime := nST;
    BPGNArray[1,2].nBTime := nST;
    bGameChange           := TRUE;
END;

procedure TBPGN.NewGameString(p:pbpgnrec; var s:string);
BEGIN
 with p^ do
 begin
         s := '[Event "' + aEvent + '"]' + EOL_STR +
              '[Site "'  + aSite  + '"]' + EOL_STR +
              '[Date "'  + aDate  + '"]' + EOL_STR +
              '[WhiteA "' + aWhiteA + '"][WhiteAElo "' + aRWA + '"]' + EOL_STR +
              '[BlackA "' + aBlackA + '"][BlackAElo "' + aRBA + '"]' + EOL_STR +
              '[WhiteB "' + aWhiteB + '"][WhiteBElo "' + aRWB + '"]' + EOL_STR +
              '[BlackB "' + aBlackB + '"][BlackBElo "' + aRBB + '"]' + EOL_STR +
              '[TimeControl "' + aTimeC + '"]' + EOL_STR +
              '[Result "' + aResult + '"]' + EOL_STR;
         if aSetup='1' then
         s:=s+'[Setup "1"]'+EOL_STR+
              '[FEN "'+aFen+'"]'+EOL_STR;
         s:=s+EOL_STR +
              aReason + EOL_STR;
 end;
END;

PROCEDURE TBPGN.NewFile(cFn, cWhiteA, cBlackA, cWhiteB, cBlackB: String);
VAR
    OutFile: TextFile;
    s:string;
    rec:tbpgnrec;
BEGIN
    AssignFile(OutFile, cFn);
    Rewrite(OutFile);
    BPGNrecFillempty(@rec);
    Rec.aWhiteA:=cWhitea;
    rec.aWhiteB:=cWhiteb;
    rec.aBlackA:=cBlackA;
    rec.aBlackB:=cBlackB;
    NewGameString(@rec,s);
    Write(OutFile,s);
    CloseFile(OutFile);
END;


PROCEDURE TBPGN.AppendNewGame(pnew:pbpgnrec);
VAR
    OutFile: TextFile;
    p:pbpgnrec;
BEGIN
    { Close BPGN file }
    CloseFile(BPGNFile);

    { Append game }
    AssignFile(OutFile, cFile);
    Append(OutFile);
    NewGameString(pnew,cActualGame);
    Write(OutFile, cActualGame);
    CloseFile(OutFile);

    { Assign game list stuff }
    alist.add(p);
    p:=alist.Last;
    p^:=pnew^;
    p^.aPosition:=nfilesize;
    if bisdb then
    begin
     bdbchange:=true;
     p^.aPositionW:=pdblist[1].Count;
     p^.aPositionB:=pdblist[2].Count;
    end;

    P:=alist.Items[nFilterCount];
    p^.aFilterN:=nHeaderCount;

    inc(nFilterCount);
    Inc(nHeaderCount);


    { Get file size }
    AssignFile(ByteFile, cFile);
    Reset(ByteFile);
    nFileSize := FileSize(ByteFile);
    CloseFile(ByteFile);

    { Open file }
    AssignFile(BPGNFile, cFile);
    Reset(BPGNFile);
END;

PROCEDURE TBPGN.DuplicateGame(nIndex: Integer);
VAR
    OutFile: TextFile;
    p,p1:pbpgnrec;
BEGIN
    { Get actual game text }
    GetActualGameText(nIndex);
    cActualGame := cActualGame + EOL_STR;

    { Close BPGN file }
    CloseFile(BPGNFile);

    { Append game }
    AssignFile(OutFile, cFile);
    Append(OutFile);
    Write(OutFile, cActualGame);
    CloseFile(OutFile);

    { Get file size }
    AssignFile(ByteFile, cFile);
    Reset(ByteFile);
    nFileSize := FileSize(ByteFile);
    CloseFile(ByteFile);

    { Assign game list stuff }
    alist.add(p);
    p:=alist.last;
    p1:=alist.Items[nindex];

    p^:=p1^;

    p^.aPosition:= nFileSize - Length(cActualGame);

    p:=alist.Items[nFilterCount];
    p^.aFilterN:=nHeaderCount;
    Inc(nHeaderCount);
    inc(nFilterCount);
    { Open file }
    AssignFile(BPGNFile, cFile);
    Reset(BPGNFile);
    {db stuff}
    if bisdb then
    begin
     pbpgnrec(alist.Items[nHeadercount-1])^.aPositionw:=pdblist[1].Count;
     pbpgnrec(alist.Items[nHeadercount-1])^.aPositionb:=pdblist[2].Count;
     updategametoposdb(nHeadercount-1);
     bdbchange:=true;
    end;
END;

FUNCTION TBPGN.CanAddNewGame: Boolean;
BEGIN
    Result := true;// (nHeaderCount < MAX_GAME);
END;

FUNCTION TBPGN.PackFile: Boolean;
VAR
    bF: Boolean;
    nP: LongInt;
    nI, nC: Integer;
    cTmpFn:  String;
    OutFile: TextFile;
    nGameChunk:longint;
    nc1,barstep,nprogcount:longint;
BEGIN
    { Did file changed? }
    IF (NOT bFileChange) THEN
    BEGIN
        Result := FALSE;
        EXIT;
    END;

    { Check to see if there is any game marked for deletion }
    bF := FALSE;
    FOR nI := 0 TO (alist.Count-1)  DO
    BEGIN
        IF (getTag(ni) = TAG_DELETE) THEN
        BEGIN
            bF := TRUE;
            BREAK;
        END;
    END;
    IF (NOT bf) THEN
    BEGIN
        Result := FALSE;
        EXIT;
    END;

    { Check to see if all games are marked for deletion }
    bF := TRUE;
    FOR nI := 0 TO (alist.Count-1) DO
    BEGIN
        IF (gettag(ni) <> TAG_DELETE) THEN
        BEGIN
            bF := FALSE;
            BREAK;
        END;
    END;
    IF (bF) THEN
    BEGIN
        { Close BPGN file }
        cTmpFn := cFile;
        CloseBPGN(TRUE);

        { Delete file }
        DeleteFile(cTmpFn);

        if bisdb then
        begin
         DeleteFile(ChangeFileExt(cTmpFn,DB_HEADER_EXT));
         DeleteFile(ChangeFileExt(cTmpFn,DB_WHITE_EXT));
         DeleteFile(ChangeFileExt(cTmpFn,DB_Black_EXT));
         bdbchange:=false;
        end;

        Result := TRUE;
        EXIT;
    END;

    { Get temporary filename }
    cTmpFn := ChangeFileExt(cFile, '.' + TMP_EXT);

    { Create temporary file }
    AssignFile(OutFile, cTmpFn);
    Rewrite(OutFile);

    { Scan for undeleted game }
    nGameChunk:=Max_STRING div 1000;
    nP    := 0;
    nC    := 0;
    cLine := '';

     barstep:=1000;
     nc1:=0;
     nProgCount:=nHeaderCount;
     if bisdb then inc(nProgCount,nProgCount);

    FOR nI := 0 TO nHeaderCount-1 DO
    BEGIN
        if nc1=barstep then
        begin
         nc1:=0; PostMessage(fMainHandle,WM_ProgressUpdate,ni,nProgCount);
        end
        else inc(nc1,1);

        IF (gettag(ni) <> TAG_DELETE) THEN
        BEGIN
            IF (nI = nHeaderCount-1) THEN
                GetActualGame(getposition(nI), nFileSize)
            ELSE
                GetActualGame(getposition(nI), getposition(nI + 1) - 1);

            cLine := cLine + cActualGame;

            setPosition(ni,nP);
            nP := nP + Length(cActualGame);

            Inc(nC);

            IF (nC >= nGameChunk) THEN
            BEGIN
                Write(OutFile, cLine);
                nC    := 0;
                cLine := '';
            END;
        END;
    END;

    IF (Length(cLine) > 0) THEN
        Write(OutFile, cLine);

    { Close file }
    CloseFile(OutFile);

    { Close BPGN file }
    CloseFile(BPGNFile);

    { Delete current BPGN file }
    DeleteFile(cFile);

    { Rename temporary file into current BPGN file }
    RenameFile(cTmpFn, cFile);

    { Open file }
    AssignFile(bytefile, cfile);
    Reset(bytefile);
    nfilesize:=Filesize(bytefile);
    close(bytefile);

    AssignFile(bpgnfile,cfile);
    Reset(bpgnfile);

    Postmessage(fMainHandle,WM_ProgressUpdate,nHeaderCount,nProgCount);
    ni:=0;nc:=0;
    while ni<alist.Count do
    begin
    if gettag(ni)=TAG_DELETE then
    begin
      if bisdb then
      begin
       RemoveGameFromDB(ni);
       ShiftGamesDB(ni);
       bdbchange:=true;
      end;
      alist.delete(ni);
    end
    else inc(ni,1);
    if nc1=barstep then
    begin
         nc1:=0; PostMessage(fMainHandle,WM_ProgressUpdate,ni+nHeaderCount,nHeaderCount+alist.Count);
    end
    else inc(nc1,1);
    end;
    PostMessage(fMainHandle,WM_ProgressUpdate,nHeaderCount,nHeaderCount);

    nHeaderCount:=alist.Count;
    Result := FALSE;
END;


FUNCTION TBPGN.GameIsValid(nIndex: Integer): Boolean;
var p:pbpgnrec;
BEGIN
    IF (NOT bOpen) THEN
    BEGIN
        Result := FALSE;
        EXIT;
    END;
    IF ((nIndex < 0) OR (nIndex > alist.Count-1)) THEN
        Result := FALSE
    else result:=true;
END;

PROCEDURE TBPGN.SetGameChange(bC: Boolean);
BEGIN
    bGameChange := bC;
END;

PROCEDURE TBPGN.SetFileChange(bC: Boolean);
BEGIN
    bFileChange := bC;
END;

PROCEDURE TBPGN.RebuildHeaders(VAR ListBox1: TListBox);
VAR
    nL,nc, nI: longint;
    barstep:integer;
BEGIN
    nL := ListBox1.ItemIndex;
    listbox1.invalidate;
    ListBox1.ItemIndex := nL;
END;

FUNCTION TBPGN.GetErrorMessage: String;
BEGIN
    Result := cResult;
END;

procedure Tbpgn.SaveHeaders(fname:string);
var i,hc,k,nbeg,nend,begofgame,endofgame:longint;
f:file;
headlist:pheadlist;
line,gametext:string;
res:integer;
bpgndate:TDateTime;
begin
 //allocating the memory for buffer
 try
  New(headlist);
 except
  Werror('Not enough memory');
  exit;
 end;

 Assign(f,fname);
 try
  Rewrite(f,1);
 except
  CloseFile(f);
  Werror('Can not write to file ' + fname);
  dispose(headlist);
  exit;
 end;

 try
 //format of file with headers:
 // number of records, bug or zh
 // sync control data;
 // then records of type theadersave
  BlockWrite(f,nHeaderCount,sizeof(nHeaderCount),res);
  blockWrite(f,bisbugdb,sizeof(bIsBugDB),res);
  //add some data for  sync control between bpgn and headers;
  blockwrite(f,nFileSize,sizeof(nfilesize),res);
  bpgndate:=FileDateToDateTime(FileAge(getbpgnfilename));
  blockwrite(f,bpgndate,sizeof(bpgndate));

  hc:=0; i:=0;nbeg:=0;nend:=0;line:='';begofgame:=0;gametext:='';
  seek(bpgnfile,0);

  While i<nheadercount do
  begin
   begofgame:=getposition(i)+1; //first char in the game in 1-base array
   if(i<>nHeadercount-1) then endofgame:=getposition(i+1)+1 //first char not in the game in 1-base array
   else endofgame:=nFileSize+1;

   if endofgame-1>nend then
   begin
    for k:=0 to MAX_STRING-1 do cfilestring[k]:=#0;
    Read(bpgnfile,cfilestring);
    line:=line+cfilestring;
    nend:=nend+sizeof(cfilestring); //number of chars read=last char read in 1-based array
   end
   else begin
    gametext:=copy(line,begofgame-nbeg,endofgame-begofgame);
    delete(line,1,endofgame-nbeg-1);
    inc(nbeg,endofgame-nbeg-1); //nbeg--number of chars deleted;
    gametoheader(gametext,i,headlist[hc]);
    inc(i,1);
    inc(hc,1);
   end;
  //write headerbuf to headerfile
   if (hc>SAVE_HEADER_BUF) then
   begin
    blockwrite(f,Headlist^,sizeof(theadlist));
    hc:=0;
   end;
  end;
 //flush headbug at the end;
  blockwrite(f,headlist^,hc*sizeof(theadersave));
 finally
  CloseFile(f);
 end;
 dispose(headlist);
end;

procedure tbpgn.gametoheader(var gametext:string;n:longint;var head:theadersave);
var s:string;
ps:integer;
begin
  head.GameStart:=getposition(n);
  if n+1<nHeaderCount then head.GameEnd:=getposition(n+1)-1
  else head.GameEnd:=nFileSize-1;
  head.aTag:=getTag(n);

  s:=getEvent(n);
  ps:=pos(s,gametext);  if ps=0 then
  begin
   head.EventSh:=0;
   head.EventLn:=0;
  end
  else begin
   head.EventSh:=ps;
   head.EventLn:=length(s);
  end;

  s:=getSite(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.SiteSh:=0;
   head.SiteLn:=0;
  end
  else begin
   head.SiteSh:=ps;
   head.SiteLn:=length(s);
  end;

  s:=getdate(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.DateSh:=0;
   head.DateLn:=0;
  end
  else begin
   head.DateSh:=ps;
   head.DateLn:=length(s);
  end;

  s:=getWhiteA(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.WhiteASh:=0;
   head.WhiteALn:=0;
  end
  else begin
   head.WhiteASh:=ps;
   head.WhiteALn:=length(s);
  end;

  s:=getWhiteB(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.WhiteBSh:=0;
   head.WhiteBLn:=0;
  end
  else begin
   head.WhiteBSh:=ps;
   head.WhitebLn:=length(s);
  end;

  s:=getBlackB(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.BlackBSh:=0;
   head.BlackBLn:=0;
  end
  else begin
   head.BlackBSh:=ps;
   head.BlackBLn:=length(s);
  end;

  s:=getBlackA(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.BlackASh:=0;
   head.BlackALn:=0;
  end
  else begin
   head.BlackASh:=ps;
   head.BlackALn:=length(s);
  end;

  head.RWA:=getRWA(n);
  head.RBA:=getRBA(n);
  head.RWB:=getRWB(n);
  head.RBB:=getRBB(n);

  s:=gettimec(n);
  head.time:=TOOLBOX_gettimeval(s);
  head.inc:=TOOLBOX_gettimeincval(s);

  s:=getresult(n);
  head.Result:=resultStrToByte(s);

  s:=getOpenA(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.OpenASh:=0;
   head.OpenALn:=0;
  end
  else begin
   head.OpenASh:=ps;
   head.OpenALn:=length(s);
  end;

  s:=getOpenB(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.OpenBSh:=0;
   head.OpenBLn:=0;
  end
  else begin
   head.OpenBSh:=ps;
   head.OpenBLn:=length(s);
  end;

  s:=getSetup(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.SetupSh:=0;
   head.SetupLn:=0;
  end
  else begin
   head.SetupSh:=ps;
   head.SetupLn:=length(s);
  end;

  s:=getFen(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.FenSh:=0;
   head.FenLn:=0;
  end
  else begin
   head.FenSh:=ps;
   head.FenLn:=length(s);
  end;

  s:=getaReason(n);
  ps:=pos(s,gametext);
  if ps=0 then
  begin
   head.ReasonSh:=0;
   head.ReasonLn:=0;
  end
  else begin
   head.ReasonSh:=ps;
   head.ReasonLn:=length(s);
  end;
  head.positionw:=pbpgnrec(alist.Items[n]).aPositionW;
  head.positionb:=pbpgnrec(alist.Items[n]).aPositionB;
end;

function tbpgn.LoadDBheaders(fname:string):boolean;
var i,k,nI,hc,rc,hr,nbeg,nend:longint;
f:file;
headlist:pheadlist;
bufsize,szheadlist:longint;
gametext,line:string;
p:pbpgnrec;
nfsz:longint;
bpgnDate,bpgn1Date:TdateTime;
barstep,nc1:longint;
//hc -> number of games in DB (saved in the bh file)
//hr -> number of headers already read from bh
// bufsize -> size of the buffer in records
// szHeadlist -> size of the buffer in bytes
// rc -> number of bytes read from file
begin
 //allocate memory
 try
  new(headlist);
 except
  cResult:= 'Not enough memory';
  SendMessage(fMainHandle,WM_BPGNError,0,0);
  Result:=False;
  exit;
 end;

 //open header file
 AssignFile(f,fname);
 try
  reset(f,1);
 except
  cResult:='Can not open file '+fname;
  SendMessage(fMainHandle,WM_BPGNError,0,0);
  dispose(headlist);
  Result:=False;
  exit;
 end;



 try
  seek(f,0);
  blockread(f,hc,sizeof(hc),rc);
  blockread(f,bisbugdb,sizeof(bisbugdb),rc);
  blockread(f,nfsz,sizeof(nfsz),rc);
  if nfsz<>nFilesize then
  begin
   cResult:='Database file ' + fname+EOL_STR+
           'and BPGN file '+ getbpgnfilename+EOL_STR+
           'do NOT match. '+
           'Open bpgn file and regenerate the database';
   SendMessage(fMainHandle,WM_BPGNError,0,0);
   dispose(headlist);
   alist.clear;
   closeFile(f);
   Result:=false; exit;
  end;
  blockread(f,bpgndate,sizeof(bpgndate),rc);
  bpgn1Date:=FileDateToDateTime(FileAge(getbpgnfilename));
  if not DBStampOk(bpgnDate,bpgn1date) then
  begin
   cResult:='Database file ' + fname+EOL_STR+
           'and BPGN file '+ getbpgnfilename+EOL_STR+
           'do NOT match. '+EOL_STR+
           'BPGN file timestamp:' +DateTimeToStr(bpgn1date)+EOL_STR+
           'Database timestamp:'+DateTimeTostr(bpgndate)+EOL_STR+
           'Open bpgn file and regenerate the database';
   SendMessage(fMainHandle,WM_BPGNError,0,0);
   dispose(headlist);
   alist.clear;
   closeFile(f);
   Result:=false; exit;
  end;

  szHeadlist:=sizeof(theadlist);
  bufsize:=SAVE_HEADER_BUF+1;

  alist.clear;
  nbeg:=0;hr:=0;
  nend:=-1; i:=1; line:='';gametext:='';
  barstep:=3000;
  nc1:=0;
  while(hc>=i*bufsize) do
  begin
   //read a chunk of header file;
   blockread(f,headlist^,szHeadlist,rc);
   if rc<>szheadlist then // corrupt file
   begin
    cResult:='Corrupted header file ' + fname;
    SendMessage(fMainHandle,WM_BPGNError,0,0);
    dispose(headlist);
    alist.clear;
    closeFile(f);
    Result:=false; exit;
   end;

   //populate alist from the chunk
   for k:=0 to bufsize-1 do
   begin
    if alist.add(p)=-1 then //not enough memory
    begin
     cResult:='Not enough memory';
     SendMessage(fMainHandle,WM_BPGNError,0,0);
     dispose(headlist);
     alist.clear;
     closefile(f);
     Result:=false;
     exit;
    end;

    if headlist[k].GameEnd>nend then  //need a new chunk from bpgn file
    begin
     for ni:=0 to MAX_STRING-1 do cfilestring[nI]:=#0;
     Read(bpgnfile,cfilestring);
     line:=line+cfilestring;
     inc(nend,sizeof(cfilestring));  //nEnd is the 0-based index of the last char read
    end;

    gametext:=copy(line,headlist[k].GameStart-nbeg+1,headlist[k].GameEnd-headlist[k].GameStart+1);
    delete(line,1,headlist[k].GameEnd-nbeg+1);
    nbeg:=headlist[k].GameEnd+1;  //nbeg is the 0-based index of the first char not in the last game
    DBtoHeader(gametext,@headlist[k],alist.last);
    inc(hr,1);
    if nc1=barstep then begin PostMessage(fMainHandle,WM_ProgressUpdate,alist.Count,hc); nc1:=0;end
    else inc(nc1,1);
    if bAbortLong then exit;
   end;
   inc(i,1);
  end;

  //read the last chunk of headers
  blockread(f,headlist^,(hc-hr)*sizeof(theadersave),rc);
  if ((hc-hr)*sizeof(theadersave)<>rc) then
  begin
   cResult:='Corrupted header file ' + fname;
   SendMessage(fMainHandle,WM_BPGNError,0,0);
   dispose(headlist);
   closefile(f);
   alist.clear;
   Result:=false; exit;
  end;

  //populate alist from the chunk
  for k:=0 to hc-hr-1 do
  begin
   if alist.add(p)=-1 then //not enough memory
   begin
    cResult:='Not enough memory';
    SendMessage(fMainHandle,WM_BPGNError,0,0);
    dispose(headlist);
    closefile(f);
    alist.clear;
    Result:=false;
    exit;
   end;

   if headlist[k].GameEnd>nend then  //need a new chunk from bpgn file
   begin
    for ni:=0 to MAX_STRING-1 do cfilestring[nI]:=#0;
    Read(bpgnfile,cfilestring);
    line:=line+cfilestring;
    inc(nend,sizeof(cfilestring));    //nend is a position of first unread character, number of characters read
   end;

   gametext:=copy(line,headlist[k].GameStart-nbeg+1,headlist[k].GameEnd-headlist[k].GameStart+1);
   delete(line,1,headlist[k].GameEnd-nbeg+1);
   nbeg:=headlist[k].GameEnd+1;   //position of first unprocessed character
   DBtoHeader(gametext,@headlist[k],alist.last);
  end;

 finally
  CloseFile(f);
 end;
 dispose(headlist);
 nHeaderCount:=hc;
// nFilterCount:=hc;
// bIsDb:=true;
 Result:=true;
end;

procedure tbpgn.DBtoHeader(var s:string;p:pheadersave;p1:pbpgnrec);
begin
 p1.aTag:=p.aTag;

 if p.EventSh=0 then p1.aEvent:=DEFAULT_EVENT
 else p1.aEvent:=copy(s,p.EventSh,p.EventLn);

 if p.SiteSh=0 then p1.aSite:=DEFAULT_SITE
 else p1.aSite:=copy(s,p.SiteSh,p.SiteLn);

 if p.DateSh=0 then p1.aDate:=DEFAULT_DATE
 else p1.aDate:=copy(s,p.DateSh,p.DateLn);

 p1.aPosition:=p.GameStart;

 if p.WhiteaSh=0 then p1.aWhitea:=DEFAULT_Whitea
 else p1.aWhitea:=copy(s,p.WhiteaSh,p.WhiteaLn);

 if p.WhitebSh=0 then p1.aWhiteb:=DEFAULT_Whiteb
 else p1.aWhiteb:=copy(s,p.WhitebSh,p.WhitebLn);

 if p.BlackASh=0 then p1.aBlackA:=DEFAULT_BlackA
 else p1.aBlacka:=copy(s,p.BlackaSh,p.BlackaLn);

 if p.BlackBSh=0 then p1.aBlackB:=DEFAULT_BlackB
 else p1.aBlackb:=copy(s,p.BlackbSh,p.BlackbLn);

 p1.aRWA:=p.RWA;
 p1.aRWB:=p.RWB;
 p1.aRBA:=p.RBA;
 p1.aRBB:=p.RBB;

 p1.aTimeC:=StrTimeC(p.time,p.inc);
 p1.aResult:=resultbytetostr(p.Result);

 if p.OpenASh=0 then p1.aOpenA:=''
 else p1.aopena:=copy(s,p.openaSh,p.openaLn);

 if p.OpenBSh=0 then p1.aOpenB:=''
 else p1.aopenb:=copy(s,p.openbSh,p.openbLn);

 if p.SetupSh=0 then p1.aSetup:=''
 else p1.aSetup:=copy(s,p.SetupSh,p.SetupLn);

 if p.FenSh=0 then p1.aFen:=''
 else p1.aFen:=copy(s,p.FenSh,p.FenLn);

 if p.ReasonSh=0 then p1.aReason:=DEFAULT_Reason
 else p1.aReason:=copy(s,p.ReasonSh,p.ReasonLn);
 p1.aPositionW:=p.positionw;
 p1.aPositionB:=p.positionb;
end;

procedure tbpgn.BPGNtoPosRec(n:longint;cboard:char; var rec:tposDBrec);
var
i,j,k:integer;
begin
  rec.mIndex:=n;
  rec.timedif:=gettimedif(n,cboard);
  k:=boardnum(cboard);
  boardtoint(bpgnarray[n,k].aboard, rec.position);
//  for i:=1 to 2 do
//  for j:=1 to 5 do
//  rec.holding [i,j]:=bpgnarray[n,k].anPiece[i,j];
 // rec.mfrom:=xytoint(bpgnarray[n,k].mPrev.nFromX,bpgnarray[n,k].mPrev.nFromY);
 // rec.mto:=xytoint(bpgnarray[n,k].mPrev.ntoX,bpgnarray[n,k].mPrev.ntoY);

 // rec.cMove
end;



function tbpgn.gettimedif(nIndex:longint;cboard:char):integer;
begin
 case cboard of
 'A': Result:=bpgnarray[nIndex,1].nWTime-bpgnarray[nIndex,2].nWTime;
 'a': Result:=bpgnarray[nIndex,1].nbTime-bpgnarray[nIndex,2].nbTime;
 'B': Result:=bpgnarray[nIndex,2].nWTime-bpgnarray[nIndex,1].nWTime;
 'b': Result:=bpgnarray[nIndex,2].nbTime-bpgnarray[nIndex,1].nbTime;
 end;
end;

function tbpgn.UpdateGameToPosDB(n:longint):integer;
begin
 removegamefromdb(n);
 pdblist[1].defaultInsertN:=pbpgnrec(alist.Items[n]).aPositionW;
 pdblist[2].defaultInsertN:=pbpgnrec(alist.Items[n]).aPositionB;
 Result:=addgametoPosDb(n);
 pdblist[1].defaultInsertN:=-1;
 pdblist[2].defaultInsertN:=-1;
end;

function tbpgn.AddGameToPosDb(n:longint):integer;
var i,k:integer;
res:integer;
rec: tPosDBrec;
s:string;
begin
 if (not loadgame(n)) then
 begin
  Result:=0;
  exit;
 end;

 res:=addchildrenPos(savedepth*2,savedepth*2,1,1,1,true);

   if (res<0) then
   begin
    Result:=-1; exit;  // not enough memory
   end;
 Result:=1;
end;

function tbpgn.addChildrenPos(aleft,bleft:integer;parentN:integer;lastA,lastb:integer;ismain:boolean):integer;
var
rec:TPosdbrec;
i:integer;
nnext:longint;
cboard:char;
res,bd:integer;
worb:integer;
label skiprec;
begin
 if not bisbugdb then bleft:=0;
 if ((aleft<1) and (bleft<1)) then begin Result:=1; exit; end;
 i:=1; nnext:=bpgnnext[parentn,1];

 if nnext = 0 then
 begin

  if (aleft>0) THEN    //add final position for board 1;
  BEGIN
   bd:=1;
   if bpgnarray[parentn,1].bWhitesMove then
   begin
    worb:=1;
    cboard:='A';
   end
   else begin
    worb:=2;
    cboard:='a';
   end;
   bpgntoPosRec(lasta,cboard, rec);
   rec.GameN:=nGameHeader;
   rec.cBoard:=bd;
   rec.mfrom:=100;
   rec.mto:=100;
   rec.cMove:='End';
   rec.bIsMain:=ismain;
   if pdblist[worb].add(@rec)<0 then begin Result:=-1; exit;end;
  END;

  if (bleft>0) THEN    //add final position for board 2;
  BEGIN
   bd:=2;
   if bpgnarray[parentn,2].bWhitesMove then
   begin
    worb:=1;
    cboard:='B';
   end
   else begin
    worb:=2;
    cboard:='b';
   end;
   bpgntoPosRec(lastb,cboard, rec);
   rec.GameN:=nGameHeader;
   rec.cBoard:=bd;
   rec.mfrom:=100;
   rec.mto:=100;
   rec.cMove:='End';
   rec.bIsMain:=ismain;
   if pdblist[worb].add(@rec)<0 then begin Result:=-1; exit;end;
  END;
  Result:=1;
  exit;
 end;

 while nnext>0 do
 begin
  cboard:=getbpgnboard(nNext);
  bd:=boardnum(cboard);
  if (((aleft<1) and (bd=1)) or ((bleft<1) and (bd=2))) then
  begin goto skiprec; end;

  if bd=1 then bpgntoPosRec(lasta,cboard, rec)
  else bpgntoPosrec(lastb,cboard,rec);
  rec.GameN:=nGameHeader;
  worb:=getworb(cboard);
  rec.cBoard:=bd;
  rec.mfrom:=xytoint(bpgnarray[nNext,bd].mPrev.nFromX,bpgnarray[nNext,bd].mPrev.nFromY);
  rec.mTo:=xytoint(bpgnarray[nNext,bd].mPrev.ntoX,bpgnarray[nNext,bd].mPrev.nToY);
  rec.cMove:=getbpgnmove(nNext);
  if ((ismain) and (BPGNmain[parentn]=nNext)) then rec.bIsMain:=true else rec.bIsMain:=false;
  if pdblist[worb].add(@rec)<0 then begin Result:=-1;exit;end else ;

skiprec:
  ismain:=(ismain and (bpgnmain[parentn]=nnext));
  if bd=1 then res:=addchildrenPos(aleft-1,bleft,nnext,nnext,lastb,ismain)
  else res:=addchildrenPos(aleft,bleft-1,nnext,lasta,nnext,ismain);
  if res<0 then begin Result:=-1;exit;end;
  inc(i,1);
  nnext:=bpgnnext[parentn,i];
 end;
 Result:=1;
end;

function tbpgn.SavePosDb(fname:string):integer;
var i:longint;
res: integer;
cnt,stepcnt,barstep:integer;
s,namew,nameb:string;
nBackupGameHeader,nBackupCurrent:longint;
ferror:tviewactualform;
sdamagedGames:string;
begin
 sDamagedGames:='';
 Cresult:='';
 namew:=changeFileExt(fname,DB_WHITE_EXT);
 nameb:=changeFileExt(fname,DB_BLACK_EXT);
 nbackupGameHeader:=nGameHeader;
 nBackupCurrent:=nCurrent;
 pdblist[1].clear;
 pdblist[2].clear;
// pdblist[1].Capacity:=savedepth*alist.Count;
// pdblist[2].Capacity:=savedepth*alist.Count;

 bisDb:=false;
 cnt:=0;
 stepcnt:=1; barstep:=100;
 for i:=0 to alist.Count-1 do
 begin
  pbpgnrec(alist.Items[i]).aPositionW:=pdblist[1].Count;
  pbpgnrec(alist.Items[i]).aPositionB:=pdblist[2].Count;
  res:=addGameToPosDB(i);
  if (res<0) then
  begin
   loadgame(nbackupGameHeader);
   nCurrent:=nbackupCurrent;
   cResult:='Creation of database failed' +EOL_STR+'Not enough memory';
   SendMessage(fMainHandle,WM_BPGNError,0,0);
   Result:=-1;
   exit;
  end;
  if res=0 then begin inc(cnt,1); str(i+1,s); sDamagedGames:=sDamagedGames+s+EOL_STR; end;
  if bAbortLong then exit;
  if stepcnt=barstep then
  begin stepcnt:=0; PostMessage(fMainHandle,WM_ProgressUpdate,i,alist.Count) ;end;
  inc(stepcnt,1);

 end;

 If cnt>0 then
 begin
   str(cnt,s);
   cResult:= s+' games were damaged' + EOL_STR+
                         'and could not be processed'+EOL_STR+
                         'The Following games have error:'+EOL_STR+
                         EOL_STR+sDamagedGames;
   SendMessage(fMainHandle,WM_BPGNinfo,0,0);
   Result:=1;
 end;

 bisDb:=true;
 loadgame(nbackupGameHeader);
 nCurrent:=nbackupCurrent;
end;

function tbpgn.checkDBtype:integer;
var i:longint;
begin
 if alist.count<1 then begin result:=0; exit; end;

 if pos(Crazyhouse_ID,lowercase(pbpgnrec(alist.First)^.aEvent))=0 then bisBuGDB:=true
 else bIsBugDB:=false;
 for i:=1 to nHeaderCount-1 do
 if (bIsBugDB and (pos(Crazyhouse_ID,lowercase(pbpgnrec(alist.items[i])^.aEvent))<>0))
  then begin Result:=-1;exit;end
 else if ((not BisBugDB) and (pos(Crazyhouse_ID,lowercase(pbpgnrec(alist.items[i])^.aEvent))=0))
 then begin Result:=-1; exit;end;

 Result:=0;

end;

procedure tbpgn.setposstats(fform:tfform);
var
n1,side:integer;
res,i,j,k,fc:longint;
worb:integer;
oside,nfromTo:integer;
p:pposdbrec;
bisrated,bb:boolean;
s:string;
begin
 with fform.StringGrid1 do
 begin
  Cells[0,0]:='<==';
  Cells[2,0]:='Win %';
  Cells[3,0]:='Av.Rating';

  if fform.bfullstat then
  begin
   Cells[1,0]:='Games (%)';
   Cells[4,0]:='Perf(Games)';
  end
  else begin
   Cells[1,0]:='Games';
   Cells[4,0]:='Perf';
  end;
 end;

 //assuming ncurrentworb and pdblist[ncurrentworb].searchrec are set before;
 for i:=1 to Max_next do
 cleardbnext(nextdblist[i]);              //clear the list of next move

 fc:=0;
 for i:=0 to nHeaderCount-1 do
 if filtermatch(i) then
 begin
  j:=pdblist[nCurrentworb].lastfound;
  //the index of the first appearance of the searched
  //position in the game #i in position db
  bisrated:=israted(i);
  repeat
    p:=pdblist[ncurrentworb].Items[j];
    nfromto:=fromtotoint(p^.mfrom,p^.mto);

    k:=1;
    if fc=0 then inc(fc,1)
    else
     while nextDbList[k].nFromTo>0 do
     if ((nfromto=nextDBList[k].nFromTo) and (p^.cMove=nextdblist[k].cMove))
     then break else begin inc(k,1);inc(fc,1);end;

    if k<=MAX_NEXT then //can add position;
    begin
     nextdblist[k].nFromTo:=nfromto;
     nextdblist[k].cMove:=p^.cMove;
 //    fform.StringGrid1.Cells[0,k]:=p^.cMove;
     inc(nextdblist[k].nGame,1); //update total number of times the move happened in current position;

     if p^.bIsMain then     //main line, hence update other counters
     begin
      side:=boardcolortoside(p^.cBoard,nCurrentWorb);      //number of wins,losses,draws
      res:=getsideresult(side,i);
      case res of
      1: begin inc(nextdblist[k].nWin,1); end;
      2: begin inc(nextdblist[k].nLoss,1); end;
      3: begin inc(nextdblist[k].nDraw,1); end;
      end;

      if bisrated then      //rated game, update rated wins,losses,draws and pard and opp ratings;
      begin
       if res>0 then                                 //ratings
       begin
        inc(nextdblist[k].nSelfRat,getsiderating(side,i));
        oside:=getopponentside(side);
        inc(nextdblist[k].nOppRat,Getsiderating(oside,i));
        if bisbugdb then begin
         inc(nextdblist[k].nOppRat,getsiderating(getpartnerside(oside),i));
         inc(nextdblist[k].nPardRat,getsiderating(getpartnerside(side),i));
        end;
       end;
       case res of       //rated wins,losses,draws
       1: begin inc(nextdblist[k].nrWin,1); end;
       2: begin inc(nextdblist[k].nrLoss,1); end;
       3: begin inc(nextdblist[k].nrDraw,1); end;
       end;
      end; {if bisrated}
     end; { if p^.bismain}
    end; {k<=Max_next}

    inc(j,1);
    if not pdblist[ncurrentworb].findPosInGame(i,j,pdblist[ncurrentworb].Count-1) then break
    else begin
     j:=pdblist[ncurrentworb].lastfound;
     s:=lowercase(trim(rfilter.Player1));
     if s<>'' then
     with pdblist[ncurrentworb] do begin
     if s=lowercase(trim(getwhitea(i))) then n1:=1;
     if s=lowercase(trim(getwhiteb(i))) then n1:=2;
     if s=lowercase(trim(getBlacka(i))) then n1:=3;
     if s=lowercase(trim(getBlackB(i))) then n1:=4;
     bb:=false;
     repeat
      if pposdbrec(items[lastfound]).cBoard=(2-(n1 mod 2)) then break;
      if not findposingame(i,LastFound+1,count-1) then
      begin
       bb:=true;
       break;
      end;
      until false;
     end;{with}
    end; {else begin}
    if bb then break;
  until pdblist[nCurrentworb].getGameN(j)>i;

 end; {for i:= }

 if fc>Max_next then fc:=max_next;
 for i:=1 to fc do
 begin
   res:=nextdblist[i].nRwin+nextdblist[i].nrDraw+nextdblist[i].nrLoss;
   if res>0 then
    nextdblist[i].nPerf:=round(perfrating(nextdblist[i].nPardRat,nextdblist[i].nOppRat,(100*nextdblist[i].nRwin+50*nextdblist[i].nrDraw)/res,res))
   else
    nextdblist[i].nPerf:=-1;
 end;
 sortmovestats(fform,ini1.INI.nMoveOrder);
end;

procedure tbpgn.setnonposstats(fform:tfform);
var i,j,fc,rwin,rdraw:longint;
s,s1:string;
side,runknown:integer;
res:real;
pardrat,opprat,selfrat,rgameN:longint;
bzh:boolean;
begin
 with fform.StringGrid1 do
 begin
  Cells[0,0]:='';
  Cells[1,0]:='Games';
  cells[2,0]:='Win %';
  cells[3,0]:='Av.Rating';
  if fform.bfullstat then cells[4,0]:='Perf(Games)'
  else cells[4,0]:='Perf';
  fform.clearstats;
 end;

 fc:=0;
 ndraw:=0;nwin:=0;nloss:=0;
 pardrat:=0;opprat:=0;rgameN:=0; rwin:=0;rdraw:=0;runknown:=0;selfrat:=0;
 for i:=0 to nHeaderCount-1 do
 if filtermatch(i) then
 begin
  inc(fc,1);
  if getresult(i)=BPGN_UNKNOWN then inc(runknown,1);
  side:=getplayerside(rfilter.Player1,i);
  if ((side<1) or (not israted(i))) then continue;

  j:=getsideresult(side,i);
  case j of
  1: inc(rwin,1);
  3: inc(rdraw,1);
  0: continue;
  end;

  inc(rgamen,1);

  inc(selfrat,getsiderating(side,i));
  j:=getopponentside(side);
  inc(opprat,getsiderating(j,i));

  if (Pos(Crazyhouse_Id,getevent(i))<>0) then bzh:=true else bzh:=false;
  if not bzh then
  begin
   inc(pardrat,getsiderating(getpartnerside(side),i));
   inc(opprat,getsiderating(getpartnerside(j),i));
  end;
 end;

 with fform.StringGrid1 do
 begin
  str(fc,s);
  cells[1,1]:=s;  //# of games
  if fc>runknown then
  begin
   res:=100*(nwin+0.5*ndraw)/(fc-runknown); str(res:5:1,s);
   if fform.bfullstat then
   begin
    str(fc-runknown,s1);
    s:=s+'('+s1+')';
   end;
   cells[2,1]:=s; //win %
  end
  else begin
   cells[2,1]:='-';
  end;

  if rfilter.Player1='' then begin cells[3,1]:='-'; cells[4,1]:='-';end
  else begin
   cells[0,1]:=rfilter.Player1;
   if rgamen>0 then
   begin
    str(round(selfrat/rgamen),s);
    cells[3,1]:=s;
    res:=(100*rwin+50*rdraw)/rgameN;
    i:=round(perfrating(pardrat,opprat,res,rgameN));
    str(i,s);
    if fform.bfullstat then
    begin
     str(rgameN,s1);
     s:=s+'('+s1+')';
    end;
    cells[4,1]:=s;
   end
   else begin cells[3,1]:='-'; cells[4,1]:='-'; end;
  end;
 end;
end;

procedure tbpgn.setStats(fform:tfform);
begin
 rfilter:=fform.filter;
 if ((bisdb) and (rfilter.possearchmode<3)) then
 begin
  if bpgnarray[ncurrent,rfilter.possearchmode].bWhitesMove then nCurrentworb:=1
  else nCurrentworb:=2;
  boardtoint(bpgnarray[ncurrent,rfilter.possearchmode].aboard,rfilter.position);
  pdblist[ncurrentWorb].searchrec.position:=rfilter.position;
  fform.filter.position:=rfilter.position;
  setposStats(fform);
 end
 else setNonPosStats(fform);
end;

procedure tbpgn.sortmovestats(fform:tfform;col:integer);
var
i,j,nlast,tmprange:integer;
rangear:array [1..MaX_NEXT] of integer;
temprec: TDBPosNext;
s:string;
cSort:string;
begin
 csort:='@';


 if (ini.nMoveOrder>0) then
 begin
  s:=fform.StringGrid1.Cells[INI.nMoveOrder,0];
  if pos(csort,s)>0 then delete(s,1,length(csort));
  fform.StringGrid1.Cells[INI.nMoveOrder,0]:=s;
 end;
 ini1.INI.nMoveOrder:=col;
 if ((col>0) and (col<5)) then
 begin
 s:=fform.StringGrid1.Cells[col,0];
 insert(csort,s,1);
 fform.StringGrid1.Cells[col,0]:=s;



 //find the number of moves in the list;
 nlast:=-1;
 for i:=1 to MAX_NEXT do
  if nextdblist[i].nFromTo<1 then
  begin
   nlast:=i-1;
   break;
  end;
 if nlast=-1 then nlast:=Max_NEXT;

 //clear rangear
 for i:=1 to nlast do
 rangear[i]:=nlast;

 //find range of each move which is not optimal but Ok since nlast should be small
 for i:=1 to nlast do
 for j:=i+1 to nlast do
  case compareNextMove(i,j,col) of
  1:dec(rangear[i],1);
  0:dec(rangear[i],1);
  -1: dec(rangear[j],1);
  end;

 // sort by ranges
 for i:=1 to nlast do
 while rangear[i]<>i do
  begin
  temprec:=nextdblist[rangear[i]];
  nextdblist[rangear[i]]:=nextdblist[i];
  nextdblist[i]:=temprec;

  tmprange:=rangear[rangear[i]];
  rangear[rangear[i]]:=rangear[i];
  rangear[i]:=tmprange;
 end;
 end; {if 0<col<5}
 displayPosstats(fform);
end;

function tbpgn.CompareNextMove(i,j,col:integer):integer;
var ri,rj:integer;
begin
 case col of
 1: Result:=sign(nextdblist[i].nGame-nextdblist[j].nGame);
 2: Begin
    ri:=(nextdblist[i].nWin+nextdblist[i].nDraw+nextdblist[i].nLoss);
    rj:=(nextdblist[j].nWin+nextdblist[j].nDraw+nextdblist[j].nLoss);
    if ((ri=0) and (rj=0)) then Result:=0
    else if ((ri>0) and (rj=0)) then Result:=1
         else if ((ri=0) and (rj>0)) then Result:=-1
              else
              Result:=sign((nextdblist[i].nWin+0.5*nextdblist[i].nDraw)/ri-
                 (nextdblist[j].nWin+0.5*nextdblist[j].nDraw)/rj);
    end;
 3: Begin
    ri:=(nextdblist[i].nRwin+nextdblist[i].nrDraw+nextdblist[i].nrLoss);
    rj:=(nextdblist[j].nRwin+nextdblist[j].nrDraw+nextdblist[j].nrLoss);
    if ((ri=0) and (rj=0)) then Result:=0
    else if ((ri>0) and (rj=0)) then Result:=1
         else if ((ri=0) and (rj>0)) then Result:=-1
              else
        Result:=sign(nextdblist[i].nSelfRat/ri - nextdblist[j].nSelfRat/rj);
    end;
 4: Result:=sign(nextdblist[i].nPerf-nextdblist[j].nPerf);
 else Result:=0;
 end;
end;

procedure tbpgn.DisplayPosStats(fform:tfform);
var res:integer;
i,j,k:longint;
s,s1:string;
fc,ntotal,nwintotal,ndrawtotal,nlosstotal,nrwintotal,nrdrawtotal,nrlosstotal:longint;
opprattotal,pardrattotal,selfrattotal,nrestotal,nrrestotal:longint;
begin
 fc:=0;
 ntotal:=0;nwintotal:=0;ndrawtotal:=0;nlosstotal:=0;
 nrwintotal:=0;nrdrawtotal:=0;nrlosstotal:=0;
 opprattotal:=0;pardrattotal:=0;selfrattotal:=0;
 fform.StringGrid1.RowCount:=Max_Next+1;

 for i:=1 to Max_NEXt do
 begin

  if nextdblist[i].nFromTo<1 then //clear stringgrid below
  begin
   for k:=i to fform.StringGrid1.RowCount-1  do
   for j:=0 to 4 do
    fform.StringGrid1.Cells[j,k]:='';
//     fform.StringGrid1.RowCount:=i+1;
   break;
  end;
  inc(fc,nextdblist[i].nGame);
  inc(opprattotal,nextdblist[i].nOppRat);
  inc(pardrattotal,nextdblist[i].nPardRat);
  inc(selfrattotal,nextdblist[i].nSelfRat);

  inc(nwintotal,nextdblist[i].nWin);
  inc(ndrawtotal,nextdblist[i].nDraw);
  inc(nlosstotal,nextdblist[i].nLoss);

  inc(nrwintotal,nextdblist[i].nrWin);
  inc(nrdrawtotal,nextdblist[i].nrDraw);
  inc(nrlosstotal,nextdblist[i].nrLoss);

  inc(ntotal,1);
  with fform.StringGrid1 do
  begin
   cells[0,i]:=nextdblist[i].cMove;

   res:=nextdblist[i].nWin+nextdblist[i].nDraw+nextdblist[i].nLoss;
   if res>0 then
   begin
    nextdblist[i].Perc:=100*(nextdblist[i].nWin+0.5*nextdblist[i].nDraw)/res;
    str(nextdblist[i].Perc:5:1,s);
    if fform.bfullstat then
    begin
     str(res,s1);
     s:=s+'('+s1+')';
    end;
   end
   else s:='-';
   cells[2,i]:=s; //win %

   res:=nextdblist[i].nRwin+nextdblist[i].nrDraw+nextdblist[i].nrLoss;
   if res>0 then
   begin
    k:=round(nextdblist[i].nSelfRat/res);
    str(k,s);
    cells[3,i]:=s;
    str(nextdblist[i].nPerf,s);
    if fform.bfullstat then
    begin
     str(res,s1);
     s:=s+'('+s1+')';
    end;
    cells[4,i]:=s;
   end
   else begin
    cells[3,i]:='-';
    cells[4,i]:='-';
   end;                      //performance rating;
  end;

 end; {for i}

 fform.StringGrid1.RowCount:=ntotal+3;
 nrestotal:=nwintotal+nlosstotal+ndrawtotal;
 nrrestotal:=nrwintotal+nrlosstotal+nrdrawtotal;

 if nrestotal>0 then
  str((100*nwintotal+50*ndrawtotal)/nresTotal:5:1,s)
 else s:='-';
 fform.StringGrid1.Cells[2,ntotal+2]:=s;
 if nrrestotal>0 then
 begin
  str(round(selfrattotal/nrrestotal),s);
  fform.StringGrid1.Cells[3,ntotal+2]:=s;
  res:=round(perfrating(pardratTotal,oppratTotal,(100*nrwinTotal+50*nrdrawtotal)/nrrestotal,nrrestotal));
  str(res,s);
  fform.StringGrid1.Cells[4,ntotal+2]:=s;
 end
 else begin
  fform.StringGrid1.Cells[3,ntotal+1]:='-';
  fform.StringGrid1.Cells[4,ntotal+1]:='-';
 end;

 fform.StringGrid1.cells[0,ntotal+2]:='Total';
 str(fc,s);             // update totals
 fform.StringGrid1.Cells[1,ntotal+2]:=s;

 for i:=1 to ntotal do
 begin
   str(nextdblist[i].nGame,s);
   if fform.bfullstat then
   begin
    str(round(100*nextdblist[i].nGame/fc),s1);
    s:=s+'('+s1+'%)';
   end;
   fform.StringGrid1.Cells[1,i]:=s;  //# of games
 end;

end;

procedure tbpgn.updatedb;
var hfilename,namew,nameb:string;
stepinc:integer;
begin
   hfileName:=ChangeFileExt(getbpgnfilename,DB_HEADER_EXT);
   namew:=changeFileExt(hfilename,DB_WHITE_EXT);
   nameb:=changeFileExt(hfilename,DB_BLACK_EXT);
   SaveHeaders(hFileName);
   PostMessage(fMainHandle,WM_ProgressUpdate,1,3);
   pdblist[1].Save(namew);
   PostMessage(fMainHandle,WM_ProgressUpdate,2,3);
   pdblist[2].save(nameb);
   PostMessage(fMainHandle,WM_ProgressUpdate,3,3);
   bdbchange:=false;
end;

procedure tbpgn.goposition(var rec:tposdbrec);
var i:longint;
curpos:tposdbrec;
worb:integer;
begin
 for i:=1 to MAX_POSITION do
 begin

  if (bpgnarray[i,1].bWhitesMove) then worb:=1 else worb:=2;
  if worb=rec.cBoard then
  begin
   boardtoint(bpgnarray[i,1].aboard,curpos.position);
   if compposition(@rec,@curpos)=0 then
   begin
    nCurrent:=i;
    rfilter.possearchmode:=1;
    exit;
   end;
  end;

  if not bisbugdb then continue;
  if (bpgnarray[i,2].bWhitesMove) then worb:=1 else worb:=2;
  if worb=rec.cboard then
  begin
   boardtoint(bpgnarray[i,2].aboard,curpos.position);
   if compposition(@rec,@curpos)=0 then
   begin
    nCurrent:=i;
    rfilter.possearchmode:=2;
    exit;
   end;
  end;

 end;{for}
end;

procedure tbpgn.killdoubles;
var gamei,gamej,iwmin,iwmax,jwmin,jwmax,ibmin,ibmax,jbmin,jbmax:longint;
bfirst:boolean;
barstep,nc:longint;
s,s1:string;
begin
 nTmpCounter:=0;
 str(nHeaderCount,s1);


 barstep:=100;
 nc:=0;
 for gamei:=0 to nHeadercount-2 do
 begin
  if bAbortLong then exit;
  if nc=barstep then
  begin
   PostMessage(fMainHandle,WM_ProgressUpdate,gamei,nHeaderCount);nc:=0;
  end else inc(nc,1);

  if gettag(gamei)=Tag_Delete then continue;
  bfirst:=true;

  for gamej:=gamei+1 to nHeaderCount-1 do
  begin
   if gettag(gamej)=Tag_delete then continue;

   if isdouble(gamei,gamej) then
   begin
    settag(gamej,TAG_DELETE);
    if bfirst then begin setfindex(nTmpCounter,gamei); inc(nTmpCounter,1); bfirst:=false;end;
    setfindex(nTmpCounter,gamej);
    inc(nTmpCounter,1);
   end;
  end;
 end;

 nfiltercount:=nTmpCounter;
 bfilterset:=false;
end;

function tbpgn.IsDouble(i,j:longint):boolean;
var
k,limk,iwmin,iwmax,ibmin,ibmax,jwmin,jwmax,jbmin,jbmax:longint;
bok:boolean;
begin
 iwmin:=pbpgnrec(alist.Items[i]).aPositionW;
 ibmin:=pbpgnrec(alist.Items[i]).aPositionb;
 if i<(alist.Count-1) then
  iwmax:=pbpgnrec(alist.Items[i+1]).aPositionW-1
 else  iwmax:=pdblist[1].Count-1;
 if i<(alist.Count-1) then
  ibmax:=pbpgnrec(alist.Items[i+1]).aPositionb-1
 else  ibmax:=pdblist[2].Count-1;

 jwmin:=pbpgnrec(alist.Items[j]).aPositionW;
 jbmin:=pbpgnrec(alist.Items[j]).aPositionb;
 if j<(alist.Count-1) then
  jwmax:=pbpgnrec(alist.Items[j+1]).aPositionW-1
 else  jwmax:=pdblist[1].Count-1;
 if j<(alist.Count-1) then
  jbmax:=pbpgnrec(alist.Items[j+1]).aPositionb-1
 else  jbmax:=pdblist[2].Count-1;

 if (((iwmax-iwmin)<>(jwmax-jwmin)) or ((ibmax-ibmin)<>(jbmax-jbmin))) then
 begin
  result:=false;exit;
 end;
 if ((min(jwmin,iwmin)<0) and ((jwmin-iwmin)<>0)) then
 begin
  result:=false;exit;
 end;
 if ((min(jbmin,ibmin)<0) and ((jbmin-ibmin)<>0)) then
 begin
  result:=false;exit;
 end;

 if ((max(jwmin,jwmax)>pdblist[1].count) or (max(jbmin,jbmax)>pdblist[2].Count)) then
 Werror('j='+inttostr(j));

 limk:=min(1,ibmax-ibmin);
  with pdblist[1] do
  if compposition(pposdbrec(items[iwmax]),pposdbrec(items[jwmax]))<>0 then
  begin
    bok:=false;
    for k:=jwmax-1 downto jwmin do
    if pposdbrec(items[k]).cBoard<>pposdbrec(items[jwmax]).cBoard then
     if compposition(pposdbrec(items[k]),pposdbrec(items[iwmax]))=0 then
     begin bok:=true; break; end
     else begin result:=false;exit;end;

    if not bok then begin result:=false; exit;end;
  end;

  with pdblist[2] do
  if compposition(pposdbrec(items[ibmax]),pposdbrec(items[jbmax]))<>0 then
  begin
    bok:=false; {find the last position on another board}
    for k:=jbmax-1 downto jbmin do
    if pposdbrec(items[k]).cBoard<>pposdbrec(items[jbmax]).cBoard then
     if compposition(pposdbrec(items[k]),pposdbrec(items[ibmax]))=0 then
     begin bok:=true; break; end
     else begin result:=false;exit;end;

    if not bok then begin result:=false; exit;end;
  end;
// end; {positional check}

{ if ((getrwa(i)<>getrwa(j)) or (getrwb(i)<>getrwb(j))
 or (getrba(i)<>getrba(j)) or (getrbb(i)<>getrbb(j))) then
 begin
  Result:=false;exit;
 end;

 if ((getwhitea(i)<>getwhitea(j)) or (getwhiteb(i)<>getwhiteb(j))
 or (getblacka(i)<>getblacka(j)) or (getblackb(i)<>getblackb(j))
 or (gettimeC(i)<>gettimec(j)) or (getResult(i)<>getResult(j))) then
 begin
  Result:=false;exit;
 end;}
 Result:=true;
end;

function tbpgn.gamexstart(gamen:longint;ncolor:integer):longint;
begin
 if ncolor=1 then result:=pbpgnrec(alist.Items[gamen]).aPositionW
 else result:=pbpgnrec(alist.Items[gamen]).aPositionB;
end;

procedure tbpgn.removeGameFromDB(Gamen:longint);
begin
 pdblist[1].RemoveGame(Gamen,gamexstart(Gamen,1));
 pdblist[2].RemoveGame(Gamen,gamexstart(gamen,2));
end;

procedure tbpgn.shiftgamesDb(Gamen:longint);
var i,wshift,bshift:longint;
begin
 if gamen>alist.Count-2 then exit;

       wshift:=pbpgnrec(alist.Items[Gamen+1])^.aPositionW-
               pbpgnrec(alist.Items[Gamen])^.aPositionW;
       bshift:=pbpgnrec(alist.Items[Gamen+1])^.aPositionB-
               pbpgnrec(alist.Items[Gamen])^.aPositionB;
       for i:=alist.Count-1 downto gamen+1 do
       with pbpgnrec(alist.Items[i])^ do
       begin
        dec(aPositionW,wshift);
        dec(aPositionB,bshift);
       end;
       pdblist[1].shiftgames(pbpgnrec(alist.Items[GameN]).aPositionW,1);
       pdblist[2].shiftgames(pbpgnrec(alist.Items[GameN]).aPositionB,1);
end;

procedure tbpgn.appendGameDB(fname:string;nIndex,nPrevSize,nPrevCount,nAddSize:longint);
var hFile:file;
bpgnname,wname,bname:string;
wlist,blist:tposdb;
bok:boolean;
j,maxj,nPrevCountB,nPrevCountW:longint;
hsave:theaderSave;
begin
 try
  wlist:=tposdb.Create;
 except
  Werror('Not enough memory');
  exit;
 end;
 try
  blist:=tposdb.Create;
 except
  Werror('Not enough memory');
  wlist.Free;exit;
 end;
 AssignFile(hfile,fname);
 bpgnname:=ChangeFileExt(fname,'.'+bpgn_ext);
 wname:=ChangeFileExt(fname,DB_WHITE_EXT);
 bname:=ChangeFileExt(fname,db_black_ext);

 try
  if nPrevCount =0 then Rewrite(hfile,1)
  else Reset(hfile,1);
  bOk:=WriteDbSyncInfo(hfile,bpgnname,nPrevSize+nAddSize,bisbugdb,nPrevCount+1);
  if not bOk then begin
   wError('Writing to file failed:'+EOL_STR+fname);
   raise EInOutError.Create('');
  end;

  nPrevCountW:=GetPositionCount(wname);
  nPrevCountB:=getPositionCount(bname);

  gametoHeader(cActualGame,nIndex,hsave);
  with hsave do begin
     GameStart:=nPrevSize;
     GameEnd:=Gamestart+nAddSize-1;
     positionw:=nPrevCountW;
     positionb:=nPrevCountB;
  end;
    {white positions}
  if nIndex<(alist.Count-1) then maxj:=pbpgnrec(alist.Items[nIndex+1]).apositionw-1
  else maxj:=pdblist[1].Count-1;

  for j:=pbpgnrec(alist.Items[nIndex])^.apositionw to maxj do
  with wlist do
  begin
   add(pdblist[1].items[j]);
   pposdbrec(last)^.GameN:=0;
  end;
    {black positions}
  if nIndex<(alist.Count-1) then maxj:=pbpgnrec(alist.Items[nIndex+1]).apositionb-1
  else maxj:=pdblist[2].Count-1;
  for j:=pbpgnrec(alist.Items[nIndex])^.apositionb to maxj do
  with blist do
  begin
    add(pdblist[2].items[j]);
    pposdbrec(last)^.GameN:=0;
  end;

  seek(hfile,FileSize(hfile));
  blockwrite(hfile,hsave,sizeof(hsave));
  wlist.Append(wname,nPrevCount);
  blist.Append(bname,nPrevCount);
 finally
  wlist.clear;blist.clear;
  wlist.Free;blist.Free;
  closeFile(hfile);
 end;
end;

function tbpgn.iszh(nIndex:longint):boolean;
begin
 Result:=(Pos(Crazyhouse_ID,getevent(nIndex))<>0);
end;

procedure tbpgn.Createdatabase(Pfname:Pointer);
begin
 saveposdb(string(pfname^));
end;

procedure tbpgn.SaveGame_(p:Pointer);
begin
 Savegame(PLongInt(p)^);
end;

procedure tbpgn.PackFile_(p:pointer);
begin
 pboolean(p)^:=packfile;
end;

procedure tbpgn.updateDB_(p:pointer);
begin
 updateDB;
end;

procedure tbpgn.killdoubles_(p:pointer);
begin
 killdoubles;
end;

procedure tbpgn.loadheader_(p:pointer);
var res:boolean;
begin
 res:=loadheader;
 if Assigned(p) then pboolean(p)^:=res;
end;

procedure tbpgn.LoadDBHeader_(p:pointer);
begin
 bisdb:=loaddbheaders(string(p^));
end;

procedure tbpgn.LoadDBPos_(p:pointer);
var hname:string;
begin
 Hname:=string(p^);

 bisdb:=pdblist[1].Load(changeFileExt(hname,DB_WHITE_EXT));
 if bAbortLong then exit;
 if bisdb then PostMessage(fMainHandle,WM_ProgressUpdate,1,2)
 else begin cResult:=pdblist[1].cLastError; SendMessage(fMainHandle,WM_BPGNError,0,0);end;

 if bisdb then bisdb:=pdblist[2].Load(ChangeFileExt(hname,DB_BLACK_EXT));
 if bisdb then PostMessage(fMainHandle,WM_ProgressUpdate,2,2)
 else begin cResult:=pdblist[2].cLastError; SendMessage(fMainHandle,WM_BPGNError,0,0);end;
end;

procedure tbpgn.SaveFiltered_(p:pointer);
var farg:TstrBoolRec;
begin
 farg:=tstrBoolRec(p^);
 savefiltered(farg.s,farg.b);
end;

procedure tbpgn.filterTag(cTag:char);
var i,nfound:longint;
begin
 bfilterSet:=false;
 nFound:=0;
 for i:=0 to GetHeaderCount-1 do
 if Gettag(i)=cTag then
 begin
  SetFindex(nFound,i);
  inc(nfound,1);
 end;
 nFilterCount:=nFound;
 rfilter.possearchmode:=3;
end;

{procedure tbpgn.killshort(nMaxMove:integer);
var i,nfound:longint;
begin
 nFound:=0;
 for i:=0 to GetHeaderCount-1 do
 if isshortgame(i,nMaxMove) then
 begin
  setFindex(nFound,i);
  setTag(i,Tag_delete);
  inc(nFound,1);
 end;
 nFilterCount:=nFound;
 rfilter.PossearchMode:=3;
end;

function tbpgn.isShortgame(i:longint; nMaxMove:integer):boolean;
var p:PBpgnRec;
nw,nb:longint;
begin
 if i<nHeadercount-1 then
 begin
  p:=alist.Items[i+1];
  with p^ do begin nw:=P^.aPositionW; nb:=p^.aPositionB; end;
 end
 else begin nw:=pdblist[1].Count; nb:=pdblist[2].Count; end;

 p:=alist.Items[i];
 with P^ do
  if (((nw-apositionW)<=nMaxMove) and ((nb-apositionB)<=nMaxMove)) then Result:=True
  else Result:=false;
end;  }

procedure tbpgn.filterBroken;
var i,nfound:longint;
begin
 bfilterSet:=false;
 nFound:=0;
 for i:=0 to GetHeaderCount-1 do
 if isBroken(i) then
 begin
  SetFindex(nFound,i);
  inc(nfound,1);
 end;
 nFilterCount:=nFound;
 rfilter.possearchmode:=3;
end;

function tbpgn.isBroken(n:integer):boolean;
var n1,n2:integer;
s,s1:string;
begin
 n1:=getsideResult(1,n);
 n2:=getRsideResult(1,n);
 if n1<>n2 then begin Result:=true; exit; end;
 //check if the result after } is correct
 s:=getaReason(n);
 n1:=pos('}',s)+1;
 s1:=trim(copy(s,n1,length(s)));
 Result:=(s1<>GetResult(n));
end;

function tbpgn.getRSideResult(side:integer;n:longint):integer;
var s,player,opponent,pardop,partner,codew,cps:string;
ops,ps:integer;
begin
  player:=trim(LowerCase(getsidePlayer(side,n)));
  ops:=getopponentside(side);
  opponent:=trim(LowerCase(getSidePlayer(ops,n)));
  partner:=trim(LowerCase(getsidePlayer(getpartnerside(side),n)));
  pardop:=trim(LowerCase(getsidePlayer(getpartnerside(ops),n)));

  S:=lowercase(getareason(n));
  if pos('draw',s)>0 then
  begin
   result:=3; exit;
  end;

  cps:='''s partner';
  codew:=' won';
  if ((pos(player+codew,s)>0) or (pos(player+cps+codew,s)>0)) then
  begin
   result:=1; exit;
  end;
  if ((pos(partner+codew,s)>0) or (pos(partner+cps+codew,s)>0)) then
  begin
   result:=1; exit;
  end;
  if ((pos(opponent+codew,s)>0) or (pos(opponent+cps+codew,s)>0)) then
  begin
   result:=2; exit;
  end;
  if ((pos(pardop+codew,s)>0) or (pos(pardop+cps+codew,s)>0)) then
  begin
   result:=2; exit;
  end;

  codew:=' forfeit';
  if ((pos(player+codew,s)>0) or (pos(player+cps+codew,s)>0))  then
  begin
   Result:=2; exit;
  end;
  if ((pos(partner+codew,s)>0) or (pos(partner+cps+codew,s)>0))  then
  begin
   Result:=2; exit;
  end;
  if ((pos(opponent+codew,s)>0) or (pos(opponent+cps+codew,s)>0))  then
  begin
   Result:=1; exit;
  end;
  if ((pos(pardop+codew,s)>0) or (pos(pardop+cps+codew,s)>0))  then
  begin
   Result:=1; exit;
  end;

  codew:=' resign';
  if ((pos(player+codew,s)>0) or (pos(player+cps+codew,s)>0))  then
  begin
   result:=2; exit;
  end;
  if ((pos(partner+codew,s)>0) or (pos(partner+cps+codew,s)>0)) then
  begin
   result:=2; exit;
  end;
  if ((pos(opponent+codew,s)>0) or (pos(opponent+cps+codew,s)>0))  then
  begin
   result:=1; exit;
  end;
  if ((pos(pardop+codew,s)>0) or (pos(pardop+cps+codew,s)>0))  then
  begin
   result:=1; exit;
  end;

  codew:=' checkmated';
  if ((pos(player+codew,s)>0) or (pos(player+cps+codew,s)>0))  then
  begin
   result:=2; exit;
  end;
  if ((pos(partner+codew,s)>0) or (pos(partner+cps+codew,s)>0)) then
  begin
   result:=2; exit;
  end;
  if ((pos(opponent+codew,s)>0) or (pos(opponent+cps+codew,s)>0))  then
  begin
   result:=1; exit;
  end;
  if ((pos(pardop+codew,s)>0) or (pos(pardop+cps+codew,s)>0))  then
  begin
   result:=1; exit;
  end;

  codew:=' disconnect';
  if ((pos(player+codew,s)>0) or (pos(player+cps+codew,s)>0))  then
  begin
   result:=2; exit;
  end;
  if ((pos(partner+codew,s)>0) or (pos(partner+cps+codew,s)>0)) then
  begin
   result:=2; exit;
  end;
  if ((pos(opponent+codew,s)>0) or (pos(opponent+cps+codew,s)>0))  then
  begin
   result:=1; exit;
  end;
  if ((pos(pardop+codew,s)>0) or (pos(pardop+cps+codew,s)>0))  then
  begin
   result:=1; exit;
  end;
  Result:=0;
end;

procedure tbpgn.DeleteFiltered;
var i,j:longint;
begin
 for i:=0 to  nFilterCount-1 do
 begin
  j:=GetFindex(i);
  SetTag(j,Tag_Delete);
 end;
end;

procedure tbpgn.SelectFiltered;
var i,j:longint;
begin
 for i:=0 to nFilterCount-1 do
 begin
  j:=GetFindex(i);
  if Gettag(j)=Tag_OK then Settag(j,Tag_Select);
 end;
end;

procedure tbpgn.untagFiltered(cTag:char);
var i,j:longint;
begin
 for i:=0 to nFilterCount-1 do
 begin
  j:=GetFindex(i);
  if Gettag(j)=cTag then Settag(j,Tag_OK);
 end;
end;

function tbpgn.SynchFilter(var filter:TfilterRec):integer;
var r1,r2:Tposdbrec;
i:integer;
begin
 if filter.possearchmode=3 then begin Result:=0; exit; end;
 // nonpositional search, always valid.
 r1.position:=filter.position;
 i:=filter.possearchmode;
 boardtoint(bpgnarray[nCurrent,i].aboard,r2.position);
 if compposition(@r1,@r2)=0 then begin Result:=0; exit; end;
 i:=3-filter.possearchmode;
 boardtoint(bpgnarray[nCurrent,i].aboard,r2.position);
 if compposition(@r1,@r2)=0 then begin Result:=i; exit; end;
 Result:=-1;
end;

procedure tbpgn.SyncResultFromTag(gameN:integer);
var resStr:string;
res:integer;
begin
 res:=getSideResult(1,gameN);
 case res of
 1: ResStr:='{'+GetWhitea(gamen)+'''s partner won} '+BPGN_WHITE_WINS;
 2: ResStr:='{'+GetBlacka(GameN)+'''s partner won} '+BPGN_BLACK_WINS;
 3: Resstr:='{Game drawn} '+BPGN_DRAW;
 0: ResStr:='{'+BPGN_UNKNOWN+'} '+BPGN_UNKNOWN;
 end;
 ResStr:=resStr;//+EOL_STR+getaReason(gameN);
 SetReason(gameN,ResStr);
 bdbchange:=true;
end;

procedure tbpgn.SyncResultFromString(gameN:integer);
var res,res1:integer;
s,resStr:string;
begin
 s:=GetaReason(gamen); res:=pos('}',s);
 if res=0 then exit; //result string is clearly broken, exiting.
 res1:=getRsideResult(1,gameN);
 case res1 of
 1: ResStr:=BPGN_WHITE_WINS;
 2: ResStr:=BPGN_BLACK_WINS;
 3: Resstr:=BPGN_DRAW;
 0: ResStr:=BPGN_UNKNOWN;
 end;
 SetResult(gameN,ResStr);
 //make sure that result string after } is correct
 delete(s,res+1,length(s));
 s:=s+' '+ResStr;
 SetReason(gamen,s);
 bdbchange:=true;
end;

procedure tbpgn.SaveUpdates;
const  nGameChunk=MAX_STRING div 1000;
var i,nc,lc:longint;
cLine,fname,bpgnName:string;
f:TextFile;
barstep,nbarc:longint;
nBkupCurrent:longint;
begin
  nBkupCurrent:=nGameHeader;
  bpgnname:=getBpgnFilename;
  nc:=0; cLine:=''; lc:=0; barstep:=100; nBarc:=0;
  fname:=uniquefilename(ExtractfilePath(getBpgnFileName));
  Assign(f,Fname);
try
  RewRite(f);
  for I:=0 to getHeaderCount-1 do
  begin
   GetUpdatedGame(i);
   cLine:=cLine+cActualgame;
   if nc=nGamechunk then
   begin
    write(f,cLine);
    cLine:=''; nc:=0;
   end else inc(nc,1);
   setPosition(i,lc);

   inc(lc,length(cActualgame));
   if nbarc=barstep then
   begin
    PostMessage(fmainHandle,WM_ProgressUpdate,i+1,getHeaderCount); nbarc:=0;
   end else inc(nbarc,1);
  end; {for i}
  write(f,cLine);
  flush(f);
except
  cResult:='Error writing to file'+EOL_STR+fname;
  SendMessage(fmainHandle,WM_BPGNError,0,0);
  closefile(f);
  loadgame(nBkupCurrent);
  exit;
end;
  CloseFile(f);
  CloseFile(bpgnFile);
  try
  DeleteFile(bpgnname);
  RenameFile(fname,bpgnname);
  except
  cResult:='Can not write to file'+EOL_STR+bpgnname;
  SendMessage(fmainHandle,WM_BPGNError,0,0);
  loadgame(nBkupCurrent);
  exit;
  end;
  try
  AssignFile(bytefile, bpgnname);
  Reset(bytefile);
  nfilesize:=Filesize(bytefile);
  close(bytefile);
  AssignFile(bpgnfile,bpgnname);
  Reset(bpgnfile);
  except
  cResult:='Can not open file'+EOL_STR+bpgnname;
  SendMessage(fmainHandle,WM_BPGNError,0,0);
  end;
  SetGameChange(false);
  loadgame(nBkupCurrent);
end;

procedure tbpgn.SaveUpdates_(p:pointer);
begin
 saveupdates;
end;

{function tbpgn.GameScoreStart(bScrewFirstMove:boolean):longint;
var bFound,bNumber,bDisplay,bComment:boolean;
begin
//assumes that getgame before set everything correctly including cGameLine
    bFound   := FALSE;
    WHILE (TRUE) DO
    BEGIN
        Backup(1);
        IF (NOT GetToken) THEN
            BREAK;

        IF ((nLastState = BPGN_INTEGER) or (nLastState = BPGN_Comment2)) THEN
        BEGIN
            bNumber := TRUE;
            Restore(1);
            bFound := TRUE;
            BREAK;
        END;
    end;
    IF (NOT bFound) THEN result:=-1 else result:=nGameIndex;
end;
}

procedure tbpgn.GetUpdatedGame(nIndex:longint);
var nBeginPos,nEndPos,nHeaderEnd,nReasonStart,ln:longint;
s,cOldReason,cNewReason:string;
ps:integer;
bbkupchange:boolean;
begin
 bbkupchange:=isGameChange;
 loadgame(nIndex);
 setgamechange(true);
 getActualgameText(nIndex);
 setgamechange(bbkupchange);
{ if (nIndex=nGameHeader) then begin getActualGameText(nIndex); exit; end;
 nBeginPos := getPosition(nIndex);
 IF (nIndex = nHeaderCount - 1) THEN
        nEndPos := nFileSize
 ELSE begin
        nEndPos := getPosition(nIndex+1) - 1;
 end;
 getGame(nBeginPos,nEndPos);
 nHeaderEnd:=gameScoreStart(true);

 if nHeaderEnd<0 then nHeaderEnd:=length(cgameLine)+1;
 cActualGame:=trim(Copy(cGameLine,nHeaderEnd,length(cgameline)));
 s:=getaReason(nIndex);
 ps:=pos(EOL_STR,s);
 if ps>0 then begin //need to update reason
  cNewReason:=trim(copy(s,1,ps-1));
  cOldReason:=trim(copy(s,ps+1,length(s)));
  ln:=length(cOldReason);
  nReasonStart:=length(cActualGame)-ln+1;
  if nReasonStart>0 then
  begin
   s:=copy(cActualgame,nReasonStart,ln);
   if s=cOldReason then Delete(cActualGame,nReasonStart,ln);
  end;
  cActualGame:=cActualGame+EOL_STR+cNewReason;
  setReason(nIndex,cNewReason);
 end;
 getUpdatedHeader(nIndex);
 cActualGame:=cGameLine+cActualGame+EOL_STR+EOL_STR;}
end;

procedure tbpgn.getUpdatedHeader(nIndex:longint);
begin
    { Event, site, and date }
    cGameLine := '[Event "' + getEvent(nIndex) + '"]' + EOL_STR +
                 '[Site "'  + getSite(nIndex) + '"]' + EOL_STR +
                 '[Date "'  + getDate(nIndex) + '"]' + EOL_STR;


    { WhiteA }
    cGameLine := cGameLine + '[WhiteA "' + getWhiteA(nIndex) + '"]';
    IF (Length(getRWA(nIndex)) > 0) THEN
        cGameLine := cGameLine + '[WhiteAElo "' + getRWA(nIndex) + '"]';
    cGameLine := cGameLine + EOL_STR;


    { BlackA }
    cGameLine := cGameLine + '[BlackA "' + getblackA(nIndex) + '"]';
    IF (Length(getRBA(nIndex)) > 0) THEN
        cGameLine := cGameLine + '[BlackAElo "' + getRBA(nIndex) + '"]';
    cGameLine := cGameLine + EOL_STR;


    { WhiteB }
    cGameLine := cGameLine + '[WhiteB "' + getWhiteB(nIndex) + '"]';
    IF (Length(getRWB(nIndex)) > 0) THEN
        cGameLine := cGameLine + '[WhiteBElo "' + getRWB(nIndex) + '"]';
    cGameLine := cGameLine + EOL_STR;


    { BlackB }
    cGameLine := cGameLine + '[BlackB "' + getBlackB(nIndex) + '"]';
    IF (Length(getRBB(nIndex)) > 0) THEN
        cGameLine := cGameLine + '[BlackBElo "' + getRBB(nIndex) + '"]';
    cGameLine := cGameLine + EOL_STR;


    { BECOA }
    IF (Length(getOpenA(nIndex)) > 0) THEN
        cGameLine := cGameLine + '[BECOA "' + getOpenA(nIndex) + '"]' + EOL_STR;


    { BECOB }
    IF (Length(getOpenB(nIndex)) > 0) THEN
        cGameLine := cGameLine + '[BECOB "' + getOpenB(nIndex) + '"]' + EOL_STR;


    { TimeControl }
    cGameLine := cGameLine + '[TimeControl "' + getTimeC(nIndex) + '"]' + EOL_STR;


    { Result }
    cGameLine := cGameLine + '[Result "' + getResult(nIndex) + '"]' + EOL_STR;

    {Bfen}
    if getSetup(nIndex)='1' then
    begin
     cGameLine := cGameLine + '[Setup "' + getSetup(nIndex) + '"]' + EOL_STR;
     cGameLine := cGameLine + '[FEN "' + getFen(nIndex) + '"]' + EOL_STR;
    end;

    { Skip a line }
    cGameLine := cGameLine + EOL_STR;

end;

function tbpgn.FindHeader(cHeaderTag:string;var Cgame:string):string;
var nC,i:longint;
nBeg,psTag,nend:longint;
s:string;
begin
 Result:='';
 nc:=1;
 psTag:=Pos(cHeaderTag,cGame);
 if psTag=0 then Exit;
 if ((psTag>1) and (cgame[psTag-1]<>']')) then exit;
 nBeg:=psTag+length(cHeaderTag);
 if cGame[nBeg]<>'"' then exit;
 nEnd:=psTag;
 for i:=nBeg+1 to length(cgame) do
 if cGame[i]='"' then
 begin
  nEnd:=i;break;
 end;
 if nEnd>nBeg then Result:=Copy(cGame,nBeg+1,nEnd-nBeg-1) else Werror('Invalig [Lag] tag');
end;

function tbpgn.GetNextLag(nBpgnIndex:longint):integer;
var er,ps:integer;
s:string;
begin
 Result:=0;
 if not BPGNismain[nBPGNIndex] then exit;
 if not blagTag then exit;
 if cLagString='x' then exit;
 if ClagString='' then
 begin
    cResult:='Lag tag is too short or invalid format';
    cLagString:='x';
    Result:=-1;
    exit;
 end;
 ps:=pos(' ',cLagString);
 if ps=0 then ps:=length(cLagString)+1;
 s:=copy(cLagString,1,ps-1);
 delete(cLagString,1,ps);
 val(s,ps,er);
 if er=0 then Result:=ps else begin REsult:=-1; cResult:='Invalid lag format:'+EOL_STR+s;end;
end;

function tbpgn.GetCurrentLag(nBoard:integer):integer;
var c:char;
n:integer;
begin
 c:=GetCurrentBoard;
 if c in ['a','A'] then n:=1 else n:=2;
 if n=nBoard then Result:=BpgnLag[nCurrent]
 else Result:=0;
end;

function Tbpgn.GetCurrentWLag(nBoard:integer):integer;
begin
 Result:=Bpgnarray[nCurrent,nBoard].nWLag;
end;

function Tbpgn.GetCurrentBLag(nBoard:integer):integer;
begin
 Result:=Bpgnarray[nCurrent,nBoard].nBLag;
end;

procedure tbpgn.invertTag(cTag:char);
var i,j:longint;
c:char;
begin
 for i:=0 to nFilterCount-1 do
 begin
  j:=GetFindex(i);
  c:=Gettag(j);
  if (c=cTag) then Settag(j,TAG_OK);
  if (c=TAG_OK) then Settag(j,Ctag);
 end;
end;


END. { BPGN40K }
