(*============================================================================*)
(* MAIN.PAS -- Main program                                                   *)
(*----------------------------------------------------------------------------*)
(*                                                                            *)
(* BPGNVIEW.EXE -- BPGN file viewer.                                          *)
(* Copyright (C) 1998  Zon Juan Jan, 2002-2005 Fermy                          *)
(*                                                                            *)
(* This program is free software; you can redistribute it and/or              *)
(* modify it under the terms of the GNU General Public License                *)
(* as published by the Free Software Foundation; either version 2             *)
(* of the License, or (at your option) any later version.                     *)
(*                                                                            *)
(* This program is distributed in the hope that it will be useful,            *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(* GNU General Public License for more details.                               *)
(*                                                                            *)
(* You should have received a copy of the GNU General Public License          *)
(* along with this program; if not, write to the Free Software Foundation,    *)
(* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *)
(*                                                                            *)
(*----------------------------------------------------------------------------*)
(* HISTORY                                                                    *)
(* -------                                                                    *)
(* 1998.11.03: First construction.                                            *)
(* 1998.11.04: 1) Removed calls to StopTimer() when window gets activated.    *)
(*             2) Added short cut keys to window activation.                  *)
(*             3) Changed Extrect bpgn function so it will not be disabled    *)
(*                when a bpgn file is open.                                   *)
(*             4) Made navigator form sizeable.                               *)
(*             5) Added previous next move.                                   *)
(* 1998.11.05: 1) Added show ratings under game list options.                 *)
(*             2) Added show time under game list options.                    *)
(*             3) Added show increment under game list options.               *)
(*             4) Added show result under game list options.                  *)
(*             5) Added show square outline under board options.              *)
(*             6) Added game # selected / # of games in game list caption.    *)
(* 1998.11.06: 1) Moved edit clock from game to move menu.                    *)
(*             2) Added flip, start, and stop to game menu.                   *)
(*             3) Added edit event, site, date, time control, and result into *)
(*                game menu.                                                  *)
(*             4) Changed all the shortcut keys form control to alt keys.     *)
(* 1998.11.07: 1) Changed version from 0.92 alpha to 0.93 alpha.              *)
(*             2) Added new file.                                             *)
(*             3) Added new game.                                             *)
(*             4) Added duplicate game.                                       *)
(*             5) Patched a bug in extract function where I didn't pass the   *)
(*                full path for the source file.                              *)
(*             6) Added in a check to make sure in extract function the       *)
(*                destination file does not get included in the source filter.*)
(* 1998.11.08: 1) Changed the code so the cursor will change to hour glass    *)
(*                during BPGN file loading.                                   *)
(* 1998.11.09: 1) Added show game # under game list options.                  *)
(*             2) Changed the extract function so it will produce 2 bpgn game *)
(*                in the destination bpgn file (white/black).                 *)
(*             3) Added view extract.err in utility menu.                     *)
(*             4) Added user select maximum # of positions per bpgn game.     *)
(* 1998.11.12: Added show event under game list options.                      *)
(* 1998.11.13: Added opponent, bpgn filename, and game # into extract's       *)
(*             occurrence tag.                                                *)
(* 1998.11.17: 1) Added FCD stuff.                                            *)
(*             2) Added view RTF file.                                        *)
(* 1998.11.18: 1) Added TXT stuff.                                            *)
(*             2) Added view TXT file.                                        *)
(* 1998.11.23: 1) Added include handle, move, annotation, time, comment,      *)
(*                display board, and result for convert to RTF/TXT function.  *)
(*             2) Added remove all destination comments before extraction.    *)
(*             3) Added generate destination occurrence comment.              *)
(*             4) Added edit move piece keymap in RTF function.               *)
(* 1998.11.24: 1) Fixed a bug in GetRTFMove() where pawn promotion does not   *)
(*                get translated properly.                                    *)
(*             2) Fixed a pawn move translation error in GetRTFMove().        *)
(*             3) Removed include move piece keymap.                          *)
(*             4) Moved edit move piece keymap from RTF function to under     *)
(*                utility menu.                                               *)
(* 1998.11.25: Added use zeros instead of Os in castling move to edit move    *)
(*             piece keymap function, RTF function, and TXT function.         *)
(* 1998.11.27: Added word wrap into comment/option.                           *)
(* 1999.01.20: 1) Added code to read in Crazyhouse games (determined by event *)
(*               tag in BPGN file: [Event "fics unrated crazyhouse match"]).  *)
(*               If it's a crazyhouse game then the captured piece's color    *)
(*               will be reversed and into player's piece buffer instead of   *)
(*               going to player partner's piece buffer as in regular         *)
(*               bughouse games.                                              *)
(*            2) Changed version ID to 1.13 beta.                             *)
(* 2001.12.12 1)Patched Smartmove bug (Double count for pieces captured with  *)
(*              smartmove). Change is in functions                            *)
(*              TMainForm.Board1/2MouseUp/Down                                *)
(*            2)Crazyhouse game now recognized by substring "crazyhouse" in   *)
(*              BPGN tag                                                      *)
(* 2002.01-05: Too many changes to list. See Readme.rtf for details           *)
(* 2004.11.13 1)Patched bug in TmainForm.formClose that caused the layout of  *)
(*              engine window to not be saved                                 *)
(*            2)Changed TmainForm.OpenBPGNfile to open also databases         *)
(* 2004.11.27 1) fForm is now created on start and is no longer modal         *)
(*            2) Added mouseclick handles for the fform: movestatclick,       *)
(*               filtergames,statgames                                        *)
(*            3) Added execmovestats which is responsible for handling clicks *)
(*               on moves in fform.                                           *)
(*            4) Convert to db now has a progress bar                         *)
(* 2004.11.28 1) Modified disableEngines to not disable the hand button       *)
(*            2)Modified Loadrecord to call statgames if positional autosearch*)
(* 2004.11.29 Added Options-->BPGN menu and a choice of handling of airdrops  *)
(* 2004.12.02 Added Options-->Bpgn--> DB Save Depth and dbdepth.pas           *)
(* 2004.12.03 1) Changed File-->Convert to Database to Generate Database      *)
(*            2) Modified file0click to disable save items in File menu if    *)
(*               no file is open                                              *)
(*            3) Added menu item Options-->Filter-->Full Stats                *)
(*            4) Modified execmovestats to check whether takeback is possible *)
(*               when the <== button is clicked                               *)
(* 2004.12.07 1) Added a popup menu to right click drop pieces on the board,  *)
(*               modified board1/2mouseDown to allow that                     *)
(*            2) Added RepaintAllforms to repaint all graphical forms before  *)
(*               a lengthy operation                                          *)
(*            3) Modified SaveAs1Click to save also databases                 *)
(* 2004.12.08 1) Added code to save layout of filter window                   *)
(*            2) Made No the default answer to save game question, for that   *)
(*               added NoYes form                                             *)
(*            3) Moved Bitbtn1 to the navigate form                           *)
(* 2004.12.12 1) Added menu Database, move Generate database there            *)
(*            2) Made capitalization in the menu consistent                   *)
(*            3) Added menu item Database--> Killdoubles                      *)
(* 2004.12.15 1) Modified filter1click to disable unfilter only when          *)
(*               bpgn.filtercount<>bpgn.headercount                           *)
(*            2) Fixed extract moves from bpgns, it didn't work since indexing*)
(*               changed, also added repaintallforms where necessary          *)
(*            3) Changed board1/2mousedown so that drop menu appears only if  *)
(*               the click was on the board                                   *)
(* 2004.12.17 1) Made File-->Save/Move Filtered, Game-->Append To save to dbs *)
(*            2) Modified Game0click to disable Append to work when work is   *)
(*               opened                                                       *)
(* 2004.12.20    Changed gamelistclick to number games from 1                 *)
(* 2004.12.23 1) Fixed bug File-->Save: game list was not updated             *)
//            2) Added Game-->Setup new functionality
// 2004.12.26    Fixed bug Game -->Setup new: Labels for whiteB and BlackA were
//               switched. Made so that offboard stuff can be entered in any case
// 2004.12.29    Fixed major bug that affected games with numbers higher than 65536.
// 2004.12.30 1) Fixed bug in opening a database with more than 65536 games
//            2) Changed the default background color or comment, next, previous
//               next and game list forms to silver
//            3) Fixed bug in Game --> Handle -->BlackA. This was setting the
//               handle for the game immediately following the current one.
//            4) Fixed bug in bpgn.LoaddbHeaders. Under certain circumstances
//               headers were shifted.
//            5) Changed FCDProc to fix the bug with misaligned handles and moves
//            6) Modified gamelistclick to fix bug: If headers were modified,
//               but then all changes to the game were discarded (e.g. click
//               another game, choose "NO" in the save game changes dialog) the
//               game list was showing modified headers instead of original.
//            7) Fixed bug in the save game changes dialog. Answer cancel had
//               the same effect as no. Modified CheckGameSaveChanges.
// 2005.01.02 1) Added Windows-->Arrange windows
//            2) Disabled Utility-->-->Split current bpgn into bug and zh parts
//               for databases (not applicable)
//            3) Modified NavigateClickx to bring to the top an appropriate form
//            4) Layout of player stats window is now saved with all others.
//            5) Fixed bug in game0click: Game-->Stop disabled whenever open file
//               button is disabled.
//            6) Fixed bug open file button is disabled after click on gamelist.
//            7) Fixed bug in Default1Click: it did nothing.
//            8) Fixed bug in LoadRecord: Titlebar of the main window disapears
//               after click on "games" button in Tree window if follow the game
//               is checked.
//            9) Coded proper long operations: Added SynchThreads to the project,
//               Added calls to doLongRun throughout
// 2005.01.08 1) Modified various functions that work with gamelist to call
//               gamelistform.setState instead of setting the caption directly.
//            2) Made Ctrl+U the shortcut to Search-->Unfilter
//            3) Fixed bug: Button >| not working properly after the use of
//               tree functions
//            4) Removed calls to CheckFileSaveChanges before searches
//            5) Added Search-->Deleted Games
//            6) Removed Options-->BPGN, added Options-->Database. Added to that
//               submenu Read only games and Open with tree
// 2005.01.10 1) Added to Search menu: Delete/Undelete all filtered, Broken result
//            2) Disabled adding/duplicating games to the databases when
//               Options --> Database --> Read only games is checked (gameOclick)
//            3) Added Selected games and 3 items to the Search menu that work
//               with selected games.
//            4) Added parameter to DoLongRun that controls if operation can be canceleed
//            5) Changed captions of items in Search menu, e.g. Delete all shown
// 2005.01.12 1) Merged filter and tree into one form (now only one is visible at a time
//            2) Moved Search --> Broken result to the utility menu, renamed to
//               Find broken results, added  to Utility menu Restore result string from
//               result tag and Restore result tag from result string in shown games
// 2005.01.16 1) Replaced "shown" by "listed" throughout Search menu
//            2) Added Game-->Replay all games
//            3) Added Utility-->Replace handle in listed games
// 2005.01.25 Modified timer1.timer and SetupNextMove to make replay faster
//            (arguably closer to real life)
// 2005.01.29 1) Fixed bug in Arrange windows: misplaced next and previous move forms
//            2) Fixed cancelling of long operations: added WMcancelOp
//            3) Added About-->View FAQ
//            4) Modified Create database to call fform.godatabase after success
// 2005.02.09 1) Changed executable version to 3.00
//            2) Fixed bug in movestatclick: Sorting criteria in the tree grid
//               was lost after click on <==
// 2005.02.10 1) Disabled right click on the board during replay: BoardxMousedown
//            2) Modified LoadRecord and timer1timer to work with Lag and decimal time
//               Set timer1.interval to 10 ms
//            3) Got rid of Board2Form. Now both boards are of the type Board1Form
//            4) Fixed bug in OpenBpgnFile: Program title shows file even if no
//               file was open (aborted operation)
//            5) Renamed Utility ==> Convert current game to ...RTF and Convert
//               current game to ...TXT to Generate ... RTF from current game and
//               Generate ... TXT from current game. Modified these items to work
//               with Lag tag and decimal time (FCDProc and TXTProc) respectively
//            6) Fixed bug in ExtractMovesFromFile (exceptions)
//            7) Added Glyph for the display button.
// 2005.02.15 Modified timer1time and setupnextmove to speed up playback
//            (less frequent updates). Timer1.interval=100 ms
// 2005.02.17 1) Fixed weird bug in Formclose: if Save layouts on exit were unchecked
//               and one of the child windows were maximized access violations appeared
//            2) Modified LoadRecord to use a faster method to refresh board
//               than boardxform.displayboard.
//            3) Added timer3. Moved clock refresh from timer1 to timer3.
//            4) Now timer1.interval is the time left to nextmove instead of fixed
//            5) Loadrecord sets timer1.interval accounting for processing delay
//            6) Modified LoadRecord to catch i
// 2005.02.22 1) Code cleanup: removed rarely used forms by reuse of similar ones
//               Gone: Piecemap,VIEWEXTR, VIEWRTF
//            2) Added ShowFile and ShowtxtFile that displays an RTF file in readme
//               form and TXT file in ViewTxtForm respectively
//            3) Moved Utility ==>Edit move piece keymap to the FCD form, where
//               it belongs.
// 2005.02.28 1) Removed onShow handler. Now it's called only once in bpgnview.pas
//               after form is created: Fixes stack overflow on win98
//            2) Added Options-->Board-->Show lag,Show tenth of seconds on the clock
//            3) Modified Setupnextmove to slow down playback of lag-free bpgns
//               Moves that take 0 seconds according to bpgn will take 0.2 s in replay
//            4) Added Database-->Online database that points to jb's site
//            5) Added Search-->Invert selection in listed
// 2005.03.02 Modified finalizesetup to call bpgn.appendnewgame to add game; bpgn
//            object is not touched directly here anymore. Fixes bug: Invalid result
//            string after adding a new game with Setup new game
// 2005.03.03 1) Fixed bug in LoadRecord call to boardx.makeMove: in zh captured
//               pieces were not redrawn properly
//            2) Fixed bug in SetupNextMove: clocks not stopping at the end of
//               the game (e.g, if rewind to the end is pressed during playback)
//            3) Added FileMode:=fmOpenRead to the formcreate: Fixes failure to
//               open Read only files
//            4) Improved File-->Save as. Should work faster, should save/overwrite
//               read only files
(*============================================================================*)
UNIT MAIN;


INTERFACE

USES
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    Menus, ExtCtrls, stdctrls, syncobjs,ComCtrls, math,
    ABOUT, ANNOTATE,EngineOut, BOARD1, BPGNx, CHESS, CLOCK, COMMENT, ENTERDT,
    ENTERTC, ENTERRC, EVENT, EXTRACT, FCD, GAMELIST, GLOBAL, HANDLE, MISC,
    NAVIGATE, NEXTMOVE, PREVMOVE, RATING, SITE, TOOLBOX, VIEWACT,
    TXT, VIEWTXT,ini1,EngComline,Sjeng,
    Engann,engtest,readme,associate,soundopt,
    SRWave, boardcfg, LbButton,Buttons, RepeatBtn, merge, split,
    filter, stat, posdb1, progressdb, dbdepth, setupall,bpgnlist, synchedThreads;


Const
EMPTY_MENU='< Empty >';

TYPE
tTestrec=record
move:string;
score:real;
end;

Ttestar=array[1..50] of ttestrec;

TEngineRec=record
 bEngineOn:boolean;
 exist:boolean;
 suspended:boolean;
end;
TEngineArr=array[1..MAX_ENGINE] of TEngineRec;

    TMainForm = CLASS(TForm)
        MainMenu1:   TMainMenu;
        File0:       TMenuItem;
        File1:       TMenuItem;
        File2:       TMenuItem;
        File90:      TMenuItem;
        File91:      TMenuItem;
        File92:      TMenuItem;
        File3:       TMenuItem;
        File4:       TMenuItem;
        File5:       TMenuItem;
        File6:       TMenuItem;
        File7:       TMenuItem;
        Game0:       TMenuItem;
        Game20:      TMenuItem;
        Game21:      TMenuItem;
        Game99:      TMenuItem;
        Game22:      TMenuItem;
        Game23:      TMenuItem;
        Game24:      TMenuItem;
        Game25:      TMenuItem;
        Game2:       TMenuItem;
        Game3:       TMenuItem;
        Game4:       TMenuItem;
        Game5:       TMenuItem;
        Game6:       TMenuItem;
        Game7:       TMenuItem;
        Game8:       TMenuItem;
        Game9:       TMenuItem;
        Game10:      TMenuItem;
        Name1:       TMenuItem;
        Name2:       TMenuItem;
        Name3:       TMenuItem;
        Name4:       TMenuItem;
        Name5:       TMenuItem;
        Game11:      TMenuItem;
        Rating1:     TMenuItem;
        Rating2:     TMenuItem;
        Rating3:     TMenuItem;
        Rating4:     TMenuItem;
        Rating5:     TMenuItem;
        Game12:      TMenuItem;
        Game13:      TMenuItem;
        Game14:      TMenuItem;
        Game15:      TMenuItem;
        Game16:      TMenuItem;
        Game17:      TMenuItem;
        Game18:      TMenuItem;
        Game19:      TMenuItem;
        Move0:       TMenuItem;
        Move1:       TMenuItem;
        Move2:       TMenuItem;
        Move3:       TMenuItem;
        Move4:       TMenuItem;
        Move5:       TMenuItem;
        Options0:    TMenuItem;
        Options1:    TMenuItem;
        Board17:     TMenuItem;
        Board18:     TMenuItem;
        Board19:     TMenuItem;
        Board20:     TMenuItem;
        Board21:     TMenuItem;
        Board22:     TMenuItem;
        Board27:     TMenuItem;
        Board29:     TMenuItem;
        Options4:    TMenuItem;
        GameList1:   TMenuItem;
        GameList2:   TMenuItem;
        GameList3:   TMenuItem;
        GameList99:  TMenuItem;
        GameList98:  TMenuItem;
        GameList4:   TMenuItem;
        GameList5:   TMenuItem;
        GameList6:   TMenuItem;
        GameList7:   TMenuItem;
        GameList8:   TMenuItem;
        GameList9:   TMenuItem;
        Options5:    TMenuItem;
        Nextmove1:   TMenuItem;
        Nextmove2:   TMenuItem;
        Nextmove3:   TMenuItem;
        Nextmove4:   TMenuItem;
        Options12:   TMenuItem;
        Prevmove1:   TMenuItem;
        Prevmove2:   TMenuItem;
        Prevmove3:   TMenuItem;
        Prevmove4:   TMenuItem;
        Options6:    TMenuItem;
        Comment1:    TMenuItem;
        Comment2:    TMenuItem;
        Comment3:    TMenuItem;
        Comment4:    TMenuItem;
        Comment5:    TMenuItem;
        Options13:   TMenuItem;
        Options8:    TMenuItem;
        Options9:    TMenuItem;
        Options10:   TMenuItem;
        Options11:   TMenuItem;
        Utility0:    TMenuItem;
        Utility1:    TMenuItem;
        Utility2:    TMenuItem;
        Utility3:    TMenuItem;
        Utility4:    TMenuItem;
        Utility5:    TMenuItem;
        Utility6:    TMenuItem;
        Utility7:    TMenuItem;
        Utility8:    TMenuItem;
        Windows0:    TMenuItem;
        Windows1:    TMenuItem;
        Windows2:    TMenuItem;
        Windows3:    TMenuItem;
        Windows4:    TMenuItem;
        Windows5:    TMenuItem;
        Windows6:    TMenuItem;
        Windows7:    TMenuItem;
        Help0:       TMenuItem;
        Help1:       TMenuItem;

        ColorDialog1: TColorDialog;
        FontDialog1:  TFontDialog;
        OpenDialog1:  TOpenDialog;
        OpenDialog2:  TOpenDialog;
        OpenDialog3:  TOpenDialog;
        OpenDialog4:  TOpenDialog;
        OpenDialog5:  TOpenDialog;
  //      SaveDialog1:  TSaveDialog;
        Timer1:       TTimer;
    Weng: TMenuItem;
    Engine2: TMenuItem;
    Font1: TMenuItem;
    Color1: TMenuItem;
    Default1: TMenuItem;
    WordWrap1: TMenuItem;
    Eng_2: TMenuItem;
    N2: TMenuItem;
    Eng_1: TMenuItem;
    Eng_3: TMenuItem;
    Eng_4: TMenuItem;
    Eng_5: TMenuItem;
    Options_1: TMenuItem;
    SetasDefault_1: TMenuItem;
    Options_2: TMenuItem;
    SetasDefault_2: TMenuItem;
    Options_3: TMenuItem;
    SetasDefault_3: TMenuItem;
    Options_4: TMenuItem;
    SetasDefault_4: TMenuItem;
    Options_5: TMenuItem;
    SetasDefault_5: TMenuItem;
    N1: TMenuItem;
    Weng_1: TMenuItem;
    Weng_2: TMenuItem;
    Weng_3: TMenuItem;
    Weng_4: TMenuItem;
    Weng_5: TMenuItem;
    Usebitmap1: TMenuItem;
    Timer2: TTimer;
    ViewReadme1: TMenuItem;
    N4: TMenuItem;
    RegisterBpgnfiles1: TMenuItem;
    SRWave1: TSRWavePlayer;
    Sound1: TMenuItem;
    ApplyGameSound1: TMenuItem;
    Configure1: TMenuItem;
    Imsound: TImage;
    Imnosound: TImage;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Image5: TImage;
    Image6: TImage;
    Image7: TImage;
    Image8: TImage;
    SaveAs1: TMenuItem;
    SaveDialog2: TSaveDialog;
    Game26: TMenuItem;
    Game27: TMenuItem;
    Utility11: TMenuItem;
    N3: TMenuItem;
    File8: TMenuItem;
    N5: TMenuItem;
    Utility12: TMenuItem;
    Utility13: TMenuItem;
    Filter1: TMenuItem;
    search1: TMenuItem;
    search2: TMenuItem;
    File10: TMenuItem;
    File11: TMenuItem;
    Search3: TMenuItem;
    AirDrops1: TMenuItem;
    DBsavedepth1: TMenuItem;
    Filter2: TMenuItem;
    FullStats1: TMenuItem;
    DonotSortMovelist1: TMenuItem;
    PopupMenu1: TPopupMenu;
    Pawn1: TMenuItem;
    Rook1: TMenuItem;
    Knight1: TMenuItem;
    Bishop1: TMenuItem;
    Queen1: TMenuItem;
    Database1: TMenuItem;
    GenerateDatabase1: TMenuItem;
    Killdoubles1: TMenuItem;
    N6: TMenuItem;
    Setupposition1: TMenuItem;
    Setupmenu: TPopupMenu;
    WhiteK1: TMenuItem;
    BlackK1: TMenuItem;
    N8: TMenuItem;
    markpromoted: TMenuItem;
    Newfromposition1: TMenuItem;
    ree1: TMenuItem;
    A1: TMenuItem;
    Showdeleted1: TMenuItem;
    Database2: TMenuItem;
    Donoteditgames1: TMenuItem;
    N7: TMenuItem;
    Showtreeonopen1: TMenuItem;
    Brokenresult1: TMenuItem;
    Markallfiltered1: TMenuItem;
    Unmarkallfiltered1: TMenuItem;
    N9: TMenuItem;
    N10: TMenuItem;
    Showselected1: TMenuItem;
    Unselectallshown1: TMenuItem;
    Selectallshow1: TMenuItem;
    Select1: TMenuItem;
    N11: TMenuItem;
    RestoreStringFromTag1: TMenuItem;
    RestoreTagFromString1: TMenuItem;
    Replayallgames1: TMenuItem;
    Replacehandleinlisted1: TMenuItem;
    ViewFAQ1: TMenuItem;
    Image9: TImage;
    Image10: TImage;
    Timer3: TTimer;
    Showlag1: TMenuItem;
    Showtenth1: TMenuItem;
    In1: TMenuItem;
    N12: TMenuItem;
    Onlinedatabase1: TMenuItem;


        PROCEDURE FormCreate(Sender: TObject);
        PROCEDURE FormShow(Sender: TObject);
        PROCEDURE FormClose(Sender: TObject; VAR Action: TCloseAction);
        PROCEDURE ResetAll(Sender: TObject);
        PROCEDURE SaveFileChanges(Sender: TObject; bAdjustGameList: Boolean);
        FUNCTION  CheckFileSaveChanges(Sender: TObject; bAdjustGameList: Boolean): Boolean;

        PROCEDURE File0Click(Sender: TObject);
        PROCEDURE File1Click(Sender: TObject);
        PROCEDURE File2Click(Sender: TObject);
        PROCEDURE File91Click(Sender: TObject);
        PROCEDURE File3Click(Sender: TObject);
        PROCEDURE File5Click(Sender: TObject);
        PROCEDURE File7Click(Sender: TObject);
        PROCEDURE Game0Click(Sender: TObject);
        PROCEDURE Game20Click(Sender: TObject);
        PROCEDURE Game21Click(Sender: TObject);
        PROCEDURE Game22Click(Sender: TObject);
        PROCEDURE Game24Click(Sender: TObject);
        PROCEDURE Game25Click(Sender: TObject);
        PROCEDURE Game3Click(Sender: TObject);
        PROCEDURE Game4Click(Sender: TObject);
        PROCEDURE Game5Click(Sender: TObject);
        PROCEDURE Game7Click(Sender: TObject);
        PROCEDURE Game8Click(Sender: TObject);
        PROCEDURE Game9Click(Sender: TObject);
        PROCEDURE Game10Click(Sender: TObject);
        PROCEDURE Name1Click(Sender: TObject);
        PROCEDURE Name2Click(Sender: TObject);
        PROCEDURE Name4Click(Sender: TObject);
        PROCEDURE Name5Click(Sender: TObject);
        PROCEDURE Game11Click(Sender: TObject);
        PROCEDURE Rating1Click(Sender: TObject);
        PROCEDURE Rating2Click(Sender: TObject);
        PROCEDURE Rating4Click(Sender: TObject);
        PROCEDURE Rating5Click(Sender: TObject);
        PROCEDURE Game12Click(Sender: TObject);
        PROCEDURE Game13Click(Sender: TObject);
        PROCEDURE Game15Click(Sender: TObject);
        PROCEDURE Game16Click(Sender: TObject);
        PROCEDURE Game17Click(Sender: TObject);
        PROCEDURE Game19Click(Sender: TObject);
        PROCEDURE Move0Click(Sender: TObject);
        PROCEDURE Move1Click(Sender: TObject);
        PROCEDURE Move3Click(Sender: TObject);
        PROCEDURE Move5Click(Sender: TObject);
        PROCEDURE Options0Click(Sender: TObject);
        PROCEDURE Board18Click(Sender: TObject);
        PROCEDURE Board19Click(Sender: TObject);
        PROCEDURE Board20Click(Sender: TObject);
        PROCEDURE Board22Click(Sender: TObject);
        PROCEDURE Board27Click(Sender: TObject);
        procedure board29click(sender: Tobject);
        PROCEDURE GameList1Click(Sender: TObject);
        PROCEDURE GameList2Click(Sender: TObject);
        PROCEDURE GameList99Click(Sender: TObject);
        PROCEDURE GameList98Click(Sender: TObject);
        PROCEDURE GameList4Click(Sender: TObject);
        PROCEDURE GameList5Click(Sender: TObject);
        PROCEDURE GameList6Click(Sender: TObject);
        PROCEDURE GameList7Click(Sender: TObject);
        PROCEDURE GameList9Click(Sender: TObject);
        PROCEDURE Nextmove1Click(Sender: TObject);
        PROCEDURE Nextmove2Click(Sender: TObject);
        PROCEDURE Nextmove4Click(Sender: TObject);
        PROCEDURE Prevmove1Click(Sender: TObject);
        PROCEDURE Prevmove2Click(Sender: TObject);
        PROCEDURE Prevmove4Click(Sender: TObject);
        PROCEDURE Comment1Click(Sender: TObject);
        PROCEDURE Comment2Click(Sender: TObject);
        PROCEDURE Comment3Click(Sender: TObject);
        PROCEDURE Comment5Click(Sender: TObject);
        PROCEDURE Options8Click(Sender: TObject);
        PROCEDURE Options9Click(Sender: TObject);
        PROCEDURE Options11Click(Sender: TObject);
        PROCEDURE Utility0Click(Sender: TObject);
        PROCEDURE Utility1Click(Sender: TObject);
        PROCEDURE Utility2Click(Sender: TObject);
        PROCEDURE Utility4Click(Sender: TObject);
        PROCEDURE Utility5Click(Sender: TObject);
        PROCEDURE Utility7Click(Sender: TObject);
        PROCEDURE Utility8Click(Sender: TObject);
        PROCEDURE Windows0Click(Sender: TObject);
        PROCEDURE Windows1Click(Sender: TObject);
        PROCEDURE Windows2Click(Sender: TObject);
        PROCEDURE Windows3Click(Sender: TObject);
        PROCEDURE Windows4Click(Sender: TObject);
        PROCEDURE Windows5Click(Sender: TObject);
        PROCEDURE Windows6Click(Sender: TObject);
        PROCEDURE Windows7Click(Sender: TObject);
        PROCEDURE Help0Click(Sender: TObject);
        PROCEDURE Help1Click(Sender: TObject);

        PROCEDURE LoadCustomColors;
        PROCEDURE SaveCustomColors;
        PROCEDURE LoadIntoINI;

        PROCEDURE OpenBPGNFile(Sender: TObject; cFn: String);
        PROCEDURE GameListClick(Sender: TObject);

        PROCEDURE NavigateClick1(Sender: TObject);
        PROCEDURE NavigateClick2(Sender: TObject);
        PROCEDURE NavigateClick3(Sender: TObject);
        PROCEDURE NavigateClick4(Sender: TObject);
        PROCEDURE NavigateClick5(Sender: TObject);
        PROCEDURE NavigateClick6(Sender: TObject);
        PROCEDURE NavigateClick7(Sender: TObject);
        PROCEDURE NavigateClick8(Sender: TObject);
        procedure NavigateClick9(Sender: TObject);
        procedure NavigateClick10(SEnder: TObject);
        Procedure NavigateClick11(Sender:Tobject);
        Procedure NavigateClick12(Sender: Tobject);
        procedure navigateclick13(Sender:Tobject);
        procedure navigateclick14(Sender: Tobject);

        PROCEDURE Board1Click1(Sender: TObject);
        PROCEDURE Board1Click2(Sender: TObject);
        PROCEDURE Board1Click3(Sender: TObject);
        PROCEDURE Board1Click4(Sender: TObject);
        PROCEDURE Board2Click1(Sender: TObject);
        PROCEDURE Board2Click2(Sender: TObject);
        PROCEDURE Board2Click3(Sender: TObject);
        PROCEDURE Board2Click4(Sender: TObject);
        PROCEDURE NextMoveClick(Sender: TObject);
        PROCEDURE PrevMoveClick(Sender: TObject);
        PROCEDURE CommentChange(Sender: TObject);

        PROCEDURE Board1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        PROCEDURE Board1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
        PROCEDURE Board1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        PROCEDURE   SMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        PROCEDURE   SMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
        PROCEDURE   SMouseUp  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);


        PROCEDURE Board2MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        PROCEDURE Board2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
        PROCEDURE Board2MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

        PROCEDURE LoadRecord(bFast:boolean=false);
        PROCEDURE MadeMove(cBoard: Char; cMove: String; nMI: Integer);

        PROCEDURE ProcessBeg(cSt: String);
        PROCEDURE ProcessMid(cSt: String);
        PROCEDURE ProcessEnd;

        PROCEDURE ExtractWriteError(nWhich: Integer);
        FUNCTION  ExtractMovesIntoFile(cMatchBoard: Char; cGame: String): Integer;
        FUNCTION  ExtractMovesFromFile(cFn, cColor: String; cMatchBoard: Char): Boolean;
        PROCEDURE ExtractMoves;

        PROCEDURE StopTimer;
        PROCEDURE SetupNextMove;
        PROCEDURE Timer1Timer(Sender: TObject);

        PROCEDURE NavigateFormKeyDown(Sender: TObject; VAR Key: Word; Shift: TShiftState);
        PROCEDURE Board1FormKeyDown(Sender: TObject; VAR Key: Word; Shift: TShiftState);
        PROCEDURE Board2FormKeyDown(Sender: TObject; VAR Key: Word; Shift: TShiftState);

        PROCEDURE MapShortCutKeys;

        FUNCTION  FCDDelLastTab(cLine: String): String;
        FUNCTION  GetRTFMove(bWhitesMove: Boolean; cMove: String): String;
        PROCEDURE FCDProc(var FCDform:Tfcdform);

        FUNCTION  GetTXTMove(bWhitesMove: Boolean; cMove: String): String;
        PROCEDURE TXTProc(var txtform:ttxtform);
    procedure WengClick(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure Color1Click(Sender: TObject);
    procedure WordWrap1Click(Sender: TObject);

    procedure Options_1Click(Sender: TObject);
    procedure SetasDefault_1Click(Sender: TObject);
    procedure uncheckEngines;
    procedure Weng_1Click(Sender: TObject);
    procedure button1click1(Sender: Tobject);
    procedure Timer2Timer(Sender: TObject);
    procedure timer3timer(Sender: Tobject);
    procedure Donothing1(Sender: TObject; VAR Key: Word; Shift: TShiftState);
    procedure donothing(Sender: Tobject);
    procedure donothing2(Sender: TObject; Button: TMouseButton;
                                    Shift: TShiftState; X, Y: Integer);
    procedure donothing3(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure doabsolutelynothing(Sender:Tobject);
    procedure ViewReadme1Click(Sender: TObject);
    procedure RegisterBpgnfiles1Click(Sender: TObject);
    procedure ApplyGameSound1Click(Sender: TObject);
    procedure Configure1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure setupboardconfig(bd:Pboardcfg);
    procedure Game27Click(Sender: TObject);
    procedure Utility11Click(Sender: TObject);
    procedure Game26Click(Sender: TObject);
    procedure File8Click(Sender: TObject);
    procedure Utility12Click(Sender: TObject);
    procedure Utility13Click(Sender: TObject);
    procedure search1Click(Sender: TObject);
    procedure search2Click(Sender: TObject);
    procedure Filter1Click(Sender: TObject);
    procedure File10Click(Sender: TObject);
    procedure File11Click(Sender: TObject);
    procedure Search3Click(Sender: TObject);
    procedure CreateDatabase1Click(Sender: TObject);
    procedure filtergames(Sender:Tobject);
    procedure statgames(Sender:Tobject);
    procedure movestatclick(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure execmovestats(xform:tfform;row:integer);
    procedure AirDrops1Click(Sender: TObject);
    procedure DBsavedepth1Click(Sender: TObject);
    procedure FullStats1Click(Sender: TObject);
    procedure DonotSortMovelist1Click(Sender: TObject);
    procedure Pawn1Click(Sender: TObject);
    procedure Rook1Click(Sender: TObject);
    procedure Knight1Click(Sender: TObject);
    procedure Bishop1Click(Sender: TObject);
    procedure Queen1Click(Sender: TObject);
    procedure dropclick(nboard,npiece:integer);
    procedure Database1Click(Sender: TObject);
    procedure Killdoubles1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure Setupposition1Click(Sender: TObject);
    procedure WhiteK1Click(Sender: TObject);
    procedure BlackK1Click(Sender: TObject);
    procedure markpromotedClick(Sender: TObject);
    procedure ree1Click(Sender: TObject);
    procedure Listbox1Data(Sender: TObject; Item: TListItem);
    procedure ListBox1KeyPress(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure A1Click(Sender: TObject);
    procedure Default1Click(Sender: TObject);
    procedure Showdeleted1Click(Sender: TObject);
    procedure Donoteditgames1Click(Sender: TObject);
    procedure Showtreeonopen1Click(Sender: TObject);
    procedure Brokenresult1Click(Sender: TObject);
    procedure Markallfiltered1Click(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure Select1Click(Sender: TObject);
    procedure RestoreStringFromTag1Click(Sender: TObject);
    procedure Replayallgames1Click(Sender: TObject);
    procedure Replacehandleinlisted1Click(Sender: TObject);
    procedure Timer4Timer(Sender:Tobject);
    procedure Showlag1Click(Sender: TObject);
    procedure Showtenth1Click(Sender: TObject);
    procedure Onlinedatabase1Click(Sender: TObject);
//    procedure Killshortgames1Click(Sender: TObject);



        PRIVATE
        WEngMenu: array[1..MAX_ENGINE] of Pmenuitem;
        EngineMenu: array[1..MAX_ENGINE] of Pmenuitem;
        PlEngMenu: array[1..MAX_ENGINE] of Pmenuitem;
        OptEngMenu: array [1..MAX_ENGINE] of Pmenuitem;
        DefEngMenu: array [1..MAX_ENGINE] of Pmenuitem;
        nEngineB : TColor;
        fEngineF :TFont;
        t2int:Cardinal;
        banno:boolean;
        setupallform:Tsetupallform;
        ProgBar: TProgBar;
        fLongThread:TWaitThread;

        procedure setEngmenu;
        procedure disableEngines;
        procedure EnableEngines;
        procedure RunEngine(i:integer);
        procedure StopEngine(i:integer);
        procedure setEngCaps;
        procedure createEngine(i:integer);
        procedure stopengines;
        procedure disableControls;
        procedure enableControls;
        procedure gotonexttest;
        function gettestar(var testar:TTestAr):integer;


        protected
        procedure WMEngineStop(var Message: TMessage); message WM_EngineStop;
        procedure WMEngineRun(var Message: TMessage); message WM_EngineRun;
        procedure WMEngineUnload(var Message: TMessage); message WM_EngineUnload;
        procedure WMEngineOptions(var Message: TMessage); message WM_EngineOptions;
        procedure WMEngineDefault(var Message: TMessage); message WM_EngineDefault;
        procedure WMEnginesuspend(var Message: TMessage); message WM_Enginesuspend;
        procedure WMEngineresume(var Message: TMessage); message WM_Engineresume;
        procedure WMEngineAnn(var Message:TMessage);message WM_EngineAnn;
        procedure WMEngineAnnStop(var Message:Tmessage);message WM_EngineAnnStop;
        procedure WMEngineTest(var Message:Tmessage);message WM_EngineTest;
        procedure WMEngineTestStop(var Message:Tmessage);message WM_EngineTestStop;
        procedure WMProgressUpdate(var Message:Tmessage);message WM_ProgressUpdate;
        procedure WMBpgnError(var message:tmessage);message WM_BPGNError;
        procedure WMBpgnInfo(var message:tmessage);message WM_BPGNInfo;
        procedure WMUpdateDB(var message:tmessage);message WM_UpdateDB;
        procedure WMupdateTree(var message:tmessage);message WM_UpdateTree;
        procedure WMCancelOp(var message:tmessage); message WM_Cancel_OP;

        { Private declarations }
        PUBLIC
        { Public declarations }
        EngStep:integer;
        bFlipBoard: Boolean;
        Engines:TEngineArr;
        EngLays:TEngineLayoutArr;
        DefEng:integer;
        EngOpt: TengineOptArr;
        humactmove:string;
        algactmove:string;
        actScore:integer;
        testscore:real;
        anneng:integer;
        bannno:boolean;
        batchmode: boolean;
        procedure showengine(i:integer);
        procedure resumeEngine(i:integer);
        procedure suspendEngine(i:integer);
        procedure ann(i:integer;mode:boolean);
        procedure test(i:integer);
        procedure annall(i:integer);
        procedure testall(i:integer);
        procedure playgamesound(i:integer);
        procedure closebpgnfile;
        procedure filtergamelist;
        procedure BoardControlSetup;
        procedure finalizesetup(Sender:Tobject);
        procedure cancelfileChanges;
        procedure raiseboards;
        procedure doLongRun(var Thread:TWaitThread; Handler:TThreadMethod; Data:pointer; Caption:string; bAllowAbort:boolean=false);
        FUNCTION  CheckGameSaveChanges(nIndex: Integer): Boolean;

    END;

VAR
    MainForm: TMainForm;

    BPGN:  TBPGN;
    BPGN1: TBPGN;

    nGameIndex:  longInt;
    nStartTime:  longInt;
    bIgnoreDisp: Boolean;
    bIgnoreMemo: Boolean;

    cOrgCaption: String;
    cParamFn:    String;

    bRunTimer: Boolean;
    nTick:     longint;
    nMaxTick:  longint;

    cExtractErrFn: String;
    bExtractErrFn: Boolean;
  FForm: TFForm;



IMPLEMENTATION
Uses ShellApi;

{$R *.DFM}
procedure ShowFile(cCaption,Cfile:string;var nL,nT,nW,nH:integer);
var ReadmeForm:TreadmeForm;
begin
 With ReadmeForm do
 try
  readmeForm:=TreadmeForm.Create(nil);
  setBounds(nL,nT,nW,nH);
  Setup(cCaption,cFile);
  showmodal;
  nL:=left; nw:=width; nT:=top; nH:=height;
 finally
  ReadmeForm.free;
 end;
end;

procedure ShowTxtFile(cCaption,Cfile:string;var nL,nT,nW,nH:integer);
var ReadmeForm:TViewTxtForm;
begin
 With ReadmeForm do
 try
  readmeForm:=TviewTxtForm.Create(nil);
  setBounds(nL,nT,nW,nH);
  Setup(cCaption,cFile);
  showmodal;
  nL:=left; nw:=width; nT:=top; nH:=height;
 finally
  ReadmeForm.free;
 end;
end;


procedure tmainform.closebpgnfile;
begin
 if banno then SendMessage(handle,WM_EngineAnnSTop,anneng,0);
 stopengines;
 ProcessBeg(Program_title+'(Updating file...)');
 BPGN.CloseBPGN(TRUE);
 processend;
 ResetAll(Self);
end;


procedure TMainForm.donothing1(Sender: TObject; VAR Key: Word; Shift: TShiftState);
var x:integer;
begin
 Wok(cCurrentInfo);
end;

procedure TMainForm.donothing(Sender: TObject);
var x:boolean;
begin
 if (Sender is TRepeatSpeedbtn) then with Sender as TRepeatSpeedbtn do
 begin
  x:=enabled;
  Enabled:=false;
 end;
 if (Sender is TRepeatlbbutton) then with Sender as TRepeatlbbutton do
 begin
  x:=enabled;
  Enabled:=false;
 end;

 Wok(cCurrentInfo);
 if (Sender is TRepeatSpeedbtn) then with Sender as TRepeatSpeedbtn do Enabled:=x;
 if (Sender is TRepeatLbButton) then with Sender as TRepeatLbbutton do Enabled:=x;
end;

procedure TMainForm.Donothing2(Sender: TObject; Button: TMouseButton;
                                    Shift: TShiftState; X, Y: Integer);
begin
x:=y+x;
end;

procedure TMainForm.Donothing3(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
 x:=x+y;
end;

procedure TmainForm.doabsolutelynothing(Sender:Tobject);
var x,y:integer;
begin
 x:=x+y;
end;

function getcapmove(s:string):string;
var ps:integer;
s1:string;
begin
 ps:=pos('(',s);
 s1:=copy(s,ps+1,length(s));
 ps:=pos(')',s1);
 Result:=copy(s1,1,ps-1);
end;


procedure Tmainform.stopengines;
var i:integer;
begin
 for i:=1 to MAX_ENGINE do
 begin
  if Engines[i].suspended then ResumeEngine(i);
  if engines[i].bengineon then stopengine(i);
 end;
end;

procedure TMainForm.setEngcaps;
var i:integer;
begin
 for i:=1 to MAX_ENGINE do
 if engines[i].exist then EngineForms[i].caption:=Engopt[i].comline;
end;

procedure TMainForm.WMEngineStop(var Message: TMessage);
var i:integer;
begin
 i:=integer(Message.WParam);
 StopEngine(i);
end;


procedure TMainForm.WMEngineAnnStop(var Message: Tmessage);
label stopan,skip;
var i,mode:integer;
begin
 i:=integer(Message.Wparam);
 mode:=integer(Message.Lparam);
 Timer2.enabled:=false;
 if ((mode=1)and batchmode) then
 begin
skip:
  if (gamelistform.ListBox1.ItemIndex<gamelistform.ListBox1.Items.Count-1)
  THEN begin
   gamelistform.ListBox1.ItemIndex:=gamelistform.listbox1.itemindex+1;
   If (not (Pos(Crazyhouse_ID,lowercase(Bpgn.getEvent(gamelistform.listbox1.itemindex+1)))>0)) then goto skip;
   mainForm.gamelistclick(nil);
   ann(i,false);
  end
  else goto stopan;
 end
 else begin
stopan:
  batchmode:=false;
  banno:=false;
  enablecontrols;
  EngineForms[i].StopAnn;
  stopEngine(i);
 end;
end;

procedure TMainForm.WMEngineTestStop(var Message: Tmessage);
label stopan,skip;
var i,mode:integer;
res:string;
begin
 i:=integer(Message.Wparam);
 mode:=integer(Message.Lparam);
 Timer2.enabled:=false;
 str(testscore:5:1,res);
 testscore:=0;
 if not navigateform.speedbutton3.enabled then
 commentform.memo1.lines.add('Test score for '+EngineForms[anneng].Engine.engname+' is '+res);
 if ((mode=1)and batchmode) then
 begin
skip:
  if (gamelistform.ListBox1.ItemIndex<gamelistform.ListBox1.Items.Count-1)
  THEN begin
   gamelistform.ListBox1.ItemIndex:=gamelistform.listbox1.itemindex+1;
   If (not (Pos(Crazyhouse_ID,lowercase(Bpgn.getEvent(gamelistform.listbox1.itemindex+1)))>0)) then goto skip;
   mainForm.gamelistclick(nil);
   test(i);
  end
  else goto stopan;
 end
 else begin
stopan:
  batchmode:=false;
  banno:=false;
  enablecontrols;
  EngineForms[i].Stoptest;
  stopEngine(i);
 end;
end;

procedure TmainForm.testall(i:integer);
begin
 IF (NOT checkgamesaveChanges(nGameIndex)) THEN
 begin
  SendMessage(handle,WM_ENGINETESTSTOP,i,0);
  EXIT;
 end;
 GameListForm.ListBox1.ItemIndex:=0;
 while (not (Pos(Crazyhouse_ID,lowercase(Bpgn.getEvent(bpgn.getfindex(gamelistform.listbox1.itemindex+1))))>0)) do
 begin
  if (gamelistform.listbox1.itemindex>(gamelistform.listbox1.Items.count-2)) then
  begin SendMessage(handle,WM_ENGINETestSTOP,i,0); exit; end;
  gamelistform.ListBox1.itemindex:=gamelistform.ListBox1.ItemIndex+1;
 end;
 batchmode:=true;
 Gamelistclick(GameListForm);
 test(i);
end;

procedure TmainForm.annall(i:integer);
begin
 IF (NOT checkgamesaveChanges(nGameIndex)) THEN
 begin
  SendMessage(handle,WM_ENGINEANNSTOP,i,0);
  EXIT;
 end;
 GameListForm.ListBox1.ItemIndex:=0;
 while (not (Pos(Crazyhouse_ID,lowercase(Bpgn.getEvent(bpgn.getfindex(gamelistform.listbox1.itemindex+1))))>0)) do
 begin
  if (gamelistform.listbox1.itemindex>(gamelistform.listbox1.Items.count-2)) then
  begin SendMessage(handle,WM_ENGINEAnnSTOP,i,0); exit; end;
  gamelistform.ListBox1.itemindex:=gamelistform.ListBox1.ItemIndex+1;
 end;
 batchmode:=true;
 Gamelistclick(GameListForm);
 ann(i,false);
end;

procedure TMainForm.Ann(i:integer;mode:boolean);
begin
 banno:=true;
 timer2.ontimer:=timer2timer;
 if not mode then begin
  navigateclick1(NavigateForm.speedbutton1);
  NavigateClick4(NavigateForm.speedbutton4);
 end;
 RunEngine(i);
 EngineForms[i].setAnnMode;
 anneng:=i;
 t2int:=round(Ini1.ini.nengineAT*1000);
 Timer2.interval:=t2int;
 actScore:=-INFTY;
 Timer2.enabled:=true;
end;

procedure TmainForm.test(i:integer);
begin
 stopengines;
 stopTimer;
 cCurrentInfo:='Engine is performing some special task, during which controls are disabled. To return to normal mode, press red led button in the engine window';
 disableControls;
 banno:=true;
 navigateclick1(NavigateForm.speedbutton1);
 gotonexttest;
 RunEngine(i);
 EngineForms[i].settestMode;
 anneng:=i; testscore:=0;
 t2int:=round(Ini1.ini.nenginetT*1000);
 Timer2.interval:=t2int;
 timer2.ontimer:=timer3timer;
 Timer2.enabled:=true;
end;

procedure TMainForm.WMEngineAnn(var Message: TMessage);
var i,cd:integer;
EngAnForm:TengAnform;
batchmode,ok,mode:boolean;
begin
 ok:=false;
 i:=integer(Message.WParam);
 try
  engAnForm:=TengAnForm.create(self);
  EngAnForm.SetupValues(Ini1.ini.nEngineAT,ini1.ini.nengineiT,Ini1.ini.nengineET,ini1.ini.bengineSM,EngineForms[i].engine.engname);
  EngAnForm.showmodal;
  if EngAnForm.bok then
  begin
    val(EngAnForm.edit1.text,Ini1.ini.nengineAt,cd);
    val(EngAnForm.edit2.text,Ini1.ini.nengineIt,cd);
    val(EngAnForm.edit3.text,Ini1.ini.nengineEt,cd);
    Ini1.ini.bEngineSM:=EngAnForm.checkbox2.checked;
  end;
  ok:=EngAnForm.Bok;
  mode:=EnganForm.checkbox1.checked;
  batchmode:=engAnForm.checkbox3.checked;
  finally
   EngAnForm.free;
  end;
  if ok then begin
   stopengines;
   stopTimer;
   cCurrentInfo:='Engine is performing some special task, during which controls are disabled. To return to normal mode, press red led button in the engine window';
   disableControls;
   if batchmode then annall(i)
   else ann(i,mode);//do some stuff;
  end;
end;

procedure TmainForm.WMEngineTest(var Message:Tmessage);
var
Engtestform:TengTestform;
ok,mode:boolean;
cd,i:integer;
begin
 ok:=false;
 i:=integer(Message.WParam);
 try
  engtestForm:=TengtestForm.create(self);
  EngtestForm.SetupValues(Ini1.ini.nEngineTT,EngineForms[i].engine.engname);
  EngtestForm.showmodal;
  if EngtestForm.bok then
  begin
    val(EngtestForm.edit1.text,Ini1.ini.nenginett,cd);
  end;
  ok:=EngtestForm.Bok;
  mode:=engtestForm.checkbox1.Checked;
 finally
  EngtestForm.free;
 end;
 if ok then
  if mode then testall(i) else test(i);//do some stuff;
end;

procedure TMainForm.WMEngineRun(var Message: TMessage);
var i:integer;
begin
 i:=integer(Message.Wparam);
 RunEngine(i);
end;

Procedure TMainForm.WMEngineUnload(var Message: TMessage);
var i:integer;
begin
 i:=integer(message.wparam);
 stopEngine(i);
 Engines[i].exist:=false;
 WengMenu[i]^.enabled:=false;
 if i=defeng then Weng.enabled:=false;
 englays[i].nEngineS:=TOOLBOX_WinStateToInt(EngineForms[i].WindowState);
 EngineForms[i].WindowState := TOOLBOX_IntToWinState(0);
 englays[i].nEngineL := EngineForms[i].Left;
 englays[i].nEngineT := EngineForms[i].Top;
 englays[i].nEngineW := EngineForms[i].Width;
 englays[i].nEngineH := EngineForms[i].Height;
 EngineForms[i].destroy;
end;

procedure TMainForm.WMEngineOptions(var Message: TMessage);
var i:integer;
begin
 i:=integer(Message.Wparam);
 Options_1Click(optEngMenu[i]^);
end;

Procedure TMainForm.WMEngineDefault(var Message: TMessage);
var i:integer;
begin
 i:=integer(Message.Wparam);
 SetAsDefault_1Click(defEngMenu[i]^);
end;

Procedure TMainForm.WMEngineSuspend(var Message: TMessage);
var i:integer;
begin
 i:=integer(Message.Wparam);
 SuspendEngine(i);
end;

Procedure TMainForm.WMEngineresume(var Message:TMessage);
var i:integer;
begin
 i:=integer(Message.Wparam);
 ResumeEngine(i);
end;

procedure TMainForm.SuspendEngine(i:integer);
begin
 engines[i].bengineOn:=false;
 engines[i].suspended:=true;
 plengmenu[i]^.enabled:=false;
 EngineForms[i].suspend;
 if i=defeng then NavigateForm.Button1.caption:=SJENG_RESUME;
end;

procedure TMainForm.ResumeEngine(i:integer);
begin
 Engines[i].bEngineOn:=true;
 plengmenu[i]^.enabled:=true;
 Engines[i].suspended:=false;
 EngineForms[i].resume;
 if i=defeng then NavigateForm.button1.caption:=SJENG_OFF;
end;


function getEngine(Sender:Tobject):integer;
var s:string;
i:integer;
begin
 with Sender as TMenuItem do s:=name;
 i:=ParseInt(s,'_');
 Result:=i;
end;

procedure TMainForm.disableEngines;
begin
 NavigateForm.Button1.enabled:=false;
 navigateForm.bitbtn1.enabled:=false;
{ for i:=1 to MAX_ENGINE do
 if engines[i].exist then EngineForms[i].bitbtn1.enabled:=false;}
end;

procedure TMainForm.enableEngines;
var i:integer;
begin
 navigateForm.button1.enabled:=true;
 navigateForm.bitbtn1.enabled:=true;
 for i:=1 to MAX_ENGINE do
 if engines[i].exist then EngineForms[i].bitbtn1.enabled:=true;
end;

procedure TMainForm.Setengmenu;
var i:integer;
begin
 EngineMenu[1]:=@Eng_1;
 EngineMenu[2]:=@Eng_2;
 EngineMenu[3]:=@Eng_3;
 EngineMenu[4]:=@Eng_4;
 EngineMenu[5]:=@Eng_5;
 WEngMenu[1]:=@Weng_1;
 WEngMenu[2]:=@Weng_2;
 WEngMenu[3]:=@Weng_3;
 WEngMenu[4]:=@Weng_4;
 WEngMenu[5]:=@Weng_5;
 PlEngMenu[1]:=@NavigateForm.Eng_1;
 PlEngMenu[2]:=@NavigateForm.Eng_2;
 PlEngMenu[3]:=@NavigateForm.Eng_3;
 PlEngMenu[4]:=@NavigateForm.Eng_4;
 PlEngMenu[5]:=@NavigateForm.Eng_5;
 OptEngMenu[1]:=@options_1;
 OptEngMenu[2]:=@options_2;
 OptEngMenu[3]:=@options_3;
 OptEngMenu[4]:=@options_4;
 OptEngMenu[5]:=@options_5;
 DefEngMenu[1]:=@SetasDefault_1;
 DefEngMenu[2]:=@SetasDefault_2;
 DefEngMenu[3]:=@SetasDefault_3;
 DefEngMenu[4]:=@SetasDefault_4;
 DefEngMenu[5]:=@SetasDefault_5;

 Enginemenu[defeng]^.checked:=true;

    for i:=1 to MAX_ENGINE do
    begin
     if engopt[i].name='' then
     begin
      Enginemenu[i]^.caption:=EMPTY_MENU;
      WengMenu[i]^.caption:=EMPTY_MENU;
      PlEngMenu[i]^.caption:=EMPTY_MENU;
      PlengMenu[i]^.enabled:=false;
     end
     else begin
      Enginemenu[i]^.caption:=engopt[i].name;
      Wengmenu[i]^.caption:=engopt[i].name;
      PlengMenu[i]^.caption:=engopt[i].name;
     end;
     if not engines[i].exist then WengMenu[i]^.enabled:=false;
    end;
    if not engines[defeng].exist then Weng.enabled:=false;

end;

Procedure TMainForm.ShowEngine(i:integer);
begin
 if engines[i].exist then
  begin
     IF (englays[i].nEngineS <> 2) THEN
     BEGIN
        EngineForms[i].SetBounds(englays[i].nEngineL, englays[i].nEngineT,
                  englays[i].nEngineW, englays[i].nEngineH);
     END;
     EngineForms[i].WindowState := TOOLBOX_IntToWinState(englays[i].nEngineS);
     EngineForms[i].show;
  end;
end;

PROCEDURE TMainForm.FormCreate(Sender: TObject);
VAR
    nI: Integer;
    i:integer;
BEGIN
    fEngineF:=TFont.create;

    GLOBAL_Init;
    engstep:=1;
    batchmode:=false;

    bg_file:=ini1.ini.bmpfile;
    usebitmap:=Ini1.ini.usebmp;
    usepiecebitmap:=ini1.ini.usepiecebmp;
    useboardbitmap:=ini1.ini.useboardbmp;
    piece_dir:=ini1.ini.p_Dir;
    board_dir:=ini1.ini.b_dir;
    piecesbg:=TbitMap.create;
    createpieces(pieces);
    createboard(boardbmp);

    if usebitmap then
    try
     piecesbg.loadfromFile(bg_file);
    except
     usebitmap:=false;
    end;
    if usepiecebitmap then
    usepiecebitmap:=readpieces(pieces,piece_dir);
    if useboardbitmap then
    useboardbitmap:=readboard(boardbmp,board_dir);


    for i:=1 to MAX_ENGINE do
    begin
     Engines[i].exist:=false;
     Engines[i].bEngineOn:=false;
     Engines[i].suspended:=false;
    end;

    defeng:=INI1.ini.defeng;
    engopt:=INI1.ini.engineopt;
    englays:=Ini1.ini.Enginelay;
    fEngineF:=Ini1.ini.fEngineF;
    nengineb:=ini1.ini.nEngineB;


    IF (INI1.INI.nMainS <> 2) THEN
    BEGIN
        SetBounds(INI1.INI.nMainL, INI1.INI.nMainT, INI1.INI.nMainW, INI1.INI.nMainH);
    END;
    WindowState := TOOLBOX_IntToWinState(INI1.INI.nMainS);
    Caption     := PROGRAM_TITLE;

    Board18.Checked   := INI1.INI.bAlwaysQ;
    Board19.Checked   := INI1.INI.bDragMove;
    Board20.Checked   := INI1.INI.bSmartMove;
    Board22.Checked   := INI1.INI.bActualBoard;
    Board27.Checked   := INI1.INI.bShowSOutline;
    Showlag1.Checked  := Ini1.INI.bShowLag;
    showtenth1.Checked:= ini1.INI.bShowTenth;

    GameList99.Checked:= INI1.INI.bShowGameNum;
    GameList98.Checked:= INI1.INI.bShowEvent;
    GameList4.Checked := INI1.INI.bShowRatings;
    GameList5.Checked := INI1.INI.bShowTime;
    GameList6.Checked := INI1.INI.bShowInc;
    GameList7.Checked := INI1.INI.bShowResult;
    Comment3.Checked  := INI1.INI.bCommentWR;
    WordWrap1.Checked := INI1.Ini.bEngineWR;
    ApplyGameSound1.Checked:=ini1.ini.bsoundon;
    Tecumseh:=INI1.Ini.tecumseh;

    Options9.Checked  := INI1.INI.bSaveOptionsOnExit;
    Options11.Checked := INI1.INI.bSaveLayoutsOnExit;
    bFlipBoard        := INI1.INI.bFlipBd;


    OpenDialog1.InitialDir := INI1.INI.cNewPath;
    OpenDialog2.InitialDir := INI1.INI.cOpenPath;
    OpenDialog3.InitialDir := INI1.INI.cDeletePath;
    OpenDialog4.InitialDir := INI1.INI.cRTFDir;
    OpenDialog5.InitialDir := INI1.INI.cTXTDir;


    BPGN := TBPGN.Create;
    BPGN.Init;
    bpgn.savedepth:=ini1.ini.savedepth;
    nGameIndex  := -1;
    nStartTime  := 0;
    bIgnoreDisp := TRUE;
    bIgnoreMemo := TRUE;

    cOrgCaption := '';

    cParamFn := '';
    FOR nI := 1 TO ParamCount DO
    BEGIN
        IF (FileExists(ParamStr(nI))) THEN
        BEGIN
            cParamFn := ParamStr(nI);
            BREAK;
        END;
    END;

    Timer1.Enabled  := FALSE;
    Timer1.Interval := 100;

    bRunTimer := FALSE;
    nTick     := 0;
    nMaxTick  := 0;

    cExtractErrFn := ExtractFilePath(ParamStr(0)) + EXTRACT_ERRFN;

    { Map shortcut keys }
    MapShortCutKeys;

    Glyphs[1]:=@Image1.picture;
    Glyphs[2]:=@image2.picture;
    Glyphs[3]:=@image3.picture;
    Glyphs[4]:=@image4.picture;
    Glyphs[5]:=@image5.picture;
    Glyphs[6]:=@image6.picture;
    glyphs[7]:=@image7.picture;
    glyphs[8]:=@image8.picture;
    glyphs[9]:=@image9.Picture;
    glyphs[10]:=@image10.Picture;

    bairdrop:=ini1.INI.bairdropini;
    airdrops1.checked:=bairdrop;
    Donoteditgames1.Checked:=ini1.INI.bROgames;
    showtreeonopen1.Checked:=ini1.INI.bOpenTree;

      Application.OnActivate:=formActivate;
  application.OnDeactivate:=formdeactivate;
  FileMode:=fmOpenRead;  //need this to allow opening read only files with reset

//    Bitbtn1.glyph:=glyphs[1]^.bitmap;

END;

procedure TMainForm.NavigateClick10(Sender: TObject);
var i:integer;
begin
 i:=getEngine(Sender);
 if Engines[i].bEngineOn then StopEngine(i)
 else RunEngine(i);
end;

procedure TMainForm.DisableControls;
Begin
    GameListForm.ListBox1.OnClick := donothing;
    gamelistform.ListBox1.OnKeyUp:=donothing1;
        NavigateForm.OnKeyDown            := donothing1;
        NavigateForm.SpeedButton1.OnClick := donothing;
        NavigateForm.SpeedButton2.OnClick := donothing;
        NavigateForm.SpeedButton3.OnClick := donothing;
        NavigateForm.SpeedButton4.OnClick := donothing;
        NavigateForm.SpeedButton5.OnClick := donothing;
        NavigateForm.SpeedButton6.OnClick := donothing;
        NavigateForm.SpeedButton7.OnClick := donothing;
        NavigateForm.SpeedButton8.OnClick := donothing;
        NavigateForm.Button1.OnClick:=donothing;
        NavigateForm.Eng_1.onClick:=donothing;
        NavigateForm.Eng_2.onClick:=donothing;
        NavigateForm.Eng_3.onClick:=donothing;
        NavigateForm.Eng_4.onClick:=donothing;
        NavigateForm.Eng_5.onClick:=donothing;
        NavigateForm.SpeedButton9.OnClick:=donothing;
        NavigateForm.Speedbutton10.onclick:=donothing;
        navigateform.speedbutton12.onclick:=donothing;
        navigateform.speedbutton13.onclick:=donothing;
        navigateform.LbButton1.OnClick:=donothing;


        Board1Form.SpeedButton1.OnClick := donothing;
        Board1Form.SpeedButton2.OnClick := donothing;
        Board1Form.SpeedButton3.OnClick := donothing;
        Board1Form.SpeedButton4.OnClick := donothing;

        Board2Form.SpeedButton1.OnClick := donothing;
        Board2Form.SpeedButton2.OnClick := donothing;
        Board2Form.SpeedButton3.OnClick := donothing;
        Board2Form.SpeedButton4.OnClick := donothing;

        NextMoveForm.ListBox1.OnClick := donothing;
        PrevMoveForm.ListBox1.OnClick := donothing;

        Board1Form.OnKeyDown             := donothing1;
        Board1Form.PaintBox1.OnMouseDown := donothing2;
        Board1Form.PaintBox1.OnMouseMove := donothing3;
        Board1Form.PaintBox1.OnMouseUp   := donothing2;

        Board2Form.OnKeyDown             := donothing1;
        Board2Form.PaintBox1.OnMouseDown := donothing2;
        Board2Form.PaintBox1.OnMouseMove := donothing3;
        Board2Form.PaintBox1.OnMouseUp   := donothing2;
        game0.enabled:=false;
        move0.enabled:=false;
        filter1.Enabled:=false;
        database1.Enabled:=false;
        utility0.Enabled:=false;
        engine2.Enabled:=false;
        file0.Enabled:=false;
        fform.Enabled:=false;

end;

procedure TMainForm.EnableControls;
Begin
       GameListForm.ListBox1.OnClick := GameListClick;
        gamelistform.ListBox1.OnKeyup:=listbox1keypress;
        NavigateForm.OnKeyDown            := NavigateFormKeyDown;
        NavigateForm.SpeedButton1.OnClick := NavigateClick1;
        NavigateForm.SpeedButton2.OnClick := NavigateClick2;
        NavigateForm.SpeedButton3.OnClick := NavigateClick3;
        NavigateForm.SpeedButton4.OnClick := NavigateClick4;
        NavigateForm.SpeedButton5.OnClick := NavigateClick5;
        NavigateForm.SpeedButton6.OnClick := NavigateClick6;
        NavigateForm.SpeedButton7.OnClick := file2click;
        NavigateForm.SpeedButton8.OnClick := NavigateClick8;
        navigateForm.speedbutton11.onclick:=ApplyGameSound1Click;
        if ini1.ini.bsoundon then navigateForm.speedbutton11.glyph:=imsound.Picture.bitmap
        else navigateForm.speedbutton11.glyph:=imnosound.Picture.bitmap;
        NavigateForm.Button1.OnClick:=NavigateClick9;
        NavigateForm.Eng_1.onClick:=NavigateClick10;
        NavigateForm.Eng_2.onClick:=NavigateClick10;
        NavigateForm.Eng_3.onClick:=NavigateClick10;
        NavigateForm.Eng_4.onClick:=NavigateClick10;
        NavigateForm.Eng_5.onClick:=NavigateClick10;
        NavigateForm.SpeedButton9.OnClick:=NavigateClick11;
        NavigateForm.SpeedButton10.onclick:=NavigateClick12;
        navigateform.speedbutton12.onclick:=navigateclick13;
        navigateform.speedbutton13.onclick:=navigateclick14;
        navigateform.LbButton1.OnClick:=button1click1;
        navigateform.BitBtn1.OnClick:=navigateform.BitBtn1Click;

        NextMoveForm.ListBox1.OnClick := NextMoveClick;
        PrevMoveForm.ListBox1.OnClick := PrevMoveClick;

        Board1Form.SpeedButton1.OnClick := Board1Click1;
        Board1Form.SpeedButton2.OnClick := Board1Click2;
        Board1Form.SpeedButton3.OnClick := Board1Click3;
        Board1Form.SpeedButton4.OnClick := Board1Click4;

        Board2Form.SpeedButton1.OnClick := Board2Click1;
        Board2Form.SpeedButton2.OnClick := Board2Click2;
        Board2Form.SpeedButton3.OnClick := Board2Click3;
        Board2Form.SpeedButton4.OnClick := Board2Click4;

        Board1Form.OnKeyDown             := Board1FormKeyDown;
        Board1Form.PaintBox1.OnMouseDown := Board1MouseDown;
        Board1Form.PaintBox1.OnMouseMove := Board1MouseMove;
        Board1Form.PaintBox1.OnMouseUp   := Board1MouseUp;

        Board2Form.OnKeyDown             := Board2FormKeyDown;
        Board2Form.PaintBox1.OnMouseDown := Board2MouseDown;
        Board2Form.PaintBox1.OnMouseMove := Board2MouseMove;
        Board2Form.PaintBox1.OnMouseUp   := Board2MouseUp;
        board1form.Label1.OnClick:=doabsolutelynothing;
        board1form.Label2.OnClick:=doabsolutelynothing;
        board2form.Label1.OnClick:=doabsolutelynothing;
        board2form.Label2.OnClick:=doabsolutelynothing;

        board1form.gonormal;
        board2form.gonormal;

        game0.enabled:=true;
        move0.enabled:=true;
        filter1.Enabled:=true;
        database1.Enabled:=true;
        utility0.Enabled:=true;
        engine2.Enabled:=true;
        file0.Enabled:=true;
        fform.Enabled:=true;

end;

PROCEDURE TMainForm.FormShow(Sender: TObject);
BEGIN
        fMainHandle:=Handle;
        gamelistform.ListBox1.OnData:=listbox1data;
        setengmenu;
        disableengines;
        setAsDefault_1Click(defengMenu[defeng]^);
        setengcaps;
        board1form.GoBoard1;
        board2form.GoBoard2;

        if ini1.INI.nNavigateSticky then navigateform.SetBounds(ini1.INI.nNavigateL,
        ini1.INI.nNavigateT,ini1.INI.nNavigateW,ini1.INI.nNavigateH);
        if ini1.INI.bNavigateSimple then navigateform.SpeedButton14click(sender)
        else Navigateform.setnavigatewidth;

        enableControls;
        CommentForm.Memo1.OnChange    := CommentChange;

        IF (Length(cParamFn) > 0) THEN
            OpenBPGNFile(Sender, cParamFn);

    with fform do begin
     Button1.OnClick:=filtergames;
     Button4.OnClick:=statgames;
     StringGrid1.OnMouseUp:=movestatclick;
     bfullstat:=ini1.INI.bfullstat;
     resetall;
    end;
    with ini1.INI do begin
     if bFilterSt then fform.CheckBox14.Checked:=true;
     fform.SetBounds(nfilterl,nfiltert,nfilterw,nfilterh);
    end;
    fullstats1.Checked:=fform.bfullstat;
END;

PROCEDURE TMainForm.FormClose(Sender: TObject; VAR Action: TCloseAction);
var i:integer;
BEGIN
    StopTimer;
    IF (NOT CheckFileSaveChanges(Sender, FALSE)) THEN
    BEGIN
        Action := caNone;
        EXIT;
    END;
    Action := caFree;

//    IF (INI1.INI.bSaveOptionsOnExit) THEN
    if options9.Checked then
    BEGIN
        Processbeg('Saving options');
        LoadIntoINI;
        INI1.INI.WriteOptions;
        processEnd;
    END;

    for i:=1 to MAX_ENGINE do
     if engines[i].exist then
     SendMessage(handle,WM_ENGINEUNLOAD,i,0);


//    IF (INI1.INI.bSaveLayoutsOnExit) THEN
    IF (options11.checked) THEN
    BEGIN
        processbeg('Saving layouts');
        MainForm.Visible := FALSE;

        INI1.INI.nMainS := TOOLBOX_WinStateToInt(WindowState);
        mainform.WindowState:=wsNormal;
        INI1.INI.nMainL := Left;
        INI1.INI.nMainT := Top;
        INI1.INI.nMainW := Width;
        INI1.INI.nMainH := Height;

        INI1.INI.nBoard1S := TOOLBOX_WinStateToInt(Board1Form.WindowState);
        Board1Form.WindowState := TOOLBOX_IntToWinState(0);
        INI1.INI.nBoard1L := Board1Form.Left;
        INI1.INI.nBoard1T := Board1Form.Top;
        INI1.INI.nBoard1W := Board1Form.Width;
        INI1.INI.nBoard1H := Board1Form.Height;

        INI1.INI.nBoard2S := TOOLBOX_WinStateToInt(Board2Form.WindowState);
        Board2Form.WindowState := TOOLBOX_IntToWinState(0);
        INI1.INI.nBoard2L := Board2Form.Left;
        INI1.INI.nBoard2T := Board2Form.Top;
        INI1.INI.nBoard2W := Board2Form.Width;
        INI1.INI.nBoard2H := Board2Form.Height;

        INI1.INI.nNavigateSticky := (navigateform.BorderStyle=bsNone);

        if not ini1.ini.nNavigateSticky then begin
          INI1.INI.nNavigateS := TOOLBOX_WinStateToInt(navigateform.WindowState);
          navigateform.WindowState := TOOLBOX_IntToWinState(0);
        end;
        INI1.INI.nNavigateL := NavigateForm.Left;
        INI1.INI.nNavigateT := NavigateForm.Top;
        ini1.ini.nNavigateW:=Navigateform.width;
        ini1.INI.nNavigateH:=navigateform.Height;
        ini1.ini.bNavigateSimple:=not navigateform.speedbutton13.Visible;
        if ini1.INI.nNavigateSticky then navigateform.Visible:=false;

        INI1.INI.nGameListS := TOOLBOX_WinStateToInt(GameListForm.WindowState);
        GameListForm.WindowState := TOOLBOX_IntToWinState(0);
        INI1.INI.nGameListL := GameListForm.Left;
        INI1.INI.nGameListT := GameListForm.Top;
        INI1.INI.nGameListW := GameListForm.Width;
        INI1.INI.nGameListH := GameListForm.Height;

        INI1.INI.nNextMoveS := TOOLBOX_WinStateToInt(NextMoveForm.WindowState);
        NextMoveForm.WindowState := TOOLBOX_IntToWinState(0);
        INI1.INI.nNextMoveL := NextMoveForm.Left;
        INI1.INI.nNextMoveT := NextMoveForm.Top;
        INI1.INI.nNextMoveW := NextMoveForm.Width;
        INI1.INI.nNextMoveH := NextMoveForm.Height;

        INI1.INI.nPrevMoveS := TOOLBOX_WinStateToInt(PrevMoveForm.WindowState);
        PrevMoveForm.WindowState := TOOLBOX_IntToWinState(0);
        INI1.INI.nPrevMoveL := PrevMoveForm.Left;
        INI1.INI.nPrevMoveT := PrevMoveForm.Top;
        INI1.INI.nPrevMoveW := PrevMoveForm.Width;
        INI1.INI.nPrevMoveH := PrevMoveForm.Height;

        INI1.INI.nCommentS := TOOLBOX_WinStateToInt(CommentForm.WindowState);
        CommentForm.WindowState := TOOLBOX_IntToWinState(0);
        INI1.INI.nCommentL := CommentForm.Left;
        INI1.INI.nCommentT := CommentForm.Top;
        INI1.INI.nCommentW := CommentForm.Width;
        INI1.INI.nCommentH := CommentForm.Height;

//        if ree1.Enabled then ini1.INI.bTreeAuto:=ftreeform.CheckBox13.checked;
        if fform.CheckBox13.Checked then begin
         fform.WindowState:=wsNormal;
//         ini1.INI.bTreeAuto:=true;
         fform.checkbox13.Checked:=false;
        end;
        INI1.INI.nFilterL := fForm.Left;
        INI1.INI.nFilterT := fForm.Top;
        INI1.INI.nFilterW := fForm.Width;
        INI1.INI.nfilterH := fForm.Height;
        ini1.INI.bFilterSt:= fform.CheckBox14.Checked;

        timer2.enabled:=false;
        for i:=1 to MAX_ENGINE do ini1.ini.enginelay[i]:=englays[i];

        INI1.INI.WriteLayout;
        processEnd;
    END
    else begin
     commentform.WindowState:=wSnormal;
     nextmoveform.WindowState:=wsNormal;
     gamelistform.WindowState:=wsNormal;
     prevmoveForm.WindowState:=wsNormal;
    end;

    piecesbg.free;
    freepieces(pieces);
    freeboard(boardbmp);

    BPGN.CloseBPGN(TRUE);
    BPGN.Destroy;
END;

PROCEDURE TMainForm.ResetAll(Sender: TObject);
BEGIN
    Caption := PROGRAM_TITLE;

    Board1Form.ResetAll;
    Board2Form.ResetAll;

    Board1Form.FormResize(Sender);
    Board2Form.FormResize(Sender);

    Board1Form.DisplayBoard;
    Board2Form.DisplayBoard;

    NavigateForm.ResetAll;
    GameListForm.ResetAll;
    NextMoveForm.ResetAll;
    PrevMoveForm.ResetAll;
    CommentForm.ResetAll;

    nGameIndex  := -1;
    nStartTime  := 0;
    bIgnoreDisp := TRUE;
    bIgnoreMemo := TRUE;
    if fform.CheckBox13.Checked then begin
//     ini1.INI.bTreeAuto:=true;
     fform.CheckBox13.Checked:=false;
     fform.Close;
    end;
END;

PROCEDURE TMainForm.SaveFileChanges(Sender: TObject; bAdjustGameList: Boolean);
var
bchangegamelist,bpackfile:boolean;
BEGIN
    bchangegamelist:=false;
    IF (BPGN.IsGameChange) THEN
    BEGIN
        ProcessBeg(PROGRAM_TITLE + ' (Saving...)');
        DoLongRun(fLongThread,bpgn.SaveGame_,@nGameIndex,'Saving game...');
        ProcessEnd;

    END;
    IF (BPGN.IsFileChange) THEN
    BEGIN

        ProcessBeg(PROGRAM_TITLE + ' (Packing file...)');
        gamelistform.clearall;
        DoLongRun(flongthread,bpgn.PackFile_,@bpackfile,'Packing file...');
        IF (bPackFile) THEN
        BEGIN
            ProcessEnd;
            ResetAll(Sender);
            EXIT;
        END;
        bpgn.SetFileChange(false);
        bpgn.resetfilter;
        bpgn.setfilter;
        ProcessEnd;
        IF (bAdjustGameList) THEN
        begin
         filtergamelist;
         if bpgn.nFilterCount<>bpgn.GetHeaderCount
         then gamelistform.SetCaption(GlFilter)
         else gamelistform.SetCaption(glNormal);
         gamelistform.ListBox1.Repaint;
         nGameIndex := -1;
         GameListClick(Sender);
        end;
    END;
END;

FUNCTION TMainForm.CheckFileSaveChanges(Sender: TObject; bAdjustGameList: Boolean): Boolean;
VAR
    nI: Integer;
    i:longint;
BEGIN
    IF (NOT BPGN.IsOpen) THEN
    BEGIN
        Result := TRUE;
        EXIT;
    END;
    IF ((NOT BPGN.IsGameChange) AND (NOT BPGN.IsFileChange)) THEN
    BEGIN
        Result := TRUE;
        EXIT;
    END;

    IF (BPGN.IsGameChange AND (NOT BPGN.IsFileChange)) THEN
    BEGIN
        if ((bpgn.bIsDB) and (donoteditgames1.Checked)) then
        begin
             bpgn.SetGameChange(false);
             exit;
        end;
        ni := wNoYesCancel('Save game changes?');
        IF (nI = 3) THEN
        BEGIN
            Result := FALSE;
            EXIT;
        END;
        IF (nI = 2) THEN
        BEGIN
            Result := TRUE;
            EXIT;
        END;
        SaveFileChanges(Sender, bAdjustGameList);
    END
    ELSE
    BEGIN
        nI := wYesNoCancel('Save file changes?');
        IF (nI = 3) THEN
        BEGIN
            Result := FALSE;
            EXIT;
        END;
        IF (nI = 2) THEN
        BEGIN
            cancelfilechanges;
            Result := TRUE;
            EXIT;
        END;
        SaveFileChanges(Sender, bAdjustGameList);
    END;

    Result := TRUE;
END;

PROCEDURE TMainForm.File0Click(Sender: TObject);
BEGIN
    File91.Enabled := BPGN.IsOpen;
    File3.Enabled  := BPGN.IsOpen;
    saveas1.Enabled:= Bpgn.Isopen;
    file10.Enabled:=bpgn.IsOpen;
    file11.Enabled:=bpgn.IsOpen;
END;

PROCEDURE TMainForm.File1Click(Sender: TObject);
VAR
    cFn: String;
BEGIN
    StopTimer;
    IF (NOT CheckFileSaveChanges(Sender, TRUE)) THEN
        EXIT;

    IF (OpenDialog1.Execute) THEN
    BEGIN
      if banno then SendMessage(handle,WM_EngineAnnSTop,anneng,0);
     stopengines;
        cFn := ChangeFileExt(OpenDialog1.Filename, '.' + BPGN_EXT);
        OpenDialog1.InitialDir := ExtractFilePath(cFn);

        IF (BPGN.IsOpen) THEN
        BEGIN
            BPGN.CloseBPGN(TRUE);
            ResetAll(Sender);
        END;

        IF (NOT FileExists(cFn)) THEN
        BEGIN
            ProcessBeg(PROGRAM_TITLE + ' (Creating...)');
            BPGN.NewFile(cFn, DEFAULT_WHITEA, DEFAULT_BLACKA, DEFAULT_WHITEB, DEFAULT_BLACKB);
            ProcessEnd;
        END;

        OpenBPGNFile(Sender, cFn);
    END;
END;

PROCEDURE TMainForm.File2Click(Sender: TObject);
VAR
    cFn: String;
BEGIN
    StopTimer;
    IF (NOT CheckFileSaveChanges(Sender, TRUE)) THEN
        EXIT;

    IF (OpenDialog2.Execute) THEN
    BEGIN
        if banno then SendMessage(handle,WM_EngineAnnSTop,anneng,0);
        stopengines;
        cFn := OpenDialog2.Filename;
        OpenDialog2.InitialDir := ExtractFilePath(cFn);
        OpenBPGNFile(Sender, cFn);
        if ((showtreeonopen1.Checked) and (bpgn.bIsDB)) then begin
         ree1click(self);
        end;
    END;
END;

PROCEDURE TMainForm.File91Click(Sender: TObject);
BEGIN
    StopTimer;
    SaveFileChanges(Sender, TRUE);
END;

PROCEDURE TMainForm.File3Click(Sender: TObject);
BEGIN
    StopTimer;
    IF (NOT CheckFileSaveChanges(Sender, FALSE)) THEN
        EXIT;
    BPGN.CloseBPGN(TRUE);
    ResetAll(Sender);
    if banno then SendMessage(handle,WM_EngineAnnSTop,anneng,0);
    stopengines;
END;


PROCEDURE TMainForm.File5Click(Sender: TObject);
VAR
    cFn: String;
BEGIN
    StopTimer;
    IF (OpenDialog3.Execute) THEN
    BEGIN
        cFn := OpenDialog3.Filename;
        OpenDialog3.InitialDir := ExtractFilePath(cFn);

        IF (BPGN.IsOpen) THEN
        BEGIN
            IF (compareText(cFn, BPGN.GetBPGNFilename)=0) THEN
            BEGIN
                BPGN.CloseBPGN(TRUE);
                ResetAll(Sender);
            END;
        END;
        if banno then SendMessage(handle,WM_EngineAnnSTop,anneng,0);
        stopengines;
        ProcessBeg(PROGRAM_TITLE + ' (Deleting...)');

        if fileExists(cfn) then DeleteFile(cFn);

        if UPpercase(extractfileExt(cfn))='.'+BPGN_EXT then
        cfn:=changefileExt(cfn,db_header_Ext) ;

        if lowercase(ExtractfileExt(cfn))=DB_HEADER_EXT then
        begin
         if fileExists(cfn) then deletefile(cfn);

         cfn:=changefileext(cfn,DB_WHITE_EXT);
         if fileExists(cfn) then deletefile(cfn);

         cfn:=changefileext(cfn,db_Black_EXt);
         if fileExists(cfn) then deletefile(cfn);
        end;
        ProcessEnd;
    END;
END;


PROCEDURE TMainForm.File7Click(Sender: TObject);
BEGIN
    Close;
END;

PROCEDURE TMainForm.Game0Click(Sender: TObject);
BEGIN
    Game24.Enabled := bpgn.GameIsValid(nGameIndex);
    Game25.Enabled := BPGN.GameIsValid(nGameIndex);
    select1.Enabled:= bpgn.GameIsValid(nGameIndex);
    game26.Enabled := (BPGN.IsOpen AND (NOT BPGN.IsEmpty));
    game27.enabled := ((game26.enabled) AND
    (not iscurrentbpgn(ini1.INI.cWorkBpgn,bpgn.GetBPGNFilename)));

    Game3.Enabled  := TRUE;
    Game4.Enabled  := (Game26.Enabled AND NavigateForm.SpeedButton6.Enabled);
    Game5.Enabled  := (Game26.Enabled AND NavigateForm.SpeedButton6.Enabled);


    Game22.Enabled := (BPGN.IsOpen AND (NOT BPGN.IsEmpty))and (not ((bpgn.bIsDB) and (donoteditgames1.Checked)));
    Game20.Enabled := (BPGN.IsOpen AND BPGN.CanAddNewGame) and game22.Enabled;
    Game21.Enabled := (game22.Enabled AND BPGN.CanAddNewGame);
    newfromposition1.Enabled:=game20.Enabled;

    Game7.Enabled  := game22.enabled;
    Game8.Enabled  := game22.enabled;
    Game9.Enabled  := game22.enabled;
    Game10.Enabled := game22.enabled;
    Game11.Enabled := game22.enabled;
    Game12.Enabled := game22.enabled;
    Game13.Enabled := game22.enabled;
    Game15.Enabled := game22.enabled;
    Game16.Enabled := game22.enabled;
    Game17.Enabled := game22.enabled;
    Game19.Enabled := game26.Enabled;
END;

PROCEDURE TMainForm.Game20Click(Sender: TObject);
var rec:tbpgnrec;
BEGIN
    StopTimer;
    IF (NOT checkgamesaveChanges(nGameIndex)) THEN
        EXIT;

    BPGNrecFillempty(@rec);
    BPGN.AppendNewGame(@rec);
    GameListForm.AddItem;
    GameListForm.ListBox1.Enabled   := TRUE;
    GameListForm.ListBox1.ItemIndex := GameListForm.ListBox1.Items.Count - 1;
    GameListClick(Sender);
END;

PROCEDURE TMainForm.Game21Click(Sender: TObject);
BEGIN
    StopTimer;
    IF (NOT checkgamesaveChanges(nGameIndex)) THEN
        EXIT;

    BPGN.DuplicateGame(nGameIndex);
    GameListForm.AddItem;
    GameListForm.ListBox1.Enabled   := TRUE;
    GameListForm.ListBox1.ItemIndex := GameListForm.ListBox1.Items.Count - 1;
    GameListClick(Sender);
END;

PROCEDURE TMainForm.Game22Click(Sender: TObject);
BEGIN
    StopTimer;
    ProcessBeg(PROGRAM_TITLE + ' (Saving...)');
    DoLongRun(fLongThread,bpgn.SaveGame_,@nGameIndex,'Saving game...');
    ProcessEnd;
END;

PROCEDURE TMainForm.Game24Click(Sender: TObject);
BEGIN
    StopTimer;
    BPGN.SetTag(nGameIndex, TAG_DELETE);
    gamelistform.listbox1.Refresh;
END;

PROCEDURE TMainForm.Game25Click(Sender: TObject);
BEGIN
    StopTimer;
    BPGN.SetTag(nGameIndex, TAG_OK);
    gamelistform.listbox1.Refresh;
END;

PROCEDURE TMainForm.Game3Click(Sender: TObject);
BEGIN
    NavigateClick5(Sender);
END;

PROCEDURE TMainForm.Game4Click(Sender: TObject);
BEGIN
    NavigateClick6(Sender);
END;

PROCEDURE TMainForm.Game5Click(Sender: TObject);
BEGIN
    NavigateClick7(Sender);
END;

PROCEDURE TMainForm.Game7Click(Sender: TObject);
var eventform:teventform;
BEGIN
    { Edit event }
    StopTimer;
    try
    eventform:=teventform.create(self);
    EventForm.Caption   := 'Edit event';
    EventForm.cOrgEvent := BPGN.GetEvent(nGameIndex);
    EventForm.cNewEvent := EventForm.cOrgEvent;

    EventForm.SetupValues;
    EventForm.ShowModal;

    IF (EventForm.bOK) THEN
    BEGIN
        BPGN.SetEvent(nGameIndex, EventForm.cNewEvent);

        if ini1.INI.bShowEvent then gamelistform.listbox1.Repaint;
    END;
    finally eventform.free;
    end;
If (Pos(Crazyhouse_ID,lowercase(Bpgn.getEvent(nGameIndex)))>0) then
enableengines
else disableengines;
END;

PROCEDURE TMainForm.Game8Click(Sender: TObject);
var siteform:Tsiteform;
BEGIN
    { Edit site }
    StopTimer;
    try
    siteform:=tsiteform.create(self);
    SiteForm.Caption   := 'Edit site';
    SiteForm.cOrgSite := BPGN.GetSite(nGameIndex);
    SiteForm.cNewSite := SiteForm.cOrgSite;

    SiteForm.SetupValues;
    SiteForm.ShowModal;

    IF (SiteForm.bOK) THEN
        BPGN.SetSite(nGameIndex, SiteForm.cNewSite);
    finally
     siteform.free;
    end;
END;

PROCEDURE TMainForm.Game9Click(Sender: TObject);
var dateform:tdateform;
BEGIN
    { Edit date }
    StopTimer;
    try
    dateform:=tdateform.create(self);
    DateForm.Caption  := 'Edit date';
    DateForm.cOrgDate := BPGN.GetDate(nGameIndex);
    DateForm.cNewDate := DateForm.cOrgDate;

    DateForm.SetupValues;
    DateForm.ShowModal;

    IF (DateForm.bOK) THEN
        BPGN.SetDate(nGameIndex, DateForm.cNewDate);
    finally dateform.free;
    end;
END;

PROCEDURE TMainForm.Game10Click(Sender: TObject);
BEGIN
    Name1.Caption := 'Board #1 white: ' + BPGN.GetWhiteA(nGameIndex);
    Name2.Caption := 'Board #1 black: ' + BPGN.GetBlackA(nGameIndex);
    Name4.Caption := 'Board #2 white: ' + BPGN.GetWhiteB(nGameIndex);
    Name5.Caption := 'Board #2 black: ' + BPGN.GetBlackB(nGameIndex);
END;

PROCEDURE TMainForm.Name1Click(Sender: TObject);
var HandleForm:ThandleForm;
BEGIN
    StopTimer;
    try
    handleForm:=ThandleForm.create(self);
    HandleForm.Caption    := HANDLE_TITLE + ' (board #1 white)';
    HandleForm.cOrgHandle := BPGN.GetWhiteA(nGameIndex);
    HandleForm.cNewHandle := HandleForm.cOrgHandle;

    HandleForm.SetupValues;
    HandleForm.ShowModal;

    IF (HandleForm.bOK) THEN
    BEGIN
        BPGN.SetWhiteA(nGameIndex, HandleForm.cNewHandle);

        Board1Form.cWHandle := BPGN.GetWhiteAHandle(nGameIndex);

        Board1Form.LoadHandle;
        Board1Form.FormResize(Sender);
        Board1Form.DisplayBoard;

        gamelistform.listbox1.Repaint;
    END;
    finally handleForm.free;
    end;
END;

PROCEDURE TMainForm.Name2Click(Sender: TObject);
var HandleForm:ThandleForm;
BEGIN
    StopTimer;
    try
    HandleForm:=thandleForm.create(self);
    HandleForm.Caption    := HANDLE_TITLE + ' (board #1 black)';
    HandleForm.cOrgHandle := BPGN.GetBlackA(nGameIndex);
    HandleForm.cNewHandle := HandleForm.cOrgHandle;

    HandleForm.SetupValues;
    HandleForm.ShowModal;

    IF (HandleForm.bOK) THEN
    BEGIN
        BPGN.SetBlackA(nGameIndex, HandleForm.cNewHandle);

        Board1Form.cBHandle := BPGN.GetBlackAHandle(nGameIndex);

        Board1Form.LoadHandle;
        Board1Form.FormResize(Sender);
        Board1Form.DisplayBoard;

        gamelistform.listbox1.Repaint;
    END;
    finally handleform.free;
    end;
END;

PROCEDURE TMainForm.Name4Click(Sender: TObject);
var HandleForm:ThandleForm;
BEGIN
    StopTimer;
    try
    handleForm:=ThandleForm.create(self);
    HandleForm.Caption    := HANDLE_TITLE + ' (board #2 white)';
    HandleForm.cOrgHandle := BPGN.GetWhiteB(nGameIndex);
    HandleForm.cNewHandle := HandleForm.cOrgHandle;

    HandleForm.SetupValues;
    HandleForm.ShowModal;

    IF (HandleForm.bOK) THEN
    BEGIN
        BPGN.SetWhiteB(nGameIndex, HandleForm.cNewHandle);

        Board2Form.cWHandle := BPGN.GetWhiteBHandle(nGameIndex);

        Board2Form.LoadHandle;
        Board2Form.FormResize(Sender);
        Board2Form.DisplayBoard;
        gamelistform.listbox1.Repaint;
    END;
    finally handleForm.free;
    end;
END;

PROCEDURE TMainForm.Name5Click(Sender: TObject);
var HandleForm:ThandleForm;
BEGIN
    StopTimer;
    try
    HandleForm:=ThandleForm.Create(self);
    HandleForm.Caption    := HANDLE_TITLE + ' (board #2 black)';
    HandleForm.cOrgHandle := BPGN.GetBlackB(nGameIndex);
    HandleForm.cNewHandle := HandleForm.cOrgHandle;

    HandleForm.SetupValues;
    HandleForm.ShowModal;

    IF (HandleForm.bOK) THEN
    BEGIN
        BPGN.SetBlackB(nGameIndex, HandleForm.cNewHandle);

        Board2Form.cBHandle := BPGN.GetBlackBHandle(nGameIndex);

        Board2Form.LoadHandle;
        Board2Form.FormResize(Sender);
        Board2Form.DisplayBoard;
        gamelistform.listbox1.Repaint;
    END;
    finally handleForm.free;
    end;
END;

PROCEDURE TMainForm.Game11Click(Sender: TObject);
BEGIN
    Rating1.Caption := 'Board #1 white: ' + BPGN.GetRWA(nGameIndex);
    Rating2.Caption := 'Board #1 black: ' + BPGN.GetRBA(nGameIndex);
    Rating4.Caption := 'Board #2 white: ' + BPGN.GetRWB(nGameIndex);
    Rating5.Caption := 'Board #2 black: ' + BPGN.GetRBB(nGameIndex);
END;

PROCEDURE TMainForm.Rating1Click(Sender: TObject);
var ratingForm:Tratingform;
BEGIN
    StopTimer;
    try
    ratingForm:=tratingform.create(self);
    RatingForm.Caption    := RATING_TITLE + ' (board #1 white)';
    RatingForm.cOrgRating := BPGN.GetRWA(nGameIndex);
    RatingForm.cNewRating := RatingForm.cOrgRating;

    RatingForm.SetupValues;
    RatingForm.ShowModal;

    IF (RatingForm.bOK) THEN
    BEGIN
        BPGN.SetRWA(nGameIndex, RatingForm.cNewRating);

        Board1Form.cWHandle := BPGN.GetWhiteAHandle(nGameIndex);

        Board1Form.LoadHandle;
        Board1Form.FormResize(Sender);
        Board1Form.DisplayBoard;

        gamelistform.listbox1.Repaint;
    END;
    finally ratingform.free;
    end;
END;

PROCEDURE TMainForm.Rating2Click(Sender: TObject);
var ratingform:Tratingform;
BEGIN
    StopTimer;
    try
    ratingForm:=tratingform.create(self);
    RatingForm.Caption    := RATING_TITLE + ' (board #1 black)';
    RatingForm.cOrgRating := BPGN.GetRBA(nGameIndex);
    RatingForm.cNewRating := RatingForm.cOrgRating;

    RatingForm.SetupValues;
    RatingForm.ShowModal;

    IF (RatingForm.bOK) THEN
    BEGIN
        BPGN.SetRBA(nGameIndex, RatingForm.cNewRating);

        Board1Form.cBHandle := BPGN.GetBlackAHandle(nGameIndex);

        Board1Form.LoadHandle;
        Board1Form.FormResize(Sender);
        Board1Form.DisplayBoard;
        gamelistform.listbox1.Repaint;
    END;
    finally ratingform.free;
    end;
END;

PROCEDURE TMainForm.Rating4Click(Sender: TObject);
var ratingform:tratingform;
BEGIN
    StopTimer;
    try
     ratingForm:=tratingform.create(self);

    RatingForm.Caption    := RATING_TITLE + ' (board #2 white)';
    RatingForm.cOrgRating := BPGN.GetRWB(nGameIndex);
    RatingForm.cNewRating := RatingForm.cOrgRating;

    RatingForm.SetupValues;
    RatingForm.ShowModal;

    IF (RatingForm.bOK) THEN
    BEGIN
        BPGN.SetRWB(nGameIndex, RatingForm.cNewRating);

        Board2Form.cWHandle := BPGN.GetWhiteBHandle(nGameIndex);

        Board2Form.LoadHandle;
        Board2Form.FormResize(Sender);
        Board2Form.DisplayBoard;
        gamelistform.listbox1.Repaint;
    END;
    finally ratingform.free;
    end;
END;

PROCEDURE TMainForm.Rating5Click(Sender: TObject);
var ratingform:tratingform;
BEGIN
    StopTimer;
    try
     ratingForm:=tratingform.create(self);
    RatingForm.Caption    := RATING_TITLE + ' (board #2 black)';
    RatingForm.cOrgRating := BPGN.GetRBB(nGameIndex);
    RatingForm.cNewRating := RatingForm.cOrgRating;

    RatingForm.SetupValues;
    RatingForm.ShowModal;

    IF (RatingForm.bOK) THEN
    BEGIN
        BPGN.SetRBB(nGameIndex, RatingForm.cNewRating);

        Board2Form.cBHandle := BPGN.GetBlackBHandle(nGameIndex);

        Board2Form.LoadHandle;
        Board2Form.FormResize(Sender);
        Board2Form.DisplayBoard;
        gamelistform.listbox1.Repaint;
    END;
    finally ratingform.free;
    end;
END;

PROCEDURE TMainForm.Game12Click(Sender: TObject);
var timeform:ttimeform;
BEGIN
    { Edit time control }
    StopTimer;
    try
    timeform:=ttimeform.create(self);
    TimeForm.Caption   := 'Edit time control';
    TimeForm.cOrgTimeC := BPGN.GetTimeC(nGameIndex);
    TimeForm.cNewTimeC := TimeForm.cOrgTimeC;

    TimeForm.SetupValues;
    TimeForm.ShowModal;

    IF (TimeForm.bOK) THEN
    BEGIN
        BPGN.SetTimeC(nGameIndex, TimeForm.cNewTimeC);

        nStartTime := 1000*TOOLBOX_GetTimeVal(BPGN.GetTimeC(nGameIndex));
        if ini1.INI.bShowTime then  gamelistform.listbox1.Repaint;

        BPGN.SetStartTime(nStartTime);

        LoadRecord;
    END;
    finally timeform.free;
    end;
END;

PROCEDURE TMainForm.Game13Click(Sender: TObject);
var resultForm:tresultform;
BEGIN
    { Edit result }
    StopTimer;
    try
    resultform:=tresultform.create(self);
    ResultForm.Caption := 'Edit result';
    ResultForm.cOrgRC  := BPGN.GetReason;
    ResultForm.cNewR1  := TOOLBOX_GetReason(ResultForm.cOrgRC);
    ResultForm.cNewR2  := TOOLBOX_GetReasult(ResultForm.cOrgRC);

    ResultForm.SetupValues;
    ResultForm.ShowModal;

    IF (ResultForm.bOK) THEN
    BEGIN
        BPGN.SetReason(nGameIndex,ResultForm.cNewR1 + ' ' + ResultForm.cNewR2);
        BPGN.SetResult(nGameIndex, ResultForm.cNewR2);
        if ini1.INI.bShowResult then gamelistform.listbox1.Repaint;
        LoadRecord;
    END;
    finally resultform.free;
    end;
END;

PROCEDURE TMainForm.Game15Click(Sender: TObject);
BEGIN
    StopTimer;
    IF (NOT wYesNo('Remove all annotations?')) THEN
        EXIT;

    ProcessBeg(PROGRAM_TITLE + ' (Removing all annotations...)');
    BPGN.DeleteAllAnnotations;
    ProcessEnd;

    IF (BPGN.IsGameChange) THEN
        LoadRecord;
END;

PROCEDURE TMainForm.Game16Click(Sender: TObject);
BEGIN
    StopTimer;
    IF (NOT wYesNo('Remove all comments?')) THEN
        EXIT;

    ProcessBeg(PROGRAM_TITLE + ' (Removing all comments...)');
    BPGN.DeleteAllComments;
    ProcessEnd;

    IF (BPGN.IsGameChange) THEN
        LoadRecord;
END;

PROCEDURE TMainForm.Game17Click(Sender: TObject);
BEGIN
    StopTimer;
    IF (NOT wYesNo('Remove all displays?')) THEN
        EXIT;

    ProcessBeg(PROGRAM_TITLE + ' (Removing all displays...)');
    BPGN.DeleteAllDisplays;
    ProcessEnd;

    IF (BPGN.IsGameChange) THEN
        LoadRecord;
END;

PROCEDURE TMainForm.Game19Click(Sender: TObject);
VAR
    cT: String;
    viewactf:tviewactualform;
BEGIN
    StopTimer;
    BPGN.GetActualGameText(nGameIndex);
    WindowsLineBreaks(BPGN.cActualGame);

    cT := BPGN.GetHeaderItem(nGameIndex);
    cT := Copy(cT, 3, Length(cT) - 2);

    try
    viewactf:=tviewactualform.create(self);
    VIEWACTf.ResetAll;
    VIEWACTf.Caption    := 'View actual text' + ' (' + cT + ')';
    VIEWACTf.Memo1.Text := BPGN.cActualGame;
    VIEWACTf.ShowModal;

    INI1.INI.nVATL := VIEWACTf.Left;
    INI1.INI.nVATT := VIEWACTf.Top;
    INI1.INI.nVATW := VIEWACTf.Width;
    INI1.INI.nVATH := VIEWACTf.Height;
    finally viewactf.free;
    end;

END;

PROCEDURE TMainForm.Move0Click(Sender: TObject);
BEGIN
    Move1.Enabled := (BPGN.IsOpen AND (NOT BPGN.IsEmpty) AND (NOT BPGN.IsAtStartingPosition))
    and (not ((bpgn.bIsDB) and (donoteditgames1.Checked)));
    Move3.Enabled := Move1.Enabled;
    Move5.Enabled := Move1.Enabled;
END;

PROCEDURE TMainForm.Move1Click(Sender: TObject);
var clockform:tclockform;
BEGIN
    StopTimer;
    try
    clockform:=tclockform.create(self);
    CASE (BPGN.GetCurrentBoard) OF
        'A':BEGIN
                ClockForm.Caption   := CLOCK_TITLE + ' (board #1 white)';
                ClockForm.cOrgClock := Board1Form.cWTime;
            END;
        'a':BEGIN
                ClockForm.Caption   := CLOCK_TITLE + ' (board #1 black)';
                ClockForm.cOrgClock := Board1Form.cBTime;
            END;
        'B':BEGIN
                ClockForm.Caption   := CLOCK_TITLE + ' (board #2 white)';
                ClockForm.cOrgClock := Board2Form.cWTime;
            END;
        'b':BEGIN
                ClockForm.Caption   := CLOCK_TITLE + ' (board #2 black)';
                ClockForm.cOrgClock := Board2Form.cBTime;
            END;
    ELSE
        EXIT;
    END;
    ClockForm.cNewClock := ClockForm.cOrgClock;

    ClockForm.SetupValues;
    ClockForm.ShowModal;

    CASE (BPGN.GetCurrentBoard) OF
        'A':BEGIN
                Board1Form.cWTime := ClockForm.cNewClock;
                Board1Form.LoadTime;
                Board1Form.FormResize(Sender);
                Board1Form.DisplayBoard;
                BPGN.SetCurrentBoard1WTime(ClockForm.nNewClock);
            END;
        'a':BEGIN
                Board1Form.cBTime := ClockForm.cNewClock;
                Board1Form.LoadTime;
                Board1Form.FormResize(Sender);
                Board1Form.DisplayBoard;
                BPGN.SetCurrentBoard1BTime(ClockForm.nNewClock);
            END;
        'B':BEGIN
                Board2Form.cWTime := ClockForm.cNewClock;
                Board2Form.LoadTime;
                Board2Form.FormResize(Sender);
                Board2Form.DisplayBoard;
                BPGN.SetCurrentBoard2WTime(ClockForm.nNewClock);
            END;
        'b':BEGIN
                Board2Form.cBTime := ClockForm.cNewClock;
                Board2Form.LoadTime;
                Board2Form.FormResize(Sender);
                Board2Form.DisplayBoard;
                BPGN.SetCurrentBoard2BTime(ClockForm.nNewClock);
            END;
    END;
    finally clockform.free;
    end;
END;

PROCEDURE TMainForm.Move3Click(Sender: TObject);
VAR
    cB: Char;
    cTmp: String;
    AnnotateForm:TannotateForm;
BEGIN
    StopTimer;
    try
    AnnotateForm:=TannotateForm.Create(self);
    AnnotateForm.Caption := 'Annotate';
    AnnotateForm.cMove   := IntToStr(BPGN.GetCurrentNumber) + BPGN.GetCurrentBoard + '. ' + BPGN.GetCurrentMove;
    AnnotateForm.cAnno   := BPGN.GetCurrentAnno;

    AnnotateForm.SetupValues;
    AnnotateForm.ShowModal;

    IF (AnnotateForm.bOK) THEN
    BEGIN
        BPGN.SetCurrentAnno(AnnotateForm.cAnno);

        cB   := UpCase(BPGN.GetCurrentBoard);
        cTmp := IntToStr(BPGN.GetCurrentNumber) + BPGN.GetCurrentBoard + '. ' +
                BPGN.GetCurrentMove + Trim(BPGN.GetCurrentAnno);

        IF (cB = 'A') THEN
        BEGIN
            IF (CompareStr(Copy(Board1Form.Caption,1,11),'Board #1 ({') <> 0) THEN
            BEGIN
                BPGN.SetCurrentBoard1Caption('Board #1 (' + cTmp + ')');
                Board1Form.Caption := BPGN.GetCurrentBoard1Caption;
            END;
        END
        ELSE
        IF (cB = 'B') THEN
        BEGIN
            IF (CompareStr(Copy(Board2Form.Caption,1,11),'Board #2 ({') <> 0) THEN
            BEGIN
                BPGN.SetCurrentBoard2Caption('Board #2 (' + cTmp + ')');
                Board2Form.Caption := BPGN.GetCurrentBoard2Caption;
            END;
        END;
    END;
    finally annotateform.free;
    end;
END;

PROCEDURE TMainForm.Move5Click(Sender: TObject);
BEGIN
    StopTimer;
    IF (NOT wYesNo('Delete move?')) THEN
        EXIT;

    ProcessBeg(PROGRAM_TITLE + ' (Deleting move...)');
    BPGN.DeleteCurrentMove;
    ProcessEnd;

    LoadRecord;
END;

PROCEDURE TMainForm.Options0Click(Sender: TObject);
BEGIN
    { Options }
END;


PROCEDURE TMainForm.Board18Click(Sender: TObject);
BEGIN
    StopTimer;
    Board18.Checked               := NOT Board18.Checked;
    Board1Form.Board.bAlwaysQueen := Board18.Checked;
    Board2Form.Board.bAlwaysQueen := Board18.Checked;
END;

PROCEDURE TMainForm.Board19Click(Sender: TObject);
BEGIN
    StopTimer;
    Board19.Checked            := NOT Board19.Checked;
    Board1Form.Board.bDragMove := Board19.Checked;
    Board2Form.Board.bDragMove := Board19.Checked;
END;

PROCEDURE TMainForm.Board20Click(Sender: TObject);
BEGIN
    StopTimer;
    Board20.Checked             := NOT Board20.Checked;
    Board1Form.Board.bSmartMove := Board20.Checked;
    Board2Form.Board.bSmartMove := Board20.Checked;
END;

PROCEDURE TMainForm.Board22Click(Sender: TObject);
BEGIN
    StopTimer;
    Board22.Checked    := NOT Board22.Checked;
    Board1Form.bActual := Board22.Checked;
    Board2Form.bActual := Board22.Checked;
    Board1Form.FormResize(Sender);
    Board2Form.FormResize(Sender);
END;


PROCEDURE TMainForm.Board27Click(Sender: TObject);
BEGIN
    StopTimer;
    Board27.Checked                := NOT Board27.Checked;
    Board1Form.Board.bShowSOutline := Board27.Checked;
    Board2Form.Board.bShowSOutline := Board27.Checked;

    Board1Form.Board.LoadPiece;
    Board1Form.FormPaint(Sender);

    Board2Form.Board.LoadPiece;
    Board2Form.FormPaint(Sender);
END;

PROCEDURE TMainForm.Board29Click(Sender: TObject);
BEGIN
    StopTimer;
    Board1Form.Board.nWhtSquare   := clWhite;
    Board1Form.Board.nWhtSOutline := clBlack;
    Board1Form.Board.nWhtOutline  := clBlack;
    Board1Form.Board.nWhtFill     := clWhite;

    Board1Form.Board.nBlkSquare   := clTeal;
    Board1Form.Board.nBlkSOutline := clBlack;
    Board1Form.Board.nBlkOutline  := clGray;
    Board1Form.Board.nBlkFill     := clBlack;

    Board1Form.Board.bOutlinePC   := TRUE;
    Board1Form.Board.nClickColor  := clRed;
    Board1Form.Board.bShwPCCount  := TRUE;
    Board1Form.Board.nNumberColor := clBlack;
    Board1Form.Board.nPcSquare    := clBtnFace;
    bFlipBoard                    := FALSE;
    Board1Form.Board.bFlip        := FALSE;
    Board1Form.Board.bAlwaysQueen := FALSE;
    Board1Form.Board.bDragMove    := TRUE;
    Board1Form.Board.bSmartMove   := FALSE;
    Board1Form.bActual            := FALSE;
    Board1Form.Board.bShowSOutline:= TRUE;
    showlag1.Checked:=true;
    ini1.INI.bShowLag:=showlag1.Checked;
    showtenth1.Checked:=true;
    ini1.INI.bShowTenth:=showtenth1.Checked;
    board1form.ShowMoveLag;
    board2form.ShowMoveLag;

    Board1Form.PaintBox2.Canvas.Pen.Color   := clBtnFace;
    Board1Form.PaintBox2.Canvas.Brush.Color := clBtnFace;
    Board1Form.PaintBox2.Canvas.Brush.Style := bsSolid;

    Board1Form.PaintBox3.Canvas.Pen.Color   := clBtnFace;
    Board1Form.PaintBox3.Canvas.Brush.Color := clBtnFace;
    Board1Form.PaintBox3.Canvas.Brush.Style := bsSolid;

    Board1Form.PaintBox4.Canvas.Pen.Color   := clBtnFace;
    Board1Form.PaintBox4.Canvas.Brush.Color := clBtnFace;
    Board1Form.PaintBox4.Canvas.Brush.Style := bsSolid;

    Board1Form.PaintBox5.Canvas.Pen.Color   := clBtnFace;
    Board1Form.PaintBox5.Canvas.Brush.Color := clBtnFace;
    Board1Form.PaintBox5.Canvas.Brush.Style := bsSolid;

    Board1Form.Color := clBtnFace;

    Board1Form.Board.LoadPiece;
//    Board1Form.FormPaint(Sender);

    Board2Form.Board.nWhtSquare   := clWhite;
    Board2Form.Board.nWhtSOutline := clBlack;
    Board2Form.Board.nWhtOutline  := clBlack;
    Board2Form.Board.nWhtFill     := clWhite;

    Board2Form.Board.nBlkSquare   := clTeal;
    Board2Form.Board.nBlkSOutline := clBlack;
    Board2Form.Board.nBlkOutline  := clGray;
    Board2Form.Board.nBlkFill     := clBlack;

    Board2Form.Board.bOutlinePC   := TRUE;
    Board2Form.Board.nClickColor  := clRed;
    Board2Form.Board.bShwPCCount  := TRUE;
    Board2Form.Board.nNumberColor := clBlack;
    Board2Form.Board.nPcSquare    := clBtnFace;
    bFlipBoard                    := FALSE;
    Board2Form.Board.bFlip        := TRUE;
    Board2Form.Board.bAlwaysQueen := FALSE;
    Board2Form.Board.bDragMove    := TRUE;
    Board2Form.Board.bSmartMove   := FALSE;
    Board2Form.bActual            := FALSE;
    Board2Form.Board.bShowSOutline:= TRUE;

    Board2Form.PaintBox2.Canvas.Pen.Color   := clBtnFace;
    Board2Form.PaintBox2.Canvas.Brush.Color := clBtnFace;
    Board2Form.PaintBox2.Canvas.Brush.Style := bsSolid;

    Board2Form.PaintBox3.Canvas.Pen.Color   := clBtnFace;
    Board2Form.PaintBox3.Canvas.Brush.Color := clBtnFace;
    Board2Form.PaintBox3.Canvas.Brush.Style := bsSolid;

    Board2Form.PaintBox4.Canvas.Pen.Color   := clBtnFace;
    Board2Form.PaintBox4.Canvas.Brush.Color := clBtnFace;
    Board2Form.PaintBox4.Canvas.Brush.Style := bsSolid;

    Board2Form.PaintBox5.Canvas.Pen.Color   := clBtnFace;
    Board2Form.PaintBox5.Canvas.Brush.Color := clBtnFace;
    Board2Form.PaintBox5.Canvas.Brush.Style := bsSolid;

    Board2Form.Color := clBtnFace;

    Board2Form.Board.LoadPiece;
//    Board2Form.FormPaint(Sender);

    Board18.Checked := Board1Form.Board.bAlwaysQueen;
    Board19.Checked := Board1Form.Board.bDragMove;
    Board20.Checked := Board1Form.Board.bSmartMove;
    Board22.Checked := Board1Form.bActual;
    Board27.Checked := Board1Form.Board.bShowSOutline;
    LoadRecord;
END;


PROCEDURE TMainForm.GameList1Click(Sender: TObject);
BEGIN
    StopTimer;
    FontDialog1.Device := fdScreen;
    FontDialog1.Font   := GameListForm.ListBox1.Font;
    IF (FontDialog1.Execute) THEN
    BEGIN
        GameListForm.ListBox1.Font := FontDialog1.Font;
    END;
END;

PROCEDURE TMainForm.GameList2Click(Sender: TObject);
BEGIN
    StopTimer;
    LoadCustomColors;
    ColorDialog1.Color := GameListForm.ListBox1.Color;
    IF (ColorDialog1.Execute) THEN
    BEGIN
        SaveCustomColors;
        GameListForm.ListBox1.Color := ColorDialog1.Color;
    END;
END;

PROCEDURE TMainForm.GameList99Click(Sender: TObject);
BEGIN
    StopTimer;
    GameList99.Checked      := (NOT GameList99.Checked);
    INI1.INI.bShowGameNum := GameList99.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.GameList98Click(Sender: TObject);
BEGIN
    StopTimer;
    GameList98.Checked    := (NOT GameList98.Checked);
    INI1.INI.bShowEvent := GameList98.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.GameList4Click(Sender: TObject);
BEGIN
    StopTimer;
    GameList4.Checked       := (NOT GameList4.Checked);
    INI1.INI.bShowRatings := GameList4.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.GameList5Click(Sender: TObject);
BEGIN
    StopTimer;
    GameList5.Checked    := (NOT GameList5.Checked);
    INI1.INI.bShowTime := GameList5.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.GameList6Click(Sender: TObject);
BEGIN
    StopTimer;
    GameList6.Checked   := (NOT GameList6.Checked);
    INI1.INI.bShowInc := GameList6.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.GameList7Click(Sender: TObject);
BEGIN
    StopTimer;
    GameList7.Checked      := (NOT GameList7.Checked);
    INI1.INI.bShowResult := GameList7.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.GameList9Click(Sender: TObject);
BEGIN
    StopTimer;
    GameListForm.ListBox1.Font.Color  := clBlack;
    GameListForm.ListBox1.Font.Height := -11;
    GameListForm.ListBox1.Font.Name   := 'MS Sans Serif';
    GameListForm.ListBox1.Font.Pitch  := fpDefault;
    GameListForm.ListBox1.Font.Size   := 10;
    GameListForm.ListBox1.Font.Style  := [];
    GameListForm.ListBox1.Color       := defFormB;
    GameList99.Checked                := FALSE;
    GameList98.Checked                := FALSE;
    GameList4.Checked                 := TRUE;
    GameList5.Checked                 := TRUE;
    GameList6.Checked                 := TRUE;
    GameList7.Checked                 := TRUE;
    INI1.INI.bShowGameNum           := GameList99.Checked;
    INI1.INI.bShowEvent             := GameList98.Checked;
    INI1.INI.bShowRatings           := GameList4.Checked;
    INI1.INI.bShowTime              := GameList5.Checked;
    INI1.INI.bShowInc               := GameList6.Checked;
    INI1.INI.bShowResult            := GameList7.Checked;

    ProcessBeg(PROGRAM_TITLE + ' (Rebuilding BPGN headers...)');
//    BPGN.RebuildHeaders(GameListForm.ListBox1);
    gamelistform.ListBox1.Invalidate;
    ProcessEnd;
END;

PROCEDURE TMainForm.Nextmove1Click(Sender: TObject);
BEGIN
    StopTimer;
    FontDialog1.Device := fdScreen;
    FontDialog1.Font   := NextMoveForm.ListBox1.Font;
    IF (FontDialog1.Execute) THEN
    BEGIN
        NextMoveForm.ListBox1.Font := FontDialog1.Font;
    END;
END;

PROCEDURE TMainForm.Nextmove2Click(Sender: TObject);
BEGIN
    StopTimer;
    LoadCustomColors;
    ColorDialog1.Color := NextMoveForm.ListBox1.Color;
    IF (ColorDialog1.Execute) THEN
    BEGIN
        SaveCustomColors;
        NextMoveForm.ListBox1.Color := ColorDialog1.Color;
    END;
END;

PROCEDURE TMainForm.Nextmove4Click(Sender: TObject);
BEGIN
    StopTimer;
    NextMoveForm.ListBox1.Font.Color  := clBlack;
    NextMoveForm.ListBox1.Font.Height := -11;
    NextMoveForm.ListBox1.Font.Name   := 'MS Sans Serif';
    NextMoveForm.ListBox1.Font.Pitch  := fpDefault;
    NextMoveForm.ListBox1.Font.Size   := 10;
    NextMoveForm.ListBox1.Font.Style  := [];
    NextMoveForm.ListBox1.Color       := defformB;
END;

PROCEDURE TMainForm.Prevmove1Click(Sender: TObject);
BEGIN
    StopTimer;
    FontDialog1.Device := fdScreen;
    FontDialog1.Font   := PrevMoveForm.ListBox1.Font;
    IF (FontDialog1.Execute) THEN
    BEGIN
        PrevMoveForm.ListBox1.Font := FontDialog1.Font;
    END;
END;

PROCEDURE TMainForm.Prevmove2Click(Sender: TObject);
BEGIN
    StopTimer;
    LoadCustomColors;
    ColorDialog1.Color := PrevMoveForm.ListBox1.Color;
    IF (ColorDialog1.Execute) THEN
    BEGIN
        SaveCustomColors;
        PrevMoveForm.ListBox1.Color := ColorDialog1.Color;
    END;
END;

PROCEDURE TMainForm.Prevmove4Click(Sender: TObject);
BEGIN
    StopTimer;
    PrevMoveForm.ListBox1.Font.Color  := clBlack;
    PrevMoveForm.ListBox1.Font.Height := -11;
    PrevMoveForm.ListBox1.Font.Name   := 'MS Sans Serif';
    PrevMoveForm.ListBox1.Font.Pitch  := fpDefault;
    PrevMoveForm.ListBox1.Font.Size   := 10;
    PrevMoveForm.ListBox1.Font.Style  := [];
    PrevMoveForm.ListBox1.Color       := defFormB;
END;

PROCEDURE TMainForm.Comment1Click(Sender: TObject);
BEGIN
    StopTimer;
    FontDialog1.Device := fdScreen;
    FontDialog1.Font   := CommentForm.Memo1.Font;
    IF (FontDialog1.Execute) THEN
    BEGIN
        CommentForm.Memo1.Font := FontDialog1.Font;
    END;
END;

PROCEDURE TMainForm.Comment2Click(Sender: TObject);
BEGIN
    StopTimer;
    LoadCustomColors;
    ColorDialog1.Color := CommentForm.Memo1.Color;
    IF (ColorDialog1.Execute) THEN
    BEGIN
        SaveCustomColors;
        CommentForm.Memo1.Color := ColorDialog1.Color;
    END;
END;

PROCEDURE TMainForm.Comment3Click(Sender: TObject);
BEGIN
    StopTimer;
    Comment3.Checked           := (NOT Comment3.Checked);
    CommentForm.Memo1.WordWrap := Comment3.Checked;

    IF (CommentForm.Memo1.WordWrap) THEN
        CommentForm.Memo1.ScrollBars := ssVertical
    ELSE
        CommentForm.Memo1.ScrollBars := ssBoth;
END;

PROCEDURE TMainForm.Comment5Click(Sender: TObject);
BEGIN
    StopTimer;
    CommentForm.Memo1.Font.Color  := clBlack;
    CommentForm.Memo1.Font.Height := -11;
    CommentForm.Memo1.Font.Name   := 'MS Sans Serif';
    CommentForm.Memo1.Font.Pitch  := fpDefault;
    CommentForm.Memo1.Font.Size   := 10;
    CommentForm.Memo1.Font.Style  := [];
    CommentForm.Memo1.Color       := defformB;
    CommentForm.Memo1.WordWrap    := FALSE;
    Comment3.Checked              := FALSE;

    IF (CommentForm.Memo1.WordWrap) THEN
        CommentForm.Memo1.ScrollBars := ssVertical
    ELSE
        CommentForm.Memo1.ScrollBars := ssBoth;
END;


PROCEDURE TMainForm.Options8Click(Sender: TObject);
BEGIN
    StopTimer;
    processbeg('Saving options');
    LoadIntoINI;
    INI1.INI.WriteOptions;
    processend;
END;

PROCEDURE TMainForm.Options9Click(Sender: TObject);
BEGIN
    StopTimer;
    Options9.Checked := NOT Options9.Checked;
END;

PROCEDURE TMainForm.Options11Click(Sender: TObject);
BEGIN
    StopTimer;
    Options11.Checked := NOT Options11.Checked;
END;

PROCEDURE TMainForm.Utility0Click(Sender: TObject);
BEGIN
    Utility2.Enabled := FileExists(cExtractErrFn);
    Utility4.Enabled := BPGN.IsOpen;
    Utility7.Enabled := BPGN.IsOpen;
    Utility13.enabled:= ((BPGN.isOpen) and (not bpgn.bIsDB));
    brokenresult1.Enabled:=(bpgn.IsOpen and (not bpgn.IsEmpty));
    restoreStringFromTag1.Enabled:=bpgn.nFilterCount>0;
    restoreTagFromString1.Enabled:=restoreStringFromTag1.Enabled;
    replaceHandleinlisted1.Enabled:=restoreStringFromTag1.Enabled;
END;

PROCEDURE TMainForm.Utility1Click(Sender: TObject);
var Extractform:textractform;
BEGIN
    StopTimer;
    try
    extractform:=textractform.create(self);
    IF (NOT CheckFileSaveChanges(Sender, FALSE)) THEN
        EXIT;
    BPGN.CloseBPGN(TRUE);
    ResetAll(Sender);

    ExtractForm.Caption        := 'Extract moves from BPGN files';
    ExtractForm.cHandle        := INI1.INI.cHandle;
    ExtractForm.cSource        := INI1.INI.cSource;
    ExtractForm.cDestination   := INI1.INI.cDestination;
    ExtractForm.nMoveNumber    := INI1.INI.nMoveNumber;
    ExtractForm.bRemoveComment := INI1.INI.bRemoveComment;
    ExtractForm.bGenOccurrence := INI1.INI.bGenOccurrence;

    ExtractForm.SetupValues;
    ExtractForm.ShowModal;

    IF (ExtractForm.bOK) THEN
    BEGIN
        INI1.INI.cHandle        := ExtractForm.cHandle;
        INI1.INI.cSource        := ExtractForm.cSource;
        INI1.INI.cDestination   := ExtractForm.cDestination;
        INI1.INI.nMoveNumber    := ExtractForm.nMoveNumber;
        INI1.INI.bRemoveComment := ExtractForm.bRemoveComment;
        INI1.INI.bGenOccurrence := ExtractForm.bGenOccurrence;

        ProcessBeg(PROGRAM_TITLE + ' (Extract moves from BPGN files...)');
        ExtractMoves;
        ProcessEnd;
    END;
    finally extractform.free;
    end;
END;

PROCEDURE TMainForm.Utility2Click(Sender: TObject);
BEGIN
  StopTimer;
  with ini1.INI do
  showTXTFile('View ' + cExtractErrFn,cExtractErrFn,nvexL,nvexT,nvexW,nvexh);
END;

PROCEDURE TMainForm.Utility4Click(Sender: TObject);
var fcdform:TFCDform;
BEGIN
    StopTimer;
    try
    fcdform:=tfcdform.create(self);

    FCDForm.Caption := 'Generate four column RTF file from current game';

    FCDForm.fHandle.Name   := INI1.INI.fFCD1.Name;
    FCDForm.fHandle.Color  := INI1.INI.fFCD1.Color;
    FCDForm.fHandle.Height := INI1.INI.fFCD1.Height;
    FCDForm.fHandle.Size   := INI1.INI.fFCD1.Size;
    FCDForm.fHandle.Pitch  := INI1.INI.fFCD1.Pitch;
    FCDForm.fHandle.Style  := INI1.INI.fFCD1.Style;

    FCDForm.fMove.Name   := INI1.INI.fFCD2.Name;
    FCDForm.fMove.Color  := INI1.INI.fFCD2.Color;
    FCDForm.fMove.Height := INI1.INI.fFCD2.Height;
    FCDForm.fMove.Size   := INI1.INI.fFCD2.Size;
    FCDForm.fMove.Pitch  := INI1.INI.fFCD2.Pitch;
    FCDForm.fMove.Style  := INI1.INI.fFCD2.Style;

    FCDForm.fComment.Name   := INI1.INI.fFCD3.Name;
    FCDForm.fComment.Color  := INI1.INI.fFCD3.Color;
    FCDForm.fComment.Height := INI1.INI.fFCD3.Height;
    FCDForm.fComment.Size   := INI1.INI.fFCD3.Size;
    FCDForm.fComment.Pitch  := INI1.INI.fFCD3.Pitch;
    FCDForm.fComment.Style  := INI1.INI.fFCD3.Style;

    FCDForm.fChess.Name   := INI1.INI.fFCD4.Name;
    FCDForm.fChess.Color  := INI1.INI.fFCD4.Color;
    FCDForm.fChess.Height := INI1.INI.fFCD4.Height;
    FCDForm.fChess.Size   := INI1.INI.fFCD4.Size;
    FCDForm.fChess.Pitch  := INI1.INI.fFCD4.Pitch;
    FCDForm.fChess.Style  := INI1.INI.fFCD4.Style;

    FCDForm.fResult.Name   := INI1.INI.fFCD5.Name;
    FCDForm.fResult.Color  := INI1.INI.fFCD5.Color;
    FCDForm.fResult.Height := INI1.INI.fFCD5.Height;
    FCDForm.fResult.Size   := INI1.INI.fFCD5.Size;
    FCDForm.fResult.Pitch  := INI1.INI.fFCD5.Pitch;
    FCDForm.fResult.Style  := INI1.INI.fFCD5.Style;

    FCDForm.bHandle     := INI1.INI.bRTFHand;
    FCDForm.bMove       := INI1.INI.bRTFMove;
    FCDForm.bAnnotation := INI1.INI.bRTFAnno;
    FCDForm.bTime       := INI1.INI.bRTFTime;
    FCDForm.bComment    := INI1.INI.bRTFNote;
    FCDForm.bDisplay    := INI1.INI.bRTFDisp;
    FCDForm.bResult     := INI1.INI.bRTFRest;
    FCDForm.bZero       := INI1.INI.bRTFZero;

    FCDForm.cRTFFile := INI1.INI.cRTFFile;

    FCDForm.SetupValues;
    FCDForm.ShowModal;

    IF (FCDForm.bOK) THEN
    BEGIN
        INI1.INI.fFCD1.Name   := FCDForm.fHandle.Name;
        INI1.INI.fFCD1.Color  := FCDForm.fHandle.Color;
        INI1.INI.fFCD1.Height := FCDForm.fHandle.Height;
        INI1.INI.fFCD1.Size   := FCDForm.fHandle.Size;
        INI1.INI.fFCD1.Pitch  := FCDForm.fHandle.Pitch;
        INI1.INI.fFCD1.Style  := FCDForm.fHandle.Style;

        INI1.INI.fFCD2.Name   := FCDForm.fMove.Name;
        INI1.INI.fFCD2.Color  := FCDForm.fMove.Color;
        INI1.INI.fFCD2.Height := FCDForm.fMove.Height;
        INI1.INI.fFCD2.Size   := FCDForm.fMove.Size;
        INI1.INI.fFCD2.Pitch  := FCDForm.fMove.Pitch;
        INI1.INI.fFCD2.Style  := FCDForm.fMove.Style;

        INI1.INI.fFCD3.Name   := FCDForm.fComment.Name;
        INI1.INI.fFCD3.Color  := FCDForm.fComment.Color;
        INI1.INI.fFCD3.Height := FCDForm.fComment.Height;
        INI1.INI.fFCD3.Size   := FCDForm.fComment.Size;
        INI1.INI.fFCD3.Pitch  := FCDForm.fComment.Pitch;
        INI1.INI.fFCD3.Style  := FCDForm.fComment.Style;

        INI1.INI.fFCD4.Name   := FCDForm.fChess.Name;
        INI1.INI.fFCD4.Color  := FCDForm.fChess.Color;
        INI1.INI.fFCD4.Height := FCDForm.fChess.Height;
        INI1.INI.fFCD4.Size   := FCDForm.fChess.Size;
        INI1.INI.fFCD4.Pitch  := FCDForm.fChess.Pitch;
        INI1.INI.fFCD4.Style  := FCDForm.fChess.Style;

        INI1.INI.fFCD5.Name   := FCDForm.fResult.Name;
        INI1.INI.fFCD5.Color  := FCDForm.fResult.Color;
        INI1.INI.fFCD5.Height := FCDForm.fResult.Height;
        INI1.INI.fFCD5.Size   := FCDForm.fResult.Size;
        INI1.INI.fFCD5.Pitch  := FCDForm.fResult.Pitch;
        INI1.INI.fFCD5.Style  := FCDForm.fResult.Style;

        INI1.INI.bRTFHand := FCDForm.bHandle;
        INI1.INI.bRTFMove := FCDForm.bMove;
        INI1.INI.bRTFAnno := FCDForm.bAnnotation;
        INI1.INI.bRTFTime := FCDForm.bTime;
        INI1.INI.bRTFNote := FCDForm.bComment;
        INI1.INI.bRTFDisp := FCDForm.bDisplay;
        INI1.INI.bRTFRest := FCDForm.bResult;
        INI1.INI.bRTFZero := FCDForm.bZero;

        INI1.INI.cRTFFile := FCDForm.cRTFFile;
        FCDProc(fcdform);
        with ini1.INI do
        ShowFile('Generated RTF: '+cRTFFile,cRTFFile,nRtfl,nrtft,nrtfw,nrtfh);
    END;

    finally fcdform.free;
    end;
END;

PROCEDURE TMainForm.Utility5Click(Sender: TObject);
VAR
    cFn: String;
BEGIN
    StopTimer;
    IF (OpenDialog4.Execute) THEN
    BEGIN
        cFn := ChangeFileExt(OpenDialog4.Filename, '.' + RTF_EXT);
        OpenDialog4.InitialDir := ExtractFilePath(cFn);
        with ini1.INI do
        ShowFile('View RTF file (' + cFn + ')',cFn,nRtfl,nrtft,nrtfw,nrtfh);
    END;
END;

PROCEDURE TMainForm.Utility7Click(Sender: TObject);
var txtform:ttxtform;
BEGIN
    StopTimer;
    try
    txtform:=ttxtform.create(self);
    with sender as TmenuItem do
    TXTForm.Caption := 'Generate four column TXT file from current game';
    TXTForm.bHandle     := INI1.INI.bTXTHand;
    TXTForm.bMove       := INI1.INI.bTXTMove;
    TXTForm.bAnnotation := INI1.INI.bTXTAnno;
    TXTForm.bTime       := INI1.INI.bTXTTime;
    TXTForm.bComment    := INI1.INI.bTXTNote;
    TXTForm.bDisplay    := INI1.INI.bTXTDisp;
    TXTForm.bResult     := INI1.INI.bTXTRest;
    TXTForm.bZero       := INI1.INI.bTXTZero;

    TXTForm.cTXTFile := INI1.INI.cTXTFile;

    TXTForm.SetupValues;
    TXTForm.ShowModal;

    IF (TXTForm.bOK) THEN
    BEGIN
        INI1.INI.bTXTHand := TXTForm.bHandle;
        INI1.INI.bTXTMove := TXTForm.bMove;
        INI1.INI.bTXTAnno := TXTForm.bAnnotation;
        INI1.INI.bTXTTime := TXTForm.bTime;
        INI1.INI.bTXTNote := TXTForm.bComment;
        INI1.INI.bTXTDisp := TXTForm.bDisplay;
        INI1.INI.bTXTRest := TXTForm.bResult;
        INI1.INI.bTXTZero := TXTForm.bZero;

        INI1.INI.cTXTFile := TXTForm.cTXTFile;
        TXTProc(txtform);
        with ini1.INI do
        ShowTxtFile('Generated TXT file: '+ cTXTfile,cTXTfile,ntxtl,ntxtt,ntxtw,ntxth);
    END;
    finally
    txtform.free;
    end;
END;

PROCEDURE TMainForm.Utility8Click(Sender: TObject);
VAR
    cFn: String;
BEGIN
    StopTimer;
    IF (OpenDialog5.Execute) THEN
    BEGIN
        cFn := ChangeFileExt(OpenDialog5.Filename, '.' + TXT_EXT);
        OpenDialog5.InitialDir := ExtractFilePath(cFn);
        with ini1.INI do
        ShowTXTFile('View RTF file (' + cFn + ')',cFn,ntxtl,ntxtt,ntxtw,ntxth);
    END;
END;

PROCEDURE TMainForm.Windows0Click(Sender: TObject);
BEGIN
    { Windows }
END;

PROCEDURE TMainForm.Windows1Click(Sender: TObject);
BEGIN
    Board1Form.Show;
END;

PROCEDURE TMainForm.Windows2Click(Sender: TObject);
BEGIN
    Board2Form.Show;
END;

PROCEDURE TMainForm.Windows3Click(Sender: TObject);
BEGIN
    NavigateForm.Show;
END;

PROCEDURE TMainForm.Windows4Click(Sender: TObject);
BEGIN
    GameListForm.Show;
END;

PROCEDURE TMainForm.Windows5Click(Sender: TObject);
BEGIN
    NextMoveForm.Show;
END;

PROCEDURE TMainForm.Windows6Click(Sender: TObject);
BEGIN
    PrevMoveForm.Show;
END;

PROCEDURE TMainForm.Windows7Click(Sender: TObject);
BEGIN
    CommentForm.Show;
END;

PROCEDURE TMainForm.Help0Click(Sender: TObject);
BEGIN
    { Help }
END;

PROCEDURE TMainForm.Help1Click(Sender: TObject);
var AboutForm:TaboutForm;
BEGIN
    StopTimer;
    try
     aboutForm:=TaboutForm.Create(self);
     AboutForm.Caption := 'About BPGN viewer';
     AboutForm.SetupValues;
     AboutForm.ShowModal;
    finally
     AboutForm.free;
    end;
END;

PROCEDURE TMainForm.LoadCustomColors;
BEGIN
    ColorDialog1.CustomColors.Clear;

    ColorDialog1.CustomColors.Add(INI1.INI.cColorA);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorB);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorC);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorD);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorE);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorF);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorG);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorH);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorI);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorJ);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorK);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorL);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorM);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorN);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorO);
    ColorDialog1.CustomColors.Add(INI1.INI.cColorP);
END;

PROCEDURE TMainForm.SaveCustomColors;
BEGIN
    INI1.INI.cColorA := ColorDialog1.CustomColors.Strings[ 0];
    INI1.INI.cColorB := ColorDialog1.CustomColors.Strings[ 1];
    INI1.INI.cColorC := ColorDialog1.CustomColors.Strings[ 2];
    INI1.INI.cColorD := ColorDialog1.CustomColors.Strings[ 3];
    INI1.INI.cColorE := ColorDialog1.CustomColors.Strings[ 4];
    INI1.INI.cColorF := ColorDialog1.CustomColors.Strings[ 5];
    INI1.INI.cColorG := ColorDialog1.CustomColors.Strings[ 6];
    INI1.INI.cColorH := ColorDialog1.CustomColors.Strings[ 7];
    INI1.INI.cColorI := ColorDialog1.CustomColors.Strings[ 8];
    INI1.INI.cColorJ := ColorDialog1.CustomColors.Strings[ 9];
    INI1.INI.cColorK := ColorDialog1.CustomColors.Strings[10];
    INI1.INI.cColorL := ColorDialog1.CustomColors.Strings[11];
    INI1.INI.cColorM := ColorDialog1.CustomColors.Strings[12];
    INI1.INI.cColorN := ColorDialog1.CustomColors.Strings[13];
    INI1.INI.cColorO := ColorDialog1.CustomColors.Strings[14];
    INI1.INI.cColorP := ColorDialog1.CustomColors.Strings[15];
END;

PROCEDURE TMainForm.LoadIntoINI;
BEGIN
    ini1.ini.engineopt:=engopt;

    INI1.INI.cNewPath    := OpenDialog1.InitialDir;
    INI1.INI.cOpenPath   := OpenDialog2.InitialDir;
    INI1.INI.cDeletePath := OpenDialog3.InitialDir;
    INI1.INI.cRTFDir     := OpenDialog4.InitialDir;
    INI1.INI.cTXTDir     := OpenDialog5.InitialDir;

    INI1.INI.nWhtFill     := Board1Form.Board.nWhtFill;
    INI1.INI.nWhtOutline  := Board1Form.Board.nWhtOutline;
    INI1.INI.nWhtSquare   := Board1Form.Board.nWhtSquare;
    INI1.INI.nWhtSOutline := Board1Form.Board.nWhtSOutline;

    INI1.INI.nBlkFill     := Board1Form.Board.nBlkFill;
    INI1.INI.nBlkOutline  := Board1Form.Board.nBlkOutline;
    INI1.INI.nBlkSquare   := Board1Form.Board.nBlkSquare;
    INI1.INI.nBlkSOutline := Board1Form.Board.nBlkSOutline;

    INI1.INI.bOutlinePC   := Board1Form.Board.bOutlinePC;
    INI1.INI.nClickColor  := Board1Form.Board.nClickColor;
    INI1.INI.bShwPCCount  := Board1Form.Board.bShwPCCount;
    INI1.INI.nNumberColor := Board1Form.Board.nNumberColor;
    INI1.INI.nPcSquare    := Board1Form.Board.nPcSquare;
    INI1.INI.bFlipBd      := bFlipBoard;
    Ini1.ini.usebmp:=usebitmap;
    ini1.ini.usepiecebmp:=usepiecebitmap;
    ini1.ini.useboardbmp:=useboardbitmap;
    ini1.ini.bmpfile:=bg_file;
    ini1.INI.p_dir:=piece_dir;
    ini1.ini.b_dir:=board_dir;
    INI1.INI.bAlwaysQ     := Board1Form.Board.bAlwaysQueen;
    INI1.INI.bDragMove    := Board1Form.Board.bDragMove;
    INI1.INI.bSmartMove   := Board1Form.Board.bSmartMove;
    INI1.INI.bActualBoard := Board1Form.bActual;
    INI1.INI.nBoardWindow := Board1Form.Color;
    INI1.INI.nBoardOutline:= Board1Form.PaintBox2.Canvas.Pen.Color;
    INI1.INI.bShowSOutline:= Board1Form.Board.bShowSOutline;
    ini1.ini.bairdropini:=bairdrop;
    ini1.INI.savedepth:=bpgn.savedepth;
    ini1.INI.bfullstat:=fform.bfullstat;

    INI1.INI.nWaitF        := Board1Form.nWaitF;
    INI1.INI.nWaitB        := Board1Form.nWaitB;
    INI1.INI.nTurnF        := Board1Form.nTurnF;
    INI1.INI.nTurnB        := Board1Form.nTurnB;
    INI1.INI.fClock.Color  := Board1Form.nWaitF;
    INI1.INI.fClock.Height := Board1Form.Label3.Font.Height;
    INI1.INI.fClock.Name   := Board1Form.Label3.Font.Name;
    INI1.INI.fClock.Pitch  := Board1Form.Label3.Font.Pitch;
    INI1.INI.fClock.Size   := Board1Form.Label3.Font.Size;
    INI1.INI.fClock.Style  := Board1Form.Label3.Font.Style;

    INI1.INI.nHandleB       := Board1Form.Label1.Color;
    INI1.INI.fHandle.Color  := Board1Form.Label1.Font.Color;
    INI1.INI.fHandle.Height := Board1Form.Label1.Font.Height;
    INI1.INI.fHandle.Name   := Board1Form.Label1.Font.Name;
    INI1.INI.fHandle.Pitch  := Board1Form.Label1.Font.Pitch;
    INI1.INI.fHandle.Size   := Board1Form.Label1.Font.Size;
    INI1.INI.fHandle.Style  := Board1Form.Label1.Font.Style;

    INI1.INI.nGameListB        := GameListForm.ListBox1.Color;
    INI1.INI.fGameListF.Color  := GameListForm.ListBox1.Font.Color;
    INI1.INI.fGameListF.Height := GameListForm.ListBox1.Font.Height;
    INI1.INI.fGameListF.Name   := GameListForm.ListBox1.Font.Name;
    INI1.INI.fGameListF.Pitch  := GameListForm.ListBox1.Font.Pitch;
    INI1.INI.fGameListF.Size   := GameListForm.ListBox1.Font.Size;
    INI1.INI.fGameListF.Style  := GameListForm.ListBox1.Font.Style;
    INI1.INI.bShowGameNum      := GameList99.Checked;
    INI1.INI.bShowEvent        := GameList98.Checked;
    INI1.INI.bShowRatings      := GameList4.Checked;
    INI1.INI.bShowTime         := GameList5.Checked;
    INI1.INI.bShowInc          := GameList6.Checked;
    INI1.INI.bShowResult       := GameList7.Checked;

    INI1.INI.nNextMoveB        := NextMoveForm.ListBox1.Color;
    INI1.INI.fNextMoveF.Color  := NextMoveForm.ListBox1.Font.Color;
    INI1.INI.fNextMoveF.Height := NextMoveForm.ListBox1.Font.Height;
    INI1.INI.fNextMoveF.Name   := NextMoveForm.ListBox1.Font.Name;
    INI1.INI.fNextMoveF.Pitch  := NextMoveForm.ListBox1.Font.Pitch;
    INI1.INI.fNextMoveF.Size   := NextMoveForm.ListBox1.Font.Size;
    INI1.INI.fNextMoveF.Style  := NextMoveForm.ListBox1.Font.Style;

    INI1.INI.nPrevMoveB        := PrevMoveForm.ListBox1.Color;
    INI1.INI.fPrevMoveF.Color  := PrevMoveForm.ListBox1.Font.Color;
    INI1.INI.fPrevMoveF.Height := PrevMoveForm.ListBox1.Font.Height;
    INI1.INI.fPrevMoveF.Name   := PrevMoveForm.ListBox1.Font.Name;
    INI1.INI.fPrevMoveF.Pitch  := PrevMoveForm.ListBox1.Font.Pitch;
    INI1.INI.fPrevMoveF.Size   := PrevMoveForm.ListBox1.Font.Size;
    INI1.INI.fPrevMoveF.Style  := PrevMoveForm.ListBox1.Font.Style;

    INI1.INI.nCommentB        := CommentForm.Memo1.Color;
    INI1.INI.fCommentF.Color  := CommentForm.Memo1.Font.Color;
    INI1.INI.fCommentF.Height := CommentForm.Memo1.Font.Height;
    INI1.INI.fCommentF.Name   := CommentForm.Memo1.Font.Name;
    INI1.INI.fCommentF.Pitch  := CommentForm.Memo1.Font.Pitch;
    INI1.INI.fCommentF.Size   := CommentForm.Memo1.Font.Size;
    INI1.INI.fCommentF.Style  := CommentForm.Memo1.Font.Style;
    INI1.INI.bCommentWR       := CommentForm.Memo1.WordWrap;
    ini1.ini.bsoundon:=ApplyGAmesound1.checked;

    Ini1.Ini.nEngineB:=nengineb;
    INI1.INI.fEngineF.Color  := fEnginef.color;
    INI1.INI.fEngineF.Height := fEngineF.Height;
    INI1.INI.fEngineF.Name   := fEnginef.Name;
    INI1.INI.fEngineF.Pitch  := FEnginef.Pitch;
    INI1.INI.fEngineF.Size   := fEnginef.Size;
    INI1.INI.fEngineF.Style  := fEnginef.Style;

    Ini1.Ini.Tecumseh :=tecumseh;

    INI1.Ini.defeng:=defeng;

    INI1.INI.bSaveOptionsOnExit := Options9.Checked;
    INI1.INI.bSaveLayoutsOnExit := Options11.Checked;
END;

PROCEDURE TMainForm.OpenBPGNFile(Sender: TObject; cFn: String);
VAR
    nI: Integer;
    hName,pfname:string;
    isdb:boolean;
    nc,barstep:longint;
BEGIN
    IF (Length(cFn) = 0) THEN
    BEGIN
        wError('Missing BPGN filename.');
        EXIT;
    END;

    IF (NOT FileExists(cFn)) THEN
    BEGIN
        wError(cFn + EOL_STR + 'BPGN file not found.');
        EXIT;
    END;

    gamelistform.ClearAll;
    fform.Close;
    BPGN.CloseBPGN(TRUE);
    bpgn.bIsDB:=false;
    isdb:=isbpgnheader(cfn);
    if isdb then
    begin
     hname:=cfn;
     cfn:=changefileext(hname,'.'+BPGN_EXT);
    end;

    IF (NOT BPGN.OpenBPGN(cFn)) THEN
        EXIT;

    Caption := PROGRAM_TITLE + ' (' + cFn + ')';

//    ProcessBeg(PROGRAM_TITLE + ' (Scanning BPGN header...)');

    if isdb then begin
     try
      DoLongRun(fLongThread,bpgn.loaddbHeader_,@hname,'Loading headers...',true);
     except
       bpgn.CloseBPGN(false); Caption:=Program_Title;
       exit;
     end;
     bpgn.nFilterCount:=bpgn.getHeaderCount;
    end;


    if (not bpgn.bIsDB) then begin
     try
      DoLongRun(fLongThread,bpgn.loadHeader_,nil,'Parsing headers...',true);
     except
      bpgn.CloseBPGN(false); Caption:=Program_Title; exit;
     end;
     bpgn.nFilterCount:=bpgn.getHeaderCount;
    end
    else
    try
      DoLongRun(fLongThread,bpgn.loadDBPos_,@hname,'Loading positions...',true);
    except
      bpgn.CloseBPGN(false); Caption:=Program_title; exit;
    end;
    bpgn.resetfilter;
//    ProcessEnd;

    gamelistform.ListBox1.items.Count:=bpgn.GetHeaderCount;

    nGameIndex  := -1;
    nStartTime  := 0;
    bIgnoreDisp := TRUE;
    bIgnoreMemo := TRUE;

    Timer1.Enabled := FALSE;
    bRunTimer      := FALSE;
    nTick          := 0;
    nMaxTick       := 0;

    IF (bpgn.GetHeaderCount > 0) THEN
    BEGIN
        GameListForm.ListBox1.Enabled   := TRUE;
        GameListForm.ListBox1.ItemIndex := 0;
        GameListClick(Sender);
    END
    ELSE
        GameListForm.ListBox1.Enabled := FALSE;
    bpgn.bdbchange:=false;

    if bpgn.bIsDB then begin

     Caption:=Program_TITLE+' ( '+hname+')';
     savedialog2.Filter:='Supported (*.bpgn,*.bh)|*.bpgn;*.bh|Bug DB|*.bh|BPGN (*.bpgn)|*.bpgn|All|*.*';
     fform.godatabase;
//     if ini1.INI.bTreeAuto then ftreeform.CheckBox13.Checked:=true;
    end
    else begin
     savedialog2.Filter:='BPGN (*.bpgn)|*.bpgn|All|*.*';
     fform.gobpgn;
    end;
END;

procedure TmainForm.filterGAmelist;
var ni:longint;
s1,s:string;
begin
  gamelistform.ListBox1.items.Count:=bpgn.nFilterCount;
  if bpgn.bfilterset then
  begin
     gamelistform.cInfo:=' Stat: +'+Inttostr(bpgn.nwin)+
     '-'+inttostr(bpgn.nloss)+
     '='+inttostr(bpgn.ndraw);
  end
  else gamelistform.cInfo:='';


//    nGameIndex  := -1;
    nStartTime  := 0;
    bIgnoreDisp := TRUE;
    bIgnoreMemo := TRUE;

    StopTimer;

    gamelistform.ListBox1.Repaint;
    IF (bpgn.nFilterCount>0) THEN
    BEGIN
        GameListForm.ListBox1.Enabled   := TRUE;
    END
    ELSE
        GameListForm.ListBox1.Enabled := FALSE;
end;


PROCEDURE TMainForm.GameListClick(Sender: TObject);
var s,s1:string;
r1:TPosDbRec;
BEGIN
    IF (GameListForm.ListBox1.Items.Count <= 0) THEN
        EXIT;

    if gamelistform.ListBox1.ItemIndex<0 then gamelistform.ListBox1.ItemIndex:=0;
    if gamelistform.ListBox1.ItemIndex> gamelistform.ListBox1.items.Count-1 then
     gamelistform.ListBox1.ItemIndex:=gamelistform.ListBox1.items.count-1;


    IF (nGameIndex = bpgn.getfindex(GameListForm.ListBox1.ItemIndex)) THEN
        EXIT;

    if batchmode then DoLongRun(fLongThread,bpgn.SaveGame_,@nGameIndex,'Saving game...')
    else
     IF (NOT checkgamesaveChanges(nGameIndex)) THEN
     begin
        gamelistform.restoreIndex;
        EXIT;
     end
     else gamelistform.Outofdate;

    nGameIndex := bpgn.getfindex(GameListForm.ListBox1.ItemIndex);
    GameListForm.setCaption(glCurrent);

    stoptimer;

    nStartTime := 1000*TOOLBOX_GetTimeVal(BPGN.GetTimeC(nGameIndex));

    IF (Length(BPGN.GetOpenA(nGameIndex)) = 0) THEN
        Board1Form.Caption := 'Board #1'
    ELSE
        Board1Form.Caption := 'Board #1 (' + BPGN.GetOpenA(nGameIndex) + ')';

    IF (Length(BPGN.GetOpenB(nGameIndex)) = 0) THEN
        Board2Form.Caption := 'Board #2'
    ELSE
        Board2Form.Caption := 'Board #2 (' + BPGN.GetOpenB(nGameIndex) + ')';


    Board1Form.cWTime := TOOLBOX_GetDispTimeStr(nStartTime);
    Board1Form.cBTime := Board1Form.cWTime;
    Board2Form.cWTime := Board1Form.cWTime;
    Board2Form.cBTime := Board1Form.cWTime;

    Board1Form.LoadTime;
    Board2Form.LoadTime;

    Board1Form.cWHandle := BPGN.GetWhiteAHandle(nGameIndex);
    Board1Form.cBHandle := BPGN.GetBlackAHandle(nGameIndex);
    Board2Form.cWHandle := BPGN.GetWhiteBHandle(nGameIndex);
    Board2Form.cBHandle := BPGN.GetBlackBHandle(nGameIndex);

    Board1Form.LoadHandle;
    Board2Form.LoadHandle;

    Board1Form.Board.Chess.ResetBoard;
    Board2Form.Board.Chess.ResetBoard;

    Board1Form.FormResize(Sender);
    Board2Form.FormResize(Sender);

    Board1Form.DisplayBoard;
    Board2Form.DisplayBoard;

    ProcessBeg(PROGRAM_TITLE + ' (Parsing BPGN game...)');
    BPGN.LoadGame(nGameIndex);
    ProcessEnd;
    with bpgn do //set Board in tree to follow the game with searched position
    if ((bfilterset) and (gamelistpos.cBoard>0)) then
    begin
     boardtoint(bpgnarray[nCurrent,1].aboard,r1.position);
     if compposition(@gamelistpos,@r1)=0 then fform.RadioGroup1.ItemIndex:=0
     else begin
      boardtoint(bpgnarray[nCurrent,2].aboard,r1.position);
      if compposition(@gamelistpos,@r1)=0 then fform.RadioGroup1.ItemIndex:=1;
     end;
    end;
    LoadRecord;

    NavigateForm.Enable(6);
    NavigateForm.Enable(8);
    If (Pos(Crazyhouse_ID,lowercase(Bpgn.getEvent(nGameIndex)))>0) then
    enableengines
    else disableEngines;

    NextMoveForm.ListBox1.Enabled := TRUE;
    PrevMoveForm.ListBox1.Enabled := TRUE;
    CommentForm.Memo1.Enabled     := TRUE;
    gamelistform.backupIndex;
END;

PROCEDURE TMainForm.NavigateClick1(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;
    BPGN.MakeAllNav1Move;
    engstep:=-INFTY;
    LoadRecord;
    engstep:=1;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.NavigateClick2(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeAllNav2Move;
    engstep:=-1;
    LoadRecord;
    engstep:=1;


    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.NavigateClick3(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeAllNav3Move;
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.NavigateClick4(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeAllNav4Move;
    Engstep:=INFTY;
    LoadRecord;
    engstep:=1;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.NavigateClick5(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    bFlipBoard             := NOT bFlipBoard;
    Board1Form.Board.bFlip := NOT Board1Form.Board.bFlip;
    Board2Form.Board.bFlip := NOT Board2Form.Board.bFlip;

    Board1Form.LoadHandle;
    Board2Form.LoadHandle;

    Board1Form.LoadTime;
    Board2Form.LoadTime;

    Board1Form.DisplayBoard;
    Board2Form.DisplayBoard;

    IF (bRunTimer) THEN
        SetupNextMove
    ELSE
        Timer1.Enabled := FALSE;
END;

PROCEDURE TMainForm.NavigateClick6(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;
 // if Engines[defeng].bEngineOn then NavigateClick9(Sender);
    startmove:=now;
{    startPlay:=startmove;
    nStartIndex:=Bpgn.GetPositionIndex; }
//    if ini1.INI.bShowTenth then timer3.Interval:=100 else timer3.Interval:=1000;
    Loadrecord;
    SetupNextMove;
    NavigateForm.Disable(8);

    bRunTimer      := TRUE;
//    Timer1.Enabled := TRUE;
    navigateForm.SpeedButton6.onclick:=navigateclick7;
    navigateform.speedbutton6.glyph:=glyphs[8]^.bitmap;
    raiseboards;
   // navigateform.speedbutton6.numglyphs:=2;
END;

PROCEDURE TMainForm.NavigateClick7(Sender: TObject);
BEGIN
    StopTimer;
    raiseboards;
END;

PROCEDURE TMainForm.NavigateClick8(Sender: TObject);
BEGIN
    IF (NOT bIgnoreDisp) THEN
    BEGIN
        IF (NavigateForm.SpeedButton8.Tag = 0) THEN
        BEGIN
            navigateform.SetDisplay(true);
            BPGN.SetCurrentDisplay(TRUE);
        END
        ELSE
        BEGIN
            navigateform.SetDisplay(false);
            BPGN.SetCurrentDisplay(FALSE);
        END;
    END;
END;

procedure Tmainform.navigateClick11(Sender:Tobject);
var s:string;
begin

 Timer1.Enabled := FALSE;

 BPGN.MakeAllNav5Move;
 raiseform(commentform);
 engstep:=INFTY;
 LoadRecord;
 engstep:=1;
 IF (bRunTimer) THEN
 SetupNextMove;
end;

procedure Tmainform.navigateClick12(Sender:Tobject);
var s:string;
begin
 Timer1.Enabled := FALSE;
 BPGN.MakeAllNav8Move;
 raiseform(commentform);
 engstep:=INFTY;
 LoadRecord;
 engstep:=1;
 IF (bRunTimer) THEN
 SetupNextMove;
end;

procedure Tmainform.navigateClick14(Sender:Tobject);
begin

 Timer1.Enabled := FALSE;
 BPGN.MakeAllNav6Move;
 engstep:=INFTY;
 LoadRecord;
 engstep:=1;
 IF (bRunTimer) THEN
 SetupNextMove;
 raiseform(nextmoveform);
end;

procedure Tmainform.navigateClick13(Sender:Tobject);
begin

 Timer1.Enabled := FALSE;
 BPGN.MakeAllNav7Move;
 engstep:=INFTY;
 LoadRecord;
 engstep:=1;
 IF (bRunTimer) THEN
 SetupNextMove;
 raiseform(nextmoveform);
end;


Procedure Joke;
Begin
WError('This programm SUCKS'+CR_ASCII_CHAR+'I REFUSE to run it');
WError('Ok, it sux less than Windows, I will give it a try');
Werror('Merry Xmas, Man');
end;

procedure TMainForm.createEngine(i:integer);
begin
 try
   EngineForms[i]:=TEngineForm.create(self);
   engines[i].exist:=true;
   EngineForms[i].homewnd:=handle;
   EngineForms[i].engineN:=longint(i);
   EngineForms[i].opt:=engopt[i];

   EngineForms[i].Memo1.font:=fengineF;
   EngineForms[i].memo1.WordWrap:=ini1.ini.bEngineWr;
   EngineForms[i].memo1.color:=nEngineb;
   showEngine(i);
   if i=defeng then EngineForms[i].bitbtn2.visible:=true;
 except
   engines[i].exist:=false;
   EngineForms[i].free;
   exit;
 end;
end;

procedure TmainForm.RunEngine(i:integer);
var ps:integer;
s:string;
begin
  if not FileExists(Engopt[i].comline) then
  begin
        wError('WARNING: Engine Command Line'+EOL_STR+
        Engopt[i].comline+EOL_STR+
        'points to nonexistant file'+EOL_STR+
        'Go to menu'+EOL_STR+'Engines --> '+engineMenu[i]^.caption+' -->Options'+EOL_STR+'to fix it');
        exit;
  end;
  if not engines[i].exist then
  createengine(i);
  if not engines[i].exist then exit;

  Wengmenu[i]^.enabled:=true;
  if i=defeng then Weng.enabled:=true;
  s:=lowercase(engopt[i].comline);
  ps:=pos('sunsetter',s);
  if ((ps>0) and tecumseh)then
  begin
   tecumseh:=false; Joke;
  end;

  Engines[i].bEngineOn:=true;
  EngineForms[i].opt:=engopt[i];
  plengMenu[i]^.checked:=true;
  EngineForms[i].Run;
  if i=defeng then NavigateForm.Button1.caption:=SJENG_OFF;
end;

procedure TMainForm.StopEngine(i:integer);
begin
 Engines[i].bEngineOn:=false;
 EngineForms[i].Stop;
 plengMenu[i]^.checked:=false;
 if i=defeng then NavigateForm.Button1.caption:=SJeng_ON;
end;

procedure TMainForm.NavigateClick9(Sender: TObject);
begin
 stopTimer;
 If not Engines[defeng].bEngineOn then
 begin
  NavigateForm.Button1.Caption:=SJENG_OFF;
  NavigateForm.button1.hint:='Turns default engine ['+engopt[defeng].name+'] OFF';
  if engines[defeng].suspended then ResumeEngine(defeng) else RunEngine(defeng);
 end
 else begin
  NavigateForm.button1.CAption:=SJENG_ON;
  NavigateForm.button1.hint:='Turns default engine ['+engopt[defeng].name+'] ON';
  StopEngine(defeng);
 end;
end;


PROCEDURE TMainForm.Board1Click1(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;
    engstep:=-INFTY;
    BPGN.MakeBoard1Nav1Move;
    LoadRecord;
    Engstep:=1;
    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board1Click2(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;
        engstep:=-1;
    BPGN.MakeBoard1Nav2Move;
    LoadRecord;
    engstep:=1;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board1Click3(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;
engstep:=1;
    BPGN.MakeBoard1Nav3Move;
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board1Click4(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;
engstep:=INFTY;
    BPGN.MakeBoard1Nav4Move;
    LoadRecord;
    engstep:=1;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board2Click1(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeBoard2Nav1Move;
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board2Click2(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeBoard2Nav2Move;
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board2Click3(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeBoard2Nav3Move;
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.Board2Click4(Sender: TObject);
BEGIN
    Timer1.Enabled := FALSE;

    BPGN.MakeBoard2Nav4Move;
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.NextMoveClick(Sender: TObject);
BEGIN
    IF (NextMoveForm.ListBox1.ItemIndex < 0) THEN
        EXIT;

    IF (NextMoveForm.ListBox1.ItemIndex >= NextMoveForm.ListBox1.Items.Count) THEN
        EXIT;

    Timer1.Enabled := FALSE;

    BPGN.MakeNextMove(NextMoveForm.ListBox1.ItemIndex + 1);
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.PrevMoveClick(Sender: TObject);
BEGIN
    IF (PrevMoveForm.ListBox1.ItemIndex < 0) THEN
        EXIT;

    IF (PrevMoveForm.ListBox1.ItemIndex >= PrevMoveForm.ListBox1.Items.Count) THEN
        EXIT;

    Timer1.Enabled := FALSE;

    BPGN.MakePrevMove(PrevMoveForm.ListBox1.ItemIndex + 1);
    LoadRecord;

    IF (bRunTimer) THEN
        SetupNextMove;
END;

PROCEDURE TMainForm.CommentChange(Sender: TObject);
BEGIN
    IF (NOT bIgnoreMemo) THEN
        BPGN.SetCurrentComment(CommentForm.Memo1.Text);
END;

PROCEDURE TMainForm.Board1MouseDown(Sender: TObject; Button: TMouseButton;
                                    Shift: TShiftState; X, Y: Integer);
var nfromx,nfromy:integer;
BEGIN
    if button=mbRight then with board1form do
    begin
     if ngameindex<0 then exit;
     if bRunTimer then exit;
     with board1form.Board do
     begin
      nfromx:=Indexxy(x-nmxoff);
      nfromy:=indexxy(y-nmyoff);
      if not (nfromx in [1..8]) then exit;
      if not (nfromy in [1..8]) then exit;
     end;
     popupmenu1.Tag:=1;
     board.nPopupDropX:=x-board.nPieceSize div 2;
     board.nPopupDropY:=y - board.nPieceSize div 2;
     popupmenu1.Popup(left+x,Top+y);
    end;
    IF (Button = mbLeft) THEN
    BEGIN
        Board1Form.Board.OnMouseDown(Board1Form.PaintBox1.Canvas, X, Y, Board1Form.Left, Board1Form.Top);
        IF (Board1Form.Board.bMadeMove) THEN
        BEGIN
            IF (Board1Form.Board.Chess.bWhitesMove) THEN
                MadeMove('a', Board1Form.Board.Chess.GetMoveStr(Board1Form.Board.nMoveI, TRUE, FALSE), Board1Form.Board.nMoveI)
            ELSE
                MadeMove('A', Board1Form.Board.Chess.GetMoveStr(Board1Form.Board.nMoveI, TRUE, FALSE), Board1Form.Board.nMoveI);
        Board1Form.Board.bMadeMove:=false; {let's see if this gonna work}
        END;
    END;
END;

PROCEDURE TMainForm.Board1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
BEGIN
    Board1Form.Board.OnMouseMove(Board1Form.PaintBox1.Canvas, X, Y);
END;

PROCEDURE TMainForm.Board1MouseUp(Sender: TObject; Button: TMouseButton;
                                  Shift: TShiftState; X, Y: Integer);
BEGIN
    Board1Form.Board.OnMouseUp(Board1Form.PaintBox1.Canvas, X, Y, Board1Form.Left, Board1Form.Top);
    IF (Board1Form.Board.bMadeMove) THEN
    BEGIN
        IF (Board1Form.Board.Chess.bWhitesMove) THEN
            MadeMove('a', Board1Form.Board.Chess.GetMoveStr(Board1Form.Board.nMoveI, TRUE, FALSE), Board1Form.Board.nMoveI)
        ELSE
            MadeMove('A', Board1Form.Board.Chess.GetMoveStr(Board1Form.Board.nMoveI, TRUE, FALSE), Board1Form.Board.nMoveI);
    Board1Form.Board.bMadeMove:=false; {let's see if this gonna work}
    END;
END;

PROCEDURE TMainForm.Board2MouseDown(Sender: TObject; Button: TMouseButton;
                                    Shift: TShiftState; X, Y: Integer);
var nfromx,nfromy:integer;
BEGIN
    if button=mbRight then with board2form do
    begin
     if nGameIndex<0 then exit;
     if bRunTimer then exit;
     with board2form.Board do
     begin
      nfromx:=Indexxy(x-nmxoff);
      nfromy:=indexxy(y-nmyoff);
      if not (nfromx in [1..8]) then exit;
      if not (nfromy in [1..8]) then exit;
     end;
     popupmenu1.Tag:=2;
     board.nPopupDropX:=x-board.nPieceSize div 2;
     board.nPopupDropY:=y - board.nPieceSize div 2;
     popupmenu1.Popup(left+x,Top+y);
    end;
    IF (Button = mbLeft) THEN
    BEGIN
        Board2Form.Board.OnMouseDown(Board2Form.PaintBox1.Canvas, X, Y, Board2Form.Left, Board2Form.Top);
        IF (Board2Form.Board.bMadeMove) THEN
        BEGIN
            IF (Board2Form.Board.Chess.bWhitesMove) THEN
                MadeMove('b', Board2Form.Board.Chess.GetMoveStr(Board2Form.Board.nMoveI, TRUE, FALSE), Board2Form.Board.nMoveI)
            ELSE
                MadeMove('B', Board2Form.Board.Chess.GetMoveStr(Board2Form.Board.nMoveI, TRUE, FALSE), Board2Form.Board.nMoveI);
        Board2Form.Board.bMadeMove:=false; {let's see if this gonna work}
        END;
    END;
END;

PROCEDURE TMainForm.Board2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
BEGIN
    Board2Form.Board.OnMouseMove(Board2Form.PaintBox1.Canvas, X, Y);
END;

PROCEDURE TMainForm.Board2MouseUp(Sender: TObject; Button: TMouseButton;
                                  Shift: TShiftState; X, Y: Integer);
BEGIN
    Board2Form.Board.OnMouseUp(Board2Form.PaintBox1.Canvas, X, Y, Board2Form.Left, Board2Form.Top);
    IF (Board2Form.Board.bMadeMove) THEN
    BEGIN
        IF (Board2Form.Board.Chess.bWhitesMove) THEN
            MadeMove('b', Board2Form.Board.Chess.GetMoveStr(Board2Form.Board.nMoveI, TRUE, FALSE), Board2Form.Board.nMoveI)
        ELSE
            MadeMove('B', Board2Form.Board.Chess.GetMoveStr(Board2Form.Board.nMoveI, TRUE, FALSE), Board2Form.Board.nMoveI);
    Board2Form.Board.bMadeMove:=false; {let's see if this gonna work}
    END;
END;

PROCEDURE TMainForm.LoadRecord(bFast:boolean=false);
VAR
    nN, nI: Integer;
    cBoard: Char;
    cMove:  String;
    i:integer;
    tmp:boolean;
    fposition:TintBoard;
    nPosIndex:longint;
BEGIN
    nPosIndex:=Bpgn.GetPositionIndex;
    IF (nPosIndex < 1) THEN
        EXIT;


    { Adjust board #1 and board #2 }
    BPGN.CopyBPGNRecordIntoBoards(Board1Form.Board.Chess, Board2Form.Board.Chess,
                                  nPosIndex);
    if ((nPosIndex>0) and (bFast)) then
    begin
      if bpgn.iszh(nGameIndex) then
      begin
       with Board1form.Board.Chess.mprev do Board1Form.MakeMove(nFromX,nFromY,nToX,nToY,board1form.Board.Chess.GetRevPiece(cCapture));
      end
      else begin
       with Board1form.Board.Chess.mPrev do Board1Form.MakeMove(nFromX,nFromY,nToX,nToY,Board2Form.Board.Chess.mPrev.cCapture);
       with Board2form.Board.Chess.mPrev do Board2Form.MakeMove(nFromX,nFromY,nToX,nToY,Board1Form.Board.Chess.mPrev.cCapture);
      end;
    end
    else begin
            Board1Form.DisplayBoard;
            Board2Form.DisplayBoard;
    end;
    Board1form.nWtime := bpgn.GetBPGNnWTime(nPosIndex,1);
    Board1form.nBtime := bpgn.GetBPGNnBTime(nPosIndex,1);
    Board1Form.cWTime := BPGN.GetCurrentBoard1WTime;
    Board1Form.cBTime := BPGN.GetCurrentBoard1BTime;
    Board1form.nLag   := Bpgn.GetCurrentLag(1);
    Board1form.nWLag  := Bpgn.GetCurrentWLag(1);
    board1form.nBLag  := Bpgn.GetCurrentBLag(1);

    Board1Form.LoadTime;
    Board1Form.ShowMoveLag;
    Board1Form.Caption := BPGN.GetCurrentBoard1Caption;

    Board2form.nWtime := bpgn.GetBPGNnWTime(nPosIndex,2);
    Board2form.nBtime := bpgn.GetBPGNnBTime(nPosIndex,2);
    Board2Form.cWTime := BPGN.GetCurrentBoard2WTime;
    Board2Form.cBTime := BPGN.GetCurrentBoard2BTime;
    Board2form.nLag   := Bpgn.GetCurrentLag(2);
    Board2form.nWLag  := Bpgn.GetCurrentWLag(2);
    board2form.nBLag  := Bpgn.GetCurrentBLag(2);


    Board2Form.LoadTime;
    Board2Form.ShowMoveLag;
    Board2Form.Caption := BPGN.GetCurrentBoard2Caption;


    { Adjust next move }
    NextMoveForm.ClearAll;
    BPGN.LoadNextMove;
    nN := BPGN.GetNextCount;
    FOR nI := 1 TO nN DO
    BEGIN
        cBoard := BPGN.GetNextBoard(nI);
        cMove  := BPGN.GetNextMove(nI);

        IF (INI1.INI.bZero) THEN
        BEGIN
            IF (CompareText(cMove, 'O-O-O') = 0) THEN
                cMove := '0-0-0'
            ELSE
            IF (CompareText(cMove, 'O-O') = 0) THEN
                cMove := '0-0';
        END;

        IF (cBoard = 'A') THEN
        BEGIN
            NextMoveForm.AddItem(IntToStr(BPGN.GetNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(TRUE, cMove) +
                                 Trim(BPGN.GetNextAnno(nI)));
        END
        ELSE
        IF (cBoard = 'a') THEN
        BEGIN
            NextMoveForm.AddItem(IntToStr(BPGN.GetNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(FALSE, cMove) +
                                 Trim(BPGN.GetNextAnno(nI)));
        END
        ELSE
        IF (cBoard = 'B') THEN
        BEGIN
            NextMoveForm.AddItem(IntToStr(BPGN.GetNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(TRUE, cMove) +
                                 Trim(BPGN.GetNextAnno(nI)));
        END
        ELSE
        IF (cBoard = 'b') THEN
        BEGIN
            NextMoveForm.AddItem(IntToStr(BPGN.GetNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(FALSE, cMove) +
                                 Trim(BPGN.GetNextAnno(nI)));
        END;
    END;


    { Adjust previous next move }
    PrevMoveForm.ClearAll;
    BPGN.LoadPNextMove;
    nN := BPGN.GetPNextCount;
    FOR nI := 1 TO nN DO
    BEGIN
        cBoard := BPGN.GetPNextBoard(nI);
        cMove  := BPGN.GetPNextMove(nI);

        IF (INI1.INI.bZero) THEN
        BEGIN
            IF (CompareText(cMove, 'O-O-O') = 0) THEN
                cMove := '0-0-0'
            ELSE
            IF (CompareText(cMove, 'O-O') = 0) THEN
                cMove := '0-0';
        END;

        IF (cBoard = 'A') THEN
        BEGIN
            PrevMoveForm.AddItem(IntToStr(BPGN.GetPNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(TRUE, cMove) +
                                 Trim(BPGN.GetPNextAnno(nI)));
        END
        ELSE
        IF (cBoard = 'a') THEN
        BEGIN
            PrevMoveForm.AddItem(IntToStr(BPGN.GetPNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(FALSE, cMove) +
                                 Trim(BPGN.GetPNextAnno(nI)));
        END
        ELSE
        IF (cBoard = 'B') THEN
        BEGIN
            PrevMoveForm.AddItem(IntToStr(BPGN.GetPNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(TRUE, cMove) +
                                 Trim(BPGN.GetPNextAnno(nI)));
        END
        ELSE
        IF (cBoard = 'b') THEN
        BEGIN
            PrevMoveForm.AddItem(IntToStr(BPGN.GetPNextNumber(nI)) + cBoard + '. ' +
                                 GetRTFMove(FALSE, cMove) +
                                 Trim(BPGN.GetPNextAnno(nI)));
        END;
    END;


    { Adjust boards }
    IF (bRunTimer) THEN
    BEGIN
        Board1Form.Board.bDisable := TRUE;
        Board2Form.Board.bDisable := TRUE;
    END
    ELSE
    BEGIN
        Board1Form.Board.bDisable := (nN >= MAX_NEXT);
        Board2Form.Board.bDisable := Board1Form.Board.bDisable;
    END;


    { Adjust comment }
    bIgnoreMemo := TRUE;
    CommentForm.Memo1.Text    := BPGN.GetCurrentComment;
    CommentForm.Memo1.Enabled := (NOT bRunTimer);
    bIgnoreMemo := FALSE;


    { Adjust display }
    bIgnoreDisp := TRUE;
    navigateform.SetDisplay(BPGN.GetCurrentDisplay);

    NavigateForm.SpeedButton8.Enabled := (NOT bRunTimer);
    bIgnoreDisp := FALSE;


    { Adjust navigators }
    BPGN.LoadNavigator;
    IF (BPGN.GetAllNavigator(1)) THEN
        NavigateForm.Enable(1)
    ELSE
        NavigateForm.Disable(1);

    IF (BPGN.GetAllNavigator(2)) THEN
        NavigateForm.Enable(2)
    ELSE
        NavigateForm.Disable(2);

    IF (BPGN.GetAllNavigator(3)) THEN
        NavigateForm.Enable(3)
    ELSE
        NavigateForm.Disable(3);

    IF (BPGN.GetAllNavigator(4)) THEN
        NavigateForm.Enable(4)
    ELSE
        NavigateForm.Disable(4);

    NavigateForm.SpeedButton9.enabled:=NavigateForm.speedbutton2.enabled;
    NavigateForm.SpeedButton10.enabled:=NavigateForm.speedbutton3.enabled;
    navigateform.Speedbutton12.enabled:=navigateForm.speedbutton3.enabled;
    navigateform.speedbutton13.enabled:=navigateform.speedbutton2.Enabled;

    if (BPGN.GetBoard1Navigator(1)<>Board1Form.SpeedButton1.Enabled) then Board1Form.SpeedButton1.Enabled:=not Board1Form.SpeedButton1.Enabled;
    if (BPGN.GetBoard1Navigator(2)<>Board1Form.SpeedButton2.Enabled) then Board1Form.SpeedButton2.Enabled:=not Board1Form.SpeedButton2.Enabled;
    if (BPGN.GetBoard1Navigator(3)<>Board1Form.SpeedButton3.Enabled) then Board1Form.SpeedButton3.Enabled:=not Board1Form.SpeedButton3.Enabled;
    if (BPGN.GetBoard1Navigator(4)<>Board1Form.SpeedButton4.Enabled) then Board1Form.SpeedButton4.Enabled:=not Board1Form.SpeedButton4.Enabled;

    if (BPGN.GetBoard2Navigator(1)<>Board2Form.SpeedButton1.Enabled) then Board2Form.SpeedButton1.Enabled:=not Board2Form.SpeedButton1.Enabled;
    if (BPGN.GetBoard2Navigator(2)<>Board2Form.SpeedButton2.Enabled) then Board2Form.SpeedButton2.Enabled:=not Board2Form.SpeedButton2.Enabled;
    if (BPGN.GetBoard2Navigator(3)<>Board2Form.SpeedButton3.Enabled) then Board2Form.SpeedButton3.Enabled:=not Board2Form.SpeedButton3.Enabled;
    if (BPGN.GetBoard2Navigator(4)<>Board2Form.SpeedButton4.Enabled) then Board2Form.SpeedButton4.Enabled:=not Board2Form.SpeedButton4.Enabled;

    lastmove:=bpgn.GetCurrentMove;
    if lastmove='' then lastmove:='Start';
    BPGN.GetBpgnLineUpTo(1);

    for i:=1 to MAX_ENGINE do
    if (engines[i].bengineon) then  EngineForms[i].updateeng(engstep);
    if fform.checkbox13.Checked then
    begin
     statgames(fform.Button4);
     if fform.WindowState=wsMinimized then fform.WindowState:=wsNormal;
    end;

END;

PROCEDURE TMainForm.MadeMove(cBoard: Char; cMove: String; nMI: Integer);
BEGIN
    BPGN.MakeBoardMove(Board1Form.Board.Chess, Board2Form.Board.Chess,
                       cBoard, cMove, nMI,
                       Board1Form.Caption, Board2Form.Caption);

    LoadRecord;
END;

PROCEDURE TMainForm.ProcessBeg(cSt: String);
BEGIN
    IF (Length(cOrgCaption) = 0) THEN
    BEGIN
        cOrgCaption   := Caption;
        Caption       := cSt;
        Screen.Cursor := crHourGlass;
    END;
END;

PROCEDURE TMainForm.ProcessMid(cSt: String);
BEGIN
    Caption := cSt;
END;

PROCEDURE TMainForm.ProcessEnd;
BEGIN
    Screen.Cursor := crDefault;
    Caption       := cOrgCaption;
    cOrgCaption   := '';
END;

PROCEDURE TMainForm.ExtractWriteError(nWhich: Integer);
BEGIN
    IF (bExtractErrFn) THEN
    BEGIN
        bExtractErrFn := FALSE;

        IF (FileExists(cExtractErrFn)) THEN
            DeleteFile(cExtractErrFn);

        IF (nWhich = 1) THEN
        BEGIN
            TOOLBOX_TxtWriteln(cExtractErrFn, FormatDateTime('yyyy.mm.dd, hh:nn', Now) + EOL_STR +
                               EOL_STR + BPGN.GetErrorMessage);
        END
        ELSE
        BEGIN
            TOOLBOX_TxtWriteln(cExtractErrFn, FormatDateTime('yyyy.mm.dd, hh:nn', Now) + EOL_STR +
                               EOL_STR + BPGN1.GetErrorMessage);
        END;
    END
    ELSE
    BEGIN
        IF (nWhich = 1) THEN
            TOOLBOX_TxtWriteln(cExtractErrFn, EOL_STR + BPGN.GetErrorMessage)
        ELSE
            TOOLBOX_TxtWriteln(cExtractErrFn, EOL_STR + BPGN1.GetErrorMessage);
    END;
END;

FUNCTION TMainForm.ExtractMovesIntoFile(cMatchBoard: Char; cGame: String): Integer;
VAR
    cB, cBoard: Char;
    nResult, nDestination, nSource, nMain, nTime, nMove: Integer;
BEGIN
    nResult       := 1;
    nDestination  := 1;
    nSource       := 1;

    WHILE (TRUE) DO
    BEGIN
        nMain := BPGN1.GetBPGNMain(nSource);
        IF (nMain = 0) THEN
            BREAK;

        cB := BPGN1.GetBPGNBoard(nMain);
        IF (UpCase(cB) = cMatchBoard) THEN
        BEGIN
            IF (cB = 'A') THEN
            BEGIN
                nMove  := BPGN1.GetBPGNNumber(nMain,1);
                nTime  := BPGN1.GetBPGNnWTime(nMain,1);
                cBoard := 'A';
            END
            ELSE
            IF (cB = 'a') THEN
            BEGIN
                nMove  := BPGN1.GetBPGNNumber(nMain,1);
                nTime  := BPGN1.GetBPGNnBTime(nMain,1);
                cBoard := 'a';
            END
            ELSE
            IF (cB = 'B') THEN
            BEGIN
                nMove  := BPGN1.GetBPGNNumber(nMain,2);
                nTime  := BPGN1.GetBPGNnWTime(nMain,2);
                cBoard := 'A';
            END
            ELSE
            IF (cB = 'b') THEN
            BEGIN
                nMove  := BPGN1.GetBPGNNumber(nMain,2);
                nTime  := BPGN1.GetBPGNnBTime(nMain,2);
                cBoard := 'a';
            END;

            IF (nMove <= INI1.INI.nMoveNumber) THEN
            BEGIN
                nDestination := BPGN.AddExtractMove(nDestination, nMove, cBoard,
                                                    BPGN1.GetBPGNMove(nMain),
                                                    nTime, cGame);
                IF (nDestination < 1) THEN
                BEGIN
                    nResult := nDestination;
                    BREAK;
                END;
            END
            ELSE
            BEGIN
                BREAK;
            END;
        END;

        nSource := nMain;
    END;

    Result := nResult;
END;

FUNCTION TMainForm.ExtractMovesFromFile(cFn, cColor: String; cMatchBoard: Char): Boolean;
VAR
    cB: Char;
    nH, nI, nR: Integer;
    br,bResult: Boolean;
    cG: String;
BEGIN
    IF (CompareText(INI1.INI.cDestination, cFn) = 0) THEN
    BEGIN
        Result := TRUE;
        EXIT;
    END;

    ProcessMid(PROGRAM_TITLE + ' (Loading source BPGN file '  + cFn + ' ' + cColor + '...)');
    IF (NOT BPGN1.OpenBPGN(cFn)) THEN
    BEGIN
        ExtractWriteError(2);
        Result := TRUE;
        EXIT;
    END;
    ProcessMid(PROGRAM_TITLE + ' (Scanning source BPGN header ' + cColor + '...)');
    try
     DoLongRun(fLongThread,bpgn1.LoadHeader_,@br,'Parsing headers...',true);
    except
     bpgn1.CloseBPGN(false); ProcessEnd; Caption:=Program_title; exit;
    end;
    IF (NOT br) THEN
        ExtractWriteError(2);

    bResult := TRUE;
    nH      := BPGN1.GetHeaderCount;
    FOR nI := 0 TO nH-1 DO
    BEGIN
        ProcessMid(PROGRAM_TITLE + ' (Processing source BPGN file [' +
                   IntToStr(nI) + '/' + IntToStr(nH) + '] ' + cColor + '...)');

        IF (cMatchBoard = 'A') THEN
        BEGIN
            IF (CompareText(INI1.INI.cHandle, BPGN1.GetWhiteA(nI)) = 0) THEN
            BEGIN
                cB := 'A';
                cG := ' - ' + BPGN1.GetWhiteA(nI) + ' vs ' + BPGN1.GetBlackA(nI) +
                      ' (' + ExtractFilename(cFn) + ', Game ' + IntToStr(nI) + ')>';
            END
            ELSE
            IF (CompareText(INI1.INI.cHandle, BPGN1.GetWhiteB(nI)) = 0) THEN
            BEGIN
                cB := 'B';
                cG := ' - ' + BPGN1.GetWhiteB(nI) + ' vs ' + BPGN1.GetBlackB(nI) +
                      ' (' + ExtractFilename(cFn) + ', Game ' + IntToStr(nI) + ')>';
            END
            ELSE
                CONTINUE;
        END
        ELSE
        BEGIN
            IF (CompareText(INI1.INI.cHandle, BPGN1.GetBlackA(nI)) = 0) THEN
            BEGIN
                cB := 'A';
                cG := ' - ' + BPGN1.GetWhiteA(nI) + ' vs ' + BPGN1.GetBlackA(nI) +
                      ' (' + ExtractFilename(cFn) + ', Game ' + IntToStr(nI) + ')>';
            END
            ELSE
            IF (CompareText(INI1.INI.cHandle, BPGN1.GetBlackB(nI)) = 0) THEN
            BEGIN
                cB := 'B';
                cG := ' - ' + BPGN1.GetWhiteB(nI) + ' vs ' + BPGN1.GetBlackB(nI) +
                      ' (' + ExtractFilename(cFn) + ', Game ' + IntToStr(nI) + ')>';
            END
            ELSE
                CONTINUE;
        END;

        IF (BPGN1.LoadGame(nI)) THEN
        BEGIN
            nR := ExtractMovesIntoFile(cB, cG);
            IF (nR < 1) THEN
            BEGIN
                ExtractWriteError(1);
                IF (nR = -1) THEN
                BEGIN
                    bResult := FALSE;
                    BREAK;
                END;
            END;
        END
        ELSE
        BEGIN
            ExtractWriteError(2);

            IF (Pos('Maximum # of positions exceeded (', BPGN1.GetErrorMessage) > 0) THEN
            BEGIN
                nR := ExtractMovesIntoFile(cB, cG);
                IF (nR < 1) THEN
                BEGIN
                    ExtractWriteError(1);
                    IF (nR = -1) THEN
                    BEGIN
                        bResult := FALSE;
                        BREAK;
                    END;
                END;
            END;
        END;
    END;

    ProcessMid(PROGRAM_TITLE + ' (Closing source BPGN file ' + cColor + '...)');
    BPGN1.CloseBPGN(TRUE);

    Result := bResult;
END;

PROCEDURE TMainForm.ExtractMoves;
VAR
    rSearch: TSearchRec;
    nResult: Integer;
    cSDir:   String;
    nL,itmp:      Integer;
    bRes:boolean;
    rec:tBPGnrec;
BEGIN
    bExtractErrFn := TRUE;

    IF (NOT FileExists(INI1.INI.cDestination)) THEN
    BEGIN
        ProcessMid(PROGRAM_TITLE + ' (Creating destination BPGN file...)');
        BPGN.NewFile(INI1.INI.cDestination, INI1.INI.cHandle, DEFAULT_BLACKA, DEFAULT_WHITEB, DEFAULT_BLACKB)
    END;

    ProcessMid(PROGRAM_TITLE + ' (Loading destination BPGN file...)');
    IF (NOT BPGN.OpenBPGN(INI1.INI.cDestination)) THEN
        EXIT;

    ProcessMid(PROGRAM_TITLE + ' (Scanning destination BPGN header...)');
    try
     DoLongRun(fLongThread,bpgn.LoadHeader_,@bres,'Parsing headers...',true);
    except
     bpgn.CloseBPGN(false); ProcessEnd; Caption:=Program_title; exit;
    end;
    bpgn.nHeaderCount:=bpgn.aList.Count;
    bpgn.nFilterCount:=bpgn.nHeaderCount;
    IF (NOT bres) THEN
    BEGIN
        BPGN.CloseBPGN(TRUE);
        EXIT;
    END;

    BpgnRecFillempty(@rec);
    ProcessMid(PROGRAM_TITLE + ' (Adjusting destination BPGN header...)');
    IF (BPGN.GetHeaderCount = 0) THEN
    BEGIN
        BPGN.AppendNewGame(@rec);
        BPGN.AppendNewGame(@rec);
    END
    ELSE
    IF (BPGN.GetHeaderCount = 1) THEN
        BPGN.AppendNewGame(@rec);

    BPGN.SetWhiteA(0, INI1.INI.cHandle);
    BPGN.SetBlackA(0, DEFAULT_BLACKA);
    BPGN.SetWhiteB(0, DEFAULT_WHITEB);
    BPGN.SetBlackB(0, DEFAULT_BLACKB);

    BPGN.SetWhiteA(1, DEFAULT_WHITEA);
    BPGN.SetBlackA(1, INI1.INI.cHandle);
    BPGN.SetWhiteB(1, DEFAULT_WHITEB);
    BPGN.SetBlackB(1, DEFAULT_BLACKB);

    ProcessMid(PROGRAM_TITLE + ' (Loading destination BPGN game [white]...)');
    IF (NOT BPGN.LoadGame(0)) THEN
    BEGIN
        BPGN.CloseBPGN(TRUE);
        EXIT;
    END;

    IF (INI1.INI.bRemoveComment) THEN
    BEGIN
        ProcessMid(PROGRAM_TITLE + ' (Removing all destination comments [white]...)');
        BPGN.DeleteAllComments;
    END;

    ProcessMid(PROGRAM_TITLE + ' (Creating source BPGN object...)');
    BPGN1 := TBPGN.Create;
    bpgn1.init;
    BPGN1.SetPromptError(FALSE);

    ProcessMid(PROGRAM_TITLE + ' (Searching source BPGN filter [white]...)');
    cSDir := ExtractFileDir(INI1.INI.cSource);
    nL    := Length(cSDir);
    IF (nL > 0) THEN
    BEGIN
        IF (cSDir[nL] <> '\') THEN
            cSDir := cSDir + '\';
    END;

    BPGN.SetPromptError(FALSE);
    nResult := FindFirst(INI1.INI.cSource, (faReadOnly + faArchive), rSearch);
    WHILE (nResult = 0) DO
    BEGIN
        IF (NOT ExtractMovesFromFile(cSDir + ExtractFileName(rSearch.Name), '[white]', 'A')) THEN
            BREAK;
        nResult := FindNext(rSearch);
    END;
    FindClose(rSearch);
    BPGN.SetPromptError(TRUE);

    ProcessMid(PROGRAM_TITLE + ' (Saving destination BPGN game [white]...)');
    itmp:=0;
    DoLongRun(fLongThread,bpgn.SaveGame_,@itmp,'Saving game...');

    ProcessMid(PROGRAM_TITLE + ' (Loading destination BPGN game [black]...)');
    IF (BPGN.LoadGame(1)) THEN
    BEGIN
        IF (INI1.INI.bRemoveComment) THEN
        BEGIN
            ProcessMid(PROGRAM_TITLE + ' (Removing all destination comments [black]...)');
            BPGN.DeleteAllComments;
        END;

        ProcessMid(PROGRAM_TITLE + ' (Searching source BPGN filter [black]...)');
        BPGN.SetPromptError(FALSE);
        nResult := FindFirst(INI1.INI.cSource, (faReadOnly + faArchive), rSearch);
        WHILE (nResult = 0) DO
        BEGIN
            IF (NOT ExtractMovesFromFile(cSDir + ExtractFileName(rSearch.Name), '[black]', 'B')) THEN
                BREAK;
            nResult := FindNext(rSearch);
        END;
        FindClose(rSearch);
        BPGN.SetPromptError(TRUE);

        ProcessMid(PROGRAM_TITLE + ' (Saving destination BPGN game [black]...)');
        itmp:=1;
        DoLongRun(fLongThread,bpgn.SaveGame_,@itmp,'Saving game...');
    END;

    ProcessMid(PROGRAM_TITLE + ' (Closing destination BPGN file...)');
    BPGN.CloseBPGN(TRUE);

    ProcessMid(PROGRAM_TITLE + ' (Destroying source BPGN object...)');
    BPGN1.Destroy;

    IF (bExtractErrFn) THEN
    BEGIN
        IF (FileExists(cExtractErrFn)) THEN
            DeleteFile(cExtractErrFn);
    END;
END;

PROCEDURE TMainForm.StopTimer;
BEGIN
    IF (bRunTimer) THEN
    BEGIN
        timer3.enabled :=false;
        Timer1.Enabled := FALSE;
        bRunTimer      := FALSE;
        nTick          := 0;
        nMaxTick       := 0;

        LoadRecord;


    navigateform.speedbutton6.onclick:=navigateclick6;
    navigateform.speedbutton6.glyph:=glyphs[7]^.bitmap;
    navigateform.speedbutton6.numglyphs:=2;
    END;
END;

PROCEDURE TMainForm.SetupNextMove;
const nDelay=5000;
VAR
    nC, nM, nT: Integer;
    cB: Char;
    nInt,playlenA,playlenB,playLen:longint;
    nowTime:TdateTime;
    tw1,tw2,tb1,tb2:integer;
BEGIN
    nC := BPGN.GetPositionIndex;
    IF (nC <= 0) THEN
    BEGIN
        nMaxTick := 0;
        EXIT;
    END;


    nM := BPGN.GetBPGNMain(nC);
    IF (nM <= 0) THEN
    BEGIN
{        EndPlay:=now;
        playlen:=Round((EndPlay-startPlay)*DATE_TIME_TO_MS);
        if nStartIndex>1 then
        begin
         tw1:=bpgn.GetBPGNnWTime(nStartIndex,1);
         tw2:=bpgn.GetBPGNnWTime(nStartIndex,2);
         tb1:=bpgn.GetBPGNnBTime(nStartIndex,1);
         tb2:=bpgn.GetBPGNnBTime(nStartIndex,2);
        end
        else begin
         tw1:=nStartTime;tw2:=nStartTime;
         tb1:=nsTartTime;tb2:=nsTartTime;
        end;
        playlenA:=tw1-bpgn.GetBPGNnWTime(nc,1)+tb1-Bpgn.GetBPGNnBTime(nc,1)+Bpgn.GetCurrentWLag(1)+bpgn.GetCurrentBlag(1)-bpgn.BPGNArray[nStartIndex,1].nWLag-bpgn.bpgnarray[nStartIndex,1].nBLag;
        playlenB:=tw2-bpgn.GetBPGNnWTime(nc,2)+tb2-Bpgn.GetBPGNnBTime(nc,2)+Bpgn.GetCurrentWLag(2)+bpgn.GetCurrentBlag(2)-bpgn.BPGNArray[nStartIndex,2].nWLag-bpgn.bpgnarray[nStartIndex,2].nBLag;

        commentform.Memo1.Text:=CommentForm.Memo1.Text+
        'Playtime='+Inttostr(playlen)+' ms'+EOL_STR+
        'Bpgn time A='+Inttostr(playlena)+ ' ms'+EOL_STR+
        'Bpgn time B='+Inttostr(playlenb)+ ' ms'+EOL_STR+
        'The difference is '+Inttostr( min( abs(playlen-playlena), abs(playlen-playlenb) ) div 1000)+' sec'+EOL_STR+
        'Ideal time:'+sIdealT+EOL_STR+
        'Delays    :'+sDelayT+EOL_STR+
        'Real time :'+sTProblem+EOL_STR;
}        nMaxTick := 0;
        timer3.Enabled:=false;
        playgamesound(1);
        if replayallgames1.Checked then {wait a bit, then play the next game}
        begin
          nowTime := now;
          repeat
          Application.ProcessMessages;
          until Now > nowTime + nDelay/DATE_TIME_TO_MS;

          with gamelistform do
          if listbox1.ItemIndex<(listbox1.Items.Count-1) then listbox1.ItemIndex:=listbox1.ItemIndex+1
          else listbox1.ItemIndex:=0;
          gamelistclick(self);
          game4click(self);
        end;

        EXIT;
    END;



    cB := BPGN.GetBPGNBoard(nM);

        IF (cB = 'A') THEN
            nT := BPGN.GetBPGNnWTime(nC,1)
        ELSE
        IF (cB = 'a') THEN
            nT := BPGN.GetBPGNnBTime(nC,1)
        ELSE
        IF (cB = 'B') THEN
            nT := BPGN.GetBPGNnWTime(nC,2)
        ELSE
        IF (cB = 'b') THEN
            nT := BPGN.GetBPGNnBTime(nC,2);

        inc(nt,bpgn.bpgnLag[nm]);

    case cB of
    'A': nMaxTick := nT - BPGN.GetBPGNnWTime(nM,1);
    'a': nMaxTick := nT - BPGN.GetBPGNnBTime(nM,1);
    'B': nMaxTick := nT - BPGN.GetBPGNnWTime(nM,2);
    'b': nMaxTick := nT - BPGN.GetBPGNnBTime(nM,2);
    END;
    if (not bpgn.bLagTag) then
    begin
     if (nMaxTick<0) then nMaxTick:=GetLagDelay(nMaxTick); //out of sync due to lag
     if (nMaxTick<MOVE_OLD_DELAY) then nMaxTick:=MOVE_OLD_DELAY;
    end;
//    sIdealT:=sIdealT+Inttostr(nMaxTick)+'+';
    nowTime:=now;
    nInt:=Round((nowTime-startMove)*DATE_TIME_TO_MS)+1;
    if nMaxTick>nInt then timer1.Interval:=nMaxTick-nInt
    else timer1.Interval:=1;
//    sDelayT:=sDelayT+Inttostr(nInt)+'+';
//    sTProblem:=STProblem+Inttostr(timer1.Interval)+' ';
    timer1.Enabled:=true;
    timer3.Enabled:=true;
END;

procedure TmainForm.playgamesound(i:integer);
begin
 if not applygamesound1.checked then exit;
 case i of
 0: srwave1.wavename:=ini1.ini.csoundmove;
 1: srwave1.wavename:=ini1.ini.csoundend;
 2: srwave1.wavename:=ini1.ini.csoundcap;
 3: srwave1.wavename:=ini1.ini.csoundcheck;
 end;
 if lowercase(srwave1.wavename)='none'  then exit;
 try
  srwave1.loop:=false;
  srwave1.async:=true;
  srwave1.Play;
 except
 end;
end;

PROCEDURE TMainForm.Timer1Timer(Sender: TObject);
VAR
    nC: Integer;lm:string;
    nElapsedUnits:longint;
BEGIN
    StartMove:=Now;
    Timer1.Enabled := FALSE;
    timer3.Enabled:= false;
    if navigateform.speedbutton3.enabled then
    BPGN.MakeAllNav3Move;
    lm:=bpgn.getcurrentmove;
    if (pos('x',lm)>0) then playgamesound(2) //capture
    else  playgamesound(0);
    if (pos('+',lm)>0) then playgamesound(3); //check
    LoadRecord(true);
    SetupNextMove;
END;

PROCEDURE TMainForm.NavigateFormKeyDown(Sender: TObject; VAR Key: Word; Shift: TShiftState);
BEGIN
    IF (Key = 37) THEN
    BEGIN
        { Left arrow key }
        IF (NavigateForm.SpeedButton2.Enabled) THEN NavigateClick2(Sender);
    END
    ELSE
    IF (Key = 38) THEN
    BEGIN
        { Up arrow key }
        IF (NavigateForm.SpeedButton1.Enabled) THEN NavigateClick1(Sender);
    END
    ELSE
    IF (Key = 39) THEN
    BEGIN
        { Right arrow key }
        IF (NavigateForm.SpeedButton3.Enabled) THEN NavigateClick3(Sender);
    END
    ELSE
    IF (Key = 40) THEN
    BEGIN
        { Down arrow key }
        IF (NavigateForm.SpeedButton4.Enabled) THEN NavigateClick4(Sender);
    END;
END;

PROCEDURE TMainForm.Board1FormKeyDown(Sender: TObject; VAR Key: Word; Shift: TShiftState);
BEGIN
    IF (Key = 37) THEN
    BEGIN
        { Left arrow key }
        IF (Board1Form.SpeedButton2.Enabled) THEN Board1Click2(Sender);
    END
    ELSE
    IF (Key = 38) THEN
    BEGIN
        { Up arrow key }
        IF (Board1Form.SpeedButton1.Enabled) THEN Board1Click1(Sender);
    END
    ELSE
    IF (Key = 39) THEN
    BEGIN
        { Right arrow key }
        IF (Board1Form.SpeedButton3.Enabled) THEN Board1Click3(Sender);
    END
    ELSE
    IF (Key = 40) THEN
    BEGIN
        { Down arrow key }
        IF (Board1Form.SpeedButton4.Enabled) THEN Board1Click4(Sender);
    END;
END;

PROCEDURE TMainForm.Board2FormKeyDown(Sender: TObject; VAR Key: Word; Shift: TShiftState);
BEGIN
    IF (Key = 37) THEN
    BEGIN
        { Left arrow key }
        IF (Board2Form.SpeedButton2.Enabled) THEN Board2Click2(Sender);
    END
    ELSE
    IF (Key = 38) THEN
    BEGIN
        { Up arrow key }
        IF (Board2Form.SpeedButton1.Enabled) THEN Board2Click1(Sender);
    END
    ELSE
    IF (Key = 39) THEN
    BEGIN
        { Right arrow key }
        IF (Board2Form.SpeedButton3.Enabled) THEN Board2Click3(Sender);
    END
    ELSE
    IF (Key = 40) THEN
    BEGIN
        { Down arrow key }
        IF (Board2Form.SpeedButton4.Enabled) THEN Board2Click4(Sender);
    END;
END;

PROCEDURE TMainForm.MapShortCutKeys;
BEGIN
    { File menu }
    File1.ShortCut  := ShortCut(Word('N'), [ssAlt]);
    File2.ShortCut  := ShortCut(Word('O'), [ssAlt]);
    File91.ShortCut := ShortCut(Word('S'), [ssAlt]);
    File7.ShortCut  := ShortCut(Word('X'), [ssAlt]);

    { Game menu }
    Game22.ShortCut := ShortCut(Word('G'), [ssAlt]);
    Game24.ShortCut := ShortCut(Word('D'), [ssAlt]);
    Game25.ShortCut := ShortCut(Word('U'), [ssAlt]);
    Game3.ShortCut  := ShortCut(Word('F'), [ssAlt]);
    Game7.ShortCut  := ShortCut(Word('E'), [ssAlt]);
    Game19.ShortCut := ShortCut(Word('V'), [ssAlt]);

    { Move menu }
    Move3.ShortCut := ShortCut(Word('A'), [ssAlt]);

    { Options menu }

    { Utility menu }

    { Windows menu }
    Windows1.ShortCut := ShortCut(Word('1'), [ssAlt]);
    Windows2.ShortCut := ShortCut(Word('2'), [ssAlt]);
    Windows3.ShortCut := ShortCut(Word('C'), [ssAlt]);
    Windows4.ShortCut := ShortCut(Word('L'), [ssAlt]);
    Windows7.ShortCut := ShortCut(Word('T'), [ssAlt]);

    { Help menu }
    Help1.ShortCut := ShortCut(Word('H'), [ssAlt]);
END;



FUNCTION TMainForm.FCDDelLastTab(cLine: String): String;
VAR
    nL: Integer;
BEGIN
    nL := Length(cLine);
    IF (nL = 0) THEN
        Result := ''
    ELSE
    BEGIN
        IF (cLine[nL] = TAB_ASCII_CHAR) THEN
            Result := Copy(cLine, 1, nL - 1)
        ELSE
            Result := cLine;
    END;
END;

FUNCTION TMainForm.GetRTFMove(bWhitesMove: Boolean; cMove: String): String;
VAR
    nL, nP: Integer;
    cP: Char;
    cM: String;
BEGIN
    nL := Length(cMove);
    IF (nL <= 1) THEN
    BEGIN
        Result := cMove;
        EXIT;
    END;

    cP := cMove[1];
    cM := Copy(cMove, 2, nL - 1);

    IF (bWhitesMove) THEN
    BEGIN
        nP := Pos('=', cM);
        IF ((nP > 0) AND (nP < nL)) THEN
        BEGIN
            IF (cM[nP + 1] = 'P') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cWP) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'N') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cWN) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'B') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cWB) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'R') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cWR) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'Q') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cWQ) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'K') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cWK) + Copy(cM, nP + 2, nL - nP - 1);
        END;

        IF (cP = 'P') THEN
            Result := Trim(INI1.INI.cWP) + cM
        ELSE
        IF (cP = 'N') THEN
            Result := Trim(INI1.INI.cWN) + cM
        ELSE
        IF (cP = 'B') THEN
            Result := Trim(INI1.INI.cWB) + cM
        ELSE
        IF (cP = 'R') THEN
            Result := Trim(INI1.INI.cWR) + cM
        ELSE
        IF (cP = 'Q') THEN
            Result := Trim(INI1.INI.cWQ) + cM
        ELSE
        IF (cP = 'K') THEN
            Result := Trim(INI1.INI.cWK) + cM
        ELSE
            Result := cP + cM;
    END
    ELSE
    BEGIN
        IF ((nP > 0) AND (nP < nL)) THEN
        BEGIN
            IF (cM[nP + 1] = 'P') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cBP) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'N') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cBN) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'B') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cBB) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'R') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cBR) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'Q') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cBQ) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'K') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cBK) + Copy(cM, nP + 2, nL - nP - 1);
        END;

        IF (cP = 'P') THEN
            Result := Trim(INI1.INI.cBP) + cM
        ELSE
        IF (cP = 'N') THEN
            Result := Trim(INI1.INI.cBN) + cM
        ELSE
        IF (cP = 'B') THEN
            Result := Trim(INI1.INI.cBB) + cM
        ELSE
        IF (cP = 'R') THEN
            Result := Trim(INI1.INI.cBR) + cM
        ELSE
        IF (cP = 'Q') THEN
            Result := Trim(INI1.INI.cBQ) + cM
        ELSE
        IF (cP = 'K') THEN
            Result := Trim(INI1.INI.cBK) + cM
        ELSE
            Result := cP + cM;
    END;
END;

PROCEDURE TMainForm.FCDProc(var fcdform:tfcdform);
VAR
    cWhiteA, cBlackB, cWhiteB, cBlackA, cRWA, cRBB, cRWB, cRBA: String;
    cwt1,cbt1,cwt2,cbt2:string;
    bwm1,bwm2:boolean;

    nSource: Integer;
    cOneA:   String;
    cOneB:   String;
    cTwoA:   String;
    cTwoB:   String;
    nLine:   Integer;

    cBoard: Char;
    nMain, nNumber, nTime: longint;
    cMove, cAnno, cComment: String;
    bDisplay: Boolean;
    cItem: String;
    bNEmpty:boolean;

    cB1, cB2, cWP1, cBP1, cWP2, cBP2: String;

    nL, nI: Integer;
BEGIN
    ProcessBeg(PROGRAM_TITLE + ' (Converting current BPGN game to four column display RTF file...)');


    { If file already exists then delete it }
    IF (FileExists(INI1.INI.cRTFFile)) THEN
        DeleteFile(INI1.INI.cRTFFile);


    { Clear richedit and comment }
    FCDForm.ClearMemo;
    FCDForm.ClearComment;
    fcdform.backupfont;
    fcdform.SetColumnWidth;


    { Check for display position }
    IF (INI1.INI.bRTFDisp) THEN
    BEGIN
        IF (BPGN.GetBPGNDisplay(1)) THEN
        BEGIN
            BPGN.GetBPGNPosition(1, cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
            FCDForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);

            FCDForm.AddBlankLine;
        END;
    END;


    { Check for comment }
    IF (INI1.INI.bRTFNote) THEN
    BEGIN
        FCDForm.Memo1.Text := Trim(BPGN.GetBPGNComment(1));
        IF (Length(FCDForm.Memo1.Text) > 0) THEN
        BEGIN
            nL := FCDForm.Memo1.Lines.Count - 1;
            FOR nI := 0 TO nL DO
            BEGIN
                IF (nI = 0) THEN
                BEGIN
                    IF (nI = nL) THEN
                        FCDForm.Add3('{' + FCDForm.Memo1.Lines[nI] + '}')
                    ELSE
                        FCDForm.Add3('{' + FCDForm.Memo1.Lines[nI]);
                END
                ELSE
                IF (nI = nL) THEN
                    FCDForm.Add3(FCDForm.Memo1.Lines[nI] + '}')
                ELSE
                    FCDForm.Add3(FCDForm.Memo1.Lines[nI]);
            END;

            FCDForm.AddBlankLine;
        END;
    END;


    { Write handles }
    cWhiteA := BPGN.GetWhiteA(nGameIndex);
    cBlackB := BPGN.GetBlackB(nGameIndex);
    cWhiteB := BPGN.GetWhiteB(nGameIndex);
    cBlackA := BPGN.GetBlackA(nGameIndex);

    cRWA := BPGN.GetRWA(nGameIndex);
    cRBB := BPGN.GetRBB(nGameIndex);
    cRWB := BPGN.GetRWB(nGameIndex);
    cRBA := BPGN.GetRBA(nGameIndex);

    IF (Length(cRWA) > 0) THEN
        cWhiteA := cWhiteA + '(' + cRWA + ')';

    IF (Length(cRBB) > 0) THEN
        cBlackB := cBlackB + '(' + cRBB + ')';

    IF (Length(cRWB) > 0) THEN
        cWhiteB := cWhiteB + '(' + cRWB + ')';

    IF (Length(cRBA) > 0) THEN
        cBlackA := cBlackA + '(' + cRBA + ')';

    IF (INI1.INI.bRTFHand) THEN
    BEGIN
        FCDForm.Add1(cWhiteA,cBlackA,cWhiteB,cBlackB);
    END;


    { Loop through rest of game }
    nSource := 1;
    cOneA   := '';
    cOneB   := '';
    cTwoA   := '';
    cTwoB   :='';
    nline:=0;
    bNempty:=false;
    WHILE (TRUE) DO
    BEGIN
        nMain := BPGN.GetBPGNMain(nSource);
        IF (nMain = 0) THEN
            BREAK;

        cBoard := BPGN.GetBPGNBoard(nMain);
        cMove  := '';

        IF (INI1.INI.bRTFMove) THEN
        BEGIN
            IF (cBoard = 'A') THEN
                cMove := GetRTFMove(TRUE, Trim(BPGN.GetBPGNMove(nMain)))
            ELSE
            IF (cBoard = 'a') THEN
                cMove := GetRTFMove(FALSE, Trim(BPGN.GetBPGNMove(nMain)))
            ELSE
            IF (cBoard = 'B') THEN
                cMove := GetRTFMove(TRUE, Trim(BPGN.GetBPGNMove(nMain)))
            ELSE
            IF (cBoard = 'b') THEN
                cMove := GetRTFMove(FALSE, Trim(BPGN.GetBPGNMove(nMain)));
        END;

        IF (INI1.INI.bRTFAnno) THEN
            cAnno := Trim(BPGN.GetBPGNAnno(nMain))
        ELSE
            cAnno := '';

        IF (INI1.INI.bRTFNote) THEN
            cComment := Trim(BPGN.GetBPGNComment(nMain))
        ELSE
            cComment := '';

        IF (INI1.INI.bRTFDisp) THEN
            bDisplay := BPGN.GetBPGNDisplay(nMain)
        ELSE
            bDisplay := FALSE;

        IF (UpCase(cBoard) = 'A') THEN
        BEGIN
            nNumber := BPGN.GetBPGNNumber(nMain,1);
            nTime   := BPGN.GetBPGNnWTime(nMain,1);
        END
        ELSE
        BEGIN
            nNumber := BPGN.GetBPGNNumber(nMain,2);
            nTime   := BPGN.GetBPGNnWTime(nMain,2);
        END;

        IF (INI1.INI.bRTFZero) THEN
        BEGIN
            IF (CompareText(cMove, 'O-O-O') = 0) THEN
                cMove := '0-0-0'
            ELSE
            IF (CompareText(cMove, 'O-O') = 0) THEN
                cMove := '0-0';
        END;

        IF (INI1.INI.bRTFTime) THEN
        BEGIN
            cItem := IntToStr(nNumber) + cBoard + '. ' + cMove + cAnno +
                     '{' + TOOLBOX_GetDispTimeStr(nTime) + '}';
        END
        ELSE
        BEGIN
            cItem := IntToStr(nNumber) + cBoard + '. ' + cMove + cAnno;
        END;

        IF (cBoard = 'A') THEN
        BEGIN
            IF (bNEmpty) THEN
             FCDForm.Add2(cOneA,cTwoA,cOneB,cTwoB);
            nline:=1;
            cOneA := cItem; cTwoA:='';cTwoB:='';cOneB:='';
            bNempty:=true;
        END
        ELSE
        IF (cBoard = 'a') THEN
        BEGIN
            IF (nLine = 1) THEN
                cTwoA := cItem
            ELSE
            BEGIN
                IF (bNempty) THEN
                    FCDForm.Add2(cOneA,cTwoA,coneB,cTwoB);

                cTwoA:=cItem;cOneA:='';
            END;
            cOneB:='';cTwob:='';
            nLine := 2;
            bNempty:=true;
        END
        ELSE
        IF (cBoard = 'B') THEN
        BEGIN
            if nline>2 then
            BEGIN
                IF (bNEmpty) THEN
                    FCDForm.Add2(cOneA,cTwoA,cOneB,cTwoB);

                cOneA:='';cTwoA:='';cTwoB:='';
            END;
            cOneB:=cItem;
            nLine := 3;
            bNempty:=true;
        END
        ELSE
        IF (cBoard = 'b') THEN
        BEGIN
            FCDForm.Add2(cOneA,cTwoA,cOneB,citem);
            cOneA:='';CtwoA:='';cTwob:='';cOneB:='';
            nLine := 0;
            bNempty:=false;
        END;

        IF (Length(cComment) > 0) THEN
        BEGIN
            IF (bNempty) THEN
                FCDForm.Add2(ConeA,cTwoA,cOneB,cTwoB);

            nLine := 0;

            IF (bDisplay) THEN
            BEGIN
                FCDForm.AddBlankLine;

                BPGN.GetBPGNPosition(nMain, cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
                FCDForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
            END;

            FCDForm.AddBlankLine;

            FCDForm.Memo1.Text := cComment;
            nL := FCDForm.Memo1.Lines.Count - 1;
            FOR nI := 0 TO nL DO
            BEGIN
                IF (nI = 0) THEN
                BEGIN
                    IF (nI = nL) THEN
                        FCDForm.Add3('{' + FCDForm.Memo1.Lines[nI] + '}')
                    ELSE
                        FCDForm.Add3('{' + FCDForm.Memo1.Lines[nI]);
                END
                ELSE
                IF (nI = nL) THEN
                    FCDForm.Add3(FCDForm.Memo1.Lines[nI] + '}')
                ELSE
                    FCDForm.Add3(FCDForm.Memo1.Lines[nI]);
            END;

            FCDForm.AddBlankLine;
        END
        ELSE
        IF (bDisplay) THEN
        BEGIN
            IF (bNempty) THEN
                FCDForm.Add2(cOneA,cTwoA,coneB,cTwob);

            nLine := 0;

            FCDForm.AddBlankLine;

            BPGN.GetBPGNPosition(nMain, cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
            FCDForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);

            FCDForm.AddBlankLine;
        END;

        nSource := nMain;
    END;
    IF (bNempty) THEN
        FCDForm.Add2(cOneA,cTwoA,cOneB,cTwob);


    { Display result }
    IF (INI1.INI.bRTFRest) THEN
    BEGIN
        FCDForm.AddBlankLine;
        FCDForm.Add5(BPGN.GetReason);
    END;

    { Save into RTF file }
    FCDForm.SaveToFile;


    { Adjust RTF file }
    FCDForm.AdjustFile;


    { Clear richedit and comment }
    FCDForm.ClearMemo;
    FCDForm.ClearComment;
    Fcdform.restorefont;

    ProcessEnd;
END;

FUNCTION TMainForm.GetTXTMove(bWhitesMove: Boolean; cMove: String): String;
VAR
    nL, nP: Integer;
    cP: Char;
    cM: String;
BEGIN
    nL := Length(cMove);
    IF (nL <= 1) THEN
    BEGIN
        Result := cMove;
        EXIT;
    END;

    cP := cMove[1];
    cM := Copy(cMove, 2, nL - 1);

    IF (bWhitesMove) THEN
    BEGIN
        nP := Pos('=', cM);
        IF ((nP > 0) AND (nP < nL)) THEN
        BEGIN
            IF (cM[nP + 1] = 'P') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTWP) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'N') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTWN) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'B') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTWB) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'R') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTWR) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'Q') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTWQ) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'K') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTWK) + Copy(cM, nP + 2, nL - nP - 1);
        END;

        IF (cP = 'P') THEN
            Result := Trim(INI1.INI.cTXTWP) + cM
        ELSE
        IF (cP = 'N') THEN
            Result := Trim(INI1.INI.cTXTWN) + cM
        ELSE
        IF (cP = 'B') THEN
            Result := Trim(INI1.INI.cTXTWB) + cM
        ELSE
        IF (cP = 'R') THEN
            Result := Trim(INI1.INI.cTXTWR) + cM
        ELSE
        IF (cP = 'Q') THEN
            Result := Trim(INI1.INI.cTXTWQ) + cM
        ELSE
        IF (cP = 'K') THEN
            Result := Trim(INI1.INI.cTXTWK) + cM
        ELSE
            Result := cP + cM;
    END
    ELSE
    BEGIN
        IF ((nP > 0) AND (nP < nL)) THEN
        BEGIN
            IF (cM[nP + 1] = 'P') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTBP) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'N') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTBN) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'B') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTBB) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'R') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTBR) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'Q') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTBQ) + Copy(cM, nP + 2, nL - nP - 1)
            ELSE
            IF (cM[nP + 1] = 'K') THEN
                cM := Copy(cM, 1, nP) + Trim(INI1.INI.cTXTBK) + Copy(cM, nP + 2, nL - nP - 1);
        END;

        IF (cP = 'P') THEN
            Result := Trim(INI1.INI.cTXTBP) + cM
        ELSE
        IF (cP = 'N') THEN
            Result := Trim(INI1.INI.cTXTBN) + cM
        ELSE
        IF (cP = 'B') THEN
            Result := Trim(INI1.INI.cTXTBB) + cM
        ELSE
        IF (cP = 'R') THEN
            Result := Trim(INI1.INI.cTXTBR) + cM
        ELSE
        IF (cP = 'Q') THEN
            Result := Trim(INI1.INI.cTXTBQ) + cM
        ELSE
        IF (cP = 'K') THEN
            Result := Trim(INI1.INI.cTXTBK) + cM
        ELSE
            Result := cP + cM;
    END;
END;

PROCEDURE TMainForm.TXTProc(var txtform:ttxtform);
VAR
    cWhiteA, cBlackB, cWhiteB, cBlackA, cRWA, cRBB, cRWB, cRBA: String;
    ncolWidth,i: integer;
    nSource: Integer;
    cSpace:  String;
    cLine:   String;
    nLine:   Integer;

    cBoard: Char;
    nMain, nNumber, nTime,nLag: Integer;
    cMove, cAnno, cComment: String;
    bDisplay: Boolean;
    cItem: String;

    s,cB1, cB2, cWP1, cBP1, cWP2, cBP2: String;
    cWt1,cBt1,cwt2,cbt2:string;
    bwm1,bwm2:boolean;
    nL, nI: Integer;
BEGIN
    ncolWidth:=30;
    ProcessBeg(PROGRAM_TITLE + ' (Converting current BPGN game to four column display TXT file...)');


    { If file already exists then delete it }
    IF (FileExists(INI1.INI.cTXTFile)) THEN
        DeleteFile(INI1.INI.cTXTFile);


    { Clear memo and comment }
    TXTForm.ClearMemo;
    TXTForm.ClearComment;


    { Check for display position }
    IF (INI1.INI.bTXTDisp) THEN
    BEGIN
        IF (BPGN.GetBPGNDisplay(1)) THEN
        BEGIN
            BPGN.GetBPGNPosition(1, cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
            TXTForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2);

            TXTForm.AddBlankLine;
        END;
    END;


    { Check for comment }
    IF (INI1.INI.bTXTNote) THEN
    BEGIN
        TXTForm.Memo2.Text := Trim(BPGN.GetBPGNComment(1));
        IF (Length(TXTForm.Memo2.Text) > 0) THEN
        BEGIN
            nL := TXTForm.Memo2.Lines.Count - 1;
            FOR nI := 0 TO nL DO
            BEGIN
                IF (nI = 0) THEN
                BEGIN
                    IF (nI = nL) THEN
                        TXTForm.AddMemo('{' + TXTForm.Memo2.Lines[nI] + '}')
                    ELSE
                        TXTForm.AddMemo('{' + TXTForm.Memo2.Lines[nI]);
                END
                ELSE
                IF (nI = nL) THEN
                    TXTForm.AddMemo(TXTForm.Memo2.Lines[nI] + '}')
                ELSE
                    TXTForm.AddMemo(TXTForm.Memo2.Lines[nI]);
            END;

            TXTForm.AddBlankLine;
        END;
    END;


    { Write handles }
    cWhiteA := BPGN.GetWhiteA(nGameIndex);
    cBlackB := BPGN.GetBlackB(nGameIndex);
    cWhiteB := BPGN.GetWhiteB(nGameIndex);
    cBlackA := BPGN.GetBlackA(nGameIndex);

    cRWA := BPGN.GetRWA(nGameIndex);
    cRBB := BPGN.GetRBB(nGameIndex);
    cRWB := BPGN.GetRWB(nGameIndex);
    cRBA := BPGN.GetRBA(nGameIndex);

    IF (Length(cRWA) > 0) THEN
        cWhiteA := cWhiteA + '(' + cRWA + ')';

    IF (Length(cRBB) > 0) THEN
        cBlackB := cBlackB + '(' + cRBB + ')';

    IF (Length(cRWB) > 0) THEN
        cWhiteB := cWhiteB + '(' + cRWB + ')';

    IF (Length(cRBA) > 0) THEN
        cBlackA := cBlackA + '(' + cRBA + ')';

    TOOLBOX_PadR(cWhiteA, nColWidth, ' ');
    TOOLBOX_PadR(cBlackB, nColWidth, ' ');
    TOOLBOX_PadR(cWhiteB, nColWidth, ' ');
    TOOLBOX_PadR(cBlackA, nColWidth, ' ');

    IF (INI1.INI.bTXTHand) THEN
    BEGIN
        TXTForm.AddMemo(cWhiteA + cBlackB + cWhiteB + cBlackA);
    END;


    { Loop through rest of game }
    nSource := 1;
    cSpace:='';
    for i:=1 to nColWidth do
    cSpace:=cSpace+' ';
    cLine   := '';
    nLine   := 0;
    WHILE (TRUE) DO
    BEGIN
        nMain := BPGN.GetBPGNMain(nSource);
        IF (nMain = 0) THEN
            BREAK;

        cBoard := BPGN.GetBPGNBoard(nMain);
        cMove  := '';

        IF (INI1.INI.bTXTMove) THEN
        BEGIN
            IF (cBoard = 'A') THEN
                cMove := GetTXTMove(TRUE, Trim(BPGN.GetBPGNMove(nMain)))
            ELSE
            IF (cBoard = 'a') THEN
                cMove := GetTXTMove(FALSE, Trim(BPGN.GetBPGNMove(nMain)))
            ELSE
            IF (cBoard = 'B') THEN
                cMove := GetTXTMove(TRUE, Trim(BPGN.GetBPGNMove(nMain)))
            ELSE
            IF (cBoard = 'b') THEN
                cMove := GetTXTMove(FALSE, Trim(BPGN.GetBPGNMove(nMain)));
        END;

        IF (INI1.INI.bTXTAnno) THEN
            cAnno := Trim(BPGN.GetBPGNAnno(nMain))
        ELSE
            cAnno := '';

        IF (INI1.INI.bTXTNote) THEN
            cComment := Trim(BPGN.GetBPGNComment(nMain))
        ELSE
            cComment := '';

        IF (INI1.INI.bTXTDisp) THEN
            bDisplay := BPGN.GetBPGNDisplay(nMain)
        ELSE
            bDisplay := FALSE;

        IF (UpCase(cBoard) = 'A') THEN
        BEGIN
            nNumber := BPGN.GetBPGNNumber(nMain,1);
            nTime   := BPGN.GetBPGNnWTime(nMain,1);
        END
        ELSE
        BEGIN
            nNumber := BPGN.GetBPGNNumber(nMain,2);
            nTime   := BPGN.GetBPGNnWTime(nMain,2);
        END;
        nLag:=Bpgn.bpgnlag[nMain];

        IF (INI1.INI.bTXTZero) THEN
        BEGIN
            IF (CompareText(cMove, 'O-O-O') = 0) THEN
                cMove := '0-0-0'
            ELSE
            IF (CompareText(cMove, 'O-O') = 0) THEN
                cMove := '0-0';
        END;

        Str(nNumber:3, cItem);
        cItem := cItem + cBoard + '. ' + cMove + cAnno;
        TOOLBOX_PadR(cItem, 15, ' ');

        IF (INI1.INI.bTXTTime) THEN
        BEGIN
            if bpgn.bLagTag then s:='+'+GetLagStr(nLag) else s:='';
            cItem := cItem + '{' + TOOLBOX_GetDispTimeStr(nTime) + s+'}';
        END;

        TOOLBOX_PadR(cItem, nColWidth, ' ');

        IF (cBoard = 'A') THEN
        BEGIN
            IF (Length(cLine) > 0) THEN
                TXTForm.AddMemo(cLine);

            cLine := cItem;
            nLine := 1;
        END
        ELSE
        IF (cBoard = 'a') THEN
        BEGIN
            IF (nLine = 1) THEN
                cLine := cLine + cItem
            ELSE
            BEGIN
                IF (Length(cLine) > 0) THEN
                    TXTForm.AddMemo(cLine);

                cLine := cSpace + cItem;
            END;
            nLine := 2;
        END
        ELSE
        IF (cBoard = 'B') THEN
        BEGIN
            IF (nLine = 1) THEN
                cLine := cLine + cSpace + cItem
            ELSE
            IF (nLine = 2) THEN
                cLine := cLine + cItem
            ELSE
            BEGIN
                IF (Length(cLine) > 0) THEN
                    TXTForm.AddMemo(cLine);

                cLine := cSpace + cSpace + cItem;
            END;
            nLine := 3;
        END
        ELSE
        IF (cBoard = 'b') THEN
        BEGIN
            IF (nLine = 1) THEN
                cLine := cLine + cSpace + cSpace + cItem
            ELSE
            IF (nLine = 2) THEN
                cLine := cLine + cSpace + cItem
            ELSE
            IF (nLine = 3) THEN
                cLine := cLine + cItem
            ELSE
            BEGIN
                IF (Length(cLine) > 0) THEN
                    TXTForm.AddMemo(cLine);

                cLine := cSpace + cSpace + cSpace + cItem;
            END;
            nLine := 4;
        END;

        IF (Length(cComment) > 0) THEN
        BEGIN
            IF (Length(cLine) > 0) THEN
                TXTForm.AddMemo(cLine);

            cLine := '';
            nLine := 0;

            IF (bDisplay) THEN
            BEGIN
                TXTForm.AddBlankLine;

                BPGN.GetBPGNPosition(nMain, cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
                TXTForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2);
            END;

            TXTForm.AddBlankLine;

            TXTForm.Memo2.Text := cComment;

            nL := TXTForm.Memo2.Lines.Count - 1;
            FOR nI := 0 TO nL DO
            BEGIN
                IF (nI = 0) THEN
                BEGIN
                    IF (nI = nL) THEN
                        TXTForm.AddMemo('{' + TXTForm.Memo2.Lines[nI] + '}')
                    ELSE
                        TXTForm.AddMemo('{' + TXTForm.Memo2.Lines[nI]);
                END
                ELSE
                IF (nI = nL) THEN
                    TXTForm.AddMemo(TXTForm.Memo2.Lines[nI] + '}')
                ELSE
                    TXTForm.AddMemo(TXTForm.Memo2.Lines[nI]);
            END;

            TXTForm.AddBlankLine;
        END
        ELSE
        IF (bDisplay) THEN
        BEGIN
            IF (Length(cLine) > 0) THEN
                TXTForm.AddMemo(cLine);

            cLine := '';
            nLine := 0;

            TXTForm.AddBlankLine;

            BPGN.GetBPGNPosition(nMain, cB1, cB2, cWP1, cBP1, cWP2, cBP2,cWt1,cBt1,cwt2,cbt2,bwm1,bwm2);
            TXTForm.DisplayBoard(cB1, cB2, cWP1, cBP1, cWP2, cBP2);

            TXTForm.AddBlankLine;
        END;

        nSource := nMain;
    END;
    IF (Length(cLine) > 0) THEN
        TXTForm.AddMemo(cLine);


    { Display result }
    IF (INI1.INI.bTXTRest) THEN
    BEGIN
        TXTForm.AddBlankLine;
        TXTForm.AddMemo(BPGN.GetReason);
    END;


    { Save into TXT file }
    TXTForm.SaveToFile;


    { Clear memo and comment }
    TXTForm.ClearMemo;
    TXTForm.ClearComment;


    ProcessEnd;
END;

procedure TMainForm.WengClick(Sender: TObject);
begin
 EngineForms[defeng].Show;
 EngineForms[defeng].bringtofront;
end;

procedure TMainForm.Font1Click(Sender: TObject);
var i:integer;
begin
    StopTimer;
    FontDialog1.Device := fdScreen;
    FontDialog1.Font   := fEngineF;
    IF (FontDialog1.Execute) THEN
    BEGIN
     for i:=1 to MAX_ENGINE do
      if Engines[i].exist then EngineForms[i].Memo1.Font := FontDialog1.Font;
     fEnginef:=FontDialog1.font;
    END;
end;

procedure TMainForm.Color1Click(Sender: TObject);
var i:integer;
begin
    StopTimer;
    LoadCustomColors;
    ColorDialog1.Color := nEngineb;
    IF (ColorDialog1.Execute) THEN
    BEGIN
        SaveCustomColors;
        for i:=1 to MAX_ENGINE  do
         if Engines[i].exist then EngineForms[i].Memo1.Color := ColorDialog1.Color;
        nengineb:=colorDialog1.Color;
    END;
end;

procedure TMainForm.WordWrap1Click(Sender: TObject);
var i:integer;
begin
    StopTimer;
    WordWrap1.Checked           := (NOT WordWrap1.Checked);
    ini1.ini.bEngineWR:= WordWrap1.checked;
    for i:=1 to MAX_ENGINE do
    if Engines[i].exist then
    begin
     EngineForms[i].Memo1.WordWrap := WordWrap1.Checked;

     IF (EngineForms[i].Memo1.WordWrap) THEN
         EngineForms[i].Memo1.ScrollBars := ssVertical
     ELSE
        EngineForms[i].Memo1.ScrollBars := ssBoth;
    end;
end;

procedure TMainForm.Options_1Click(Sender: TObject);
var i:integer;
comlineform:tcomlineform;
begin
 StopTimer;
 i:=getengine(Sender);
 try
 comlineform:=tcomlineform.create(self);
 ComLineForm.SetupValues(EngOpt[i]);
 ComLineForm.ShowModal;
 if ComLineForm.bOk then
 Begin
  EngOpt[i]:=ComLineForm.options;
  if Engines[i].exist then EngineForms[i].opt:=engopt[i];
  EngineMenu[i]^.Caption:=ComlineForm.options.name;
  WengMenu[i]^.Caption:=ComlineForm.options.name;
  PlengMenu[i]^.enabled:=true;
  plEngMenu[i]^.caption:=comlineForm.options.name;
 end;
 finally
  comlineform.free;
 end;
end;

procedure TMainForm.SetasDefault_1Click(Sender: TObject);
var i:integer;
begin
 i:=getEngine(Sender);
 //uncheckEngines;
 EngineMenu[defeng].checked:=false;
 DefEngMenu[defeng].enabled:=true;
 If Engines[defeng].exist then EngineForms[defeng].bitbtn2.visible:=false;
 defeng:=i;
 EngineMenu[i]^.checked:=true;
 DefEngMenu[i]^.enabled:=false;
 if Engines[defeng].suspended then
 begin
  NavigateForm.button1.caption:=Sjeng_Resume;
  NavigateForm.Button1.hint:='Resumes default engine ['+engopt[defeng].name+']';
 end;

 if Engines[defeng].bEngineOn then
 begin
  NavigateForm.button1.caption:=Sjeng_OFF;
  NavigateForm.Button1.hint:='Turns default engine ['+engopt[defeng].name+'] OFF';
 end
 else begin
  NavigateForm.Button1.caption:=Sjeng_ON;
  NavigateForm.button1.hint:='Turns default engine ['+engopt[defeng].name+'] ON';
 end;
 if Engines[defeng].exist then EngineForms[defeng].bitbtn2.visible:=true;
end;


procedure TMainForm.UncheckEngines;
var i:integer;
begin
 for i:=1 to MAX_ENGINE do enginemenu[i]^.checked:=false;
end;

procedure TMainForm.Weng_1Click(Sender: TObject);
var i:integer;
begin
 i:=getEngine(Sender);
 EngineForms[i].Show;
 EngineForms[i].BringToFront;
end;

procedure tmainform.setupboardconfig(bd:Pboardcfg);
begin
 bd^.nwhtsquare:=board1form.board.nwhtsquare;
 bd^.nblksquare:=board1form.board.nblksquare;
 bd^.nnumbercolor:=board1form.board.nnumbercolor;
 bd^.PaintBox6.color:=board1form.Color;

 bd^.nboardoutline:=board1form.PaintBox2.Canvas.Pen.color;
 bd^.nPcSquare:=board1form.board.nPcSquare;
 bd^.fclock:=board1form.label3.Font;
 bd^.nWaitf:=board1form.nwaitf;
 bd^.nwaitb:=board1form.nwaitb;
 bd^.nturnb:=board1form.nTurnB;
 bd^.nturnf:=board1form.nturnf;
 bd^.fhandle:=board1form.Label1.font;
 bd^.nHandleB:=board1form.Label1.Color;


end;

procedure tmainform.button1click1(Sender:Tobject);
var
bdform:Tboardcfg;
tmp:string;
begin
 try
  bdform:=tboardcfg.create(self);
  setupboardconfig(@bdform);
  bdform.showmodal;
  if bdform.bok then
  begin
   board1form.board.nwhtsquare:=bdform.nwhtsquare;
   board2form.board.nwhtsquare:=bdform.nwhtsquare;
   board1form.board.nblksquare:=bdform.nblksquare;
   board2form.board.nblksquare:=bdform.nblksquare;
   board1form.board.nnumbercolor:=bdform.nnumbercolor;
   board2form.board.nnumbercolor:=bdform.nnumbercolor;
   board1form.color:=bdform.paintbox6.color;
   board2form.color:=bdform.paintbox6.color;


   Board1Form.PaintBox2.Canvas.Pen.Color   := bdform.nboardoutline;
   Board1Form.PaintBox2.Canvas.Brush.Color := bdform.nboardoutline;
   Board1Form.PaintBox2.Canvas.Brush.Style := bsSolid;

   Board1Form.PaintBox3.Canvas.Pen.Color   := bdform.nboardoutline;
   Board1Form.PaintBox3.Canvas.Brush.Color := bdform.nboardoutline;
   Board1Form.PaintBox3.Canvas.Brush.Style := bsSolid;

   Board1Form.PaintBox4.Canvas.Pen.Color   := bdform.nboardoutline;
   Board1Form.PaintBox4.Canvas.Brush.Color := bdform.nboardoutline;
   Board1Form.PaintBox4.Canvas.Brush.Style := bsSolid;

   Board1Form.PaintBox5.Canvas.Pen.Color   := bdform.nboardoutline;
   Board1Form.PaintBox5.Canvas.Brush.Color := bdform.nboardoutline;
   Board1Form.PaintBox5.Canvas.Brush.Style := bsSolid;

   Board2Form.PaintBox2.Canvas.Pen.Color   := bdform.nboardoutline;
   Board2Form.PaintBox2.Canvas.Brush.Color := bdform.nboardoutline;
   Board2Form.PaintBox2.Canvas.Brush.Style := bsSolid;

   Board2Form.PaintBox3.Canvas.Pen.Color   := bdform.nboardoutline;
   Board2Form.PaintBox3.Canvas.Brush.Color := bdform.nboardoutline;
   Board2Form.PaintBox3.Canvas.Brush.Style := bsSolid;

   Board2Form.PaintBox4.Canvas.Pen.Color   := bdform.nboardoutline;
   Board2Form.PaintBox4.Canvas.Brush.Color := bdform.nboardoutline;
   Board2Form.PaintBox4.Canvas.Brush.Style := bsSolid;

   Board2Form.PaintBox5.Canvas.Pen.Color   := bdform.nboardoutline;
   Board2Form.PaintBox5.Canvas.Brush.Color := bdform.nboardoutline;
   Board2Form.PaintBox5.Canvas.Brush.Style := bsSolid;

   board1form.board.npcsquare:=bdform.npcsquare;
   board2form.board.npcsquare:=bdform.npcsquare;
   board1form.nwaitf:=bdform.nwaitf;
   board2form.nwaitf:=bdform.nwaitf;
   board1form.nwaitb:=bdform.nwaitb;
   board2form.nwaitb:=bdform.nwaitb;
   board1form.nturnf:=bdform.nturnf;
   board2form.nturnf:=bdform.nturnf;
   board1form.nturnb:=bdform.nturnb;
   board2form.nturnb:=bdform.nturnb;

   Board1Form.label1.setfont(bdform.fhandle);
   Board1Form.label2.setfont(bdform.fhandle);
   Board2Form.label1.setfont(bdform.fhandle);
   Board2Form.label2.setfont(bdform.fhandle);

   Board1Form.Label1.Color := bdform.nhandleb;
   Board1Form.Label2.Color := bdform.nhandleb;
   Board2Form.Label1.Color := bdform.nhandleb;
   Board2Form.Label2.Color := bdform.nhandleb;

   Board1Form.Label3.setfont(bdform.lbbutton3.font);
   Board1Form.Label4.setfont(bdform.lbbutton3.font);

   Board2Form.Label3.setfont(bdform.lbbutton3.font);
   Board2Form.Label4.setfont(bdform.lbbutton3.font);

   usebitmap:=bdform.usebmpbg;
   usepiecebitmap:=bdform.usebmppiece;
   useboardbitmap:=bdform.usebmpsquare;

   if usepiecebitmap then usepiecebitmap:=readpieces(pieces,bdform.bmppiece);
   if usepiecebitmap then piece_dir:=bdform.bmppiece;

   if useboardbitmap then useboardbitmap:=readboard(boardbmp,bdform.bmpsquare);
   if useboardbitmap then board_dir:=bdform.bmpsquare;
   tmp:=bdform.bmpbg;
   if usebitmap then
   try
     piecesbg.loadfromfile(tmp);
     bg_file:=tmp;
    except
     usebitmap:=false;
    end;
 board1form.Board.LoadPiece;
 board2form.board.loadpiece;
 board1form.invalidate;
 board2form.invalidate;
 board1form.formresize(Self);
 board2form.formresize(self);
  end;
 finally
  bdform.free;
 end;
end;


function div100(s:string):string;
begin
 Result:=copy(s,1,length(s)-2)+'.'+copy(s,length(s)-1,2);
end;

procedure TmainForm.gotonexttest;
var ps,ps1:integer;
found:boolean;
begin
 found:=false;
 ps:=pos('\test',commentForm.memo1.text);
 ps1:=pos('\endtest',commentform.memo1.text);
 if ((ps>0) and (ps1>0)) then found:=true;
 while ((not found) and (navigateform.speedbutton3.enabled)) do
 begin
  navigateclick3(Self);
  ps:=pos('\test',commentForm.memo1.text);
  ps1:=pos('\endtest',commentform.memo1.text);
  if ((ps>0) and (ps1>0)) then found:=true;
 end;
end;

function TmainForm.gettestar(var testar:TTestAr):integer;
var ps,ps1,count,cd:integer;
res:real;
s,s1,s2,tstart,tend:string;
begin
 tstart:='\test';
 tend:='\endtest';
 ps:=pos(tstart,commentForm.memo1.text);
 ps1:=pos(tend,commentform.memo1.text);
 s:=Copy(CommentForm.memo1.text,ps+length(tstart),ps1-ps-length(tstart));
 count:=0;
  //delete CR's
 ps:=pos(CR_ASCII_CHAR,s);
 while ps>0 do begin delete(s,ps,1); ps:=pos(CR_ASCII_CHAR,s); end;
 //Delete first LF; add final lf;
 s:=trim(s);
 if s[1]=LF_ASCII_CHAR then delete(s,1,1);
 if s[length(s)]<>LF_ASCII_CHAR then s:=s+LF_ASCII_CHAR;
//now parse lines
 ps:=pos(LF_ASCII_CHAR,s);
 while ps>0 do
 begin
  s1:=copy(s,1,ps-1);
  delete(s,1,ps);
  ps1:=pos(',',s1);
  s2:=trim(copy(s1,ps1+1,length(s1)));
  val(s2,res,cd);
  if cd>0 then begin ps:=pos(LF_ASCII_CHAR,s);continue;end;
  inc(count,1);
  testar[count].score:=res;
  testar[count].move:=trim(copy(s1,1,ps1-1));
  ps:=pos(LF_ASCII_CHAR,s);
 end;
 result:=count;
end;

procedure TmainForm.timer3timer(Sender: Tobject);
var testar:TTestar;
i,n:integer;
t:TengLine;
mv:string;
begin
 timer2.enabled:=false;
 t:=EngineForms[anneng].getbestline;
 mv:=lowercase(t.move);
 n:=gettestar(testar);
 for i:=1 to n do
 if mv=testar[i].move then
 begin
  testScore:=testscore+testar[i].score;
  break;
 end;
 CommentForm.memo1.lines.add(EngineForms[anneng].engine.engname+': test move='+t.move);
 navigateclick3(self);
 gotonexttest;
 if navigateform.SpeedButton3.Enabled then
 begin
  Timer2.interval:=t2int;
  timer2.enabled:=true;
 end
 else SendMessage(handle,WM_ENGINETestSTOP,anneng,1);
end;

procedure TMainForm.Timer2Timer(Sender: TObject);
var N,score,err:integer;
t:TEngLine;
s1,pscore,move,ascore,almove:string;
mmissed:boolean;
begin
 mmissed:=false;
 timer2.enabled:=false;
 t:=EngineForms[anneng].getBestLine;
 move:=standmove(t.move); //  moves like Bxc3 are case sensitive;
 almove:=lowercase(t.move);  //coordinate moves like d2c3 are always lowercase;
 val(t.score,score,err);
 if actscore=-INFTY then //It's a last move in the game. Let's add comment
 begin
  pscore:=div100(t.score);
  if abs(score) > MATE_EVAL then
  begin
   if score>0 then s1:='Even nitemove can not blow this' else s1:='I swear, I would never get into such shit';
   commentForm.memo1.lines.add(EngineForms[anneng].Engine.engname+':Forced mate. '+s1);
  end
  else
   commentForm.memo1.lines.add(EngineForms[anneng].Engine.engname+': Evaluation is'+pscore);
 end;
 if ((ini1.ini.bengineSM) and (almove<>algactmove) and (move<>humactmove) and (actscore<MATE_EVAL)and (score>MATE_EVAL)) then //mate missed
 begin
  mmissed:=true;
  //find out how quick was mate
  s1:='Patzer missed the mate here: '+ t.line + ' and gg. Whatabigfish!';
  commentForm.memo1.lines.add(EngineForms[anneng].Engine.engname+': '+s1);
 end;
 if not ((almove=algactmove) or (move=humactmove) or
 ((score-actscore-100*ini1.ini.nEngineIT)<0) or
 (abs(actScore)>100*ini1.ini.nEngineET) or (actScore=-INFTY) or (mmissed)) then
 begin
  pscore:=div100(t.score);
  str(actscore,ascore);  ascore:=div100(ascore);
  commentForm.memo1.lines.add(EngineForms[anneng].Engine.engname+': Now better were '+move+'='+pscore+' '+
  '.For example, '+t.line+'. Score for the played move ='+ascore+'.');
  //include comment;
 end;
 if linecnt[1]>0 then
 begin
  actscore:=-score;//since move changes sides;
  algactmove:=lowercase(sjengmove(line[1,1],linecnt[1]));
  humactmove:=standmove(bpgn.GetCurrentMove);
  Timer2.interval:=t2int;
  navigateclick2(Sender);
  if banno then Timer2.enabled:=true;
 end
 else SendMessage(handle,WM_ENGINEANNSTOP,anneng,1);
end;

procedure TMainForm.ViewReadme1Click(Sender: TObject);
begin
  StopTimer;
  with ini1.INI do
  if Sender=ViewReadme1 then ShowFile('Readme',ExtractFilePath(paramstr(0))+'readme.rtf',nRTFL,nRtfT,nRtfW,nrtfh)
  else showFile('FAQ',ExtractFilePath(paramstr(0))+'FAQ.rtf',nRTFL,nRtfT,nRtfW,nrtfh);
end;

procedure TMainForm.RegisterBpgnfiles1Click(Sender: TObject);
begin
 associatefiles;
end;

procedure TMainForm.ApplyGameSound1Click(Sender: TObject);
begin
 ApplyGameSound1.Checked:=not ApplyGameSound1.checked;
 ini1.ini.bsoundon:=Applygamesound1.checked;
 if applygamesound1.checked then navigateform.speedbutton11.glyph:=imsound.Picture.Bitmap
 else navigateform.speedbutton11.glyph:=imnosound.Picture.Bitmap;
end;

procedure TMainForm.Configure1Click(Sender: TObject);
var sform:Tsoundform;
begin
 try
  sform:=Tsoundform.create(self);
  with sform do
  begin
   edit1.text:=ini1.ini.csoundmove;
   edit2.text:=ini1.ini.csoundend;
   edit3.text:=ini1.ini.csoundcap;
   edit4.text:=ini1.ini.csoundcheck;
  end;
  sform.setupvalues;
  sform.showmodal;
  if sform.bok then
  with sform do
  begin
   if (lowercase(edit1.text)='none') then edit1.text:='none';
   ini1.ini.csoundmove:=edit1.text;
   if (lowercase(edit2.text)='none') then edit2.text:='none';
   ini1.ini.csoundend:=edit2.text;
   if (lowercase(edit3.text)='none') then edit3.text:='none';
   ini1.ini.csoundcap:=edit3.text;
   if (lowercase(edit4.text)='none') then edit4.text:='none';
   ini1.ini.csoundcheck:=edit4.text;
  end;
 finally
  sform.free;
 end;
end;

procedure TMainForm.SaveAs1Click(Sender: TObject);
var tmpname,sourcename,destname,tmph,tmpb,tmpw,s,dext:string;
source,dest,tmp:array[0..512] of char;
bdb:boolean;
srcDate,destDate:tdatetime;
begin
 StopTimer;
 if SaveDialog2.Execute then
 begin
  bdb:=bpgn.bIsDB;
  destname:=savedialog2.filename;
//  tmpname:=uniquefilename(ExtractFilePath(destname));
//  strpcopy(tmp,tmpname);
  dext:=ExtractFileExt(destname);
  if ((dext=DB_HEADER_EXT) or (Dext=DB_WHITE_EXT) or (Dext=db_Black_ext)) then
  destname:=changefileExt(destname,'.'+BPGN_EXT);
  sourcename:=BPGN.GetBPGNFilename;

  if not fileexists(sourcename) then exit;
  if fileexists(destname) then
  begin
    if not wokCancel('File '+destname+' already exists. Overwrite?') then exit;
  end;

  if not FileSaveAS(sourcename,destname) then
  begin
   Werror('Error writing file'+EOL_STR+destname);
   exit;
  end;

  if bdb then //backup 3 more files
  begin
   if not FileSaveAs(changeFileExt(sourcename,DB_HEADER_EXT),ChangeFileExt(destname,DB_HEADER_EXT))
   then begin Werror('Error writing file'+EOL_STR+ChangeFileExt(destname,DB_HEADER_EXT)); exit; end;
   if not FileSaveAs(changeFileExt(sourcename,DB_WHITE_EXT),ChangeFileExt(destname,DB_WHITE_EXT))
   then begin Werror('Error writing file'+EOL_STR+ChangeFileExt(destname,DB_WHITE_EXT)); exit; end;
   if not FileSaveAs(changeFileExt(sourcename,DB_BLACK_EXT),ChangeFileExt(destname,DB_BLACK_EXT))
   then begin Werror('Error writing file'+EOL_STR+ChangeFileExt(destname,DB_BLACK_EXT)); exit; end;
  end;

  with bpgn do //close source, reopen destination bpgn.
  begin
    closeFile(BPGNFile);
    cFile:=destname;
    AssignFile(BPGNFile, cFile);
    Reset(BPGNFile);
  end;
  if bpgn.bIsDB then Caption:=Program_TITLE+' ( '+ChangeFileExt(destname,DB_HEADER_EXT)+')'
  else Caption:=Program_TITLE+' ( '+destname+')';
  SaveFileChanges(Sender, TRUE);
 end;
end;


procedure TMainForm.Game27Click(Sender: TObject);
begin
 ProcessBeg(' (Appending...)');
 bpgn.AppendGame(nGameIndex,ini1.ini.cworkbpgn);
 processEnd;
end;

procedure TMainForm.Utility11Click(Sender: TObject);
var s:string;
begin
 s:=savedialog2.Title;
 saveDialog2.title:='Set work BPGN file';
 if SaveDialog2.Execute then ini1.ini.cWorkBpgn:=SaveDialog2.FileName;
 savedialog2.title:=s;
end;

procedure TMainForm.Game26Click(Sender: TObject);
var s,fname:string;
bc:boolean;
begin
 StopTimer;
 s:=savedialog2.Title;
 saveDialog2.title:='Append game to';
 if SaveDialog2.Execute then
 begin
  fname:=SaveDialog2.FileName;
  bc:=not iscurrentbpgn(fname,bpgn.GetBPGNFilename);
  if bc then begin
   ProcessBeg(PROGRAM_TITLE + ' (Appending...)');
   bpgn.AppendGame(ngameIndex,fname);
   ProcessEnd;
  end
  else Game21Click(Sender);
 end;
 savedialog2.title:=s;
end;


procedure TMainForm.File8Click(Sender: TObject);
BEGIN
    StopTimer;
    if fileexists(ini1.ini.cWorkBpgn) then
    begin
        IF (NOT CheckFileSaveChanges(Sender, TRUE)) THEN
        EXIT;
        if banno then SendMessage(handle,WM_EngineAnnSTop,anneng,0);
        stopengines;
        OpenBPGNFile(Sender, ini1.ini.cworkbpgn);
    END
    Else Werror('Work file ('+ini1.ini.cworkbpgn+') does not exist!');
end;

procedure TMainForm.Utility12Click(Sender: TObject);
var mergeform:tmergeform;
cur,odir:string;
BEGIN
    StopTimer;
    try
    mergeform:=tmergeform.create(self);
    mergeform.Caption  := 'Merge files';

    mergeform.SetupValues(opendialog2.InitialDir);
    mergeform.ShowModal;

    IF (mergeform.bOK) THEN
    begin
     odir:=trim(ExtractFilePath(mergeform.edit1.text));
     opendialog2.initialdir:=odir;
     cur:=bpgn.getbpgnfilename;
     if ((odir=ExtractFilePath(cur)) and (ExtractFileext(mergeform.edit1.text)=ExtractfileExt(cur)) or (mergeform.edit2.text=cur)) then
     begin
      IF (NOT CheckFileSaveChanges(Sender, FALSE)) THEN
        EXIT;
      closebpgnfile;
     end;
     ProcessBeg(PROGRAM_TITLE + ' (Merging...)') ;
     mergefiles(mergeform.edit1.Text,mergeform.edit2.text);
     processEnd;
    end;
    finally mergeform.free;
    end;
END;

procedure TMainForm.Utility13Click(Sender: TObject);
var splitform:tsplitform;
cur,d1,d2,tmp:string;
source,tmpfname:array[0..512] of char;
deltmp:boolean;
BEGIN
    StopTimer;
    deltmp:=false;
    try
    splitform:=tsplitform.create(self);

    splitform.SetupValues(bpgn.GetBPGNFilename);
    splitform.ShowModal;

    IF (splitform.bOK) THEN
    begin
     cur:=bpgn.getbpgnfilename;d1:=splitform.edit1.text;d2:=splitform.edit2.text;
     if ((cur=d1) or (cur=d2)) then
     begin
      IF (NOT CheckFileSaveChanges(Sender, FALSE)) THEN
        EXIT;
      closebpgnfile;
      tmp:=uniquefilename(extractfilepath(cur));
      strpcopy(tmpfname,tmp);
      if not fileexists(cur) then exit;
      strpcopy(source,cur);
      copyfile(source,tmpfname,false);
      OpenBPGNFile(Sender, tmp);
      deltmp:=true;
     end;
     ProcessBeg(PROGRAM_TITLE + ' (Splitting '+cur+' ...)') ;
     bpgn.splitbpgn(d1,d2,splitform.ot1,splitform.ot2);
     processEnd;
     if deltmp then
     begin
      closebpgnfile;
      deletefile(tmp);
      openbpgnFile(Sender,cur);
     end;
    end;
    finally splitform.free;
    end;
end;

procedure TMainForm.search1Click(Sender: TObject);
var r1:tposdbrec;
res:integer;
begin
 stoptimer;
//  if bpgn.bIsDB then fform.RadioGroup1.Enabled:=true;
  fform.ChangeState(fsFilter);
  res:=bpgn.SynchFilter(fform.filter);
  if res<0 then fform.clearstats;
  if res>0 then fform.RadioGroup1.ItemIndex:=res-1;
  fform.Show;
  if fform.WindowState=wsMinimized then fform.WindowState:=wsNormal;
end;

procedure tmainForm.statgames(Sender:Tobject);
begin
 processBeg('Building tree');
 with fform do begin
  Button1Click(Sender);
  if bOk then
  begin
   bpgn.setstats(fform);
  end;
 end;
 ProcessEnd;
end;

procedure tmainForm.filtergames(Sender:Tobject);
var
fGl:tGlstate;
begin
 stopengines;

   IF (NOT CheckGameSaveChanges(nGameIndex)) THEN
   EXIT else bpgn.setGameChange(false);
   fform.Button1Click(Sender);
   if fform.bOk then
   begin
   bpgn.rfilter:=fform.filter;
   bpgn.setfilter;
   fform.filter.position:=bpgn.RFilter.position;
   filtergamelist;
   if bpgn.nFilterCount=Bpgn.nHeaderCount then fGl:=GlNormal else fgl:=glFilter;
   gamelistform.SetCaption(fgl);
   nGameIndex:=-1;
   gamelistclick(self);
   fform.Close;
   end;
end;

procedure TMainForm.search2Click(Sender: TObject);
begin
 stoptimer;
 IF (NOT CheckGameSaveChanges(ngameIndex)) THEN
 EXIT;
 bpgn.resetfilter;
 filtergamelist;
 gamelistform.SetCaption(glNormal);
 gamelistclick(self);
end;

procedure TMainForm.Filter1Click(Sender: TObject);
begin
 search1.Enabled := (BPGN.IsOpen AND (NOT BPGN.IsEmpty));
 search2.enabled := (Bpgn.IsOpen and (bpgn.nFilterCount<>bpgn.nHeaderCount));
 search3.enabled := search1.enabled;
 showselected1.Enabled:=search1.Enabled;
 selectallshow1.Enabled:=search1.Enabled and (bpgn.nFilterCount>0);
 unselectallshown1.Enabled:=showselected1.Enabled;
 showdeleted1.Enabled := search1.Enabled;
 markallfiltered1.Enabled:=search1.Enabled and (bpgn.nFilterCount>0);
 unmarkallfiltered1.Enabled:=markallfiltered1.Enabled;
end;

procedure TMainForm.File10Click(Sender: TObject);
var fname:string;
rec:TstrBoolRec;
begin
 StopTimer;
 if SaveDialog2.Execute then
 begin
  IF (NOT CheckFileSaveChanges(Sender, TRUE)) THEN
 EXIT;
  fname:=savedialog2.filename;
  if (iscurrentbpgn(fname,bpgn.GetBPGNFilename)) then
  begin
   wError('Can not copy games into the current file');
   exit;
  end;
  rec.s:=fname; rec.b:=false;
  doLongRun(fLongthread, bpgn.savefiltered_,@rec,'Saving filtered...');
 end;
end;



procedure TMainForm.File11Click(Sender: TObject);
var
fname:string;
rec:TstrBoolrec;
begin
 StopTimer;
 if SaveDialog2.Execute then
 begin
  IF (NOT CheckFileSaveChanges(Sender, TRUE)) THEN   EXIT;
  fname:=savedialog2.filename;
  if iscurrentbpgn(fname,bpgn.getbpgnfilename) then
  begin
   wError('Can not copy games into the current file');
   exit;
  end;
  rec.s:=fname; rec.b:=true;
  doLongRun(fLongthread, bpgn.savefiltered_,@rec,'Moving filtered...');
  filtergamelist;
  gamelistform.SetCaption(GLCurrent);
  gamelistclick(self);
 end;
end;

procedure TMainForm.Search3Click(Sender: TObject);
var sform:TstatsForm;
begin
 stoptimer;
 IF (NOT CheckGameSaveChanges(NGameIndex)) THEN
 EXIT;

 try
  sform:=TstatsForm.create(Self);
  sform.top:=ini1.ini.nStatT;
  sform.left:=ini1.ini.nstatL;

  sform.bp:=@bpgn;
  sform.ShowModal;
  INI1.INI.nstatL := sform.Left;
  INI1.INI.nstatT := sform.Top;
  INI1.INI.nstatW := sform.Width;
  INI1.INI.nstatH := sform.Height;
 finally
  sform.free;
 end;
end;

procedure TMainForm.CreateDatabase1Click(Sender: TObject);
var hFileName,fname:string;
bpe:boolean;
BEGIN
   StopTimer;
   stopengines;
   IF (NOT CheckFileSaveChanges(Sender, FALSE)) THEN
        EXIT;
   fname:=bpgn.getBPGNFilename;
   hfileName:=ChangeFileExt(fname,DB_HEADER_EXT);
   if not fileexists(fname) then exit;
   if fileexists(hFileName) then
   begin
    if not wokCancel('File '+hFileName+' already exists. Overwrite?') then exit;
   end;

   ProcessBeg('Checking BPGN file...');
   if bpgn.checkDBType<0 then
   begin
    if wOkCancel('File '+fname+ ' contains both Bug and Zh games'+EOL_STR+
             'Such files cannot be converted to the database format'+ EOL_STR+
             'Split the file into Bug and Zh parts?')
    then begin Utility13Click(Sender); end;
    processEnd;
    exit;
   end;
   processEnd;

   bpe:=bpgn.bPromptError;
   bpgn.SetPromptError(false);
   try
    doLongRun(fLongthread, bpgn.createdatabase,@fname,'Creating database...',true);
   except
    with bpgn do
     if not bIsDB then begin pdblist[1].clear; pdblist[2].clear; end;
    gamelistclick(Self);
    exit;
   end;
   bpgn.SetPromptError(bpe);

   DoLongRun(fLongThread,bpgn.UpdateDB_,nil,'Saving DB to disk...');
   if bpgn.bIsDB then begin
    Caption:=Program_TITLE+' ( '+hFileName+')';
    savedialog2.Filter:='Supported (*.bpgn,*.bh)|*.bpgn;*.bh|Bug DB|*.bh|BPGN (*.bpgn)|*.bpgn|All|*.*';
    fform.godatabase;
   end;
END;

procedure tmainform.movestatclick(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var column,row:integer;
begin
 fform.StringGrid1.MouseToCell(X, Y, Column, Row);
 if ((row>0) or (column=0)) then execmovestats(fform,row)
 else bpgn.sortmovestats(fform,column);
end;

procedure tmainform.execmovestats(xform:tfform;row:integer);
var r1:TPosdbrec;
fbd,nFromX,NfromY,ntox,ntoy,nfromto,nfrom,nto:integer;
Button:TMouseButton;
Shift:TShiftState;
begin;
 fbd:=xform.RadioGroup1.ItemIndex+1;
 if fbd>2 then exit; //nonpositional search
 xform.Button1Click(xform.Button4);
 if row=0 then //1 move back on board
 begin
  stoptimer;
  if ((fbd=1) and (board1form.SpeedButton2.Enabled)) then Board1click2(Self)
  else if board2form.SpeedButton2.Enabled then board2click2(Self);
  if not xform.filter.bautosearch then
  begin
   if bpgn.SynchFilter(xform.filter)=0 then exit; //already in sync
   processBeg('Building tree...');
   bpgn.setStats(xform);
   processEnd;
  end;
  exit;
 end;

 if xform.NotaMove(row) then exit; //click on empty cell;
 if not bpgn.SynchFilter(xform.filter)=0 then
 begin
  processBeg('Building tree...');
  bpgn.setStats(xform);
  processEnd;
  exit;
 end;

 nfromto:=bpgn.nextdblist[row].nFromTo;
 inttofromto(nfromto,nfrom,nto);

 if xform.filter.possearchmode=1 then
 begin
   board1form.board.chesstopixels(nfrom,NfromX,NfromY,bpgn.nextdblist[row].cMove);
   board1form.board.chesstopixels(nto,Ntox,nToY,bpgn.nextdblist[row].cMove);
   Board1MouseDown(Self, Button, Shift,nFromX,nFromY);
   Board1MouseUp(Self, Button, Shift, nToX,nToY);
 end
 else begin
   board2form.board.chesstopixels(nfrom,Nfromx,NfromY,bpgn.nextdblist[row].cMove);
   board2form.Board.chesstopixels(nto,Ntox,nToY,bpgn.nextdblist[row].cMove);
   Board2MouseDown(Self, Button, Shift, nFromX,nFromY);
   Board2MouseUp(Self, Button, Shift, nToX,nToY);
 end;
 bpgn.setStats(xform);
end;

procedure TMainForm.AirDrops1Click(Sender: TObject);
begin
 bairdrop:=not bairdrop;
 airdrops1.Checked:=bairdrop;
end;

procedure TMainForm.DBsavedepth1Click(Sender: TObject);
var sdf:TDBDepthForm;
begin
 try
  sdf:=TDBDepthForm.Create(Self);
  sdf.SetupValues(bpgn.savedepth);
  sdf.ShowModal;
  if sdf.bOk then bpgn.savedepth:=sdf.savedepthvalue;
 finally sdf.Free;
 end;
end;

procedure TMainForm.FullStats1Click(Sender: TObject);
begin
 fullstats1.Checked:=not fform.bfullstat;
 fform.bfullstat:=fullstats1.Checked;
end;

procedure TMainForm.DonotSortMovelist1Click(Sender: TObject);
begin
 ini.nMoveOrder:=0;
end;



procedure TMainForm.Pawn1Click(Sender: TObject);
begin
 dropclick(popupmenu1.Tag,PAWN_INDEX);
end;

procedure TMainForm.Rook1Click(Sender: TObject);
begin
 dropclick(popupmenu1.Tag,ROOK_INDEX);
end;

procedure TMainForm.Knight1Click(Sender: TObject);
begin
 dropclick(popupmenu1.Tag,KNIGHT_INDEX);
end;

procedure TMainForm.Bishop1Click(Sender: TObject);
begin
 dropclick(popupmenu1.Tag,BISHOP_INDEX);
end;

procedure TMainForm.Queen1Click(Sender: TObject);
begin
 dropclick(popupmenu1.Tag,QUEEN_INDEX);
end;

procedure Tmainform.dropclick(nboard,npiece:integer);
var button:TMousebutton;
Shift:TShiftState;
ncolor:integer;
nfromX,nfromY,x,y:integer;
begin
 button:=mbLeft;
 if nboard=1 then
 with board1form.Board do
 begin
   if chess.bWhitesMove then ncolor:=White_Color
   else ncolor:=Black_color;
   dropchesstoboard(ncolor,npiece,X,Y);
   nFromX := PieceXY(X)+1;
   nfromY := PieceXY(Y)+1;
   Board1Mousedown(Self, Button, Shift,nFromX, nFromY);
   Board1mouseUp(Self,button,Shift,nPopupDropx,npopupDropy);
 end
 else with board2form.Board do
 begin
   if chess.bWhitesMove then ncolor:=White_Color
   else ncolor:=Black_color;
   dropchesstoboard(ncolor,npiece,X,Y);
   nFromX := PieceXY(X)+1;
   nfromY := PieceXY(Y)+1;
   Board2MouseDown(self,button,shift, nFromX, nFromY);
   Board2MouseUp(self,Button,Shift,nPopupdropx,nPopupdropY);
 end;
end;


procedure TMainForm.Database1Click(Sender: TObject);
begin
  Generatedatabase1.Enabled:=bpgn.IsOpen;
  killdoubles1.Enabled:=bpgn.IsOpen and bpgn.bIsDB;
  ree1.Enabled:=killdoubles1.Enabled;
  if bpgn.bIsDB then generatedatabase1.Caption:='Rebuild' else generatedatabase1.Caption:='Create';
//  killshortgames1.Enabled:=killdoubles1.Enabled;
end;

procedure TMainForm.Killdoubles1Click(Sender: TObject);
var glState:tglstate;
begin
 bpgn.resetfilter;
 try
  DoLongRun(fLongThread,bpgn.killdoubles_,nil,'Looking up doubles...',true);
 except
  bpgn.nFilterCount:=bpgn.nTmpCounter;
  filtergamelist;
  gamelistform.SetCaption(glDAbort);
  exit;
 end;
 if bpgn.nFilterCount<1 then
 begin
  wOk('No doubles found');
  bpgn.resetfilter;
  glstate:=GLnormal;
 end else glState:=GlDoubles;
 filtergamelist;
 gamelistform.SetCaption(glstate);
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
 if navigateform.BorderStyle=bsnone then
 navigateform.Visible:=true;
end;

procedure TMainForm.FormDeactivate(Sender: TObject);
begin
 if navigateform.BorderStyle=bsnone then
 navigateform.Visible:=false;
end;

procedure TMainForm.Setupposition1Click(Sender: TObject);
begin
 IF (NOT checkgamesaveChanges(nGameIndex)) THEN
        EXIT;
 boardcontrolSetup;
end;

procedure tmainform.BoardControlSetup;
begin
       try
        setupallform:=Tsetupallform.Create(self);
       except
        werror('Not enough memory');
        exit;
       end;
       cCurrentInfo:='You are currently in setup mode'+EOL_STR+
       'This mode allows you to setup an arbitrary starting position for a game'+EOL_STR+
       'In the setup mode, most of the controls do not work'+EOL_STR+
       'To finish setup and return to normal mode click Done button on any of the boards';
       disablecontrols;
       with setupallform do
       if nGameIndex>=0 then
       begin
        edit9.Text:=bpgn.GetWhiteA(nGameindex);
        edit10.Text:=bpgn.GetBlackA(nGameindex);
        edit11.Text:=bpgn.GetWhiteB(nGameindex);
        edit12.Text:=bpgn.GetBlackB(nGameindex);
        edit13.Text:=bpgn.GetEvent(ngameIndex);
        edit14.Text:=bpgn.GetSite(nGameIndex);
        edit15.Text:=bpgn.GetDate(nGameIndex);
        edit16.Text:=IntToStr(TOOLBOX_GetTimeVal(bpgn.GetTimeC(nGameIndex)));
        edit17.Text:=IntToStr(TOOLBOX_GetTimeIncVal(bpgn.GetTimeC(nGameIndex)));
        edit18.Text:=bpgn.GetRWA(nGameindex);
        edit19.Text:=bpgn.GetRBA(nGameindex);
        edit20.Text:=bpgn.GetRWB(nGameindex);
        edit21.Text:=bpgn.GetRBB(nGameindex);
        edit22.Text:='1';
        edit23.Text:='1';
        radiogroup1.ItemIndex:=resultstrtobyte(bpgn.GetResult(ngameIndex))-1;
       end
       else begin
       end;
       with setupallform do
       begin
        if board1form.Board.Chess.bWhitesMove then Checkbox1.Checked:=true
        else Checkbox1.Checked:=false;
        if board2form.Board.chess.bWhitesMove then Checkbox3.Checked:=true
        else Checkbox3.Checked:=false;

        edit5.Text:=bpgn.GetCurrentBoard1WTime;
        edit6.Text:=bpgn.GetCurrentBoard1bTime;
        edit7.Text:=bpgn.GetCurrentBoard2WTime;
        edit8.Text:=bpgn.GetCurrentBoard2BTime;
        edit1.Text:=board1form.Board.Chess.GetWhitePieceStr;
        edit2.Text:=board1form.Board.Chess.GetBlackPieceStr;
        edit3.Text:=board2form.Board.Chess.GetWhitePieceStr;
        edit4.Text:=board2form.Board.Chess.GetBlackPieceStr;

        {Castling board 1}
        checkbox2.Checked:=Board1form.Board.Chess.aKCastle[1];
        checkbox4.Checked:=Board1form.Board.Chess.aKCastle[2];
        checkbox5.Checked:=Board1form.Board.Chess.aQCastle[1];
        checkbox10.Checked:=Board1form.Board.Chess.aQCastle[2];
                {Castling board 2}
        checkbox7.Checked:=Board2form.Board.Chess.aKCastle[1];
        checkbox8.Checked:=Board2form.Board.Chess.aKCastle[2];
        checkbox9.Checked:=Board2form.Board.Chess.aQCastle[1];
        checkbox6.Checked:=Board2form.Board.Chess.aQCastle[2];
        {enpassant}
        edit24.Text:=board1form.Board.Chess.getEnpassantsq;
        edit25.Text:=board2form.Board.Chess.getEnpassantsq;

        edit26.Text:=extractDelimitedStr(bpgn.GetReason,'{','}');
       end;

        Board1Form.PaintBox1.OnMouseDown :=SMouseDown;
        Board1Form.PaintBox1.OnMouseMove :=sMouseMove;
        Board1Form.PaintBox1.OnMouseUp   :=SMouseUp;

        Board2Form.PaintBox1.OnMouseDown :=sMouseDown;
        Board2Form.PaintBox1.OnMouseMove :=SMouseMove;
        Board2Form.PaintBox1.OnMouseUp   :=SMouseUp;

        Caption:='Setup position';
        board1form.gosetup;
        board2form.gosetup;
        board1form.Label2.OnClick:=finalizesetup;
        board2form.Label2.OnClick:=finalizesetup;
        navigateform.Bitbtn1.OnClick:=donothing;
end;

PROCEDURE   tmainform.SMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var res:integer;
begin
 if button=mbRight then
  if sender=board1form.PaintBox1 then
  with board1form.Board do begin
       res:=isOnboard(x,y);
       case res of
       0: exit;
       1: markpromoted.Enabled:=false;
       2: with markpromoted do begin enabled:=true; Caption:='Mark as promoted';end;
       else with markpromoted do begin enabled:=true; Caption:='Unmark promoted piece';end;
       end;
       setupmenu.Tag:=1;
       nPopupDropX:=Indexxy(x-nmxoff);
       nPopupDropY:=Indexxy(y-nmyoff);
       setupmenu.Popup(board1form.Left+x,board1form.Top+y);
  end
  else with board2form.board do begin
       res:=isOnboard(x,y);
       case res of
       0: exit;
       1: markpromoted.Enabled:=false;
       2: with markpromoted do begin enabled:=true; Caption:='Mark as promoted';end;
       else with markpromoted do begin enabled:=true; Caption:='Unmark promoted piece';end;
       end;
       setupmenu.Tag:=2;
       nPopupDropX:=Indexxy(x-nmxoff);
       nPopupDropY:=Indexxy(y-nmyoff);
       setupmenu.Popup(board2form.Left+x,board2form.Top+y);
  end;

 IF (Button = mbLeft) THEN
 BEGIN
   If sender=board1form.PaintBox1 then
   with board1form do
        Board.SMouseDown(PaintBox1.Canvas, X, Y, Left, Top)
   else with board2form do Board.SMouseDown(PaintBox1.Canvas, X, Y, Left, Top);
 END;
end;

PROCEDURE   tmainform.SMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if Sender=Board1form.PaintBox1 then
     Board1Form.Board.OnMouseMove(Board1Form.PaintBox1.Canvas, X, Y)
  else with board2form do
     Board.OnMouseMove(PaintBox1.Canvas, X, Y);
end;

PROCEDURE   tmainform.SMouseUp  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if Sender=Board1form.PaintBox1 then
    with board1form do
     Board.sMouseUp(PaintBox1.Canvas, X, Y, Left, Top)
    else with board2form do
     Board.sMouseUp(PaintBox1.Canvas, X, Y, Left, Top);
end;

procedure tmainform.finalizesetup(Sender:Tobject);
var fen1,fen2,s,fen:string;
ps:integer;
rec:tbpgnrec;
begin
 with board1form.Board.Chess do SetPieces(cSaveWPieces,cSaveBPieces);
 with board1form do Board.drawboard(paintbox1.Canvas);
 with board2form.Board.Chess do SetPieces(cSaveWPieces,cSaveBPieces);
 with board2form do Board.drawboard(paintbox1.Canvas);
 setupallform.ShowModal;
 case setupallform.ModalResult of
 MrCancel:
  begin
   with board1form.Board.Chess do SetPieces('QRBNP','qrbnp');
   with board1form do Board.drawboard(paintbox1.Canvas);
   with board2form.Board.Chess do SetPieces('QRBNP','qrbnp');
   with board2form do Board.drawboard(paintbox1.Canvas);
   exit;
  end;
 MrOk:
  Begin
   with board1form.Board.chess do
   begin
    bwhitesmove:=setupallform.CheckBox1.Checked;
    aKCastle[1]:=setupallform.checkbox2.checked;
    aKCastle[2]:=setupallform.checkbox4.checked;
    aQCastle[1]:=setupallform.checkbox5.checked;
    aQCastle[2]:=setupallform.checkbox10.checked;
    setpieces(UpperCase(setupallform.Edit1.Text),LowerCase(setupallform.Edit2.Text));
    mprev.nFromX:=0;mprev.nFromY:=0;
    fen1:=getBFenStr; delete(fen1,length(fen1),1);
   end;
   with board2form.Board.chess do
   begin
    bwhitesmove:=setupallform.CheckBox3.Checked;
    aKCastle[1]:=setupallform.checkbox7.checked;
    aKCastle[2]:=setupallform.checkbox8.checked;
    aQCastle[1]:=setupallform.checkbox9.checked;
    aQCastle[2]:=setupallform.checkbox6.checked;
    setpieces(UpperCase(setupallform.Edit3.Text),LowerCase(setupallform.Edit4.Text));
    mprev.nFromX:=0;mprev.nFromY:=0;
    fen2:=getBFenStr; delete(fen2,length(fen2),1);
   end;
   with setupallform do begin
    s:=trim(setupallform.Edit24.Text);
    if length(s)=0 then fen1:=fen1+' -' else fen1:=' '+fen1+s;
    s:=trim(setupallform.Edit25.Text);
    if length(s)=0 then fen2:=fen2+' -' else fen2:=' '+fen2+s;
    fen:=fen1+' ' +disptimetosec(edit5.Text)+' '+disptimetosec(edit6.Text)+
    ' | '+fen2+' '+disptimeToSec(edit7.Text)+' '+disptimetosec(edit8.Text);
   end;


   setupallform.genBPGNrec(@rec);
   rec.aFen:=fen;
   bpgn.Appendnewgame(@rec);

   enablecontrols;
   GameListForm.AddItem;
   GameListForm.ListBox1.Enabled   := TRUE;
   GameListForm.ListBox1.ItemIndex := GameListForm.ListBox1.Items.Count - 1;
   GameListClick(Sender);
  end;
 mrAbort:
  begin
   board1form.Caption:='Board #1';
   board2form.Caption:='Board #2';
   board1form.LoadHandle;
   board2form.LoadHandle;
   board1form.undoClear;
   board2form.undoclear;
   enablecontrols;
  end;
 end;
 setupallform.Free;
 if bpgn.bIsDB then Caption:=Program_title+' ('+changefileExt(bpgn.GetBPGNFilename,DB_HEADER_EXT)+')'
 else Caption:=Program_title+' ('+bpgn.GetBPGNFilename+')'

end;

procedure TMainForm.WhiteK1Click(Sender: TObject);
begin
 if setupmenu.Tag=1 then
  with board1form do Board.DropPopupPiece(WHITE_KING,Paintbox1.canvas)
 else with board2form do
  Board.DropPopupPiece(WHITE_KING,Paintbox1.canvas);
end;

procedure TMainForm.BlackK1Click(Sender: TObject);
begin
 if setupmenu.Tag=1 then
  with board1form do Board.DropPopupPiece(BLACK_KING,Paintbox1.canvas)
 else with board2form do
  Board.DropPopupPiece(BLACK_KING,Paintbox1.canvas);
end;

procedure TMainForm.markpromotedClick(Sender: TObject);
begin
 if setupmenu.Tag=1 then with board1form.Board do switchpromoted(nPopupdropx,npopupdropy)
 else with board2form.Board do switchpromoted(nPopupdropx,npopupdropy)
end;

procedure TMainForm.ree1Click(Sender: TObject);
var res:integer;
begin
  stoptimer;
//  ftreeform.SetupValues(bpgn.RFilter);
  fform.ChangeState(fsTree);
  res:=bpgn.SynchFilter(fform.filter);
  if res<0 then  statgames(fform.Button4);
  if res>0 then fform.RadioGroup1.ItemIndex:=res-1;
  fform.Show;
  if fform.WindowState=wsMinimized then fform.WindowState:=wsNormal;
end;

procedure tmainform.cancelfileChanges;
var I,j:longint;
begin
 for i:=0 to bpgn.nFiltercount-1 do
 begin
  j:=bpgn.getFIndex(i);
  if bpgn.GetTag(j)=TAG_DELETE then
  begin
   bpgn.SetTag(j,TAG_OK);
//   gamelistform.ListBox1.Items[i]:=bpgn.GetHeaderItem(j);
  end;
 end;
 gamelistform.ListBox1.Invalidate;
 bpgn.SetFileChange(false);
end;

procedure tmainform.Listbox1Data(Sender: TObject; Item: TListItem);
begin
 with bpgn do item.Caption:=getHeaderItem(getfindex(item.Index));
end;

procedure TMainForm.ListBox1KeyPress(Sender: TObject; var Key: Word; Shift: TShiftState);
var ncur:longint;
begin
gamelistclick(Sender);
end;

procedure tmainform.raiseboards;
begin
 if not bpgn.iszh(nGameIndex) then raiseform(board2form);
 raiseform(board1form);
end;

procedure TMainForm.A1Click(Sender: TObject);
var i,n,np:integer;
nGoalGLW,nGoalGLH,nGoalNMH,nGoalNMW,nGoalCW,nGoalCH:integer;
nRealGLW,nRealGLH,nRealNMH,nRealNMW,nrealCW,nrealCH:integer;
nDeltaW,nDeltaH,nLO,bh:integer;
s:string;
begin
 nDeltaW:=Board1form.Width div 5;
 nDeltaH:=20;
 nGoalNMH:=3;
 nGoalnMW:=13;
 nGoalGLW:=80;
 nGoalGLH:=10;
 nGoalCH:=3;
 nGoalCW:=20;
 s:='A';
 bh:=5;

 if navigateform.FormStyle<>fsMDIChild then NavigateForm.SpeedButton14MouseEnter(self);
 for i:=1 to nGoalGLW-1 do s:=s+'a';
 nRealGLW:=gamelistform.Canvas.TextWidth(s);
 nRealGLH:=nGoalGLH*gamelistform.Canvas.TextHeight('A');
 nRealCW:=nGoalCW* commentform.Canvas.TextWidth('a') ;
 nRealCH:=nGoalCH* commentform.Canvas.TextHeight('A');
 nRealnMw:=112;
 nRealnMH:=ngoalnmh * nextmoveform.Canvas.TextHeight('A');


 with board1form do SetBounds(1,1,Width,height);
 board1form.bActual:=true;
 board1form.FormResize(Self);
 with board2form do SetBounds(board1form.Width+1,1,board1form.Width,board1form.Height);
 board1form.bActual:=board22.Checked;
 n:=board1form.Top+board1form.Height+1;

 if (n+navigateform.Height)<ClientHeight then with navigateform do setbounds(1,n,width,height)
 else navigateform.SetBounds(1,ClientHeight-navigateform.Height,navigateform.Width,navigateform.Height);

 if (nRealGLW+2*board1form.Width+bh-nDeltaW)<ClientWidth then
 begin
  n:=ClientWidth-board2form.Left -board2form.Width-bh;
  if n<nRealGLW then n:=nRealGLW;
  gamelistform.Setbounds(clientWidth-n-bh,1,n,clientHeight-bh);
  nLO:=1;
 end
 else
 begin
  n:=clientHeight-board1form.Left -board1form.Height;
  if n<nREalGLH then n:=nRealGLH;
  gamelistform.Setbounds(ClientWidth-nRealGLW-bh,ClientHeight-n-bh, nRealGLW,n);
  nLO:=2;
 end;

 if nlo=1 then
 begin
  i:=ClientHeight-navigateform.Height-navigateform.Top-bh;
  if i<nRealCh then i:=nRealCh;
  n:=gamelistform.Left-2*nRealnMW-1;
  if n<nRealCW then n:=nRealCW;
  Commentform.SetBounds(1,navigateform.Top+navigateform.Height+1,n,i);
  nextmoveform.SetBounds(commentform.Left+ commentform.Width, commentform.Top, nRealnMW,i);
  prevmoveform.SetBounds(nextmoveform.Left+nextmoveform.Width,commentform.Top, nRealnMw,i);
 end
 else begin
  i:=ClientHeight-navigateform.Height-navigateform.Top-bh;
  if i<nRealCh then i:=nRealCh;
  n:=gamelistform.Left-2;
  if n<nRealCW then n:=nRealCW;
  Commentform.SetBounds(1,navigateform.Top+navigateform.Height+1,n,i);
  i:=(gamelistform.Top-3) div 2;
  if i<nRealNMH then i:=nRealnMH;
  if (ClientWidth-board2form.Left-board2form.Width-2) div 2 > nRealnmW then
  begin
   nextmoveform.SetBounds(board2form.Left+board2form.Width+1,1,nRealnMW,2*i);
   prevmoveform.SetBounds(nextmoveform.Left+nextmoveform.Width+1,1,nRealnMW,2*i);
  end
  else begin
   n:=ClientWidth-board2form.left - board2form.Width-bh;
   if n<nRealnMw then n:=nRealnMW;
   nextmoveform.SetBounds(board2form.Left+board2form.Width+1,1,n,i);
   prevmoveform.SetBounds(nextmoveform.left,nextmoveform.Top+nextmoveform.Height+1,n,i);
  end;
 end;
end;

procedure tmainform.WMProgressUpdate(var message:Tmessage);
begin
 if Assigned(progbar) then Progbar.ProgressBar1.Position:=1+100*message.WParam div message.LParam;
end;

procedure tmainform.WMBpgnError(var message:tmessage);
begin
 Werror(bpgn.cResult);
 if message.WParam=0 then bpgn.cResult:='';
end;

procedure tmainform.WMBpgnInfo(var message:tmessage);
var ferror:Tviewactualform;
begin
   try
    ferror:=tviewactualform.create(nil);
    ferror.ResetAll;
    if message.WParam=0 then ferror.Caption    := 'Errors';
    ferror.Memo1.Text := bpgn.cResult;
    ferror.ShowModal;
   finally ferror.free;
   end;
   if message.WParam=0 then bpgn.cResult:='';
end;

procedure tmainform.WMUpdateDB(var message:tmessage);
begin
 DoLongRun(fLongThread,bpgn.updateDB_,nil,'Saving DB updates...');
end;

procedure TmainForm.doLongRun(var Thread:TWaitThread; Handler:TThreadMethod; Data:pointer; Caption:string; bAllowAbort:boolean=false);
var babort:boolean;
begin
    try
    progbar:=tprogbar.Create(nil);
    progbar.Caption:=Caption;
    progbar.setAbort(bAllowAbort);
    progbar.Show;
    enabled:=false;
    bpgn.bAbortLong:=false;
    MsgWaitForThread(Thread,Handler,Data);
//    handler(data);
    if bpgn.bAbortLong then Abort;
   finally
    enabled:=true;
    progbar.Free; progbar:=nil;
   end;
end;

FUNCTION TMainForm.CheckGameSaveChanges(nIndex: Integer): Boolean;
VAR
    nI: Integer;
BEGIN
with bpgn do
begin
    IF (NOT bOpen) THEN
    BEGIN
        Result := TRUE;
        EXIT;
    END;
    IF ((nIndex >= 0) AND (nIndex < nHeaderCount)) THEN
    BEGIN
        IF (isgamechange) THEN
        BEGIN
            if ((bIsDB) and (donoteditgames1.Checked)) then
            begin
             setGameChange(false);
             exit;
            end;
            nI := wNoYesCancel('Save game changes?');
            IF (nI = 3) THEN
            BEGIN
                Result := FALSE;
                EXIT;
            END;
            IF (nI = 1) THEN
            begin
                DoLongRun(fLongThread,bpgn.SaveGame_,@nIndex,'Saving game...');
                bdbchange:=true;
            end
            ELSE begin
                setGameChange(FALSE);
                alist.restore(nIndex);
            end;
        END;
    END;
    Result := TRUE;
END;

end;

procedure TMainForm.Default1Click(Sender: TObject);
var i:integer;
begin
 nEngineB:=defFormB;
 fEngineF.Color  := clBlack;
 fEngineF.Height := -11;
 fEngineF.Name   := 'Courier New';
 fEngineF.Pitch  := fpDefault;
 fEngineF.Size   := 10;
 fEngineF.Style  := [];
 ini1.ini.bEngineWR := False;
 WordWrap1.Checked:= ini1.ini.bEngineWR;
 for i:=1 to MAX_ENGINE do
 if Engines[i].exist then with engineforms[i].Memo1 do
 begin
     color:=nEngineB;
     WordWrap := ini1.ini.bEngineWR;
     IF (WordWrap) THEN
         ScrollBars := ssVertical
     ELSE
         ScrollBars := ssBoth;
     Font:=fEngineF;
 end;
end;

procedure tmainform.WMupdateTree(var message:tmessage);
begin
 fform.Repaint;
 statgames(fform.Button4);
end;

procedure TMainForm.Showdeleted1Click(Sender: TObject);
var cTag:char;
fst:TGlState;
begin
  stoptimer;
  IF (NOT CheckGameSaveChanges(nGameIndex)) THEN
  EXIT else bpgn.setGameChange(false);
  stopengines;
  cTag:=Tag_ok;
  if Sender=showdeleted1 then begin cTag:=Tag_delete; fst:=GlDelete; end;
  if Sender=showselected1 then begin cTag:=Tag_Select; fst:=glSelect; end;
  bpgn.filterTag(cTag);
  filtergamelist;
  gamelistform.SetCaption(fst);
  nGameIndex:=-1;
  gamelistclick(self);
end;

procedure TMainForm.Donoteditgames1Click(Sender: TObject);
begin
 with donoteditgames1 do begin
  Checked:=not checked;
  ini1.INI.bROgames:=Checked;
 end;
end;

procedure TMainForm.Showtreeonopen1Click(Sender: TObject);
begin
 with showtreeonopen1 do begin
  Checked:=not Checked;
  ini1.INI.bOpenTree:=Checked;
 end;
end;

procedure TMainForm.Brokenresult1Click(Sender: TObject);
begin
  stoptimer;
  IF (NOT CheckGameSaveChanges(nGameIndex)) THEN
  EXIT else bpgn.setGameChange(false);
  stopengines;
  bpgn.filterBroken;
  filtergamelist;
  gamelistform.SetCaption(glBroken);
  nGameIndex:=-1;
  gamelistclick(self);
end;

procedure TMainForm.Markallfiltered1Click(Sender: TObject);
var ctag:char;
begin
  IF (NOT CheckGameSaveChanges(nGameIndex)) THEN
  EXIT else bpgn.setGameChange(false);
  if Sender=Markallfiltered1 then bpgn.DeleteFiltered;
  if Sender=SelectAllshow1 then bpgn.SelectFiltered;
  if Sender=UnmarkAllfiltered1 then bpgn.untagFiltered(TAG_Delete);
  if Sender=UnselectAllshown1 then bpgn.untagFiltered(TAG_Select);
  if Sender=In1 then bpgn.invertTag(TAG_SELECT);
  gamelistform.Outofdate;
end;

procedure TMainForm.PopupMenu1Popup(Sender: TObject);
begin
 setupposition1.Enabled:=(BPGN.IsOpen AND (NOT BPGN.IsEmpty))and (not ((bpgn.bIsDB) and (donoteditgames1.Checked)) AND BPGN.CanAddNewGame);
end;

procedure tmainform.WMCancelOp(var message:tmessage);
begin
//  if flongThread<>nil then flongThread.AbortThread;
bpgn.bAbortLong:=true;
end;

procedure TMainForm.Select1Click(Sender: TObject);
begin
    StopTimer;
    BPGN.SetTag(nGameIndex, TAG_Select);
    gamelistform.listbox1.Refresh;
end;

procedure TMainForm.RestoreStringFromTag1Click(Sender: TObject);
var i,nc,n:longint;
begin
 IF (NOT CheckFileSaveChanges(Sender, TRUE)) THEN
 EXIT;
 nc:=0;
 if Sender=RestoreStringFromTag1 then
 with bpgn do
 for i:=0 to nFilterCount-1  do
 begin
  n:=getFindex(i);
  if isbroken(n) then begin inc(nc,1); SyncResultFromTag(n); end;
 end
 else with bpgn do
 for i:=0 to nFilterCount-1  do
 begin
  n:=getFindex(i);
  if isbroken(n) then begin inc(nc,1); SyncResultFromString(n); end;
 end;
 if nc>0 then DoLongRun(fLongThread,bpgn.SaveUpdates_,nil,'Saving changes to BPGN');
end;

procedure TMainForm.Replayallgames1Click(Sender: TObject);
var i:longint;
begin
 With Sender as TMenuItem do checked:=not Checked;
end;

procedure TMainForm.Replacehandleinlisted1Click(Sender: TObject);
var HandleForm:ThandleForm;
cFind,s:String;
i,j,ps,cnt:longint;
bfound:boolean;
begin
    StopTimer;
    If not CheckGameSaveChanges(nGameIndex) then exit;
    try
    handleForm:=ThandleForm.create(self);
    HandleForm.Caption    := 'Replace handle';
    HandleForm.cOrgHandle := BPGN.GetWhiteA(nGameIndex);
    HandleForm.cNewHandle := HandleForm.cOrgHandle;
    HandleForm.Edit2.Visible:=true;
    HandleForm.SetupValues;
    HandleForm.ShowModal;

    IF (HandleForm.bOK) THEN
    BEGIN
        cFind:=LowerCase(trim(HandleForm.Edit2.Text));
        cnt:=length(cFind);
        with bpgn do
        for j:=0 to nFilterCount-1 do
        Begin
         i:=getFindex(j); bfound:=false;
         if (cFind=LowerCase(trim(GetWhiteA(i)))) then begin SetWhiteA(i, HandleForm.cNewHandle); bfound:=true; end;
         if (cFind=LowerCase(trim(GetWhiteB(i)))) then begin SetWhiteB(i, HandleForm.cNewHandle); bfound:=true; end;
         if (cFind=LowerCase(trim(GetBlackA(i)))) then begin SetBlackA(i, HandleForm.cNewHandle); bfound:=true; end;
         if (cFind=LowerCase(trim(GetBlackB(i)))) then begin SetBlackB(i, HandleForm.cNewHandle); bfound:=true; end;
         if bfound then
         begin //adjust reason
                 s:=GetaReason(i);
                 ps:=pos(cFind+' ',lowercase(s));
                 if ps>0 then  //replace
                 begin
                  delete(s,ps,cnt); insert(HandleForm.cNewHandle,s,ps);
                 end;
                 ps:=pos(cFind+'''s',lowercase(s));
                 if ps>0 then  //replace
                 begin
                  delete(s,ps,cnt); insert(HandleForm.cNewHandle,s,ps);
                 end;

                 SetReason(i,s);
                 bdbchange:=true;
         end;
        end; {for}

        Board1Form.cWHandle := BPGN.GetWhiteAHandle(nGameIndex);
        Board1Form.cBHandle := BPGN.GetBlackAHandle(nGameIndex);
        Board1Form.LoadHandle;
        Board1Form.FormResize(Sender);
        Board1Form.DisplayBoard;

        Board2Form.cWHandle := BPGN.GetWhiteBHandle(nGameIndex);
        Board2Form.cBHandle := BPGN.GetBlackBHandle(nGameIndex);
        Board2Form.LoadHandle;
        Board2Form.FormResize(Sender);
        Board2Form.DisplayBoard;

        gamelistform.listbox1.Repaint;
        if bpgn.IsGameChange then //save changes
        begin
             DoLongRun(fLongThread,bpgn.SaveUpdates_,nil,'Saving changes to BPGN');
             bpgn.SetGameChange(false);
        end;
    END;
    finally handleForm.free;
    end;
end;

procedure TMainform.Timer4Timer(sender:Tobject);
var nInt:longint;
begin
 with Sender as TTimer do nInt:=Interval;
 with board1form do
 begin
  if board.Chess.bWhitesMove then
  begin
   dec(nWtime,nInt);
   cWtime:=TOOLBOX_GetDispTimeStr(nWtime);
  end
  else begin
   dec(nBtime,nInt);
   cBtime:=TOOLBOX_GetDispTimeStr(nBtime);
  end;
  LoadTime;
 end;

 with board2form do
 begin
  if board.Chess.bWhitesMove then
  begin
   dec(nWtime,nInt);
   cWtime:=TOOLBOX_GetDispTimeStr(nWtime);
  end
  else begin
   dec(nBtime,nInt);
   cBtime:=TOOLBOX_GetDispTimeStr(nBtime);
  end;
  LoadTime;
 end;
end;


procedure TMainForm.Showlag1Click(Sender: TObject);
begin
    StopTimer;
    showlag1.Checked                := NOT showlag1.Checked;
    ini1.INI.bShowLag := showLag1.Checked;
    loadRecord;
end;

procedure TMainForm.Showtenth1Click(Sender: TObject);
begin
    StopTimer;
    showtenth1.Checked                := NOT showtenth1.Checked;
    ini1.INI.bShowtenth := showtenth1.Checked;
    loadRecord;
    Board1Form.FormResize(Self);
    Board2form.FormResize(Self);
end;

procedure TMainForm.Onlinedatabase1Click(Sender: TObject);
begin
    ShellExecute(0, 'open', PChar(ini1.INI.cDBSite), nil, nil, SW_ShowNormal);
end;

END.
